@@ -7,13 +7,25 @@
 #ifndef ThirdPartyUtil_h__
 #define ThirdPartyUtil_h__
 
-#include "nsCOMPtr.h"
-#include "nsString.h"
+#include <cstdint>
+#include "ErrorList.h"
 #include "mozIThirdPartyUtil.h"
-#include "nsEffectiveTLDService.h"
-#include "mozilla/Attributes.h"
+#include "mozilla/RefPtr.h"
+#include "nsISupports.h"
+#include "nsString.h"
 
+class mozIDOMWindowProxy;
+class nsEffectiveTLDService;
+class nsIChannel;
+class nsIPrincipal;
 class nsIURI;
+class nsPIDOMWindowOuter;
+
+namespace mozilla {
+namespace dom {
+class WindowGlobalParent;
+}
+}  // namespace mozilla
 
 class ThirdPartyUtil final : public mozIThirdPartyUtil {
  public:
@@ -25,6 +37,9 @@ class ThirdPartyUtil final : public mozIThirdPartyUtil {
   static void Startup();
   static ThirdPartyUtil* GetInstance();
 
+  nsresult IsThirdPartyGlobal(mozilla::dom::WindowGlobalParent* aWindowGlobal,
+                              bool* aResult);
+
  private:
   ~ThirdPartyUtil();
 
@@ -36,15 +51,7 @@ class ThirdPartyUtil final : public mozIThirdPartyUtil {
   nsresult IsThirdPartyInternal(const nsCString& aFirstDomain,
                                 nsIURI* aSecondURI, bool* aResult);
 
-  nsCString GetBaseDomainFromWindow(nsPIDOMWindowOuter* aWindow) {
-    mozilla::dom::Document* doc = aWindow ? aWindow->GetExtantDoc() : nullptr;
-
-    if (!doc) {
-      return EmptyCString();
-    }
-
-    return doc->GetBaseDomain();
-  }
+  nsCString GetBaseDomainFromWindow(nsPIDOMWindowOuter* aWindow);
 
   RefPtr<nsEffectiveTLDService> mTLDService;
 };
