@@ -7,21 +7,22 @@
 #include "nsDOMNavigationTiming.h"
 
 #include "GeckoProfiler.h"
+#include "ipc/IPCMessageUtilsSpecializations.h"
+#include "mozilla/ProfilerMarkers.h"
+#include "mozilla/Telemetry.h"
+#include "mozilla/TimeStamp.h"
+#include "mozilla/dom/Document.h"
+#include "mozilla/dom/PerformanceNavigation.h"
+#include "mozilla/ipc/IPDLParamTraits.h"
+#include "mozilla/ipc/URIUtils.h"
 #include "nsCOMPtr.h"
 #include "nsContentUtils.h"
 #include "nsDocShell.h"
 #include "nsHttp.h"
-#include "nsIDocShellTreeItem.h"
 #include "nsIScriptSecurityManager.h"
-#include "prtime.h"
 #include "nsIURI.h"
 #include "nsPrintfCString.h"
-#include "mozilla/dom/PerformanceNavigation.h"
-#include "mozilla/TimeStamp.h"
-#include "mozilla/Telemetry.h"
-#ifdef MOZ_GECKO_PROFILER
-#  include "ProfilerMarkerPayload.h"
-#endif
+#include "prtime.h"
 
 using namespace mozilla;
 
@@ -39,7 +40,7 @@ nsDOMNavigationTiming::nsDOMNavigationTiming(nsDocShell* aDocShell) {
   mDocShell = aDocShell;
 }
 
-nsDOMNavigationTiming::~nsDOMNavigationTiming() {}
+nsDOMNavigationTiming::~nsDOMNavigationTiming() = default;
 
 void nsDOMNavigationTiming::Clear() {
   mNavigationType = TYPE_RESERVED;
@@ -61,6 +62,14 @@ void nsDOMNavigationTiming::Clear() {
   mDocShellHasBeenActiveSinceNavigationStart = false;
 }
 
+void nsDOMNavigationTiming::Anonymize(nsIURI* aFinalURI) {
+  mLoadedURI = aFinalURI;
+  mUnloadedURI = nullptr;
+  mBeforeUnloadStart = TimeStamp();
+  mUnloadStart = TimeStamp();
+  mUnloadEnd = TimeStamp();
+}
+
 DOMTimeMilliSec nsDOMNavigationTiming::TimeStampToDOM(TimeStamp aStamp) const {
   if (aStamp.IsNull()) {
     return 0;
@@ -76,7 +85,8 @@ void nsDOMNavigationTiming::NotifyNavigationStart(
   mNavigationStart = TimeStamp::Now();
   mDocShellHasBeenActiveSinceNavigationStart =
       (aDocShellState == DocShellState::eActive);
-  PROFILER_ADD_MARKER("Navigation::Start", DOM);
+  PROFILER_MARKER_UNTYPED("Navigation::Start", DOM,
+                          MarkerInnerWindowIdFromDocShell(mDocShell));
 }
 
 void nsDOMNavigationTiming::NotifyFetchStart(nsIURI* aURI,
@@ -87,6 +97,10 @@ void nsDOMNavigationTiming::NotifyFetchStart(nsIURI* aURI,
   mLoadedURI = aURI;
 }
 
+void nsDOMNavigationTiming::NotifyRestoreStart() {
+  mNavigationType = TYPE_BACK_FORWARD;
+}
+
 void nsDOMNavigationTiming::NotifyBeforeUnload() {
   mBeforeUnloadStart = TimeStamp::Now();
 }
@@ -98,14 +112,18 @@ void nsDOMNavigationTiming::NotifyUnloadAccepted(nsIURI* aOldURI) {
 
 void nsDOMNavigationTiming::NotifyUnloadEventStart() {
   mUnloadStart = TimeStamp::Now();
-  PROFILER_TRACING_DOCSHELL("Navigation", "Unload", NETWORK,
-                            TRACING_INTERVAL_START, mDocShell);
+  PROFILER_MARKER("Unload", NETWORK,
+                  MarkerOptions(MarkerTiming::IntervalStart(),
+                                MarkerInnerWindowIdFromDocShell(mDocShell)),
+                  Tracing, "Navigation");
 }
 
 void nsDOMNavigationTiming::NotifyUnloadEventEnd() {
   mUnloadEnd = TimeStamp::Now();
-  PROFILER_TRACING_DOCSHELL("Navigation", "Unload", NETWORK,
-                            TRACING_INTERVAL_END, mDocShell);
+  PROFILER_MARKER("Unload", NETWORK,
+                  MarkerOptions(MarkerTiming::IntervalEnd(),
+                                MarkerInnerWindowIdFromDocShell(mDocShell)),
+                  Tracing, "Navigation");
 }
 
 void nsDOMNavigationTiming::NotifyLoadEventStart() {
@@ -114,8 +132,10 @@ void nsDOMNavigationTiming::NotifyLoadEventStart() {
   }
   mLoadEventStart = TimeStamp::Now();
 
-  PROFILER_TRACING_DOCSHELL("Navigation", "Load", NETWORK,
-                            TRACING_INTERVAL_START, mDocShell);
+  PROFILER_MARKER("Load", NETWORK,
+                  MarkerOptions(MarkerTiming::IntervalStart(),
+                                MarkerInnerWindowIdFromDocShell(mDocShell)),
+                  Tracing, "Navigation");
 
   if (IsTopLevelContentDocumentInContentProcess()) {
     TimeStamp now = TimeStamp::Now();
@@ -144,12 +164,13 @@ void nsDOMNavigationTiming::NotifyLoadEventEnd() {
   }
   mLoadEventEnd = TimeStamp::Now();
 
-  PROFILER_TRACING_DOCSHELL("Navigation", "Load", NETWORK, TRACING_INTERVAL_END,
-                            mDocShell);
+  PROFILER_MARKER("Load", NETWORK,
+                  MarkerOptions(MarkerTiming::IntervalEnd(),
+                                MarkerInnerWindowIdFromDocShell(mDocShell)),
+                  Tracing, "Navigation");
 
   if (IsTopLevelContentDocumentInContentProcess()) {
-#ifdef MOZ_GECKO_PROFILER
-    if (profiler_is_active() || PAGELOAD_LOG_ENABLED()) {
+    if (profiler_can_accept_markers() || PAGELOAD_LOG_ENABLED()) {
       TimeDuration elapsed = mLoadEventEnd - mNavigationStart;
       TimeDuration duration = mLoadEventEnd - mLoadEventStart;
       nsAutoCString spec;
@@ -159,14 +180,13 @@ void nsDOMNavigationTiming::NotifyLoadEventEnd() {
       nsPrintfCString marker(
           "Document %s loaded after %dms, load event duration %dms", spec.get(),
           int(elapsed.ToMilliseconds()), int(duration.ToMilliseconds()));
-      DECLARE_DOCSHELL_AND_HISTORY_ID(mDocShell);
       PAGELOAD_LOG(("%s", marker.get()));
-      profiler_add_marker(
-          "DocumentLoad", JS::ProfilingCategoryPair::DOM,
-          MakeUnique<TextMarkerPayload>(marker, mNavigationStart, mLoadEventEnd,
-                                        docShellId, docShellHistoryId));
+      PROFILER_MARKER_TEXT(
+          "DocumentLoad", DOM,
+          MarkerOptions(MarkerTiming::Interval(mNavigationStart, mLoadEventEnd),
+                        MarkerInnerWindowIdFromDocShell(mDocShell)),
+          marker);
     }
-#endif
     Telemetry::AccumulateTimeDelta(Telemetry::TIME_TO_LOAD_EVENT_END_MS,
                                    mNavigationStart);
   }
@@ -188,7 +208,8 @@ void nsDOMNavigationTiming::NotifyDOMLoading(nsIURI* aURI) {
   mLoadedURI = aURI;
   mDOMLoading = TimeStamp::Now();
 
-  PROFILER_ADD_MARKER("Navigation::DOMLoading", DOM);
+  PROFILER_MARKER_UNTYPED("Navigation::DOMLoading", DOM,
+                          MarkerInnerWindowIdFromDocShell(mDocShell));
 }
 
 void nsDOMNavigationTiming::NotifyDOMInteractive(nsIURI* aURI) {
@@ -198,7 +219,8 @@ void nsDOMNavigationTiming::NotifyDOMInteractive(nsIURI* aURI) {
   mLoadedURI = aURI;
   mDOMInteractive = TimeStamp::Now();
 
-  PROFILER_ADD_MARKER("Navigation::DOMInteractive", DOM);
+  PROFILER_MARKER_UNTYPED("Navigation::DOMInteractive", DOM,
+                          MarkerInnerWindowIdFromDocShell(mDocShell));
 }
 
 void nsDOMNavigationTiming::NotifyDOMComplete(nsIURI* aURI) {
@@ -208,7 +230,8 @@ void nsDOMNavigationTiming::NotifyDOMComplete(nsIURI* aURI) {
   mLoadedURI = aURI;
   mDOMComplete = TimeStamp::Now();
 
-  PROFILER_ADD_MARKER("Navigation::DOMComplete", DOM);
+  PROFILER_MARKER_UNTYPED("Navigation::DOMComplete", DOM,
+                          MarkerInnerWindowIdFromDocShell(mDocShell));
 }
 
 void nsDOMNavigationTiming::NotifyDOMContentLoadedStart(nsIURI* aURI) {
@@ -219,8 +242,10 @@ void nsDOMNavigationTiming::NotifyDOMContentLoadedStart(nsIURI* aURI) {
   mLoadedURI = aURI;
   mDOMContentLoadedEventStart = TimeStamp::Now();
 
-  PROFILER_TRACING_DOCSHELL("Navigation", "DOMContentLoaded", NETWORK,
-                            TRACING_INTERVAL_START, mDocShell);
+  PROFILER_MARKER("DOMContentLoaded", NETWORK,
+                  MarkerOptions(MarkerTiming::IntervalStart(),
+                                MarkerInnerWindowIdFromDocShell(mDocShell)),
+                  Tracing, "Navigation");
 
   if (IsTopLevelContentDocumentInContentProcess()) {
     TimeStamp now = TimeStamp::Now();
@@ -251,8 +276,10 @@ void nsDOMNavigationTiming::NotifyDOMContentLoadedEnd(nsIURI* aURI) {
   mLoadedURI = aURI;
   mDOMContentLoadedEventEnd = TimeStamp::Now();
 
-  PROFILER_TRACING_DOCSHELL("Navigation", "DOMContentLoaded", NETWORK,
-                            TRACING_INTERVAL_END, mDocShell);
+  PROFILER_MARKER("DOMContentLoaded", NETWORK,
+                  MarkerOptions(MarkerTiming::IntervalEnd(),
+                                MarkerInnerWindowIdFromDocShell(mDocShell)),
+                  Tracing, "Navigation");
 
   if (IsTopLevelContentDocumentInContentProcess()) {
     Telemetry::AccumulateTimeDelta(Telemetry::TIME_TO_DOM_CONTENT_LOADED_END_MS,
@@ -341,8 +368,7 @@ void nsDOMNavigationTiming::TTITimeout(nsITimer* aTimer) {
 
   mTTITimer = nullptr;
 
-#ifdef MOZ_GECKO_PROFILER
-  if (profiler_is_active() || PAGELOAD_LOG_ENABLED()) {
+  if (profiler_can_accept_markers() || PAGELOAD_LOG_ENABLED()) {
     TimeDuration elapsed = mTTFI - mNavigationStart;
     MOZ_ASSERT(elapsed.ToMilliseconds() > 0);
     TimeDuration elapsedLongTask =
@@ -355,14 +381,12 @@ void nsDOMNavigationTiming::TTITimeout(nsITimer* aTimer) {
                            int(elapsed.ToMilliseconds()),
                            int(elapsedLongTask.ToMilliseconds()), spec.get());
 
-    DECLARE_DOCSHELL_AND_HISTORY_ID(mDocShell);
-    profiler_add_marker(
-        "TTFI", JS::ProfilingCategoryPair::DOM,
-        MakeUnique<TextMarkerPayload>(marker, mNavigationStart, mTTFI,
-                                      docShellId, docShellHistoryId));
+    PROFILER_MARKER_TEXT(
+        "TimeToFirstInteractive (TTFI)", DOM,
+        MarkerOptions(MarkerTiming::Interval(mNavigationStart, mTTFI),
+                      MarkerInnerWindowIdFromDocShell(mDocShell)),
+        marker);
   }
-#endif
-  return;
 }
 
 void nsDOMNavigationTiming::NotifyNonBlankPaintForRootContentDocument() {
@@ -375,7 +399,6 @@ void nsDOMNavigationTiming::NotifyNonBlankPaintForRootContentDocument() {
 
   mNonBlankPaint = TimeStamp::Now();
 
-#ifdef MOZ_GECKO_PROFILER
   if (profiler_thread_is_being_profiled() || PAGELOAD_LOG_ENABLED()) {
     TimeDuration elapsed = mNonBlankPaint - mNavigationStart;
     nsAutoCString spec;
@@ -390,13 +413,12 @@ void nsDOMNavigationTiming::NotifyNonBlankPaintForRootContentDocument() {
             : "this tab was inactive some of the time between navigation start "
               "and first non-blank paint");
     PAGELOAD_LOG(("%s", marker.get()));
-    DECLARE_DOCSHELL_AND_HISTORY_ID(mDocShell);
-    profiler_add_marker(
-        "FirstNonBlankPaint", JS::ProfilingCategoryPair::DOM,
-        MakeUnique<TextMarkerPayload>(marker, mNavigationStart, mNonBlankPaint,
-                                      docShellId, docShellHistoryId));
+    PROFILER_MARKER_TEXT(
+        "FirstNonBlankPaint", DOM,
+        MarkerOptions(MarkerTiming::Interval(mNavigationStart, mNonBlankPaint),
+                      MarkerInnerWindowIdFromDocShell(mDocShell)),
+        marker);
   }
-#endif
 
   if (mDocShellHasBeenActiveSinceNavigationStart) {
     if (net::nsHttp::IsBeforeLastActiveTabLoadOptimization(mNavigationStart)) {
@@ -425,8 +447,7 @@ void nsDOMNavigationTiming::NotifyContentfulPaintForRootContentDocument(
 
   mContentfulPaint = aCompositeEndTime;
 
-#ifdef MOZ_GECKO_PROFILER
-  if (profiler_is_active() || PAGELOAD_LOG_ENABLED()) {
+  if (profiler_can_accept_markers() || PAGELOAD_LOG_ENABLED()) {
     TimeDuration elapsed = mContentfulPaint - mNavigationStart;
     nsAutoCString spec;
     if (mLoadedURI) {
@@ -439,14 +460,14 @@ void nsDOMNavigationTiming::NotifyContentfulPaintForRootContentDocument(
             ? "foreground tab"
             : "this tab was inactive some of the time between navigation start "
               "and first non-blank paint");
-    DECLARE_DOCSHELL_AND_HISTORY_ID(mDocShell);
     PAGELOAD_LOG(("%s", marker.get()));
-    profiler_add_marker("FirstContentfulPaint", JS::ProfilingCategoryPair::DOM,
-                        MakeUnique<TextMarkerPayload>(
-                            marker, mNavigationStart, mContentfulPaint,
-                            docShellId, docShellHistoryId));
+    PROFILER_MARKER_TEXT(
+        "FirstContentfulPaint", DOM,
+        MarkerOptions(
+            MarkerTiming::Interval(mNavigationStart, mContentfulPaint),
+            MarkerInnerWindowIdFromDocShell(mDocShell)),
+        marker);
   }
-#endif
 
   if (!mTTITimer) {
     mTTITimer = NS_NewTimer();
@@ -475,7 +496,6 @@ void nsDOMNavigationTiming::NotifyDOMContentFlushedForRootContentDocument() {
 
   mDOMContentFlushed = TimeStamp::Now();
 
-#ifdef MOZ_GECKO_PROFILER
   if (profiler_thread_is_being_profiled() || PAGELOAD_LOG_ENABLED()) {
     TimeDuration elapsed = mDOMContentFlushed - mNavigationStart;
     nsAutoCString spec;
@@ -489,14 +509,14 @@ void nsDOMNavigationTiming::NotifyDOMContentFlushedForRootContentDocument() {
             ? "foreground tab"
             : "this tab was inactive some of the time between navigation start "
               "and DOMContentFlushed");
-    DECLARE_DOCSHELL_AND_HISTORY_ID(mDocShell);
     PAGELOAD_LOG(("%s", marker.get()));
-    profiler_add_marker("DOMContentFlushed", JS::ProfilingCategoryPair::DOM,
-                        MakeUnique<TextMarkerPayload>(
-                            marker, mNavigationStart, mDOMContentFlushed,
-                            docShellId, docShellHistoryId));
+    PROFILER_MARKER_TEXT(
+        "DOMContentFlushed", DOM,
+        MarkerOptions(
+            MarkerTiming::Interval(mNavigationStart, mDOMContentFlushed),
+            MarkerInnerWindowIdFromDocShell(mDocShell)),
+        marker);
   }
-#endif
 }
 
 void nsDOMNavigationTiming::NotifyDocShellStateChanged(
@@ -534,10 +554,101 @@ bool nsDOMNavigationTiming::IsTopLevelContentDocumentInContentProcess() const {
   if (!XRE_IsContentProcess()) {
     return false;
   }
-  nsCOMPtr<nsIDocShellTreeItem> rootItem;
-  Unused << mDocShell->GetSameTypeRootTreeItem(getter_AddRefs(rootItem));
-  if (rootItem.get() != static_cast<nsIDocShellTreeItem*>(mDocShell.get())) {
+  return mDocShell->GetBrowsingContext()->IsTopContent();
+}
+
+nsDOMNavigationTiming::nsDOMNavigationTiming(nsDocShell* aDocShell,
+                                             nsDOMNavigationTiming* aOther)
+    : mDocShell(aDocShell),
+      mUnloadedURI(aOther->mUnloadedURI),
+      mLoadedURI(aOther->mLoadedURI),
+      mNavigationType(aOther->mNavigationType),
+      mNavigationStartHighRes(aOther->mNavigationStartHighRes),
+      mNavigationStart(aOther->mNavigationStart),
+      mNonBlankPaint(aOther->mNonBlankPaint),
+      mContentfulPaint(aOther->mContentfulPaint),
+      mDOMContentFlushed(aOther->mDOMContentFlushed),
+      mBeforeUnloadStart(aOther->mBeforeUnloadStart),
+      mUnloadStart(aOther->mUnloadStart),
+      mUnloadEnd(aOther->mUnloadEnd),
+      mLoadEventStart(aOther->mLoadEventStart),
+      mLoadEventEnd(aOther->mLoadEventEnd),
+      mDOMLoading(aOther->mDOMLoading),
+      mDOMInteractive(aOther->mDOMInteractive),
+      mDOMContentLoadedEventStart(aOther->mDOMContentLoadedEventStart),
+      mDOMContentLoadedEventEnd(aOther->mDOMContentLoadedEventEnd),
+      mDOMComplete(aOther->mDOMComplete),
+      mTTFI(aOther->mTTFI),
+      mDocShellHasBeenActiveSinceNavigationStart(
+          aOther->mDocShellHasBeenActiveSinceNavigationStart) {}
+
+/* static */
+void mozilla::ipc::IPDLParamTraits<nsDOMNavigationTiming*>::Write(
+    IPC::Message* aMsg, IProtocol* aActor, nsDOMNavigationTiming* aParam) {
+  RefPtr<nsIURI> unloadedURI = aParam->mUnloadedURI.get();
+  RefPtr<nsIURI> loadedURI = aParam->mLoadedURI.get();
+  WriteIPDLParam(aMsg, aActor, unloadedURI ? Some(unloadedURI) : Nothing());
+  WriteIPDLParam(aMsg, aActor, loadedURI ? Some(loadedURI) : Nothing());
+  WriteIPDLParam(aMsg, aActor, uint32_t(aParam->mNavigationType));
+  WriteIPDLParam(aMsg, aActor, aParam->mNavigationStartHighRes);
+  WriteIPDLParam(aMsg, aActor, aParam->mNavigationStart);
+  WriteIPDLParam(aMsg, aActor, aParam->mNonBlankPaint);
+  WriteIPDLParam(aMsg, aActor, aParam->mContentfulPaint);
+  WriteIPDLParam(aMsg, aActor, aParam->mDOMContentFlushed);
+  WriteIPDLParam(aMsg, aActor, aParam->mBeforeUnloadStart);
+  WriteIPDLParam(aMsg, aActor, aParam->mUnloadStart);
+  WriteIPDLParam(aMsg, aActor, aParam->mUnloadEnd);
+  WriteIPDLParam(aMsg, aActor, aParam->mLoadEventStart);
+  WriteIPDLParam(aMsg, aActor, aParam->mLoadEventEnd);
+  WriteIPDLParam(aMsg, aActor, aParam->mDOMLoading);
+  WriteIPDLParam(aMsg, aActor, aParam->mDOMInteractive);
+  WriteIPDLParam(aMsg, aActor, aParam->mDOMContentLoadedEventStart);
+  WriteIPDLParam(aMsg, aActor, aParam->mDOMContentLoadedEventEnd);
+  WriteIPDLParam(aMsg, aActor, aParam->mDOMComplete);
+  WriteIPDLParam(aMsg, aActor, aParam->mTTFI);
+  WriteIPDLParam(aMsg, aActor,
+                 aParam->mDocShellHasBeenActiveSinceNavigationStart);
+}
+
+/* static */
+bool mozilla::ipc::IPDLParamTraits<nsDOMNavigationTiming*>::Read(
+    const IPC::Message* aMsg, PickleIterator* aIter, IProtocol* aActor,
+    RefPtr<nsDOMNavigationTiming>* aResult) {
+  auto timing = MakeRefPtr<nsDOMNavigationTiming>(nullptr);
+  uint32_t type;
+  Maybe<RefPtr<nsIURI>> unloadedURI;
+  Maybe<RefPtr<nsIURI>> loadedURI;
+  if (!ReadIPDLParam(aMsg, aIter, aActor, &unloadedURI) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &loadedURI) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &type) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &timing->mNavigationStartHighRes) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &timing->mNavigationStart) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &timing->mNonBlankPaint) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &timing->mContentfulPaint) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &timing->mDOMContentFlushed) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &timing->mBeforeUnloadStart) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &timing->mUnloadStart) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &timing->mUnloadEnd) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &timing->mLoadEventStart) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &timing->mLoadEventEnd) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &timing->mDOMLoading) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &timing->mDOMInteractive) ||
+      !ReadIPDLParam(aMsg, aIter, aActor,
+                     &timing->mDOMContentLoadedEventStart) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &timing->mDOMContentLoadedEventEnd) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &timing->mDOMComplete) ||
+      !ReadIPDLParam(aMsg, aIter, aActor, &timing->mTTFI) ||
+      !ReadIPDLParam(aMsg, aIter, aActor,
+                     &timing->mDocShellHasBeenActiveSinceNavigationStart)) {
     return false;
   }
-  return rootItem->ItemType() == nsIDocShellTreeItem::typeContent;
+  timing->mNavigationType = nsDOMNavigationTiming::Type(type);
+  if (unloadedURI) {
+    timing->mUnloadedURI = std::move(*unloadedURI);
+  }
+  if (loadedURI) {
+    timing->mLoadedURI = std::move(*loadedURI);
+  }
+  *aResult = std::move(timing);
+  return true;
 }
