@@ -7,7 +7,7 @@
 #define _mozilla_dom_BindingIPCUtils_h
 
 #include "mozilla/dom/BindingDeclarations.h"
-#include "ipc/IPCMessageUtils.h"
+#include "ipc/EnumSerializer.h"
 
 namespace IPC {
 template <>
@@ -16,4 +16,4 @@ struct ParamTraits<mozilla::dom::CallerType>
           mozilla::dom::CallerType, mozilla::dom::CallerType::System,
           mozilla::dom::CallerType::NonSystem> {};
 }  // namespace IPC
-#endif  // _mozilla_dom_BindingIPCUtils_h
\ No newline at end of file
+#endif  // _mozilla_dom_BindingIPCUtils_h
