@@ -92,48 +92,52 @@ class WebIDLCodegenManagerState(dict):
     VERSION = 3
 
     def __init__(self, fh=None):
-        self['version'] = self.VERSION
-        self['webidls'] = {}
-        self['global_depends'] = {}
+        self["version"] = self.VERSION
+        self["webidls"] = {}
+        self["global_depends"] = {}
 
         if not fh:
             return
 
         state = json.load(fh)
-        if state['version'] != self.VERSION:
-            raise Exception('Unknown state version: %s' % state['version'])
+        if state["version"] != self.VERSION:
+            raise Exception("Unknown state version: %s" % state["version"])
 
-        self['version'] = state['version']
-        self['global_depends'] = state['global_depends']
+        self["version"] = state["version"]
+        self["global_depends"] = state["global_depends"]
 
-        for k, v in state['webidls'].items():
-            self['webidls'][k] = v
+        for k, v in state["webidls"].items():
+            self["webidls"][k] = v
 
             # Sets are converted to lists for serialization because JSON
             # doesn't support sets.
-            self['webidls'][k]['inputs'] = set(v['inputs'])
-            self['webidls'][k]['outputs'] = set(v['outputs'])
+            self["webidls"][k]["inputs"] = set(v["inputs"])
+            self["webidls"][k]["outputs"] = set(v["outputs"])
 
-        self['dictionaries_convertible_to_js'] = set(
-            state['dictionaries_convertible_to_js'])
+        self["dictionaries_convertible_to_js"] = set(
+            state["dictionaries_convertible_to_js"]
+        )
 
-        self['dictionaries_convertible_from_js'] = set(
-            state['dictionaries_convertible_from_js'])
+        self["dictionaries_convertible_from_js"] = set(
+            state["dictionaries_convertible_from_js"]
+        )
 
     def dump(self, fh):
         """Dump serialized state to a file handle."""
         normalized = deepcopy(self)
 
-        for k, v in self['webidls'].items():
+        for k, v in self["webidls"].items():
             # Convert sets to lists because JSON doesn't support sets.
-            normalized['webidls'][k]['outputs'] = sorted(v['outputs'])
-            normalized['webidls'][k]['inputs'] = sorted(v['inputs'])
+            normalized["webidls"][k]["outputs"] = sorted(v["outputs"])
+            normalized["webidls"][k]["inputs"] = sorted(v["inputs"])
 
-        normalized['dictionaries_convertible_to_js'] = sorted(
-            self['dictionaries_convertible_to_js'])
+        normalized["dictionaries_convertible_to_js"] = sorted(
+            self["dictionaries_convertible_to_js"]
+        )
 
-        normalized['dictionaries_convertible_from_js'] = sorted(
-            self['dictionaries_convertible_from_js'])
+        normalized["dictionaries_convertible_from_js"] = sorted(
+            self["dictionaries_convertible_from_js"]
+        )
 
         json.dump(normalized, fh, sort_keys=True)
 
@@ -147,34 +151,43 @@ class WebIDLCodegenManager(LoggingMixin):
 
     # Global parser derived declaration files.
     GLOBAL_DECLARE_FILES = {
-        'GeneratedAtomList.h',
-        'GeneratedEventList.h',
-        'PrototypeList.h',
-        'RegisterBindings.h',
-        'RegisterWorkerBindings.h',
-        'RegisterWorkerDebuggerBindings.h',
-        'RegisterWorkletBindings.h',
-        'UnionConversions.h',
-        'UnionTypes.h',
-        'WebIDLPrefs.h',
-        'WebIDLSerializable.h',
+        "GeneratedAtomList.h",
+        "GeneratedEventList.h",
+        "PrototypeList.h",
+        "RegisterBindings.h",
+        "RegisterWorkerBindings.h",
+        "RegisterWorkerDebuggerBindings.h",
+        "RegisterWorkletBindings.h",
+        "UnionConversions.h",
+        "UnionTypes.h",
+        "WebIDLPrefs.h",
+        "WebIDLSerializable.h",
     }
 
     # Global parser derived definition files.
     GLOBAL_DEFINE_FILES = {
-        'RegisterBindings.cpp',
-        'RegisterWorkerBindings.cpp',
-        'RegisterWorkerDebuggerBindings.cpp',
-        'RegisterWorkletBindings.cpp',
-        'UnionTypes.cpp',
-        'PrototypeList.cpp',
-        'WebIDLPrefs.cpp',
-        'WebIDLSerializable.cpp',
+        "RegisterBindings.cpp",
+        "RegisterWorkerBindings.cpp",
+        "RegisterWorkerDebuggerBindings.cpp",
+        "RegisterWorkletBindings.cpp",
+        "UnionTypes.cpp",
+        "PrototypeList.cpp",
+        "WebIDLPrefs.cpp",
+        "WebIDLSerializable.cpp",
     }
 
-    def __init__(self, config_path, webidl_root, inputs, exported_header_dir,
-                 codegen_dir, state_path, cache_dir=None, make_deps_path=None,
-                 make_deps_target=None):
+    def __init__(
+        self,
+        config_path,
+        webidl_root,
+        inputs,
+        exported_header_dir,
+        codegen_dir,
+        state_path,
+        cache_dir=None,
+        make_deps_path=None,
+        make_deps_target=None,
+    ):
         """Create an instance that manages WebIDLs in the build system.
 
         config_path refers to a WebIDL config file (e.g. Bindings.conf).
@@ -211,22 +224,29 @@ class WebIDLCodegenManager(LoggingMixin):
         self._make_deps_path = make_deps_path
         self._make_deps_target = make_deps_target
 
-        if ((make_deps_path and not make_deps_target) or
-                (not make_deps_path and make_deps_target)):
-            raise Exception('Must define both make_deps_path and make_deps_target '
-                            'if one is defined.')
+        if (make_deps_path and not make_deps_target) or (
+            not make_deps_path and make_deps_target
+        ):
+            raise Exception(
+                "Must define both make_deps_path and make_deps_target "
+                "if one is defined."
+            )
 
         self._parser_results = None
         self._config = None
         self._state = WebIDLCodegenManagerState()
 
         if os.path.exists(state_path):
-            with io.open(state_path, 'r') as fh:
+            with io.open(state_path, "r") as fh:
                 try:
                     self._state = WebIDLCodegenManagerState(fh=fh)
                 except Exception as e:
-                    self.log(logging.WARN, 'webidl_bad_state', {'msg': str(e)},
-                                           'Bad WebIDL state: {msg}')
+                    self.log(
+                        logging.WARN,
+                        "webidl_bad_state",
+                        {"msg": str(e)},
+                        "Bad WebIDL state: {msg}",
+                    )
 
     @property
     def config(self):
@@ -287,11 +307,13 @@ class WebIDLCodegenManager(LoggingMixin):
         else:
             changed_inputs = self._compute_changed_inputs()
 
-        self._state['global_depends'] = global_hashes
-        self._state['dictionaries_convertible_to_js'] = set(
-            d.identifier.name for d in self._config.getDictionariesConvertibleToJS())
-        self._state['dictionaries_convertible_from_js'] = set(
-            d.identifier.name for d in self._config.getDictionariesConvertibleFromJS())
+        self._state["global_depends"] = global_hashes
+        self._state["dictionaries_convertible_to_js"] = set(
+            d.identifier.name for d in self._config.getDictionariesConvertibleToJS()
+        )
+        self._state["dictionaries_convertible_from_js"] = set(
+            d.identifier.name for d in self._config.getDictionariesConvertibleFromJS()
+        )
 
         # Generate bindings from .webidl files.
         for filename in sorted(changed_inputs):
@@ -302,7 +324,7 @@ class WebIDLCodegenManager(LoggingMixin):
             result.updated |= written[1]
             result.unchanged |= written[2]
 
-            self._state['webidls'][basename] = dict(
+            self._state["webidls"][basename] = dict(
                 filename=filename,
                 outputs=written[0] | written[1] | written[2],
                 inputs=set(deps),
@@ -320,10 +342,10 @@ class WebIDLCodegenManager(LoggingMixin):
         if self._make_deps_path:
             mk = Makefile()
             codegen_rule = mk.create_rule([self._make_deps_target])
-            codegen_rule.add_dependencies(six.ensure_text(s) for s in
-                                          global_hashes.keys())
-            codegen_rule.add_dependencies(six.ensure_text(p) for p in
-                                          self._input_paths)
+            codegen_rule.add_dependencies(
+                six.ensure_text(s) for s in global_hashes.keys()
+            )
+            codegen_rule.add_dependencies(six.ensure_text(p) for p in self._input_paths)
 
             with FileAvoidWrite(self._make_deps_path) as fh:
                 mk.dump(fh)
@@ -348,15 +370,18 @@ class WebIDLCodegenManager(LoggingMixin):
         import WebIDL
         from Configuration import Configuration
 
-        self.log(logging.INFO, 'webidl_parse',
-                 {'count': len(self._input_paths)},
-                 'Parsing {count} WebIDL files.')
+        self.log(
+            logging.INFO,
+            "webidl_parse",
+            {"count": len(self._input_paths)},
+            "Parsing {count} WebIDL files.",
+        )
 
         hashes = {}
         parser = WebIDL.Parser(self._cache_dir)
 
         for path in sorted(self._input_paths):
-            with io.open(path, 'r', encoding='utf-8') as fh:
+            with io.open(path, "r", encoding="utf-8") as fh:
                 data = fh.read()
                 hashes[path] = hashlib.sha1(six.ensure_binary(data)).hexdigest()
                 parser.parse(data, path)
@@ -379,16 +404,19 @@ class WebIDLCodegenManager(LoggingMixin):
         )
 
         self._parser_results = parser.finish()
-        self._config = Configuration(self._config_path, web_roots,
-                                     self._parser_results,
-                                     self._generated_events_stems_as_array)
+        self._config = Configuration(
+            self._config_path,
+            web_roots,
+            self._parser_results,
+            self._generated_events_stems_as_array,
+        )
         self._input_hashes = hashes
 
     def _write_global_derived(self):
         from Codegen import GlobalGenRoots
 
-        things = [('declare', f) for f in self.GLOBAL_DECLARE_FILES]
-        things.extend(('define', f) for f in self.GLOBAL_DEFINE_FILES)
+        things = [("declare", f) for f in self.GLOBAL_DECLARE_FILES]
+        things.extend(("define", f) for f in self.GLOBAL_DEFINE_FILES)
 
         result = (set(), set(), set())
 
@@ -396,14 +424,14 @@ class WebIDLCodegenManager(LoggingMixin):
             stem = mozpath.splitext(filename)[0]
             root = getattr(GlobalGenRoots, stem)(self._config)
 
-            if what == 'declare':
+            if what == "declare":
                 code = root.declare()
                 output_root = self._exported_header_dir
-            elif what == 'define':
+            elif what == "define":
                 code = root.define()
                 output_root = self._codegen_dir
             else:
-                raise Exception('Unknown global gen type: %s' % what)
+                raise Exception("Unknown global gen type: %s" % what)
 
             output_path = mozpath.join(output_root, filename)
             self._maybe_write_file(output_path, code, result)
@@ -425,8 +453,7 @@ class WebIDLCodegenManager(LoggingMixin):
         # a lot of extra work and most build systems don't do that anyway.
 
         # Now we move on to the input files.
-        old_hashes = {v['filename']: v['sha1']
-                      for v in self._state['webidls'].values()}
+        old_hashes = {v["filename"]: v["sha1"] for v in self._state["webidls"].values()}
 
         old_filenames = set(old_hashes.keys())
         new_filenames = self._input_paths
@@ -447,19 +474,23 @@ class WebIDLCodegenManager(LoggingMixin):
         # Inherit dependencies from previous run. The full set of dependencies
         # is associated with each record, so we don't need to perform any fancy
         # graph traversal.
-        for v in self._state['webidls'].values():
-            if any(dep for dep in v['inputs'] if dep in changed_inputs):
-                changed_inputs.add(v['filename'])
+        for v in self._state["webidls"].values():
+            if any(dep for dep in v["inputs"] if dep in changed_inputs):
+                changed_inputs.add(v["filename"])
 
         # Now check for changes to the set of dictionaries that are convertible to JS
-        oldDictionariesConvertibleToJS = self._state['dictionaries_convertible_to_js']
+        oldDictionariesConvertibleToJS = self._state["dictionaries_convertible_to_js"]
         newDictionariesConvertibleToJS = self._config.getDictionariesConvertibleToJS()
         newNames = set(d.identifier.name for d in newDictionariesConvertibleToJS)
         changedDictionaryNames = oldDictionariesConvertibleToJS ^ newNames
 
         # Now check for changes to the set of dictionaries that are convertible from JS
-        oldDictionariesConvertibleFromJS = self._state['dictionaries_convertible_from_js']
-        newDictionariesConvertibleFromJS = self._config.getDictionariesConvertibleFromJS()
+        oldDictionariesConvertibleFromJS = self._state[
+            "dictionaries_convertible_from_js"
+        ]
+        newDictionariesConvertibleFromJS = (
+            self._config.getDictionariesConvertibleFromJS()
+        )
         newNames = set(d.identifier.name for d in newDictionariesConvertibleFromJS)
         changedDictionaryNames |= oldDictionariesConvertibleFromJS ^ newNames
 
@@ -486,7 +517,7 @@ class WebIDLCodegenManager(LoggingMixin):
         """
         basename = mozpath.basename(p)
         stem = mozpath.splitext(basename)[0]
-        binding_stem = '%sBinding' % stem
+        binding_stem = "%sBinding" % stem
 
         if stem in self._exported_stems:
             header_dir = self._exported_header_dir
@@ -496,18 +527,19 @@ class WebIDLCodegenManager(LoggingMixin):
         is_event = stem in self._generated_events_stems
 
         files = (
-            mozpath.join(header_dir, '%s.h' % binding_stem),
-            mozpath.join(self._codegen_dir, '%s.cpp' % binding_stem),
-            mozpath.join(header_dir, '%s.h' % stem) if is_event else None,
-            mozpath.join(self._codegen_dir, '%s.cpp' % stem) if is_event else None,
+            mozpath.join(header_dir, "%s.h" % binding_stem),
+            mozpath.join(self._codegen_dir, "%s.cpp" % binding_stem),
+            mozpath.join(header_dir, "%s.h" % stem) if is_event else None,
+            mozpath.join(self._codegen_dir, "%s.cpp" % stem) if is_event else None,
         )
 
         return stem, binding_stem, is_event, header_dir, files
 
     def _example_paths(self, interface):
         return (
-            mozpath.join(self._codegen_dir, '%s-example.h' % interface),
-            mozpath.join(self._codegen_dir, '%s-example.cpp' % interface))
+            mozpath.join(self._codegen_dir, "%s-example.h" % interface),
+            mozpath.join(self._codegen_dir, "%s-example.cpp" % interface),
+        )
 
     def expected_build_output_files(self):
         """Obtain the set of files generate_build_files() should write."""
@@ -535,9 +567,12 @@ class WebIDLCodegenManager(LoggingMixin):
             CGEventRoot,
         )
 
-        self.log(logging.INFO, 'webidl_generate_build_for_input',
-                 {'filename': filename},
-                 'Generating WebIDL files derived from {filename}')
+        self.log(
+            logging.INFO,
+            "webidl_generate_build_for_input",
+            {"filename": filename},
+            "Generating WebIDL files derived from {filename}",
+        )
 
         stem, binding_stem, is_event, header_dir, files = self._binding_info(filename)
         root = CGBindingRoot(self._config, binding_stem, filename)
@@ -546,8 +581,9 @@ class WebIDLCodegenManager(LoggingMixin):
 
         if is_event:
             generated_event = CGEventRoot(self._config, stem)
-            result = self._maybe_write_codegen(generated_event, files[2],
-                                               files[3], result)
+            result = self._maybe_write_codegen(
+                generated_event, files[2], files[3], result
+            )
 
         return result, root.deps()
 
@@ -565,22 +601,22 @@ class WebIDLCodegenManager(LoggingMixin):
         for f in current_files:
             # This will fail if the file doesn't exist. If a current global
             # dependency doesn't exist, something else is wrong.
-            with io.open(f, 'rb') as fh:
+            with io.open(f, "rb") as fh:
                 current_hashes[f] = hashlib.sha1(fh.read()).hexdigest()
 
         # The set of files has changed.
-        if current_files ^ set(self._state['global_depends'].keys()):
+        if current_files ^ set(self._state["global_depends"].keys()):
             return True, current_hashes
 
         # Compare hashes.
         for f, sha1 in current_hashes.items():
-            if sha1 != self._state['global_depends'][f]:
+            if sha1 != self._state["global_depends"][f]:
                 return True, current_hashes
 
         return False, current_hashes
 
     def _save_state(self):
-        with io.open(self._state_path, 'w', newline='\n') as fh:
+        with io.open(self._state_path, "w", newline="\n") as fh:
             self._state.dump(fh)
 
     def _maybe_write_codegen(self, obj, declare_path, define_path, result=None):
@@ -608,17 +644,21 @@ class WebIDLCodegenManager(LoggingMixin):
 
 def create_build_system_manager(topsrcdir, topobjdir, dist_dir):
     """Create a WebIDLCodegenManager for use by the build system."""
-    src_dir = os.path.join(topsrcdir, 'dom', 'bindings')
-    obj_dir = os.path.join(topobjdir, 'dom', 'bindings')
-    webidl_root = os.path.join(topsrcdir, 'dom', 'webidl')
+    src_dir = os.path.join(topsrcdir, "dom", "bindings")
+    obj_dir = os.path.join(topobjdir, "dom", "bindings")
+    webidl_root = os.path.join(topsrcdir, "dom", "webidl")
 
-    with io.open(os.path.join(obj_dir, 'file-lists.json'), 'r') as fh:
+    with io.open(os.path.join(obj_dir, "file-lists.json"), "r") as fh:
         files = json.load(fh)
 
-    inputs = (files['webidls'], files['exported_stems'],
-              files['generated_events_stems'], files['example_interfaces'])
+    inputs = (
+        files["webidls"],
+        files["exported_stems"],
+        files["generated_events_stems"],
+        files["example_interfaces"],
+    )
 
-    cache_dir = os.path.join(obj_dir, '_cache')
+    cache_dir = os.path.join(obj_dir, "_cache")
     try:
         os.makedirs(cache_dir)
     except OSError as e:
@@ -626,24 +666,25 @@ def create_build_system_manager(topsrcdir, topobjdir, dist_dir):
             raise
 
     return WebIDLCodegenManager(
-        os.path.join(src_dir, 'Bindings.conf'),
+        os.path.join(src_dir, "Bindings.conf"),
         webidl_root,
         inputs,
-        os.path.join(dist_dir, 'include', 'mozilla', 'dom'),
+        os.path.join(dist_dir, "include", "mozilla", "dom"),
         obj_dir,
-        os.path.join(obj_dir, 'codegen.json'),
+        os.path.join(obj_dir, "codegen.json"),
         cache_dir=cache_dir,
         # The make rules include a codegen.pp file containing dependencies.
-        make_deps_path=os.path.join(obj_dir, 'codegen.pp'),
-        make_deps_target='webidl.stub',
+        make_deps_path=os.path.join(obj_dir, "codegen.pp"),
+        make_deps_target="webidl.stub",
     )
 
 
 class BuildSystemWebIDL(MozbuildObject):
     @property
     def manager(self):
-        if not hasattr(self, '_webidl_manager'):
+        if not hasattr(self, "_webidl_manager"):
             self._webidl_manager = create_build_system_manager(
-                self.topsrcdir, self.topobjdir, self.distdir)
+                self.topsrcdir, self.topobjdir, self.distdir
+            )
 
         return self._webidl_manager
