@@ -1,33 +1,42 @@
 import WebIDL
 import traceback
-def WebIDLTest(parser, harness):
 
+
+def WebIDLTest(parser, harness):
     def shouldPass(prefix, iface, expectedMembers, numProductions=1):
         p = parser.reset()
         p.parse(iface)
         results = p.finish()
-        harness.check(len(results), numProductions,
-                      "%s - Should have production count %d" % (prefix, numProductions))
-        harness.ok(isinstance(results[0], WebIDL.IDLInterface),
-                   "%s - Should be an IDLInterface" % (prefix))
+        harness.check(
+            len(results),
+            numProductions,
+            "%s - Should have production count %d" % (prefix, numProductions),
+        )
+        harness.ok(
+            isinstance(results[0], WebIDL.IDLInterface),
+            "%s - Should be an IDLInterface" % (prefix),
+        )
         # Make a copy, since we plan to modify it
         expectedMembers = list(expectedMembers)
         for m in results[0].members:
             name = m.identifier.name
             if (name, type(m)) in expectedMembers:
-                harness.ok(True, "%s - %s - Should be a %s" % (prefix, name,
-                                                               type(m)))
+                harness.ok(True, "%s - %s - Should be a %s" % (prefix, name, type(m)))
                 expectedMembers.remove((name, type(m)))
             else:
-                harness.ok(False, "%s - %s - Unknown symbol of type %s" %
-                           (prefix, name, type(m)))
+                harness.ok(
+                    False,
+                    "%s - %s - Unknown symbol of type %s" % (prefix, name, type(m)),
+                )
         # A bit of a hoop because we can't generate the error string if we pass
         if len(expectedMembers) == 0:
             harness.ok(True, "Found all the members")
         else:
-            harness.ok(False,
-                       "Expected member not found: %s of type %s" %
-                       (expectedMembers[0][0], expectedMembers[0][1]))
+            harness.ok(
+                False,
+                "Expected member not found: %s of type %s"
+                % (expectedMembers[0][0], expectedMembers[0][1]),
+            )
         return results
 
     def shouldFail(prefix, iface):
@@ -35,44 +44,46 @@ def WebIDLTest(parser, harness):
             p = parser.reset()
             p.parse(iface)
             p.finish()
-            harness.ok(False,
-                       prefix + " - Interface passed when should've failed")
+            harness.ok(False, prefix + " - Interface passed when should've failed")
         except WebIDL.WebIDLError as e:
-            harness.ok(True,
-                       prefix + " - Interface failed as expected")
+            harness.ok(True, prefix + " - Interface failed as expected")
         except Exception as e:
-            harness.ok(False,
-                       prefix + " - Interface failed but not as a WebIDLError exception: %s" % e)
+            harness.ok(
+                False,
+                prefix
+                + " - Interface failed but not as a WebIDLError exception: %s" % e,
+            )
 
-    iterableMembers = [(x, WebIDL.IDLMethod) for x in ["entries", "keys",
-                                                       "values", "forEach"]]
-    setROMembers = ([(x, WebIDL.IDLMethod) for x in ["has"]] +
-                    [("__setlike", WebIDL.IDLMaplikeOrSetlike)] +
-                    iterableMembers)
+    iterableMembers = [
+        (x, WebIDL.IDLMethod) for x in ["entries", "keys", "values", "forEach"]
+    ]
+    setROMembers = (
+        [(x, WebIDL.IDLMethod) for x in ["has"]]
+        + [("__setlike", WebIDL.IDLMaplikeOrSetlike)]
+        + iterableMembers
+    )
     setROMembers.extend([("size", WebIDL.IDLAttribute)])
-    setRWMembers = ([(x, WebIDL.IDLMethod) for x in ["add",
-                                                     "clear",
-                                                     "delete"]] +
-                    setROMembers)
-    setROChromeMembers = ([(x, WebIDL.IDLMethod) for x in ["__add",
-                                                           "__clear",
-                                                           "__delete"]] +
-                          setROMembers)
-    setRWChromeMembers = ([(x, WebIDL.IDLMethod) for x in ["__add",
-                                                           "__clear",
-                                                           "__delete"]] +
-                          setRWMembers)
-    mapROMembers = ([(x, WebIDL.IDLMethod) for x in ["get", "has"]] +
-                    [("__maplike", WebIDL.IDLMaplikeOrSetlike)] +
-                    iterableMembers)
+    setRWMembers = [
+        (x, WebIDL.IDLMethod) for x in ["add", "clear", "delete"]
+    ] + setROMembers
+    setROChromeMembers = [
+        (x, WebIDL.IDLMethod) for x in ["__add", "__clear", "__delete"]
+    ] + setROMembers
+    setRWChromeMembers = [
+        (x, WebIDL.IDLMethod) for x in ["__add", "__clear", "__delete"]
+    ] + setRWMembers
+    mapROMembers = (
+        [(x, WebIDL.IDLMethod) for x in ["get", "has"]]
+        + [("__maplike", WebIDL.IDLMaplikeOrSetlike)]
+        + iterableMembers
+    )
     mapROMembers.extend([("size", WebIDL.IDLAttribute)])
-    mapRWMembers = ([(x, WebIDL.IDLMethod) for x in ["set",
-                                                     "clear",
-                                                     "delete"]] + mapROMembers)
-    mapRWChromeMembers = ([(x, WebIDL.IDLMethod) for x in ["__set",
-                                                           "__clear",
-                                                           "__delete"]] +
-                          mapRWMembers)
+    mapRWMembers = [
+        (x, WebIDL.IDLMethod) for x in ["set", "clear", "delete"]
+    ] + mapROMembers
+    mapRWChromeMembers = [
+        (x, WebIDL.IDLMethod) for x in ["__set", "__clear", "__delete"]
+    ] + mapRWMembers
 
     # OK, now that we've used iterableMembers to set up the above, append
     # __iterable to it for the iterable<> case.
@@ -88,15 +99,18 @@ def WebIDLTest(parser, harness):
     disallowedNonMethodNames = ["clear", "delete"]
     mapDisallowedNonMethodNames = ["set"] + disallowedNonMethodNames
     setDisallowedNonMethodNames = ["add"] + disallowedNonMethodNames
-    unrelatedMembers = [("unrelatedAttribute", WebIDL.IDLAttribute),
-                        ("unrelatedMethod", WebIDL.IDLMethod)]
+    unrelatedMembers = [
+        ("unrelatedAttribute", WebIDL.IDLAttribute),
+        ("unrelatedMethod", WebIDL.IDLMethod),
+    ]
 
     #
     # Simple Usage Tests
     #
 
-    shouldPass("Iterable (key only)",
-               """
+    shouldPass(
+        "Iterable (key only)",
+        """
                interface Foo1 {
                iterable<long>;
                readonly attribute unsigned long length;
@@ -104,10 +118,13 @@ def WebIDLTest(parser, harness):
                attribute long unrelatedAttribute;
                long unrelatedMethod();
                };
-               """, valueIterableMembers + unrelatedMembers)
+               """,
+        valueIterableMembers + unrelatedMembers,
+    )
 
-    shouldPass("Iterable (key only) inheriting from parent",
-               """
+    shouldPass(
+        "Iterable (key only) inheriting from parent",
+        """
                interface Foo1 : Foo2 {
                iterable<long>;
                readonly attribute unsigned long length;
@@ -117,21 +134,28 @@ def WebIDLTest(parser, harness):
                attribute long unrelatedAttribute;
                long unrelatedMethod();
                };
-               """, valueIterableMembers, numProductions=2)
+               """,
+        valueIterableMembers,
+        numProductions=2,
+    )
 
-    shouldPass("Iterable (key and value)",
-               """
+    shouldPass(
+        "Iterable (key and value)",
+        """
                interface Foo1 {
                iterable<long, long>;
                attribute long unrelatedAttribute;
                long unrelatedMethod();
                };
-               """, iterableMembers + unrelatedMembers,
-               # numProductions == 2 because of the generated iterator iface,
-               numProductions=2)
+               """,
+        iterableMembers + unrelatedMembers,
+        # numProductions == 2 because of the generated iterator iface,
+        numProductions=2,
+    )
 
-    shouldPass("Iterable (key and value) inheriting from parent",
-               """
+    shouldPass(
+        "Iterable (key and value) inheriting from parent",
+        """
                interface Foo1 : Foo2 {
                iterable<long, long>;
                };
@@ -139,21 +163,27 @@ def WebIDLTest(parser, harness):
                attribute long unrelatedAttribute;
                long unrelatedMethod();
                };
-               """, iterableMembers,
-               # numProductions == 3 because of the generated iterator iface,
-               numProductions=3)
+               """,
+        iterableMembers,
+        # numProductions == 3 because of the generated iterator iface,
+        numProductions=3,
+    )
 
-    shouldPass("Maplike (readwrite)",
-               """
+    shouldPass(
+        "Maplike (readwrite)",
+        """
                interface Foo1 {
                maplike<long, long>;
                attribute long unrelatedAttribute;
                long unrelatedMethod();
                };
-               """, mapRWMembers + unrelatedMembers)
+               """,
+        mapRWMembers + unrelatedMembers,
+    )
 
-    shouldPass("Maplike (readwrite) inheriting from parent",
-               """
+    shouldPass(
+        "Maplike (readwrite) inheriting from parent",
+        """
                interface Foo1 : Foo2 {
                maplike<long, long>;
                };
@@ -161,19 +191,26 @@ def WebIDLTest(parser, harness):
                attribute long unrelatedAttribute;
                long unrelatedMethod();
                };
-               """, mapRWMembers, numProductions=2)
+               """,
+        mapRWMembers,
+        numProductions=2,
+    )
 
-    shouldPass("Maplike (readwrite)",
-               """
+    shouldPass(
+        "Maplike (readwrite)",
+        """
                interface Foo1 {
                maplike<long, long>;
                attribute long unrelatedAttribute;
                long unrelatedMethod();
                };
-               """, mapRWMembers + unrelatedMembers)
+               """,
+        mapRWMembers + unrelatedMembers,
+    )
 
-    shouldPass("Maplike (readwrite) inheriting from parent",
-               """
+    shouldPass(
+        "Maplike (readwrite) inheriting from parent",
+        """
                interface Foo1 : Foo2 {
                maplike<long, long>;
                };
@@ -181,19 +218,26 @@ def WebIDLTest(parser, harness):
                attribute long unrelatedAttribute;
                long unrelatedMethod();
                };
-               """, mapRWMembers, numProductions=2)
+               """,
+        mapRWMembers,
+        numProductions=2,
+    )
 
-    shouldPass("Maplike (readonly)",
-               """
+    shouldPass(
+        "Maplike (readonly)",
+        """
                interface Foo1 {
                readonly maplike<long, long>;
                attribute long unrelatedAttribute;
                long unrelatedMethod();
                };
-               """, mapROMembers + unrelatedMembers)
+               """,
+        mapROMembers + unrelatedMembers,
+    )
 
-    shouldPass("Maplike (readonly) inheriting from parent",
-               """
+    shouldPass(
+        "Maplike (readonly) inheriting from parent",
+        """
                interface Foo1 : Foo2 {
                readonly maplike<long, long>;
                };
@@ -201,19 +245,26 @@ def WebIDLTest(parser, harness):
                attribute long unrelatedAttribute;
                long unrelatedMethod();
                };
-               """, mapROMembers, numProductions=2)
+               """,
+        mapROMembers,
+        numProductions=2,
+    )
 
-    shouldPass("Setlike (readwrite)",
-               """
+    shouldPass(
+        "Setlike (readwrite)",
+        """
                interface Foo1 {
                setlike<long>;
                attribute long unrelatedAttribute;
                long unrelatedMethod();
                };
-               """, setRWMembers + unrelatedMembers)
+               """,
+        setRWMembers + unrelatedMembers,
+    )
 
-    shouldPass("Setlike (readwrite) inheriting from parent",
-               """
+    shouldPass(
+        "Setlike (readwrite) inheriting from parent",
+        """
                interface Foo1 : Foo2 {
                setlike<long>;
                };
@@ -221,19 +272,26 @@ def WebIDLTest(parser, harness):
                attribute long unrelatedAttribute;
                long unrelatedMethod();
                };
-               """, setRWMembers, numProductions=2)
+               """,
+        setRWMembers,
+        numProductions=2,
+    )
 
-    shouldPass("Setlike (readonly)",
-               """
+    shouldPass(
+        "Setlike (readonly)",
+        """
                interface Foo1 {
                readonly setlike<long>;
                attribute long unrelatedAttribute;
                long unrelatedMethod();
                };
-               """, setROMembers + unrelatedMembers)
+               """,
+        setROMembers + unrelatedMembers,
+    )
 
-    shouldPass("Setlike (readonly) inheriting from parent",
-               """
+    shouldPass(
+        "Setlike (readonly) inheriting from parent",
+        """
                interface Foo1 : Foo2 {
                readonly setlike<long>;
                };
@@ -241,95 +299,119 @@ def WebIDLTest(parser, harness):
                attribute long unrelatedAttribute;
                long unrelatedMethod();
                };
-               """, setROMembers, numProductions=2)
+               """,
+        setROMembers,
+        numProductions=2,
+    )
 
-    shouldPass("Inheritance of maplike/setlike",
-               """
+    shouldPass(
+        "Inheritance of maplike/setlike",
+        """
                interface Foo1 {
                maplike<long, long>;
                };
                interface Foo2 : Foo1 {
                };
-               """, mapRWMembers, numProductions=2)
+               """,
+        mapRWMembers,
+        numProductions=2,
+    )
 
-    shouldPass("JS Implemented maplike interface",
-               """
+    shouldFail(
+        "JS Implemented maplike interface",
+        """
                [JSImplementation="@mozilla.org/dom/test-interface-js-maplike;1"]
                interface Foo1 {
                constructor();
                setlike<long>;
                };
-               """, setRWChromeMembers)
+               """,
+    )
 
-    shouldPass("JS Implemented maplike interface",
-               """
+    shouldFail(
+        "JS Implemented maplike interface",
+        """
                [JSImplementation="@mozilla.org/dom/test-interface-js-maplike;1"]
                interface Foo1 {
                constructor();
                maplike<long, long>;
                };
-               """, mapRWChromeMembers)
+               """,
+    )
 
     #
     # Multiple maplike/setlike tests
     #
 
-    shouldFail("Two maplike/setlikes on same interface",
-               """
+    shouldFail(
+        "Two maplike/setlikes on same interface",
+        """
                interface Foo1 {
                setlike<long>;
                maplike<long, long>;
                };
-               """)
+               """,
+    )
 
-    shouldFail("Two iterable/setlikes on same interface",
-               """
+    shouldFail(
+        "Two iterable/setlikes on same interface",
+        """
                interface Foo1 {
                iterable<long>;
                maplike<long, long>;
                };
-               """)
+               """,
+    )
 
-    shouldFail("Two iterables on same interface",
-               """
+    shouldFail(
+        "Two iterables on same interface",
+        """
                interface Foo1 {
                iterable<long>;
                iterable<long, long>;
                };
-               """)
+               """,
+    )
 
-    shouldFail("Two maplike/setlikes in partials",
-               """
+    shouldFail(
+        "Two maplike/setlikes in partials",
+        """
                interface Foo1 {
                maplike<long, long>;
                };
                partial interface Foo1 {
                setlike<long>;
                };
-               """)
+               """,
+    )
 
-    shouldFail("Conflicting maplike/setlikes across inheritance",
-               """
+    shouldFail(
+        "Conflicting maplike/setlikes across inheritance",
+        """
                interface Foo1 {
                maplike<long, long>;
                };
                interface Foo2 : Foo1 {
                setlike<long>;
                };
-               """)
+               """,
+    )
 
-    shouldFail("Conflicting maplike/iterable across inheritance",
-               """
+    shouldFail(
+        "Conflicting maplike/iterable across inheritance",
+        """
                interface Foo1 {
                maplike<long, long>;
                };
                interface Foo2 : Foo1 {
                iterable<long>;
                };
-               """)
+               """,
+    )
 
-    shouldFail("Conflicting maplike/setlikes across multistep inheritance",
-               """
+    shouldFail(
+        "Conflicting maplike/setlikes across multistep inheritance",
+        """
                interface Foo1 {
                maplike<long, long>;
                };
@@ -338,7 +420,8 @@ def WebIDLTest(parser, harness):
                interface Foo3 : Foo2 {
                setlike<long>;
                };
-               """)
+               """,
+    )
 
     #
     # Member name collision tests
@@ -353,61 +436,83 @@ def WebIDLTest(parser, harness):
 
         """
         if methodPasses:
-            shouldPass("Conflicting method: %s and %s" % (likeMember, conflictName),
-                       """
+            shouldPass(
+                "Conflicting method: %s and %s" % (likeMember, conflictName),
+                """
                        interface Foo1 {
                        %s;
                        [Throws]
                        void %s(long test1, double test2, double test3);
                        };
-                       """ % (likeMember, conflictName), expectedMembers)
+                       """
+                % (likeMember, conflictName),
+                expectedMembers,
+            )
         else:
-            shouldFail("Conflicting method: %s and %s" % (likeMember, conflictName),
-                       """
+            shouldFail(
+                "Conflicting method: %s and %s" % (likeMember, conflictName),
+                """
                        interface Foo1 {
                        %s;
                        [Throws]
                        void %s(long test1, double test2, double test3);
                        };
-                       """ % (likeMember, conflictName))
+                       """
+                % (likeMember, conflictName),
+            )
         # Inherited conflicting methods should ALWAYS fail
-        shouldFail("Conflicting inherited method: %s and %s" % (likeMember, conflictName),
-                   """
+        shouldFail(
+            "Conflicting inherited method: %s and %s" % (likeMember, conflictName),
+            """
                    interface Foo1 {
                    void %s(long test1, double test2, double test3);
                    };
                    interface Foo2 : Foo1 {
                    %s;
                    };
-                   """ % (conflictName, likeMember))
-        shouldFail("Conflicting static method: %s and %s" % (likeMember, conflictName),
                    """
+            % (conflictName, likeMember),
+        )
+        shouldFail(
+            "Conflicting static method: %s and %s" % (likeMember, conflictName),
+            """
                    interface Foo1 {
                    %s;
                    static void %s(long test1, double test2, double test3);
                    };
-                   """ % (likeMember, conflictName))
-        shouldFail("Conflicting attribute: %s and %s" % (likeMember, conflictName),
                    """
+            % (likeMember, conflictName),
+        )
+        shouldFail(
+            "Conflicting attribute: %s and %s" % (likeMember, conflictName),
+            """
                    interface Foo1 {
                    %s
                    attribute double %s;
                    };
-                   """ % (likeMember, conflictName))
-        shouldFail("Conflicting const: %s and %s" % (likeMember, conflictName),
                    """
+            % (likeMember, conflictName),
+        )
+        shouldFail(
+            "Conflicting const: %s and %s" % (likeMember, conflictName),
+            """
                    interface Foo1 {
                    %s;
                    const double %s = 0;
                    };
-                   """ % (likeMember, conflictName))
-        shouldFail("Conflicting static attribute: %s and %s" % (likeMember, conflictName),
                    """
+            % (likeMember, conflictName),
+        )
+        shouldFail(
+            "Conflicting static attribute: %s and %s" % (likeMember, conflictName),
+            """
                    interface Foo1 {
                    %s;
                    static attribute long %s;
                    };
-                   """ % (likeMember, conflictName))
+                   """
+            % (likeMember, conflictName),
+        )
 
     for member in disallowedIterableNames:
         testConflictingMembers("iterable<long, long>", member, iterableMembers, False)
@@ -420,18 +525,23 @@ def WebIDLTest(parser, harness):
     for member in setDisallowedNonMethodNames:
         testConflictingMembers("setlike<long>", member, setRWMembers, True)
 
-    shouldPass("Inheritance of maplike/setlike with child member collision",
-               """
+    shouldPass(
+        "Inheritance of maplike/setlike with child member collision",
+        """
                interface Foo1 {
                maplike<long, long>;
                };
                interface Foo2 : Foo1 {
                void entries();
                };
-               """, mapRWMembers, numProductions=2)
+               """,
+        mapRWMembers,
+        numProductions=2,
+    )
 
-    shouldPass("Inheritance of multi-level maplike/setlike with child member collision",
-               """
+    shouldPass(
+        "Inheritance of multi-level maplike/setlike with child member collision",
+        """
                interface Foo1 {
                maplike<long, long>;
                };
@@ -440,10 +550,14 @@ def WebIDLTest(parser, harness):
                interface Foo3 : Foo2 {
                void entries();
                };
-               """, mapRWMembers, numProductions=3)
+               """,
+        mapRWMembers,
+        numProductions=3,
+    )
 
-    shouldFail("Maplike interface with mixin member collision",
-               """
+    shouldFail(
+        "Maplike interface with mixin member collision",
+        """
                interface Foo1 {
                maplike<long, long>;
                };
@@ -451,10 +565,12 @@ def WebIDLTest(parser, harness):
                void entries();
                };
                Foo1 includes Foo2;
-               """)
+               """,
+    )
 
-    shouldPass("Inherited Maplike interface with consequential interface member collision",
-               """
+    shouldPass(
+        "Inherited Maplike interface with consequential interface member collision",
+        """
                interface Foo1 {
                maplike<long, long>;
                };
@@ -464,20 +580,26 @@ def WebIDLTest(parser, harness):
                interface Foo3 : Foo1 {
                };
                Foo3 includes Foo2;
-               """, mapRWMembers, numProductions=4)
+               """,
+        mapRWMembers,
+        numProductions=4,
+    )
 
-    shouldFail("Inheritance of name collision with child maplike/setlike",
-               """
+    shouldFail(
+        "Inheritance of name collision with child maplike/setlike",
+        """
                interface Foo1 {
                void entries();
                };
                interface Foo2 : Foo1 {
                maplike<long, long>;
                };
-               """)
+               """,
+    )
 
-    shouldFail("Inheritance of multi-level name collision with child maplike/setlike",
-               """
+    shouldFail(
+        "Inheritance of multi-level name collision with child maplike/setlike",
+        """
                interface Foo1 {
                void entries();
                };
@@ -486,20 +608,26 @@ def WebIDLTest(parser, harness):
                interface Foo3 : Foo2 {
                maplike<long, long>;
                };
-               """)
+               """,
+    )
 
-    shouldPass("Inheritance of attribute collision with parent maplike/setlike",
-               """
+    shouldPass(
+        "Inheritance of attribute collision with parent maplike/setlike",
+        """
                interface Foo1 {
                maplike<long, long>;
                };
                interface Foo2 : Foo1 {
                attribute double size;
                };
-               """, mapRWMembers, numProductions=2)
+               """,
+        mapRWMembers,
+        numProductions=2,
+    )
 
-    shouldPass("Inheritance of multi-level attribute collision with parent maplike/setlike",
-               """
+    shouldPass(
+        "Inheritance of multi-level attribute collision with parent maplike/setlike",
+        """
                interface Foo1 {
                maplike<long, long>;
                };
@@ -508,20 +636,26 @@ def WebIDLTest(parser, harness):
                interface Foo3 : Foo2 {
                attribute double size;
                };
-               """, mapRWMembers, numProductions=3)
+               """,
+        mapRWMembers,
+        numProductions=3,
+    )
 
-    shouldFail("Inheritance of attribute collision with child maplike/setlike",
-               """
+    shouldFail(
+        "Inheritance of attribute collision with child maplike/setlike",
+        """
                interface Foo1 {
                attribute double size;
                };
                interface Foo2 : Foo1 {
                maplike<long, long>;
                };
-               """)
+               """,
+    )
 
-    shouldFail("Inheritance of multi-level attribute collision with child maplike/setlike",
-               """
+    shouldFail(
+        "Inheritance of multi-level attribute collision with child maplike/setlike",
+        """
                interface Foo1 {
                attribute double size;
                };
@@ -530,53 +664,65 @@ def WebIDLTest(parser, harness):
                interface Foo3 : Foo2 {
                maplike<long, long>;
                };
-               """)
+               """,
+    )
 
-    shouldFail("Inheritance of attribute/rw function collision with child maplike/setlike",
-               """
+    shouldFail(
+        "Inheritance of attribute/rw function collision with child maplike/setlike",
+        """
                interface Foo1 {
                attribute double set;
                };
                interface Foo2 : Foo1 {
                maplike<long, long>;
                };
-               """)
+               """,
+    )
 
-    shouldFail("Inheritance of const/rw function collision with child maplike/setlike",
-               """
+    shouldFail(
+        "Inheritance of const/rw function collision with child maplike/setlike",
+        """
                interface Foo1 {
                const double set = 0;
                };
                interface Foo2 : Foo1 {
                maplike<long, long>;
                };
-               """)
+               """,
+    )
 
-    shouldPass("Inheritance of rw function with same name in child maplike/setlike",
-               """
+    shouldPass(
+        "Inheritance of rw function with same name in child maplike/setlike",
+        """
                interface Foo1 {
                maplike<long, long>;
                };
                interface Foo2 : Foo1 {
                void clear();
                };
-               """, mapRWMembers, numProductions=2)
+               """,
+        mapRWMembers,
+        numProductions=2,
+    )
 
-    shouldFail("Inheritance of unforgeable attribute collision with child maplike/setlike",
-               """
+    shouldFail(
+        "Inheritance of unforgeable attribute collision with child maplike/setlike",
+        """
                interface Foo1 {
-               [Unforgeable]
+               [LegacyUnforgeable]
                attribute double size;
                };
                interface Foo2 : Foo1 {
                maplike<long, long>;
                };
-               """)
+               """,
+    )
 
-    shouldFail("Inheritance of multi-level unforgeable attribute collision with child maplike/setlike",
-               """
+    shouldFail(
+        "Inheritance of multi-level unforgeable attribute collision with child maplike/setlike",
+        """
                interface Foo1 {
-               [Unforgeable]
+               [LegacyUnforgeable]
                attribute double size;
                };
                interface Foo2 : Foo1 {
@@ -584,49 +730,40 @@ def WebIDLTest(parser, harness):
                interface Foo3 : Foo2 {
                maplike<long, long>;
                };
-               """)
+               """,
+    )
 
-    shouldPass("Interface with readonly allowable overrides",
-               """
+    shouldPass(
+        "Interface with readonly allowable overrides",
+        """
                interface Foo1 {
                readonly setlike<long>;
                readonly attribute boolean clear;
                };
-               """, setROMembers + [("clear", WebIDL.IDLAttribute)])
+               """,
+        setROMembers + [("clear", WebIDL.IDLAttribute)],
+    )
 
-    shouldPass("JS Implemented read-only interface with readonly allowable overrides",
-               """
-               [JSImplementation="@mozilla.org/dom/test-interface-js-maplike;1"]
-               interface Foo1 {
-               constructor();
-               readonly setlike<long>;
-               readonly attribute boolean clear;
-               };
-               """, setROChromeMembers + [("clear", WebIDL.IDLAttribute)])
-
-    shouldFail("JS Implemented read-write interface with non-readwrite allowable overrides",
-               """
-               [JSImplementation="@mozilla.org/dom/test-interface-js-maplike;1"]
-               interface Foo1 {
-               constructor();
-               setlike<long>;
-               readonly attribute boolean clear;
-               };
-               """)
-
-    r = shouldPass("Check proper override of clear/delete/set",
-                   """
+    r = shouldPass(
+        "Check proper override of clear/delete/set",
+        """
                    interface Foo1 {
                    maplike<long, long>;
                    long clear(long a, long b, double c, double d);
                    long set(long a, long b, double c, double d);
                    long delete(long a, long b, double c, double d);
                    };
-                   """, mapRWMembers)
+                   """,
+        mapRWMembers,
+    )
 
     for m in r[0].members:
         if m.identifier.name in ["clear", "set", "delete"]:
             harness.ok(m.isMethod(), "%s should be a method" % m.identifier.name)
-            harness.check(m.maxArgCount, 4, "%s should have 4 arguments" % m.identifier.name)
-            harness.ok(not m.isMaplikeOrSetlikeOrIterableMethod(),
-                       "%s should not be a maplike/setlike function" % m.identifier.name)
+            harness.check(
+                m.maxArgCount, 4, "%s should have 4 arguments" % m.identifier.name
+            )
+            harness.ok(
+                not m.isMaplikeOrSetlikeOrIterableMethod(),
+                "%s should not be a maplike/setlike function" % m.identifier.name,
+            )
