@@ -102,32 +102,40 @@ class MOZ_STACK_CLASS ContentEventHandler {
   explicit ContentEventHandler(nsPresContext* aPresContext);
 
   // Handle aEvent in the current process.
-  nsresult HandleQueryContentEvent(WidgetQueryContentEvent* aEvent);
+  MOZ_CAN_RUN_SCRIPT nsresult
+  HandleQueryContentEvent(WidgetQueryContentEvent* aEvent);
 
   // eQuerySelectedText event handler
-  nsresult OnQuerySelectedText(WidgetQueryContentEvent* aEvent);
+  MOZ_CAN_RUN_SCRIPT nsresult
+  OnQuerySelectedText(WidgetQueryContentEvent* aEvent);
   // eQueryTextContent event handler
-  nsresult OnQueryTextContent(WidgetQueryContentEvent* aEvent);
+  MOZ_CAN_RUN_SCRIPT nsresult
+  OnQueryTextContent(WidgetQueryContentEvent* aEvent);
   // eQueryCaretRect event handler
-  nsresult OnQueryCaretRect(WidgetQueryContentEvent* aEvent);
+  MOZ_CAN_RUN_SCRIPT nsresult OnQueryCaretRect(WidgetQueryContentEvent* aEvent);
   // eQueryTextRect event handler
-  nsresult OnQueryTextRect(WidgetQueryContentEvent* aEvent);
+  MOZ_CAN_RUN_SCRIPT nsresult OnQueryTextRect(WidgetQueryContentEvent* aEvent);
   // eQueryTextRectArray event handler
-  nsresult OnQueryTextRectArray(WidgetQueryContentEvent* aEvent);
+  MOZ_CAN_RUN_SCRIPT nsresult
+  OnQueryTextRectArray(WidgetQueryContentEvent* aEvent);
   // eQueryEditorRect event handler
-  nsresult OnQueryEditorRect(WidgetQueryContentEvent* aEvent);
+  MOZ_CAN_RUN_SCRIPT nsresult
+  OnQueryEditorRect(WidgetQueryContentEvent* aEvent);
   // eQueryContentState event handler
-  nsresult OnQueryContentState(WidgetQueryContentEvent* aEvent);
+  MOZ_CAN_RUN_SCRIPT nsresult
+  OnQueryContentState(WidgetQueryContentEvent* aEvent);
   // eQuerySelectionAsTransferable event handler
-  nsresult OnQuerySelectionAsTransferable(WidgetQueryContentEvent* aEvent);
+  MOZ_CAN_RUN_SCRIPT nsresult
+  OnQuerySelectionAsTransferable(WidgetQueryContentEvent* aEvent);
   // eQueryCharacterAtPoint event handler
-  nsresult OnQueryCharacterAtPoint(WidgetQueryContentEvent* aEvent);
+  MOZ_CAN_RUN_SCRIPT nsresult
+  OnQueryCharacterAtPoint(WidgetQueryContentEvent* aEvent);
   // eQueryDOMWidgetHittest event handler
-  nsresult OnQueryDOMWidgetHittest(WidgetQueryContentEvent* aEvent);
+  MOZ_CAN_RUN_SCRIPT nsresult
+  OnQueryDOMWidgetHittest(WidgetQueryContentEvent* aEvent);
 
   // NS_SELECTION_* event
-  MOZ_CAN_RUN_SCRIPT
-  nsresult OnSelectionEvent(WidgetSelectionEvent* aEvent);
+  MOZ_CAN_RUN_SCRIPT nsresult OnSelectionEvent(WidgetSelectionEvent* aEvent);
 
  protected:
   RefPtr<dom::Document> mDocument;
@@ -140,19 +148,20 @@ class MOZ_STACK_CLASS ContentEventHandler {
   RawRange mFirstSelectedRawRange;
   nsCOMPtr<nsIContent> mRootContent;
 
-  nsresult Init(WidgetQueryContentEvent* aEvent);
-  nsresult Init(WidgetSelectionEvent* aEvent);
+  MOZ_CAN_RUN_SCRIPT nsresult Init(WidgetQueryContentEvent* aEvent);
+  MOZ_CAN_RUN_SCRIPT nsresult Init(WidgetSelectionEvent* aEvent);
 
   nsresult InitBasic(bool aRequireFlush = true);
-  nsresult InitCommon(SelectionType aSelectionType = SelectionType::eNormal,
-                      bool aRequireFlush = true);
+  MOZ_CAN_RUN_SCRIPT nsresult
+  InitCommon(SelectionType aSelectionType = SelectionType::eNormal,
+             bool aRequireFlush = true);
   /**
    * InitRootContent() computes the root content of current focused editor.
    *
    * @param aNormalSelection    This must be a Selection instance whose type is
    *                            SelectionType::eNormal.
    */
-  nsresult InitRootContent(Selection* aNormalSelection);
+  MOZ_CAN_RUN_SCRIPT nsresult InitRootContent(Selection* aNormalSelection);
 
  public:
   // FlatText means the text that is generated from DOM tree. The BR elements
@@ -283,8 +292,6 @@ class MOZ_STACK_CLASS ContentEventHandler {
   static LineBreakType GetLineBreakType(bool aUseNativeLineBreak);
   // Returns focused content (including its descendant documents).
   nsIContent* GetFocusedContent();
-  // Returns true if the content is a plugin host.
-  bool IsPlugin(nsIContent* aContent);
   // QueryContentRect() sets the rect of aContent's frame(s) to aEvent.
   nsresult QueryContentRect(nsIContent* aContent,
                             WidgetQueryContentEvent* aEvent);
