@@ -8,20 +8,18 @@
 
 #include "nsContentUtils.h"
 #include "nsIGlobalObject.h"
-#include "nsIClipboard.h"
 #include "nsIScriptObjectPrincipal.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIScriptContext.h"
-#include "nsISupportsPrimitives.h"
 #include "nsQueryObject.h"
 #include "nsVariant.h"
+#include "mozilla/BasePrincipal.h"
 #include "mozilla/ContentEvents.h"
 #include "mozilla/EventForwards.h"
 #include "mozilla/storage/Variant.h"
 #include "mozilla/dom/DataTransferItemListBinding.h"
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE(DataTransferItemList, mDataTransfer,
                                       mItems, mIndexedItems, mFiles)
@@ -207,12 +205,24 @@ already_AddRefed<FileList> DataTransferItemList::Files(
   // checked by DataTransferItem::Data(), however with files, we keep a cached
   // live copy of the files list for spec compliance.
   //
-  // A DataTransfer is only exposed to one webpage, and chrome code. The chrome
-  // code should be able to see all files on the DataTransfer, while the webpage
-  // should only be able to see the files it can see. As chrome code doesn't
-  // need as strict spec compliance as web visible code, we generate a new
-  // FileList object every time you access the Files list from chrome code, but
-  // re-use the cached one when accessing from non-chrome code.
+  // A DataTransfer is only exposed to one webpage, chrome code and expanded
+  // principals related to WebExtensions content scripts or user scripts.
+  // The chrome code should be able to see all files on the DataTransfer, while
+  // the webpage and WebExtensions content scripts and user scripts should only
+  // be able to see the files they can see.
+  //
+  // As chrome code doesn't need as strict spec compliance as web visible code,
+  // we generate a new FileList object every time you access the Files list from
+  // chrome code, but re-use the cached one when accessing from content code.
+  //
+  // For WebExtensions content scripts (expanded principals subsuming both
+  // the attached web page principal and the extension principal) and
+  // WebExtensions user scripts (expanded principals subsuming the attached
+  // web page principal but not the extension principal) we also don't cache
+  // the FileList as for chrome code (because the webpage principal and other
+  // extension content scripts/user scripts principals would not be able to
+  // access the cached FileList when accessed by a different expanded principal
+  // first, see Bug 1707214).
   //
   // It is not legal to expose an identical DataTransfer object is to multiple
   // different principals without using the `Clone` method or similar to copy it
@@ -220,7 +230,9 @@ already_AddRefed<FileList> DataTransferItemList::Files(
   // release builds. If this functionality is required in the future, a more
   // advanced caching mechanism for the FileList objects will be required.
   RefPtr<FileList> files;
-  if (nsContentUtils::IsSystemPrincipal(aPrincipal)) {
+  if (aPrincipal->IsSystemPrincipal() ||
+      // WebExtensions content scripts and user scripts.
+      nsContentUtils::IsExpandedPrincipal(aPrincipal)) {
     files = new FileList(mDataTransfer);
     GenerateFiles(files, aPrincipal);
     return files.forget();
@@ -414,6 +426,71 @@ DataTransferItem* DataTransferItemList::AppendNewItem(uint32_t aIndex,
   return item;
 }
 
+void DataTransferItemList::GetTypes(nsTArray<nsString>& aTypes,
+                                    CallerType aCallerType) const {
+  MOZ_ASSERT(aTypes.IsEmpty());
+
+  if (mIndexedItems.IsEmpty()) {
+    return;
+  }
+
+  bool foundFile = false;
+  for (const RefPtr<DataTransferItem>& item : mIndexedItems[0]) {
+    MOZ_ASSERT(item);
+
+    // XXX Why don't we check the caller type with item's permission only
+    //     for "Files"?
+    if (!foundFile) {
+      foundFile = item->Kind() == DataTransferItem::KIND_FILE;
+    }
+
+    if (item->ChromeOnly() && aCallerType != CallerType::System) {
+      continue;
+    }
+
+    // NOTE: The reason why we get the internal type here is because we want
+    // kFileMime to appear in the types list for backwards compatibility
+    // reasons.
+    nsAutoString type;
+    item->GetInternalType(type);
+    if (item->Kind() != DataTransferItem::KIND_FILE ||
+        type.EqualsASCII(kFileMime)) {
+      aTypes.AppendElement(type);
+    }
+  }
+
+  if (foundFile) {
+    aTypes.AppendElement(u"Files"_ns);
+  }
+}
+
+bool DataTransferItemList::HasType(const nsAString& aType) const {
+  MOZ_ASSERT(!aType.EqualsASCII("Files"), "Use HasFile instead");
+  if (mIndexedItems.IsEmpty()) {
+    return false;
+  }
+
+  for (const RefPtr<DataTransferItem>& item : mIndexedItems[0]) {
+    if (item->IsInternalType(aType)) {
+      return true;
+    }
+  }
+  return false;
+}
+
+bool DataTransferItemList::HasFile() const {
+  if (mIndexedItems.IsEmpty()) {
+    return false;
+  }
+
+  for (const RefPtr<DataTransferItem>& item : mIndexedItems[0]) {
+    if (item->Kind() == DataTransferItem::KIND_FILE) {
+      return true;
+    }
+  }
+  return false;
+}
+
 const nsTArray<RefPtr<DataTransferItem>>* DataTransferItemList::MozItemsAt(
     uint32_t aIndex)  // -- INDEXED
 {
@@ -528,8 +605,7 @@ void DataTransferItemList::GenerateFiles(FileList* aFiles,
 
   // For non-system principals, the Files list should be empty if the
   // DataTransfer is protected.
-  if (!nsContentUtils::IsSystemPrincipal(aFilesPrincipal) &&
-      mDataTransfer->IsProtected()) {
+  if (!aFilesPrincipal->IsSystemPrincipal() && mDataTransfer->IsProtected()) {
     return;
   }
 
@@ -549,5 +625,4 @@ void DataTransferItemList::GenerateFiles(FileList* aFiles,
   }
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
