@@ -15,6 +15,7 @@
 #include "nsINode.h"
 #include "nsIScriptObjectPrincipal.h"
 #include "nsPIDOMWindow.h"
+#include "nsRefreshDriver.h"
 #include "AnimationEvent.h"
 #include "BeforeUnloadEvent.h"
 #include "ClipboardEvent.h"
@@ -22,7 +23,6 @@
 #include "CompositionEvent.h"
 #include "DeviceMotionEvent.h"
 #include "DragEvent.h"
-#include "GeckoProfiler.h"
 #include "KeyboardEvent.h"
 #include "Layers.h"
 #include "mozilla/BasePrincipal.h"
@@ -39,6 +39,7 @@
 #include "mozilla/dom/MutationEvent.h"
 #include "mozilla/dom/NotifyPaintEvent.h"
 #include "mozilla/dom/PageTransitionEvent.h"
+#include "mozilla/dom/PerformanceEventTiming.h"
 #include "mozilla/dom/PointerEvent.h"
 #include "mozilla/dom/RootedDictionary.h"
 #include "mozilla/dom/ScrollAreaEvent.h"
@@ -57,22 +58,14 @@
 #include "mozilla/ipc/MessageChannel.h"
 #include "mozilla/MiscEvents.h"
 #include "mozilla/MouseEvents.h"
+#include "mozilla/ProfilerLabels.h"
+#include "mozilla/ProfilerMarkers.h"
+#include "mozilla/ScopeExit.h"
 #include "mozilla/Telemetry.h"
 #include "mozilla/TextEvents.h"
 #include "mozilla/TouchEvents.h"
 #include "mozilla/Unused.h"
 
-#ifdef MOZ_TASK_TRACER
-#  include "GeckoTaskTracer.h"
-#  include "mozilla/dom/Element.h"
-#  include "mozilla/Likely.h"
-using namespace mozilla::tasktracer;
-#endif
-
-#ifdef MOZ_GECKO_PROFILER
-#  include "ProfilerMarkerPayload.h"
-#endif
-
 namespace mozilla {
 
 using namespace dom;
@@ -149,9 +142,8 @@ class EventTargetChainItem {
 
   static void DestroyLast(nsTArray<EventTargetChainItem>& aChain,
                           EventTargetChainItem* aItem) {
-    uint32_t lastIndex = aChain.Length() - 1;
-    MOZ_ASSERT(&aChain[lastIndex] == aItem);
-    aChain.RemoveElementAt(lastIndex);
+    MOZ_ASSERT(&aChain.LastElement() == aItem);
+    aChain.RemoveLastElement();
   }
 
   static EventTargetChainItem* GetFirstCanHandleEventTarget(
@@ -740,32 +732,16 @@ nsresult EventDispatcher::Dispatch(nsISupports* aTarget,
   NS_ENSURE_TRUE(!nsContentUtils::IsInStableOrMetaStableState(),
                  NS_ERROR_DOM_INVALID_STATE_ERR);
 
-#ifdef MOZ_TASK_TRACER
-  if (MOZ_UNLIKELY(mozilla::tasktracer::IsStartLogging())) {
-    nsAutoCString eventType;
-    nsAutoString eventTypeU16;
-    if (aDOMEvent) {
-      aDOMEvent->GetType(eventTypeU16);
-    } else {
-      Event::GetWidgetEventType(aEvent, eventTypeU16);
-    }
-    eventType = NS_ConvertUTF16toUTF8(eventTypeU16);
-
-    nsCOMPtr<Element> element = do_QueryInterface(aTarget);
-    nsAutoString elementId;
-    nsAutoString elementTagName;
-    if (element) {
-      element->GetId(elementId);
-      element->GetTagName(elementTagName);
-    }
-    AddLabel("Event [%s] dispatched at target [id:%s tag:%s]", eventType.get(),
-             NS_ConvertUTF16toUTF8(elementId).get(),
-             NS_ConvertUTF16toUTF8(elementTagName).get());
-  }
-#endif
-
   nsCOMPtr<EventTarget> target = do_QueryInterface(aTarget);
 
+  RefPtr<PerformanceEventTiming> eventTimingEntry;
+  // Similar to PerformancePaintTiming, we don't need to
+  // expose them for printing documents
+  if (aPresContext && !aPresContext->IsPrintingOrPrintPreview()) {
+    eventTimingEntry =
+        PerformanceEventTiming::TryGenerateEventTiming(target, aEvent);
+  }
+
   bool retargeted = false;
 
   if (aEvent->mFlags.mRetargetToNonNativeAnonymous) {
@@ -850,7 +826,7 @@ nsresult EventDispatcher::Dispatch(nsISupports* aTarget,
     if (!sCachedMainThreadChain) {
       sCachedMainThreadChain = new nsTArray<EventTargetChainItem>();
     }
-    chain.SwapElements(*sCachedMainThreadChain);
+    chain = std::move(*sCachedMainThreadChain);
     chain.SetCapacity(128);
   }
 
@@ -898,7 +874,10 @@ nsresult EventDispatcher::Dispatch(nsISupports* aTarget,
 
   // Create visitor object and start event dispatching.
   // GetEventTargetParent for the original target.
-  nsEventStatus status = aEventStatus ? *aEventStatus : nsEventStatus_eIgnore;
+  nsEventStatus status = aDOMEvent && aDOMEvent->DefaultPrevented()
+                             ? nsEventStatus_eConsumeNoDefault
+                         : aEventStatus ? *aEventStatus
+                                        : nsEventStatus_eIgnore;
   nsCOMPtr<EventTarget> targetForPreVisitor = aEvent->mTarget;
   EventChainPreVisitor preVisitor(aPresContext, aEvent, aDOMEvent, status,
                                   isInAnon, targetForPreVisitor);
@@ -1004,6 +983,22 @@ nsresult EventDispatcher::Dispatch(nsISupports* aTarget,
           chain[i].PreHandleEvent(preVisitor);
         }
 
+        RefPtr<nsRefreshDriver> refreshDriver;
+        if (aPresContext && aPresContext->GetRootPresContext() &&
+            aEvent->IsTrusted() &&
+            (aEvent->mMessage == eKeyPress ||
+             aEvent->mMessage == eMouseClick)) {
+          refreshDriver = aPresContext->GetRootPresContext()->RefreshDriver();
+          if (refreshDriver) {
+            refreshDriver->EnterUserInputProcessing();
+          }
+        }
+        auto cleanup = MakeScopeExit([&] {
+          if (refreshDriver) {
+            refreshDriver->ExitUserInputProcessing();
+          }
+        });
+
         clearTargets = ShouldClearTargets(aEvent);
 
         // Handle the chain.
@@ -1011,7 +1006,6 @@ nsresult EventDispatcher::Dispatch(nsISupports* aTarget,
         MOZ_RELEASE_ASSERT(!aEvent->mPath);
         aEvent->mPath = &chain;
 
-#ifdef MOZ_GECKO_PROFILER
         if (profiler_is_active()) {
           // Add a profiler label and a profiler marker for the actual
           // dispatch of the event.
@@ -1021,8 +1015,8 @@ nsresult EventDispatcher::Dispatch(nsISupports* aTarget,
             // This is tiny bit slow, but happens only once per event.
             // Similar code also in EventListenerManager.
             nsCOMPtr<EventTarget> et = aEvent->mOriginalTarget;
-            RefPtr<Event> event = EventDispatcher::CreateEvent(
-                et, aPresContext, aEvent, EmptyString());
+            RefPtr<Event> event =
+                EventDispatcher::CreateEvent(et, aPresContext, aEvent, u""_ns);
             event.swap(postVisitor.mDOMEvent);
           }
           nsAutoString typeStr;
@@ -1032,43 +1026,88 @@ nsresult EventDispatcher::Dispatch(nsISupports* aTarget,
 
           nsCOMPtr<nsIDocShell> docShell;
           docShell = nsContentUtils::GetDocShellForEventTarget(aEvent->mTarget);
-          Maybe<uint64_t> innerWindowID;
+          MarkerInnerWindowId innerWindowId;
           if (nsCOMPtr<nsPIDOMWindowInner> inner =
                   do_QueryInterface(aEvent->mTarget->GetOwnerGlobal())) {
-            innerWindowID = Some(inner->WindowID());
+            innerWindowId = MarkerInnerWindowId{inner->WindowID()};
           }
-          PROFILER_ADD_MARKER_WITH_PAYLOAD(
-              "DOMEvent", DOM, DOMEventMarkerPayload,
-              (typeStr, aEvent->mTimeStamp, "DOMEvent", TRACING_INTERVAL_START,
-               innerWindowID));
+
+          struct DOMEventMarker {
+            static constexpr Span<const char> MarkerTypeName() {
+              return MakeStringSpan("DOMEvent");
+            }
+            static void StreamJSONMarkerData(
+                baseprofiler::SpliceableJSONWriter& aWriter,
+                const ProfilerString16View& aEventType,
+                const TimeStamp& aStartTime, const TimeStamp& aEventTimeStamp) {
+              aWriter.StringProperty(
+                  "eventType", NS_ConvertUTF16toUTF8(aEventType.Data(),
+                                                     aEventType.Length()));
+              // This is the event processing latency, which is the time from
+              // when the event was created, to when it was started to be
+              // processed. Note that the computation of this latency is
+              // deferred until serialization time, at the expense of some extra
+              // memory.
+              aWriter.DoubleProperty(
+                  "latency", (aStartTime - aEventTimeStamp).ToMilliseconds());
+            }
+            static MarkerSchema MarkerTypeDisplay() {
+              using MS = MarkerSchema;
+              MS schema{MS::Location::markerChart, MS::Location::markerTable,
+                        MS::Location::timelineOverview};
+              schema.SetChartLabel("{marker.data.eventType}");
+              schema.SetTooltipLabel("{marker.data.eventType} - DOMEvent");
+              schema.SetTableLabel("{marker.data.eventType}");
+              schema.AddKeyLabelFormat("latency", "Latency",
+                                       MS::Format::duration);
+              return schema;
+            }
+          };
+
+          auto startTime = TimeStamp::NowUnfuzzed();
+          profiler_add_marker("DOMEvent", geckoprofiler::category::DOM,
+                              {MarkerTiming::IntervalStart(),
+                               MarkerInnerWindowId(innerWindowId)},
+                              DOMEventMarker{}, typeStr, startTime,
+                              aEvent->mTimeStamp);
 
           EventTargetChainItem::HandleEventTargetChain(chain, postVisitor,
                                                        aCallback, cd);
 
-          PROFILER_ADD_MARKER_WITH_PAYLOAD(
-              "DOMEvent", DOM, DOMEventMarkerPayload,
-              (typeStr, aEvent->mTimeStamp, "DOMEvent", TRACING_INTERVAL_END,
-               innerWindowID));
-        } else
-#endif
-        {
+          profiler_add_marker(
+              "DOMEvent", geckoprofiler::category::DOM,
+              {MarkerTiming::IntervalEnd(), std::move(innerWindowId)},
+              DOMEventMarker{}, typeStr, startTime, aEvent->mTimeStamp);
+        } else {
           EventTargetChainItem::HandleEventTargetChain(chain, postVisitor,
                                                        aCallback, cd);
         }
         aEvent->mPath = nullptr;
 
-        if (aEvent->mMessage == eKeyPress && aEvent->IsTrusted()) {
-          if (aPresContext && aPresContext->GetRootPresContext()) {
-            nsRefreshDriver* driver =
-                aPresContext->GetRootPresContext()->RefreshDriver();
-            if (driver && driver->ViewManagerFlushIsPending()) {
-              nsIWidget* widget = aPresContext->GetRootWidget();
-              layers::LayerManager* lm =
-                  widget ? widget->GetLayerManager() : nullptr;
-              if (lm) {
-                lm->RegisterPayload({layers::CompositionPayloadType::eKeyPress,
-                                     aEvent->mTimeStamp});
+        if (aPresContext && aPresContext->GetRootPresContext() &&
+            aEvent->IsTrusted()) {
+          nsRefreshDriver* driver =
+              aPresContext->GetRootPresContext()->RefreshDriver();
+          if (driver && driver->HasPendingTick()) {
+            switch (aEvent->mMessage) {
+              case eKeyPress:
+                driver->RegisterCompositionPayload(
+                    {layers::CompositionPayloadType::eKeyPress,
+                     aEvent->mTimeStamp});
+                break;
+              case eMouseClick: {
+                if (aEvent->AsMouseEvent()->mInputSource ==
+                        MouseEvent_Binding::MOZ_SOURCE_MOUSE ||
+                    aEvent->AsMouseEvent()->mInputSource ==
+                        MouseEvent_Binding::MOZ_SOURCE_TOUCH) {
+                  driver->RegisterCompositionPayload(
+                      {layers::CompositionPayloadType::eMouseUpFollowedByClick,
+                       aEvent->mTimeStamp});
+                }
+                break;
               }
+              default:
+                break;
             }
           }
         }
@@ -1088,6 +1127,9 @@ nsresult EventDispatcher::Dispatch(nsISupports* aTarget,
   aEvent->mFlags.mIsBeingDispatched = false;
   aEvent->mFlags.mDispatchedAtLeastOnce = true;
 
+  if (eventTimingEntry) {
+    eventTimingEntry->FinalizeEventTiming(aEvent->mTarget);
+  }
   // https://dom.spec.whatwg.org/#concept-event-dispatch
   // step 10. If clearTargets, then:
   //          1. Set event's target to null.
@@ -1102,7 +1144,7 @@ nsresult EventDispatcher::Dispatch(nsISupports* aTarget,
   }
 
   if (!externalDOMEvent && preVisitor.mDOMEvent) {
-    // An dom::Event was created while dispatching the event.
+    // A dom::Event was created while dispatching the event.
     // Duplicate private data if someone holds a pointer to it.
     nsrefcnt rc = 0;
     NS_RELEASE2(preVisitor.mDOMEvent, rc);
@@ -1254,7 +1296,7 @@ nsresult EventDispatcher::DispatchDOMEvent(nsISupports* aTarget,
   if (aEventType.LowerCaseEqualsLiteral("deviceorientationevent")) {
     DeviceOrientationEventInit init;
     RefPtr<Event> event =
-        DeviceOrientationEvent::Constructor(aOwner, EmptyString(), init);
+        DeviceOrientationEvent::Constructor(aOwner, u""_ns, init);
     event->MarkUninitialized();
     return event.forget();
   }
@@ -1286,8 +1328,7 @@ nsresult EventDispatcher::DispatchDOMEvent(nsISupports* aTarget,
   }
   if (aEventType.LowerCaseEqualsLiteral("hashchangeevent")) {
     HashChangeEventInit init;
-    RefPtr<Event> event =
-        HashChangeEvent::Constructor(aOwner, EmptyString(), init);
+    RefPtr<Event> event = HashChangeEvent::Constructor(aOwner, u""_ns, init);
     event->MarkUninitialized();
     return event.forget();
   }
@@ -1296,7 +1337,7 @@ nsresult EventDispatcher::DispatchDOMEvent(nsISupports* aTarget,
   }
   if (aEventType.LowerCaseEqualsLiteral("storageevent")) {
     RefPtr<Event> event =
-        StorageEvent::Constructor(aOwner, EmptyString(), StorageEventInit());
+        StorageEvent::Constructor(aOwner, u""_ns, StorageEventInit());
     event->MarkUninitialized();
     return event.forget();
   }
