@@ -21,6 +21,7 @@
 #include "nsContentUtils.h"
 #include "nsGkAtoms.h"
 #include "nsAtom.h"
+#include "nsDocShell.h"
 #include "nsIContent.h"
 #include "mozilla/dom/Document.h"
 #include "nsIFrame.h"
@@ -66,69 +67,6 @@ static nsIContent* PointBefore(nsINode* aContainer, nsIContent* aContent) {
   return aContainer->GetLastChild();
 }
 
-class WritingModeToString final : public nsAutoCString {
- public:
-  explicit WritingModeToString(const WritingMode& aWritingMode) {
-    if (!aWritingMode.IsVertical()) {
-      AssignLiteral("Horizontal");
-      return;
-    }
-    if (aWritingMode.IsVerticalLR()) {
-      AssignLiteral("Vertical (LR)");
-      return;
-    }
-    AssignLiteral("Vertical (RL)");
-  }
-  virtual ~WritingModeToString() {}
-};
-
-class SelectionChangeDataToString final : public nsAutoCString {
- public:
-  explicit SelectionChangeDataToString(
-      const IMENotification::SelectionChangeDataBase& aData) {
-    if (!aData.IsValid()) {
-      AppendLiteral("{ IsValid()=false }");
-      return;
-    }
-    AppendPrintf("{ mOffset=%u, ", aData.mOffset);
-    if (aData.mString->Length() > 20) {
-      AppendPrintf("mString.Length()=%u, ", aData.mString->Length());
-    } else {
-      AppendPrintf("mString=\"%s\" (Length()=%u), ",
-                   NS_ConvertUTF16toUTF8(*aData.mString).get(),
-                   aData.mString->Length());
-    }
-    AppendPrintf(
-        "GetWritingMode()=%s, mReversed=%s, mCausedByComposition=%s, "
-        "mCausedBySelectionEvent=%s }",
-        WritingModeToString(aData.GetWritingMode()).get(),
-        ToChar(aData.mReversed), ToChar(aData.mCausedByComposition),
-        ToChar(aData.mCausedBySelectionEvent));
-  }
-  virtual ~SelectionChangeDataToString() {}
-};
-
-class TextChangeDataToString final : public nsAutoCString {
- public:
-  explicit TextChangeDataToString(
-      const IMENotification::TextChangeDataBase& aData) {
-    if (!aData.IsValid()) {
-      AppendLiteral("{ IsValid()=false }");
-      return;
-    }
-    AppendPrintf(
-        "{ mStartOffset=%u, mRemovedEndOffset=%u, mAddedEndOffset=%u, "
-        "mCausedOnlyByComposition=%s, "
-        "mIncludingChangesDuringComposition=%s, "
-        "mIncludingChangesWithoutComposition=%s }",
-        aData.mStartOffset, aData.mRemovedEndOffset, aData.mAddedEndOffset,
-        ToChar(aData.mCausedOnlyByComposition),
-        ToChar(aData.mIncludingChangesDuringComposition),
-        ToChar(aData.mIncludingChangesWithoutComposition));
-  }
-  virtual ~TextChangeDataToString() {}
-};
-
 /******************************************************************************
  * mozilla::IMEContentObserver
  ******************************************************************************/
@@ -153,6 +91,7 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(IMEContentObserver)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mDocumentObserver)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mEndOfAddedTextCache.mContainerNode)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mStartOfRemovingTextRangeCache.mContainerNode)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK_WEAK_REFERENCE
 
   tmp->mIMENotificationRequests = nullptr;
   tmp->mESM = nullptr;
@@ -186,7 +125,6 @@ NS_IMPL_CYCLE_COLLECTING_RELEASE(IMEContentObserver)
 IMEContentObserver::IMEContentObserver()
     : mESM(nullptr),
       mIMENotificationRequests(nullptr),
-      mPreAttrChangeLength(0),
       mSuppressNotifications(0),
       mPreCharacterDataChangeLength(-1),
       mSendingNotification(NOTIFY_IME_OF_NOTHING),
@@ -203,8 +141,8 @@ IMEContentObserver::IMEContentObserver()
 #endif
 }
 
-void IMEContentObserver::Init(nsIWidget* aWidget, nsPresContext* aPresContext,
-                              nsIContent* aContent, EditorBase* aEditorBase) {
+void IMEContentObserver::Init(nsIWidget& aWidget, nsPresContext& aPresContext,
+                              nsIContent* aContent, EditorBase& aEditorBase) {
   State state = GetState();
   if (NS_WARN_IF(state == eState_Observing)) {
     return;  // Nothing to do.
@@ -218,22 +156,19 @@ void IMEContentObserver::Init(nsIWidget* aWidget, nsPresContext* aPresContext,
     Clear();
   }
 
-  mESM = aPresContext->EventStateManager();
+  mESM = aPresContext.EventStateManager();
   mESM->OnStartToObserveContent(this);
 
-  mWidget = aWidget;
+  mWidget = &aWidget;
   mIMENotificationRequests = &mWidget->IMENotificationRequestsRef();
 
-  if (aWidget->GetInputContext().mIMEState.mEnabled == IMEState::PLUGIN) {
-    if (!InitWithPlugin(aPresContext, aContent)) {
-      Clear();
-      return;
-    }
-  } else {
-    if (!InitWithEditor(aPresContext, aContent, aEditorBase)) {
-      Clear();
-      return;
-    }
+  if (!InitWithEditor(aPresContext, aContent, aEditorBase)) {
+    MOZ_LOG(sIMECOLog, LogLevel::Debug,
+            ("0x%p IMEContentObserver::Init() FAILED, due to InitWithEditor() "
+             "failure",
+             this));
+    Clear();
+    return;
   }
 
   if (firstInitialization) {
@@ -290,22 +225,17 @@ void IMEContentObserver::OnIMEReceivedFocus() {
   FlushMergeableNotifications();
 }
 
-bool IMEContentObserver::InitWithEditor(nsPresContext* aPresContext,
+bool IMEContentObserver::InitWithEditor(nsPresContext& aPresContext,
                                         nsIContent* aContent,
-                                        EditorBase* aEditorBase) {
-  MOZ_ASSERT(aEditorBase);
-
-  mEditableNode = IMEStateManager::GetRootEditableNode(aPresContext, aContent);
+                                        EditorBase& aEditorBase) {
+  mEditableNode = IMEStateManager::GetRootEditableNode(&aPresContext, aContent);
   if (NS_WARN_IF(!mEditableNode)) {
     return false;
   }
 
-  mEditorBase = aEditorBase;
-  if (NS_WARN_IF(!mEditorBase)) {
-    return false;
-  }
+  mEditorBase = &aEditorBase;
 
-  PresShell* presShell = aPresContext->GetPresShell();
+  RefPtr<PresShell> presShell = aPresContext.GetPresShell();
 
   // get selection and root content
   nsCOMPtr<nsISelectionController> selCon;
@@ -315,7 +245,7 @@ bool IMEContentObserver::InitWithEditor(nsPresContext* aPresContext,
       return false;
     }
 
-    frame->GetSelectionController(aPresContext, getter_AddRefs(selCon));
+    frame->GetSelectionController(&aPresContext, getter_AddRefs(selCon));
   } else {
     // mEditableNode is a document
     selCon = presShell;
@@ -330,15 +260,16 @@ bool IMEContentObserver::InitWithEditor(nsPresContext* aPresContext,
     return false;
   }
 
-  if (nsRange* selRange = mSelection->GetRangeAt(0)) {
+  if (const nsRange* selRange = mSelection->GetRangeAt(0)) {
     if (NS_WARN_IF(!selRange->GetStartContainer())) {
       return false;
     }
 
-    mRootContent =
-        selRange->GetStartContainer()->GetSelectionRootContent(presShell);
+    nsCOMPtr<nsINode> startContainer = selRange->GetStartContainer();
+    mRootContent = startContainer->GetSelectionRootContent(presShell);
   } else {
-    mRootContent = mEditableNode->GetSelectionRootContent(presShell);
+    nsCOMPtr<nsINode> editableNode = mEditableNode;
+    mRootContent = editableNode->GetSelectionRootContent(presShell);
   }
   if (!mRootContent && mEditableNode->IsDocument()) {
     // The document node is editable, but there are no contents, this document
@@ -350,61 +281,16 @@ bool IMEContentObserver::InitWithEditor(nsPresContext* aPresContext,
     return false;
   }
 
-  mDocShell = aPresContext->GetDocShell();
+  mDocShell = aPresContext.GetDocShell();
   if (NS_WARN_IF(!mDocShell)) {
     return false;
   }
 
   mDocumentObserver = new DocumentObserver(*this);
 
-  MOZ_ASSERT(!WasInitializedWithPlugin());
-
   return true;
 }
 
-bool IMEContentObserver::InitWithPlugin(nsPresContext* aPresContext,
-                                        nsIContent* aContent) {
-  if (NS_WARN_IF(!aContent) ||
-      NS_WARN_IF(aContent->GetDesiredIMEState().mEnabled != IMEState::PLUGIN)) {
-    return false;
-  }
-  nsIFrame* frame = aContent->GetPrimaryFrame();
-  if (NS_WARN_IF(!frame)) {
-    return false;
-  }
-  nsCOMPtr<nsISelectionController> selCon;
-  frame->GetSelectionController(aPresContext, getter_AddRefs(selCon));
-  if (NS_WARN_IF(!selCon)) {
-    return false;
-  }
-  mSelection = selCon->GetSelection(nsISelectionController::SELECTION_NORMAL);
-  if (NS_WARN_IF(!mSelection)) {
-    return false;
-  }
-
-  mEditorBase = nullptr;
-  mEditableNode = aContent;
-  mRootContent = aContent;
-  // Should be safe to clear mDocumentObserver here even though it *might*
-  // grab this instance because this is called by Init() and the callers of
-  // it and MaybeReinitialize() grabs this instance with local RefPtr.
-  // So, this won't cause refcount of this instance become 0.
-  mDocumentObserver = nullptr;
-
-  mDocShell = aPresContext->GetDocShell();
-  if (NS_WARN_IF(!mDocShell)) {
-    return false;
-  }
-
-  MOZ_ASSERT(WasInitializedWithPlugin());
-
-  return true;
-}
-
-bool IMEContentObserver::WasInitializedWithPlugin() const {
-  return mDocShell && !mEditorBase;
-}
-
 void IMEContentObserver::Clear() {
   mEditorBase = nullptr;
   mSelection = nullptr;
@@ -441,18 +327,13 @@ void IMEContentObserver::ObserveEditableNode() {
     mEditorBase->SetIMEContentObserver(this);
   }
 
-  if (!WasInitializedWithPlugin()) {
-    // Add text change observer only when this starts to observe
-    // non-plugin content since we cannot detect text changes in
-    // plugins.
-    mRootContent->AddMutationObserver(this);
-    // If it's in a document (should be so), we can use document observer to
-    // reduce redundant computation of text change offsets.
-    Document* doc = mRootContent->GetComposedDoc();
-    if (doc) {
-      RefPtr<DocumentObserver> documentObserver = mDocumentObserver;
-      documentObserver->Observe(doc);
-    }
+  mRootContent->AddMutationObserver(this);
+  // If it's in a document (should be so), we can use document observer to
+  // reduce redundant computation of text change offsets.
+  dom::Document* doc = mRootContent->GetComposedDoc();
+  if (doc) {
+    RefPtr<DocumentObserver> documentObserver = mDocumentObserver;
+    documentObserver->Observe(doc);
   }
 
   if (mDocShell) {
@@ -538,6 +419,9 @@ nsPresContext* IMEContentObserver::GetPresContext() const {
 void IMEContentObserver::Destroy() {
   // WARNING: When you change this method, you have to check Unlink() too.
 
+  // Note that don't send any notifications later from here.  I.e., notify
+  // IMEStateManager of the blur synchronously because IMEStateManager needs to
+  // stop notifying the main process if this is requested by the main process.
   NotifyIMEOfBlur();
   UnregisterObservers();
   Clear();
@@ -555,18 +439,18 @@ bool IMEContentObserver::Destroyed() const { return !mWidget; }
 
 void IMEContentObserver::DisconnectFromEventStateManager() { mESM = nullptr; }
 
-bool IMEContentObserver::MaybeReinitialize(nsIWidget* aWidget,
-                                           nsPresContext* aPresContext,
+bool IMEContentObserver::MaybeReinitialize(nsIWidget& aWidget,
+                                           nsPresContext& aPresContext,
                                            nsIContent* aContent,
-                                           EditorBase* aEditorBase) {
-  if (!IsObservingContent(aPresContext, aContent)) {
+                                           EditorBase& aEditorBase) {
+  if (!IsObservingContent(&aPresContext, aContent)) {
     return false;
   }
 
   if (GetState() == eState_StoppedObserving) {
     Init(aWidget, aPresContext, aContent, aEditorBase);
   }
-  return IsManaging(aPresContext, aContent);
+  return IsManaging(&aPresContext, aContent);
 }
 
 bool IMEContentObserver::IsManaging(nsPresContext* aPresContext,
@@ -605,10 +489,8 @@ IMEContentObserver::State IMEContentObserver::GetState() const {
 
 bool IMEContentObserver::IsObservingContent(nsPresContext* aPresContext,
                                             nsIContent* aContent) const {
-  return IsInitializedWithPlugin()
-             ? mRootContent == aContent && mRootContent != nullptr
-             : mEditableNode ==
-                   IMEStateManager::GetRootEditableNode(aPresContext, aContent);
+  return mEditableNode ==
+         IMEStateManager::GetRootEditableNode(aPresContext, aContent);
 }
 
 bool IMEContentObserver::IsEditorHandlingEventForComposition() const {
@@ -646,7 +528,7 @@ nsresult IMEContentObserver::GetSelectionAndRoot(
   return NS_OK;
 }
 
-void IMEContentObserver::OnSelectionChange(Selection& aSelection) {
+void IMEContentObserver::OnSelectionChange(dom::Selection& aSelection) {
   if (!mIsObserving) {
     return;
   }
@@ -705,17 +587,18 @@ nsresult IMEContentObserver::HandleQueryContentEvent(
                                    !mNeedsToNotifyIMEOfSelectionChange;
   if (isSelectionCacheAvailable && aEvent->mMessage == eQuerySelectedText &&
       aEvent->mInput.mSelectionType == SelectionType::eNormal) {
-    aEvent->mReply.mContentsRoot = mRootContent;
-    aEvent->mReply.mHasSelection = !mSelectionData.IsCollapsed();
-    aEvent->mReply.mOffset = mSelectionData.mOffset;
-    aEvent->mReply.mString = mSelectionData.String();
-    aEvent->mReply.mWritingMode = mSelectionData.GetWritingMode();
-    aEvent->mReply.mReversed = mSelectionData.mReversed;
-    aEvent->mSucceeded = true;
+    aEvent->EmplaceReply();
+    aEvent->mReply->mOffsetAndData.emplace(mSelectionData.mOffset,
+                                           mSelectionData.String(),
+                                           OffsetAndDataFor::SelectedString);
+    aEvent->mReply->mContentsRoot = mRootContent;
+    aEvent->mReply->mHasSelection = !mSelectionData.IsCollapsed();
+    aEvent->mReply->mWritingMode = mSelectionData.GetWritingMode();
+    aEvent->mReply->mReversed = mSelectionData.mReversed;
     MOZ_LOG(sIMECOLog, LogLevel::Debug,
             ("0x%p IMEContentObserver::HandleQueryContentEvent(aEvent={ "
-             "mMessage=%s })",
-             this, ToChar(aEvent->mMessage)));
+             "mMessage=%s, mReply=%s })",
+             this, ToChar(aEvent->mMessage), ToString(aEvent->mReply).c_str()));
     return NS_OK;
   }
 
@@ -757,7 +640,7 @@ nsresult IMEContentObserver::HandleQueryContentEvent(
   if (NS_WARN_IF(Destroyed())) {
     // If this has already destroyed during querying the content, the query
     // is outdated even if it's succeeded.  So, make the query fail.
-    aEvent->mSucceeded = false;
+    aEvent->mReply.reset();
     MOZ_LOG(sIMECOLog, LogLevel::Warning,
             ("0x%p IMEContentObserver::HandleQueryContentEvent(), WARNING, "
              "IMEContentObserver has been destroyed during the query, "
@@ -766,10 +649,10 @@ nsresult IMEContentObserver::HandleQueryContentEvent(
     return rv;
   }
 
-  if (!IsInitializedWithPlugin() &&
-      NS_WARN_IF(aEvent->mReply.mContentsRoot != mRootContent)) {
+  if (aEvent->Succeeded() &&
+      NS_WARN_IF(aEvent->mReply->mContentsRoot != mRootContent)) {
     // Focus has changed unexpectedly, so make the query fail.
-    aEvent->mSucceeded = false;
+    aEvent->mReply.reset();
   }
   return rv;
 }
@@ -798,13 +681,13 @@ bool IMEContentObserver::OnMouseButtonEvent(nsPresContext* aPresContext,
 
   RefPtr<IMEContentObserver> kungFuDeathGrip(this);
 
-  WidgetQueryContentEvent charAtPt(true, eQueryCharacterAtPoint,
-                                   aMouseEvent->mWidget);
-  charAtPt.mRefPoint = aMouseEvent->mRefPoint;
+  WidgetQueryContentEvent queryCharAtPointEvent(true, eQueryCharacterAtPoint,
+                                                aMouseEvent->mWidget);
+  queryCharAtPointEvent.mRefPoint = aMouseEvent->mRefPoint;
   ContentEventHandler handler(aPresContext);
-  handler.OnQueryCharacterAtPoint(&charAtPt);
-  if (NS_WARN_IF(!charAtPt.mSucceeded) ||
-      charAtPt.mReply.mOffset == WidgetQueryContentEvent::NOT_FOUND) {
+  handler.OnQueryCharacterAtPoint(&queryCharAtPointEvent);
+  if (NS_WARN_IF(queryCharAtPointEvent.Failed()) ||
+      queryCharAtPointEvent.DidNotFindChar()) {
     return false;
   }
 
@@ -818,23 +701,26 @@ bool IMEContentObserver::OnMouseButtonEvent(nsPresContext* aPresContext,
   // We should notify it with offset in the widget.
   nsIWidget* topLevelWidget = mWidget->GetTopLevelWidget();
   if (topLevelWidget && topLevelWidget != mWidget) {
-    charAtPt.mReply.mRect.MoveBy(topLevelWidget->WidgetToScreenOffset() -
-                                 mWidget->WidgetToScreenOffset());
+    queryCharAtPointEvent.mReply->mRect.MoveBy(
+        topLevelWidget->WidgetToScreenOffset() -
+        mWidget->WidgetToScreenOffset());
   }
   // The refPt is relative to its widget.
   // We should notify it with offset in the widget.
   if (aMouseEvent->mWidget != mWidget) {
-    charAtPt.mRefPoint += aMouseEvent->mWidget->WidgetToScreenOffset() -
-                          mWidget->WidgetToScreenOffset();
+    queryCharAtPointEvent.mRefPoint +=
+        aMouseEvent->mWidget->WidgetToScreenOffset() -
+        mWidget->WidgetToScreenOffset();
   }
 
   IMENotification notification(NOTIFY_IME_OF_MOUSE_BUTTON_EVENT);
   notification.mMouseButtonEventData.mEventMessage = aMouseEvent->mMessage;
-  notification.mMouseButtonEventData.mOffset = charAtPt.mReply.mOffset;
-  notification.mMouseButtonEventData.mCursorPos.Set(
-      charAtPt.mRefPoint.ToUnknownPoint());
-  notification.mMouseButtonEventData.mCharRect.Set(
-      charAtPt.mReply.mRect.ToUnknownRect());
+  notification.mMouseButtonEventData.mOffset =
+      queryCharAtPointEvent.mReply->StartOffset();
+  notification.mMouseButtonEventData.mCursorPos =
+      queryCharAtPointEvent.mRefPoint;
+  notification.mMouseButtonEventData.mCharRect =
+      queryCharAtPointEvent.mReply->mRect;
   notification.mMouseButtonEventData.mButton = aMouseEvent->mButton;
   notification.mMouseButtonEventData.mButtons = aMouseEvent->mButtons;
   notification.mMouseButtonEventData.mModifiers = aMouseEvent->mModifiers;
@@ -853,7 +739,10 @@ bool IMEContentObserver::OnMouseButtonEvent(nsPresContext* aPresContext,
 
 void IMEContentObserver::CharacterDataWillChange(
     nsIContent* aContent, const CharacterDataChangeInfo& aInfo) {
-  NS_ASSERTION(aContent->IsText(), "character data changed for non-text node");
+  if (!aContent->IsText()) {
+    return;  // Ignore if it's a comment node or something other invisible data
+             // node.
+  }
   MOZ_ASSERT(mPreCharacterDataChangeLength < 0,
              "CharacterDataChanged() should've reset "
              "mPreCharacterDataChangeLength");
@@ -881,7 +770,10 @@ void IMEContentObserver::CharacterDataWillChange(
 
 void IMEContentObserver::CharacterDataChanged(
     nsIContent* aContent, const CharacterDataChangeInfo& aInfo) {
-  NS_ASSERTION(aContent->IsText(), "character data changed for non-text node");
+  if (!aContent->IsText()) {
+    return;  // Ignore if it's a comment node or something other invisible data
+             // node.
+  }
 
   if (!NeedsTextChangeNotification() ||
       !nsContentUtils::IsInSameAnonymousTree(mRootContent, aContent)) {
@@ -1085,51 +977,6 @@ void IMEContentObserver::ContentRemoved(nsIContent* aChild,
   MaybeNotifyIMEOfTextChange(data);
 }
 
-void IMEContentObserver::AttributeWillChange(dom::Element* aElement,
-                                             int32_t aNameSpaceID,
-                                             nsAtom* aAttribute,
-                                             int32_t aModType) {
-  if (!NeedsTextChangeNotification()) {
-    return;
-  }
-
-  mPreAttrChangeLength =
-      ContentEventHandler::GetNativeTextLengthBefore(aElement, mRootContent);
-}
-
-void IMEContentObserver::AttributeChanged(dom::Element* aElement,
-                                          int32_t aNameSpaceID,
-                                          nsAtom* aAttribute, int32_t aModType,
-                                          const nsAttrValue* aOldValue) {
-  if (!NeedsTextChangeNotification()) {
-    return;
-  }
-
-  mEndOfAddedTextCache.Clear();
-  mStartOfRemovingTextRangeCache.Clear();
-
-  uint32_t postAttrChangeLength =
-      ContentEventHandler::GetNativeTextLengthBefore(aElement, mRootContent);
-  if (postAttrChangeLength == mPreAttrChangeLength) {
-    return;
-  }
-  // First, compute text range which were added during a document change.
-  MaybeNotifyIMEOfAddedTextDuringDocumentChange();
-  // Then, compute the new text changed caused by this attribute change.
-  uint32_t start;
-  nsresult rv = ContentEventHandler::GetFlatTextLengthInRange(
-      NodePosition(mRootContent, 0), NodePositionBefore(aElement, 0),
-      mRootContent, &start, LINE_BREAK_TYPE_NATIVE);
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return;
-  }
-
-  TextChangeData data(
-      start, start + mPreAttrChangeLength, start + postAttrChangeLength,
-      IsEditorHandlingEventForComposition(), IsEditorComposing());
-  MaybeNotifyIMEOfTextChange(data);
-}
-
 void IMEContentObserver::ClearAddedNodesDuringDocumentChange() {
   mFirstAddedContainer = mLastAddedContainer = nullptr;
   mFirstAddedContent = mLastAddedContent = nullptr;
@@ -1307,7 +1154,7 @@ void IMEContentObserver::PostTextChangeNotification() {
   MOZ_LOG(sIMECOLog, LogLevel::Debug,
           ("0x%p IMEContentObserver::PostTextChangeNotification("
            "mTextChangeData=%s)",
-           this, TextChangeDataToString(mTextChangeData).get()));
+           this, ToString(mTextChangeData).c_str()));
 
   MOZ_ASSERT(mTextChangeData.IsValid(),
              "mTextChangeData must have text change data");
@@ -1338,7 +1185,7 @@ void IMEContentObserver::MaybeNotifyIMEOfTextChange(
   MOZ_LOG(sIMECOLog, LogLevel::Debug,
           ("0x%p IMEContentObserver::MaybeNotifyIMEOfTextChange("
            "aTextChangeData=%s)",
-           this, TextChangeDataToString(aTextChangeData).get()));
+           this, ToString(aTextChangeData).c_str()));
 
   mTextChangeData += aTextChangeData;
   PostTextChangeNotification();
@@ -1406,35 +1253,35 @@ void IMEContentObserver::MaybeNotifyCompositionEventHandled() {
 bool IMEContentObserver::UpdateSelectionCache(bool aRequireFlush /* = true */) {
   MOZ_ASSERT(IsSafeToNotifyIME());
 
-  if (WasInitializedWithPlugin()) {
-    return false;
-  }
-
   mSelectionData.ClearSelectionData();
 
   // XXX Cannot we cache some information for reducing the cost to compute
   //     selection offset and writing mode?
-  WidgetQueryContentEvent selection(true, eQuerySelectedText, mWidget);
-  selection.mNeedsToFlushLayout = aRequireFlush;
+  WidgetQueryContentEvent querySelectedTextEvent(true, eQuerySelectedText,
+                                                 mWidget);
+  querySelectedTextEvent.mNeedsToFlushLayout = aRequireFlush;
   ContentEventHandler handler(GetPresContext());
-  handler.OnQuerySelectedText(&selection);
-  if (NS_WARN_IF(!selection.mSucceeded) ||
-      NS_WARN_IF(selection.mReply.mContentsRoot != mRootContent)) {
+  handler.OnQuerySelectedText(&querySelectedTextEvent);
+  if (NS_WARN_IF(querySelectedTextEvent.DidNotFindSelection()) ||
+      NS_WARN_IF(querySelectedTextEvent.mReply->mContentsRoot !=
+                 mRootContent)) {
     return false;
   }
+  MOZ_ASSERT(querySelectedTextEvent.mReply->mOffsetAndData.isSome());
 
-  mFocusedWidget = selection.mReply.mFocusedWidget;
-  mSelectionData.mOffset = selection.mReply.mOffset;
-  *mSelectionData.mString = selection.mReply.mString;
-  mSelectionData.SetWritingMode(selection.GetWritingMode());
-  mSelectionData.mReversed = selection.mReply.mReversed;
+  mFocusedWidget = querySelectedTextEvent.mReply->mFocusedWidget;
+  mSelectionData.mOffset = querySelectedTextEvent.mReply->StartOffset();
+  *mSelectionData.mString = querySelectedTextEvent.mReply->DataRef();
+  mSelectionData.SetWritingMode(
+      querySelectedTextEvent.mReply->WritingModeRef());
+  mSelectionData.mReversed = querySelectedTextEvent.mReply->mReversed;
 
   // WARNING: Don't modify the reason of selection change here.
 
   MOZ_LOG(sIMECOLog, LogLevel::Debug,
           ("0x%p IMEContentObserver::UpdateSelectionCache(), "
            "mSelectionData=%s",
-           this, SelectionChangeDataToString(mSelectionData).get()));
+           this, ToString(mSelectionData).c_str()));
 
   return mSelectionData.IsValid();
 }
@@ -1726,6 +1573,14 @@ IMEContentObserver::IMENotificationSender::Run() {
   if (observer->mNeedsToNotifyIMEOfTextChange) {
     observer->mNeedsToNotifyIMEOfTextChange = false;
     SendTextChange();
+    // Even if the observer hasn't received selection change, let's try to send
+    // selection change notification to IME because selection start offset may
+    // be changed if the previous contents of selection start are changed.  For
+    // example, when previous `<p>` element of another `<p>` element which
+    // contains caret is removed by a DOM mutation, selection change event
+    // won't be fired, but selection start offset should be decreased by the
+    // length of removed `<p>` element.
+    observer->mNeedsToNotifyIMEOfSelectionChange = true;
   }
 
   // If a text change notification causes another text change again, we should
@@ -1912,7 +1767,7 @@ void IMEContentObserver::IMENotificationSender::SendSelectionChange() {
           ("0x%p IMEContentObserver::IMENotificationSender::"
            "SendSelectionChange(), sending NOTIFY_IME_OF_SELECTION_CHANGE... "
            "newSelChangeData=%s",
-           this, SelectionChangeDataToString(newSelChangeData).get()));
+           this, ToString(newSelChangeData).c_str()));
 
   IMENotification notification(NOTIFY_IME_OF_SELECTION_CHANGE);
   notification.SetData(observer->mSelectionData);
@@ -1967,7 +1822,7 @@ void IMEContentObserver::IMENotificationSender::SendTextChange() {
           ("0x%p IMEContentObserver::IMENotificationSender::"
            "SendTextChange(), sending NOTIFY_IME_OF_TEXT_CHANGE... "
            "mIMEContentObserver={ mTextChangeData=%s }",
-           this, TextChangeDataToString(observer->mTextChangeData).get()));
+           this, ToString(observer->mTextChangeData).c_str()));
 
   IMENotification notification(NOTIFY_IME_OF_TEXT_CHANGE);
   notification.SetData(observer->mTextChangeData);
@@ -2108,16 +1963,16 @@ NS_INTERFACE_MAP_END
 NS_IMPL_CYCLE_COLLECTING_ADDREF(IMEContentObserver::DocumentObserver)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(IMEContentObserver::DocumentObserver)
 
-void IMEContentObserver::DocumentObserver::Observe(Document* aDocument) {
+void IMEContentObserver::DocumentObserver::Observe(dom::Document* aDocument) {
   MOZ_ASSERT(aDocument);
 
   // Guarantee that aDocument won't be destroyed during a call of
   // StopObserving().
-  RefPtr<Document> newDocument = aDocument;
+  RefPtr<dom::Document> newDocument = aDocument;
 
   StopObserving();
 
-  mDocument = newDocument.forget();
+  mDocument = std::move(newDocument);
   mDocument->AddObserver(this);
 }
 
@@ -2127,10 +1982,10 @@ void IMEContentObserver::DocumentObserver::StopObserving() {
   }
 
   // Grab IMEContentObserver which could be destroyed during method calls.
-  RefPtr<IMEContentObserver> observer = mIMEContentObserver.forget();
+  RefPtr<IMEContentObserver> observer = std::move(mIMEContentObserver);
 
   // Stop observing the document first.
-  RefPtr<Document> document = mDocument.forget();
+  RefPtr<dom::Document> document = std::move(mDocument);
   document->RemoveObserver(this);
 
   // Notify IMEContentObserver of ending of document updates if this already
@@ -2149,7 +2004,8 @@ void IMEContentObserver::DocumentObserver::Destroy() {
   mIMEContentObserver = nullptr;
 }
 
-void IMEContentObserver::DocumentObserver::BeginUpdate(Document* aDocument) {
+void IMEContentObserver::DocumentObserver::BeginUpdate(
+    dom::Document* aDocument) {
   if (NS_WARN_IF(Destroyed()) || NS_WARN_IF(!IsObserving())) {
     return;
   }
@@ -2157,7 +2013,7 @@ void IMEContentObserver::DocumentObserver::BeginUpdate(Document* aDocument) {
   mIMEContentObserver->BeginDocumentUpdate();
 }
 
-void IMEContentObserver::DocumentObserver::EndUpdate(Document* aDocument) {
+void IMEContentObserver::DocumentObserver::EndUpdate(dom::Document* aDocument) {
   if (NS_WARN_IF(Destroyed()) || NS_WARN_IF(!IsObserving()) ||
       NS_WARN_IF(!IsUpdating())) {
     return;
