@@ -9,9 +9,11 @@
 #include "mozilla/IMEStateManager.h"
 
 #include "mozilla/Attributes.h"
+#include "mozilla/AutoRestore.h"
 #include "mozilla/EditorBase.h"
 #include "mozilla/EventListenerManager.h"
 #include "mozilla/EventStates.h"
+#include "mozilla/EventStateManager.h"
 #include "mozilla/MouseEvents.h"
 #include "mozilla/PresShell.h"
 #include "mozilla/StaticPrefs_dom.h"
@@ -76,6 +78,10 @@ InputContext::Origin IMEStateManager::sOrigin = InputContext::ORIGIN_MAIN;
 InputContext IMEStateManager::sActiveChildInputContext;
 bool IMEStateManager::sInstalledMenuKeyboardListener = false;
 bool IMEStateManager::sIsGettingNewIMEState = false;
+bool IMEStateManager::sCleaningUpForStoppingIMEStateManagement = false;
+bool IMEStateManager::sIsActive = false;
+Maybe<IMEStateManager::PendingFocusedBrowserSwitchingData>
+    IMEStateManager::sPendingFocusedBrowserSwitchingData;
 
 // static
 void IMEStateManager::Init() {
@@ -88,9 +94,12 @@ void IMEStateManager::Init() {
 void IMEStateManager::Shutdown() {
   MOZ_LOG(
       sISMLog, LogLevel::Info,
-      ("Shutdown(), sTextCompositions=0x%p, sTextCompositions->Length()=%zu",
-       sTextCompositions, sTextCompositions ? sTextCompositions->Length() : 0));
+      ("Shutdown(), sTextCompositions=0x%p, sTextCompositions->Length()=%zu, "
+       "sPendingFocusedBrowserSwitchingData.isSome()=%s",
+       sTextCompositions, sTextCompositions ? sTextCompositions->Length() : 0,
+       GetBoolName(sPendingFocusedBrowserSwitchingData.isSome())));
 
+  sPendingFocusedBrowserSwitchingData.reset();
   MOZ_ASSERT(!sTextCompositions || !sTextCompositions->Length());
   delete sTextCompositions;
   sTextCompositions = nullptr;
@@ -105,6 +114,43 @@ void IMEStateManager::OnFocusMovedBetweenBrowsers(BrowserParent* aBlur,
   MOZ_ASSERT(aBlur != aFocus);
   MOZ_ASSERT(XRE_IsParentProcess());
 
+  if (sPendingFocusedBrowserSwitchingData.isSome()) {
+    MOZ_ASSERT(aBlur ==
+               sPendingFocusedBrowserSwitchingData.ref().mBrowserParentFocused);
+    // If focus is not changed between browsers actually, we need to do
+    // nothing here.  Let's cancel handling what this method does.
+    if (sPendingFocusedBrowserSwitchingData.ref().mBrowserParentBlurred ==
+        aFocus) {
+      sPendingFocusedBrowserSwitchingData.reset();
+      MOZ_LOG(sISMLog, LogLevel::Info,
+              ("  OnFocusMovedBetweenBrowsers(), canceled all pending focus "
+               "moves between browsers"));
+      return;
+    }
+    aBlur = sPendingFocusedBrowserSwitchingData.ref().mBrowserParentBlurred;
+    sPendingFocusedBrowserSwitchingData.ref().mBrowserParentFocused = aFocus;
+    MOZ_ASSERT(aBlur != aFocus);
+  }
+
+  // If application was inactive, but is now activated, and the last focused
+  // this is called by BrowserParent::UnsetTopLevelWebFocusAll() from
+  // nsFocusManager::WindowRaised().  If a content has focus in a remote
+  // process and it has composition, it may get focus back later and the
+  // composition shouldn't be commited now.  Therefore, we should put off to
+  // handle this until getting another call of this method or a call of
+  //`OnFocusChangeInternal()`.
+  if (aBlur && !aFocus && !sIsActive && sWidget && sTextCompositions &&
+      sTextCompositions->GetCompositionFor(sWidget)) {
+    if (sPendingFocusedBrowserSwitchingData.isNothing()) {
+      sPendingFocusedBrowserSwitchingData.emplace(aBlur, aFocus);
+    }
+    MOZ_LOG(sISMLog, LogLevel::Debug,
+            ("  OnFocusMovedBetweenBrowsers(), put off to handle it until "
+             "next OnFocusChangeInternal() call"));
+    return;
+  }
+  sPendingFocusedBrowserSwitchingData.reset();
+
   nsCOMPtr<nsIWidget> oldWidget = sWidget;
   nsCOMPtr<nsIWidget> newWidget =
       aFocus ? aFocus->GetTextInputHandlingWidget() : nullptr;
@@ -203,12 +249,22 @@ void IMEStateManager::StopIMEStateManagement() {
   // NOTE: Don't set input context from here since this has already lost
   //       the rights to change input context.
 
+  // The requestee of this method in the main process must destroy its
+  // active IMEContentObserver for making existing composition end and
+  // make it be possible to start new composition in new focused process.
+  // Therefore, we shouldn't notify the main process of any changes which
+  // occurred after here.
+  AutoRestore<bool> restoreStoppingIMEStateManagementState(
+      sCleaningUpForStoppingIMEStateManagement);
+  sCleaningUpForStoppingIMEStateManagement = true;
+
   if (sTextCompositions && sPresContext) {
     NotifyIME(REQUEST_TO_COMMIT_COMPOSITION, sPresContext, nullptr);
   }
   sActiveInputContextWidget = nullptr;
   sPresContext = nullptr;
   sContent = nullptr;
+  sIsActive = false;
   DestroyIMEContentObserver();
 }
 
@@ -375,8 +431,16 @@ nsresult IMEStateManager::OnChangeFocus(nsPresContext* aPresContext,
 nsresult IMEStateManager::OnChangeFocusInternal(nsPresContext* aPresContext,
                                                 nsIContent* aContent,
                                                 InputContextAction aAction) {
-  bool remoteHasFocus =
-      BrowserParent::GetFrom(aContent) || BrowserBridgeChild::GetFrom(aContent);
+  bool remoteHasFocus = EventStateManager::IsRemoteTarget(aContent);
+  // If we've handled focused content, we were inactive but now active,
+  // a remote process has focus, and setting focus to same content in the main
+  // process, it means that we're restoring focus without changing DOM focus
+  // both in the main process and the remote process.
+  const bool restoringContextForRemoteContent =
+      XRE_IsParentProcess() && remoteHasFocus && !sIsActive && aPresContext &&
+      sPresContext && sContent && sPresContext.get() == aPresContext &&
+      sContent.get() == aContent &&
+      aAction.mFocusChange != InputContextAction::MENU_GOT_PSEUDO_FOCUS;
 
   MOZ_LOG(sISMLog, LogLevel::Info,
           ("OnChangeFocusInternal(aPresContext=0x%p (available: %s), "
@@ -384,14 +448,40 @@ nsresult IMEStateManager::OnChangeFocusInternal(nsPresContext* aPresContext,
            "mFocusChange=%s }), "
            "sPresContext=0x%p (available: %s), sContent=0x%p, "
            "sWidget=0x%p (available: %s), BrowserParent::GetFocused()=0x%p, "
-           "sActiveIMEContentObserver=0x%p, sInstalledMenuKeyboardListener=%s",
+           "sActiveIMEContentObserver=0x%p, sInstalledMenuKeyboardListener=%s, "
+           "sIsActive=%s, restoringContextForRemoteContent=%s",
            aPresContext, GetBoolName(CanHandleWith(aPresContext)), aContent,
            GetBoolName(remoteHasFocus), ToString(aAction.mCause).c_str(),
            ToString(aAction.mFocusChange).c_str(), sPresContext.get(),
            GetBoolName(CanHandleWith(sPresContext)), sContent.get(), sWidget,
            GetBoolName(sWidget && !sWidget->Destroyed()),
            BrowserParent::GetFocused(), sActiveIMEContentObserver.get(),
-           GetBoolName(sInstalledMenuKeyboardListener)));
+           GetBoolName(sInstalledMenuKeyboardListener), GetBoolName(sIsActive),
+           GetBoolName(restoringContextForRemoteContent)));
+
+  sIsActive = !!aPresContext;
+  if (sPendingFocusedBrowserSwitchingData.isSome()) {
+    MOZ_ASSERT(XRE_IsParentProcess());
+    nsCOMPtr<nsIContent> currentContent = sContent.get();
+    RefPtr<nsPresContext> currentPresContext = sPresContext.get();
+    RefPtr<BrowserParent> browserParentBlurred =
+        sPendingFocusedBrowserSwitchingData.ref().mBrowserParentBlurred;
+    RefPtr<BrowserParent> browserParentFocused =
+        sPendingFocusedBrowserSwitchingData.ref().mBrowserParentFocused;
+    OnFocusMovedBetweenBrowsers(browserParentBlurred, browserParentFocused);
+    // If another call of this method happens during the
+    // OnFocusMovedBetweenBrowsers call, we shouldn't take back focus to
+    // the old one.
+    if (currentContent != sContent.get() ||
+        currentPresContext != sPresContext.get()) {
+      MOZ_LOG(sISMLog, LogLevel::Debug,
+              ("  OnChangeFocusInternal(aPresContext=0x%p, aContent=0x%p) "
+               "stoped handling it because the focused content was changed to "
+               "sPresContext=0x%p, sContent=0x%p by another call",
+               aPresContext, aContent, sPresContext.get(), sContent.get()));
+      return NS_OK;
+    }
+  }
 
   // If new aPresShell has been destroyed, this should handle the focus change
   // as nobody is getting focus.
@@ -409,7 +499,7 @@ nsresult IMEStateManager::OnChangeFocusInternal(nsPresContext* aPresContext,
   bool focusActuallyChanging =
       (sContent != aContent || sPresContext != aPresContext ||
        oldWidget != newWidget ||
-       (remoteHasFocus &&
+       (remoteHasFocus && !restoringContextForRemoteContent &&
         (aAction.mFocusChange != InputContextAction::MENU_GOT_PSEUDO_FOCUS)));
 
   // If old widget has composition, we may need to commit composition since
@@ -456,15 +546,14 @@ nsresult IMEStateManager::OnChangeFocusInternal(nsPresContext* aPresContext,
 
   if (!aPresContext) {
     MOZ_LOG(sISMLog, LogLevel::Debug,
-            ("  OnChangeFocusInternal(), "
-             "no nsPresContext is being activated"));
+            ("  OnChangeFocusInternal(), no nsPresContext is being activated"));
     return NS_OK;
   }
 
   if (NS_WARN_IF(!newWidget)) {
     MOZ_LOG(sISMLog, LogLevel::Error,
-            ("  OnChangeFocusInternal(), FAILED due to "
-             "no widget to manage its IME state"));
+            ("  OnChangeFocusInternal(), FAILED due to no widget to manage its "
+             "IME state"));
     return NS_OK;
   }
 
@@ -475,7 +564,7 @@ nsresult IMEStateManager::OnChangeFocusInternal(nsPresContext* aPresContext,
   // If a child process has focus, we should disable IME state until the child
   // process actually gets focus because if user types keys before that they
   // are handled by IME.
-  IMEState newState = remoteHasFocus ? IMEState(IMEState::DISABLED)
+  IMEState newState = remoteHasFocus ? IMEState(IMEEnabled::Disabled)
                                      : GetNewIMEState(aPresContext, aContent);
   bool setIMEState = true;
 
@@ -497,22 +586,21 @@ nsresult IMEStateManager::OnChangeFocusInternal(nsPresContext* aPresContext,
       }
     } else if (focusActuallyChanging) {
       InputContext context = newWidget->GetInputContext();
-      if (context.mIMEState.mEnabled == IMEState::DISABLED &&
+      if (context.mIMEState.mEnabled == IMEEnabled::Disabled &&
           context.mOrigin == InputContext::ORIGIN_CONTENT) {
         setIMEState = false;
         MOZ_LOG(sISMLog, LogLevel::Debug,
-                ("  OnChangeFocusInternal(), doesn't set IME "
-                 "state because focused element (or document) is in a child "
-                 "process "
-                 "and the IME state is already disabled by a remote process"));
+                ("  OnChangeFocusInternal(), doesn't set IME state because "
+                 "focused element (or document) is in a child process and the "
+                 "IME state is already disabled by a remote process"));
       } else {
         // When new remote process gets focus, we should forget input context
         // coming from old focused remote process.
         ResetActiveChildInputContext();
         MOZ_LOG(sISMLog, LogLevel::Debug,
-                ("  OnChangeFocusInternal(), will disable IME "
-                 "until new focused element (or document) in the child process "
-                 "will get focus actually"));
+                ("  OnChangeFocusInternal(), will disable IME until new "
+                 "focused element (or document) in the child process will get "
+                 "focus actually"));
       }
     } else if (newWidget->GetInputContext().mOrigin !=
                InputContext::ORIGIN_MAIN) {
@@ -522,11 +610,10 @@ nsresult IMEStateManager::OnChangeFocusInternal(nsPresContext* aPresContext,
       // composition.  Then, we shouldn't commit the composition with making
       // IME state disabled.
       setIMEState = false;
-      MOZ_LOG(sISMLog, LogLevel::Debug,
-              ("  OnChangeFocusInternal(), doesn't set IME "
-               "state because focused element (or document) is already in the "
-               "child "
-               "process"));
+      MOZ_LOG(
+          sISMLog, LogLevel::Debug,
+          ("  OnChangeFocusInternal(), doesn't set IME state because focused "
+           "element (or document) is already in the child process"));
     }
   } else {
     // When this process gets focus, we should forget input context coming
@@ -541,8 +628,8 @@ nsresult IMEStateManager::OnChangeFocusInternal(nsPresContext* aPresContext,
       InputContext context = newWidget->GetInputContext();
       if (context.mIMEState.mEnabled == newState.mEnabled) {
         MOZ_LOG(sISMLog, LogLevel::Debug,
-                ("  OnChangeFocusInternal(), "
-                 "neither focus nor IME state is changing"));
+                ("  OnChangeFocusInternal(), neither focus nor IME state is "
+                 "changing"));
         return NS_OK;
       }
       aAction.mFocusChange = InputContextAction::FOCUS_NOT_CHANGED;
@@ -556,7 +643,7 @@ nsresult IMEStateManager::OnChangeFocusInternal(nsPresContext* aPresContext,
     } else if (aAction.mFocusChange == InputContextAction::FOCUS_NOT_CHANGED) {
       // If aContent isn't null or aContent is null but editable, somebody gets
       // focus.
-      bool gotFocus = aContent || (newState.mEnabled == IMEState::ENABLED);
+      bool gotFocus = aContent || (newState.mEnabled == IMEEnabled::Enabled);
       aAction.mFocusChange = gotFocus ? InputContextAction::GOT_FOCUS
                                       : InputContextAction::LOST_FOCUS;
     }
@@ -577,20 +664,8 @@ nsresult IMEStateManager::OnChangeFocusInternal(nsPresContext* aPresContext,
   sPresContext = aPresContext;
   sContent = aContent;
 
-  // Don't call CreateIMEContentObserver() here except when a plugin gets
-  // focus because it will be called from the focus event handler of focused
-  // editor.
-  if (newState.mEnabled == IMEState::PLUGIN) {
-    CreateIMEContentObserver(nullptr);
-    if (sActiveIMEContentObserver) {
-      MOZ_LOG(
-          sISMLog, LogLevel::Debug,
-          ("  OnChangeFocusInternal(), an "
-           "IMEContentObserver instance is created for plugin and trying to "
-           "flush its pending notifications..."));
-      sActiveIMEContentObserver->TryToFlushPendingNotifications(false);
-    }
-  }
+  // Don't call CreateIMEContentObserver() here  because it will be called from
+  // the focus event handler of focused editor.
 
   return NS_OK;
 }
@@ -601,18 +676,10 @@ void IMEStateManager::OnInstalledMenuKeyboardListener(bool aInstalling) {
       sISMLog, LogLevel::Info,
       ("OnInstalledMenuKeyboardListener(aInstalling=%s), "
        "sInstalledMenuKeyboardListener=%s, BrowserParent::GetFocused()=0x%p, "
-       "sActiveChildInputContext={ mIMEState={ mEnabled=%s, mOpen=%s }, "
-       "mHTMLInputType=\"%s\", mHTMLInputInputmode=\"%s\", mActionHint=\"%s\", "
-       "mInPrivateBrowsing=%s }",
+       "sActiveChildInputContext=%s",
        GetBoolName(aInstalling), GetBoolName(sInstalledMenuKeyboardListener),
        BrowserParent::GetFocused(),
-       ToString(sActiveChildInputContext.mIMEState.mEnabled).c_str(),
-       ToString(sActiveChildInputContext.mIMEState.mOpen).c_str(),
-       NS_ConvertUTF16toUTF8(sActiveChildInputContext.mHTMLInputType).get(),
-       NS_ConvertUTF16toUTF8(sActiveChildInputContext.mHTMLInputInputmode)
-           .get(),
-       NS_ConvertUTF16toUTF8(sActiveChildInputContext.mActionHint).get(),
-       GetBoolName(sActiveChildInputContext.mInPrivateBrowsing)));
+       ToString(sActiveChildInputContext).c_str()));
 
   sInstalledMenuKeyboardListener = aInstalling;
 
@@ -660,8 +727,8 @@ bool IMEStateManager::OnMouseButtonEventInEditor(
     return false;
   }
 
-  bool consumed =
-      sActiveIMEContentObserver->OnMouseButtonEvent(aPresContext, aMouseEvent);
+  RefPtr<IMEContentObserver> observer = sActiveIMEContentObserver;
+  bool consumed = observer->OnMouseButtonEvent(aPresContext, aMouseEvent);
 
   if (MOZ_LOG_TEST(sISMLog, LogLevel::Info)) {
     nsAutoString eventType;
@@ -766,7 +833,7 @@ void IMEStateManager::OnFocusInEditor(nsPresContext* aPresContext,
     DestroyIMEContentObserver();
   }
 
-  CreateIMEContentObserver(&aEditorBase);
+  CreateIMEContentObserver(aEditorBase);
 
   // Let's flush the focus notification now.
   if (sActiveIMEContentObserver) {
@@ -805,18 +872,61 @@ void IMEStateManager::OnEditorDestroying(EditorBase& aEditorBase) {
   sActiveIMEContentObserver->SuppressNotifyingIME();
 }
 
+void IMEStateManager::OnReFocus(nsPresContext* aPresContext,
+                                nsIContent& aContent) {
+  MOZ_LOG(sISMLog, LogLevel::Info,
+          ("OnReFocus(aPresContext=0x%p, aContent=0x%p), "
+           "sActiveIMEContentObserver=0x%p, sContent=0x%p",
+           aPresContext, &aContent, sActiveIMEContentObserver.get(),
+           sContent.get()));
+
+  if (NS_WARN_IF(!sWidget) || NS_WARN_IF(sWidget->Destroyed())) {
+    return;
+  }
+
+  if (!sActiveIMEContentObserver ||
+      !sActiveIMEContentObserver->IsManaging(aPresContext, &aContent)) {
+    MOZ_LOG(sISMLog, LogLevel::Debug,
+            ("  OnReFocus(), there is no valid IMEContentObserver, so we don't "
+             "manage this. Ignore this"));
+    return;
+  }
+
+  MOZ_ASSERT(&aContent == sContent.get());
+
+  if (!UserActivation::IsHandlingUserInput() ||
+      UserActivation::IsHandlingKeyboardInput()) {
+    return;
+  }
+
+  nsCOMPtr<nsIWidget> widget(sWidget);
+
+  // Don't update IME state during composition.
+  if (sTextCompositions) {
+    if (TextComposition* composition =
+            sTextCompositions->GetCompositionFor(widget)) {
+      if (composition->IsComposing()) {
+        return;
+      }
+    }
+  }
+
+  InputContextAction action(InputContextAction::CAUSE_UNKNOWN,
+                            InputContextAction::FOCUS_NOT_CHANGED);
+  IMEState newState = GetNewIMEState(aPresContext, &aContent);
+  SetIMEState(newState, aPresContext, &aContent, widget, action, sOrigin);
+}
+
 // static
 void IMEStateManager::UpdateIMEState(const IMEState& aNewIMEState,
                                      nsIContent* aContent,
-                                     EditorBase* aEditorBase) {
+                                     EditorBase& aEditorBase) {
   MOZ_LOG(
       sISMLog, LogLevel::Info,
-      ("UpdateIMEState(aNewIMEState={ mEnabled=%s, "
-       "mOpen=%s }, aContent=0x%p, aEditorBase=0x%p), "
+      ("UpdateIMEState(aNewIMEState=%s, aContent=0x%p, aEditorBase=0x%p), "
        "sPresContext=0x%p, sContent=0x%p, sWidget=0x%p (available: %s), "
        "sActiveIMEContentObserver=0x%p, sIsGettingNewIMEState=%s",
-       ToString(aNewIMEState.mEnabled).c_str(),
-       ToString(aNewIMEState.mOpen).c_str(), aContent, aEditorBase,
+       ToString(aNewIMEState).c_str(), aContent, &aEditorBase,
        sPresContext.get(), sContent.get(), sWidget,
        GetBoolName(sWidget && !sWidget->Destroyed()),
        sActiveIMEContentObserver.get(), GetBoolName(sIsGettingNewIMEState)));
@@ -828,15 +938,7 @@ void IMEStateManager::UpdateIMEState(const IMEState& aNewIMEState,
     return;
   }
 
-  RefPtr<PresShell> presShell;
-  if (!aEditorBase) {
-    MOZ_ASSERT(aContent, "we must have content");
-    Document* doc = aContent->OwnerDoc();
-    presShell = doc->GetPresShell();
-  } else {
-    presShell = aEditorBase->GetPresShell();
-  }
-
+  RefPtr<PresShell> presShell(aEditorBase.GetPresShell());
   if (NS_WARN_IF(!presShell)) {
     MOZ_LOG(sISMLog, LogLevel::Error,
             ("  UpdateIMEState(), FAILED due to "
@@ -883,11 +985,15 @@ void IMEStateManager::UpdateIMEState(const IMEState& aNewIMEState,
     return;
   }
 
-  nsCOMPtr<nsIWidget> widget(sWidget);
+  OwningNonNull<nsIWidget> widget(*sWidget);
 
   MOZ_ASSERT(!sPresContext->GetTextInputHandlingWidget() ||
              sPresContext->GetTextInputHandlingWidget() == widget);
 
+  // TODO: Investigate if we could put off to initialize IMEContentObserver
+  //       later because a lot of callers need to be marked as
+  //       MOZ_CAN_RUN_SCRIPT otherwise.
+
   // Even if there is active IMEContentObserver, it may not be observing the
   // editor with current editable root content due to reframed.  In such case,
   // We should try to reinitialize the IMEContentObserver.
@@ -896,7 +1002,8 @@ void IMEStateManager::UpdateIMEState(const IMEState& aNewIMEState,
             ("  UpdateIMEState(), try to reinitialize the "
              "active IMEContentObserver"));
     RefPtr<IMEContentObserver> contentObserver = sActiveIMEContentObserver;
-    if (!contentObserver->MaybeReinitialize(widget, sPresContext, aContent,
+    OwningNonNull<nsPresContext> presContext(*sPresContext);
+    if (!contentObserver->MaybeReinitialize(widget, presContext, aContent,
                                             aEditorBase)) {
       MOZ_LOG(sISMLog, LogLevel::Error,
               ("  UpdateIMEState(), failed to reinitialize the "
@@ -952,6 +1059,10 @@ void IMEStateManager::UpdateIMEState(const IMEState& aNewIMEState,
     }
   }
 
+  // XXX Update sContent when aContent is focused content?
+  NS_ASSERTION(sContent.get() == aContent,
+               "sContent and aContent are mismatched.");
+
   if (createTextStateManager) {
     // XXX In this case, it might not be enough safe to notify IME of anything.
     //     So, don't try to flush pending notifications of IMEContentObserver
@@ -971,25 +1082,25 @@ IMEState IMEStateManager::GetNewIMEState(nsPresContext* aPresContext,
 
   if (!CanHandleWith(aPresContext)) {
     MOZ_LOG(sISMLog, LogLevel::Debug,
-            ("  GetNewIMEState() returns DISABLED because "
+            ("  GetNewIMEState() returns IMEEnabled::Disabled because "
              "the nsPresContext has been destroyed"));
-    return IMEState(IMEState::DISABLED);
+    return IMEState(IMEEnabled::Disabled);
   }
 
   // On Printing or Print Preview, we don't need IME.
   if (aPresContext->Type() == nsPresContext::eContext_PrintPreview ||
       aPresContext->Type() == nsPresContext::eContext_Print) {
     MOZ_LOG(sISMLog, LogLevel::Debug,
-            ("  GetNewIMEState() returns DISABLED because "
+            ("  GetNewIMEState() returns IMEEnabled::Disabled because "
              "the nsPresContext is for print or print preview"));
-    return IMEState(IMEState::DISABLED);
+    return IMEState(IMEEnabled::Disabled);
   }
 
   if (sInstalledMenuKeyboardListener) {
     MOZ_LOG(sISMLog, LogLevel::Debug,
-            ("  GetNewIMEState() returns DISABLED because "
+            ("  GetNewIMEState() returns IMEEnabled::Disabled because "
              "menu keyboard listener was installed"));
-    return IMEState(IMEState::DISABLED);
+    return IMEState(IMEEnabled::Disabled);
   }
 
   if (!aContent) {
@@ -998,14 +1109,14 @@ IMEState IMEStateManager::GetNewIMEState(nsPresContext* aPresContext,
     Document* doc = aPresContext->Document();
     if (doc && doc->HasFlag(NODE_IS_EDITABLE)) {
       MOZ_LOG(sISMLog, LogLevel::Debug,
-              ("  GetNewIMEState() returns ENABLED because "
+              ("  GetNewIMEState() returns IMEEnabled::Enabled because "
                "design mode editor has focus"));
-      return IMEState(IMEState::ENABLED);
+      return IMEState(IMEEnabled::Enabled);
     }
     MOZ_LOG(sISMLog, LogLevel::Debug,
-            ("  GetNewIMEState() returns DISABLED because "
+            ("  GetNewIMEState() returns IMEEnabled::Disabled because "
              "no content has focus"));
-    return IMEState(IMEState::DISABLED);
+    return IMEState(IMEEnabled::Disabled);
   }
 
   // nsIContent::GetDesiredIMEState() may cause a call of UpdateIMEState()
@@ -1017,10 +1128,7 @@ IMEState IMEStateManager::GetNewIMEState(nsPresContext* aPresContext,
 
   IMEState newIMEState = aContent->GetDesiredIMEState();
   MOZ_LOG(sISMLog, LogLevel::Debug,
-          ("  GetNewIMEState() returns { mEnabled=%s, "
-           "mOpen=%s }",
-           ToString(newIMEState.mEnabled).c_str(),
-           ToString(newIMEState.mOpen).c_str()));
+          ("  GetNewIMEState() returns %s", ToString(newIMEState).c_str()));
   return newIMEState;
 }
 
@@ -1043,12 +1151,12 @@ static bool MayBeIMEUnawareWebApp(nsINode* aNode) {
 
 // static
 void IMEStateManager::ResetActiveChildInputContext() {
-  sActiveChildInputContext.mIMEState.mEnabled = IMEState::UNKNOWN;
+  sActiveChildInputContext.mIMEState.mEnabled = IMEEnabled::Unknown;
 }
 
 // static
 bool IMEStateManager::HasActiveChildSetInputContext() {
-  return sActiveChildInputContext.mIMEState.mEnabled != IMEState::UNKNOWN;
+  return sActiveChildInputContext.mIMEState.mEnabled != IMEEnabled::Unknown;
 }
 
 // static
@@ -1058,17 +1166,10 @@ void IMEStateManager::SetInputContextForChildProcess(
   MOZ_LOG(
       sISMLog, LogLevel::Info,
       ("SetInputContextForChildProcess(aBrowserParent=0x%p, "
-       "aInputContext={ mIMEState={ mEnabled=%s, mOpen=%s }, "
-       "mHTMLInputType=\"%s\", mHTMLInputInputmode=\"%s\", mActionHint=\"%s\", "
-       "mInPrivateBrowsing=%s }, aAction={ mCause=%s, mAction=%s }), "
+       "aInputContext=%s , aAction={ mCause=%s, mAction=%s }), "
        "sPresContext=0x%p (available: %s), sWidget=0x%p (available: %s), "
        "BrowserParent::GetFocused()=0x%p, sInstalledMenuKeyboardListener=%s",
-       aBrowserParent, ToString(aInputContext.mIMEState.mEnabled).c_str(),
-       ToString(aInputContext.mIMEState.mOpen).c_str(),
-       NS_ConvertUTF16toUTF8(aInputContext.mHTMLInputType).get(),
-       NS_ConvertUTF16toUTF8(aInputContext.mHTMLInputInputmode).get(),
-       NS_ConvertUTF16toUTF8(aInputContext.mActionHint).get(),
-       GetBoolName(aInputContext.mInPrivateBrowsing),
+       aBrowserParent, ToString(aInputContext).c_str(),
        ToString(aAction.mCause).c_str(), ToString(aAction.mFocusChange).c_str(),
        sPresContext.get(), GetBoolName(CanHandleWith(sPresContext)), sWidget,
        GetBoolName(sWidget && !sWidget->Destroyed()),
@@ -1166,10 +1267,17 @@ static bool IsNextFocusableElementTextControl(Element* aInputContent) {
 }
 
 static void GetActionHint(nsIContent& aContent, nsAString& aActionHint) {
+  // If enterkeyhint is set, we don't infer action hint.
+  if (!aActionHint.IsEmpty()) {
+    return;
+  }
+
+  // XXX This is old compatibility, but we might be able to remove this.
   aContent.AsElement()->GetAttr(kNameSpaceID_None, nsGkAtoms::moz_action_hint,
                                 aActionHint);
 
   if (!aActionHint.IsEmpty()) {
+    ToLowerCase(aActionHint);
     return;
   }
 
@@ -1181,30 +1289,24 @@ static void GetActionHint(nsIContent& aContent, nsAString& aActionHint) {
   }
 
   // If we don't have an action hint and
-  // return won't submit the form, use "next".
+  // return won't submit the form, use "maybenext".
   bool willSubmit = false;
   bool isLastElement = false;
   nsCOMPtr<nsIFormControl> control(do_QueryInterface(inputContent));
-  Element* formElement = nullptr;
   if (control) {
-    formElement = control->GetFormElement();
+    HTMLFormElement* formElement = control->GetFormElement();
     // is this a form and does it have a default submit element?
     if (formElement) {
-      HTMLFormElement* htmlForm = nullptr;
-      if (formElement->IsHTMLElement(nsGkAtoms::form)) {
-        htmlForm = HTMLFormElement::FromNodeOrNull(formElement);
-        if (htmlForm->IsLastActiveElement(control)) {
-          isLastElement = true;
-        }
+      if (formElement->IsLastActiveElement(control)) {
+        isLastElement = true;
       }
 
-      nsCOMPtr<nsIForm> form = do_QueryInterface(formElement);
-      if (form && form->GetDefaultSubmitElement()) {
+      if (formElement->GetDefaultSubmitElement()) {
         willSubmit = true;
         // is this an html form...
-      } else if (htmlForm) {
+      } else {
         // ... and does it only have a single text input element ?
-        if (!htmlForm->ImplicitSubmissionIsDisabled() ||
+        if (!formElement->ImplicitSubmissionIsDisabled() ||
             // ... or is this the last non-disabled element?
             isLastElement) {
           willSubmit = true;
@@ -1218,7 +1320,7 @@ static void GetActionHint(nsIContent& aContent, nsAString& aActionHint) {
       if (IsNextFocusableElementTextControl(inputContent->AsElement())) {
         // This is focusable text control
         // XXX What good hint for read only field?
-        aActionHint.AssignLiteral("next");
+        aActionHint.AssignLiteral("maybenext");
         return;
       }
     }
@@ -1228,7 +1330,7 @@ static void GetActionHint(nsIContent& aContent, nsAString& aActionHint) {
     return;
   }
 
-  if (control->ControlType() == NS_FORM_INPUT_SEARCH) {
+  if (control->ControlType() == FormControlType::InputSearch) {
     aActionHint.AssignLiteral("search");
     return;
   }
@@ -1242,15 +1344,12 @@ void IMEStateManager::SetIMEState(const IMEState& aState,
                                   nsIContent* aContent, nsIWidget* aWidget,
                                   InputContextAction aAction,
                                   InputContext::Origin aOrigin) {
-  MOZ_LOG(
-      sISMLog, LogLevel::Info,
-      ("SetIMEState(aState={ mEnabled=%s, mOpen=%s }, "
-       "aContent=0x%p (BrowserParent=0x%p), aWidget=0x%p, aAction={ mCause=%s, "
-       "mFocusChange=%s }, aOrigin=%s)",
-       ToString(aState.mEnabled).c_str(), ToString(aState.mOpen).c_str(),
-       aContent, BrowserParent::GetFrom(aContent), aWidget,
-       ToString(aAction.mCause).c_str(), ToString(aAction.mFocusChange).c_str(),
-       ToChar(aOrigin)));
+  MOZ_LOG(sISMLog, LogLevel::Info,
+          ("SetIMEState(aState=%s, aContent=0x%p (BrowserParent=0x%p), "
+           "aWidget=0x%p, aAction={ mCause=%s, mFocusChange=%s }, aOrigin=%s)",
+           ToString(aState).c_str(), aContent, BrowserParent::GetFrom(aContent),
+           aWidget, ToString(aAction.mCause).c_str(),
+           ToString(aAction.mFocusChange).c_str(), ToChar(aOrigin)));
 
   NS_ENSURE_TRUE_VOID(aWidget);
 
@@ -1270,16 +1369,27 @@ void IMEStateManager::SetIMEState(const IMEState& aState,
       aPresContext &&
       nsContentUtils::IsInPrivateBrowsing(aPresContext->Document());
 
-  if (aContent) {
+  if (aContent && aContent->IsHTMLElement()) {
+    if (aState.IsEditable() && StaticPrefs::dom_forms_enterkeyhint()) {
+      nsGenericHTMLElement::FromNode(aContent)->GetEnterKeyHint(
+          context.mActionHint);
+    }
+
     if (aContent->IsHTMLElement(nsGkAtoms::input)) {
-      HTMLInputElement::FromNode(aContent)->GetType(context.mHTMLInputType);
+      HTMLInputElement* inputElement = HTMLInputElement::FromNode(aContent);
+      if (inputElement->HasBeenTypePassword() && aState.IsEditable()) {
+        context.mHTMLInputType.AssignLiteral("password");
+      } else {
+        inputElement->GetType(context.mHTMLInputType);
+      }
+
       GetActionHint(*aContent, context.mActionHint);
     } else if (aContent->IsHTMLElement(nsGkAtoms::textarea)) {
       context.mHTMLInputType.Assign(nsGkAtoms::textarea->GetUTF16String());
       GetActionHint(*aContent, context.mActionHint);
     }
 
-    if (aContent->IsHTMLElement() && aState.IsEditable() &&
+    if (aState.IsEditable() &&
         (StaticPrefs::dom_forms_inputmode() ||
          nsContentUtils::IsChromeDoc(aContent->OwnerDoc()))) {
       aContent->AsElement()->GetAttr(kNameSpaceID_None, nsGkAtoms::inputmode,
@@ -1295,6 +1405,13 @@ void IMEStateManager::SetIMEState(const IMEState& aState,
         ToLowerCase(context.mHTMLInputInputmode);
       }
     }
+
+    if (aContent->IsHTMLElement() && aState.IsEditable() &&
+        StaticPrefs::dom_forms_autocapitalize() &&
+        context.IsAutocapitalizeSupported()) {
+      nsGenericHTMLElement::FromNode(aContent)->GetAutocapitalize(
+          context.mAutocapitalize);
+    }
   }
 
   if (aAction.mCause == InputContextAction::CAUSE_UNKNOWN &&
@@ -1320,17 +1437,9 @@ void IMEStateManager::SetInputContext(nsIWidget* aWidget,
                                       const InputContextAction& aAction) {
   MOZ_LOG(
       sISMLog, LogLevel::Info,
-      ("SetInputContext(aWidget=0x%p, aInputContext={ "
-       "mIMEState={ mEnabled=%s, mOpen=%s }, mHTMLInputType=\"%s\", "
-       "mHTMLInputInputmode=\"%s\", mActionHint=\"%s\", "
-       "mInPrivateBrowsing=%s }, "
+      ("SetInputContext(aWidget=0x%p, aInputContext=%s, "
        "aAction={ mCause=%s, mAction=%s }), BrowserParent::GetFocused()=0x%p",
-       aWidget, ToString(aInputContext.mIMEState.mEnabled).c_str(),
-       ToString(aInputContext.mIMEState.mOpen).c_str(),
-       NS_ConvertUTF16toUTF8(aInputContext.mHTMLInputType).get(),
-       NS_ConvertUTF16toUTF8(aInputContext.mHTMLInputInputmode).get(),
-       NS_ConvertUTF16toUTF8(aInputContext.mActionHint).get(),
-       GetBoolName(aInputContext.mInPrivateBrowsing),
+       aWidget, ToString(aInputContext).c_str(),
        ToString(aAction.mCause).c_str(), ToString(aAction.mFocusChange).c_str(),
        BrowserParent::GetFocused()));
 
@@ -1459,10 +1568,12 @@ nsIContent* IMEStateManager::GetRootContent(nsPresContext* aPresContext) {
 void IMEStateManager::HandleSelectionEvent(
     nsPresContext* aPresContext, nsIContent* aEventTargetContent,
     WidgetSelectionEvent* aSelectionEvent) {
-  nsIContent* eventTargetContent =
-      aEventTargetContent ? aEventTargetContent : GetRootContent(aPresContext);
-  RefPtr<BrowserParent> browserParent =
-      eventTargetContent ? BrowserParent::GetFrom(eventTargetContent) : nullptr;
+  RefPtr<BrowserParent> browserParent = GetActiveBrowserParent();
+  if (!browserParent) {
+    browserParent = BrowserParent::GetFrom(aEventTargetContent
+                                               ? aEventTargetContent
+                                               : GetRootContent(aPresContext));
+  }
 
   MOZ_LOG(
       sISMLog, LogLevel::Info,
@@ -1556,12 +1667,14 @@ nsresult IMEStateManager::NotifyIME(const IMENotification& aNotification,
            "aWidget=0x%p, aBrowserParent=0x%p), sFocusedIMEWidget=0x%p, "
            "BrowserParent::GetFocused()=0x%p, sFocusedIMEBrowserParent=0x%p, "
            "aBrowserParent == BrowserParent::GetFocused()=%s, "
-           "aBrowserParent == sFocusedIMEBrowserParent=%s",
+           "aBrowserParent == sFocusedIMEBrowserParent=%s, "
+           "CanSendNotificationToWidget()=%s",
            ToChar(aNotification.mMessage), aWidget, aBrowserParent,
            sFocusedIMEWidget, BrowserParent::GetFocused(),
            sFocusedIMEBrowserParent.get(),
            GetBoolName(aBrowserParent == BrowserParent::GetFocused()),
-           GetBoolName(aBrowserParent == sFocusedIMEBrowserParent)));
+           GetBoolName(aBrowserParent == sFocusedIMEBrowserParent),
+           GetBoolName(CanSendNotificationToWidget())));
 
   if (NS_WARN_IF(!aWidget)) {
     MOZ_LOG(sISMLog, LogLevel::Error,
@@ -1571,6 +1684,8 @@ nsresult IMEStateManager::NotifyIME(const IMENotification& aNotification,
 
   switch (aNotification.mMessage) {
     case NOTIFY_IME_OF_FOCUS: {
+      MOZ_ASSERT(CanSendNotificationToWidget());
+
       // If focus notification comes from a remote browser which already lost
       // focus, we shouldn't accept the focus notification.  Then, following
       // notifications from the process will be ignored.
@@ -1638,7 +1753,10 @@ nsresult IMEStateManager::NotifyIME(const IMENotification& aNotification,
       nsCOMPtr<nsIWidget> focusedIMEWidget(sFocusedIMEWidget);
       sFocusedIMEWidget = nullptr;
       sFocusedIMEBrowserParent = nullptr;
-      return focusedIMEWidget->NotifyIME(IMENotification(NOTIFY_IME_OF_BLUR));
+      return CanSendNotificationToWidget()
+                 ? focusedIMEWidget->NotifyIME(
+                       IMENotification(NOTIFY_IME_OF_BLUR))
+                 : NS_OK;
     }
     case NOTIFY_IME_OF_SELECTION_CHANGE:
     case NOTIFY_IME_OF_TEXT_CHANGE:
@@ -1666,6 +1784,9 @@ nsresult IMEStateManager::NotifyIME(const IMENotification& aNotification,
              "is ignored because it's not for current focused IME widget"));
         return NS_OK;
       }
+      if (!CanSendNotificationToWidget()) {
+        return NS_OK;
+      }
       nsCOMPtr<nsIWidget> widget(aWidget);
       return widget->NotifyIME(aNotification);
     }
@@ -1780,7 +1901,7 @@ nsINode* IMEStateManager::GetRootEditableNode(nsPresContext* aPresContext,
 
 // static
 bool IMEStateManager::IsIMEObserverNeeded(const IMEState& aState) {
-  return aState.MaybeEditable();
+  return aState.IsEditable();
 }
 
 // static
@@ -1804,13 +1925,13 @@ void IMEStateManager::DestroyIMEContentObserver() {
 }
 
 // static
-void IMEStateManager::CreateIMEContentObserver(EditorBase* aEditorBase) {
+void IMEStateManager::CreateIMEContentObserver(EditorBase& aEditorBase) {
   MOZ_LOG(sISMLog, LogLevel::Info,
           ("CreateIMEContentObserver(aEditorBase=0x%p), "
            "sPresContext=0x%p, sContent=0x%p, sWidget=0x%p (available: %s), "
            "sActiveIMEContentObserver=0x%p, "
            "sActiveIMEContentObserver->IsManaging(sPresContext, sContent)=%s",
-           aEditorBase, sPresContext.get(), sContent.get(), sWidget,
+           &aEditorBase, sPresContext.get(), sContent.get(), sWidget,
            GetBoolName(sWidget && !sWidget->Destroyed()),
            sActiveIMEContentObserver.get(),
            GetBoolName(sActiveIMEContentObserver
@@ -1833,7 +1954,7 @@ void IMEStateManager::CreateIMEContentObserver(EditorBase* aEditorBase) {
     return;  // Sometimes, there are no widgets.
   }
 
-  nsCOMPtr<nsIWidget> widget(sWidget);
+  OwningNonNull<nsIWidget> widget(*sWidget);
 
   // If it's not text editable, we don't need to create IMEContentObserver.
   if (!IsIMEObserverNeeded(widget->GetInputContext().mIMEState)) {
@@ -1850,6 +1971,13 @@ void IMEStateManager::CreateIMEContentObserver(EditorBase* aEditorBase) {
     return;
   }
 
+  if (NS_WARN_IF(!sPresContext)) {
+    MOZ_LOG(sISMLog, LogLevel::Error,
+            ("  CreateIMEContentObserver(), FAILED due to "
+             "the nsPresContext is nullptr"));
+    return;
+  }
+
   MOZ_ASSERT(sPresContext->GetTextInputHandlingWidget() == widget);
 
   MOZ_LOG(sISMLog, LogLevel::Debug,
@@ -1862,7 +1990,9 @@ void IMEStateManager::CreateIMEContentObserver(EditorBase* aEditorBase) {
   // We should hold the current instance here.
   RefPtr<IMEContentObserver> activeIMEContentObserver(
       sActiveIMEContentObserver);
-  activeIMEContentObserver->Init(widget, sPresContext, sContent, aEditorBase);
+  OwningNonNull<nsPresContext> presContext(*sPresContext);
+  RefPtr<nsIContent> content = sContent;
+  activeIMEContentObserver->Init(widget, presContext, content, aEditorBase);
 }
 
 // static
