@@ -8,8 +8,7 @@
 #include "mozilla/dom/ElementInternalsBinding.h"
 #include "nsGenericHTMLElement.h"
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE(ElementInternals, mTarget)
 NS_IMPL_CYCLE_COLLECTING_ADDREF(ElementInternals)
@@ -29,5 +28,4 @@ JSObject* ElementInternals::WrapObject(JSContext* aCx,
   return ElementInternals_Binding::Wrap(aCx, this, aGivenProto);
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
