@@ -9,8 +9,7 @@
 
 NS_IMPL_NS_NEW_HTML_ELEMENT(DataList)
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 HTMLDataListElement::~HTMLDataListElement() = default;
 
@@ -33,5 +32,4 @@ bool HTMLDataListElement::MatchOptions(Element* aElement, int32_t aNamespaceID,
          !aElement->HasAttr(kNameSpaceID_None, nsGkAtoms::disabled);
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
