@@ -127,8 +127,7 @@ class HTMLScriptElement final : public nsGenericHTMLElement,
     SetHTMLAttr(nsGkAtoms::referrerpolicy, aReferrerPolicy, aError);
   }
   void GetReferrerPolicy(nsAString& aReferrerPolicy) {
-    GetEnumAttr(nsGkAtoms::referrerpolicy, EmptyCString().get(),
-                aReferrerPolicy);
+    GetEnumAttr(nsGkAtoms::referrerpolicy, "", aReferrerPolicy);
   }
 
  protected:
