@@ -12,8 +12,7 @@
 
 NS_IMPL_NS_NEW_HTML_ELEMENT(Time)
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 HTMLTimeElement::HTMLTimeElement(
     already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo)
@@ -28,5 +27,4 @@ JSObject* HTMLTimeElement::WrapNode(JSContext* cx,
   return HTMLTimeElement_Binding::Wrap(cx, this, aGivenProto);
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
