@@ -7,6 +7,7 @@
 #include "IDBIndex.h"
 
 #include "IDBCursorType.h"
+#include "IDBDatabase.h"
 #include "IDBEvents.h"
 #include "IDBKeyRange.h"
 #include "IDBObjectStore.h"
@@ -22,14 +23,14 @@
 // Include this last to avoid path problems on Windows.
 #include "ActorsChild.h"
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 using namespace mozilla::dom::indexedDB;
 
 namespace {
 
-RefPtr<IDBRequest> GenerateRequest(JSContext* aCx, IDBIndex* aIndex) {
+MovingNotNull<RefPtr<IDBRequest>> GenerateRequest(JSContext* aCx,
+                                                  IDBIndex* aIndex) {
   MOZ_ASSERT(aIndex);
   aIndex->AssertIsOnOwningThread();
 
@@ -56,7 +57,6 @@ IDBIndex::~IDBIndex() {
   AssertIsOnOwningThread();
 
   if (mRooted) {
-    mCachedKeyPath.setUndefined();
     mozilla::DropJSObjects(this);
   }
 }
@@ -211,10 +211,11 @@ void IDBIndex::SetName(const nsAString& aName, ErrorResult& aRv) {
   IDB_LOG_MARK_CHILD_TRANSACTION_REQUEST(
       "database(%s).transaction(%s).objectStore(%s).index(%s)."
       "rename(%s)",
-      "IDBIndex.rename()", transaction.LoggingSerialNumber(),
-      requestSerialNumber, IDB_LOG_STRINGIFY(transaction.Database()),
-      IDB_LOG_STRINGIFY(transaction), IDB_LOG_STRINGIFY(mObjectStore),
-      loggingOldIndex.get(), IDB_LOG_STRINGIFY(this));
+      "IDBIndex.rename(%.0s%.0s%.0s%.0s%.0s)",
+      transaction.LoggingSerialNumber(), requestSerialNumber,
+      IDB_LOG_STRINGIFY(transaction.Database()), IDB_LOG_STRINGIFY(transaction),
+      IDB_LOG_STRINGIFY(mObjectStore), loggingOldIndex.get(),
+      IDB_LOG_STRINGIFY(this));
 
   mObjectStore->MutableTransactionRef().RenameIndex(mObjectStore, indexId,
                                                     aName);
@@ -346,15 +347,14 @@ RefPtr<IDBRequest> IDBIndex::GetInternal(bool aKeyOnly, JSContext* aCx,
     params = IndexGetParams(objectStoreId, indexId, serializedKeyRange);
   }
 
-  auto request = GenerateRequest(aCx, this);
-  MOZ_ASSERT(request);
+  auto request = GenerateRequest(aCx, this).unwrap();
 
   if (aKeyOnly) {
     IDB_LOG_MARK_CHILD_TRANSACTION_REQUEST(
         "database(%s).transaction(%s).objectStore(%s).index(%s)."
         "getKey(%s)",
-        "IDBIndex.getKey()", transaction.LoggingSerialNumber(),
-        request->LoggingSerialNumber(),
+        "IDBIndex.getKey(%.0s%.0s%.0s%.0s%.0s)",
+        transaction.LoggingSerialNumber(), request->LoggingSerialNumber(),
         IDB_LOG_STRINGIFY(transaction.Database()),
         IDB_LOG_STRINGIFY(transaction), IDB_LOG_STRINGIFY(mObjectStore),
         IDB_LOG_STRINGIFY(this), IDB_LOG_STRINGIFY(keyRange));
@@ -362,7 +362,7 @@ RefPtr<IDBRequest> IDBIndex::GetInternal(bool aKeyOnly, JSContext* aCx,
     IDB_LOG_MARK_CHILD_TRANSACTION_REQUEST(
         "database(%s).transaction(%s).objectStore(%s).index(%s)."
         "get(%s)",
-        "IDBIndex.get()", transaction.LoggingSerialNumber(),
+        "IDBIndex.get(%.0s%.0s%.0s%.0s%.0s)", transaction.LoggingSerialNumber(),
         request->LoggingSerialNumber(),
         IDB_LOG_STRINGIFY(transaction.Database()),
         IDB_LOG_STRINGIFY(transaction), IDB_LOG_STRINGIFY(mObjectStore),
@@ -422,15 +422,14 @@ RefPtr<IDBRequest> IDBIndex::GetAllInternal(bool aKeysOnly, JSContext* aCx,
                 : RequestParams{IndexGetAllParams(objectStoreId, indexId,
                                                   optionalKeyRange, limit)};
 
-  auto request = GenerateRequest(aCx, this);
-  MOZ_ASSERT(request);
+  auto request = GenerateRequest(aCx, this).unwrap();
 
   if (aKeysOnly) {
     IDB_LOG_MARK_CHILD_TRANSACTION_REQUEST(
         "database(%s).transaction(%s).objectStore(%s).index(%s)."
         "getAllKeys(%s, %s)",
-        "IDBIndex.getAllKeys()", transaction.LoggingSerialNumber(),
-        request->LoggingSerialNumber(),
+        "IDBIndex.getAllKeys(%.0s%.0s%.0s%.0s%.0s%.0s)",
+        transaction.LoggingSerialNumber(), request->LoggingSerialNumber(),
         IDB_LOG_STRINGIFY(transaction.Database()),
         IDB_LOG_STRINGIFY(transaction), IDB_LOG_STRINGIFY(mObjectStore),
         IDB_LOG_STRINGIFY(this), IDB_LOG_STRINGIFY(keyRange),
@@ -439,8 +438,8 @@ RefPtr<IDBRequest> IDBIndex::GetAllInternal(bool aKeysOnly, JSContext* aCx,
     IDB_LOG_MARK_CHILD_TRANSACTION_REQUEST(
         "database(%s).transaction(%s).objectStore(%s).index(%s)."
         "getAll(%s, %s)",
-        "IDBIndex.getAll()", transaction.LoggingSerialNumber(),
-        request->LoggingSerialNumber(),
+        "IDBIndex.getAll(%.0s%.0s%.0s%.0s%.0s%.0s)",
+        transaction.LoggingSerialNumber(), request->LoggingSerialNumber(),
         IDB_LOG_STRINGIFY(transaction.Database()),
         IDB_LOG_STRINGIFY(transaction), IDB_LOG_STRINGIFY(mObjectStore),
         IDB_LOG_STRINGIFY(this), IDB_LOG_STRINGIFY(keyRange),
@@ -501,15 +500,14 @@ RefPtr<IDBRequest> IDBIndex::OpenCursorInternal(bool aKeysOnly, JSContext* aCx,
       aKeysOnly ? OpenCursorParams{IndexOpenKeyCursorParams{commonIndexParams}}
                 : OpenCursorParams{IndexOpenCursorParams{commonIndexParams}};
 
-  auto request = GenerateRequest(aCx, this);
-  MOZ_ASSERT(request);
+  auto request = GenerateRequest(aCx, this).unwrap();
 
   if (aKeysOnly) {
     IDB_LOG_MARK_CHILD_TRANSACTION_REQUEST(
         "database(%s).transaction(%s).objectStore(%s).index(%s)."
         "openKeyCursor(%s, %s)",
-        "IDBIndex.openKeyCursor()", transaction.LoggingSerialNumber(),
-        request->LoggingSerialNumber(),
+        "IDBIndex.openKeyCursor(%.0s%.0s%.0s%.0s%.0s%.0s)",
+        transaction.LoggingSerialNumber(), request->LoggingSerialNumber(),
         IDB_LOG_STRINGIFY(transaction.Database()),
         IDB_LOG_STRINGIFY(transaction), IDB_LOG_STRINGIFY(mObjectStore),
         IDB_LOG_STRINGIFY(this), IDB_LOG_STRINGIFY(keyRange),
@@ -518,8 +516,8 @@ RefPtr<IDBRequest> IDBIndex::OpenCursorInternal(bool aKeysOnly, JSContext* aCx,
     IDB_LOG_MARK_CHILD_TRANSACTION_REQUEST(
         "database(%s).transaction(%s).objectStore(%s).index(%s)."
         "openCursor(%s, %s)",
-        "IDBIndex.openCursor()", transaction.LoggingSerialNumber(),
-        request->LoggingSerialNumber(),
+        "IDBIndex.openCursor(%.0s%.0s%.0s%.0s%.0s%.0s)",
+        transaction.LoggingSerialNumber(), request->LoggingSerialNumber(),
         IDB_LOG_STRINGIFY(transaction.Database()),
         IDB_LOG_STRINGIFY(transaction), IDB_LOG_STRINGIFY(mObjectStore),
         IDB_LOG_STRINGIFY(this), IDB_LOG_STRINGIFY(keyRange),
@@ -577,13 +575,12 @@ RefPtr<IDBRequest> IDBIndex::Count(JSContext* aCx, JS::Handle<JS::Value> aKey,
     params.optionalKeyRange().emplace(serializedKeyRange);
   }
 
-  auto request = GenerateRequest(aCx, this);
-  MOZ_ASSERT(request);
+  auto request = GenerateRequest(aCx, this).unwrap();
 
   IDB_LOG_MARK_CHILD_TRANSACTION_REQUEST(
       "database(%s).transaction(%s).objectStore(%s).index(%s)."
       "count(%s)",
-      "IDBIndex.count()", transaction.LoggingSerialNumber(),
+      "IDBIndex.count(%.0s%.0s%.0s%.0s%.0s)", transaction.LoggingSerialNumber(),
       request->LoggingSerialNumber(), IDB_LOG_STRINGIFY(transaction.Database()),
       IDB_LOG_STRINGIFY(transaction), IDB_LOG_STRINGIFY(mObjectStore),
       IDB_LOG_STRINGIFY(this), IDB_LOG_STRINGIFY(keyRange));
@@ -637,5 +634,4 @@ JSObject* IDBIndex::WrapObject(JSContext* aCx,
   return IDBIndex_Binding::Wrap(aCx, this, aGivenProto);
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
