@@ -16,32 +16,35 @@
 #include "mozilla/EventForwards.h"
 #include "mozilla/RefPtr.h"
 #include "mozilla/dom/BrowserBridgeParent.h"
-#include "mozilla/dom/File.h"
 #include "mozilla/dom/PBrowserParent.h"
-#include "mozilla/dom/PContent.h"
-#include "mozilla/dom/PFilePickerParent.h"
 #include "mozilla/dom/TabContext.h"
+#include "mozilla/dom/VsyncParent.h"
 #include "mozilla/dom/ipc/IdType.h"
-#include "mozilla/gfx/CrossProcessPaint.h"
-#include "mozilla/layers/CompositorBridgeParent.h"
 #include "mozilla/layout/RemoteLayerTreeOwner.h"
 #include "nsCOMPtr.h"
 #include "nsIAuthPromptProvider.h"
 #include "nsIBrowserDOMWindow.h"
 #include "nsIDOMEventListener.h"
-#include "nsIKeyEventInPluginCallback.h"
+#include "nsIRemoteTab.h"
 #include "nsIWidget.h"
-#include "nsIXULBrowserWindow.h"
-#include "nsRefreshDriver.h"
 #include "nsWeakReference.h"
 
+class imgIContainer;
+class nsCycleCollectionTraversalCallback;
+class nsDocShellLoadState;
 class nsFrameLoader;
+class nsIBrowser;
 class nsIContent;
-class nsIPrincipal;
-class nsIURI;
-class nsILoadContext;
+class nsIContentSecurityPolicy;
 class nsIDocShell;
+class nsILoadContext;
+class nsIPrincipal;
+class nsIRequest;
+class nsIURI;
 class nsIWebBrowserPersistDocumentReceiver;
+class nsIWebProgress;
+class nsIXULBrowserWindow;
+class nsPIDOMWindowOuter;
 
 namespace mozilla {
 
@@ -49,17 +52,12 @@ namespace a11y {
 class DocAccessibleParent;
 }
 
-namespace layers {
-struct TextureFactoryIdentifier;
-}  // namespace layers
-
 namespace widget {
 struct IMENotification;
 }  // namespace widget
 
 namespace gfx {
 class SourceSurface;
-class DataSourceSurface;
 }  // namespace gfx
 
 namespace dom {
@@ -83,7 +81,6 @@ class StructuredCloneData;
 class BrowserParent final : public PBrowserParent,
                             public nsIDOMEventListener,
                             public nsIAuthPromptProvider,
-                            public nsIKeyEventInPluginCallback,
                             public nsSupportsWeakReference,
                             public TabContext,
                             public LiveResizeListener {
@@ -110,9 +107,6 @@ class BrowserParent final : public PBrowserParent,
                 CanonicalBrowsingContext* aBrowsingContext,
                 uint32_t aChromeFlags);
 
-  // Call from LayoutStatics only
-  static void InitializeStatics();
-
   /**
    * Returns the focused BrowserParent or nullptr if chrome or another app
    * is focused.
@@ -220,9 +214,9 @@ class BrowserParent final : public PBrowserParent,
   template <typename Callback>
   void VisitAllDescendants(Callback aCallback) {
     const auto& browserBridges = ManagedPBrowserBridgeParent();
-    for (auto iter = browserBridges.ConstIter(); !iter.Done(); iter.Next()) {
+    for (const auto& key : browserBridges) {
       BrowserBridgeParent* browserBridge =
-          static_cast<BrowserBridgeParent*>(iter.Get()->GetKey());
+          static_cast<BrowserBridgeParent*>(key);
       BrowserParent* browserParent = browserBridge->GetBrowserParent();
 
       aCallback(browserParent);
@@ -236,9 +230,9 @@ class BrowserParent final : public PBrowserParent,
   template <typename Callback>
   void VisitChildren(Callback aCallback) {
     const auto& browserBridges = ManagedPBrowserBridgeParent();
-    for (auto iter = browserBridges.ConstIter(); !iter.Done(); iter.Next()) {
+    for (const auto& key : browserBridges) {
       BrowserBridgeParent* browserBridge =
-          static_cast<BrowserBridgeParent*>(iter.Get()->GetKey());
+          static_cast<BrowserBridgeParent*>(key);
       aCallback(browserBridge);
     }
   }
@@ -285,59 +279,44 @@ class BrowserParent final : public PBrowserParent,
                                                       nsIURI* aDocURI);
 
   mozilla::ipc::IPCResult RecvOnStateChange(
-      const Maybe<WebProgressData>& awebProgressData,
-      const RequestData& aRequestData, const uint32_t aStateFlags,
-      const nsresult aStatus,
+      const WebProgressData& aWebProgressData, const RequestData& aRequestData,
+      const uint32_t aStateFlags, const nsresult aStatus,
       const Maybe<WebProgressStateChangeData>& aStateChangeData);
 
-  mozilla::ipc::IPCResult RecvOnProgressChange(
-      const Maybe<WebProgressData>& aWebProgressData,
-      const RequestData& aRequestData, const int32_t aCurSelfProgress,
-      const int32_t aMaxSelfProgress, const int32_t aCurTotalProgres,
-      const int32_t aMaxTotalProgress);
+  mozilla::ipc::IPCResult RecvOnProgressChange(const int32_t aCurTotalProgres,
+                                               const int32_t aMaxTotalProgress);
 
   mozilla::ipc::IPCResult RecvOnLocationChange(
-      const Maybe<WebProgressData>& aWebProgressData,
-      const RequestData& aRequestData, nsIURI* aLocation, const uint32_t aFlags,
-      const bool aCanGoBack, const bool aCanGoForward,
+      const WebProgressData& aWebProgressData, const RequestData& aRequestData,
+      nsIURI* aLocation, const uint32_t aFlags, const bool aCanGoBack,
+      const bool aCanGoForward,
       const Maybe<WebProgressLocationChangeData>& aLocationChangeData);
 
-  mozilla::ipc::IPCResult RecvOnStatusChange(
-      const Maybe<WebProgressData>& aWebProgressData,
-      const RequestData& aRequestData, const nsresult aStatus,
-      const nsString& aMessage);
+  mozilla::ipc::IPCResult RecvOnStatusChange(const nsString& aMessage);
 
   mozilla::ipc::IPCResult RecvNotifyContentBlockingEvent(
       const uint32_t& aEvent, const RequestData& aRequestData,
       const bool aBlocked, const nsACString& aTrackingOrigin,
       nsTArray<nsCString>&& aTrackingFullHashes,
-      const Maybe<mozilla::ContentBlockingNotifier::StorageAccessGrantedReason>&
-          aReason);
+      const Maybe<mozilla::ContentBlockingNotifier::
+                      StorageAccessPermissionGrantedReason>& aReason);
 
-  mozilla::ipc::IPCResult RecvReportBlockedEmbedderNodeByClassifier();
+  mozilla::ipc::IPCResult RecvSetAllowDeprecatedTls(bool value);
 
   mozilla::ipc::IPCResult RecvNavigationFinished();
 
-  bool GetWebProgressListener(nsIBrowser** aOutBrowser,
-                              nsIWebProgress** aOutManager,
-                              nsIWebProgressListener** aOutListener);
+  already_AddRefed<nsIBrowser> GetBrowser();
 
-  void ReconstructWebProgressAndRequest(
-      nsIWebProgress* aManager, const Maybe<WebProgressData>& aWebProgressData,
-      const RequestData& aRequestData, nsIWebProgress** aOutWebProgress,
-      nsIRequest** aOutRequest);
+  already_AddRefed<CanonicalBrowsingContext> BrowsingContextForWebProgress(
+      const WebProgressData& aWebProgressData);
 
   mozilla::ipc::IPCResult RecvSessionStoreUpdate(
       const Maybe<nsCString>& aDocShellCaps, const Maybe<bool>& aPrivatedMode,
-      nsTArray<nsCString>&& aPositions,
-      nsTArray<int32_t>&& aPositionDescendants,
-      const nsTArray<InputFormData>& aInputs,
-      const nsTArray<CollectedInputDataValue>& aIdVals,
-      const nsTArray<CollectedInputDataValue>& aXPathVals,
-      nsTArray<nsCString>&& aOrigins, nsTArray<nsString>&& aKeys,
-      nsTArray<nsString>&& aValues, const bool aIsFullStorage,
-      const bool aNeedCollectSHistory, const uint32_t& aFlushId,
-      const bool& aIsFinal, const uint32_t& aEpoch);
+      const bool aNeedCollectSHistory, const uint32_t& aEpoch);
+
+  mozilla::ipc::IPCResult RecvIntrinsicSizeOrRatioChanged(
+      const Maybe<IntrinsicSize>& aIntrinsicSize,
+      const Maybe<AspectRatio>& aIntrinsicRatio);
 
   mozilla::ipc::IPCResult RecvSyncMessage(
       const nsString& aMessage, const ClonedMessageData& aData,
@@ -379,32 +358,12 @@ class BrowserParent final : public PBrowserParent,
   mozilla::ipc::IPCResult RecvRequestIMEToCommitComposition(
       const bool& aCancel, bool* aIsCommitted, nsString* aCommittedString);
 
-  mozilla::ipc::IPCResult RecvStartPluginIME(
-      const WidgetKeyboardEvent& aKeyboardEvent, const int32_t& aPanelX,
-      const int32_t& aPanelY, nsString* aCommitted);
-
-  mozilla::ipc::IPCResult RecvSetPluginFocused(const bool& aFocused);
-
-  mozilla::ipc::IPCResult RecvSetCandidateWindowForPlugin(
-      const widget::CandidateWindowPosition& aPosition);
-  mozilla::ipc::IPCResult RecvEnableIMEForPlugin(const bool& aEnable);
-
-  mozilla::ipc::IPCResult RecvDefaultProcOfPluginEvent(
-      const WidgetPluginEvent& aEvent);
-
   mozilla::ipc::IPCResult RecvGetInputContext(widget::IMEState* aIMEState);
 
   mozilla::ipc::IPCResult RecvSetInputContext(
       const widget::InputContext& aContext,
       const widget::InputContextAction& aAction);
 
-  // See nsIKeyEventInPluginCallback
-  virtual void HandledWindowedPluginKeyEvent(
-      const NativeEventData& aKeyEventData, bool aIsConsumed) override;
-
-  mozilla::ipc::IPCResult RecvOnWindowedPluginKeyEvent(
-      const NativeEventData& aKeyEventData);
-
   mozilla::ipc::IPCResult RecvRequestFocus(const bool& aCanRaise,
                                            const CallerType aCallerType);
 
@@ -421,9 +380,11 @@ class BrowserParent final : public PBrowserParent,
 
   mozilla::ipc::IPCResult RecvSetCursor(
       const nsCursor& aValue, const bool& aHasCustomCursor,
-      const nsCString& aUri, const uint32_t& aWidth, const uint32_t& aHeight,
-      const uint32_t& aStride, const gfx::SurfaceFormat& aFormat,
-      const uint32_t& aHotspotX, const uint32_t& aHotspotY, const bool& aForce);
+      const nsCString& aCursorData, const uint32_t& aWidth,
+      const uint32_t& aHeight, const float& aResolutionX,
+      const float& aResolutionY, const uint32_t& aStride,
+      const gfx::SurfaceFormat& aFormat, const uint32_t& aHotspotX,
+      const uint32_t& aHotspotY, const bool& aForce);
 
   mozilla::ipc::IPCResult RecvSetLinkStatus(const nsString& aStatus);
 
@@ -434,9 +395,6 @@ class BrowserParent final : public PBrowserParent,
 
   mozilla::ipc::IPCResult RecvHideTooltip();
 
-  mozilla::ipc::IPCResult RecvSetNativeChildOfShareableWindow(
-      const uintptr_t& childWindow);
-
   mozilla::ipc::IPCResult RecvDispatchFocusToTopLevelWindow();
 
   mozilla::ipc::IPCResult RecvRespondStartSwipeEvent(
@@ -451,6 +409,9 @@ class BrowserParent final : public PBrowserParent,
   mozilla::ipc::IPCResult RecvDispatchKeyboardEvent(
       const mozilla::WidgetKeyboardEvent& aEvent);
 
+  mozilla::ipc::IPCResult RecvDispatchTouchEvent(
+      const mozilla::WidgetTouchEvent& aEvent);
+
   mozilla::ipc::IPCResult RecvScrollRectIntoView(
       const nsRect& aRect, const ScrollAxis& aVertical,
       const ScrollAxis& aHorizontal, const ScrollFlags& aScrollFlags,
@@ -461,6 +422,10 @@ class BrowserParent final : public PBrowserParent,
 
   bool DeallocPColorPickerParent(PColorPickerParent* aColorPicker);
 
+  PVsyncParent* AllocPVsyncParent();
+
+  bool DeallocPVsyncParent(PVsyncParent* aActor);
+
 #ifdef ACCESSIBILITY
   PDocAccessibleParent* AllocPDocAccessibleParent(PDocAccessibleParent*,
                                                   const uint64_t&,
@@ -485,7 +450,7 @@ class BrowserParent final : public PBrowserParent,
       const uint64_t& aOuterWindowID,
       IsWindowSupportingWebVRResolver&& aResolve);
 
-  void LoadURL(nsIURI* aURI, nsIPrincipal* aTriggeringPrincipal);
+  void LoadURL(nsDocShellLoadState* aLoadState);
 
   void ResumeLoad(uint64_t aPendingSwitchID);
 
@@ -501,6 +466,11 @@ class BrowserParent final : public PBrowserParent,
 
   nsresult UpdatePosition();
 
+  // Notify position update to all descendant documents in this browser parent.
+  // NOTE: This should use only for browsers in popup windows attached to the
+  // main browser window.
+  void NotifyPositionUpdatedForContentsInPopup();
+
   void SizeModeChanged(const nsSizeMode& aSizeMode);
 
   void HandleAccessKey(const WidgetKeyboardEvent& aEvent,
@@ -511,9 +481,9 @@ class BrowserParent final : public PBrowserParent,
   void DynamicToolbarOffsetChanged(ScreenIntCoord aOffset);
 #endif
 
-  void Activate();
+  void Activate(uint64_t aActionId);
 
-  void Deactivate(bool aWindowLowering);
+  void Deactivate(bool aWindowLowering, uint64_t aActionId);
 
   void MouseEnterIntoWidget();
 
@@ -524,9 +494,10 @@ class BrowserParent final : public PBrowserParent,
 
   LayoutDeviceToCSSScale GetLayoutDeviceToCSSScale();
 
-  mozilla::ipc::IPCResult RecvRequestNativeKeyBindings(
-      const uint32_t& aType, const mozilla::WidgetKeyboardEvent& aEvent,
-      nsTArray<mozilla::CommandInt>* aCommands);
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY mozilla::ipc::IPCResult
+  RecvRequestNativeKeyBindings(const uint32_t& aType,
+                               const mozilla::WidgetKeyboardEvent& aEvent,
+                               nsTArray<mozilla::CommandInt>* aCommands);
 
   mozilla::ipc::IPCResult RecvSynthesizeNativeKeyEvent(
       const int32_t& aNativeKeyboardLayout, const int32_t& aNativeKeyCode,
@@ -535,7 +506,8 @@ class BrowserParent final : public PBrowserParent,
 
   mozilla::ipc::IPCResult RecvSynthesizeNativeMouseEvent(
       const LayoutDeviceIntPoint& aPoint, const uint32_t& aNativeMessage,
-      const uint32_t& aModifierFlags, const uint64_t& aObserverId);
+      const int16_t& aButton, const uint32_t& aModifierFlags,
+      const uint64_t& aObserverId);
 
   mozilla::ipc::IPCResult RecvSynthesizeNativeMouseMove(
       const LayoutDeviceIntPoint& aPoint, const uint64_t& aObserverId);
@@ -551,6 +523,10 @@ class BrowserParent final : public PBrowserParent,
       const LayoutDeviceIntPoint& aPoint, const double& aPointerPressure,
       const uint32_t& aPointerOrientation, const uint64_t& aObserverId);
 
+  mozilla::ipc::IPCResult RecvSynthesizeNativeTouchPadPinch(
+      const TouchpadPinchPhase& aEventPhase, const float& aScale,
+      const LayoutDeviceIntPoint& aPoint, const int32_t& aModifierFlags);
+
   mozilla::ipc::IPCResult RecvSynthesizeNativeTouchTap(
       const LayoutDeviceIntPoint& aPoint, const bool& aLongTap,
       const uint64_t& aObserverId);
@@ -558,8 +534,18 @@ class BrowserParent final : public PBrowserParent,
   mozilla::ipc::IPCResult RecvClearNativeTouchSequence(
       const uint64_t& aObserverId);
 
-  void SendMouseEvent(const nsAString& aType, float aX, float aY,
-                      int32_t aButton, int32_t aClickCount, int32_t aModifiers);
+  mozilla::ipc::IPCResult RecvSynthesizeNativePenInput(
+      const uint32_t& aPointerId, const TouchPointerState& aPointerState,
+      const LayoutDeviceIntPoint& aPoint, const double& aPressure,
+      const uint32_t& aRotation, const int32_t& aTiltX, const int32_t& aTiltY,
+      const uint64_t& aObserverId);
+
+  mozilla::ipc::IPCResult RecvSynthesizeNativeTouchpadDoubleTap(
+      const LayoutDeviceIntPoint& aPoint, const uint32_t& aModifierFlags);
+
+  mozilla::ipc::IPCResult RecvLockNativePointer();
+
+  mozilla::ipc::IPCResult RecvUnlockNativePointer();
 
   /**
    * The following Send*Event() marks aEvent as posted to remote process if
@@ -574,12 +560,15 @@ class BrowserParent final : public PBrowserParent,
 
   void SendMouseWheelEvent(WidgetWheelEvent& aEvent);
 
-  void SendRealKeyEvent(WidgetKeyboardEvent& aEvent);
+  /**
+   * Only when the event is synthesized, retrieving writing mode may flush
+   * the layout.
+   */
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY void SendRealKeyEvent(
+      WidgetKeyboardEvent& aEvent);
 
   void SendRealTouchEvent(WidgetTouchEvent& aEvent);
 
-  void SendPluginEvent(WidgetPluginEvent& aEvent);
-
   /**
    * Different from above Send*Event(), these methods return true if the
    * event has been posted to the remote process or failed to do that but
@@ -591,9 +580,9 @@ class BrowserParent final : public PBrowserParent,
 
   bool SendSelectionEvent(mozilla::WidgetSelectionEvent& aEvent);
 
-  bool SendHandleTap(TapType aType, const LayoutDevicePoint& aPoint,
-                     Modifiers aModifiers, const ScrollableLayerGuid& aGuid,
-                     uint64_t aInputBlockId);
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY bool SendHandleTap(
+      TapType aType, const LayoutDevicePoint& aPoint, Modifiers aModifiers,
+      const ScrollableLayerGuid& aGuid, uint64_t aInputBlockId);
 
   PFilePickerParent* AllocPFilePickerParent(const nsString& aTitle,
                                             const int16_t& aMode);
@@ -611,10 +600,12 @@ class BrowserParent final : public PBrowserParent,
 
   bool HandleQueryContentEvent(mozilla::WidgetQueryContentEvent& aEvent);
 
+  bool SendInsertText(const nsString& aStringToInsert);
+
   bool SendPasteTransferable(const IPCDataTransfer& aDataTransfer,
                              const bool& aIsPrivateData,
                              nsIPrincipal* aRequestingPrincipal,
-                             const uint32_t& aContentPolicyType);
+                             const nsContentPolicyType& aContentPolicyType);
 
   // Helper for transforming a point
   LayoutDeviceIntPoint TransformPoint(
@@ -661,13 +652,6 @@ class BrowserParent final : public PBrowserParent,
 
   void StopIMEStateManagement();
 
-  /**
-   * Native widget remoting protocol for use with windowed plugins with e10s.
-   */
-  PPluginWidgetParent* AllocPPluginWidgetParent();
-
-  bool DeallocPPluginWidgetParent(PPluginWidgetParent* aActor);
-
   PPaymentRequestParent* AllocPPaymentRequestParent();
 
   bool DeallocPPaymentRequestParent(PPaymentRequestParent* aActor);
@@ -681,7 +665,8 @@ class BrowserParent final : public PBrowserParent,
       nsTArray<IPCDataTransfer>&& aTransfers, const uint32_t& aAction,
       Maybe<Shmem>&& aVisualDnDData, const uint32_t& aStride,
       const gfx::SurfaceFormat& aFormat, const LayoutDeviceIntRect& aDragRect,
-      nsIPrincipal* aPrincipal, nsIContentSecurityPolicy* aCsp);
+      nsIPrincipal* aPrincipal, nsIContentSecurityPolicy* aCsp,
+      const CookieJarSettingsArgs& aCookieJarSettingsArgs);
 
   void AddInitialDnDDataTo(DataTransfer* aDataTransfer,
                            nsIPrincipal** aPrincipal);
@@ -704,9 +689,6 @@ class BrowserParent final : public PBrowserParent,
   bool GetDocShellIsActive();
   void SetDocShellIsActive(bool aDocShellIsActive);
 
-  bool GetSuspendMediaWhenInactive() const;
-  void SetSuspendMediaWhenInactive(bool aSuspendMediaWhenInactive);
-
   bool GetHasPresented();
   bool GetHasLayers();
   bool GetRenderLayers();
@@ -714,19 +696,16 @@ class BrowserParent final : public PBrowserParent,
   void PreserveLayers(bool aPreserveLayers);
   void NotifyResolutionChanged();
 
-  void Deprioritize();
-
   bool StartApzAutoscroll(float aAnchorX, float aAnchorY, nsViewID aScrollId,
                           uint32_t aPresShellId);
   void StopApzAutoscroll(nsViewID aScrollId, uint32_t aPresShellId);
 
-  // Suspend nsIWebProgressListener events until after the next STATE_START
-  // onStateChange. This is used to block STATE_STOP events from the old process
-  // when process switching away, as well as the initial about:blank and
-  // STATE_START from the new process.
-  void SuspendProgressEventsUntilAfterNextLoadStarts() {
-    mSuspendedProgressEvents = true;
-  }
+  bool CanCancelContentJS(nsIRemoteTab::NavigationType aNavigationType,
+                          int32_t aNavigationIndex,
+                          nsIURI* aNavigationURI) const;
+
+  // Called when the BrowserParent is being destroyed or entering bfcache.
+  void Deactivated();
 
  protected:
   friend BrowserBridgeParent;
@@ -739,18 +718,12 @@ class BrowserParent final : public PBrowserParent,
       const nsString& aMessage, bool aSync, ipc::StructuredCloneData* aData,
       nsTArray<ipc::StructuredCloneData>* aJSONRetVal = nullptr);
 
-  mozilla::ipc::IPCResult RecvAsyncAuthPrompt(const nsCString& aUri,
-                                              const nsString& aRealm,
-                                              const uint64_t& aCallbackId);
-
   virtual mozilla::ipc::IPCResult Recv__delete__() override;
 
   virtual void ActorDestroy(ActorDestroyReason why) override;
 
   mozilla::ipc::IPCResult RecvRemotePaintIsReady();
 
-  mozilla::ipc::IPCResult RecvNotifyCompositorTransaction();
-
   mozilla::ipc::IPCResult RecvRemoteIsReadyToHandleInputEvents();
 
   mozilla::ipc::IPCResult RecvPaintWhileInterruptingJSNoOp(
@@ -770,11 +743,19 @@ class BrowserParent final : public PBrowserParent,
                                        const uint32_t& aFlags);
 
   mozilla::ipc::IPCResult RecvQueryVisitedState(
-      const nsTArray<RefPtr<nsIURI>>&& aURIs);
+      nsTArray<RefPtr<nsIURI>>&& aURIs);
 
   mozilla::ipc::IPCResult RecvMaybeFireEmbedderLoadEvents(
       EmbedderElementEventType aFireEventAtEmbeddingElement);
 
+  mozilla::ipc::IPCResult RecvRequestPointerLock(
+      RequestPointerLockResolver&& aResolve);
+  mozilla::ipc::IPCResult RecvReleasePointerLock();
+
+  mozilla::ipc::IPCResult RecvRequestPointerCapture(
+      const uint32_t& aPointerId, RequestPointerCaptureResolver&& aResolve);
+  mozilla::ipc::IPCResult RecvReleasePointerCapture(const uint32_t& aPointerId);
+
  private:
   void SuppressDisplayport(bool aEnabled);
 
@@ -806,11 +787,14 @@ class BrowserParent final : public PBrowserParent,
   // and have to ensure that the child did not modify links to be loaded.
   bool QueryDropLinksForVerification();
 
+  void UnlockNativePointer();
+
+  void UpdateNativePointerLockCenter(nsIWidget* aWidget);
+
  private:
   // This is used when APZ needs to find the BrowserParent associated with a
   // layer to dispatch events.
-  typedef nsDataHashtable<nsUint64HashKey, BrowserParent*>
-      LayerToBrowserParentTable;
+  typedef nsTHashMap<nsUint64HashKey, BrowserParent*> LayerToBrowserParentTable;
   static LayerToBrowserParentTable* sLayerToBrowserParentTable;
 
   static void AddBrowserParentToTable(layers::LayersId aLayersId,
@@ -861,6 +845,8 @@ class BrowserParent final : public PBrowserParent,
   void SendRealTouchMoveEvent(WidgetTouchEvent& aEvent, APZData& aAPZData,
                               uint32_t aConsecutiveTouchMoveCount);
 
+  void UpdateVsyncParentVsyncSource();
+
  public:
   // Unsets sTopLevelWebFocus regardless of its current value.
   static void UnsetTopLevelWebFocusAll();
@@ -900,6 +886,7 @@ class BrowserParent final : public PBrowserParent,
   LayersObserverEpoch mLayerTreeEpoch;
 
   Maybe<LayoutDeviceToLayoutDeviceMatrix4x4> mChildToParentConversionMatrix;
+  Maybe<ScreenRect> mRemoteDocumentRect;
 
   nsIntRect mRect;
   ScreenIntSize mDimensions;
@@ -926,10 +913,8 @@ class BrowserParent final : public PBrowserParent,
   // receiving a LoadURL message before returning from ProvideWindow.
   //
   // The mCreatingWindow flag is set while dispatching CreateWindow. During
-  // that time, any LoadURL calls are skipped and the URL is stored in
-  // mSkippedURL.
+  // that time, any LoadURL calls are skipped.
   bool mCreatingWindow;
-  nsCString mDelayedURL;
 
   // When loading a new tab or window via window.open, we want to ensure that
   // frame scripts for that tab are loaded before any scripts start to run in
@@ -940,43 +925,38 @@ class BrowserParent final : public PBrowserParent,
   // CreateWindow response. Then BrowserChild loads them immediately.
   nsTArray<FrameScriptInfo> mDelayedFrameScripts;
 
-  // Cached cursor setting from BrowserChild.  When the cursor is over the tab,
-  // it should take this appearance.
-  nsCursor mCursor;
-  nsCOMPtr<imgIContainer> mCustomCursor;
-  uint32_t mCustomCursorHotspotX, mCustomCursorHotspotY;
+  // Cached cursor setting from BrowserChild.  When the cursor is over the
+  // tab, it should take this appearance.
+  nsIWidget::Cursor mCursor;
 
   nsTArray<nsString> mVerifyDropLinks;
 
+  RefPtr<VsyncParent> mVsyncParent;
+
 #ifdef DEBUG
   int32_t mActiveSupressDisplayportCount = 0;
 #endif
 
-  // Cached value indicating the docshell active state of the remote browser.
-  bool mDocShellIsActive : 1;
-
   // When true, we've initiated normal shutdown and notified our managing
   // PContent.
   bool mMarkedDestroying : 1;
-  // When true, the BrowserParent is invalid and we should not send IPC messages
-  // anymore.
+  // When true, the BrowserParent is invalid and we should not send IPC
+  // messages anymore.
   bool mIsDestroyed : 1;
   // True if the cursor changes from the BrowserChild should change the widget
-  // cursor.  This happens whenever the cursor is in the tab's region.
-  bool mTabSetsCursor : 1;
+  // cursor.  This happens whenever the cursor is in the remote target's
+  // region.
+  bool mRemoteTargetSetsCursor : 1;
 
   // If this flag is set, then the tab's layers will be preserved even when
   // the tab's docshell is inactive.
-  bool mPreserveLayers : 1;
+  bool mIsPreservingLayers : 1;
 
   // Holds the most recent value passed to the RenderLayers function. This
   // does not necessarily mean that the layers have finished rendering
   // and have uploaded - for that, use mHasLayers.
   bool mRenderLayers : 1;
 
-  // Whether this is active for the ProcessPriorityManager or not.
-  bool mActiveInPriorityManager : 1;
-
   // True if the compositor has reported that the BrowserChild has uploaded
   // layers.
   bool mHasLayers : 1;
@@ -988,40 +968,27 @@ class BrowserParent final : public PBrowserParent,
   // True when the remote browser is created and ready to handle input events.
   bool mIsReadyToHandleInputEvents : 1;
 
-  // True if we suppress the eMouseEnterIntoWidget event due to the BrowserChild
-  // was not ready to handle it. We will resend it when the next time we fire a
-  // mouse event and the BrowserChild is ready.
+  // True if we suppress the eMouseEnterIntoWidget event due to the
+  // BrowserChild was not ready to handle it. We will resend it when the next
+  // time we fire a mouse event and the BrowserChild is ready.
   bool mIsMouseEnterIntoWidgetEventSuppressed : 1;
 
-  // Set to true if we're currently suspending nsIWebProgress events.
-  // We keep suspending until we get a STATE_START onStateChange event
-  // (for something that isn't the initial about:blank) and then start
-  // allowing future events.
-  bool mSuspendedProgressEvents : 1;
-
-  // True if the media in the remote docshell should be suspended when the
-  // remote docshell is inactive.
-  bool mSuspendMediaWhenInactive : 1;
+  // True after RecvLockNativePointer has been called and until
+  // UnlockNativePointer has been called.
+  bool mLockedNativePointer : 1;
 };
 
 struct MOZ_STACK_CLASS BrowserParent::AutoUseNewTab final {
  public:
-  AutoUseNewTab(BrowserParent* aNewTab, nsCString* aURLToLoad)
-      : mNewTab(aNewTab), mURLToLoad(aURLToLoad) {
+  explicit AutoUseNewTab(BrowserParent* aNewTab) : mNewTab(aNewTab) {
     MOZ_ASSERT(!aNewTab->mCreatingWindow);
-
     aNewTab->mCreatingWindow = true;
-    aNewTab->mDelayedURL.Truncate();
   }
 
-  ~AutoUseNewTab() {
-    mNewTab->mCreatingWindow = false;
-    *mURLToLoad = mNewTab->mDelayedURL;
-  }
+  ~AutoUseNewTab() { mNewTab->mCreatingWindow = false; }
 
  private:
   RefPtr<BrowserParent> mNewTab;
-  nsCString* mURLToLoad;
 };
 
 }  // namespace dom
