@@ -25,11 +25,13 @@
 #include "GMPVideoHost.h"
 #include "mozilla/Algorithm.h"
 #include "mozilla/ipc/CrashReporterClient.h"
+#include "mozilla/ipc/Endpoint.h"
 #include "mozilla/ipc/ProcessChild.h"
 #include "mozilla/TextUtils.h"
 #include "nsDebugImpl.h"
 #include "nsExceptionHandler.h"
 #include "nsIFile.h"
+#include "nsReadableUtils.h"
 #include "nsXULAppAPI.h"
 #include "prio.h"
 #ifdef XP_WIN
@@ -39,6 +41,10 @@
 #  include <unistd.h>  // for _exit()
 #endif
 
+#if defined(MOZ_SANDBOX) && defined(MOZ_DEBUG) && defined(ENABLE_TESTS)
+#  include "nsThreadManager.h"
+#endif
+
 using namespace mozilla::ipc;
 
 namespace mozilla {
@@ -99,13 +105,11 @@ static bool GetPluginFile(const nsAString& aPluginPath,
   GetFileBase(aPluginPath, aLibDirectory, aLibFile, baseName);
 
 #if defined(XP_MACOSX)
-  nsAutoString binaryName =
-      NS_LITERAL_STRING("lib") + baseName + NS_LITERAL_STRING(".dylib");
+  nsAutoString binaryName = u"lib"_ns + baseName + u".dylib"_ns;
 #elif defined(OS_POSIX)
-  nsAutoString binaryName =
-      NS_LITERAL_STRING("lib") + baseName + NS_LITERAL_STRING(".so");
+  nsAutoString binaryName = u"lib"_ns + baseName + u".so"_ns;
 #elif defined(XP_WIN)
-  nsAutoString binaryName = baseName + NS_LITERAL_STRING(".dll");
+  nsAutoString binaryName = baseName + u".dll"_ns;
 #else
 #  error not defined
 #endif
@@ -155,11 +159,18 @@ static bool GetPluginPaths(const nsAString& aPluginPath,
 #endif    // XP_MACOSX
 
 bool GMPChild::Init(const nsAString& aPluginPath, base::ProcessId aParentPid,
-                    MessageLoop* aIOLoop, UniquePtr<IPC::Channel> aChannel) {
+                    mozilla::ipc::ScopedPort aPort) {
   GMP_CHILD_LOG_DEBUG("%s pluginPath=%s", __FUNCTION__,
                       NS_ConvertUTF16toUTF8(aPluginPath).get());
+#if defined(MOZ_SANDBOX) && defined(MOZ_DEBUG) && defined(ENABLE_TESTS)
+  // GMPChild does not use nsThreadManager outside of tests, so only init it
+  // here for the sandbox tests.
+  if (NS_WARN_IF(NS_FAILED(nsThreadManager::get().Init()))) {
+    return false;
+  }
+#endif
 
-  if (NS_WARN_IF(!Open(std::move(aChannel), aParentPid, aIOLoop))) {
+  if (NS_WARN_IF(!Open(std::move(aPort), aParentPid))) {
     return false;
   }
 
@@ -221,8 +232,14 @@ mozilla::ipc::IPCResult GMPChild::RecvPreloadLibs(const nsCString& aLibs) {
   }
 #elif defined(XP_LINUX)
   constexpr static const char* whitelist[] = {
+      // NSS libraries used by clearkey.
       "libfreeblpriv3.so",
       "libsoftokn3.so",
+      // glibc libraries merged into libc.so.6; see bug 1725828 and
+      // the corresponding code in GMPParent.cpp.
+      "libdl.so.2",
+      "libpthread.so.0",
+      "librt.so.1",
   };
 
   nsTArray<nsCString> libs;
@@ -234,7 +251,18 @@ mozilla::ipc::IPCResult GMPChild::RecvPreloadLibs(const nsCString& aLibs) {
         if (libHandle) {
           mLibHandles.AppendElement(libHandle);
         } else {
-          MOZ_CRASH("Couldn't load lib needed by NSS");
+          // TODO(bug 1698718): remove the logging once we've identified
+          // the cause of the load failure.
+          const char* error = dlerror();
+          if (error) {
+            // We should always have an error, but gracefully handle just in
+            // case.
+            nsAutoCString nsError{error};
+            CrashReporter::AppendAppNotesToCrashReport(nsError);
+          }
+          // End bug 1698718 logging.
+
+          MOZ_CRASH("Couldn't load lib needed by media plugin");
         }
       }
     }
@@ -281,7 +309,7 @@ bool GMPChild::GetUTF8LibPath(nsACString& aOutLibPath) {
 
   nsAutoString path;
   libFile->GetPath(path);
-  aOutLibPath = NS_ConvertUTF16toUTF8(path);
+  CopyUTF16toUTF8(path, aOutLibPath);
 
   return true;
 #endif
@@ -315,14 +343,14 @@ static bool IsFileLeafEqualToASCII(const nsCOMPtr<nsIFile>& aFile,
 #endif
 
 #if defined(XP_WIN)
-#  define FIREFOX_FILE NS_LITERAL_STRING("firefox.exe")
-#  define XUL_LIB_FILE NS_LITERAL_STRING("xul.dll")
+#  define FIREFOX_FILE u"firefox.exe"_ns
+#  define XUL_LIB_FILE u"xul.dll"_ns
 #elif defined(XP_MACOSX)
-#  define FIREFOX_FILE NS_LITERAL_STRING("firefox")
-#  define XUL_LIB_FILE NS_LITERAL_STRING("XUL")
+#  define FIREFOX_FILE u"firefox"_ns
+#  define XUL_LIB_FILE u"XUL"_ns
 #else
-#  define FIREFOX_FILE NS_LITERAL_STRING("firefox")
-#  define XUL_LIB_FILE NS_LITERAL_STRING("libxul.so")
+#  define FIREFOX_FILE u"firefox"_ns
+#  define XUL_LIB_FILE u"libxul.so"_ns
 #endif
 
 static nsCOMPtr<nsIFile> GetFirefoxAppPath(
@@ -374,7 +402,7 @@ static bool GetSigPath(const int aRelativeLayers,
   }
   MOZ_ASSERT(path);
   aOutSigPath = path;
-  return NS_SUCCEEDED(path->Append(NS_LITERAL_STRING("Resources"))) &&
+  return NS_SUCCEEDED(path->Append(u"Resources"_ns)) &&
          NS_SUCCEEDED(path->Append(aTargetSigFileName));
 }
 #endif
@@ -394,21 +422,19 @@ static bool AppendHostPath(nsCOMPtr<nsIFile>& aFile,
   if (NS_FAILED(aFile->GetLeafName(binary))) {
     return false;
   }
-  binary.Append(NS_LITERAL_STRING(".sig"));
+  binary.Append(u".sig"_ns);
   nsCOMPtr<nsIFile> sigFile;
   if (GetSigPath(2, binary, aFile, sigFile) &&
       NS_SUCCEEDED(sigFile->GetPath(str))) {
-    sigFilePath = NS_ConvertUTF16toUTF8(str);
+    CopyUTF16toUTF8(str, sigFilePath);
   } else {
     // Cannot successfully get the sig file path.
     // Assume it is located at the same place as plugin-container
     // alternatively.
-    sigFilePath =
-        nsCString(NS_ConvertUTF16toUTF8(str) + NS_LITERAL_CSTRING(".sig"));
+    sigFilePath = nsCString(NS_ConvertUTF16toUTF8(str) + ".sig"_ns);
   }
 #else
-  sigFilePath =
-      nsCString(NS_ConvertUTF16toUTF8(str) + NS_LITERAL_CSTRING(".sig"));
+  sigFilePath = nsCString(NS_ConvertUTF16toUTF8(str) + ".sig"_ns);
 #endif
   aPaths.AppendElement(
       std::make_pair(std::move(filePath), std::move(sigFilePath)));
@@ -424,9 +450,9 @@ GMPChild::MakeCDMHostVerificationPaths() {
   nsString str;
   if (GetPluginFile(mPluginPath, path) && FileExists(path) &&
       ResolveLinks(path) && NS_SUCCEEDED(path->GetPath(str))) {
-    paths.AppendElement(std::make_pair(
-        nsCString(NS_ConvertUTF16toUTF8(str)),
-        nsCString(NS_ConvertUTF16toUTF8(str) + NS_LITERAL_CSTRING(".sig"))));
+    paths.AppendElement(
+        std::make_pair(nsCString(NS_ConvertUTF16toUTF8(str)),
+                       nsCString(NS_ConvertUTF16toUTF8(str) + ".sig"_ns)));
   }
 
   // Plugin-container binary path.
@@ -435,7 +461,7 @@ GMPChild::MakeCDMHostVerificationPaths() {
   const std::string pluginContainer =
       WideToUTF8(CommandLine::ForCurrentProcess()->program());
   path = nullptr;
-  str = NS_ConvertUTF8toUTF16(nsDependentCString(pluginContainer.c_str()));
+  CopyUTF8toUTF16(nsDependentCString(pluginContainer.c_str()), str);
   if (NS_FAILED(NS_NewLocalFile(str, true, /* aFollowLinks */
                                 getter_AddRefs(path))) ||
       !AppendHostPath(path, paths)) {
@@ -476,16 +502,10 @@ GMPChild::MakeCDMHostVerificationPaths() {
   return paths;
 }
 
-static nsCString ToCString(
-    const nsTArray<std::pair<nsCString, nsCString>>& aPairs) {
-  nsCString result;
-  for (const auto& p : aPairs) {
-    if (!result.IsEmpty()) {
-      result.AppendLiteral(",");
-    }
-    result.Append(nsPrintfCString("(%s,%s)", p.first.get(), p.second.get()));
-  }
-  return result;
+static auto ToCString(const nsTArray<std::pair<nsCString, nsCString>>& aPairs) {
+  return StringJoin(","_ns, aPairs, [](nsACString& dest, const auto& p) {
+    dest.AppendPrintf("(%s,%s)", p.first.get(), p.second.get());
+  });
 }
 
 mozilla::ipc::IPCResult GMPChild::AnswerStartPlugin(const nsString& aAdapter) {
@@ -655,12 +675,9 @@ mozilla::ipc::IPCResult GMPChild::RecvInitGMPContentChild(
 
 void GMPChild::GMPContentChildActorDestroy(GMPContentChild* aGMPContentChild) {
   for (uint32_t i = mGMPContentChildren.Length(); i > 0; i--) {
-    UniquePtr<GMPContentChild>& toDestroy = mGMPContentChildren[i - 1];
-    if (toDestroy.get() == aGMPContentChild) {
+    RefPtr<GMPContentChild>& destroyedActor = mGMPContentChildren[i - 1];
+    if (destroyedActor.get() == aGMPContentChild) {
       SendPGMPContentChildDestroyed();
-      RefPtr<DeleteTask<GMPContentChild>> task =
-          new DeleteTask<GMPContentChild>(toDestroy.release());
-      MessageLoop::current()->PostTask(task.forget());
       mGMPContentChildren.RemoveElementAt(i - 1);
       break;
     }
