@@ -60,7 +60,7 @@ class GMPParent final
  public:
   NS_INLINE_DECL_THREADSAFE_REFCOUNTING(GMPParent)
 
-  explicit GMPParent(AbstractThread* aMainThread);
+  GMPParent();
 
   RefPtr<GenericPromise> Init(GeckoMediaPluginServiceParent* aService,
                               nsIFile* aPluginDir);
@@ -150,7 +150,7 @@ class GMPParent final
   RefPtr<GenericPromise> ReadGMPMetaData();
   RefPtr<GenericPromise> ReadGMPInfoFile(nsIFile* aFile);
   RefPtr<GenericPromise> ParseChromiumManifest(
-      const nsAString& aJSON);  // Main thread.
+      const nsAString& aJSON);  // Worker thread.
   RefPtr<GenericPromise> ReadChromiumManifestFile(
       nsIFile* aFile);  // GMP thread.
   void AddCrashAnnotations();
@@ -175,6 +175,23 @@ class GMPParent final
   void ResolveGetContentParentPromises();
   void RejectGetContentParentPromises();
 
+#if defined(XP_MACOSX) && defined(__aarch64__)
+  // We pre-translate XUL and our plugin file to avoid x64 child process
+  // startup delays caused by translation for instances when the child
+  // process binary translations have not already been cached. i.e., the
+  // first time we launch an x64 child process after installation or
+  // update. Measured by binary size of a recent XUL and Widevine plugin,
+  // this makes up 94% of the translation needed. Re-translating the
+  // same binary does not cause translation to occur again.
+  void PreTranslateBins();
+  void PreTranslateBinsWorker();
+#endif
+
+#if defined(XP_MACOSX)
+  nsresult GetPluginFileArch(nsIFile* aPluginDir, nsAutoString& aLeafName,
+                             uint32_t& aArchSet);
+#endif
+
   GMPState mState;
   nsCOMPtr<nsIFile> mDirectory;  // plugin directory on disk
   nsString mName;  // base name of plugin on disk, UTF-16 because used for paths
@@ -185,7 +202,7 @@ class GMPParent final
   nsCString mLibs;
 #endif
   nsString mAdapter;
-  uint32_t mPluginId;
+  const uint32_t mPluginId;
   nsTArray<GMPCapability> mCapabilities;
   GMPProcessParent* mProcess;
   bool mDeleteProcessOnlyOnUnload;
@@ -214,7 +231,13 @@ class GMPParent final
   // to terminate gracefully.
   bool mHoldingSelfRef;
 
-  const RefPtr<AbstractThread> mMainThread;
+#if defined(XP_MACOSX) && defined(__aarch64__)
+  // The child process architecture to use.
+  uint32_t mChildLaunchArch;
+  nsCString mPluginFilePath;
+#endif
+
+  const nsCOMPtr<nsISerialEventTarget> mMainThread;
 };
 
 }  // namespace gmp
