@@ -38,6 +38,11 @@ namespace mozilla {
 
 const uint32_t kKeyIdSize = 16;
 
+// We ensure there are no gaps in samples' CTS between the last sample in a
+// Moof, and the first sample in the next Moof, if they're within these many
+// Microseconds of each other.
+const Microseconds CROSS_MOOF_CTS_MERGE_THRESHOLD = 1;
+
 bool MoofParser::RebuildFragmentedIndex(const MediaByteRangeSet& aByteRanges) {
   BoxContext context(mSource, aByteRanges);
   return RebuildFragmentedIndex(context);
@@ -73,7 +78,7 @@ bool MoofParser::RebuildFragmentedIndex(BoxContext& aContext) {
       ParseMoov(box);
     } else if (box.IsType("moof")) {
       Moof moof(box, mTrackParseMode, mTrex, mMvhd, mMdhd, mEdts, mSinf,
-                &mLastDecodeTime, mIsAudio);
+                &mLastDecodeTime, mIsAudio, mTracksEndCts);
 
       if (!moof.IsValid() && !box.Next().IsAvailable()) {
         // Moof isn't valid abort search for now.
@@ -88,8 +93,8 @@ bool MoofParser::RebuildFragmentedIndex(BoxContext& aContext) {
         mMoofs.LastElement().FixRounding(moof);
       }
 
-      mMoofs.AppendElement(moof);
       mMediaRanges.AppendElement(moof.mRange);
+      mMoofs.AppendElement(std::move(moof));
       foundValidMoof = true;
     } else if (box.IsType("mdat") && !Moofs().IsEmpty()) {
       // Check if we have all our data from last moof.
@@ -424,7 +429,8 @@ class CtsComparator {
 
 Moof::Moof(Box& aBox, const TrackParseMode& aTrackParseMode, Trex& aTrex,
            Mvhd& aMvhd, Mdhd& aMdhd, Edts& aEdts, Sinf& aSinf,
-           uint64_t* aDecodeTime, bool aIsAudio)
+           uint64_t* aDecodeTime, bool aIsAudio,
+           nsTArray<TrackEndCts>& aTracksEndCts)
     : mRange(aBox.Range()), mTfhd(aTrex), mMaxRoundingError(35000) {
   LOG_DEBUG(
       Moof,
@@ -457,8 +463,7 @@ Moof::Moof(Box& aBox, const TrackParseMode& aTrackParseMode, Trex& aTrex,
       mPsshes.AppendElement();
     }
     nsTArray<uint8_t>& pssh = mPsshes.LastElement();
-    pssh.AppendElements(box.Header());
-    pssh.AppendElements(box.Read());
+    pssh.AppendElements(std::move(box.ReadCompleteBox()));
   }
 
   if (IsValid()) {
@@ -475,6 +480,36 @@ Moof::Moof(Box& aBox, const TrackParseMode& aTrackParseMode, Trex& aTrex,
             ctsOrder[i]->mCompositionRange.start;
       }
 
+      // Ensure that there are no gaps between the first sample in this
+      // Moof and the preceeding Moof.
+      if (!ctsOrder.IsEmpty()) {
+        bool found = false;
+        // Track ID of the track we're parsing.
+        const uint32_t trackId = aTrex.mTrackId;
+        // Find the previous CTS end time of Moof preceeding the Moofs we just
+        // parsed, for the track we're parsing.
+        for (auto& prevCts : aTracksEndCts) {
+          if (prevCts.mTrackId == trackId) {
+            // We have previously parsed a Moof for this track. Smooth the gap
+            // between samples for this track across the Moof bounary.
+            if (ctsOrder[0]->mCompositionRange.start > prevCts.mCtsEndTime &&
+                ctsOrder[0]->mCompositionRange.start - prevCts.mCtsEndTime <=
+                    CROSS_MOOF_CTS_MERGE_THRESHOLD) {
+              ctsOrder[0]->mCompositionRange.start = prevCts.mCtsEndTime;
+            }
+            prevCts.mCtsEndTime = ctsOrder.LastElement()->mCompositionRange.end;
+            found = true;
+            break;
+          }
+        }
+        if (!found) {
+          // We've not parsed a Moof for this track yet. Save its CTS end
+          // time for the next Moof we parse.
+          aTracksEndCts.AppendElement(TrackEndCts(
+              trackId, ctsOrder.LastElement()->mCompositionRange.end));
+        }
+      }
+
       // In MP4, the duration of a sample is defined as the delta between two
       // decode timestamps. The operation above has updated the duration of each
       // sample as a Sample's duration is mCompositionRange.end -
@@ -487,7 +522,7 @@ Moof::Moof(Box& aBox, const TrackParseMode& aTrackParseMode, Trex& aTrex,
           aMdhd.ToMicroseconds((int64_t)*aDecodeTime - aEdts.mMediaStart);
       auto offsetOffset = aMvhd.ToMicroseconds(aEdts.mEmptyOffset);
       int64_t endDecodeTime =
-          decodeOffset.isOk() & offsetOffset.isOk()
+          (decodeOffset.isOk() && offsetOffset.isOk())
               ? decodeOffset.unwrap() + offsetOffset.unwrap()
               : 0;
       int64_t decodeDuration = endDecodeTime - mIndex[0].mDecodeTime;
@@ -821,6 +856,7 @@ Result<Ok, nsresult> Tkhd::Parse(Box& aBox) {
     MOZ_TRY_VAR(reserved, reader->ReadU32());
     MOZ_TRY_VAR(duration, reader->ReadU32());
 
+    (void)reserved;
     NS_ASSERTION(!reserved, "reserved should be 0");
 
     mCreationTime = creationTime;
@@ -832,6 +868,7 @@ Result<Ok, nsresult> Tkhd::Parse(Box& aBox) {
     MOZ_TRY_VAR(mModificationTime, reader->ReadU64());
     MOZ_TRY_VAR(mTrackId, reader->ReadU32());
     MOZ_TRY_VAR(reserved, reader->ReadU32());
+    (void)reserved;
     NS_ASSERTION(!reserved, "reserved should be 0");
     MOZ_TRY_VAR(mDuration, reader->ReadU64());
   }
@@ -1143,7 +1180,6 @@ Result<Ok, nsresult> Sgpd::Parse(Box& aBox) {
   uint32_t flags;
   MOZ_TRY_VAR(flags, reader->ReadU32());
   const uint8_t version = flags >> 24;
-  flags = flags & 0xffffff;
 
   uint32_t type;
   MOZ_TRY_VAR(type, reader->ReadU32());
