@@ -10,13 +10,31 @@
 #  include "AppleEncoderModule.h"
 #endif
 
+#ifdef MOZ_WIDGET_ANDROID
+#  include "AndroidEncoderModule.h"
+#endif
+
+#ifdef XP_WIN
+#  include "WMFEncoderModule.h"
+#endif
+
 namespace mozilla {
 
+LazyLogModule sPEMLog("PlatformEncoderModule");
+
 PEMFactory::PEMFactory() {
 #ifdef MOZ_APPLEMEDIA
   RefPtr<PlatformEncoderModule> m(new AppleEncoderModule());
   mModules.AppendElement(m);
 #endif
+
+#ifdef MOZ_WIDGET_ANDROID
+  mModules.AppendElement(new AndroidEncoderModule());
+#endif
+
+#ifdef XP_WIN
+  mModules.AppendElement(new WMFEncoderModule());
+#endif
 }
 
 bool PEMFactory::SupportsMimeType(const nsACString& aMimeType) const {
