@@ -17,8 +17,7 @@
 #include "mozilla/PodOperations.h"
 #include <deque>
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 // The maximum latency, in seconds, that we can live with before dropping
 // buffers.
@@ -536,5 +535,4 @@ void ScriptProcessorNode::UpdateConnectedStatus() {
   engine->GetSharedBuffers()->NotifyNodeIsConnected(isConnected);
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
