@@ -53,9 +53,8 @@ LogModule* GetSpeechRecognitionLog();
 class SpeechRecognition final : public DOMEventTargetHelper,
                                 public nsIObserver,
                                 public DOMMediaStream::TrackListener,
-                                public SupportsWeakPtr<SpeechRecognition> {
+                                public SupportsWeakPtr {
  public:
-  MOZ_DECLARE_WEAKREFERENCE_TYPENAME(SpeechRecognition)
   explicit SpeechRecognition(nsPIDOMWindowInner* aOwnerWindow);
 
   NS_DECL_ISUPPORTS_INHERITED
@@ -135,10 +134,17 @@ class SpeechRecognition final : public DOMEventTargetHelper,
   };
 
   void NotifyTrackAdded(const RefPtr<MediaStreamTrack>& aTrack) override;
-
+  // aMessage should be valid UTF-8, but invalid UTF-8 byte sequences are
+  // replaced with the REPLACEMENT CHARACTER on conversion to UTF-16.
   void DispatchError(EventType aErrorType,
                      SpeechRecognitionErrorCode aErrorCode,
-                     const nsAString& aMessage);
+                     const nsACString& aMessage);
+  template <int N>
+  void DispatchError(EventType aErrorType,
+                     SpeechRecognitionErrorCode aErrorCode,
+                     const char (&aMessage)[N]) {
+    DispatchError(aErrorType, aErrorCode, nsLiteralCString(aMessage));
+  }
   uint32_t FillSamplesBuffer(const int16_t* aSamples, uint32_t aSampleCount);
   uint32_t SplitSamplesBuffer(const int16_t* aSamplesBuffer,
                               uint32_t aSampleCount,
