@@ -17,6 +17,7 @@
 #include "nsReadableUtils.h"
 #include "nsServiceManagerUtils.h"
 #include "nsThreadUtils.h"
+#include "nsXULAppAPI.h"
 #include "prlink.h"
 
 #include <math.h>
@@ -121,8 +122,7 @@ struct nsSpeechDispatcherDynamicFunction {
   nsSpeechDispatcherFunc* function;
 };
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 StaticRefPtr<SpeechDispatcherService> SpeechDispatcherService::sSingleton;
 
@@ -398,9 +398,9 @@ void SpeechDispatcherService::Setup() {
 
       uri.Append(NS_ConvertUTF8toUTF16(lang));
 
-      mVoices.Put(uri, MakeRefPtr<SpeechDispatcherVoice>(
-                           NS_ConvertUTF8toUTF16(list[i]->name),
-                           NS_ConvertUTF8toUTF16(lang)));
+      mVoices.InsertOrUpdate(uri, MakeRefPtr<SpeechDispatcherVoice>(
+                                      NS_ConvertUTF8toUTF16(list[i]->name),
+                                      NS_ConvertUTF8toUTF16(lang)));
     }
   }
 
@@ -415,14 +415,14 @@ void SpeechDispatcherService::Setup() {
 
 void SpeechDispatcherService::RegisterVoices() {
   RefPtr<nsSynthVoiceRegistry> registry = nsSynthVoiceRegistry::GetInstance();
-  for (auto iter = mVoices.Iter(); !iter.Done(); iter.Next()) {
-    RefPtr<SpeechDispatcherVoice>& voice = iter.Data();
+  for (const auto& entry : mVoices) {
+    const RefPtr<SpeechDispatcherVoice>& voice = entry.GetData();
 
     // This service can only speak one utterance at a time, so we set
     // aQueuesUtterances to true in order to track global state and schedule
     // access to this service.
     DebugOnly<nsresult> rv =
-        registry->AddVoice(this, iter.Key(), voice->mName, voice->mLanguage,
+        registry->AddVoice(this, entry.GetKey(), voice->mName, voice->mLanguage,
                            voice->mName.EqualsLiteral("default"), true);
 
     NS_WARNING_ASSERTION(NS_SUCCEEDED(rv), "Failed to add voice");
@@ -502,7 +502,7 @@ SpeechDispatcherService::Speak(const nsAString& aText, const nsAString& aUri,
       return NS_ERROR_FAILURE;
     }
 
-    mCallbacks.Put(msg_id, std::move(callback));
+    mCallbacks.InsertOrUpdate(msg_id, std::move(callback));
   } else {
     // Speech dispatcher does not work well with empty strings.
     // In that case, don't send empty string to speechd,
@@ -553,5 +553,4 @@ void SpeechDispatcherService::EventNotify(uint32_t aMsgId, uint32_t aState) {
   }
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
