@@ -38,6 +38,7 @@
 #include "nsMimeTypes.h"
 #include "nsHtml5SVGLoadDispatcher.h"
 #include "nsTextNode.h"
+#include "mozilla/dom/AutoEntryScript.h"
 #include "mozilla/dom/CDATASection.h"
 #include "mozilla/dom/Comment.h"
 #include "mozilla/dom/DocumentType.h"
@@ -48,6 +49,7 @@
 #include "mozilla/dom/ScriptLoader.h"
 #include "mozilla/LoadInfo.h"
 #include "mozilla/PresShell.h"
+#include "mozilla/ProfilerLabels.h"
 
 #include "nsXULPrototypeCache.h"
 #include "nsXULElement.h"
@@ -119,7 +121,8 @@ nsresult PrototypeDocumentContentSink::Init(Document* aDoc, nsIURI* aURI,
 }
 
 NS_IMPL_CYCLE_COLLECTION(PrototypeDocumentContentSink, mParser, mDocumentURI,
-                         mDocument, mScriptLoader, mCurrentPrototype)
+                         mDocument, mScriptLoader, mContextStack,
+                         mCurrentPrototype)
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(PrototypeDocumentContentSink)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIContentSink)
@@ -185,13 +188,27 @@ void PrototypeDocumentContentSink::ContinueInterruptedParsingAsync() {
 PrototypeDocumentContentSink::ContextStack::ContextStack()
     : mTop(nullptr), mDepth(0) {}
 
-PrototypeDocumentContentSink::ContextStack::~ContextStack() {
+PrototypeDocumentContentSink::ContextStack::~ContextStack() { Clear(); }
+
+void PrototypeDocumentContentSink::ContextStack::Traverse(
+    nsCycleCollectionTraversalCallback& aCallback, const char* aName,
+    uint32_t aFlags) {
+  aFlags |= CycleCollectionEdgeNameArrayFlag;
+  Entry* current = mTop;
+  while (current) {
+    CycleCollectionNoteChild(aCallback, current->mElement, aName, aFlags);
+    current = current->mNext;
+  }
+}
+
+void PrototypeDocumentContentSink::ContextStack::Clear() {
   while (mTop) {
     Entry* doomed = mTop;
     mTop = mTop->mNext;
     NS_IF_RELEASE(doomed->mElement);
     delete doomed;
   }
+  mDepth = 0;
 }
 
 nsresult PrototypeDocumentContentSink::ContextStack::Push(
@@ -307,8 +324,11 @@ nsresult PrototypeDocumentContentSink::PrepareToWalk() {
   rv = CreateElementFromPrototype(proto, getter_AddRefs(root), nullptr);
   if (NS_FAILED(rv)) return rv;
 
-  rv = mDocument->AppendChildTo(root, false);
-  if (NS_FAILED(rv)) return rv;
+  ErrorResult error;
+  mDocument->AppendChildTo(root, false, error);
+  if (error.Failed()) {
+    return error.StealNSResult();
+  }
 
   // TODO(emilio): Should this really notify? We don't notify of appends anyhow,
   // and we just appended the root so no styles can possibly depend on it.
@@ -348,9 +368,11 @@ nsresult PrototypeDocumentContentSink::CreateAndInsertPI(
     rv = InsertXMLStylesheetPI(aProtoPI, aParent, aBeforeThis, pi);
   } else {
     // No special processing, just add the PI to the document.
-    rv = aParent->InsertChildBefore(
-        node->AsContent(), aBeforeThis ? aBeforeThis->AsContent() : nullptr,
-        false);
+    ErrorResult error;
+    aParent->InsertChildBefore(node->AsContent(),
+                               aBeforeThis ? aBeforeThis->AsContent() : nullptr,
+                               false, error);
+    rv = error.StealNSResult();
   }
 
   return rv;
@@ -359,16 +381,17 @@ nsresult PrototypeDocumentContentSink::CreateAndInsertPI(
 nsresult PrototypeDocumentContentSink::InsertXMLStylesheetPI(
     const nsXULPrototypePI* aProtoPI, nsINode* aParent, nsINode* aBeforeThis,
     XMLStylesheetProcessingInstruction* aPINode) {
-  nsresult rv;
-
   // We want to be notified when the style sheet finishes loading, so
   // disable style sheet loading for now.
   aPINode->SetEnableUpdates(false);
   aPINode->OverrideBaseURI(mCurrentPrototype->GetURI());
 
-  rv = aParent->InsertChildBefore(
-      aPINode, aBeforeThis ? aBeforeThis->AsContent() : nullptr, false);
-  if (NS_FAILED(rv)) return rv;
+  ErrorResult rv;
+  aParent->InsertChildBefore(
+      aPINode, aBeforeThis ? aBeforeThis->AsContent() : nullptr, false, rv);
+  if (rv.Failed()) {
+    return rv.StealNSResult();
+  }
 
   aPINode->SetEnableUpdates(true);
 
@@ -393,21 +416,49 @@ nsresult PrototypeDocumentContentSink::InsertXMLStylesheetPI(
   return NS_OK;
 }
 
-void PrototypeDocumentContentSink::CloseElement(Element* aElement) {
+void PrototypeDocumentContentSink::CloseElement(Element* aElement,
+                                                bool aHadChildren) {
   if (nsIContent::RequiresDoneAddingChildren(
           aElement->NodeInfo()->NamespaceID(),
           aElement->NodeInfo()->NameAtom())) {
     aElement->DoneAddingChildren(false);
   }
+
+  if (!aHadChildren) {
+    return;
+  }
+
+  // See bug 370111 and bug 1495946. We don't cache inline styles nor module
+  // scripts in the prototype cache, and we don't notify on node insertion, so
+  // we need to do this for the stylesheet / script to be properly processed.
+  // This kinda sucks, but notifying was a pretty sizeable perf regression so...
+  if (aElement->IsHTMLElement(nsGkAtoms::script) ||
+      aElement->IsSVGElement(nsGkAtoms::script)) {
+    nsCOMPtr<nsIScriptElement> sele = do_QueryInterface(aElement);
+    MOZ_ASSERT(sele, "Node didn't QI to script.");
+    if (sele->GetScriptIsModule()) {
+      DebugOnly<bool> block = sele->AttemptToExecute();
+      MOZ_ASSERT(!block, "<script type=module> shouldn't block the parser");
+    }
+  }
+
+  if (aElement->IsHTMLElement(nsGkAtoms::style) ||
+      aElement->IsSVGElement(nsGkAtoms::style)) {
+    auto* linkStyle = LinkStyle::FromNode(*aElement);
+    NS_ASSERTION(linkStyle,
+                 "<html:style> doesn't implement "
+                 "nsIStyleSheetLinkingElement?");
+    Unused << linkStyle->UpdateStyleSheet(nullptr);
+  }
 }
 
 nsresult PrototypeDocumentContentSink::ResumeWalk() {
   nsresult rv = ResumeWalkInternal();
   if (NS_FAILED(rv)) {
     nsContentUtils::ReportToConsoleNonLocalized(
-        NS_LITERAL_STRING("Failed to load document from prototype document."),
-        nsIScriptError::errorFlag, NS_LITERAL_CSTRING("Prototype Document"),
-        mDocument, mDocumentURI);
+        u"Failed to load document from prototype document."_ns,
+        nsIScriptError::errorFlag, "Prototype Document"_ns, mDocument,
+        mDocumentURI);
   }
   return rv;
 }
@@ -446,18 +497,7 @@ nsresult PrototypeDocumentContentSink::ResumeWalkInternal() {
       if (indx >= (int32_t)proto->mChildren.Length()) {
         if (element) {
           // We've processed all of the prototype's children.
-          CloseElement(element->AsElement());
-          if (element->NodeInfo()->Equals(nsGkAtoms::style,
-                                          kNameSpaceID_XHTML) ||
-              element->NodeInfo()->Equals(nsGkAtoms::style, kNameSpaceID_SVG)) {
-            // XXX sucks that we have to do this -
-            // see bug 370111
-            auto* linkStyle = LinkStyle::FromNode(*element);
-            NS_ASSERTION(linkStyle,
-                         "<html:style> doesn't implement "
-                         "nsIStyleSheetLinkingElement?");
-            Unused << linkStyle->UpdateStyleSheet(nullptr);
-          }
+          CloseElement(element->AsElement(), /* aHadChildren = */ true);
         }
         // Now pop the context stack back up to the parent
         // element and continue the prototype walk.
@@ -489,8 +529,11 @@ nsresult PrototypeDocumentContentSink::ResumeWalkInternal() {
           if (NS_FAILED(rv)) return rv;
 
           // ...and append it to the content model.
-          rv = nodeToPushTo->AppendChildTo(child, false);
-          if (NS_FAILED(rv)) return rv;
+          ErrorResult error;
+          nodeToPushTo->AppendChildTo(child, false, error);
+          if (error.Failed()) {
+            return error.StealNSResult();
+          }
 
           if (nsIContent::RequiresDoneCreatingElement(
                   protoele->mNodeInfo->NamespaceID(),
@@ -505,7 +548,7 @@ nsresult PrototypeDocumentContentSink::ResumeWalkInternal() {
             if (NS_FAILED(rv)) return rv;
           } else {
             // If there are no children, close the element immediately.
-            CloseElement(child);
+            CloseElement(child, /* aHadChildren = */ false);
           }
         } break;
 
@@ -538,8 +581,11 @@ nsresult PrototypeDocumentContentSink::ResumeWalkInternal() {
           auto* textproto = static_cast<nsXULPrototypeText*>(childproto);
           text->SetText(textproto->mValue, false);
 
-          rv = nodeToPushTo->AppendChildTo(text, false);
-          NS_ENSURE_SUCCESS(rv, rv);
+          ErrorResult error;
+          nodeToPushTo->AppendChildTo(text, false, error);
+          if (error.Failed()) {
+            return error.StealNSResult();
+          }
         } break;
 
         case nsXULPrototypeNode::eType_PI: {
@@ -551,10 +597,10 @@ nsresult PrototypeDocumentContentSink::ResumeWalkInternal() {
           if (piProto->mTarget.EqualsLiteral("xml-stylesheet")) {
             AutoTArray<nsString, 1> params = {piProto->mTarget};
 
-            nsContentUtils::ReportToConsole(
-                nsIScriptError::warningFlag, NS_LITERAL_CSTRING("XUL Document"),
-                nullptr, nsContentUtils::eXUL_PROPERTIES, "PINotInProlog",
-                params, docURI);
+            nsContentUtils::ReportToConsole(nsIScriptError::warningFlag,
+                                            "XUL Document"_ns, nullptr,
+                                            nsContentUtils::eXUL_PROPERTIES,
+                                            "PINotInProlog", params, docURI);
           }
 
           nsIContent* parent = element.get();
@@ -608,10 +654,9 @@ nsresult PrototypeDocumentContentSink::DoneWalking() {
     mDocument->SetReadyStateInternal(Document::READYSTATE_INTERACTIVE);
     mDocument->NotifyPossibleTitleChange(false);
 
-    nsContentUtils::DispatchTrustedEvent(
-        mDocument, ToSupports(mDocument),
-        NS_LITERAL_STRING("MozBeforeInitialXULLayout"), CanBubble::eYes,
-        Cancelable::eNo);
+    nsContentUtils::DispatchEventOnlyToChrome(mDocument, ToSupports(mDocument),
+                                              u"MozBeforeInitialXULLayout"_ns,
+                                              CanBubble::eYes, Cancelable::eNo);
   }
 
   if (mScriptLoader) {
@@ -734,11 +779,12 @@ nsresult PrototypeDocumentContentSink::LoadScript(
 
     // Note: the loader will keep itself alive while it's loading.
     nsCOMPtr<nsIStreamLoader> loader;
-    rv = NS_NewStreamLoader(getter_AddRefs(loader), aScriptProto->mSrcURI,
-                            this,       // aObserver
-                            mDocument,  // aRequestingContext
-                            nsILoadInfo::SEC_REQUIRE_SAME_ORIGIN_DATA_INHERITS,
-                            nsIContentPolicy::TYPE_INTERNAL_SCRIPT, group);
+    rv = NS_NewStreamLoader(
+        getter_AddRefs(loader), aScriptProto->mSrcURI,
+        this,       // aObserver
+        mDocument,  // aRequestingContext
+        nsILoadInfo::SEC_REQUIRE_SAME_ORIGIN_INHERITS_SEC_CONTEXT,
+        nsIContentPolicy::TYPE_INTERNAL_SCRIPT, group);
 
     if (NS_FAILED(rv)) {
       mCurrentScriptProto = nullptr;
@@ -805,7 +851,7 @@ PrototypeDocumentContentSink::OnStreamComplete(nsIStreamLoader* aLoader,
                                         !mOffThreadCompileStringBuf),
                "PrototypeDocument can't load multiple scripts at once");
 
-    rv = ScriptLoader::ConvertToUTF16(channel, string, stringLen, EmptyString(),
+    rv = ScriptLoader::ConvertToUTF16(channel, string, stringLen, u""_ns,
                                       mDocument, mOffThreadCompileStringBuf,
                                       mOffThreadCompileStringLength);
     if (NS_SUCCEEDED(rv)) {
@@ -835,6 +881,12 @@ PrototypeDocumentContentSink::OnStreamComplete(nsIStreamLoader* aLoader,
 NS_IMETHODIMP
 PrototypeDocumentContentSink::OnScriptCompileComplete(JSScript* aScript,
                                                       nsresult aStatus) {
+  // The mCurrentScriptProto may have been cleared out by another
+  // PrototypeDocumentContentSink.
+  if (!mCurrentScriptProto) {
+    return NS_OK;
+  }
+
   // When compiling off thread the script will not have been attached to the
   // script proto yet.
   if (aScript && !mCurrentScriptProto->HasScriptObject())
@@ -1035,15 +1087,25 @@ nsresult PrototypeDocumentContentSink::CreateElementFromPrototype(
     if (isScript) {
       nsCOMPtr<nsIScriptElement> sele = do_QueryInterface(result);
       MOZ_ASSERT(sele, "Node didn't QI to script.");
+
+      sele->FreezeExecutionAttrs(doc);
       // Script loading is handled by the this content sink, so prevent the
       // script from loading when it is bound to the document.
-      sele->PreventExecution();
+      //
+      // NOTE(emilio): This is only done for non-module scripts, because we
+      // don't support caching modules properly yet, see the comment in
+      // XULContentSinkImpl::OpenScript. For non-inline scripts, this is enough,
+      // since we can start the load when the node is inserted. Non-inline
+      // scripts need another special-case in CloseElement.
+      if (!sele->GetScriptIsModule()) {
+        sele->PreventExecution();
+      }
     }
   }
 
   // FIXME(bug 1627474): Is this right if this is inside an <html:template>?
   if (result->HasAttr(kNameSpaceID_None, nsGkAtoms::datal10nid)) {
-    mDocument->mL10nProtoElements.Put(result, RefPtr{aPrototype});
+    mDocument->mL10nProtoElements.InsertOrUpdate(result, RefPtr{aPrototype});
     result->SetElementCreatedFromPrototypeAndHasUnmodifiedL10n();
   }
   result.forget(aResult);
