@@ -104,7 +104,6 @@ inline CSPDirective CSP_StringToCSPDirective(const nsAString& aDir) {
       return static_cast<CSPDirective>(i);
     }
   }
-  NS_ASSERTION(false, "Can not convert unknown Directive to Integer");
   return nsIContentSecurityPolicy::NO_DIRECTIVE;
 }
 
@@ -198,7 +197,6 @@ class nsCSPHostSrc;
 
 nsCSPHostSrc* CSP_CreateHostSrcFromSelfURI(nsIURI* aSelfURI);
 bool CSP_IsEmptyDirective(const nsAString& aValue, const nsAString& aDir);
-bool CSP_IsValidDirective(const nsAString& aDir);
 bool CSP_IsDirective(const nsAString& aValue, CSPDirective aDir);
 bool CSP_IsKeyword(const nsAString& aValue, enum CSPKeyword aKey);
 bool CSP_IsQuotelessKeyword(const nsAString& aKey);
@@ -454,8 +452,6 @@ class nsCSPDirective {
     mSrcs = aSrcs.Clone();
   }
 
-  virtual bool restrictsContentType(nsContentPolicyType aContentType) const;
-
   inline bool isDefaultDirective() const {
     return mDirective == nsIContentSecurityPolicy::DEFAULT_SRC_DIRECTIVE;
   }
@@ -492,9 +488,6 @@ class nsCSPChildSrcDirective : public nsCSPDirective {
 
   void setRestrictWorkers() { mRestrictWorkers = true; }
 
-  virtual bool restrictsContentType(
-      nsContentPolicyType aContentType) const override;
-
   virtual bool equals(CSPDirective aDirective) const override;
 
  private:
@@ -516,9 +509,6 @@ class nsCSPScriptSrcDirective : public nsCSPDirective {
 
   void setRestrictWorkers() { mRestrictWorkers = true; }
 
-  virtual bool restrictsContentType(
-      nsContentPolicyType aContentType) const override;
-
   virtual bool equals(CSPDirective aDirective) const override;
 
  private:
@@ -559,7 +549,7 @@ class nsBlockAllMixedContentDirective : public nsCSPDirective {
 /*
  * Upgrading insecure requests includes the following actors:
  * (1) CSP:
- *     The CSP implementation whitelists the http-request
+ *     The CSP implementation allowlists the http-request
  *     in case the policy is executed in enforcement mode.
  *     The CSP implementation however does not allow http
  *     requests to succeed if executed in report-only mode.
@@ -567,7 +557,7 @@ class nsBlockAllMixedContentDirective : public nsCSPDirective {
  *     error back to the page.
  *
  * (2) MixedContent:
- *     The evalution of MixedContent whitelists all http
+ *     The evalution of MixedContent allowlists all http
  *     requests with the promise that the http requests
  *     gets upgraded to https before any data is fetched
  *     from the network.
@@ -622,10 +612,8 @@ class nsCSPPolicy {
   bool permits(CSPDirective aDirective, nsIURI* aUri, const nsAString& aNonce,
                bool aWasRedirected, bool aSpecific, bool aParserCreated,
                nsAString& outViolatedDirective) const;
-  bool permits(CSPDirective aDir, nsIURI* aUri, bool aSpecific) const;
-  bool allows(nsContentPolicyType aContentType, enum CSPKeyword aKeyword,
+  bool allows(CSPDirective aDirective, enum CSPKeyword aKeyword,
               const nsAString& aHashOrNonce, bool aParserCreated) const;
-  bool allows(nsContentPolicyType aContentType, enum CSPKeyword aKeyword) const;
   void toString(nsAString& outStr) const;
   void toDomCSPStruct(mozilla::dom::CSP& outCSP) const;
 
@@ -655,7 +643,7 @@ class nsCSPPolicy {
   void getReportURIs(nsTArray<nsString>& outReportURIs) const;
 
   void getDirectiveStringAndReportSampleForContentType(
-      nsContentPolicyType aContentType, nsAString& outDirective,
+      CSPDirective aDirective, nsAString& outDirective,
       bool* aReportSample) const;
 
   void getDirectiveAsString(CSPDirective aDir, nsAString& outDirective) const;
@@ -667,7 +655,7 @@ class nsCSPPolicy {
   bool visitDirectiveSrcs(CSPDirective aDir, nsCSPSrcVisitor* aVisitor) const;
 
   bool allowsNavigateTo(nsIURI* aURI, bool aWasRedirected,
-                        bool aEnforceWhitelist) const;
+                        bool aEnforceAllowlist) const;
 
  private:
   nsUpgradeInsecureDirective* mUpgradeInsecDir;
