@@ -8,22 +8,51 @@
 
 #include "nsContentSecurityUtils.h"
 
+#include "mozilla/Components.h"
+#include "mozilla/dom/nsMixedContentBlocker.h"
+#include "mozilla/dom/ScriptSettings.h"
+#include "nsComponentManagerUtils.h"
 #include "nsIContentSecurityPolicy.h"
 #include "nsIChannel.h"
 #include "nsIHttpChannel.h"
 #include "nsIMultiPartChannel.h"
 #include "nsIURI.h"
+#include "nsITransfer.h"
+#include "nsNetUtil.h"
+#include "nsSandboxFlags.h"
 #if defined(XP_WIN)
 #  include "WinUtils.h"
 #  include <wininet.h>
 #endif
 
+#include "FramingChecker.h"
 #include "js/Array.h"  // JS::GetArrayLength
+#include "js/ContextOptions.h"
+#include "js/RegExp.h"
+#include "js/RegExpFlags.h"  // JS::RegExpFlags
 #include "mozilla/ExtensionPolicyService.h"
 #include "mozilla/Logging.h"
+#include "mozilla/Preferences.h"
 #include "mozilla/dom/Document.h"
+#include "mozilla/dom/nsCSPContext.h"
+#include "mozilla/StaticPrefs_security.h"
+#include "LoadInfo.h"
 #include "mozilla/StaticPrefs_extensions.h"
 #include "mozilla/StaticPrefs_dom.h"
+#include "mozilla/Telemetry.h"
+#include "mozilla/TelemetryComms.h"
+#include "mozilla/TelemetryEventEnums.h"
+#include "nsIConsoleService.h"
+#include "nsIStringBundle.h"
+
+using namespace mozilla;
+using namespace mozilla::dom;
+using namespace mozilla::Telemetry;
+
+extern mozilla::LazyLogModule sCSMLog;
+extern Atomic<bool, mozilla::Relaxed> sJSHacksChecked;
+extern Atomic<bool, mozilla::Relaxed> sJSHacksPresent;
+extern Atomic<bool, mozilla::Relaxed> sTelemetryEventEnabled;
 
 // Helper function for IsConsideredSameOriginForUIR which makes
 // Principals of scheme 'http' return Principals of scheme 'https'.
@@ -102,13 +131,16 @@ nsresult RegexEval(const nsAString& aPattern, const nsAString& aString,
   MOZ_ASSERT(NS_IsMainThread());
   aMatchResult = false;
 
-  AutoJSAPI jsapi;
+  mozilla::dom::AutoJSAPI jsapi;
   jsapi.Init();
 
   JSContext* cx = jsapi.cx();
-  AutoDisableJSInterruptCallback disabler(cx);
+  mozilla::AutoDisableJSInterruptCallback disabler(cx);
 
-  JSAutoRealm ar(cx, xpc::UnprivilegedJunkScope());
+  // We can use the junk scope here, because we're just using it for regexp
+  // evaluation, not actual script execution, and we disable statics so that the
+  // evaluation does not interact with the execution global.
+  JSAutoRealm ar(cx, xpc::PrivilegedJunkScope());
 
   JS::RootedObject regexp(
       cx, JS::NewUCRegExpObject(cx, aPattern.BeginReading(), aPattern.Length(),
@@ -180,12 +212,9 @@ nsString OptimizeFileName(const nsAString& aFileName) {
       sCSMLog, LogLevel::Verbose,
       ("Optimizing FileName: %s", NS_ConvertUTF16toUTF8(optimizedName).get()));
 
-  optimizedName.ReplaceSubstring(NS_LITERAL_STRING(".xpi!"),
-                                 NS_LITERAL_STRING("!"));
-  optimizedName.ReplaceSubstring(NS_LITERAL_STRING("shield.mozilla.org!"),
-                                 NS_LITERAL_STRING("s!"));
-  optimizedName.ReplaceSubstring(NS_LITERAL_STRING("mozilla.org!"),
-                                 NS_LITERAL_STRING("m!"));
+  optimizedName.ReplaceSubstring(u".xpi!"_ns, u"!"_ns);
+  optimizedName.ReplaceSubstring(u"shield.mozilla.org!"_ns, u"s!"_ns);
+  optimizedName.ReplaceSubstring(u"mozilla.org!"_ns, u"m!"_ns);
   if (optimizedName.Length() > 80) {
     optimizedName.Truncate(80);
   }
@@ -212,41 +241,48 @@ nsString OptimizeFileName(const nsAString& aFileName) {
 FilenameTypeAndDetails nsContentSecurityUtils::FilenameToFilenameType(
     const nsString& fileName, bool collectAdditionalExtensionData) {
   // These are strings because the Telemetry Events API only accepts strings
-  static NS_NAMED_LITERAL_CSTRING(kChromeURI, "chromeuri");
-  static NS_NAMED_LITERAL_CSTRING(kResourceURI, "resourceuri");
-  static NS_NAMED_LITERAL_CSTRING(kBlobUri, "bloburi");
-  static NS_NAMED_LITERAL_CSTRING(kDataUri, "dataurl");
-  static NS_NAMED_LITERAL_CSTRING(kSingleString, "singlestring");
-  static NS_NAMED_LITERAL_CSTRING(kMozillaExtension, "mozillaextension");
-  static NS_NAMED_LITERAL_CSTRING(kOtherExtension, "otherextension");
-  static NS_NAMED_LITERAL_CSTRING(kSuspectedUserChromeJS,
-                                  "suspectedUserChromeJS");
+  static constexpr auto kChromeURI = "chromeuri"_ns;
+  static constexpr auto kResourceURI = "resourceuri"_ns;
+  static constexpr auto kBlobUri = "bloburi"_ns;
+  static constexpr auto kDataUri = "dataurl"_ns;
+  static constexpr auto kDataUriWebExtCStyle =
+      "dataurl-extension-contentstyle"_ns;
+  static constexpr auto kSingleString = "singlestring"_ns;
+  static constexpr auto kMozillaExtension = "mozillaextension"_ns;
+  static constexpr auto kOtherExtension = "otherextension"_ns;
+  static constexpr auto kSuspectedUserChromeJS = "suspectedUserChromeJS"_ns;
 #if defined(XP_WIN)
-  static NS_NAMED_LITERAL_CSTRING(kSanitizedWindowsURL, "sanitizedWindowsURL");
-  static NS_NAMED_LITERAL_CSTRING(kSanitizedWindowsPath,
-                                  "sanitizedWindowsPath");
+  static constexpr auto kSanitizedWindowsURL = "sanitizedWindowsURL"_ns;
+  static constexpr auto kSanitizedWindowsPath = "sanitizedWindowsPath"_ns;
 #endif
-  static NS_NAMED_LITERAL_CSTRING(kOther, "other");
-  static NS_NAMED_LITERAL_CSTRING(kOtherWorker, "other-on-worker");
-  static NS_NAMED_LITERAL_CSTRING(kRegexFailure, "regexfailure");
+  static constexpr auto kOther = "other"_ns;
+  static constexpr auto kOtherWorker = "other-on-worker"_ns;
+  static constexpr auto kRegexFailure = "regexfailure"_ns;
 
-  static NS_NAMED_LITERAL_STRING(kUCJSRegex, "(.+).uc.js\\?*[0-9]*$");
-  static NS_NAMED_LITERAL_STRING(kExtensionRegex, "extensions/(.+)@(.+)!(.+)$");
-  static NS_NAMED_LITERAL_STRING(kSingleFileRegex, "^[a-zA-Z0-9.?]+$");
+  static constexpr auto kUCJSRegex = u"(.+).uc.js\\?*[0-9]*$"_ns;
+  static constexpr auto kExtensionRegex = u"extensions/(.+)@(.+)!(.+)$"_ns;
+  static constexpr auto kSingleFileRegex = u"^[a-zA-Z0-9.?]+$"_ns;
+
+  if (fileName.IsEmpty()) {
+    return FilenameTypeAndDetails(kOther, Nothing());
+  }
 
   // resource:// and chrome://
-  if (StringBeginsWith(fileName, NS_LITERAL_STRING("chrome://"))) {
+  if (StringBeginsWith(fileName, u"chrome://"_ns)) {
     return FilenameTypeAndDetails(kChromeURI, Some(fileName));
   }
-  if (StringBeginsWith(fileName, NS_LITERAL_STRING("resource://"))) {
+  if (StringBeginsWith(fileName, u"resource://"_ns)) {
     return FilenameTypeAndDetails(kResourceURI, Some(fileName));
   }
 
   // blob: and data:
-  if (StringBeginsWith(fileName, NS_LITERAL_STRING("blob:"))) {
+  if (StringBeginsWith(fileName, u"blob:"_ns)) {
     return FilenameTypeAndDetails(kBlobUri, Nothing());
   }
-  if (StringBeginsWith(fileName, NS_LITERAL_STRING("data:"))) {
+  if (StringBeginsWith(fileName, u"data:text/css;extension=style;"_ns)) {
+    return FilenameTypeAndDetails(kDataUriWebExtCStyle, Nothing());
+  }
+  if (StringBeginsWith(fileName, u"data:"_ns)) {
     return FilenameTypeAndDetails(kDataUri, Nothing());
   }
 
@@ -264,10 +300,9 @@ FilenameTypeAndDetails nsContentSecurityUtils::FilenameToFilenameType(
     return FilenameTypeAndDetails(kRegexFailure, Nothing());
   }
   if (regexMatch) {
-    nsCString type =
-        StringEndsWith(regexResults[2], NS_LITERAL_STRING("mozilla.org.xpi"))
-            ? kMozillaExtension
-            : kOtherExtension;
+    nsCString type = StringEndsWith(regexResults[2], u"mozilla.org.xpi"_ns)
+                         ? kMozillaExtension
+                         : kOtherExtension;
     auto& extensionNameAndPath =
         Substring(regexResults[0], ArrayLength("extensions/") - 1);
     return FilenameTypeAndDetails(type,
@@ -300,18 +335,33 @@ FilenameTypeAndDetails nsContentSecurityUtils::FilenameToFilenameType(
   if (widget::WinUtils::PreparePathForTelemetry(strSanitizedPath, flags)) {
     DWORD cchDecodedUrl = INTERNET_MAX_URL_LENGTH;
     WCHAR szOut[INTERNET_MAX_URL_LENGTH];
-    HRESULT hr =
-        ::CoInternetParseUrl(fileName.get(), PARSE_SCHEMA, 0, szOut,
-                             INTERNET_MAX_URL_LENGTH, &cchDecodedUrl, 0);
+    HRESULT hr;
+    SAFECALL_URLMON_FUNC(CoInternetParseUrl, fileName.get(), PARSE_SCHEMA, 0,
+                         szOut, INTERNET_MAX_URL_LENGTH, &cchDecodedUrl, 0);
     if (hr == S_OK && cchDecodedUrl) {
       nsAutoString sanitizedPathAndScheme;
       sanitizedPathAndScheme.Append(szOut);
-      if (sanitizedPathAndScheme == NS_LITERAL_STRING("file")) {
-        sanitizedPathAndScheme.Append(NS_LITERAL_STRING("://.../"));
+      if (sanitizedPathAndScheme == u"about"_ns) {
+        int32_t desired_length = fileName.Length();
+        int32_t possible_new_length = 0;
+
+        possible_new_length = fileName.FindChar('?');
+        if (possible_new_length != -1 && possible_new_length < desired_length) {
+          desired_length = possible_new_length;
+        }
+
+        possible_new_length = fileName.FindChar('#');
+        if (possible_new_length != -1 && possible_new_length < desired_length) {
+          desired_length = possible_new_length;
+        }
+
+        sanitizedPathAndScheme = Substring(fileName, 0, desired_length);
+      } else if (sanitizedPathAndScheme == u"file"_ns) {
+        sanitizedPathAndScheme.Append(u"://.../"_ns);
         sanitizedPathAndScheme.Append(strSanitizedPath);
-      } else if (sanitizedPathAndScheme == NS_LITERAL_STRING("moz-extension") &&
+      } else if (sanitizedPathAndScheme == u"moz-extension"_ns &&
                  collectAdditionalExtensionData) {
-        sanitizedPathAndScheme.Append(NS_LITERAL_STRING("://["));
+        sanitizedPathAndScheme.Append(u"://["_ns);
 
         nsCOMPtr<nsIURI> uri;
         nsresult rv = NS_NewURI(getter_AddRefs(uri), fileName);
@@ -331,17 +381,15 @@ FilenameTypeAndDetails nsContentSecurityUtils::FilenameToFilenameType(
             policy->GetId(addOnId);
 
             sanitizedPathAndScheme.Append(addOnId);
-            sanitizedPathAndScheme.Append(NS_LITERAL_STRING(": "));
+            sanitizedPathAndScheme.Append(u": "_ns);
             sanitizedPathAndScheme.Append(policy->Name());
           } else {
-            sanitizedPathAndScheme.Append(
-                NS_LITERAL_STRING("failed finding addon by host"));
+            sanitizedPathAndScheme.Append(u"failed finding addon by host"_ns);
           }
         } else {
-          sanitizedPathAndScheme.Append(
-              NS_LITERAL_STRING("can't get addon off main thread"));
+          sanitizedPathAndScheme.Append(u"can't get addon off main thread"_ns);
         }
-        sanitizedPathAndScheme.Append(NS_LITERAL_STRING("]"));
+        sanitizedPathAndScheme.Append(u"]"_ns);
         sanitizedPathAndScheme.Append(url.FilePath());
       }
       return FilenameTypeAndDetails(kSanitizedWindowsURL,
@@ -385,11 +433,6 @@ class EvalUsageNotificationRunnable final : public Runnable {
   uint32_t mColumnNumber;
 };
 
-// The Web Extension process pref may be toggled during a session, at which
-// point stuff may be loaded in the parent process but we would send telemetry
-// for it. Avoid this by observing if the pref ever was disabled.
-static bool sWebExtensionsRemoteWasEverDisabled = false;
-
 /* static */
 bool nsContentSecurityUtils::IsEvalAllowed(JSContext* cx,
                                            bool aIsSystemPrincipal,
@@ -399,32 +442,32 @@ bool nsContentSecurityUtils::IsEvalAllowed(JSContext* cx,
   // exclusively used in testing contexts.
   static nsLiteralCString evalAllowlist[] = {
       // Test-only third-party library
-      NS_LITERAL_CSTRING("resource://testing-common/sinon-7.2.7.js"),
+      "resource://testing-common/sinon-7.2.7.js"_ns,
       // Test-only third-party library
-      NS_LITERAL_CSTRING("resource://testing-common/ajv-4.1.1.js"),
+      "resource://testing-common/ajv-4.1.1.js"_ns,
       // Test-only utility
-      NS_LITERAL_CSTRING("resource://testing-common/content-task.js"),
+      "resource://testing-common/content-task.js"_ns,
 
       // Tracked by Bug 1584605
-      NS_LITERAL_CSTRING("resource:///modules/translation/cld-worker.js"),
+      "resource:///modules/translation/cld-worker.js"_ns,
 
       // require.js implements a script loader for workers. It uses eval
       // to load the script; but injection is only possible in situations
       // that you could otherwise control script that gets executed, so
       // it is okay to allow eval() as it adds no additional attack surface.
       // Bug 1584564 tracks requiring safe usage of require.js
-      NS_LITERAL_CSTRING("resource://gre/modules/workers/require.js"),
+      "resource://gre/modules/workers/require.js"_ns,
 
       // The Browser Toolbox/Console
-      NS_LITERAL_CSTRING("debugger"),
+      "debugger"_ns,
   };
 
   // We also permit two specific idioms in eval()-like contexts. We'd like to
   // elminate these too; but there are in-the-wild Mozilla privileged extensions
   // that use them.
-  static NS_NAMED_LITERAL_STRING(sAllowedEval1, "this");
-  static NS_NAMED_LITERAL_STRING(sAllowedEval2,
-                                 "function anonymous(\n) {\nreturn this\n}");
+  static constexpr auto sAllowedEval1 = u"this"_ns;
+  static constexpr auto sAllowedEval2 =
+      u"function anonymous(\n) {\nreturn this\n}"_ns;
 
   if (MOZ_LIKELY(!aIsSystemPrincipal && !XRE_IsE10sParentProcess())) {
     // We restrict eval in the system principal and parent process.
@@ -454,16 +497,12 @@ bool nsContentSecurityUtils::IsEvalAllowed(JSContext* cx,
     return true;
   }
 
-  // We only perform a check of this preference on the Main Thread
+  // We can only perform the check of this preference on the Main Thread
   // (because a String-based preference check is only safe on Main Thread.)
-  // The consequence of this is that if a user is using userChromeJS _and_
-  // the scripts they use start a worker and that worker uses eval - we will
-  // enter this function, skip over this pref check that would normally cause
-  // us to allow the eval usage - and we will block it.
-  // While not ideal, we do not officially support userChromeJS, and hopefully
-  // the usage of workers and eval in workers is even lower that userChromeJS
-  // usage.
-  if (NS_IsMainThread()) {
+  // In theory, it would be possible that a separate thread could get here
+  // before the main thread, resulting in the other thread not being able to
+  // perform this check, but the odds of that are small (and probably zero.)
+  if (MOZ_UNLIKELY(!sJSHacksChecked) && NS_IsMainThread()) {
     // This preference is a file used for autoconfiguration of Firefox
     // by administrators. It has also been (ab)used by the userChromeJS
     // project to run legacy-style 'extensions', some of which use eval,
@@ -471,29 +510,37 @@ bool nsContentSecurityUtils::IsEvalAllowed(JSContext* cx,
     nsAutoString jsConfigPref;
     Preferences::GetString("general.config.filename", jsConfigPref);
     if (!jsConfigPref.IsEmpty()) {
-      MOZ_LOG(sCSMLog, LogLevel::Debug,
-              ("Allowing eval() %s because of "
-               "general.config.filename",
-               (aIsSystemPrincipal ? "with System Principal"
-                                   : "in parent process")));
-      return true;
+      sJSHacksPresent = true;
     }
+
+    // This preference is required by bootstrapLoader.xpi, which is an
+    // alternate way to load legacy-style extensions. It only works on
+    // DevEdition/Nightly.
+    bool xpinstallSignatures;
+    Preferences::GetBool("xpinstall.signatures.required", &xpinstallSignatures);
+    if (!xpinstallSignatures) {
+      sJSHacksPresent = true;
+    }
+
+    sJSHacksChecked = true;
+  }
+
+  if (MOZ_UNLIKELY(sJSHacksPresent)) {
+    MOZ_LOG(
+        sCSMLog, LogLevel::Debug,
+        ("Allowing eval() %s because some "
+         "JS hacks may be present.",
+         (aIsSystemPrincipal ? "with System Principal" : "in parent process")));
+    return true;
   }
 
   if (XRE_IsE10sParentProcess() &&
       !StaticPrefs::extensions_webextensions_remote()) {
-    sWebExtensionsRemoteWasEverDisabled = true;
     MOZ_LOG(sCSMLog, LogLevel::Debug,
             ("Allowing eval() in parent process because the web extension "
              "process is disabled"));
     return true;
   }
-  if (XRE_IsE10sParentProcess() && sWebExtensionsRemoteWasEverDisabled) {
-    MOZ_LOG(sCSMLog, LogLevel::Debug,
-            ("Allowing eval() in parent process because the web extension "
-             "process was disabled at some point"));
-    return true;
-  }
 
   // We permit these two common idioms to get access to the global JS object
   if (!aScript.IsEmpty() &&
@@ -512,7 +559,7 @@ bool nsContentSecurityUtils::IsEvalAllowed(JSContext* cx,
   uint32_t lineNumber = 0, columnNumber = 0;
   nsJSUtils::GetCallingLocation(cx, fileName, &lineNumber, &columnNumber);
   if (fileName.IsEmpty()) {
-    fileName = NS_LITERAL_CSTRING("unknown-file");
+    fileName = "unknown-file"_ns;
   }
 
   NS_ConvertUTF8toUTF16 fileNameA(fileName);
@@ -567,13 +614,7 @@ bool nsContentSecurityUtils::IsEvalAllowed(JSContext* cx,
       fileName.get(), trimmedScript.get());
 #endif
 
-#ifdef EARLY_BETA_OR_EARLIER
-  // Until we understand the events coming from release, we don't want to
-  // enforce eval restrictions on release. Limiting to Nightly and early beta.
   return false;
-#else
-  return true;
-#endif
 }
 
 /* static */
@@ -592,14 +633,14 @@ void nsContentSecurityUtils::NotifyEvalUsage(bool aIsSystemPrincipal,
   mozilla::Maybe<nsTArray<EventExtraEntry>> extra;
   if (fileNameTypeAndDetails.second.isSome()) {
     extra = Some<nsTArray<EventExtraEntry>>({EventExtraEntry{
-        NS_LITERAL_CSTRING("fileinfo"),
+        "fileinfo"_ns,
         NS_ConvertUTF16toUTF8(fileNameTypeAndDetails.second.value())}});
   } else {
     extra = Nothing();
   }
   if (!sTelemetryEventEnabled.exchange(true)) {
     sTelemetryEventEnabled = true;
-    Telemetry::SetEventRecordingEnabled(NS_LITERAL_CSTRING("security"), true);
+    Telemetry::SetEventRecordingEnabled("security"_ns, true);
   }
   Telemetry::RecordEvent(eventType, mozilla::Some(fileNameTypeAndDetails.first),
                          extra);
@@ -616,7 +657,7 @@ void nsContentSecurityUtils::NotifyEvalUsage(bool aIsSystemPrincipal,
   }
   nsCOMPtr<nsIStringBundle> bundle;
   nsCOMPtr<nsIStringBundleService> stringService =
-      mozilla::services::GetStringBundleService();
+      mozilla::components::StringBundle::Service();
   if (!stringService) {
     return;
   }
@@ -634,7 +675,7 @@ void nsContentSecurityUtils::NotifyEvalUsage(bool aIsSystemPrincipal,
     return;
   }
 
-  rv = error->InitWithWindowID(message, aFileNameA, EmptyString(), aLineNumber,
+  rv = error->InitWithWindowID(message, aFileNameA, u""_ns, aLineNumber,
                                aColumnNumber, nsIScriptError::errorFlag,
                                "BrowserEvalUsage", aWindowID,
                                true /* From chrome context */);
@@ -689,22 +730,21 @@ nsresult ParseCSPAndEnforceFrameAncestorCheck(
   }
 
   nsCOMPtr<nsILoadInfo> loadInfo = aChannel->LoadInfo();
-  nsContentPolicyType contentType = loadInfo->GetExternalContentPolicyType();
+  ExtContentPolicyType contentType = loadInfo->GetExternalContentPolicyType();
   // frame-ancestor check only makes sense for subdocument and object loads,
   // if this is not a load of such type, there is nothing to do here.
-  if (contentType != nsIContentPolicy::TYPE_SUBDOCUMENT &&
-      contentType != nsIContentPolicy::TYPE_OBJECT) {
+  if (contentType != ExtContentPolicy::TYPE_SUBDOCUMENT &&
+      contentType != ExtContentPolicy::TYPE_OBJECT) {
     return NS_OK;
   }
 
   nsAutoCString tCspHeaderValue, tCspROHeaderValue;
 
-  Unused << httpChannel->GetResponseHeader(
-      NS_LITERAL_CSTRING("content-security-policy"), tCspHeaderValue);
+  Unused << httpChannel->GetResponseHeader("content-security-policy"_ns,
+                                           tCspHeaderValue);
 
   Unused << httpChannel->GetResponseHeader(
-      NS_LITERAL_CSTRING("content-security-policy-report-only"),
-      tCspROHeaderValue);
+      "content-security-policy-report-only"_ns, tCspROHeaderValue);
 
   // if there are no CSP values, then there is nothing to do here.
   if (tCspHeaderValue.IsEmpty() && tCspROHeaderValue.IsEmpty()) {
@@ -818,6 +858,9 @@ void nsContentSecurityUtils::AssertAboutPageHasCSP(Document* aDocument) {
   bool foundObjectSrc = false;
   bool foundUnsafeEval = false;
   bool foundUnsafeInline = false;
+  bool foundScriptSrc = false;
+  bool foundWorkerSrc = false;
+  bool foundWebScheme = false;
   if (csp) {
     uint32_t policyCount = 0;
     csp->GetPolicyCount(&policyCount);
@@ -836,6 +879,16 @@ void nsContentSecurityUtils::AssertAboutPageHasCSP(Document* aDocument) {
       if (parsedPolicyStr.Find("'unsafe-inline'") >= 0) {
         foundUnsafeInline = true;
       }
+      if (parsedPolicyStr.Find("script-src") >= 0) {
+        foundScriptSrc = true;
+      }
+      if (parsedPolicyStr.Find("worker-src") >= 0) {
+        foundWorkerSrc = true;
+      }
+      if (parsedPolicyStr.Find("http:") >= 0 ||
+          parsedPolicyStr.Find("https:") >= 0) {
+        foundWebScheme = true;
+      }
     }
   }
 
@@ -854,17 +907,17 @@ void nsContentSecurityUtils::AssertAboutPageHasCSP(Document* aDocument) {
   // render without a CSP applied.
   static nsLiteralCString sAllowedAboutPagesWithNoCSP[] = {
     // about:blank is a special about page -> no CSP
-    NS_LITERAL_CSTRING("about:blank"),
+    "about:blank"_ns,
     // about:srcdoc is a special about page -> no CSP
-    NS_LITERAL_CSTRING("about:srcdoc"),
+    "about:srcdoc"_ns,
     // about:sync-log displays plain text only -> no CSP
-    NS_LITERAL_CSTRING("about:sync-log"),
+    "about:sync-log"_ns,
     // about:printpreview displays plain text only -> no CSP
-    NS_LITERAL_CSTRING("about:printpreview"),
+    "about:printpreview"_ns,
     // about:logo just displays the firefox logo -> no CSP
-    NS_LITERAL_CSTRING("about:logo"),
+    "about:logo"_ns,
 #  if defined(ANDROID)
-    NS_LITERAL_CSTRING("about:config"),
+    "about:config"_ns,
 #  endif
   };
 
@@ -882,6 +935,38 @@ void nsContentSecurityUtils::AssertAboutPageHasCSP(Document* aDocument) {
   MOZ_ASSERT(foundObjectSrc,
              "about: page must contain a CSP denying object-src");
 
+  // preferences and downloads allow legacy inline scripts through hash src.
+  MOZ_ASSERT(!foundScriptSrc ||
+                 StringBeginsWith(aboutSpec, "about:preferences"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:downloads"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:newtab"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:logins"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:compat"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:welcome"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:profiling"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:studies"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:home"_ns),
+             "about: page must not contain a CSP including script-src");
+
+  MOZ_ASSERT(!foundWorkerSrc,
+             "about: page must not contain a CSP including worker-src");
+
+  // addons, preferences, debugging, ion, devtools all have to allow some
+  // remote web resources
+  MOZ_ASSERT(!foundWebScheme ||
+                 StringBeginsWith(aboutSpec, "about:preferences"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:addons"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:newtab"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:debugging"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:ion"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:compat"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:logins"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:home"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:welcome"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:devtools"_ns) ||
+                 StringBeginsWith(aboutSpec, "about:pocket-saved"_ns),
+             "about: page must not contain a CSP including a web scheme");
+
   if (aDocument->IsExtensionPage()) {
     // Extensions have two CSP policies applied where the baseline CSP
     // includes 'unsafe-eval' and 'unsafe-inline', hence we have to skip
@@ -896,16 +981,16 @@ void nsContentSecurityUtils::AssertAboutPageHasCSP(Document* aDocument) {
   static nsLiteralCString sLegacyUnsafeInlineAllowList[] = {
       // Bug 1579160: Remove 'unsafe-inline' from style-src within
       // about:preferences
-      NS_LITERAL_CSTRING("about:preferences"),
+      "about:preferences"_ns,
       // Bug 1571346: Remove 'unsafe-inline' from style-src within about:addons
-      NS_LITERAL_CSTRING("about:addons"),
+      "about:addons"_ns,
       // Bug 1584485: Remove 'unsafe-inline' from style-src within:
       // * about:newtab
       // * about:welcome
       // * about:home
-      NS_LITERAL_CSTRING("about:newtab"),
-      NS_LITERAL_CSTRING("about:welcome"),
-      NS_LITERAL_CSTRING("about:home"),
+      "about:newtab"_ns,
+      "about:welcome"_ns,
+      "about:home"_ns,
   };
 
   for (const nsLiteralCString& aUnsafeInlineEntry :
@@ -926,7 +1011,6 @@ void nsContentSecurityUtils::AssertAboutPageHasCSP(Document* aDocument) {
 /* static */
 bool nsContentSecurityUtils::ValidateScriptFilename(const char* aFilename,
                                                     bool aIsSystemRealm) {
-  static Maybe<bool> sGeneralConfigFilenameSet;
   // If the pref is permissive, allow everything
   if (StaticPrefs::security_allow_parent_unrestricted_js_loads()) {
     return true;
@@ -937,67 +1021,69 @@ bool nsContentSecurityUtils::ValidateScriptFilename(const char* aFilename,
     return true;
   }
 
-  // We only perform a check of this preference on the Main Thread
+  // We can only perform the check of this preference on the Main Thread
   // (because a String-based preference check is only safe on Main Thread.)
-  // The consequence of this is that if a user is using userChromeJS _and_
-  // the scripts they use start a worker - we will enter this function,
-  // skip over this pref check that would normally cause us to allow the
-  // load - and we will block it.
-  // While not ideal, we do not officially support userChromeJS, and hopefully
-  // the usage of workers is even lower than userChromeJS usage.
-  if (NS_IsMainThread()) {
+  // In theory, it would be possible that a separate thread could get here
+  // before the main thread, resulting in the other thread not being able to
+  // perform this check, but the odds of that are small (and probably zero.)
+  if (MOZ_UNLIKELY(!sJSHacksChecked) && NS_IsMainThread()) {
     // This preference is a file used for autoconfiguration of Firefox
-    // by administrators. It will also run in the parent process and throw
-    // assumptions about what can run where out of the window.
-    if (!sGeneralConfigFilenameSet.isSome()) {
-      nsAutoString jsConfigPref;
-      Preferences::GetString("general.config.filename", jsConfigPref);
-      sGeneralConfigFilenameSet.emplace(!jsConfigPref.IsEmpty());
+    // by administrators. It has also been (ab)used by the userChromeJS
+    // project to run legacy-style 'extensions', some of which use eval,
+    // all of which run in the System Principal context.
+    nsAutoString jsConfigPref;
+    Preferences::GetString("general.config.filename", jsConfigPref);
+    if (!jsConfigPref.IsEmpty()) {
+      sJSHacksPresent = true;
     }
-    if (sGeneralConfigFilenameSet.value()) {
-      MOZ_LOG(sCSMLog, LogLevel::Debug,
-              ("Allowing a javascript load of %s because "
-               "general.config.filename is set",
-               aFilename));
-      return true;
+
+    // This preference is required by bootstrapLoader.xpi, which is an
+    // alternate way to load legacy-style extensions. It only works on
+    // DevEdition/Nightly.
+    bool xpinstallSignatures;
+    Preferences::GetBool("xpinstall.signatures.required", &xpinstallSignatures);
+    if (!xpinstallSignatures) {
+      sJSHacksPresent = true;
     }
+
+    sJSHacksChecked = true;
+  }
+
+  if (MOZ_UNLIKELY(sJSHacksPresent)) {
+    MOZ_LOG(sCSMLog, LogLevel::Debug,
+            ("Allowing a javascript load of %s because "
+             "some JS hacks may be present",
+             aFilename));
+    return true;
   }
 
   if (XRE_IsE10sParentProcess() &&
       !StaticPrefs::extensions_webextensions_remote()) {
-    sWebExtensionsRemoteWasEverDisabled = true;
     MOZ_LOG(sCSMLog, LogLevel::Debug,
             ("Allowing a javascript load of %s because the web extension "
              "process is disabled.",
              aFilename));
     return true;
   }
-  if (XRE_IsE10sParentProcess() && sWebExtensionsRemoteWasEverDisabled) {
-    MOZ_LOG(sCSMLog, LogLevel::Debug,
-            ("Allowing a javascript load of %s because the web extension "
-             "process was disabled at some point.",
-             aFilename));
-    return true;
-  }
 
   NS_ConvertUTF8toUTF16 filenameU(aFilename);
-  if (StringBeginsWith(filenameU, NS_LITERAL_STRING("chrome://"))) {
+  if (StringBeginsWith(filenameU, u"chrome://"_ns)) {
     // If it's a chrome:// url, allow it
     return true;
   }
-  if (StringBeginsWith(filenameU, NS_LITERAL_STRING("resource://"))) {
+  if (StringBeginsWith(filenameU, u"resource://"_ns)) {
     // If it's a resource:// url, allow it
     return true;
   }
-  if (StringBeginsWith(filenameU, NS_LITERAL_STRING("file://"))) {
+  if (StringBeginsWith(filenameU, u"file://"_ns)) {
     // We will temporarily allow all file:// URIs through for now
     return true;
   }
-  if (StringBeginsWith(filenameU, NS_LITERAL_STRING("jar:file://"))) {
+  if (StringBeginsWith(filenameU, u"jar:file://"_ns)) {
     // We will temporarily allow all jar URIs through for now
     return true;
   }
-  if (filenameU.Equals(NS_LITERAL_STRING("about:sync-log"))) {
+  if (filenameU.Equals(u"about:sync-log"_ns)) {
     // about:sync-log runs in the parent process and displays a directory
     // listing. The listing has inline javascript that executes on load.
     return true;
@@ -1018,7 +1104,7 @@ bool nsContentSecurityUtils::ValidateScriptFilename(const char* aFilename,
   mozilla::Maybe<nsTArray<EventExtraEntry>> extra;
   if (fileNameTypeAndDetails.second.isSome()) {
     extra = Some<nsTArray<EventExtraEntry>>({EventExtraEntry{
-        NS_LITERAL_CSTRING("fileinfo"),
+        "fileinfo"_ns,
         NS_ConvertUTF16toUTF8(fileNameTypeAndDetails.second.value())}});
   } else {
     extra = Nothing();
@@ -1026,7 +1112,7 @@ bool nsContentSecurityUtils::ValidateScriptFilename(const char* aFilename,
 
   if (!sTelemetryEventEnabled.exchange(true)) {
     sTelemetryEventEnabled = true;
-    Telemetry::SetEventRecordingEnabled(NS_LITERAL_CSTRING("security"), true);
+    Telemetry::SetEventRecordingEnabled("security"_ns, true);
   }
   Telemetry::RecordEvent(eventType, mozilla::Some(fileNameTypeAndDetails.first),
                          extra);
@@ -1036,3 +1122,99 @@ bool nsContentSecurityUtils::ValidateScriptFilename(const char* aFilename,
   // builds and return false to prevent execution in non-debug builds.
   return true;
 }
+
+/* static */
+void nsContentSecurityUtils::LogMessageToConsole(nsIHttpChannel* aChannel,
+                                                 const char* aMsg) {
+  nsCOMPtr<nsIURI> uri;
+  nsresult rv = aChannel->GetURI(getter_AddRefs(uri));
+  if (NS_FAILED(rv)) {
+    return;
+  }
+
+  uint64_t windowID = 0;
+  rv = aChannel->GetTopLevelContentWindowId(&windowID);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return;
+  }
+  if (!windowID) {
+    nsCOMPtr<nsILoadInfo> loadInfo = aChannel->LoadInfo();
+    loadInfo->GetInnerWindowID(&windowID);
+  }
+
+  nsAutoString localizedMsg;
+  nsAutoCString spec;
+  uri->GetSpec(spec);
+  AutoTArray<nsString, 1> params = {NS_ConvertUTF8toUTF16(spec)};
+  rv = nsContentUtils::FormatLocalizedString(
+      nsContentUtils::eSECURITY_PROPERTIES, aMsg, params, localizedMsg);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return;
+  }
+
+  nsContentUtils::ReportToConsoleByWindowID(
+      localizedMsg, nsIScriptError::warningFlag, "Security"_ns, windowID, uri);
+}
+
+/* static */
+long nsContentSecurityUtils::ClassifyDownload(
+    nsIChannel* aChannel, const nsAutoCString& aMimeTypeGuess) {
+  MOZ_ASSERT(aChannel, "IsDownloadAllowed without channel?");
+
+  nsCOMPtr<nsILoadInfo> loadInfo = aChannel->LoadInfo();
+
+  nsCOMPtr<nsIURI> contentLocation;
+  aChannel->GetURI(getter_AddRefs(contentLocation));
+
+  nsCOMPtr<nsIPrincipal> loadingPrincipal = loadInfo->GetLoadingPrincipal();
+  if (!loadingPrincipal) {
+    loadingPrincipal = loadInfo->TriggeringPrincipal();
+  }
+  // Creating a fake Loadinfo that is just used for the MCB check.
+  nsCOMPtr<nsILoadInfo> secCheckLoadInfo = new mozilla::net::LoadInfo(
+      loadingPrincipal, loadInfo->TriggeringPrincipal(), nullptr,
+      nsILoadInfo::SEC_ONLY_FOR_EXPLICIT_CONTENTSEC_CHECK,
+      nsIContentPolicy::TYPE_FETCH);
+
+  int16_t decission = nsIContentPolicy::ACCEPT;
+  nsMixedContentBlocker::ShouldLoad(false,  //  aHadInsecureImageRedirect
+                                    contentLocation,   //  aContentLocation,
+                                    secCheckLoadInfo,  //  aLoadinfo
+                                    aMimeTypeGuess,    //  aMimeGuess,
+                                    false,             //  aReportError
+                                    &decission         // aDecision
+  );
+  Telemetry::Accumulate(mozilla::Telemetry::MIXED_CONTENT_DOWNLOADS,
+                        decission != nsIContentPolicy::ACCEPT);
+
+  if (StaticPrefs::dom_block_download_insecure() &&
+      decission != nsIContentPolicy::ACCEPT) {
+    nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(aChannel);
+    if (httpChannel) {
+      LogMessageToConsole(httpChannel, "MixedContentBlockedDownload");
+    }
+    return nsITransfer::DOWNLOAD_POTENTIALLY_UNSAFE;
+  }
+
+  if (loadInfo->TriggeringPrincipal()->IsSystemPrincipal()) {
+    return nsITransfer::DOWNLOAD_ACCEPTABLE;
+  }
+
+  if (!StaticPrefs::dom_block_download_in_sandboxed_iframes()) {
+    return nsITransfer::DOWNLOAD_ACCEPTABLE;
+  }
+
+  uint32_t triggeringFlags = loadInfo->GetTriggeringSandboxFlags();
+  uint32_t currentflags = loadInfo->GetSandboxFlags();
+
+  if ((triggeringFlags & SANDBOXED_ALLOW_DOWNLOADS) ||
+      (currentflags & SANDBOXED_ALLOW_DOWNLOADS)) {
+    nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(aChannel);
+    if (httpChannel) {
+      LogMessageToConsole(httpChannel, "IframeSandboxBlockedDownload");
+    }
+    return nsITransfer::DOWNLOAD_FORBIDDEN;
+  }
+
+  return nsITransfer::DOWNLOAD_ACCEPTABLE;
+}
