@@ -55,7 +55,7 @@ already_AddRefed<nsIFile> GetFile() {
     return nullptr;
   }
 
-  file->Append(NS_LITERAL_STRING(SERVICEWORKERREGISTRAR_FILE));
+  file->Append(nsLiteralString(SERVICEWORKERREGISTRAR_FILE));
   return file.forget();
 }
 
@@ -108,7 +108,7 @@ TEST(ServiceWorkerRegistrar, TestNoFile)
 
 TEST(ServiceWorkerRegistrar, TestEmptyFile)
 {
-  ASSERT_TRUE(CreateFile(EmptyCString()))
+  ASSERT_TRUE(CreateFile(""_ns))
   << "CreateFile should not fail";
 
   RefPtr<ServiceWorkerRegistrarTest> swr = new ServiceWorkerRegistrarTest;
@@ -123,8 +123,7 @@ TEST(ServiceWorkerRegistrar, TestEmptyFile)
 
 TEST(ServiceWorkerRegistrar, TestRightVersionFile)
 {
-  ASSERT_TRUE(
-      CreateFile(NS_LITERAL_CSTRING(SERVICEWORKERREGISTRAR_VERSION "\n")))
+  ASSERT_TRUE(CreateFile(nsLiteralCString(SERVICEWORKERREGISTRAR_VERSION "\n")))
   << "CreateFile should not fail";
 
   RefPtr<ServiceWorkerRegistrarTest> swr = new ServiceWorkerRegistrarTest;
@@ -141,7 +140,7 @@ TEST(ServiceWorkerRegistrar, TestRightVersionFile)
 TEST(ServiceWorkerRegistrar, TestWrongVersionFile)
 {
   ASSERT_TRUE(
-      CreateFile(NS_LITERAL_CSTRING(SERVICEWORKERREGISTRAR_VERSION "bla\n")))
+      CreateFile(nsLiteralCString(SERVICEWORKERREGISTRAR_VERSION "bla\n")))
   << "CreateFile should not fail";
 
   RefPtr<ServiceWorkerRegistrarTest> swr = new ServiceWorkerRegistrarTest;
@@ -172,6 +171,9 @@ TEST(ServiceWorkerRegistrar, TestReadData)
   buffer.AppendLiteral("\n");
   buffer.AppendInt(0);
   buffer.AppendLiteral("\n");
+  buffer.AppendInt(0);
+  buffer.AppendLiteral("\n");
+  buffer.AppendLiteral("true\n");
   buffer.Append(SERVICEWORKERREGISTRAR_TERMINATOR "\n");
 
   buffer.AppendLiteral("\n");
@@ -187,6 +189,9 @@ TEST(ServiceWorkerRegistrar, TestReadData)
   buffer.AppendLiteral("\n");
   buffer.AppendInt(ts);
   buffer.AppendLiteral("\n");
+  buffer.AppendInt(1);
+  buffer.AppendLiteral("\n");
+  buffer.AppendLiteral("false\n");
   buffer.Append(SERVICEWORKERREGISTRAR_TERMINATOR "\n");
 
   ASSERT_TRUE(CreateFile(buffer))
@@ -219,6 +224,8 @@ TEST(ServiceWorkerRegistrar, TestReadData)
   ASSERT_EQ((int64_t)0, data[0].currentWorkerInstalledTime());
   ASSERT_EQ((int64_t)0, data[0].currentWorkerActivatedTime());
   ASSERT_EQ((int64_t)0, data[0].lastUpdateTime());
+  ASSERT_EQ(false, data[0].navigationPreloadState().enabled());
+  ASSERT_STREQ("true", data[0].navigationPreloadState().headerValue().get());
 
   const mozilla::ipc::PrincipalInfo& info1 = data[1].principal();
   ASSERT_EQ(info1.type(), mozilla::ipc::PrincipalInfo::TContentPrincipalInfo)
@@ -239,11 +246,13 @@ TEST(ServiceWorkerRegistrar, TestReadData)
   ASSERT_EQ((int64_t)ts, data[1].currentWorkerInstalledTime());
   ASSERT_EQ((int64_t)ts, data[1].currentWorkerActivatedTime());
   ASSERT_EQ((int64_t)ts, data[1].lastUpdateTime());
+  ASSERT_EQ(true, data[1].navigationPreloadState().enabled());
+  ASSERT_STREQ("false", data[1].navigationPreloadState().headerValue().get());
 }
 
 TEST(ServiceWorkerRegistrar, TestDeleteData)
 {
-  ASSERT_TRUE(CreateFile(NS_LITERAL_CSTRING("Foobar")))
+  ASSERT_TRUE(CreateFile("Foobar"_ns))
   << "CreateFile should not fail";
 
   RefPtr<ServiceWorkerRegistrarTest> swr = new ServiceWorkerRegistrarTest;
@@ -876,7 +885,7 @@ TEST(ServiceWorkerRegistrar, TestDedupeWrite)
     for (int i = 0; i < 2; ++i) {
       ServiceWorkerRegistrationData reg;
 
-      reg.scope() = NS_LITERAL_CSTRING("https://scope_write.dedupe");
+      reg.scope() = "https://scope_write.dedupe"_ns;
       reg.currentWorkerURL() = nsPrintfCString("currentWorkerURL write %d", i);
       reg.currentWorkerHandlesFetch() = true;
       reg.cacheName() =
