@@ -33,8 +33,8 @@ using namespace mozilla::dom;
 
 namespace mozilla {
 
-typedef CopyableAutoTArray<RefPtr<RawServoAnimationValue>, 1>
-    ServoAnimationValues;
+using ServoAnimationValues =
+    CopyableAutoTArray<RefPtr<RawServoAnimationValue>, 1>;
 
 /*static*/
 SMILCSSValueType SMILCSSValueType::sSingleton;
@@ -425,7 +425,8 @@ static ServoAnimationValues ValueFromStringHelper(nsCSSPropertyID aPropID,
   ServoCSSParser::ParsingEnvironment env =
       ServoCSSParser::GetParsingEnvironment(doc);
   RefPtr<RawServoDeclarationBlock> servoDeclarationBlock =
-      ServoCSSParser::ParseProperty(aPropID, aString, env,
+      ServoCSSParser::ParseProperty(aPropID, NS_ConvertUTF16toUTF8(aString),
+                                    env,
                                     ParsingMode::AllowUnitlessLength |
                                         ParsingMode::AllowAllNumericValues);
   if (!servoDeclarationBlock) {
@@ -490,8 +491,7 @@ SMILValue SMILCSSValueType::ValueFromAnimationValue(
   // string passed to CSPAllowsInlineStyle is only used for reporting violations
   // and an intermediate CSS value is not likely to be particularly useful
   // in that case, we just use a generic placeholder string instead.
-  static const nsLiteralString kPlaceholderText =
-      NS_LITERAL_STRING("[SVG animation of CSS]");
+  static const nsLiteralString kPlaceholderText = u"[SVG animation of CSS]"_ns;
   if (doc && !nsStyleUtil::CSPAllowsInlineStyle(nullptr, doc, nullptr, 0, 0,
                                                 kPlaceholderText, nullptr)) {
     return result;
