@@ -10,6 +10,7 @@
 #include "mozilla/ClearOnShutdown.h"
 #include "mozilla/Unused.h"
 #include "nsTextFormatter.h"
+#include "nsWindowsHelpers.h"
 #include "winwebauthn/webauthn.h"
 #include "WinWebAuthnManager.h"
 
@@ -23,19 +24,17 @@ static HMODULE gWinWebAuthnModule = 0;
 
 static decltype(WebAuthNIsUserVerifyingPlatformAuthenticatorAvailable)*
     gWinWebauthnIsUVPAA = nullptr;
-static decltype(
-    WebAuthNAuthenticatorMakeCredential)* gWinWebauthnMakeCredential = nullptr;
-static decltype(
-    WebAuthNFreeCredentialAttestation)* gWinWebauthnFreeCredentialAttestation =
-    nullptr;
+static decltype(WebAuthNAuthenticatorMakeCredential)*
+    gWinWebauthnMakeCredential = nullptr;
+static decltype(WebAuthNFreeCredentialAttestation)*
+    gWinWebauthnFreeCredentialAttestation = nullptr;
 static decltype(WebAuthNAuthenticatorGetAssertion)* gWinWebauthnGetAssertion =
     nullptr;
 static decltype(WebAuthNFreeAssertion)* gWinWebauthnFreeAssertion = nullptr;
 static decltype(WebAuthNGetCancellationId)* gWinWebauthnGetCancellationId =
     nullptr;
-static decltype(
-    WebAuthNCancelCurrentOperation)* gWinWebauthnCancelCurrentOperation =
-    nullptr;
+static decltype(WebAuthNCancelCurrentOperation)*
+    gWinWebauthnCancelCurrentOperation = nullptr;
 static decltype(WebAuthNGetErrorName)* gWinWebauthnGetErrorName = nullptr;
 static decltype(WebAuthNGetApiVersionNumber)* gWinWebauthnGetApiVersionNumber =
     nullptr;
@@ -53,11 +52,11 @@ WinWebAuthnManager::WinWebAuthnManager() {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(!gWinWebAuthnModule);
 
-  gWinWebAuthnModule = LoadLibrary(L"webauthn.dll");
+  gWinWebAuthnModule = LoadLibrarySystem32(L"webauthn.dll");
 
   if (gWinWebAuthnModule) {
-    gWinWebauthnIsUVPAA = reinterpret_cast<decltype(
-        WebAuthNIsUserVerifyingPlatformAuthenticatorAvailable)*>(
+    gWinWebauthnIsUVPAA = reinterpret_cast<
+        decltype(WebAuthNIsUserVerifyingPlatformAuthenticatorAvailable)*>(
         GetProcAddress(
             gWinWebAuthnModule,
             "WebAuthNIsUserVerifyingPlatformAuthenticatorAvailable"));
@@ -730,14 +729,18 @@ void WinWebAuthnManager::MaybeAbortSign(const uint64_t& aTransactionId,
 }
 
 void WinWebAuthnManager::Cancel(PWebAuthnTransactionParent* aParent,
-                                const uint64_t& aTransactionId) {
+                                const Tainted<uint64_t>& aTransactionId) {
   if (mTransactionParent != aParent) {
     return;
   }
 
   ClearTransaction();
 
-  auto iter = mCancellationIds.find(aTransactionId);
+  auto iter = mCancellationIds.find(
+      MOZ_NO_VALIDATE(aTransactionId,
+                      "Transaction ID is checked against a global container, "
+                      "so an invalid entry can affect another origin's "
+                      "request. This issue is filed as Bug 1696159."));
   if (iter != mCancellationIds.end()) {
     gWinWebauthnCancelCurrentOperation(iter->second);
   }
