@@ -7,26 +7,20 @@
 #ifndef mozilla_dom_workerscope_h__
 #define mozilla_dom_workerscope_h__
 
+#include "js/TypeDecls.h"
 #include "mozilla/Assertions.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/DOMEventTargetHelper.h"
-#include "mozilla/ErrorResult.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/NotNull.h"
 #include "mozilla/RefPtr.h"
 #include "mozilla/UniquePtr.h"
-#include "mozilla/dom/ClientSource.h"
-#include "mozilla/dom/Console.h"
-#include "mozilla/dom/DOMString.h"
-#include "mozilla/dom/EventCallbackDebuggerNotification.h"
 #include "mozilla/dom/ImageBitmapSource.h"
-#include "mozilla/dom/RequestBinding.h"
-#include "mozilla/dom/RequestBinding.h"
+#include "mozilla/dom/SafeRefPtr.h"
 #include "nsCOMPtr.h"
-#include "nsContentUtils.h"
 #include "nsCycleCollectionParticipant.h"
 #include "nsIGlobalObject.h"
-#include "nsISupportsImpl.h"
+#include "nsISupports.h"
 #include "nsWeakReference.h"
 
 #ifdef XP_WIN
@@ -37,30 +31,45 @@ class nsAtom;
 class nsISerialEventTarget;
 
 namespace mozilla {
+class ErrorResult;
+
+namespace extensions {
+
+class ExtensionBrowser;
+
+}  // namespace extensions
+
 namespace dom {
 
 class AnyCallback;
 enum class CallerType : uint32_t;
 class ClientInfo;
+class ClientSource;
 class Clients;
 class Console;
 class Crypto;
 class DOMString;
 class DebuggerNotificationManager;
+enum class EventCallbackDebuggerNotificationType : uint8_t;
+class EventHandlerNonNull;
 class Function;
 class IDBFactory;
 class OnErrorEventHandlerNonNull;
 template <typename T>
 class Optional;
+class Performance;
 struct PostMessageOptions;
 class Promise;
+class RequestOrUSVString;
 template <typename T>
 class Sequence;
 class ServiceWorkerDescriptor;
+class ServiceWorkerRegistration;
 class ServiceWorkerRegistrationDescriptor;
 class WorkerLocation;
 class WorkerNavigator;
 class WorkerPrivate;
+struct RequestInit;
 
 namespace cache {
 
@@ -96,13 +105,9 @@ class WorkerGlobalScopeBase : public DOMEventTargetHelper,
 
   bool IsSharedMemoryAllowed() const final;
 
-  Maybe<ClientInfo> GetClientInfo() const final {
-    return Some(mClientSource->Info());
-  }
+  Maybe<ClientInfo> GetClientInfo() const final;
 
-  Maybe<ServiceWorkerDescriptor> GetController() const final {
-    return mClientSource->GetController();
-  }
+  Maybe<ServiceWorkerDescriptor> GetController() const final;
 
   virtual void Control(const ServiceWorkerDescriptor& aServiceWorker);
 
@@ -138,7 +143,7 @@ class WorkerGlobalScopeBase : public DOMEventTargetHelper,
   void WorkerPrivateSaysAllowScript() { StopForbiddingScript(); }
 
  protected:
-  ~WorkerGlobalScopeBase() = default;
+  ~WorkerGlobalScopeBase();
 
   const NotNull<WorkerPrivate*> mWorkerPrivate;
 
@@ -154,7 +159,7 @@ class NamedWorkerGlobalScopeMixin {
  public:
   explicit NamedWorkerGlobalScopeMixin(const nsAString& aName) : mName(aName) {}
 
-  void GetName(DOMString& aName) const { aName.AsAString() = mName; }
+  void GetName(DOMString& aName) const;
 
  protected:
   ~NamedWorkerGlobalScopeMixin() = default;
@@ -186,9 +191,7 @@ class WorkerGlobalScope : public WorkerGlobalScopeBase,
   DebuggerNotificationManager* GetExistingDebuggerNotificationManager() final;
 
   Maybe<EventCallbackDebuggerNotificationType> GetDebuggerNotificationType()
-      const final {
-    return Some(EventCallbackDebuggerNotificationType::Global);
-  }
+      const final;
 
   // WorkerGlobalScope WebIDL implementation
   WorkerGlobalScope* Self() { return this; }
@@ -279,10 +282,10 @@ class WorkerGlobalScope : public WorkerGlobalScopeBase,
 
   void ConsumeWindowInteraction();
 
-  void FirstPartyStorageAccessGranted();
+  void StorageAccessPermissionGranted();
 
  protected:
-  ~WorkerGlobalScope() = default;
+  ~WorkerGlobalScope();
 
  private:
   MOZ_CAN_RUN_SCRIPT
@@ -371,6 +374,8 @@ class ServiceWorkerGlobalScope final : public WorkerGlobalScope {
 
   already_AddRefed<Promise> SkipWaiting(ErrorResult& aRv);
 
+  SafeRefPtr<extensions::ExtensionBrowser> AcquireExtensionBrowser();
+
   IMPL_EVENT_HANDLER(install)
   IMPL_EVENT_HANDLER(activate)
 
@@ -390,13 +395,14 @@ class ServiceWorkerGlobalScope final : public WorkerGlobalScope {
   IMPL_EVENT_HANDLER(pushsubscriptionchange)
 
  private:
-  ~ServiceWorkerGlobalScope() = default;
+  ~ServiceWorkerGlobalScope();
 
   void NoteFetchHandlerWasAdded() const;
 
   RefPtr<Clients> mClients;
   const nsString mScope;
   RefPtr<ServiceWorkerRegistration> mRegistration;
+  SafeRefPtr<extensions::ExtensionBrowser> mExtensionBrowser;
 };
 
 class WorkerDebuggerGlobalScope final : public WorkerGlobalScopeBase {
