@@ -6,19 +6,27 @@
 
 #include "WorkerThread.h"
 
-#include "EventQueue.h"
+#include <utility>
 #include "WorkerPrivate.h"
 #include "WorkerRunnable.h"
-#include "mozilla/AbstractThread.h"
 #include "mozilla/Assertions.h"
+#include "mozilla/Atomics.h"
+#include "mozilla/CycleCollectedJSContext.h"
+#include "mozilla/EventQueue.h"
+#include "mozilla/MacroForEach.h"
+#include "mozilla/NotNull.h"
 #include "mozilla/PerformanceCounter.h"
 #include "mozilla/ThreadEventQueue.h"
+#include "mozilla/UniquePtr.h"
 #include "mozilla/ipc/BackgroundChild.h"
+#include "nsCOMPtr.h"
+#include "nsDebug.h"
+#include "nsICancelableRunnable.h"
+#include "nsIEventTarget.h"
+#include "nsIRunnable.h"
 #include "nsIThreadInternal.h"
-
-#ifdef DEBUG
-#  include "nsThreadManager.h"
-#endif
+#include "nsString.h"
+#include "prthread.h"
 
 namespace mozilla {
 
@@ -65,10 +73,10 @@ class WorkerThread::Observer final : public nsIThreadObserver {
   NS_DECL_NSITHREADOBSERVER
 };
 
-WorkerThread::WorkerThread()
-    : nsThread(MakeNotNull<ThreadEventQueue<mozilla::EventQueue>*>(
-                   MakeUnique<mozilla::EventQueue>()),
-               nsThread::NOT_MAIN_THREAD, kWorkerStackSize),
+WorkerThread::WorkerThread(ConstructorKey)
+    : nsThread(
+          MakeNotNull<ThreadEventQueue*>(MakeUnique<mozilla::EventQueue>()),
+          nsThread::NOT_MAIN_THREAD, kWorkerStackSize),
       mLock("WorkerThread::mLock"),
       mWorkerPrivateCondVar(mLock, "WorkerThread::mWorkerPrivateCondVar"),
       mWorkerPrivate(nullptr),
@@ -87,17 +95,16 @@ WorkerThread::~WorkerThread() {
 }
 
 // static
-already_AddRefed<WorkerThread> WorkerThread::Create(
+SafeRefPtr<WorkerThread> WorkerThread::Create(
     const WorkerThreadFriendKey& /* aKey */) {
-  RefPtr<WorkerThread> thread = new WorkerThread();
-  if (NS_FAILED(thread->Init(NS_LITERAL_CSTRING("DOM Worker")))) {
+  SafeRefPtr<WorkerThread> thread =
+      MakeSafeRefPtr<WorkerThread>(ConstructorKey());
+  if (NS_FAILED(thread->Init("DOM Worker"_ns))) {
     NS_WARNING("Failed to create new thread!");
     return nullptr;
   }
-  thread->mAbstractThread = AbstractThread::CreateXPCOMThreadWrapper(
-      thread, false /* aRequireTailDispatch */);
 
-  return thread.forget();
+  return thread;
 }
 
 void WorkerThread::SetWorker(const WorkerThreadFriendKey& /* aKey */,
@@ -149,11 +156,8 @@ void WorkerThread::SetWorker(const WorkerThreadFriendKey& /* aKey */,
 void WorkerThread::IncrementDispatchCounter() {
   MutexAutoLock lock(mLock);
   if (mWorkerPrivate) {
-    PerformanceCounter* performanceCounter =
-        mWorkerPrivate->GetPerformanceCounter();
-    if (performanceCounter) {
-      performanceCounter->IncrementDispatchCounter(DispatchCategory::Worker);
-    }
+    mWorkerPrivate->MutablePerformanceCounterRef().IncrementDispatchCounter(
+        DispatchCategory::Worker);
   }
 }
 
@@ -240,15 +244,16 @@ WorkerThread::Dispatch(already_AddRefed<nsIRunnable> aRunnable,
 
 #ifdef DEBUG
   if (runnable && !onWorkerThread) {
-    nsCOMPtr<nsICancelableRunnable> cancelable = do_QueryInterface(runnable);
+    nsCOMPtr<nsIDiscardableRunnable> discardable = do_QueryInterface(runnable);
 
     {
       MutexAutoLock lock(mLock);
 
-      // Only enforce cancelable runnables after we've started the worker loop.
+      // Only enforce discardable runnables after we've started the worker loop.
       if (!mAcceptingNonWorkerRunnables) {
-        MOZ_ASSERT(cancelable,
-                   "Only nsICancelableRunnable may be dispatched to a worker!");
+        MOZ_ASSERT(
+            discardable,
+            "Only nsIDiscardableRunnable may be dispatched to a worker!");
       }
     }
   }
@@ -327,20 +332,9 @@ uint32_t WorkerThread::RecursionDepth(
   return mNestedEventLoopDepth;
 }
 
-PerformanceCounter* WorkerThread::GetPerformanceCounter(
-    nsIRunnable* aEvent) const {
-  if (mWorkerPrivate) {
-    return mWorkerPrivate->GetPerformanceCounter();
-  }
-  return nullptr;
-}
-
-NS_IMETHODIMP
-WorkerThread::Shutdown() {
-  MOZ_ALWAYS_SUCCEEDS(nsThread::Shutdown());
-  // We need to break the cycle.
-  mAbstractThread = nullptr;
-  return NS_OK;
+PerformanceCounter* WorkerThread::GetPerformanceCounter(nsIRunnable*) const {
+  return mWorkerPrivate ? &mWorkerPrivate->MutablePerformanceCounterRef()
+                        : nullptr;
 }
 
 NS_IMPL_ISUPPORTS(WorkerThread::Observer, nsIThreadObserver)
