@@ -21,6 +21,7 @@
 #include "nsXULAppAPI.h"
 
 #include "RemoteWorkerService.h"
+#include "mozilla/ArrayAlgorithm.h"
 #include "mozilla/Assertions.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/BasePrincipal.h"
@@ -32,6 +33,7 @@
 #include "mozilla/dom/FetchEventOpProxyChild.h"
 #include "mozilla/dom/IndexedDatabaseManager.h"
 #include "mozilla/dom/MessagePort.h"
+#include "mozilla/dom/RemoteWorkerManager.h"  // RemoteWorkerManager::IsRemoteTypeAllowed
 #include "mozilla/dom/RemoteWorkerTypes.h"
 #include "mozilla/dom/ServiceWorkerDescriptor.h"
 #include "mozilla/dom/ServiceWorkerInterceptController.h"
@@ -104,7 +106,7 @@ class SharedWorkerInterfaceRequestor final : public nsIInterfaceRequestor {
 
   SharedWorkerInterfaceRequestor() {
     // This check must match the code nsDocShell::Create.
-    if (!ServiceWorkerParentInterceptEnabled() || XRE_IsParentProcess()) {
+    if (XRE_IsParentProcess()) {
       mSWController = new ServiceWorkerInterceptController();
     }
   }
@@ -251,7 +253,7 @@ RemoteWorkerChild::RemoteWorkerChild(const RemoteWorkerData& aData)
     : mState(VariantType<Pending>(), "RemoteWorkerChild::mState"),
       mIsServiceWorker(aData.serviceWorkerData().type() ==
                        OptionalServiceWorkerData::TServiceWorkerData),
-      mOwningEventTarget(GetCurrentThreadSerialEventTarget()) {
+      mOwningEventTarget(GetCurrentSerialEventTarget()) {
   MOZ_ASSERT(RemoteWorkerService::Thread()->IsOnCurrentThread());
   MOZ_ASSERT(mOwningEventTarget);
 }
@@ -268,7 +270,7 @@ nsISerialEventTarget* RemoteWorkerChild::GetOwningEventTarget() const {
 }
 
 void RemoteWorkerChild::ActorDestroy(ActorDestroyReason) {
-  MOZ_ACCESS_THREAD_BOUND(mLauncherData, launcherData);
+  auto launcherData = mLauncherData.Access();
   launcherData->mIPCActive = false;
 
   Unused << NS_WARN_IF(!launcherData->mTerminationPromise.IsEmpty());
@@ -285,7 +287,7 @@ void RemoteWorkerChild::ActorDestroy(ActorDestroyReason) {
 void RemoteWorkerChild::ExecWorker(const RemoteWorkerData& aData) {
 #ifdef DEBUG
   MOZ_ASSERT(GetOwningEventTarget()->IsOnCurrentThread());
-  MOZ_ACCESS_THREAD_BOUND(mLauncherData, launcherData);
+  auto launcherData = mLauncherData.Access();
   MOZ_ASSERT(launcherData->mIPCActive);
 #endif
 
@@ -312,6 +314,14 @@ nsresult RemoteWorkerChild::ExecWorkerOnMainThread(RemoteWorkerData&& aData) {
 
   auto scopeExit = MakeScopeExit([&] { TransitionStateToTerminated(); });
 
+  // Verify the the RemoteWorker should be really allowed to run in this
+  // process, and fail if it shouldn't (This shouldn't normally happen,
+  // unless the RemoteWorkerData has been tempered in the process it was
+  // sent from).
+  if (!RemoteWorkerManager::IsRemoteTypeAllowed(aData)) {
+    return NS_ERROR_UNEXPECTED;
+  }
+
   auto principalOrErr = PrincipalInfoToPrincipal(aData.principalInfo());
   if (NS_WARN_IF(principalOrErr.isErr())) {
     return principalOrErr.unwrapErr();
@@ -325,10 +335,10 @@ nsresult RemoteWorkerChild::ExecWorkerOnMainThread(RemoteWorkerData&& aData) {
     return loadingPrincipalOrErr.unwrapErr();
   }
 
-  auto storagePrincipalOrErr =
-      PrincipalInfoToPrincipal(aData.storagePrincipalInfo());
-  if (NS_WARN_IF(storagePrincipalOrErr.isErr())) {
-    return storagePrincipalOrErr.unwrapErr();
+  auto partitionedPrincipalOrErr =
+      PrincipalInfoToPrincipal(aData.partitionedPrincipalInfo());
+  if (NS_WARN_IF(partitionedPrincipalOrErr.isErr())) {
+    return partitionedPrincipalOrErr.unwrapErr();
   }
 
   WorkerLoadInfo info;
@@ -336,18 +346,22 @@ nsresult RemoteWorkerChild::ExecWorkerOnMainThread(RemoteWorkerData&& aData) {
   info.mResolvedScriptURI = DeserializeURI(aData.resolvedScriptURL());
 
   info.mPrincipalInfo = MakeUnique<PrincipalInfo>(aData.principalInfo());
-  info.mStoragePrincipalInfo =
-      MakeUnique<PrincipalInfo>(aData.storagePrincipalInfo());
+  info.mPartitionedPrincipalInfo =
+      MakeUnique<PrincipalInfo>(aData.partitionedPrincipalInfo());
 
   info.mReferrerInfo = aData.referrerInfo();
   info.mDomain = aData.domain();
   info.mPrincipal = principal;
-  info.mStoragePrincipal = storagePrincipalOrErr.unwrap();
+  info.mPartitionedPrincipal = partitionedPrincipalOrErr.unwrap();
   info.mLoadingPrincipal = loadingPrincipalOrErr.unwrap();
   info.mStorageAccess = aData.storageAccess();
+  info.mUseRegularPrincipal = aData.useRegularPrincipal();
+  info.mHasStorageAccessPermissionGranted =
+      aData.hasStorageAccessPermissionGranted();
   info.mOriginAttributes =
       BasePrincipal::Cast(principal)->OriginAttributesRef();
-  info.mCookieJarSettings = net::CookieJarSettings::Create();
+  net::CookieJarSettings::Deserialize(aData.cookieJarSettings(),
+                                      getter_AddRefs(info.mCookieJarSettings));
 
   // Default CSP permissions for now.  These will be overrided if necessary
   // based on the script CSP headers during load in ScriptLoader.
@@ -383,12 +397,12 @@ nsresult RemoteWorkerChild::ExecWorkerOnMainThread(RemoteWorkerData&& aData) {
   }
 
   rv = info.SetPrincipalsAndCSPOnMainThread(
-      info.mPrincipal, info.mStoragePrincipal, info.mLoadGroup, info.mCSP);
+      info.mPrincipal, info.mPartitionedPrincipal, info.mLoadGroup, info.mCSP);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
 
-  nsString workerPrivateId = EmptyString();
+  nsString workerPrivateId;
 
   if (mIsServiceWorker) {
     ServiceWorkerData& data = aData.serviceWorkerData().get_ServiceWorkerData();
@@ -422,7 +436,7 @@ nsresult RemoteWorkerChild::ExecWorkerOnMainThread(RemoteWorkerData&& aData) {
   ErrorResult error;
   RefPtr<WorkerPrivate> workerPrivate = WorkerPrivate::Constructor(
       jsapi.cx(), aData.originalScriptURL(), false,
-      mIsServiceWorker ? WorkerTypeService : WorkerTypeShared, aData.name(),
+      mIsServiceWorker ? WorkerKindService : WorkerKindShared, aData.name(),
       VoidCString(), &info, error, std::move(workerPrivateId));
 
   if (NS_WARN_IF(error.Failed())) {
@@ -553,7 +567,7 @@ void RemoteWorkerChild::ShutdownOnWorker() {
 
   nsCOMPtr<nsIRunnable> r =
       NS_NewRunnableFunction(__func__, [self = std::move(self)] {
-        MOZ_ACCESS_THREAD_BOUND(self->mLauncherData, launcherData);
+        auto launcherData = self->mLauncherData.Access();
 
         if (!launcherData->mIPCActive) {
           return;
@@ -567,7 +581,7 @@ void RemoteWorkerChild::ShutdownOnWorker() {
 }
 
 RefPtr<GenericNonExclusivePromise> RemoteWorkerChild::GetTerminationPromise() {
-  MOZ_ACCESS_THREAD_BOUND(mLauncherData, launcherData);
+  auto launcherData = mLauncherData.Access();
   return launcherData->mTerminationPromise.Ensure(__func__);
 }
 
@@ -592,7 +606,7 @@ void RemoteWorkerChild::CreationSucceededOrFailedOnAnyThread(
   nsCOMPtr<nsIRunnable> r = NS_NewRunnableFunction(
       __func__,
       [self = std::move(self), didCreationSucceed = aDidCreationSucceed] {
-        MOZ_ACCESS_THREAD_BOUND(self->mLauncherData, launcherData);
+        auto launcherData = self->mLauncherData.Access();
 
         if (!launcherData->mIPCActive) {
           return;
@@ -628,7 +642,7 @@ void RemoteWorkerChild::CloseWorkerOnMainThread(State& aState) {
 void RemoteWorkerChild::ErrorPropagation(const ErrorValue& aValue) {
   MOZ_ASSERT(GetOwningEventTarget()->IsOnCurrentThread());
 
-  MOZ_ACCESS_THREAD_BOUND(mLauncherData, launcherData);
+  auto launcherData = mLauncherData.Access();
 
   if (!launcherData->mIPCActive) {
     return;
@@ -654,16 +668,13 @@ void RemoteWorkerChild::ErrorPropagationOnMainThread(
 
   ErrorValue value;
   if (aIsErrorEvent) {
-    nsTArray<ErrorDataNote> notes;
-    for (size_t i = 0, len = aReport->mNotes.Length(); i < len; i++) {
-      const WorkerErrorNote& note = aReport->mNotes.ElementAt(i);
-      notes.AppendElement(ErrorDataNote(note.mLineNumber, note.mColumnNumber,
-                                        note.mMessage, note.mFilename));
-    }
-
-    ErrorData data(aReport->mIsWarning, aReport->mLineNumber,
-                   aReport->mColumnNumber, aReport->mMessage,
-                   aReport->mFilename, aReport->mLine, notes);
+    ErrorData data(
+        aReport->mIsWarning, aReport->mLineNumber, aReport->mColumnNumber,
+        aReport->mMessage, aReport->mFilename, aReport->mLine,
+        TransformIntoNewArray(aReport->mNotes, [](const WorkerErrorNote& note) {
+          return ErrorDataNote(note.mLineNumber, note.mColumnNumber,
+                               note.mMessage, note.mFilename);
+        }));
     value = data;
   } else {
     value = void_t();
@@ -801,7 +812,7 @@ void RemoteWorkerChild::TransitionStateToTerminated(State& aState) {
 
   nsCOMPtr<nsIRunnable> r =
       NS_NewRunnableFunction(__func__, [self = SelfHolder(this)]() {
-        MOZ_ACCESS_THREAD_BOUND(self->mLauncherData, launcherData);
+        auto launcherData = self->mLauncherData.Access();
         launcherData->mTerminationPromise.ResolveIfExists(true, __func__);
       });
 
@@ -828,7 +839,7 @@ class RemoteWorkerChild::SharedWorkerOp : public RemoteWorkerChild::Op {
     MOZ_ASSERT(!mStarted);
     MOZ_ASSERT(aOwner);
 
-    MOZ_ACCESS_THREAD_BOUND(aOwner->mLauncherData, launcherData);
+    auto launcherData = aOwner->mLauncherData.Access();
 
     if (NS_WARN_IF(!launcherData->mIPCActive)) {
       Unused << NS_WARN_IF(!aState.is<Terminated>());
@@ -1007,7 +1018,7 @@ RemoteWorkerChild::MaybeSendSetServiceWorkerSkipWaitingFlag() {
   nsCOMPtr<nsIRunnable> r = NS_NewRunnableFunction(__func__, [self = std::move(
                                                                   self),
                                                               promise] {
-    MOZ_ACCESS_THREAD_BOUND(self->mLauncherData, launcherData);
+    auto launcherData = self->mLauncherData.Access();
 
     if (!launcherData->mIPCActive) {
       promise->Reject(NS_ERROR_DOM_ABORT_ERR, __func__);
@@ -1015,7 +1026,7 @@ RemoteWorkerChild::MaybeSendSetServiceWorkerSkipWaitingFlag() {
     }
 
     self->SendSetServiceWorkerSkipWaitingFlag()->Then(
-        GetCurrentThreadSerialEventTarget(), __func__,
+        GetCurrentSerialEventTarget(), __func__,
         [promise](
             const SetServiceWorkerSkipWaitingFlagPromise::ResolveOrRejectValue&
                 aResult) {
