@@ -49,6 +49,8 @@
 
 static mozilla::LazyLogModule gContentSinkLog("nsXULContentSink");
 
+using namespace mozilla;
+using namespace mozilla::dom;
 //----------------------------------------------------------------------
 
 XULContentSinkImpl::ContextStack::ContextStack() : mTop(nullptr), mDepth(0) {}
@@ -569,8 +571,8 @@ XULContentSinkImpl::ReportError(const char16_t* aErrorText,
 
   const char16_t* noAtts[] = {0, 0};
 
-  NS_NAMED_LITERAL_STRING(
-      errorNs, "http://www.mozilla.org/newlayout/xml/parsererror.xml");
+  constexpr auto errorNs =
+      u"http://www.mozilla.org/newlayout/xml/parsererror.xml"_ns;
 
   nsAutoString parsererror(errorNs);
   parsererror.Append((char16_t)0xFFFF);
@@ -699,6 +701,9 @@ nsresult XULContentSinkImpl::OpenScript(const char16_t** aAttributes,
         NS_ENSURE_SUCCESS(rv, rv);
       }
 
+      // NOTE(emilio): Module scripts don't pass this test, aren't cached yet.
+      // If they become cached, then we need to tweak
+      // PrototypeDocumentContentSink and remove the special cases there.
       if (nsContentUtils::IsJavascriptMIMEType(mimeType)) {
         isJavaScript = true;
 
@@ -708,11 +713,10 @@ nsresult XULContentSinkImpl::OpenScript(const char16_t** aAttributes,
 
         if (NS_SUCCEEDED(rv)) {
           nsContentUtils::ReportToConsoleNonLocalized(
-              NS_LITERAL_STRING(
-                  "Versioned JavaScripts are no longer supported. "
-                  "Please remove the version parameter."),
-              nsIScriptError::errorFlag, NS_LITERAL_CSTRING("XUL Document"),
-              nullptr, mDocumentURL, EmptyString(), aLineNumber);
+              u"Versioned JavaScripts are no longer supported. "
+              "Please remove the version parameter."_ns,
+              nsIScriptError::errorFlag, "XUL Document"_ns, nullptr,
+              mDocumentURL, u""_ns, aLineNumber);
           isJavaScript = false;
         } else if (rv != NS_ERROR_INVALID_ARG) {
           return rv;
