@@ -166,111 +166,116 @@ static void ProcessMarginRightValue(const nsAString* aInputString,
   }
 }
 
+#define CSS_EQUIV_TABLE_NONE \
+  { CSSEditUtils::eCSSEditableProperty_NONE, 0 }
+
 const CSSEditUtils::CSSEquivTable boldEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_font_weight, ProcessBValue, nullptr,
-     nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_font_weight, true, false, ProcessBValue,
+     nullptr, nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable italicEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_font_style, ProcessDefaultValue,
-     "italic", nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_font_style, true, false,
+     ProcessDefaultValue, "italic", nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable underlineEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_text_decoration, ProcessDefaultValue,
-     "underline", nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_text_decoration, true, false,
+     ProcessDefaultValue, "underline", nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable strikeEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_text_decoration, ProcessDefaultValue,
-     "line-through", nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_text_decoration, true, false,
+     ProcessDefaultValue, "line-through", nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable ttEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_font_family, ProcessDefaultValue,
-     "monospace", nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_font_family, true, false,
+     ProcessDefaultValue, "monospace", nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable fontColorEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_color, ProcessSameValue, nullptr,
-     nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_color, true, false, ProcessSameValue,
+     nullptr, nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable fontFaceEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_font_family, ProcessSameValue, nullptr,
-     nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_font_family, true, false,
+     ProcessSameValue, nullptr, nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable bgcolorEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_background_color, ProcessSameValue,
-     nullptr, nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_background_color, true, false,
+     ProcessSameValue, nullptr, nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable backgroundImageEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_background_image, ProcessExtendedValue,
-     nullptr, "url(", ")", true, true},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_background_image, true, true,
+     ProcessExtendedValue, nullptr, "url(", ")"},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable textColorEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_color, ProcessSameValue, nullptr,
-     nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_color, true, false, ProcessSameValue,
+     nullptr, nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable borderEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_border, ProcessExtendedValue, nullptr,
-     nullptr, "px solid", true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_border, true, false,
+     ProcessExtendedValue, nullptr, nullptr, "px solid"},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable textAlignEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_text_align, ProcessSameValue, nullptr,
-     nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_text_align, true, false,
+     ProcessSameValue, nullptr, nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable captionAlignEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_caption_side, ProcessSameValue, nullptr,
-     nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_caption_side, true, false,
+     ProcessSameValue, nullptr, nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable verticalAlignEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_vertical_align, ProcessSameValue,
-     nullptr, nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_vertical_align, true, false,
+     ProcessSameValue, nullptr, nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable nowrapEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_whitespace, ProcessDefaultValue,
-     "nowrap", nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_whitespace, true, false,
+     ProcessDefaultValue, "nowrap", nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable widthEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_width, ProcessLengthValue, nullptr,
-     nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_width, true, false, ProcessLengthValue,
+     nullptr, nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable heightEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_height, ProcessLengthValue, nullptr,
-     nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_height, true, false, ProcessLengthValue,
+     nullptr, nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable listStyleTypeEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_list_style_type,
-     ProcessListStyleTypeValue, nullptr, nullptr, nullptr, true, true},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_list_style_type, true, true,
+     ProcessListStyleTypeValue, nullptr, nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable tableAlignEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_text_align, ProcessDefaultValue, "left",
-     nullptr, nullptr, false, false},
-    {CSSEditUtils::eCSSEditableProperty_margin_left, ProcessMarginLeftValue,
-     nullptr, nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_margin_right, ProcessMarginRightValue,
-     nullptr, nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_text_align, false, false,
+     ProcessDefaultValue, "left", nullptr, nullptr},
+    {CSSEditUtils::eCSSEditableProperty_margin_left, true, false,
+     ProcessMarginLeftValue, nullptr, nullptr, nullptr},
+    {CSSEditUtils::eCSSEditableProperty_margin_right, true, false,
+     ProcessMarginRightValue, nullptr, nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
 
 const CSSEditUtils::CSSEquivTable hrAlignEquivTable[] = {
-    {CSSEditUtils::eCSSEditableProperty_margin_left, ProcessMarginLeftValue,
-     nullptr, nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_margin_right, ProcessMarginRightValue,
-     nullptr, nullptr, nullptr, true, false},
-    {CSSEditUtils::eCSSEditableProperty_NONE, 0}};
+    {CSSEditUtils::eCSSEditableProperty_margin_left, true, false,
+     ProcessMarginLeftValue, nullptr, nullptr, nullptr},
+    {CSSEditUtils::eCSSEditableProperty_margin_right, true, false,
+     ProcessMarginRightValue, nullptr, nullptr, nullptr},
+    CSS_EQUIV_TABLE_NONE};
+
+#undef CSS_EQUIV_TABLE_NONE
 
 CSSEditUtils::CSSEditUtils(HTMLEditor* aHTMLEditor)
     : mHTMLEditor(aHTMLEditor), mIsCSSPrefChecked(true) {
@@ -379,53 +384,13 @@ bool CSSEditUtils::IsCSSEditableProperty(nsINode* aNode, nsAtom* aProperty,
 }
 
 // The lowest level above the transaction; adds the CSS declaration
-// "aProperty : aValue" to the inline styles carried by aElement
-nsresult CSSEditUtils::SetCSSProperty(Element& aElement, nsAtom& aProperty,
-                                      const nsAString& aValue,
-                                      bool aSuppressTxn) {
-  nsCOMPtr<nsStyledElement> styledElement = do_QueryInterface(&aElement);
-  if (NS_WARN_IF(!styledElement)) {
-    return NS_ERROR_INVALID_ARG;
-  }
+// "aProperty : aValue" to the inline styles carried by aStyledElement
+nsresult CSSEditUtils::SetCSSPropertyInternal(nsStyledElement& aStyledElement,
+                                              nsAtom& aProperty,
+                                              const nsAString& aValue,
+                                              bool aSuppressTxn) {
   RefPtr<ChangeStyleTransaction> transaction =
-      ChangeStyleTransaction::Create(*styledElement, aProperty, aValue);
-  if (aSuppressTxn) {
-    return transaction->DoTransaction();
-  }
-  if (NS_WARN_IF(!mHTMLEditor)) {
-    return NS_ERROR_NOT_AVAILABLE;
-  }
-  RefPtr<HTMLEditor> htmlEditor(mHTMLEditor);
-  nsresult rv = htmlEditor->DoTransactionInternal(transaction);
-  NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
-                       "EditorBase::DoTransactionInternal() failed");
-  return rv;
-}
-
-nsresult CSSEditUtils::SetCSSPropertyPixels(Element& aElement,
-                                            nsAtom& aProperty,
-                                            int32_t aIntValue) {
-  nsAutoString s;
-  s.AppendInt(aIntValue);
-  nsresult rv = SetCSSProperty(aElement, aProperty, s + NS_LITERAL_STRING("px"),
-                               /* suppress txn */ false);
-  NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
-                       "CSSEditUtils::SetCSSProperty() failed");
-  return rv;
-}
-
-// The lowest level above the transaction; removes the value aValue from the
-// list of values specified for the CSS property aProperty, or totally remove
-// the declaration if this property accepts only one value
-nsresult CSSEditUtils::RemoveCSSProperty(Element& aElement, nsAtom& aProperty,
-                                         const nsAString& aValue,
-                                         bool aSuppressTxn) {
-  nsCOMPtr<nsStyledElement> styledElement = do_QueryInterface(&aElement);
-  if (NS_WARN_IF(!styledElement)) {
-    return NS_ERROR_INVALID_ARG;
-  }
-  RefPtr<ChangeStyleTransaction> transaction =
-      ChangeStyleTransaction::CreateToRemove(*styledElement, aProperty, aValue);
+      ChangeStyleTransaction::Create(aStyledElement, aProperty, aValue);
   if (aSuppressTxn) {
     nsresult rv = transaction->DoTransaction();
     NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
@@ -437,6 +402,69 @@ nsresult CSSEditUtils::RemoveCSSProperty(Element& aElement, nsAtom& aProperty,
   }
   RefPtr<HTMLEditor> htmlEditor(mHTMLEditor);
   nsresult rv = htmlEditor->DoTransactionInternal(transaction);
+  if (NS_WARN_IF(htmlEditor->Destroyed())) {
+    return NS_ERROR_EDITOR_DESTROYED;
+  }
+  NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
+                       "EditorBase::DoTransactionInternal() failed");
+  return rv;
+}
+
+nsresult CSSEditUtils::SetCSSPropertyPixelsWithTransaction(
+    nsStyledElement& aStyledElement, nsAtom& aProperty, int32_t aIntValue) {
+  nsAutoString s;
+  s.AppendInt(aIntValue);
+  nsresult rv =
+      SetCSSPropertyWithTransaction(aStyledElement, aProperty, s + u"px"_ns);
+  NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
+                       "CSSEditUtils::SetCSSPropertyWithTransaction() failed");
+  return rv;
+}
+
+nsresult CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(
+    nsStyledElement& aStyledElement, const nsAtom& aProperty,
+    int32_t aIntValue) {
+  nsCOMPtr<nsICSSDeclaration> cssDecl = aStyledElement.Style();
+
+  nsAutoCString propertyNameString;
+  aProperty.ToUTF8String(propertyNameString);
+
+  nsAutoCString s;
+  s.AppendInt(aIntValue);
+  s.AppendLiteral("px");
+
+  ErrorResult error;
+  cssDecl->SetProperty(propertyNameString, s, EmptyCString(), error);
+  if (error.Failed()) {
+    NS_WARNING("nsICSSDeclaration::SetProperty() failed");
+    return error.StealNSResult();
+  }
+
+  return NS_OK;
+}
+
+// The lowest level above the transaction; removes the value aValue from the
+// list of values specified for the CSS property aProperty, or totally remove
+// the declaration if this property accepts only one value
+nsresult CSSEditUtils::RemoveCSSPropertyInternal(
+    nsStyledElement& aStyledElement, nsAtom& aProperty, const nsAString& aValue,
+    bool aSuppressTxn) {
+  RefPtr<ChangeStyleTransaction> transaction =
+      ChangeStyleTransaction::CreateToRemove(aStyledElement, aProperty, aValue);
+  if (aSuppressTxn) {
+    nsresult rv = transaction->DoTransaction();
+    NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
+                         "ChangeStyleTransaction::DoTransaction() failed");
+    return rv;
+  }
+  if (NS_WARN_IF(!mHTMLEditor)) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+  RefPtr<HTMLEditor> htmlEditor(mHTMLEditor);
+  nsresult rv = htmlEditor->DoTransactionInternal(transaction);
+  if (NS_WARN_IF(htmlEditor->Destroyed())) {
+    return NS_ERROR_EDITOR_DESTROYED;
+  }
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
                        "EditorBase::DoTransactionInternal() failed");
   return rv;
@@ -487,9 +515,13 @@ nsresult CSSEditUtils::GetComputedCSSInlinePropertyBase(nsIContent& aContent,
   //
   // FIXME(bug 1606994): nsAtomCString copies, we should just keep around the
   // property id.
+  //
+  // FIXME: Maybe we can avoid copying aValue too, though it's no worse than
+  // what we used to do.
+  nsAutoCString value;
   MOZ_ALWAYS_SUCCEEDS(
-      computedDOMStyle->GetPropertyValue(nsAtomCString(&aCSSProperty), aValue));
-
+      computedDOMStyle->GetPropertyValue(nsAtomCString(&aCSSProperty), value));
+  CopyUTF8toUTF16(value, aValue);
   return NS_OK;
 }
 
@@ -509,12 +541,14 @@ nsresult CSSEditUtils::GetSpecifiedCSSInlinePropertyBase(nsIContent& aContent,
     return NS_OK;
   }
 
+  // FIXME: Same comments as above.
   nsCSSPropertyID prop =
       nsCSSProps::LookupProperty(nsAtomCString(&aCSSProperty));
   MOZ_ASSERT(prop != eCSSProperty_UNKNOWN);
 
-  decl->GetPropertyValueByID(prop, aValue);
-
+  nsAutoCString value;
+  decl->GetPropertyValueByID(prop, value);
+  CopyUTF8toUTF16(value, aValue);
   return NS_OK;
 }
 
@@ -529,30 +563,33 @@ already_AddRefed<nsComputedDOMStyle> CSSEditUtils::GetComputedStyle(
   }
 
   RefPtr<nsComputedDOMStyle> computedDOMStyle =
-      NS_NewComputedDOMStyle(aElement, EmptyString(), document);
+      NS_NewComputedDOMStyle(aElement, u""_ns, document);
   return computedDOMStyle.forget();
 }
 
 // remove the CSS style "aProperty : aPropertyValue" and possibly remove the
 // whole node if it is a span and if its only attribute is _moz_dirty
-nsresult CSSEditUtils::RemoveCSSInlineStyle(nsINode& aNode, nsAtom* aProperty,
-                                            const nsAString& aPropertyValue) {
-  OwningNonNull<Element> element(*aNode.AsElement());
-
+nsresult CSSEditUtils::RemoveCSSInlineStyleWithTransaction(
+    nsStyledElement& aStyledElement, nsAtom* aProperty,
+    const nsAString& aPropertyValue) {
   // remove the property from the style attribute
-  nsresult rv = RemoveCSSProperty(element, *aProperty, aPropertyValue);
+  nsresult rv = RemoveCSSPropertyWithTransaction(aStyledElement, *aProperty,
+                                                 aPropertyValue);
   if (NS_FAILED(rv)) {
-    NS_WARNING("CSSEditUtils::RemoveCSSProperty() failed");
+    NS_WARNING("CSSEditUtils::RemoveCSSPropertyWithTransaction() failed");
     return rv;
   }
 
-  if (!element->IsHTMLElement(nsGkAtoms::span) ||
-      HTMLEditor::HasAttributes(element)) {
+  if (!aStyledElement.IsHTMLElement(nsGkAtoms::span) ||
+      HTMLEditor::HasAttributes(&aStyledElement)) {
     return NS_OK;
   }
 
   OwningNonNull<HTMLEditor> htmlEditor(*mHTMLEditor);
-  rv = htmlEditor->RemoveContainerWithTransaction(element);
+  rv = htmlEditor->RemoveContainerWithTransaction(aStyledElement);
+  if (NS_WARN_IF(htmlEditor->Destroyed())) {
+    return NS_ERROR_EDITOR_DESTROYED;
+  }
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
                        "HTMLEditor::RemoveContainerWithTransaction() failed");
   return rv;
@@ -815,14 +852,10 @@ void CSSEditUtils::GenerateCSSDeclarationsFromHTMLStyle(
 // Add to aNode the CSS inline style equivalent to HTMLProperty/aAttribute/
 // aValue for the node, and return in aCount the number of CSS properties set
 // by the call.  The Element version returns aCount instead.
-int32_t CSSEditUtils::SetCSSEquivalentToHTMLStyle(Element* aElement,
-                                                  nsAtom* aHTMLProperty,
-                                                  nsAtom* aAttribute,
-                                                  const nsAString* aValue,
-                                                  bool aSuppressTransaction) {
-  MOZ_ASSERT(aElement);
-
-  if (!IsCSSEditableProperty(aElement, aHTMLProperty, aAttribute)) {
+Result<int32_t, nsresult> CSSEditUtils::SetCSSEquivalentToHTMLStyleInternal(
+    nsStyledElement& aStyledElement, nsAtom* aHTMLProperty, nsAtom* aAttribute,
+    const nsAString* aValue, bool aSuppressTransaction) {
+  if (!IsCSSEditableProperty(&aStyledElement, aHTMLProperty, aAttribute)) {
     return 0;
   }
 
@@ -832,19 +865,19 @@ int32_t CSSEditUtils::SetCSSEquivalentToHTMLStyle(Element* aElement,
   // Find the CSS equivalence to the HTML style
   nsTArray<nsStaticAtom*> cssPropertyArray;
   nsTArray<nsString> cssValueArray;
-  GenerateCSSDeclarationsFromHTMLStyle(*aElement, aHTMLProperty, aAttribute,
-                                       aValue, cssPropertyArray, cssValueArray,
-                                       false);
+  GenerateCSSDeclarationsFromHTMLStyle(aStyledElement, aHTMLProperty,
+                                       aAttribute, aValue, cssPropertyArray,
+                                       cssValueArray, false);
 
   // set the individual CSS inline styles
-  size_t count = cssPropertyArray.Length();
+  const size_t count = cssPropertyArray.Length();
   for (size_t index = 0; index < count; index++) {
-    nsresult rv =
-        SetCSSProperty(*aElement, MOZ_KnownLive(*cssPropertyArray[index]),
-                       cssValueArray[index], aSuppressTransaction);
+    nsresult rv = SetCSSPropertyInternal(
+        aStyledElement, MOZ_KnownLive(*cssPropertyArray[index]),
+        cssValueArray[index], aSuppressTransaction);
     if (NS_FAILED(rv)) {
-      NS_WARNING("CSSEditUtils::SetCSSProperty() failed");
-      return 0;
+      NS_WARNING("CSSEditUtils::SetCSSPropertyInternal() failed");
+      return Err(rv);
     }
   }
   return count;
@@ -852,14 +885,10 @@ int32_t CSSEditUtils::SetCSSEquivalentToHTMLStyle(Element* aElement,
 
 // Remove from aNode the CSS inline style equivalent to
 // HTMLProperty/aAttribute/aValue for the node
-nsresult CSSEditUtils::RemoveCSSEquivalentToHTMLStyle(
-    Element* aElement, nsAtom* aHTMLProperty, nsAtom* aAttribute,
+nsresult CSSEditUtils::RemoveCSSEquivalentToHTMLStyleInternal(
+    nsStyledElement& aStyledElement, nsAtom* aHTMLProperty, nsAtom* aAttribute,
     const nsAString* aValue, bool aSuppressTransaction) {
-  if (NS_WARN_IF(!aElement)) {
-    return NS_OK;
-  }
-
-  if (!IsCSSEditableProperty(aElement, aHTMLProperty, aAttribute)) {
+  if (!IsCSSEditableProperty(&aStyledElement, aHTMLProperty, aAttribute)) {
     return NS_OK;
   }
 
@@ -869,18 +898,21 @@ nsresult CSSEditUtils::RemoveCSSEquivalentToHTMLStyle(
   // Find the CSS equivalence to the HTML style
   nsTArray<nsStaticAtom*> cssPropertyArray;
   nsTArray<nsString> cssValueArray;
-  GenerateCSSDeclarationsFromHTMLStyle(*aElement, aHTMLProperty, aAttribute,
-                                       aValue, cssPropertyArray, cssValueArray,
-                                       true);
+  GenerateCSSDeclarationsFromHTMLStyle(aStyledElement, aHTMLProperty,
+                                       aAttribute, aValue, cssPropertyArray,
+                                       cssValueArray, true);
 
   // remove the individual CSS inline styles
-  int32_t count = cssPropertyArray.Length();
-  for (int32_t index = 0; index < count; index++) {
-    nsresult rv =
-        RemoveCSSProperty(*aElement, MOZ_KnownLive(*cssPropertyArray[index]),
-                          cssValueArray[index], aSuppressTransaction);
+  const size_t count = cssPropertyArray.Length();
+  if (!count) {
+    return NS_OK;
+  }
+  for (size_t index = 0; index < count; index++) {
+    nsresult rv = RemoveCSSPropertyInternal(
+        aStyledElement, MOZ_KnownLive(*cssPropertyArray[index]),
+        cssValueArray[index], aSuppressTransaction);
     if (NS_FAILED(rv)) {
-      NS_WARNING("CSSEditUtils::RemoveCSSProperty() failed");
+      NS_WARNING("CSSEditUtils::RemoveCSSPropertyWithoutTransaction() failed");
       return rv;
     }
   }
@@ -936,7 +968,7 @@ nsresult CSSEditUtils::GetCSSEquivalentToHTMLInlineStyleSetInternal(
         return rv;
       }
     }
-    // append the value to aValue (possibly with a leading whitespace)
+    // append the value to aValue (possibly with a leading white-space)
     if (index) {
       aValue.Append(char16_t(' '));
     }
@@ -1015,13 +1047,11 @@ bool CSSEditUtils::IsCSSEquivalentToHTMLInlineStyleSetInternal(
         isSet = true;
       }
     } else if (nsGkAtoms::u == aHTMLProperty) {
-      nsAutoString val;
-      val.AssignLiteral("underline");
-      isSet = ChangeStyleTransaction::ValueIncludes(aValue, val);
+      isSet = ChangeStyleTransaction::ValueIncludes(
+          NS_ConvertUTF16toUTF8(aValue), "underline"_ns);
     } else if (nsGkAtoms::strike == aHTMLProperty) {
-      nsAutoString val;
-      val.AssignLiteral("line-through");
-      isSet = ChangeStyleTransaction::ValueIncludes(aValue, val);
+      isSet = ChangeStyleTransaction::ValueIncludes(
+          NS_ConvertUTF16toUTF8(aValue), "line-through"_ns);
     } else if ((nsGkAtoms::font == aHTMLProperty &&
                 aAttribute == nsGkAtoms::color) ||
                aAttribute == nsGkAtoms::bgcolor) {
@@ -1044,7 +1074,7 @@ bool CSSEditUtils::IsCSSEquivalentToHTMLInlineStyleSetInternal(
           } else {
             htmlColor.AppendLiteral("rgb(");
 
-            NS_NAMED_LITERAL_STRING(comma, ", ");
+            constexpr auto comma = u", "_ns;
 
             tmpStr.AppendInt(NS_GET_R(rgba), 10);
             htmlColor.Append(tmpStr + comma);
@@ -1067,7 +1097,7 @@ bool CSSEditUtils::IsCSSEquivalentToHTMLInlineStyleSetInternal(
         }
       }
     } else if (nsGkAtoms::tt == aHTMLProperty) {
-      isSet = StringBeginsWith(aValue, NS_LITERAL_STRING("monospace"));
+      isSet = StringBeginsWith(aValue, u"monospace"_ns);
     } else if (nsGkAtoms::font == aHTMLProperty && aAttribute &&
                aAttribute == nsGkAtoms::face) {
       if (!htmlValueString.IsEmpty()) {
@@ -1167,10 +1197,10 @@ bool CSSEditUtils::IsCSSPrefChecked() const { return mIsCSSPrefChecked; }
 // class
 
 // static
-bool CSSEditUtils::DoElementsHaveSameStyle(const Element& aElement,
-                                           const Element& aOtherElement) {
-  if (aElement.HasAttr(kNameSpaceID_None, nsGkAtoms::id) ||
-      aOtherElement.HasAttr(kNameSpaceID_None, nsGkAtoms::id)) {
+bool CSSEditUtils::DoStyledElementsHaveSameStyle(
+    nsStyledElement& aStyledElement, nsStyledElement& aOtherStyledElement) {
+  if (aStyledElement.HasAttr(kNameSpaceID_None, nsGkAtoms::id) ||
+      aOtherStyledElement.HasAttr(kNameSpaceID_None, nsGkAtoms::id)) {
     // at least one of the spans carries an ID ; suspect a CSS rule applies to
     // it and refuse to merge the nodes
     return false;
@@ -1178,9 +1208,9 @@ bool CSSEditUtils::DoElementsHaveSameStyle(const Element& aElement,
 
   nsAutoString firstClass, otherClass;
   bool isElementClassSet =
-      aElement.GetAttr(kNameSpaceID_None, nsGkAtoms::_class, firstClass);
-  bool isOtherElementClassSet =
-      aOtherElement.GetAttr(kNameSpaceID_None, nsGkAtoms::_class, otherClass);
+      aStyledElement.GetAttr(kNameSpaceID_None, nsGkAtoms::_class, firstClass);
+  bool isOtherElementClassSet = aOtherStyledElement.GetAttr(
+      kNameSpaceID_None, nsGkAtoms::_class, otherClass);
   if (isElementClassSet && isOtherElementClassSet) {
     // both spans carry a class, let's compare them
     if (!firstClass.Equals(otherClass)) {
@@ -1197,21 +1227,21 @@ bool CSSEditUtils::DoElementsHaveSameStyle(const Element& aElement,
     return false;
   }
 
-  nsCOMPtr<nsICSSDeclaration> firstCSSDecl, otherCSSDecl;
-  uint32_t firstLength, otherLength;
-  nsresult rv =
-      GetInlineStyles(aElement, getter_AddRefs(firstCSSDecl), &firstLength);
-  if (NS_FAILED(rv) || !firstCSSDecl) {
-    NS_WARNING("CSSEditUtils::GetInlineStyles() failed");
+  // XXX If `GetPropertyValue()` won't run script, we can stop using
+  //     nsCOMPtr here.
+  nsCOMPtr<nsICSSDeclaration> firstCSSDecl = aStyledElement.Style();
+  if (!firstCSSDecl) {
+    NS_WARNING("nsStyledElement::Style() failed");
     return false;
   }
-  rv = GetInlineStyles(aOtherElement, getter_AddRefs(otherCSSDecl),
-                       &otherLength);
-  if (NS_FAILED(rv) || !otherCSSDecl) {
-    NS_WARNING("CSSEditUtils::GetInlineStyles() failed");
+  nsCOMPtr<nsICSSDeclaration> otherCSSDecl = aOtherStyledElement.Style();
+  if (!otherCSSDecl) {
+    NS_WARNING("nsStyledElement::Style() failed");
     return false;
   }
 
+  const uint32_t firstLength = firstCSSDecl->Length();
+  const uint32_t otherLength = otherCSSDecl->Length();
   if (firstLength != otherLength) {
     // early way out if we can
     return false;
@@ -1222,9 +1252,9 @@ bool CSSEditUtils::DoElementsHaveSameStyle(const Element& aElement,
     return true;
   }
 
-  nsAutoCString propertyNameString;
-  nsAutoString firstValue, otherValue;
   for (uint32_t i = 0; i < firstLength; i++) {
+    nsAutoCString firstValue, otherValue;
+    nsAutoCString propertyNameString;
     firstCSSDecl->Item(i, propertyNameString);
     DebugOnly<nsresult> rvIgnored =
         firstCSSDecl->GetPropertyValue(propertyNameString, firstValue);
@@ -1240,6 +1270,8 @@ bool CSSEditUtils::DoElementsHaveSameStyle(const Element& aElement,
     }
   }
   for (uint32_t i = 0; i < otherLength; i++) {
+    nsAutoCString firstValue, otherValue;
+    nsAutoCString propertyNameString;
     otherCSSDecl->Item(i, propertyNameString);
     DebugOnly<nsresult> rvIgnored =
         otherCSSDecl->GetPropertyValue(propertyNameString, otherValue);
@@ -1258,27 +1290,4 @@ bool CSSEditUtils::DoElementsHaveSameStyle(const Element& aElement,
   return true;
 }
 
-// static
-nsresult CSSEditUtils::GetInlineStyles(const Element& aElement,
-                                       nsICSSDeclaration** aCssDecl,
-                                       uint32_t* aLength) {
-  if (NS_WARN_IF(!aLength)) {
-    return NS_ERROR_INVALID_ARG;
-  }
-  *aLength = 0;
-  // TODO: Perhaps, this method should take nsStyledElement& instead.
-  nsCOMPtr<nsStyledElement> styledElement =
-      do_QueryInterface(const_cast<Element*>(&aElement));
-  if (NS_WARN_IF(!styledElement)) {
-    return NS_ERROR_INVALID_ARG;
-  }
-
-  nsCOMPtr<nsICSSDeclaration> cssDecl = styledElement->Style();
-  MOZ_ASSERT(cssDecl);
-
-  cssDecl.forget(aCssDecl);
-  *aLength = (*aCssDecl)->Length();
-  return NS_OK;
-}
-
 }  // namespace mozilla
