@@ -9,7 +9,6 @@
 #include "mozilla/EditorDOMPoint.h"
 #include "mozilla/EditTransactionBase.h"
 #include "mozilla/RefPtr.h"
-#include "mozilla/dom/Element.h"
 #include "nsCycleCollectionParticipant.h"
 #include "nsISupportsImpl.h"
 
@@ -19,6 +18,9 @@ class nsAtom;
  * A transaction that creates a new node in the content tree.
  */
 namespace mozilla {
+namespace dom {
+class Element;
+}
 
 class EditorBase;
 
@@ -56,6 +58,9 @@ class CreateElementTransaction final : public EditTransactionBase {
 
   dom::Element* GetNewElement() const { return mNewElement; }
 
+  friend std::ostream& operator<<(std::ostream& aStream,
+                                  const CreateElementTransaction& aTransaction);
+
  protected:
   virtual ~CreateElementTransaction() = default;
 
