@@ -151,47 +151,45 @@ nsresult EditorEventListener::InstallToEditor() {
 
 #ifdef HANDLE_NATIVE_TEXT_DIRECTION_SWITCH
   eventListenerManager->AddEventListenerByType(
-      this, NS_LITERAL_STRING("keydown"), TrustedEventsAtSystemGroupBubble());
+      this, u"keydown"_ns, TrustedEventsAtSystemGroupBubble());
   eventListenerManager->AddEventListenerByType(
-      this, NS_LITERAL_STRING("keyup"), TrustedEventsAtSystemGroupBubble());
+      this, u"keyup"_ns, TrustedEventsAtSystemGroupBubble());
 #endif
   eventListenerManager->AddEventListenerByType(
-      this, NS_LITERAL_STRING("keypress"), TrustedEventsAtSystemGroupBubble());
+      this, u"keypress"_ns, TrustedEventsAtSystemGroupBubble());
   eventListenerManager->AddEventListenerByType(
-      this, NS_LITERAL_STRING("dragenter"), TrustedEventsAtSystemGroupBubble());
+      this, u"dragenter"_ns, TrustedEventsAtSystemGroupBubble());
   eventListenerManager->AddEventListenerByType(
-      this, NS_LITERAL_STRING("dragover"), TrustedEventsAtSystemGroupBubble());
+      this, u"dragover"_ns, TrustedEventsAtSystemGroupBubble());
   eventListenerManager->AddEventListenerByType(
-      this, NS_LITERAL_STRING("dragexit"), TrustedEventsAtSystemGroupBubble());
+      this, u"dragexit"_ns, TrustedEventsAtSystemGroupBubble());
   eventListenerManager->AddEventListenerByType(
-      this, NS_LITERAL_STRING("drop"), TrustedEventsAtSystemGroupBubble());
+      this, u"drop"_ns, TrustedEventsAtSystemGroupBubble());
   // XXX We should add the mouse event listeners as system event group.
   //     E.g., web applications cannot prevent middle mouse paste by
   //     preventDefault() of click event at bubble phase.
   //     However, if we do so, all click handlers in any frames and frontend
   //     code need to check if it's editable.  It makes easier create new bugs.
-  eventListenerManager->AddEventListenerByType(
-      this, NS_LITERAL_STRING("mousedown"), TrustedEventsAtCapture());
-  eventListenerManager->AddEventListenerByType(
-      this, NS_LITERAL_STRING("mouseup"), TrustedEventsAtCapture());
-  eventListenerManager->AddEventListenerByType(this, NS_LITERAL_STRING("click"),
+  eventListenerManager->AddEventListenerByType(this, u"mousedown"_ns,
+                                               TrustedEventsAtCapture());
+  eventListenerManager->AddEventListenerByType(this, u"mouseup"_ns,
+                                               TrustedEventsAtCapture());
+  eventListenerManager->AddEventListenerByType(this, u"click"_ns,
                                                TrustedEventsAtCapture());
   eventListenerManager->AddEventListenerByType(
-      this, NS_LITERAL_STRING("auxclick"), TrustedEventsAtSystemGroupCapture());
+      this, u"auxclick"_ns, TrustedEventsAtSystemGroupCapture());
   // Focus event doesn't bubble so adding the listener to capturing phase as
   // system event group.
   eventListenerManager->AddEventListenerByType(
-      this, NS_LITERAL_STRING("blur"), TrustedEventsAtSystemGroupCapture());
+      this, u"blur"_ns, TrustedEventsAtSystemGroupCapture());
   eventListenerManager->AddEventListenerByType(
-      this, NS_LITERAL_STRING("focus"), TrustedEventsAtSystemGroupCapture());
+      this, u"focus"_ns, TrustedEventsAtSystemGroupCapture());
   eventListenerManager->AddEventListenerByType(
-      this, NS_LITERAL_STRING("text"), TrustedEventsAtSystemGroupBubble());
+      this, u"text"_ns, TrustedEventsAtSystemGroupBubble());
   eventListenerManager->AddEventListenerByType(
-      this, NS_LITERAL_STRING("compositionstart"),
-      TrustedEventsAtSystemGroupBubble());
+      this, u"compositionstart"_ns, TrustedEventsAtSystemGroupBubble());
   eventListenerManager->AddEventListenerByType(
-      this, NS_LITERAL_STRING("compositionend"),
-      TrustedEventsAtSystemGroupBubble());
+      this, u"compositionend"_ns, TrustedEventsAtSystemGroupBubble());
 
   return NS_OK;
 }
@@ -236,40 +234,38 @@ void EditorEventListener::UninstallFromEditor() {
 
 #ifdef HANDLE_NATIVE_TEXT_DIRECTION_SWITCH
   eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("keydown"), TrustedEventsAtSystemGroupBubble());
+      this, u"keydown"_ns, TrustedEventsAtSystemGroupBubble());
   eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("keyup"), TrustedEventsAtSystemGroupBubble());
+      this, u"keyup"_ns, TrustedEventsAtSystemGroupBubble());
 #endif
   eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("keypress"), TrustedEventsAtSystemGroupBubble());
+      this, u"keypress"_ns, TrustedEventsAtSystemGroupBubble());
   eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("dragenter"), TrustedEventsAtSystemGroupBubble());
+      this, u"dragenter"_ns, TrustedEventsAtSystemGroupBubble());
   eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("dragover"), TrustedEventsAtSystemGroupBubble());
+      this, u"dragover"_ns, TrustedEventsAtSystemGroupBubble());
   eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("dragexit"), TrustedEventsAtSystemGroupBubble());
+      this, u"dragexit"_ns, TrustedEventsAtSystemGroupBubble());
   eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("drop"), TrustedEventsAtSystemGroupBubble());
+      this, u"drop"_ns, TrustedEventsAtSystemGroupBubble());
+  eventListenerManager->RemoveEventListenerByType(this, u"mousedown"_ns,
+                                                  TrustedEventsAtCapture());
+  eventListenerManager->RemoveEventListenerByType(this, u"mouseup"_ns,
+                                                  TrustedEventsAtCapture());
+  eventListenerManager->RemoveEventListenerByType(this, u"click"_ns,
+                                                  TrustedEventsAtCapture());
   eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("mousedown"), TrustedEventsAtCapture());
+      this, u"auxclick"_ns, TrustedEventsAtSystemGroupCapture());
   eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("mouseup"), TrustedEventsAtCapture());
+      this, u"blur"_ns, TrustedEventsAtSystemGroupCapture());
   eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("click"), TrustedEventsAtCapture());
+      this, u"focus"_ns, TrustedEventsAtSystemGroupCapture());
   eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("auxclick"), TrustedEventsAtSystemGroupCapture());
+      this, u"text"_ns, TrustedEventsAtSystemGroupBubble());
   eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("blur"), TrustedEventsAtSystemGroupCapture());
+      this, u"compositionstart"_ns, TrustedEventsAtSystemGroupBubble());
   eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("focus"), TrustedEventsAtSystemGroupCapture());
-  eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("text"), TrustedEventsAtSystemGroupBubble());
-  eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("compositionstart"),
-      TrustedEventsAtSystemGroupBubble());
-  eventListenerManager->RemoveEventListenerByType(
-      this, NS_LITERAL_STRING("compositionend"),
-      TrustedEventsAtSystemGroupBubble());
+      this, u"compositionend"_ns, TrustedEventsAtSystemGroupBubble());
 }
 
 PresShell* EditorEventListener::GetPresShell() const {
@@ -445,7 +441,7 @@ NS_IMETHODIMP EditorEventListener::HandleEvent(Event* aEvent) {
     case eMouseClick: {
       WidgetMouseEvent* widgetMouseEvent = internalEvent->AsMouseEvent();
       // Don't handle non-primary click events
-      if (widgetMouseEvent->mButton != MouseButton::eLeft) {
+      if (widgetMouseEvent->mButton != MouseButton::ePrimary) {
         return NS_OK;
       }
       [[fallthrough]];
@@ -576,7 +572,7 @@ nsresult EditorEventListener::KeyUp(const WidgetKeyboardEvent* aKeyboardEvent) {
   RefPtr<EditorBase> editorBase(mEditorBase);
   if ((aKeyboardEvent->mKeyCode == NS_VK_SHIFT ||
        aKeyboardEvent->mKeyCode == NS_VK_CONTROL) &&
-      mShouldSwitchTextDirection && editorBase->IsPlaintextEditor()) {
+      mShouldSwitchTextDirection && editorBase->IsInPlaintextMode()) {
     editorBase->SwitchTextDirectionTo(mSwitchToRTL
                                           ? EditorBase::TextDirection::eRTL
                                           : EditorBase::TextDirection::eLTR);
@@ -670,9 +666,9 @@ nsresult EditorEventListener::MouseClick(WidgetMouseEvent* aMouseClickEvent) {
     return NS_OK;
   }
   // nothing to do if editor isn't editable or clicked on out of the editor.
-  RefPtr<TextEditor> textEditor = mEditorBase->AsTextEditor();
-  if (textEditor->IsReadonly() ||
-      !textEditor->IsAcceptableInputEvent(aMouseClickEvent)) {
+  OwningNonNull<EditorBase> editorBase = *mEditorBase;
+  if (editorBase->IsReadonly() ||
+      !editorBase->IsAcceptableInputEvent(aMouseClickEvent)) {
     return NS_OK;
   }
 
@@ -727,7 +723,7 @@ nsresult EditorEventListener::MouseClick(WidgetMouseEvent* aMouseClickEvent) {
   nsEventStatus status = nsEventStatus_eIgnore;
   RefPtr<EventStateManager> esm = presContext->EventStateManager();
   DebugOnly<nsresult> rvIgnored = esm->HandleMiddleClickPaste(
-      presShell, aMouseClickEvent, &status, textEditor);
+      presShell, aMouseClickEvent, &status, editorBase);
   NS_WARNING_ASSERTION(
       NS_SUCCEEDED(rvIgnored),
       "EventStateManager::HandleMiddleClickPaste() failed, but ignored");
@@ -748,12 +744,13 @@ bool EditorEventListener::NotifyIMEOfMouseButtonEvent(
     return false;
   }
 
-  nsPresContext* presContext = GetPresContext();
+  RefPtr<nsPresContext> presContext = GetPresContext();
   if (NS_WARN_IF(!presContext)) {
     return false;
   }
+  nsCOMPtr<nsIContent> focusedRootContent = GetFocusedRootContent();
   return IMEStateManager::OnMouseButtonEventInEditor(
-      presContext, GetFocusedRootContent(), aMouseEvent);
+      presContext, focusedRootContent, aMouseEvent);
 }
 
 nsresult EditorEventListener::MouseDown(MouseEvent* aMouseEvent) {
@@ -807,9 +804,10 @@ void EditorEventListener::RefuseToDropAndHideCaret(DragEvent* aDragEvent) {
 
   aDragEvent->PreventDefault();
   aDragEvent->StopImmediatePropagation();
-  MOZ_ASSERT(aDragEvent->GetDataTransfer());
-  aDragEvent->GetDataTransfer()->SetDropEffectInt(
-      nsIDragService::DRAGDROP_ACTION_NONE);
+  DataTransfer* dataTransfer = aDragEvent->GetDataTransfer();
+  if (dataTransfer) {
+    dataTransfer->SetDropEffectInt(nsIDragService::DRAGDROP_ACTION_NONE);
+  }
   if (mCaret) {
     mCaret->SetVisible(false);
   }
@@ -833,7 +831,7 @@ nsresult EditorEventListener::DragOverOrDrop(DragEvent* aDragEvent) {
   int32_t dropOffset = -1;
   nsCOMPtr<nsIContent> dropParentContent =
       aDragEvent->GetRangeParentContentAndOffset(&dropOffset);
-  if (NS_WARN_IF(!dropParentContent)) {
+  if (NS_WARN_IF(!dropParentContent) || NS_WARN_IF(dropOffset < 0)) {
     return NS_ERROR_FAILURE;
   }
   if (DetachedFromEditor()) {
@@ -861,7 +859,7 @@ nsresult EditorEventListener::DragOverOrDrop(DragEvent* aDragEvent) {
   if (notEditable) {
     // If we're a text control element which is readonly or disabled,
     // we should refuse to drop.
-    if (!mEditorBase->AsHTMLEditor()) {
+    if (mEditorBase->IsTextEditor()) {
       RefuseToDropAndHideCaret(aDragEvent);
       return NS_OK;
     }
@@ -892,9 +890,10 @@ nsresult EditorEventListener::DragOverOrDrop(DragEvent* aDragEvent) {
   aDragEvent->StopImmediatePropagation();
 
   if (aDragEvent->WidgetEventPtr()->mMessage == eDrop) {
-    RefPtr<TextEditor> textEditor = mEditorBase->AsTextEditor();
-    nsresult rv = textEditor->OnDrop(aDragEvent);
-    NS_WARNING_ASSERTION(NS_SUCCEEDED(rv), "TextEditor::OnDrop() failed");
+    RefPtr<EditorBase> editorBase = mEditorBase;
+    nsresult rv = editorBase->HandleDropEvent(aDragEvent);
+    NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
+                         "EditorBase::HandleDropEvent() failed");
     return rv;
   }
 
@@ -905,9 +904,9 @@ nsresult EditorEventListener::DragOverOrDrop(DragEvent* aDragEvent) {
   // because once DataTransfer is retrieved, DragEvent has initialized it
   // with nsContentUtils::SetDataTransferInEvent() but it does not check
   // whether the content is movable or not.
-  MOZ_ASSERT(aDragEvent->GetDataTransfer());
   DataTransfer* dataTransfer = aDragEvent->GetDataTransfer();
-  if (dataTransfer->DropEffectInt() == nsIDragService::DRAGDROP_ACTION_MOVE) {
+  if (dataTransfer &&
+      dataTransfer->DropEffectInt() == nsIDragService::DRAGDROP_ACTION_MOVE) {
     nsCOMPtr<nsINode> dragSource = dataTransfer->GetMozSourceNode();
     if (dragSource && !dragSource->IsEditable()) {
       // In this case, we shouldn't allow "move" because the drag source
@@ -968,11 +967,16 @@ bool EditorEventListener::DragEventHasSupportingData(
   // Plaintext editors only support dropping text. Otherwise, HTML and files
   // can be dropped as well.
   DataTransfer* dataTransfer = aDragEvent->GetDataTransfer();
-  return dataTransfer->HasType(NS_LITERAL_STRING(kTextMime)) ||
-         dataTransfer->HasType(NS_LITERAL_STRING(kMozTextInternal)) ||
-         (!mEditorBase->IsPlaintextEditor() &&
-          (dataTransfer->HasType(NS_LITERAL_STRING(kHTMLMime)) ||
-           dataTransfer->HasType(NS_LITERAL_STRING(kFileMime))));
+  if (!dataTransfer) {
+    NS_WARNING("No data transfer returned");
+    return false;
+  }
+  return dataTransfer->HasType(NS_LITERAL_STRING_FROM_CSTRING(kTextMime)) ||
+         dataTransfer->HasType(
+             NS_LITERAL_STRING_FROM_CSTRING(kMozTextInternal)) ||
+         (!mEditorBase->IsInPlaintextMode() &&
+          (dataTransfer->HasType(NS_LITERAL_STRING_FROM_CSTRING(kHTMLMime)) ||
+           dataTransfer->HasType(NS_LITERAL_STRING_FROM_CSTRING(kFileMime))));
 }
 
 bool EditorEventListener::CanInsertAtDropPosition(DragEvent* aDragEvent) {
@@ -981,11 +985,15 @@ bool EditorEventListener::CanInsertAtDropPosition(DragEvent* aDragEvent) {
   MOZ_ASSERT(!mEditorBase->IsReadonly());
   MOZ_ASSERT(DragEventHasSupportingData(aDragEvent));
 
+  DataTransfer* dataTransfer = aDragEvent->GetDataTransfer();
+  if (NS_WARN_IF(!dataTransfer)) {
+    return false;
+  }
+
   // If there is no source node, this is probably an external drag and the
   // drop is allowed. The later checks rely on checking if the drag target
   // is the same as the drag source.
-  nsCOMPtr<nsINode> sourceNode =
-      aDragEvent->GetDataTransfer()->GetMozSourceNode();
+  nsCOMPtr<nsINode> sourceNode = dataTransfer->GetMozSourceNode();
   if (!sourceNode) {
     return true;
   }
@@ -1024,30 +1032,13 @@ bool EditorEventListener::CanInsertAtDropPosition(DragEvent* aDragEvent) {
   int32_t dropOffset = -1;
   nsCOMPtr<nsIContent> dropParentContent =
       aDragEvent->GetRangeParentContentAndOffset(&dropOffset);
-  if (!dropParentContent || NS_WARN_IF(DetachedFromEditor())) {
+  if (!dropParentContent || NS_WARN_IF(dropOffset < 0) ||
+      NS_WARN_IF(DetachedFromEditor())) {
     return false;
   }
 
-  uint32_t rangeCount = selection->RangeCount();
-  IgnoredErrorResult ignoredError;
-  for (uint32_t i = 0; i < rangeCount; i++) {
-    RefPtr<const nsRange> range = selection->GetRangeAt(i);
-    if (!range) {
-      // Don't bail yet, iterate through them all
-      continue;
-    }
-
-    bool inRange =
-        range->IsPointInRange(*dropParentContent, dropOffset, ignoredError);
-    NS_WARNING_ASSERTION(!ignoredError.Failed(),
-                         "nsRange::IsPointInRange() failed");
-    if (!ignoredError.Failed() && inRange) {
-      // Okay, now you can bail, we are over the orginal selection
-      return false;
-    }
-    ignoredError.SuppressException();
-  }
-  return true;
+  return !EditorUtils::IsPointInSelection(*selection, *dropParentContent,
+                                          dropOffset);
 }
 
 nsresult EditorEventListener::HandleStartComposition(
@@ -1058,17 +1049,17 @@ nsresult EditorEventListener::HandleStartComposition(
   if (DetachedFromEditor()) {
     return NS_OK;
   }
-  RefPtr<TextEditor> textEditor = mEditorBase->AsTextEditor();
-  if (!textEditor->IsAcceptableInputEvent(aCompositionStartEvent)) {
+  RefPtr<EditorBase> editorBase(mEditorBase);
+  if (!editorBase->IsAcceptableInputEvent(aCompositionStartEvent)) {
     return NS_OK;
   }
   // Although, "compositionstart" should be cancelable, but currently,
   // eCompositionStart event coming from widget is not cancelable.
   MOZ_ASSERT(!aCompositionStartEvent->DefaultPrevented(),
              "eCompositionStart shouldn't be cancelable");
-  nsresult rv = textEditor->OnCompositionStart(*aCompositionStartEvent);
+  nsresult rv = editorBase->OnCompositionStart(*aCompositionStartEvent);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
-                       "TextEditor::OnCompositionStart() failed");
+                       "EditorBase::OnCompositionStart() failed");
   return rv;
 }
 
@@ -1082,19 +1073,19 @@ nsresult EditorEventListener::HandleChangeComposition(
   if (DetachedFromEditor()) {
     return NS_OK;
   }
-  RefPtr<TextEditor> textEditor = mEditorBase->AsTextEditor();
-  if (!textEditor->IsAcceptableInputEvent(aCompositionChangeEvent)) {
+  RefPtr<EditorBase> editorBase(mEditorBase);
+  if (!editorBase->IsAcceptableInputEvent(aCompositionChangeEvent)) {
     return NS_OK;
   }
 
   // if we are readonly, then do nothing.
-  if (textEditor->IsReadonly()) {
+  if (editorBase->IsReadonly()) {
     return NS_OK;
   }
 
-  nsresult rv = textEditor->OnCompositionChange(*aCompositionChangeEvent);
+  nsresult rv = editorBase->OnCompositionChange(*aCompositionChangeEvent);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
-                       "TextEditor::OnCompositionChange() failed");
+                       "EditorBase::OnCompositionChange() failed");
   return rv;
 }
 
@@ -1103,14 +1094,14 @@ void EditorEventListener::HandleEndComposition(
   if (NS_WARN_IF(!aCompositionEndEvent) || DetachedFromEditor()) {
     return;
   }
-  RefPtr<TextEditor> textEditor = mEditorBase->AsTextEditor();
-  if (!textEditor->IsAcceptableInputEvent(aCompositionEndEvent)) {
+  RefPtr<EditorBase> editorBase(mEditorBase);
+  if (!editorBase->IsAcceptableInputEvent(aCompositionEndEvent)) {
     return;
   }
   MOZ_ASSERT(!aCompositionEndEvent->DefaultPrevented(),
              "eCompositionEnd shouldn't be cancelable");
 
-  textEditor->OnCompositionEnd(*aCompositionEndEvent);
+  editorBase->OnCompositionEnd(*aCompositionEndEvent);
 }
 
 nsresult EditorEventListener::Focus(InternalFocusEvent* aFocusEvent) {
@@ -1122,9 +1113,7 @@ nsresult EditorEventListener::Focus(InternalFocusEvent* aFocusEvent) {
 
   // Spell check a textarea the first time that it is focused.
   SpellCheckIfNeeded();
-  if (!editorBase) {
-    // In e10s, this can cause us to flush notifications, which can destroy
-    // the node we're about to focus.
+  if (DetachedFromEditor()) {
     return NS_OK;
   }
 
@@ -1181,7 +1170,7 @@ nsresult EditorEventListener::Focus(InternalFocusEvent* aFocusEvent) {
     return NS_OK;
   }
 
-  nsPresContext* presContext = GetPresContext();
+  RefPtr<nsPresContext> presContext = GetPresContext();
   if (NS_WARN_IF(!presContext)) {
     return NS_OK;
   }
@@ -1205,6 +1194,16 @@ nsresult EditorEventListener::Blur(InternalFocusEvent* aBlurEvent) {
 
   Element* focusedElement = focusManager->GetFocusedElement();
   if (!focusedElement) {
+    // If it's in the designMode, and blur occurs, the target must be the
+    // window.  If a blur event is fired and the target is an element, it
+    // must be delayed blur event at initializing the `HTMLEditor`.
+    if (mEditorBase->IsHTMLEditor() &&
+        mEditorBase->AsHTMLEditor()->IsInDesignMode()) {
+      if (nsCOMPtr<Element> targetElement =
+              do_QueryInterface(aBlurEvent->mTarget)) {
+        return NS_OK;
+      }
+    }
     RefPtr<EditorBase> editorBase(mEditorBase);
     DebugOnly<nsresult> rvIgnored = editorBase->FinalizeSelection();
     NS_WARNING_ASSERTION(NS_SUCCEEDED(rvIgnored),
@@ -1241,7 +1240,7 @@ bool EditorEventListener::IsFileControlTextBox() {
     return false;
   }
   nsCOMPtr<nsIFormControl> formControl = do_QueryInterface(parent);
-  return formControl->ControlType() == NS_FORM_INPUT_FILE;
+  return formControl->ControlType() == FormControlType::InputFile;
 }
 
 bool EditorEventListener::ShouldHandleNativeKeyBindings(
@@ -1262,13 +1261,12 @@ bool EditorEventListener::ShouldHandleNativeKeyBindings(
     return false;
   }
 
-  RefPtr<EditorBase> editorBase(mEditorBase);
-  HTMLEditor* htmlEditor = editorBase->AsHTMLEditor();
+  RefPtr<HTMLEditor> htmlEditor = HTMLEditor::GetFrom(mEditorBase);
   if (!htmlEditor) {
     return false;
   }
 
-  RefPtr<Document> doc = editorBase->GetDocument();
+  RefPtr<Document> doc = htmlEditor->GetDocument();
   if (doc->HasFlag(NODE_IS_EDITABLE)) {
     // Don't need to perform any checks in designMode documents.
     return true;
