@@ -10,8 +10,8 @@
 #include "mozilla/EditAction.h"
 #include "mozilla/EditorBase.h"
 #include "mozilla/EditorDOMPoint.h"
-#include "mozilla/GuardObjects.h"
 #include "mozilla/RangeBoundary.h"
+#include "mozilla/Result.h"
 #include "mozilla/dom/HTMLBRElement.h"
 #include "mozilla/dom/Selection.h"
 #include "mozilla/dom/StaticRange.h"
@@ -20,6 +20,7 @@
 #include "nsContentUtils.h"
 #include "nscore.h"
 #include "nsDebug.h"
+#include "nsDirection.h"
 #include "nsRange.h"
 #include "nsString.h"
 
@@ -700,9 +701,8 @@ class MOZ_STACK_CLASS SplitRangeOffResult final {
 class MOZ_RAII AutoTransactionBatchExternal final {
  public:
   MOZ_CAN_RUN_SCRIPT explicit AutoTransactionBatchExternal(
-      EditorBase& aEditorBase MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
+      EditorBase& aEditorBase)
       : mEditorBase(aEditorBase) {
-    MOZ_GUARD_OBJECT_NOTIFIER_INIT;
     MOZ_KnownLive(mEditorBase).BeginTransaction();
   }
 
@@ -712,32 +712,272 @@ class MOZ_RAII AutoTransactionBatchExternal final {
 
  private:
   EditorBase& mEditorBase;
-  MOZ_DECL_USE_GUARD_OBJECT_NOTIFIER
 };
 
-class MOZ_STACK_CLASS AutoRangeArray final {
+/******************************************************************************
+ * AutoSelectionRangeArray stores all ranges in `aSelection`.
+ * Note that modifying the ranges means modifing the selection ranges.
+ *****************************************************************************/
+class MOZ_STACK_CLASS AutoSelectionRangeArray final {
  public:
-  explicit AutoRangeArray(dom::Selection* aSelection) {
-    if (!aSelection) {
-      return;
-    }
-    uint32_t rangeCount = aSelection->RangeCount();
+  explicit AutoSelectionRangeArray(dom::Selection& aSelection) {
+    uint32_t rangeCount = aSelection.RangeCount();
     for (uint32_t i = 0; i < rangeCount; i++) {
-      mRanges.AppendElement(*aSelection->GetRangeAt(i));
+      mRanges.AppendElement(*aSelection.GetRangeAt(i));
     }
   }
 
   AutoTArray<mozilla::OwningNonNull<nsRange>, 8> mRanges;
 };
 
+/******************************************************************************
+ * AutoRangeArray stores ranges which do no belong any `Selection`.
+ * So, different from `AutoSelectionRangeArray`, this can be used for
+ * ranges which may need to be modified before touching the DOM tree,
+ * but does not want to modify `Selection` for the performance.
+ *****************************************************************************/
+class MOZ_STACK_CLASS AutoRangeArray final {
+ public:
+  explicit AutoRangeArray(const dom::Selection& aSelection) {
+    Initialize(aSelection);
+  }
+
+  void Initialize(const dom::Selection& aSelection) {
+    mDirection = aSelection.GetDirection();
+    mRanges.Clear();
+    for (uint32_t i = 0; i < aSelection.RangeCount(); i++) {
+      mRanges.AppendElement(aSelection.GetRangeAt(i)->CloneRange());
+      if (aSelection.GetRangeAt(i) == aSelection.GetAnchorFocusRange()) {
+        mAnchorFocusRange = mRanges.LastElement();
+      }
+    }
+  }
+
+  /**
+   * EnsureOnlyEditableRanges() removes ranges which cannot modify.
+   * Note that this is designed only for `HTMLEditor` because this must not
+   * be required by `TextEditor`.
+   */
+  void EnsureOnlyEditableRanges(const dom::Element& aEditingHost);
+  static bool IsEditableRange(const dom::AbstractRange& aRange,
+                              const dom::Element& aEditingHost);
+
+  /**
+   * IsAtLeastOneContainerOfRangeBoundariesInclusiveDescendantOf() returns true
+   * if at least one of the containers of the range boundaries is an inclusive
+   * descendant of aContent.
+   */
+  bool IsAtLeastOneContainerOfRangeBoundariesInclusiveDescendantOf(
+      const nsIContent& aContent) const {
+    for (const OwningNonNull<nsRange>& range : mRanges) {
+      nsINode* startContainer = range->GetStartContainer();
+      if (startContainer &&
+          startContainer->IsInclusiveDescendantOf(&aContent)) {
+        return true;
+      }
+      nsINode* endContainer = range->GetEndContainer();
+      if (startContainer == endContainer) {
+        continue;
+      }
+      if (endContainer && endContainer->IsInclusiveDescendantOf(&aContent)) {
+        return true;
+      }
+    }
+    return false;
+  }
+
+  auto& Ranges() { return mRanges; }
+  const auto& Ranges() const { return mRanges; }
+  auto& FirstRangeRef() { return mRanges[0]; }
+  const auto& FirstRangeRef() const { return mRanges[0]; }
+
+  template <template <typename> typename StrongPtrType>
+  AutoTArray<StrongPtrType<nsRange>, 8> CloneRanges() const {
+    AutoTArray<StrongPtrType<nsRange>, 8> ranges;
+    for (const auto& range : mRanges) {
+      ranges.AppendElement(range->CloneRange());
+    }
+    return ranges;
+  }
+
+  EditorDOMPoint GetStartPointOfFirstRange() const {
+    if (mRanges.IsEmpty() || !mRanges[0]->IsPositioned()) {
+      return EditorDOMPoint();
+    }
+    return EditorDOMPoint(mRanges[0]->StartRef());
+  }
+  EditorDOMPoint GetEndPointOfFirstRange() const {
+    if (mRanges.IsEmpty() || !mRanges[0]->IsPositioned()) {
+      return EditorDOMPoint();
+    }
+    return EditorDOMPoint(mRanges[0]->EndRef());
+  }
+
+  nsresult SelectNode(nsINode& aNode) {
+    mRanges.Clear();
+    if (!mAnchorFocusRange) {
+      mAnchorFocusRange = nsRange::Create(&aNode);
+      if (!mAnchorFocusRange) {
+        return NS_ERROR_FAILURE;
+      }
+    }
+    ErrorResult error;
+    mAnchorFocusRange->SelectNode(aNode, error);
+    if (error.Failed()) {
+      mAnchorFocusRange = nullptr;
+      return error.StealNSResult();
+    }
+    mRanges.AppendElement(*mAnchorFocusRange);
+    return NS_OK;
+  }
+
+  /**
+   * ExtendAnchorFocusRangeFor() extends the anchor-focus range for deleting
+   * content for aDirectionAndAmount.  The range won't be extended to outer of
+   * selection limiter.  Note that if a range is extened, the range is
+   * recreated.  Therefore, caller cannot cache pointer of any ranges before
+   * calling this.
+   */
+  [[nodiscard]] MOZ_CAN_RUN_SCRIPT Result<nsIEditor::EDirection, nsresult>
+  ExtendAnchorFocusRangeFor(const EditorBase& aEditorBase,
+                            nsIEditor::EDirection aDirectionAndAmount);
+
+  /**
+   * For compatiblity with the other browsers, we should shrink ranges to
+   * start from an atomic content and/or end after one instead of start
+   * from end of a preceding text node and end by start of a follwing text
+   * node.  Returns true if this modifies a range.
+   */
+  enum class IfSelectingOnlyOneAtomicContent {
+    Collapse,  // Collapse to the range selecting only one atomic content to
+               // start or after of it.  Whether to collapse start or after
+               // it depends on aDirectionAndAmount.  This is ignored if
+               // there are multiple ranges.
+    KeepSelecting,  // Won't collapse the range.
+  };
+  Result<bool, nsresult> ShrinkRangesIfStartFromOrEndAfterAtomicContent(
+      const HTMLEditor& aHTMLEditor, nsIEditor::EDirection aDirectionAndAmount,
+      IfSelectingOnlyOneAtomicContent aIfSelectingOnlyOneAtomicContent,
+      const dom::Element* aEditingHost);
+
+  /**
+   * The following methods are same as `Selection`'s methods.
+   */
+  bool IsCollapsed() const {
+    return mRanges.IsEmpty() ||
+           (mRanges.Length() == 1 && mRanges[0]->Collapsed());
+  }
+  template <typename PT, typename CT>
+  nsresult Collapse(const EditorDOMPointBase<PT, CT>& aPoint) {
+    mRanges.Clear();
+    if (!mAnchorFocusRange) {
+      ErrorResult error;
+      mAnchorFocusRange = nsRange::Create(aPoint.ToRawRangeBoundary(),
+                                          aPoint.ToRawRangeBoundary(), error);
+      if (error.Failed()) {
+        mAnchorFocusRange = nullptr;
+        return error.StealNSResult();
+      }
+    } else {
+      nsresult rv = mAnchorFocusRange->CollapseTo(aPoint.ToRawRangeBoundary());
+      if (NS_FAILED(rv)) {
+        mAnchorFocusRange = nullptr;
+        return rv;
+      }
+    }
+    mRanges.AppendElement(*mAnchorFocusRange);
+    return NS_OK;
+  }
+  template <typename SPT, typename SCT, typename EPT, typename ECT>
+  nsresult SetStartAndEnd(const EditorDOMPointBase<SPT, SCT>& aStart,
+                          const EditorDOMPointBase<EPT, ECT>& aEnd) {
+    mRanges.Clear();
+    if (!mAnchorFocusRange) {
+      ErrorResult error;
+      mAnchorFocusRange = nsRange::Create(aStart.ToRawRangeBoundary(),
+                                          aEnd.ToRawRangeBoundary(), error);
+      if (error.Failed()) {
+        mAnchorFocusRange = nullptr;
+        return error.StealNSResult();
+      }
+    } else {
+      nsresult rv = mAnchorFocusRange->SetStartAndEnd(
+          aStart.ToRawRangeBoundary(), aEnd.ToRawRangeBoundary());
+      if (NS_FAILED(rv)) {
+        mAnchorFocusRange = nullptr;
+        return rv;
+      }
+    }
+    mRanges.AppendElement(*mAnchorFocusRange);
+    return NS_OK;
+  }
+  const nsRange* GetAnchorFocusRange() const { return mAnchorFocusRange; }
+  nsDirection GetDirection() const { return mDirection; }
+
+  const RangeBoundary& AnchorRef() const {
+    if (!mAnchorFocusRange) {
+      static RangeBoundary sEmptyRangeBoundary;
+      return sEmptyRangeBoundary;
+    }
+    return mDirection == nsDirection::eDirNext ? mAnchorFocusRange->StartRef()
+                                               : mAnchorFocusRange->EndRef();
+  }
+  nsINode* GetAnchorNode() const {
+    return AnchorRef().IsSet() ? AnchorRef().Container() : nullptr;
+  }
+  uint32_t GetAnchorOffset() const {
+    return AnchorRef().IsSet()
+               ? AnchorRef()
+                     .Offset(RangeBoundary::OffsetFilter::kValidOffsets)
+                     .valueOr(0)
+               : 0;
+  }
+  nsIContent* GetChildAtAnchorOffset() const {
+    return AnchorRef().IsSet() ? AnchorRef().GetChildAtOffset() : nullptr;
+  }
+
+  const RangeBoundary& FocusRef() const {
+    if (!mAnchorFocusRange) {
+      static RangeBoundary sEmptyRangeBoundary;
+      return sEmptyRangeBoundary;
+    }
+    return mDirection == nsDirection::eDirNext ? mAnchorFocusRange->EndRef()
+                                               : mAnchorFocusRange->StartRef();
+  }
+  nsINode* GetFocusNode() const {
+    return FocusRef().IsSet() ? FocusRef().Container() : nullptr;
+  }
+  uint32_t FocusOffset() const {
+    return FocusRef().IsSet()
+               ? FocusRef()
+                     .Offset(RangeBoundary::OffsetFilter::kValidOffsets)
+                     .valueOr(0)
+               : 0;
+  }
+  nsIContent* GetChildAtFocusOffset() const {
+    return FocusRef().IsSet() ? FocusRef().GetChildAtOffset() : nullptr;
+  }
+
+  void RemoveAllRanges() {
+    mRanges.Clear();
+    mAnchorFocusRange = nullptr;
+    mDirection = nsDirection::eDirNext;
+  }
+
+ private:
+  AutoTArray<mozilla::OwningNonNull<nsRange>, 8> mRanges;
+  RefPtr<nsRange> mAnchorFocusRange;
+  nsDirection mDirection = nsDirection::eDirNext;
+};
+
 /******************************************************************************
  * some helper classes for iterating the dom tree
  *****************************************************************************/
 
 class MOZ_RAII DOMIterator {
  public:
-  explicit DOMIterator(MOZ_GUARD_OBJECT_NOTIFIER_ONLY_PARAM);
-  explicit DOMIterator(nsINode& aNode MOZ_GUARD_OBJECT_NOTIFIER_PARAM);
+  explicit DOMIterator();
+  explicit DOMIterator(nsINode& aNode);
   virtual ~DOMIterator() = default;
 
   nsresult Init(nsRange& aRange);
@@ -766,22 +1006,92 @@ class MOZ_RAII DOMIterator {
  protected:
   ContentIteratorBase* mIter;
   PostContentIterator mPostOrderIter;
-  MOZ_DECL_USE_GUARD_OBJECT_NOTIFIER
 };
 
 class MOZ_RAII DOMSubtreeIterator final : public DOMIterator {
  public:
-  explicit DOMSubtreeIterator(MOZ_GUARD_OBJECT_NOTIFIER_ONLY_PARAM);
+  explicit DOMSubtreeIterator();
   virtual ~DOMSubtreeIterator() = default;
 
   nsresult Init(nsRange& aRange);
 
  private:
   ContentSubtreeIterator mSubtreeIter;
-  explicit DOMSubtreeIterator(nsINode& aNode MOZ_GUARD_OBJECT_NOTIFIER_PARAM) =
-      delete;
+  explicit DOMSubtreeIterator(nsINode& aNode) = delete;
 };
 
+/**
+ * ReplaceRangeDataBase() represents range to be replaced and replacing string.
+ */
+template <typename EditorDOMPointType>
+class MOZ_STACK_CLASS ReplaceRangeDataBase final {
+ public:
+  ReplaceRangeDataBase() = default;
+  template <typename OtherEditorDOMRangeType>
+  ReplaceRangeDataBase(const OtherEditorDOMRangeType& aRange,
+                       const nsAString& aReplaceString)
+      : mRange(aRange), mReplaceString(aReplaceString) {}
+  template <typename StartPointType, typename EndPointType>
+  ReplaceRangeDataBase(const StartPointType& aStart, const EndPointType& aEnd,
+                       const nsAString& aReplaceString)
+      : mRange(aStart, aEnd), mReplaceString(aReplaceString) {}
+
+  bool IsSet() const { return mRange.IsPositioned(); }
+  bool IsSetAndValid() const { return mRange.IsPositionedAndValid(); }
+  bool Collapsed() const { return mRange.Collapsed(); }
+  bool HasReplaceString() const { return !mReplaceString.IsEmpty(); }
+  const EditorDOMPointType& StartRef() const { return mRange.StartRef(); }
+  const EditorDOMPointType& EndRef() const { return mRange.EndRef(); }
+  const EditorDOMRangeBase<EditorDOMPointType>& RangeRef() const {
+    return mRange;
+  }
+  const nsString& ReplaceStringRef() const { return mReplaceString; }
+
+  template <typename PointType>
+  MOZ_NEVER_INLINE_DEBUG void SetStart(const PointType& aStart) {
+    mRange.SetStart(aStart);
+  }
+  template <typename PointType>
+  MOZ_NEVER_INLINE_DEBUG void SetEnd(const PointType& aEnd) {
+    mRange.SetEnd(aEnd);
+  }
+  template <typename StartPointType, typename EndPointType>
+  MOZ_NEVER_INLINE_DEBUG void SetStartAndEnd(const StartPointType& aStart,
+                                             const EndPointType& aEnd) {
+    mRange.SetRange(aStart, aEnd);
+  }
+  template <typename OtherEditorDOMRangeType>
+  MOZ_NEVER_INLINE_DEBUG void SetRange(const OtherEditorDOMRangeType& aRange) {
+    mRange = aRange;
+  }
+  void SetReplaceString(const nsAString& aReplaceString) {
+    mReplaceString = aReplaceString;
+  }
+  template <typename StartPointType, typename EndPointType>
+  MOZ_NEVER_INLINE_DEBUG void SetStartAndEnd(const StartPointType& aStart,
+                                             const EndPointType& aEnd,
+                                             const nsAString& aReplaceString) {
+    SetStartAndEnd(aStart, aEnd);
+    SetReplaceString(aReplaceString);
+  }
+  template <typename OtherEditorDOMRangeType>
+  MOZ_NEVER_INLINE_DEBUG void Set(const OtherEditorDOMRangeType& aRange,
+                                  const nsAString& aReplaceString) {
+    SetRange(aRange);
+    SetReplaceString(aReplaceString);
+  }
+
+ private:
+  EditorDOMRangeBase<EditorDOMPointType> mRange;
+  // This string may be used with ReplaceTextTransaction.  Therefore, for
+  // avoiding memory copy, we should store it with nsString rather than
+  // nsAutoString.
+  nsString mReplaceString;
+};
+
+using ReplaceRangeData = ReplaceRangeDataBase<EditorDOMPoint>;
+using ReplaceRangeInTextsData = ReplaceRangeDataBase<EditorDOMPointInText>;
+
 class EditorUtils final {
  public:
   using EditorType = EditorBase::EditorType;
@@ -828,18 +1138,13 @@ class EditorUtils final {
    */
   static bool IsEditableContent(const nsIContent& aContent,
                                 EditorType aEditorType) {
-    if ((aEditorType == EditorType::HTML && !aContent.IsEditable()) ||
-        EditorUtils::IsPaddingBRElementForEmptyEditor(aContent)) {
+    if (aEditorType == EditorType::HTML && !aContent.IsEditable()) {
+      // FIXME(emilio): Why only for HTML editors? All content from the root
+      // content in text editors is also editable, so afaict we can remove the
+      // special-case.
       return false;
     }
-
-    // In HTML editors, if we're dealing with an element, then ask it
-    // whether it's editable.
-    if (aContent.IsElement()) {
-      return aEditorType == EditorType::HTML ? aContent.IsEditable() : true;
-    }
-    // Text nodes are considered to be editable by both typed of editors.
-    return aContent.IsText();
+    return IsElementOrText(aContent);
   }
 
   /**
@@ -851,8 +1156,7 @@ class EditorUtils final {
     if (aContent.IsText()) {
       return true;
     }
-    return aContent.IsElement() &&
-           !EditorUtils::IsPaddingBRElementForEmptyEditor(aContent);
+    return aContent.IsElement() && !IsPaddingBRElementForEmptyEditor(aContent);
   }
 
   /**
@@ -893,8 +1197,10 @@ class EditorUtils final {
    * computed with nsFrameSelection that also requires flushed layout
    * information.
    */
+  template <typename SelectionOrAutoRangeArray>
   static bool IsFrameSelectionRequiredToExtendSelection(
-      nsIEditor::EDirection aDirectionAndAmount, Selection& aSelection) {
+      nsIEditor::EDirection aDirectionAndAmount,
+      SelectionOrAutoRangeArray& aSelectionOrAutoRangeArray) {
     switch (aDirectionAndAmount) {
       case nsIEditor::eNextWord:
       case nsIEditor::ePreviousWord:
@@ -903,11 +1209,24 @@ class EditorUtils final {
         return true;
       case nsIEditor::ePrevious:
       case nsIEditor::eNext:
-        return aSelection.IsCollapsed();
+        return aSelectionOrAutoRangeArray.IsCollapsed();
       default:
         return false;
     }
   }
+
+  /**
+   * Returns true if aSelection includes the point in aParentContent.
+   */
+  static bool IsPointInSelection(const Selection& aSelection,
+                                 const nsINode& aParentNode, uint32_t aOffset);
+
+  /**
+   * Create an nsITransferable instance which has kUnicodeMime and
+   * kMozTextInternal flavors.
+   */
+  static Result<nsCOMPtr<nsITransferable>, nsresult>
+  CreateTransferableForPlainText(const dom::Document& aDocument);
 };
 
 }  // namespace mozilla
