@@ -47,11 +47,8 @@ static nsresult GetListState(HTMLEditor* aHTMLEditor, bool* aMixed,
  *****************************************************************************/
 
 bool StateUpdatingCommandBase::IsCommandEnabled(Command aCommand,
-                                                TextEditor* aTextEditor) const {
-  if (!aTextEditor) {
-    return false;
-  }
-  HTMLEditor* htmlEditor = aTextEditor->AsHTMLEditor();
+                                                EditorBase* aEditorBase) const {
+  HTMLEditor* htmlEditor = HTMLEditor::GetFrom(aEditorBase);
   if (!htmlEditor) {
     return false;
   }
@@ -65,9 +62,9 @@ bool StateUpdatingCommandBase::IsCommandEnabled(Command aCommand,
 }
 
 nsresult StateUpdatingCommandBase::DoCommand(Command aCommand,
-                                             TextEditor& aTextEditor,
+                                             EditorBase& aEditorBase,
                                              nsIPrincipal* aPrincipal) const {
-  HTMLEditor* htmlEditor = aTextEditor.AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase.GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_ERROR_FAILURE;
   }
@@ -83,12 +80,12 @@ nsresult StateUpdatingCommandBase::DoCommand(Command aCommand,
 }
 
 nsresult StateUpdatingCommandBase::GetCommandStateParams(
-    Command aCommand, nsCommandParams& aParams, TextEditor* aTextEditor,
+    Command aCommand, nsCommandParams& aParams, EditorBase* aEditorBase,
     nsIEditingSession* aEditingSession) const {
-  if (!aTextEditor) {
+  if (!aEditorBase) {
     return NS_OK;
   }
-  HTMLEditor* htmlEditor = aTextEditor->AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase->GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_ERROR_FAILURE;
   }
@@ -110,11 +107,8 @@ nsresult StateUpdatingCommandBase::GetCommandStateParams(
 StaticRefPtr<PasteNoFormattingCommand> PasteNoFormattingCommand::sInstance;
 
 bool PasteNoFormattingCommand::IsCommandEnabled(Command aCommand,
-                                                TextEditor* aTextEditor) const {
-  if (!aTextEditor) {
-    return false;
-  }
-  HTMLEditor* htmlEditor = aTextEditor->AsHTMLEditor();
+                                                EditorBase* aEditorBase) const {
+  HTMLEditor* htmlEditor = HTMLEditor::GetFrom(aEditorBase);
   if (!htmlEditor) {
     return false;
   }
@@ -122,9 +116,9 @@ bool PasteNoFormattingCommand::IsCommandEnabled(Command aCommand,
 }
 
 nsresult PasteNoFormattingCommand::DoCommand(Command aCommand,
-                                             TextEditor& aTextEditor,
+                                             EditorBase& aEditorBase,
                                              nsIPrincipal* aPrincipal) const {
-  HTMLEditor* htmlEditor = aTextEditor.AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase.GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_ERROR_FAILURE;
   }
@@ -138,10 +132,10 @@ nsresult PasteNoFormattingCommand::DoCommand(Command aCommand,
 }
 
 nsresult PasteNoFormattingCommand::GetCommandStateParams(
-    Command aCommand, nsCommandParams& aParams, TextEditor* aTextEditor,
+    Command aCommand, nsCommandParams& aParams, EditorBase* aEditorBase,
     nsIEditingSession* aEditingSession) const {
   return aParams.SetBool(STATE_ENABLED,
-                         IsCommandEnabled(aCommand, aTextEditor));
+                         IsCommandEnabled(aCommand, aEditorBase));
 }
 
 /*****************************************************************************
@@ -162,7 +156,7 @@ nsresult StyleUpdatingCommand::GetCurrentState(nsAtom* aTagName,
   bool allOfSelectionHasProp = false;
 
   nsresult rv = aHTMLEditor->GetInlineProperty(
-      aTagName, nullptr, EmptyString(), &firstOfSelectionHasProp,
+      aTagName, nullptr, u""_ns, &firstOfSelectionHasProp,
       &anyOfSelectionHasProp, &allOfSelectionHasProp);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
                        "HTMLEditor::GetInlineProperty() failed");
@@ -209,8 +203,8 @@ nsresult StyleUpdatingCommand::ToggleState(nsStaticAtom& aTagName,
     return rv;
   }
 
-  nsresult rv = aHTMLEditor.SetInlinePropertyAsAction(
-      aTagName, nullptr, EmptyString(), aPrincipal);
+  nsresult rv = aHTMLEditor.SetInlinePropertyAsAction(aTagName, nullptr, u""_ns,
+                                                      aPrincipal);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
                        "HTMLEditor::SetInlinePropertyAsAction() failed");
   return rv;
@@ -268,8 +262,7 @@ nsresult ListCommand::ToggleState(nsStaticAtom& aTagName,
   }
 
   rv = aHTMLEditor.MakeOrChangeListAsAction(
-      aTagName, EmptyString(), HTMLEditor::SelectAllOfCurrentList::No,
-      aPrincipal);
+      aTagName, u""_ns, HTMLEditor::SelectAllOfCurrentList::No, aPrincipal);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
                        "HTMLEditor::MakeOrChangeListAsAction() failed");
   return rv;
@@ -362,12 +355,8 @@ nsresult ListItemCommand::ToggleState(nsStaticAtom& aTagName,
 StaticRefPtr<RemoveListCommand> RemoveListCommand::sInstance;
 
 bool RemoveListCommand::IsCommandEnabled(Command aCommand,
-                                         TextEditor* aTextEditor) const {
-  if (!aTextEditor) {
-    return false;
-  }
-
-  HTMLEditor* htmlEditor = aTextEditor->AsHTMLEditor();
+                                         EditorBase* aEditorBase) const {
+  HTMLEditor* htmlEditor = HTMLEditor::GetFrom(aEditorBase);
   if (!htmlEditor) {
     return false;
   }
@@ -384,25 +373,25 @@ bool RemoveListCommand::IsCommandEnabled(Command aCommand,
   return NS_SUCCEEDED(rv) && (bMixed || !localName.IsEmpty());
 }
 
-nsresult RemoveListCommand::DoCommand(Command aCommand, TextEditor& aTextEditor,
+nsresult RemoveListCommand::DoCommand(Command aCommand, EditorBase& aEditorBase,
                                       nsIPrincipal* aPrincipal) const {
-  HTMLEditor* htmlEditor = aTextEditor.AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase.GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_OK;
   }
   // This removes any list type
   nsresult rv =
-      MOZ_KnownLive(htmlEditor)->RemoveListAsAction(EmptyString(), aPrincipal);
+      MOZ_KnownLive(htmlEditor)->RemoveListAsAction(u""_ns, aPrincipal);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
                        "HTMLEditor::RemoveListAsAction() failed");
   return rv;
 }
 
 nsresult RemoveListCommand::GetCommandStateParams(
-    Command aCommand, nsCommandParams& aParams, TextEditor* aTextEditor,
+    Command aCommand, nsCommandParams& aParams, EditorBase* aEditorBase,
     nsIEditingSession* aEditingSession) const {
   return aParams.SetBool(STATE_ENABLED,
-                         IsCommandEnabled(aCommand, aTextEditor));
+                         IsCommandEnabled(aCommand, aEditorBase));
 }
 
 /*****************************************************************************
@@ -412,20 +401,17 @@ nsresult RemoveListCommand::GetCommandStateParams(
 StaticRefPtr<IndentCommand> IndentCommand::sInstance;
 
 bool IndentCommand::IsCommandEnabled(Command aCommand,
-                                     TextEditor* aTextEditor) const {
-  if (!aTextEditor) {
-    return false;
-  }
-  HTMLEditor* htmlEditor = aTextEditor->AsHTMLEditor();
+                                     EditorBase* aEditorBase) const {
+  HTMLEditor* htmlEditor = HTMLEditor::GetFrom(aEditorBase);
   if (!htmlEditor) {
     return false;
   }
   return htmlEditor->IsSelectionEditable();
 }
 
-nsresult IndentCommand::DoCommand(Command aCommand, TextEditor& aTextEditor,
+nsresult IndentCommand::DoCommand(Command aCommand, EditorBase& aEditorBase,
                                   nsIPrincipal* aPrincipal) const {
-  HTMLEditor* htmlEditor = aTextEditor.AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase.GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_OK;
   }
@@ -435,10 +421,10 @@ nsresult IndentCommand::DoCommand(Command aCommand, TextEditor& aTextEditor,
 }
 
 nsresult IndentCommand::GetCommandStateParams(
-    Command aCommand, nsCommandParams& aParams, TextEditor* aTextEditor,
+    Command aCommand, nsCommandParams& aParams, EditorBase* aEditorBase,
     nsIEditingSession* aEditingSession) const {
   return aParams.SetBool(STATE_ENABLED,
-                         IsCommandEnabled(aCommand, aTextEditor));
+                         IsCommandEnabled(aCommand, aEditorBase));
 }
 
 /*****************************************************************************
@@ -448,20 +434,17 @@ nsresult IndentCommand::GetCommandStateParams(
 StaticRefPtr<OutdentCommand> OutdentCommand::sInstance;
 
 bool OutdentCommand::IsCommandEnabled(Command aCommand,
-                                      TextEditor* aTextEditor) const {
-  if (!aTextEditor) {
-    return false;
-  }
-  HTMLEditor* htmlEditor = aTextEditor->AsHTMLEditor();
+                                      EditorBase* aEditorBase) const {
+  HTMLEditor* htmlEditor = HTMLEditor::GetFrom(aEditorBase);
   if (!htmlEditor) {
     return false;
   }
   return htmlEditor->IsSelectionEditable();
 }
 
-nsresult OutdentCommand::DoCommand(Command aCommand, TextEditor& aTextEditor,
+nsresult OutdentCommand::DoCommand(Command aCommand, EditorBase& aEditorBase,
                                    nsIPrincipal* aPrincipal) const {
-  HTMLEditor* htmlEditor = aTextEditor.AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase.GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_OK;
   }
@@ -472,10 +455,10 @@ nsresult OutdentCommand::DoCommand(Command aCommand, TextEditor& aTextEditor,
 }
 
 nsresult OutdentCommand::GetCommandStateParams(
-    Command aCommand, nsCommandParams& aParams, TextEditor* aTextEditor,
+    Command aCommand, nsCommandParams& aParams, EditorBase* aEditorBase,
     nsIEditingSession* aEditingSession) const {
   return aParams.SetBool(STATE_ENABLED,
-                         IsCommandEnabled(aCommand, aTextEditor));
+                         IsCommandEnabled(aCommand, aEditorBase));
 }
 
 /*****************************************************************************
@@ -483,11 +466,8 @@ nsresult OutdentCommand::GetCommandStateParams(
  *****************************************************************************/
 
 bool MultiStateCommandBase::IsCommandEnabled(Command aCommand,
-                                             TextEditor* aTextEditor) const {
-  if (!aTextEditor) {
-    return false;
-  }
-  HTMLEditor* htmlEditor = aTextEditor->AsHTMLEditor();
+                                             EditorBase* aEditorBase) const {
+  HTMLEditor* htmlEditor = HTMLEditor::GetFrom(aEditorBase);
   if (!htmlEditor) {
     return false;
   }
@@ -496,7 +476,7 @@ bool MultiStateCommandBase::IsCommandEnabled(Command aCommand,
 }
 
 nsresult MultiStateCommandBase::DoCommand(Command aCommand,
-                                          TextEditor& aTextEditor,
+                                          EditorBase& aEditorBase,
                                           nsIPrincipal* aPrincipal) const {
   NS_WARNING(
       "who is calling MultiStateCommandBase::DoCommand (no implementation)?");
@@ -505,12 +485,12 @@ nsresult MultiStateCommandBase::DoCommand(Command aCommand,
 
 nsresult MultiStateCommandBase::DoCommandParam(Command aCommand,
                                                const nsAString& aStringParam,
-                                               TextEditor& aTextEditor,
+                                               EditorBase& aEditorBase,
                                                nsIPrincipal* aPrincipal) const {
   NS_WARNING_ASSERTION(aCommand != Command::FormatJustify,
                        "Command::FormatJustify should be used only for "
                        "IsCommandEnabled() and GetCommandStateParams()");
-  HTMLEditor* htmlEditor = aTextEditor.AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase.GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_ERROR_FAILURE;
   }
@@ -521,12 +501,12 @@ nsresult MultiStateCommandBase::DoCommandParam(Command aCommand,
 }
 
 nsresult MultiStateCommandBase::GetCommandStateParams(
-    Command aCommand, nsCommandParams& aParams, TextEditor* aTextEditor,
+    Command aCommand, nsCommandParams& aParams, EditorBase* aEditorBase,
     nsIEditingSession* aEditingSession) const {
-  if (!aTextEditor) {
+  if (!aEditorBase) {
     return NS_OK;
   }
-  HTMLEditor* htmlEditor = aTextEditor->AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase->GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_ERROR_FAILURE;
   }
@@ -557,7 +537,7 @@ nsresult ParagraphStateCommand::GetCurrentState(
   aParams.SetBool(STATE_MIXED, state.IsMixed());
   if (NS_WARN_IF(!state.GetFirstParagraphStateAtSelection())) {
     // XXX This is odd behavior, we should fix this later.
-    aParams.SetCString(STATE_ATTRIBUTE, NS_LITERAL_CSTRING("x"));
+    aParams.SetCString(STATE_ATTRIBUTE, "x"_ns);
   } else {
     nsCString paragraphState;  // Don't use `nsAutoCString` for avoiding copy.
     state.GetFirstParagraphStateAtSelection()->ToUTF8String(paragraphState);
@@ -641,8 +621,8 @@ nsresult FontSizeStateCommand::GetCurrentState(HTMLEditor* aHTMLEditor,
   nsAutoString outStateString;
   bool firstHas, anyHas, allHas;
   nsresult rv = aHTMLEditor->GetInlinePropertyWithAttrValue(
-      nsGkAtoms::font, nsGkAtoms::size, EmptyString(), &firstHas, &anyHas,
-      &allHas, outStateString);
+      nsGkAtoms::font, nsGkAtoms::size, u""_ns, &firstHas, &anyHas, &allHas,
+      outStateString);
   if (NS_FAILED(rv)) {
     NS_WARNING(
         "HTMLEditor::GetInlinePropertyWithAttrValue(nsGkAtoms::font, "
@@ -859,7 +839,7 @@ nsresult AlignCommand::GetCurrentState(HTMLEditor* aHTMLEditor,
       // make sense.  Additionally, WPT loves our behavior.
       error.SuppressException();
       aParams.SetBool(STATE_MIXED, false);
-      aParams.SetCString(STATE_ATTRIBUTE, EmptyCString());
+      aParams.SetCString(STATE_ATTRIBUTE, ""_ns);
       return NS_OK;
     }
     NS_WARNING("AlignStateAtSelection failed");
@@ -911,15 +891,14 @@ nsresult AbsolutePositioningCommand::GetCurrentState(
 
   if (!aHTMLEditor->IsAbsolutePositionEditorEnabled()) {
     aParams.SetBool(STATE_MIXED, false);
-    aParams.SetCString(STATE_ATTRIBUTE, EmptyCString());
+    aParams.SetCString(STATE_ATTRIBUTE, ""_ns);
     return NS_OK;
   }
 
   RefPtr<Element> container =
       aHTMLEditor->GetAbsolutelyPositionedSelectionContainer();
   aParams.SetBool(STATE_MIXED, false);
-  aParams.SetCString(STATE_ATTRIBUTE, container ? NS_LITERAL_CSTRING("absolute")
-                                                : EmptyCString());
+  aParams.SetCString(STATE_ATTRIBUTE, container ? "absolute"_ns : ""_ns);
   return NS_OK;
 }
 
@@ -943,11 +922,8 @@ nsresult AbsolutePositioningCommand::ToggleState(
 StaticRefPtr<DecreaseZIndexCommand> DecreaseZIndexCommand::sInstance;
 
 bool DecreaseZIndexCommand::IsCommandEnabled(Command aCommand,
-                                             TextEditor* aTextEditor) const {
-  if (!aTextEditor) {
-    return false;
-  }
-  RefPtr<HTMLEditor> htmlEditor = aTextEditor->AsHTMLEditor();
+                                             EditorBase* aEditorBase) const {
+  RefPtr<HTMLEditor> htmlEditor = HTMLEditor::GetFrom(aEditorBase);
   if (!htmlEditor) {
     return false;
   }
@@ -962,9 +938,9 @@ bool DecreaseZIndexCommand::IsCommandEnabled(Command aCommand,
 }
 
 nsresult DecreaseZIndexCommand::DoCommand(Command aCommand,
-                                          TextEditor& aTextEditor,
+                                          EditorBase& aEditorBase,
                                           nsIPrincipal* aPrincipal) const {
-  HTMLEditor* htmlEditor = aTextEditor.AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase.GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_ERROR_FAILURE;
   }
@@ -975,10 +951,10 @@ nsresult DecreaseZIndexCommand::DoCommand(Command aCommand,
 }
 
 nsresult DecreaseZIndexCommand::GetCommandStateParams(
-    Command aCommand, nsCommandParams& aParams, TextEditor* aTextEditor,
+    Command aCommand, nsCommandParams& aParams, EditorBase* aEditorBase,
     nsIEditingSession* aEditingSession) const {
   return aParams.SetBool(STATE_ENABLED,
-                         IsCommandEnabled(aCommand, aTextEditor));
+                         IsCommandEnabled(aCommand, aEditorBase));
 }
 
 /*****************************************************************************
@@ -988,11 +964,8 @@ nsresult DecreaseZIndexCommand::GetCommandStateParams(
 StaticRefPtr<IncreaseZIndexCommand> IncreaseZIndexCommand::sInstance;
 
 bool IncreaseZIndexCommand::IsCommandEnabled(Command aCommand,
-                                             TextEditor* aTextEditor) const {
-  if (!aTextEditor) {
-    return false;
-  }
-  HTMLEditor* htmlEditor = aTextEditor->AsHTMLEditor();
+                                             EditorBase* aEditorBase) const {
+  HTMLEditor* htmlEditor = HTMLEditor::GetFrom(aEditorBase);
   if (!htmlEditor) {
     return false;
   }
@@ -1003,9 +976,9 @@ bool IncreaseZIndexCommand::IsCommandEnabled(Command aCommand,
 }
 
 nsresult IncreaseZIndexCommand::DoCommand(Command aCommand,
-                                          TextEditor& aTextEditor,
+                                          EditorBase& aEditorBase,
                                           nsIPrincipal* aPrincipal) const {
-  HTMLEditor* htmlEditor = aTextEditor.AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase.GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_ERROR_FAILURE;
   }
@@ -1016,10 +989,10 @@ nsresult IncreaseZIndexCommand::DoCommand(Command aCommand,
 }
 
 nsresult IncreaseZIndexCommand::GetCommandStateParams(
-    Command aCommand, nsCommandParams& aParams, TextEditor* aTextEditor,
+    Command aCommand, nsCommandParams& aParams, EditorBase* aEditorBase,
     nsIEditingSession* aEditingSession) const {
   return aParams.SetBool(STATE_ENABLED,
-                         IsCommandEnabled(aCommand, aTextEditor));
+                         IsCommandEnabled(aCommand, aEditorBase));
 }
 
 /*****************************************************************************
@@ -1029,11 +1002,8 @@ nsresult IncreaseZIndexCommand::GetCommandStateParams(
 StaticRefPtr<RemoveStylesCommand> RemoveStylesCommand::sInstance;
 
 bool RemoveStylesCommand::IsCommandEnabled(Command aCommand,
-                                           TextEditor* aTextEditor) const {
-  if (!aTextEditor) {
-    return false;
-  }
-  HTMLEditor* htmlEditor = aTextEditor->AsHTMLEditor();
+                                           EditorBase* aEditorBase) const {
+  HTMLEditor* htmlEditor = HTMLEditor::GetFrom(aEditorBase);
   if (!htmlEditor) {
     return false;
   }
@@ -1042,9 +1012,9 @@ bool RemoveStylesCommand::IsCommandEnabled(Command aCommand,
 }
 
 nsresult RemoveStylesCommand::DoCommand(Command aCommand,
-                                        TextEditor& aTextEditor,
+                                        EditorBase& aEditorBase,
                                         nsIPrincipal* aPrincipal) const {
-  HTMLEditor* htmlEditor = aTextEditor.AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase.GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_OK;
   }
@@ -1057,10 +1027,10 @@ nsresult RemoveStylesCommand::DoCommand(Command aCommand,
 }
 
 nsresult RemoveStylesCommand::GetCommandStateParams(
-    Command aCommand, nsCommandParams& aParams, TextEditor* aTextEditor,
+    Command aCommand, nsCommandParams& aParams, EditorBase* aEditorBase,
     nsIEditingSession* aEditingSession) const {
   return aParams.SetBool(STATE_ENABLED,
-                         IsCommandEnabled(aCommand, aTextEditor));
+                         IsCommandEnabled(aCommand, aEditorBase));
 }
 
 /*****************************************************************************
@@ -1070,11 +1040,8 @@ nsresult RemoveStylesCommand::GetCommandStateParams(
 StaticRefPtr<IncreaseFontSizeCommand> IncreaseFontSizeCommand::sInstance;
 
 bool IncreaseFontSizeCommand::IsCommandEnabled(Command aCommand,
-                                               TextEditor* aTextEditor) const {
-  if (!aTextEditor) {
-    return false;
-  }
-  HTMLEditor* htmlEditor = aTextEditor->AsHTMLEditor();
+                                               EditorBase* aEditorBase) const {
+  HTMLEditor* htmlEditor = HTMLEditor::GetFrom(aEditorBase);
   if (!htmlEditor) {
     return false;
   }
@@ -1083,9 +1050,9 @@ bool IncreaseFontSizeCommand::IsCommandEnabled(Command aCommand,
 }
 
 nsresult IncreaseFontSizeCommand::DoCommand(Command aCommand,
-                                            TextEditor& aTextEditor,
+                                            EditorBase& aEditorBase,
                                             nsIPrincipal* aPrincipal) const {
-  HTMLEditor* htmlEditor = aTextEditor.AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase.GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_OK;
   }
@@ -1096,10 +1063,10 @@ nsresult IncreaseFontSizeCommand::DoCommand(Command aCommand,
 }
 
 nsresult IncreaseFontSizeCommand::GetCommandStateParams(
-    Command aCommand, nsCommandParams& aParams, TextEditor* aTextEditor,
+    Command aCommand, nsCommandParams& aParams, EditorBase* aEditorBase,
     nsIEditingSession* aEditingSession) const {
   return aParams.SetBool(STATE_ENABLED,
-                         IsCommandEnabled(aCommand, aTextEditor));
+                         IsCommandEnabled(aCommand, aEditorBase));
 }
 
 /*****************************************************************************
@@ -1109,11 +1076,8 @@ nsresult IncreaseFontSizeCommand::GetCommandStateParams(
 StaticRefPtr<DecreaseFontSizeCommand> DecreaseFontSizeCommand::sInstance;
 
 bool DecreaseFontSizeCommand::IsCommandEnabled(Command aCommand,
-                                               TextEditor* aTextEditor) const {
-  if (!aTextEditor) {
-    return false;
-  }
-  HTMLEditor* htmlEditor = aTextEditor->AsHTMLEditor();
+                                               EditorBase* aEditorBase) const {
+  HTMLEditor* htmlEditor = HTMLEditor::GetFrom(aEditorBase);
   if (!htmlEditor) {
     return false;
   }
@@ -1122,9 +1086,9 @@ bool DecreaseFontSizeCommand::IsCommandEnabled(Command aCommand,
 }
 
 nsresult DecreaseFontSizeCommand::DoCommand(Command aCommand,
-                                            TextEditor& aTextEditor,
+                                            EditorBase& aEditorBase,
                                             nsIPrincipal* aPrincipal) const {
-  HTMLEditor* htmlEditor = aTextEditor.AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase.GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_OK;
   }
@@ -1135,10 +1099,10 @@ nsresult DecreaseFontSizeCommand::DoCommand(Command aCommand,
 }
 
 nsresult DecreaseFontSizeCommand::GetCommandStateParams(
-    Command aCommand, nsCommandParams& aParams, TextEditor* aTextEditor,
+    Command aCommand, nsCommandParams& aParams, EditorBase* aEditorBase,
     nsIEditingSession* aEditingSession) const {
   return aParams.SetBool(STATE_ENABLED,
-                         IsCommandEnabled(aCommand, aTextEditor));
+                         IsCommandEnabled(aCommand, aEditorBase));
 }
 
 /*****************************************************************************
@@ -1148,28 +1112,25 @@ nsresult DecreaseFontSizeCommand::GetCommandStateParams(
 StaticRefPtr<InsertHTMLCommand> InsertHTMLCommand::sInstance;
 
 bool InsertHTMLCommand::IsCommandEnabled(Command aCommand,
-                                         TextEditor* aTextEditor) const {
-  if (!aTextEditor) {
-    return false;
-  }
-  HTMLEditor* htmlEditor = aTextEditor->AsHTMLEditor();
+                                         EditorBase* aEditorBase) const {
+  HTMLEditor* htmlEditor = HTMLEditor::GetFrom(aEditorBase);
   if (!htmlEditor) {
     return false;
   }
   return htmlEditor->IsSelectionEditable();
 }
 
-nsresult InsertHTMLCommand::DoCommand(Command aCommand, TextEditor& aTextEditor,
+nsresult InsertHTMLCommand::DoCommand(Command aCommand, EditorBase& aEditorBase,
                                       nsIPrincipal* aPrincipal) const {
   // If InsertHTMLCommand is called with no parameters, it was probably called
   // with an empty string parameter ''. In this case, it should act the same as
   // the delete command
-  HTMLEditor* htmlEditor = aTextEditor.AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase.GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_ERROR_FAILURE;
   }
   nsresult rv =
-      MOZ_KnownLive(htmlEditor)->InsertHTMLAsAction(EmptyString(), aPrincipal);
+      MOZ_KnownLive(htmlEditor)->InsertHTMLAsAction(u""_ns, aPrincipal);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
                        "HTMLEditor::InsertHTMLAsAction() failed");
   return rv;
@@ -1177,13 +1138,13 @@ nsresult InsertHTMLCommand::DoCommand(Command aCommand, TextEditor& aTextEditor,
 
 nsresult InsertHTMLCommand::DoCommandParam(Command aCommand,
                                            const nsAString& aStringParam,
-                                           TextEditor& aTextEditor,
+                                           EditorBase& aEditorBase,
                                            nsIPrincipal* aPrincipal) const {
   if (NS_WARN_IF(aStringParam.IsVoid())) {
     return NS_ERROR_INVALID_ARG;
   }
 
-  HTMLEditor* htmlEditor = aTextEditor.AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase.GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_ERROR_FAILURE;
   }
@@ -1195,10 +1156,10 @@ nsresult InsertHTMLCommand::DoCommandParam(Command aCommand,
 }
 
 nsresult InsertHTMLCommand::GetCommandStateParams(
-    Command aCommand, nsCommandParams& aParams, TextEditor* aTextEditor,
+    Command aCommand, nsCommandParams& aParams, EditorBase* aEditorBase,
     nsIEditingSession* aEditingSession) const {
   return aParams.SetBool(STATE_ENABLED,
-                         IsCommandEnabled(aCommand, aTextEditor));
+                         IsCommandEnabled(aCommand, aEditorBase));
 }
 
 /*****************************************************************************
@@ -1208,11 +1169,8 @@ nsresult InsertHTMLCommand::GetCommandStateParams(
 StaticRefPtr<InsertTagCommand> InsertTagCommand::sInstance;
 
 bool InsertTagCommand::IsCommandEnabled(Command aCommand,
-                                        TextEditor* aTextEditor) const {
-  if (!aTextEditor) {
-    return false;
-  }
-  HTMLEditor* htmlEditor = aTextEditor->AsHTMLEditor();
+                                        EditorBase* aEditorBase) const {
+  HTMLEditor* htmlEditor = HTMLEditor::GetFrom(aEditorBase);
   if (!htmlEditor) {
     return false;
   }
@@ -1220,14 +1178,14 @@ bool InsertTagCommand::IsCommandEnabled(Command aCommand,
 }
 
 // corresponding STATE_ATTRIBUTE is: src (img) and href (a)
-nsresult InsertTagCommand::DoCommand(Command aCommand, TextEditor& aTextEditor,
+nsresult InsertTagCommand::DoCommand(Command aCommand, EditorBase& aEditorBase,
                                      nsIPrincipal* aPrincipal) const {
   nsAtom* tagName = GetTagName(aCommand);
   if (NS_WARN_IF(tagName != nsGkAtoms::hr)) {
     return NS_ERROR_NOT_IMPLEMENTED;
   }
 
-  HTMLEditor* htmlEditor = aTextEditor.AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase.GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_ERROR_FAILURE;
   }
@@ -1248,7 +1206,7 @@ nsresult InsertTagCommand::DoCommand(Command aCommand, TextEditor& aTextEditor,
 
 nsresult InsertTagCommand::DoCommandParam(Command aCommand,
                                           const nsAString& aStringParam,
-                                          TextEditor& aTextEditor,
+                                          EditorBase& aEditorBase,
                                           nsIPrincipal* aPrincipal) const {
   MOZ_ASSERT(aCommand != Command::InsertHorizontalRule);
 
@@ -1260,7 +1218,7 @@ nsresult InsertTagCommand::DoCommandParam(Command aCommand,
     return NS_ERROR_UNEXPECTED;
   }
 
-  HTMLEditor* htmlEditor = aTextEditor.AsHTMLEditor();
+  HTMLEditor* htmlEditor = aEditorBase.GetAsHTMLEditor();
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_ERROR_FAILURE;
   }
@@ -1310,10 +1268,10 @@ nsresult InsertTagCommand::DoCommandParam(Command aCommand,
 }
 
 nsresult InsertTagCommand::GetCommandStateParams(
-    Command aCommand, nsCommandParams& aParams, TextEditor* aTextEditor,
+    Command aCommand, nsCommandParams& aParams, EditorBase* aEditorBase,
     nsIEditingSession* aEditingSession) const {
   return aParams.SetBool(STATE_ENABLED,
-                         IsCommandEnabled(aCommand, aTextEditor));
+                         IsCommandEnabled(aCommand, aEditorBase));
 }
 
 /*****************************************************************************
