@@ -20,16 +20,15 @@
 #include "nsHttpNegotiateAuth.h"
 
 #include "nsIHttpAuthenticableChannel.h"
-#include "nsIProxiedChannel.h"
 #include "nsIAuthModule.h"
-#include "nsIServiceManager.h"
-#include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
+#include "nsIPrefService.h"
 #include "nsIProxyInfo.h"
 #include "nsIURI.h"
 #include "nsCOMPtr.h"
 #include "nsString.h"
 #include "nsNetCID.h"
+#include "nsProxyRelease.h"
 #include "plbase64.h"
 #include "plstr.h"
 #include "mozilla/Base64.h"
@@ -50,6 +49,7 @@
 #include "nsUnicharUtils.h"
 #include "mozilla/net/HttpAuthUtils.h"
 #include "mozilla/ClearOnShutdown.h"
+#include "mozilla/net/DNS.h"
 
 using mozilla::Base64Decode;
 
@@ -307,6 +307,10 @@ class GetNextTokenCompleteEvent final : public nsIRunnable,
     MOZ_ASSERT(NS_IsMainThread());
 
     mCancelled = true;
+    nsCOMPtr<nsIHttpAuthenticatorCallback> callback = std::move(mCallback);
+    if (callback) {
+      callback->OnCredsGenerated(mCreds, mFlags, aReason, nullptr, nullptr);
+    }
     return NS_OK;
   }
 
@@ -326,18 +330,18 @@ NS_IMPL_ISUPPORTS(GetNextTokenCompleteEvent, nsIRunnable, nsICancelable)
 // GetNextTokenRunnable
 //
 // This runnable is created by GenerateCredentialsAsync and it runs
-// in nsHttpNegotiateAuth::mNegotiateThread and calling GenerateCredentials.
+// on the background thread pool and calls GenerateCredentials.
 //
 class GetNextTokenRunnable final : public mozilla::Runnable {
   ~GetNextTokenRunnable() override = default;
 
  public:
-  GetNextTokenRunnable(nsIHttpAuthenticableChannel* authChannel,
-                       const char* challenge, bool isProxyAuth,
-                       const char16_t* domain, const char16_t* username,
-                       const char16_t* password, nsISupports* sessionState,
-                       nsISupports* continuationState,
-                       GetNextTokenCompleteEvent* aCompleteEvent)
+  GetNextTokenRunnable(
+      nsMainThreadPtrHandle<nsIHttpAuthenticableChannel>& authChannel,
+      const char* challenge, bool isProxyAuth, const char16_t* domain,
+      const char16_t* username, const char16_t* password,
+      nsISupports* sessionState, nsISupports* continuationState,
+      GetNextTokenCompleteEvent* aCompleteEvent)
       : mozilla::Runnable("GetNextTokenRunnable"),
         mAuthChannel(authChannel),
         mChallenge(challenge),
@@ -407,7 +411,7 @@ class GetNextTokenRunnable final : public mozilla::Runnable {
   }
 
  private:
-  nsCOMPtr<nsIHttpAuthenticableChannel> mAuthChannel;
+  nsMainThreadPtrHandle<nsIHttpAuthenticableChannel> mAuthChannel;
   nsCString mChallenge;
   bool mIsProxyAuth;
   nsString mDomain;
@@ -433,20 +437,18 @@ nsHttpNegotiateAuth::GenerateCredentialsAsync(
   RefPtr<GetNextTokenCompleteEvent> cancelEvent =
       new GetNextTokenCompleteEvent(aCallback);
 
+  nsMainThreadPtrHandle<nsIHttpAuthenticableChannel> handle(
+      new nsMainThreadPtrHolder<nsIHttpAuthenticableChannel>(
+          "nsIHttpAuthenticableChannel", authChannel, false));
   nsCOMPtr<nsIRunnable> getNextTokenRunnable = new GetNextTokenRunnable(
-      authChannel, challenge, isProxyAuth, domain, username, password,
-      sessionState, continuationState, cancelEvent);
-  cancelEvent.forget(aCancelable);
+      handle, challenge, isProxyAuth, domain, username, password, sessionState,
+      continuationState, cancelEvent);
 
-  nsresult rv;
-  if (!mNegotiateThread) {
-    mNegotiateThread = new mozilla::LazyIdleThread(
-        DEFAULT_THREAD_TIMEOUT_MS, NS_LITERAL_CSTRING("NegotiateAuth"));
-    NS_ENSURE_TRUE(mNegotiateThread, NS_ERROR_OUT_OF_MEMORY);
-  }
-  rv = mNegotiateThread->Dispatch(getNextTokenRunnable, NS_DISPATCH_NORMAL);
+  nsresult rv = NS_DispatchBackgroundTask(
+      getNextTokenRunnable, nsIEventTarget::DISPATCH_EVENT_MAY_BLOCK);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  cancelEvent.forget(aCancelable);
   return NS_OK;
 }
 
@@ -497,8 +499,10 @@ nsHttpNegotiateAuth::GenerateCredentials(
     while (*challenge == ' ') challenge++;
     len = strlen(challenge);
 
+    if (!len) return NS_ERROR_UNEXPECTED;
+
     // strip off any padding (see bug 230351)
-    while (challenge[len - 1] == '=') len--;
+    while (len && challenge[len - 1] == '=') len--;
 
     //
     // Decode the response that followed the "Negotiate" token
@@ -561,13 +565,16 @@ bool nsHttpNegotiateAuth::TestBoolPref(const char* pref) {
 
 bool nsHttpNegotiateAuth::TestNonFqdn(nsIURI* uri) {
   nsAutoCString host;
-  PRNetAddr addr;
 
-  if (!TestBoolPref(kNegotiateAuthAllowNonFqdn)) return false;
+  if (!TestBoolPref(kNegotiateAuthAllowNonFqdn)) {
+    return false;
+  }
 
-  if (NS_FAILED(uri->GetAsciiHost(host))) return false;
+  if (NS_FAILED(uri->GetAsciiHost(host))) {
+    return false;
+  }
 
   // return true if host does not contain a dot and is not an ip address
   return !host.IsEmpty() && !host.Contains('.') &&
-         PR_StringToNetAddr(host.BeginReading(), &addr) != PR_SUCCESS;
+         !mozilla::net::HostIsIPLiteral(host);
 }
