@@ -13,15 +13,17 @@
 
 #include "mozilla/ComputedStyleInlines.h"
 #include "mozilla/PresShell.h"
+#include "mozilla/ProfilerLabels.h"
 #include "mozilla/RestyleManager.h"
 #include "nsGkAtoms.h"
 #include "nsIScrollableFrame.h"
-#include "nsSubDocumentFrame.h"
-#include "nsCanvasFrame.h"
 #include "nsAbsoluteContainingBlock.h"
-#include "GeckoProfiler.h"
+#include "nsCanvasFrame.h"
+#include "nsLayoutUtils.h"
+#include "nsSubDocumentFrame.h"
 #include "nsIMozBrowserFrame.h"
 #include "nsPlaceholderFrame.h"
+#include "MobileViewportManager.h"
 
 using namespace mozilla;
 typedef nsAbsoluteContainingBlock::AbsPosReflowFlags AbsPosReflowFlags;
@@ -55,12 +57,27 @@ void ViewportFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
   AUTO_PROFILER_LABEL("ViewportFrame::BuildDisplayList",
                       GRAPHICS_DisplayListBuilding);
 
-  if (nsIFrame* kid = mFrames.FirstChild()) {
-    // make the kid's BorderBackground our own. This ensures that the canvas
-    // frame's background becomes our own background and therefore appears
-    // below negative z-index elements.
-    BuildDisplayListForChild(aBuilder, kid, aLists);
+  nsIFrame* kid = mFrames.FirstChild();
+  if (!kid) {
+    return;
   }
+
+  nsDisplayListCollection set(aBuilder);
+  BuildDisplayListForChild(aBuilder, kid, set);
+
+  // If we have a scrollframe then it takes care of creating the display list
+  // for the top layer, but otherwise we need to do it here.
+  if (!kid->IsScrollFrame()) {
+    bool isOpaque = false;
+    if (auto* list = BuildDisplayListForTopLayer(aBuilder, &isOpaque)) {
+      if (isOpaque) {
+        set.DeleteAll(aBuilder);
+      }
+      set.PositionedDescendants()->AppendToTop(list);
+    }
+  }
+
+  set.MoveTo(aLists);
 }
 
 #ifdef DEBUG
@@ -68,7 +85,7 @@ void ViewportFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
  * Returns whether we are going to put an element in the top layer for
  * fullscreen. This function should matches the CSS rule in ua.css.
  */
-static bool ShouldInTopLayerForFullscreen(Element* aElement) {
+static bool ShouldInTopLayerForFullscreen(dom::Element* aElement) {
   if (!aElement->GetParent()) {
     return false;
   }
@@ -85,7 +102,7 @@ static void BuildDisplayListForTopLayerFrame(nsDisplayListBuilder* aBuilder,
                                              nsDisplayList* aList) {
   nsRect visible;
   nsRect dirty;
-  DisplayListClipState::AutoClipMultiple clipState(aBuilder);
+  DisplayListClipState::AutoSaveRestore clipState(aBuilder);
   nsDisplayListBuilder::AutoCurrentActiveScrolledRootSetter asrSetter(aBuilder);
   nsDisplayListBuilder::OutOfFlowDisplayData* savedOutOfFlowData =
       nsDisplayListBuilder::GetOutOfFlowData(aFrame);
@@ -101,8 +118,6 @@ static void BuildDisplayListForTopLayerFrame(nsDisplayListBuilder* aBuilder,
     // root scroll frame.
     clipState.SetClipChainForContainingBlockDescendants(
         savedOutOfFlowData->mCombinedClipChain);
-    clipState.ClipContainingBlockDescendantsExtra(
-        visible + aBuilder->ToReferenceFrame(aFrame), nullptr);
     asrSetter.SetCurrentActiveScrolledRoot(
         savedOutOfFlowData->mContainingBlockActiveScrolledRoot);
   }
@@ -114,61 +129,121 @@ static void BuildDisplayListForTopLayerFrame(nsDisplayListBuilder* aBuilder,
   aList->AppendToTop(&list);
 }
 
-void ViewportFrame::BuildDisplayListForTopLayer(nsDisplayListBuilder* aBuilder,
-                                                nsDisplayList* aList) {
-  nsTArray<Element*> fullscreenStack =
-      PresContext()->Document()->GetFullscreenStack();
-  for (Element* elem : fullscreenStack) {
-    if (nsIFrame* frame = elem->GetPrimaryFrame()) {
-      // There are two cases where an element in fullscreen is not in
-      // the top layer:
-      // 1. When building display list for purpose other than painting,
-      //    it is possible that there is inconsistency between the style
-      //    info and the content tree.
-      // 2. This is an element which we are not going to put in the top
-      //    layer for fullscreen. See ShouldInTopLayerForFullscreen().
-      // In both cases, we want to skip the frame here and paint it in
-      // the normal path.
-      if (frame->StyleDisplay()->mTopLayer == NS_STYLE_TOP_LAYER_NONE) {
-        MOZ_ASSERT(!aBuilder->IsForPainting() ||
-                   !ShouldInTopLayerForFullscreen(elem));
-        continue;
-      }
-      MOZ_ASSERT(ShouldInTopLayerForFullscreen(elem));
-      // Inner SVG, MathML elements, as well as children of some XUL
-      // elements are not allowed to be out-of-flow. They should not
-      // be handled as top layer element here.
-      if (!(frame->GetStateBits() & NS_FRAME_OUT_OF_FLOW)) {
-        MOZ_ASSERT(!elem->GetParent()->IsHTMLElement(),
-                   "HTML element should always be out-of-flow if in the top "
-                   "layer");
-        continue;
-      }
-      if (nsIFrame* backdropPh =
-              frame->GetChildList(kBackdropList).FirstChild()) {
-        MOZ_ASSERT(backdropPh->IsPlaceholderFrame());
-        MOZ_ASSERT(!backdropPh->GetNextSibling(), "more than one ::backdrop?");
-        MOZ_ASSERT(backdropPh->HasAnyStateBits(NS_FRAME_FIRST_REFLOW),
-                   "did you intend to reflow ::backdrop placeholders?");
-        nsIFrame* backdropFrame =
-            static_cast<nsPlaceholderFrame*>(backdropPh)->GetOutOfFlowFrame();
-        MOZ_ASSERT(backdropFrame);
-        BuildDisplayListForTopLayerFrame(aBuilder, backdropFrame, aList);
-      }
-      BuildDisplayListForTopLayerFrame(aBuilder, frame, aList);
+static bool BackdropListIsOpaque(ViewportFrame* aFrame,
+                                 nsDisplayListBuilder* aBuilder,
+                                 nsDisplayList* aList) {
+  // The common case for ::backdrop elements on the top layer is a single
+  // fixed position container, holding an opaque background color covering
+  // the whole viewport.
+  if (aList->Count() != 1 ||
+      aList->GetTop()->GetType() != DisplayItemType::TYPE_FIXED_POSITION) {
+    return false;
+  }
+
+  // Make sure the fixed position container isn't clipped or scrollable.
+  nsDisplayFixedPosition* fixed =
+      static_cast<nsDisplayFixedPosition*>(aList->GetTop());
+  if (fixed->GetActiveScrolledRoot() || fixed->GetClipChain()) {
+    return false;
+  }
+
+  nsDisplayList* children = fixed->GetChildren();
+  if (!children->GetTop() ||
+      children->GetTop()->GetType() != DisplayItemType::TYPE_BACKGROUND_COLOR) {
+    return false;
+  }
+
+  nsDisplayBackgroundColor* child =
+      static_cast<nsDisplayBackgroundColor*>(children->GetTop());
+  if (child->GetActiveScrolledRoot() || child->GetClipChain()) {
+    return false;
+  }
+
+  // Check that the background color is both opaque, and covering the
+  // whole viewport.
+  bool dummy;
+  nsRegion opaque = child->GetOpaqueRegion(aBuilder, &dummy);
+  return opaque.Contains(aFrame->GetRect());
+}
+
+nsDisplayWrapList* ViewportFrame::BuildDisplayListForTopLayer(
+    nsDisplayListBuilder* aBuilder, bool* aIsOpaque) {
+  nsDisplayList topLayerList;
+
+  nsTArray<dom::Element*> topLayer = PresContext()->Document()->GetTopLayer();
+  for (dom::Element* elem : topLayer) {
+    nsIFrame* frame = elem->GetPrimaryFrame();
+    if (!frame) {
+      continue;
     }
+    // There are two cases where an element in fullscreen is not in
+    // the top layer:
+    // 1. When building display list for purpose other than painting,
+    //    it is possible that there is inconsistency between the style
+    //    info and the content tree.
+    // 2. This is an element which we are not going to put in the top
+    //    layer for fullscreen. See ShouldInTopLayerForFullscreen().
+    // In both cases, we want to skip the frame here and paint it in
+    // the normal path.
+    if (frame->StyleDisplay()->mTopLayer == StyleTopLayer::None) {
+      MOZ_ASSERT(!aBuilder->IsForPainting() ||
+                 !ShouldInTopLayerForFullscreen(elem));
+      continue;
+    }
+    MOZ_ASSERT(ShouldInTopLayerForFullscreen(elem));
+    // Inner SVG, MathML elements, as well as children of some XUL
+    // elements are not allowed to be out-of-flow. They should not
+    // be handled as top layer element here.
+    if (!frame->HasAnyStateBits(NS_FRAME_OUT_OF_FLOW)) {
+      MOZ_ASSERT(!elem->GetParent()->IsHTMLElement(),
+                 "HTML element should always be out-of-flow if in the top "
+                 "layer");
+      continue;
+    }
+    if (nsIFrame* backdropPh =
+            frame->GetChildList(kBackdropList).FirstChild()) {
+      MOZ_ASSERT(!backdropPh->GetNextSibling(), "more than one ::backdrop?");
+      MOZ_ASSERT(backdropPh->HasAnyStateBits(NS_FRAME_FIRST_REFLOW),
+                 "did you intend to reflow ::backdrop placeholders?");
+      nsIFrame* backdropFrame =
+          nsPlaceholderFrame::GetRealFrameForPlaceholder(backdropPh);
+      BuildDisplayListForTopLayerFrame(aBuilder, backdropFrame, &topLayerList);
+
+      if (aIsOpaque) {
+        *aIsOpaque = BackdropListIsOpaque(this, aBuilder, &topLayerList);
+      }
+    }
+    BuildDisplayListForTopLayerFrame(aBuilder, frame, &topLayerList);
   }
 
   if (nsCanvasFrame* canvasFrame = PresShell()->GetCanvasFrame()) {
-    if (Element* container = canvasFrame->GetCustomContentContainer()) {
+    if (dom::Element* container = canvasFrame->GetCustomContentContainer()) {
       if (nsIFrame* frame = container->GetPrimaryFrame()) {
-        MOZ_ASSERT(frame->StyleDisplay()->mTopLayer != NS_STYLE_TOP_LAYER_NONE,
+        MOZ_ASSERT(frame->StyleDisplay()->mTopLayer != StyleTopLayer::None,
                    "ua.css should ensure this");
-        MOZ_ASSERT(frame->GetStateBits() & NS_FRAME_OUT_OF_FLOW);
-        BuildDisplayListForTopLayerFrame(aBuilder, frame, aList);
+        MOZ_ASSERT(frame->HasAnyStateBits(NS_FRAME_OUT_OF_FLOW));
+        BuildDisplayListForTopLayerFrame(aBuilder, frame, &topLayerList);
       }
     }
   }
+  if (topLayerList.IsEmpty()) {
+    return nullptr;
+  }
+  nsPoint offset = aBuilder->GetCurrentFrame()->GetOffsetTo(this);
+  nsDisplayListBuilder::AutoBuildingDisplayList buildingDisplayList(
+      aBuilder, this, aBuilder->GetVisibleRect() + offset,
+      aBuilder->GetDirtyRect() + offset);
+  // Wrap the whole top layer in a single item with maximum z-index,
+  // and append it at the very end, so that it stays at the topmost.
+  nsDisplayWrapList* wrapList = MakeDisplayItemWithIndex<nsDisplayWrapper>(
+      aBuilder, this, 2, &topLayerList, aBuilder->CurrentActiveScrolledRoot(),
+      false);
+  if (!wrapList) {
+    return nullptr;
+  }
+  wrapList->SetOverrideZIndex(
+      std::numeric_limits<decltype(wrapList->ZIndex())>::max());
+  return wrapList;
 }
 
 #ifdef DEBUG
@@ -179,10 +254,12 @@ void ViewportFrame::AppendFrames(ChildListID aListID, nsFrameList& aFrameList) {
 }
 
 void ViewportFrame::InsertFrames(ChildListID aListID, nsIFrame* aPrevFrame,
+                                 const nsLineList::iterator* aPrevFrameLine,
                                  nsFrameList& aFrameList) {
   NS_ASSERTION(aListID == kPrincipalList, "unexpected child list");
   NS_ASSERTION(GetChildList(aListID).IsEmpty(), "Shouldn't have any kids!");
-  nsContainerFrame::InsertFrames(aListID, aPrevFrame, aFrameList);
+  nsContainerFrame::InsertFrames(aListID, aPrevFrame, aPrevFrameLine,
+                                 aFrameList);
 }
 
 void ViewportFrame::RemoveFrame(ChildListID aListID, nsIFrame* aOldFrame) {
@@ -282,24 +359,24 @@ void ViewportFrame::Reflow(nsPresContext* aPresContext,
   if (mFrames.NotEmpty()) {
     // Deal with a non-incremental reflow or an incremental reflow
     // targeted at our one-and-only principal child frame.
-    if (aReflowInput.ShouldReflowAllKids() || aReflowInput.IsBResize() ||
-        NS_SUBTREE_DIRTY(mFrames.FirstChild())) {
+    if (aReflowInput.ShouldReflowAllKids() ||
+        mFrames.FirstChild()->IsSubtreeDirty()) {
       // Reflow our one-and-only principal child frame
       nsIFrame* kidFrame = mFrames.FirstChild();
       ReflowOutput kidDesiredSize(aReflowInput);
-      WritingMode wm = kidFrame->GetWritingMode();
-      LogicalSize availableSpace = aReflowInput.AvailableSize(wm);
+      const WritingMode kidWM = kidFrame->GetWritingMode();
+      LogicalSize availableSpace = aReflowInput.AvailableSize(kidWM);
       ReflowInput kidReflowInput(aPresContext, aReflowInput, kidFrame,
                                  availableSpace);
 
       // Reflow the frame
       kidReflowInput.SetComputedBSize(aReflowInput.ComputedBSize());
       ReflowChild(kidFrame, aPresContext, kidDesiredSize, kidReflowInput, 0, 0,
-                  0, aStatus);
+                  ReflowChildFlags::Default, aStatus);
       kidBSize = kidDesiredSize.BSize(wm);
 
-      FinishReflowChild(kidFrame, aPresContext, kidDesiredSize, nullptr, 0, 0,
-                        0);
+      FinishReflowChild(kidFrame, aPresContext, kidDesiredSize, &kidReflowInput,
+                        0, 0, ReflowChildFlags::Default);
     } else {
       kidBSize = LogicalSize(wm, mFrames.FirstChild()->GetSize()).BSize(wm);
     }
@@ -347,7 +424,7 @@ void ViewportFrame::Reflow(nsPresContext* aPresContext,
   }
 
   // If we were dirty then do a repaint
-  if (GetStateBits() & NS_FRAME_IS_DIRTY) {
+  if (HasAnyStateBits(NS_FRAME_IS_DIRTY)) {
     InvalidateFrame();
   }
 
@@ -385,9 +462,15 @@ nsSize ViewportFrame::AdjustViewportSizeForFixedPosition(
   // Layout fixed position elements to the visual viewport size if and only if
   // it has been set and it is larger than the computed size, otherwise use the
   // computed size.
-  if (presShell->IsVisualViewportSizeSet() &&
-      result < presShell->GetVisualViewportSize()) {
-    result = presShell->GetVisualViewportSize();
+  if (presShell->IsVisualViewportSizeSet()) {
+    if (presShell->GetDynamicToolbarState() == DynamicToolbarState::Collapsed &&
+        result < presShell->GetVisualViewportSizeUpdatedByDynamicToolbar()) {
+      // We need to use the viewport size updated by the dynamic toolbar in the
+      // case where the dynamic toolbar is completely hidden.
+      result = presShell->GetVisualViewportSizeUpdatedByDynamicToolbar();
+    } else if (result < presShell->GetVisualViewportSize()) {
+      result = presShell->GetVisualViewportSize();
+    }
   }
   // Expand the size to the layout viewport size if necessary.
   const nsSize layoutViewportSize = presShell->GetLayoutViewportSize();
@@ -400,6 +483,6 @@ nsSize ViewportFrame::AdjustViewportSizeForFixedPosition(
 
 #ifdef DEBUG_FRAME_DUMP
 nsresult ViewportFrame::GetFrameName(nsAString& aResult) const {
-  return MakeFrameName(NS_LITERAL_STRING("Viewport"), aResult);
+  return MakeFrameName(u"Viewport"_ns, aResult);
 }
 #endif
