@@ -34,7 +34,6 @@
 #include "mozilla/ServoStyleSetInlines.h"
 #include "mozilla/dom/Element.h"
 #include "nsDisplayList.h"
-#include "nsNodeUtils.h"
 #include "mozAutoDocUpdate.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/dom/ChildIterator.h"
@@ -134,7 +133,7 @@ class nsHTMLFramesetBlankFrame final : public nsLeafFrame {
 
 #ifdef DEBUG_FRAME_DUMP
   virtual nsresult GetFrameName(nsAString& aResult) const override {
-    return MakeFrameName(NS_LITERAL_STRING("FramesetBlank"), aResult);
+    return MakeFrameName(u"FramesetBlank"_ns, aResult);
   }
 #endif
 
@@ -183,7 +182,7 @@ nsHTMLFramesetFrame::nsHTMLFramesetFrame(ComputedStyle* aStyle,
   mEdgeColors.Set(NO_COLOR);
 }
 
-nsHTMLFramesetFrame::~nsHTMLFramesetFrame() {}
+nsHTMLFramesetFrame::~nsHTMLFramesetFrame() = default;
 
 NS_QUERYFRAME_HEAD(nsHTMLFramesetFrame)
   NS_QUERYFRAME_ENTRY(nsHTMLFramesetFrame)
@@ -286,7 +285,7 @@ void nsHTMLFramesetFrame::Init(nsIContent* aContent, nsContainerFrame* aParent,
     //
     // Maybe we should change that though.
     RefPtr<ComputedStyle> kidStyle =
-        presShell->StyleSet()->ResolveServoStyle(*child->AsElement());
+        ServoStyleSet::ResolveServoStyle(*child->AsElement());
     nsIFrame* frame;
     if (child->IsHTMLElement(nsGkAtoms::frameset)) {
       frame = NS_NewHTMLFramesetFrame(presShell, kidStyle);
@@ -316,8 +315,7 @@ void nsHTMLFramesetFrame::Init(nsIContent* aContent, nsContainerFrame* aParent,
   mNonBlankChildCount = mChildCount;
   // add blank frames for frameset cells that had no content provided
   for (int blankX = mChildCount; blankX < numCells; blankX++) {
-    RefPtr<ComputedStyle> pseudoComputedStyle;
-    pseudoComputedStyle =
+    RefPtr<ComputedStyle> pseudoComputedStyle =
         presShell->StyleSet()->ResolveNonInheritingAnonymousBoxStyle(
             PseudoStyleType::framesetBlank);
 
@@ -618,7 +616,7 @@ nsresult nsHTMLFramesetFrame::HandleEvent(nsPresContext* aPresContext,
         MouseDrag(aPresContext, aEvent);
         break;
       case eMouseUp:
-        if (aEvent->AsMouseEvent()->mButton == MouseButton::eLeft) {
+        if (aEvent->AsMouseEvent()->mButton == MouseButton::ePrimary) {
           EndMouseDrag(aPresContext);
         }
         break;
@@ -670,14 +668,14 @@ void nsHTMLFramesetFrame::ReflowPlaceChild(nsIFrame* aChild,
   nsReflowStatus status;
 
   ReflowChild(aChild, aPresContext, reflowOutput, reflowInput, aOffset.x,
-              aOffset.y, 0, status);
+              aOffset.y, ReflowChildFlags::Default, status);
   NS_ASSERTION(status.IsComplete(), "bad status");
 
   // Place and size the child
   reflowOutput.Width() = aSize.width;
   reflowOutput.Height() = aSize.height;
-  FinishReflowChild(aChild, aPresContext, reflowOutput, nullptr, aOffset.x,
-                    aOffset.y, 0);
+  FinishReflowChild(aChild, aPresContext, reflowOutput, &reflowInput, aOffset.x,
+                    aOffset.y, ReflowChildFlags::Default);
 }
 
 static nsFrameborder GetFrameBorderHelper(nsGenericHTMLElement* aContent) {
@@ -934,11 +932,10 @@ void nsHTMLFramesetFrame::Reflow(nsPresContext* aPresContext,
     if (firstTime) {
       int32_t childVis;
       nsHTMLFramesetFrame* framesetFrame = do_QueryFrame(child);
-      nsSubDocumentFrame* subdocFrame;
       if (framesetFrame) {
         childVis = framesetFrame->mEdgeVisibility;
         mChildBorderColors[childX] = framesetFrame->mEdgeColors;
-      } else if ((subdocFrame = do_QueryFrame(child))) {
+      } else if (child->IsSubDocumentFrame()) {
         if (eFrameborder_Yes == mChildFrameborder[childX]) {
           childVis = ALL_VIS;
         } else if (eFrameborder_No == mChildFrameborder[childX]) {
@@ -1059,7 +1056,7 @@ void nsHTMLFramesetFrame::Reflow(nsPresContext* aPresContext,
 
 #ifdef DEBUG_FRAME_DUMP
 nsresult nsHTMLFramesetFrame::GetFrameName(nsAString& aResult) const {
-  return MakeFrameName(NS_LITERAL_STRING("Frameset"), aResult);
+  return MakeFrameName(u"Frameset"_ns, aResult);
 }
 #endif
 
@@ -1335,11 +1332,7 @@ class nsDisplayFramesetBorder : public nsPaintedDisplayItem {
       : nsPaintedDisplayItem(aBuilder, aFrame) {
     MOZ_COUNT_CTOR(nsDisplayFramesetBorder);
   }
-#ifdef NS_BUILD_REFCNT_LOGGING
-  virtual ~nsDisplayFramesetBorder() {
-    MOZ_COUNT_DTOR(nsDisplayFramesetBorder);
-  }
-#endif
+  MOZ_COUNTED_DTOR_OVERRIDE(nsDisplayFramesetBorder)
 
   // REVIEW: see old GetFrameForPoint
   // Receives events in its bounds
@@ -1370,17 +1363,17 @@ void nsHTMLFramesetBorderFrame::PaintBorder(DrawTarget* aDrawTarget,
 
   if (widthInPixels <= 0) return;
 
-  ColorPattern bgColor(ToDeviceColor(LookAndFeel::GetColor(
-      LookAndFeel::ColorID::WidgetBackground, NS_RGB(200, 200, 200))));
+  ColorPattern bgColor(ToDeviceColor(LookAndFeel::Color(
+      LookAndFeel::ColorID::WidgetBackground, this, NS_RGB(200, 200, 200))));
 
-  ColorPattern fgColor(ToDeviceColor(LookAndFeel::GetColor(
-      LookAndFeel::ColorID::WidgetForeground, NS_RGB(0, 0, 0))));
+  ColorPattern fgColor(ToDeviceColor(LookAndFeel::Color(
+      LookAndFeel::ColorID::WidgetForeground, this, NS_RGB(0, 0, 0))));
 
-  ColorPattern hltColor(ToDeviceColor(LookAndFeel::GetColor(
-      LookAndFeel::ColorID::Widget3DHighlight, NS_RGB(255, 255, 255))));
+  ColorPattern hltColor(ToDeviceColor(LookAndFeel::Color(
+      LookAndFeel::ColorID::Widget3DHighlight, this, NS_RGB(255, 255, 255))));
 
-  ColorPattern sdwColor(ToDeviceColor(LookAndFeel::GetColor(
-      LookAndFeel::ColorID::Widget3DShadow, NS_RGB(128, 128, 128))));
+  ColorPattern sdwColor(ToDeviceColor(LookAndFeel::Color(
+      LookAndFeel::ColorID::Widget3DShadow, this, NS_RGB(128, 128, 128))));
 
   ColorPattern color(ToDeviceColor(NS_RGB(255, 255, 255)));  // default to white
   if (mVisibility) {
@@ -1454,7 +1447,7 @@ nsresult nsHTMLFramesetBorderFrame::HandleEvent(nsPresContext* aPresContext,
   }
 
   if (aEvent->mMessage == eMouseDown &&
-      aEvent->AsMouseEvent()->mButton == MouseButton::eLeft) {
+      aEvent->AsMouseEvent()->mButton == MouseButton::ePrimary) {
     nsHTMLFramesetFrame* parentFrame = do_QueryFrame(GetParent());
     if (parentFrame) {
       parentFrame->StartMouseDrag(aPresContext, this, aEvent);
@@ -1474,7 +1467,7 @@ Maybe<nsIFrame::Cursor> nsHTMLFramesetBorderFrame::GetCursor(const nsPoint&) {
 
 #ifdef DEBUG_FRAME_DUMP
 nsresult nsHTMLFramesetBorderFrame::GetFrameName(nsAString& aResult) const {
-  return MakeFrameName(NS_LITERAL_STRING("FramesetBorder"), aResult);
+  return MakeFrameName(u"FramesetBorder"_ns, aResult);
 }
 #endif
 
@@ -1522,9 +1515,7 @@ class nsDisplayFramesetBlank : public nsPaintedDisplayItem {
       : nsPaintedDisplayItem(aBuilder, aFrame) {
     MOZ_COUNT_CTOR(nsDisplayFramesetBlank);
   }
-#ifdef NS_BUILD_REFCNT_LOGGING
-  virtual ~nsDisplayFramesetBlank() { MOZ_COUNT_DTOR(nsDisplayFramesetBlank); }
-#endif
+  MOZ_COUNTED_DTOR_OVERRIDE(nsDisplayFramesetBlank)
 
   virtual void Paint(nsDisplayListBuilder* aBuilder, gfxContext* aCtx) override;
   NS_DISPLAY_DECL_NAME("FramesetBlank", TYPE_FRAMESET_BLANK)
@@ -1536,7 +1527,7 @@ void nsDisplayFramesetBlank::Paint(nsDisplayListBuilder* aBuilder,
   int32_t appUnitsPerDevPixel = mFrame->PresContext()->AppUnitsPerDevPixel();
   Rect rect =
       NSRectToSnappedRect(GetPaintRect(), appUnitsPerDevPixel, *drawTarget);
-  ColorPattern white(ToDeviceColor(Color(1.f, 1.f, 1.f, 1.f)));
+  ColorPattern white(ToDeviceColor(sRGBColor::OpaqueWhite()));
   drawTarget->FillRect(rect, white);
 }
 
