@@ -49,7 +49,7 @@ nsContainerFrame* NS_NewRubyBaseContainerFrame(PresShell* aPresShell,
 
 #ifdef DEBUG_FRAME_DUMP
 nsresult nsRubyBaseContainerFrame::GetFrameName(nsAString& aResult) const {
-  return MakeFrameName(NS_LITERAL_STRING("RubyBaseContainer"), aResult);
+  return MakeFrameName(u"RubyBaseContainer"_ns, aResult);
 }
 #endif
 
@@ -237,7 +237,8 @@ void nsRubyBaseContainerFrame::AddInlinePrefISize(
 
 /* virtual */
 bool nsRubyBaseContainerFrame::IsFrameOfType(uint32_t aFlags) const {
-  if (aFlags & (eSupportsCSSTransforms | eSupportsContainLayoutAndPaint)) {
+  if (aFlags & (eSupportsCSSTransforms | eSupportsContainLayoutAndPaint |
+                eSupportsAspectRatio)) {
     return false;
   }
   return nsContainerFrame::IsFrameOfType(aFlags &
@@ -248,14 +249,15 @@ bool nsRubyBaseContainerFrame::IsFrameOfType(uint32_t aFlags) const {
 bool nsRubyBaseContainerFrame::CanContinueTextRun() const { return true; }
 
 /* virtual */
-LogicalSize nsRubyBaseContainerFrame::ComputeSize(
+nsIFrame::SizeComputationResult nsRubyBaseContainerFrame::ComputeSize(
     gfxContext* aRenderingContext, WritingMode aWM, const LogicalSize& aCBSize,
     nscoord aAvailableISize, const LogicalSize& aMargin,
-    const LogicalSize& aBorder, const LogicalSize& aPadding,
+    const LogicalSize& aBorderPadding, const StyleSizeOverrides& aSizeOverrides,
     ComputeSizeFlags aFlags) {
   // Ruby base container frame is inline,
   // hence don't compute size before reflow.
-  return LogicalSize(aWM, NS_UNCONSTRAINEDSIZE, NS_UNCONSTRAINEDSIZE);
+  return {LogicalSize(aWM, NS_UNCONSTRAINEDSIZE, NS_UNCONSTRAINEDSIZE),
+          AspectRatioUsage::None};
 }
 
 /* virtual */
@@ -359,12 +361,13 @@ void nsRubyBaseContainerFrame::Reflow(nsPresContext* aPresContext,
   GetIsLineBreakAllowed(this, aReflowInput.mLineLayout->LineIsBreakable(),
                         &allowInitialLineBreak, &allowLineBreak);
 
-  nscoord isize = 0;
   // Reflow columns excluding any span
   RubyReflowInput reflowInput = {allowInitialLineBreak,
                                  allowLineBreak && !hasSpan, textContainers,
                                  aReflowInput, reflowInputs};
-  isize = ReflowColumns(reflowInput, aStatus);
+  aDesiredSize.BSize(lineWM) = 0;
+  aDesiredSize.SetBlockStartAscent(0);
+  nscoord isize = ReflowColumns(reflowInput, aDesiredSize, aStatus);
   DebugOnly<nscoord> lineSpanSize = aReflowInput.mLineLayout->EndSpan(this);
   aDesiredSize.ISize(lineWM) = isize;
   // When there are no frames inside the ruby base container, EndSpan
@@ -409,12 +412,18 @@ void nsRubyBaseContainerFrame::Reflow(nsPresContext* aPresContext,
     lineLayout->EndLineReflow();
   }
 
-  // Border and padding are suppressed on ruby base container,
-  // create a fake borderPadding for setting BSize.
-  WritingMode frameWM = aReflowInput.GetWritingMode();
-  LogicalMargin borderPadding(frameWM);
-  nsLayoutUtils::SetBSizeFromFontMetrics(this, aDesiredSize, borderPadding,
-                                         lineWM, frameWM);
+  // If this ruby base container is empty, size it as if there were
+  // an empty inline child inside.
+  if (mFrames.IsEmpty()) {
+    // Border and padding are suppressed on ruby base container, so we
+    // create a dummy zero-sized borderPadding for setting BSize.
+    WritingMode frameWM = aReflowInput.GetWritingMode();
+    LogicalMargin borderPadding(frameWM);
+    nsLayoutUtils::SetBSizeFromFontMetrics(this, aDesiredSize, borderPadding,
+                                           lineWM, frameWM);
+  }
+
+  ReflowAbsoluteFrames(aPresContext, aDesiredSize, aReflowInput, aStatus);
 }
 
 /**
@@ -432,7 +441,8 @@ struct MOZ_STACK_CLASS nsRubyBaseContainerFrame::PullFrameState {
 };
 
 nscoord nsRubyBaseContainerFrame::ReflowColumns(
-    const RubyReflowInput& aReflowInput, nsReflowStatus& aStatus) {
+    const RubyReflowInput& aReflowInput, ReflowOutput& aDesiredSize,
+    nsReflowStatus& aStatus) {
   nsLineLayout* lineLayout = aReflowInput.mBaseReflowInput.mLineLayout;
   const uint32_t rtcCount = aReflowInput.mTextContainers.Length();
   nscoord icoord = lineLayout->GetCurrentICoord();
@@ -446,7 +456,8 @@ nscoord nsRubyBaseContainerFrame::ReflowColumns(
   RubyColumnEnumerator e(this, aReflowInput.mTextContainers);
   for (; !e.AtEnd(); e.Next()) {
     e.GetColumn(column);
-    icoord += ReflowOneColumn(aReflowInput, columnIndex, column, reflowStatus);
+    icoord += ReflowOneColumn(aReflowInput, columnIndex, column, aDesiredSize,
+                              reflowStatus);
     if (!reflowStatus.IsInlineBreakBefore()) {
       columnIndex++;
     }
@@ -470,7 +481,8 @@ nscoord nsRubyBaseContainerFrame::ReflowColumns(
       // No more frames can be pulled.
       break;
     }
-    icoord += ReflowOneColumn(aReflowInput, columnIndex, column, reflowStatus);
+    icoord += ReflowOneColumn(aReflowInput, columnIndex, column, aDesiredSize,
+                              reflowStatus);
     if (!reflowStatus.IsInlineBreakBefore()) {
       columnIndex++;
     }
@@ -540,7 +552,8 @@ nscoord nsRubyBaseContainerFrame::ReflowColumns(
 
 nscoord nsRubyBaseContainerFrame::ReflowOneColumn(
     const RubyReflowInput& aReflowInput, uint32_t aColumnIndex,
-    const RubyColumn& aColumn, nsReflowStatus& aStatus) {
+    const RubyColumn& aColumn, ReflowOutput& aDesiredSize,
+    nsReflowStatus& aStatus) {
   const ReflowInput& baseReflowInput = aReflowInput.mBaseReflowInput;
   const auto& textReflowInputs = aReflowInput.mTextReflowInputs;
   nscoord istart = baseReflowInput.mLineLayout->GetCurrentICoord();
@@ -585,18 +598,23 @@ nscoord nsRubyBaseContainerFrame::ReflowOneColumn(
   for (uint32_t i = 0; i < rtcCount; i++) {
     nsRubyTextFrame* textFrame = aColumn.mTextFrames[i];
     if (textFrame) {
-      nsAutoString annotationText;
-      nsLayoutUtils::GetFrameTextContent(textFrame, annotationText);
-
-      // Per CSS Ruby spec, the content comparison for auto-hiding
-      // takes place prior to white spaces collapsing (white-space)
-      // and text transformation (text-transform), and ignores elements
-      // (considers only the textContent of the boxes). Which means
-      // using the content tree text comparison is correct.
-      if (annotationText.Equals(baseText)) {
-        textFrame->AddStateBits(NS_RUBY_TEXT_FRAME_AUTOHIDE);
+      bool isCollapsed = false;
+      if (textFrame->StyleVisibility()->mVisible == StyleVisibility::Collapse) {
+        isCollapsed = true;
       } else {
-        textFrame->RemoveStateBits(NS_RUBY_TEXT_FRAME_AUTOHIDE);
+        // Per CSS Ruby spec, the content comparison for auto-hiding
+        // takes place prior to white spaces collapsing (white-space)
+        // and text transformation (text-transform), and ignores elements
+        // (considers only the textContent of the boxes). Which means
+        // using the content tree text comparison is correct.
+        nsAutoString annotationText;
+        nsLayoutUtils::GetFrameTextContent(textFrame, annotationText);
+        isCollapsed = annotationText.Equals(baseText);
+      }
+      if (isCollapsed) {
+        textFrame->AddStateBits(NS_RUBY_TEXT_FRAME_COLLAPSED);
+      } else {
+        textFrame->RemoveStateBits(NS_RUBY_TEXT_FRAME_COLLAPSED);
       }
       RubyUtils::ClearReservedISize(textFrame);
 
@@ -624,8 +642,10 @@ nscoord nsRubyBaseContainerFrame::ReflowOneColumn(
     bool pushedFrame;
     nsReflowStatus reflowStatus;
     nsLineLayout* lineLayout = baseReflowInput.mLineLayout;
+    WritingMode lineWM = lineLayout->GetWritingMode();
     nscoord baseIStart = lineLayout->GetCurrentICoord();
-    lineLayout->ReflowFrame(aColumn.mBaseFrame, reflowStatus, nullptr,
+    ReflowOutput metrics(lineWM);
+    lineLayout->ReflowFrame(aColumn.mBaseFrame, reflowStatus, &metrics,
                             pushedFrame);
     if (MOZ_UNLIKELY(reflowStatus.IsInlineBreak() || pushedFrame)) {
       MOZ_ASSERT_UNREACHABLE(
@@ -636,6 +656,17 @@ nscoord nsRubyBaseContainerFrame::ReflowOneColumn(
     }
     nscoord baseISize = lineLayout->GetCurrentICoord() - baseIStart;
     columnISize = std::max(columnISize, baseISize);
+    // Calculate ascent & descent of the base frame and update desired
+    // size of this base container accordingly.
+    nscoord oldAscent = aDesiredSize.BlockStartAscent();
+    nscoord oldDescent = aDesiredSize.BSize(lineWM) - oldAscent;
+    nscoord baseAscent = metrics.BlockStartAscent();
+    nscoord baseDesent = metrics.BSize(lineWM) - baseAscent;
+    LogicalMargin margin = aColumn.mBaseFrame->GetLogicalUsedMargin(lineWM);
+    nscoord newAscent = std::max(baseAscent + margin.BStart(lineWM), oldAscent);
+    nscoord newDescent = std::max(baseDesent + margin.BEnd(lineWM), oldDescent);
+    aDesiredSize.SetBlockStartAscent(newAscent);
+    aDesiredSize.BSize(lineWM) = newAscent + newDescent;
   }
 
   // Align all the line layout to the new coordinate.
@@ -656,7 +687,7 @@ nscoord nsRubyBaseContainerFrame::ReflowOneColumn(
     nscoord deltaISize = icoord - lineLayout->GetCurrentICoord();
     if (deltaISize > 0) {
       lineLayout->AdvanceICoord(deltaISize);
-      if (textFrame && !textFrame->IsAutoHidden()) {
+      if (textFrame && !textFrame->IsCollapsed()) {
         RubyUtils::SetReservedISize(textFrame, deltaISize);
       }
     }
@@ -746,8 +777,7 @@ void nsRubyBaseContainerFrame::PullOneColumn(nsLineLayout* aLineLayout,
     MOZ_ASSERT(newFloatCB, "Must have a float containing block");
     if (oldFloatCB != newFloatCB) {
       for (nsIFrame* frame : aColumn) {
-        newFloatCB->ReparentFloats(frame, oldFloatCB, false,
-                                   ReparentingDirection::Backwards);
+        newFloatCB->ReparentFloats(frame, oldFloatCB, false);
       }
     }
   }
