@@ -12,9 +12,11 @@
 #include "mozilla/ComputedStyle.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/PresShell.h"
+#include "mozilla/StaticPrefs_layout.h"
 #include "mozilla/WritingModes.h"
 #include "nsLineLayout.h"
 #include "nsPresContext.h"
+#include "nsContainerFrameInlines.h"
 #include "nsRubyBaseContainerFrame.h"
 #include "nsRubyTextContainerFrame.h"
 
@@ -51,30 +53,32 @@ bool nsRubyFrame::IsFrameOfType(uint32_t aFlags) const {
 
 #ifdef DEBUG_FRAME_DUMP
 nsresult nsRubyFrame::GetFrameName(nsAString& aResult) const {
-  return MakeFrameName(NS_LITERAL_STRING("Ruby"), aResult);
+  return MakeFrameName(u"Ruby"_ns, aResult);
 }
 #endif
 
 /* virtual */
 void nsRubyFrame::AddInlineMinISize(gfxContext* aRenderingContext,
                                     nsIFrame::InlineMinISizeData* aData) {
-  for (nsIFrame* frame = this; frame; frame = frame->GetNextInFlow()) {
+  auto handleChildren = [aRenderingContext](auto frame, auto data) {
     for (RubySegmentEnumerator e(static_cast<nsRubyFrame*>(frame)); !e.AtEnd();
          e.Next()) {
-      e.GetBaseContainer()->AddInlineMinISize(aRenderingContext, aData);
+      e.GetBaseContainer()->AddInlineMinISize(aRenderingContext, data);
     }
-  }
+  };
+  DoInlineIntrinsicISize(aData, handleChildren);
 }
 
 /* virtual */
 void nsRubyFrame::AddInlinePrefISize(gfxContext* aRenderingContext,
                                      nsIFrame::InlinePrefISizeData* aData) {
-  for (nsIFrame* frame = this; frame; frame = frame->GetNextInFlow()) {
+  auto handleChildren = [aRenderingContext](auto frame, auto data) {
     for (RubySegmentEnumerator e(static_cast<nsRubyFrame*>(frame)); !e.AtEnd();
          e.Next()) {
-      e.GetBaseContainer()->AddInlinePrefISize(aRenderingContext, aData);
+      e.GetBaseContainer()->AddInlinePrefISize(aRenderingContext, data);
     }
-  }
+  };
+  DoInlineIntrinsicISize(aData, handleChildren);
   aData->mLineIsEmpty = false;
 }
 
@@ -112,33 +116,14 @@ void nsRubyFrame::Reflow(nsPresContext* aPresContext,
   // Clear leadings
   mLeadings.Reset();
 
-  // Since the ruby base container is going to reflow not only the ruby
-  // base frames, but also the ruby text frames, and then *afterwards*
-  // we're going to reflow the ruby text containers (which do not reflow
-  // their children), we need to transfer NS_FRAME_IS_DIRTY status from
-  // the ruby text containers to their child ruby texts now, both so
-  // that the ruby texts are marked dirty if needed, and so that the
-  // ruby text container doesn't mark the ruby text frames dirty *after*
-  // they're reflowed and leave dirty bits in a clean tree (suppressing
-  // future reflows, due to lack of a queued reflow to clean them).
-  for (nsIFrame* child : PrincipalChildList()) {
-    if (child->HasAnyStateBits(NS_FRAME_IS_DIRTY) &&
-        child->IsRubyTextContainerFrame()) {
-      for (nsIFrame* grandchild : child->PrincipalChildList()) {
-        grandchild->AddStateBits(NS_FRAME_IS_DIRTY);
-      }
-      // Replace NS_FRAME_IS_DIRTY with NS_FRAME_HAS_DIRTY_CHILDREN so
-      // we still have a dirty marking, but one that we won't transfer
-      // to children again.
-      child->RemoveStateBits(NS_FRAME_IS_DIRTY);
-      child->AddStateBits(NS_FRAME_HAS_DIRTY_CHILDREN);
-    }
-  }
-
   // Begin the span for the ruby frame
   WritingMode frameWM = aReflowInput.GetWritingMode();
   WritingMode lineWM = aReflowInput.mLineLayout->GetWritingMode();
-  LogicalMargin borderPadding = aReflowInput.ComputedLogicalBorderPadding();
+  LogicalMargin borderPadding =
+      aReflowInput.ComputedLogicalBorderPadding(frameWM);
+  nsLayoutUtils::SetBSizeFromFontMetrics(this, aDesiredSize, borderPadding,
+                                         lineWM, frameWM);
+
   nscoord startEdge = 0;
   const bool boxDecorationBreakClone =
       StyleBorder()->mBoxDecorationBreak == StyleBoxDecorationBreak::Clone;
@@ -147,13 +132,13 @@ void nsRubyFrame::Reflow(nsPresContext* aPresContext,
   }
   NS_ASSERTION(aReflowInput.AvailableISize() != NS_UNCONSTRAINEDSIZE,
                "should no longer use available widths");
-  nscoord availableISize = aReflowInput.AvailableISize();
-  availableISize -= startEdge + borderPadding.IEnd(frameWM);
-  aReflowInput.mLineLayout->BeginSpan(this, &aReflowInput, startEdge,
-                                      availableISize, &mBaseline);
+  nscoord endEdge = aReflowInput.AvailableISize() - borderPadding.IEnd(frameWM);
+  aReflowInput.mLineLayout->BeginSpan(this, &aReflowInput, startEdge, endEdge,
+                                      &mBaseline);
 
   for (RubySegmentEnumerator e(this); !e.AtEnd(); e.Next()) {
-    ReflowSegment(aPresContext, aReflowInput, e.GetBaseContainer(), aStatus);
+    ReflowSegment(aPresContext, aReflowInput, aDesiredSize.BlockStartAscent(),
+                  aDesiredSize.BSize(lineWM), e.GetBaseContainer(), aStatus);
 
     if (aStatus.IsInlineBreak()) {
       // A break occurs when reflowing the segment.
@@ -170,7 +155,8 @@ void nsRubyFrame::Reflow(nsPresContext* aPresContext,
       // No more continuations after, finish now.
       break;
     }
-    ReflowSegment(aPresContext, aReflowInput, baseContainer, aStatus);
+    ReflowSegment(aPresContext, aReflowInput, aDesiredSize.BlockStartAscent(),
+                  aDesiredSize.BSize(lineWM), baseContainer, aStatus);
   }
   // We never handle overflow in ruby.
   MOZ_ASSERT(!aStatus.IsOverflowIncomplete());
@@ -188,19 +174,18 @@ void nsRubyFrame::Reflow(nsPresContext* aPresContext,
     rbc->UpdateDescendantLeadings(mLeadings);
   }
 
-  nsLayoutUtils::SetBSizeFromFontMetrics(this, aDesiredSize, borderPadding,
-                                         lineWM, frameWM);
+  ReflowAbsoluteFrames(aPresContext, aDesiredSize, aReflowInput, aStatus);
 }
 
 void nsRubyFrame::ReflowSegment(nsPresContext* aPresContext,
                                 const ReflowInput& aReflowInput,
+                                nscoord aBlockStartAscent, nscoord aBlockSize,
                                 nsRubyBaseContainerFrame* aBaseContainer,
                                 nsReflowStatus& aStatus) {
   WritingMode lineWM = aReflowInput.mLineLayout->GetWritingMode();
   LogicalSize availSize(lineWM, aReflowInput.AvailableISize(),
                         aReflowInput.AvailableBSize());
-  WritingMode rubyWM = GetWritingMode();
-  NS_ASSERTION(!rubyWM.IsOrthogonalTo(lineWM),
+  NS_ASSERTION(!GetWritingMode().IsOrthogonalTo(lineWM),
                "Ruby frame writing-mode shouldn't be orthogonal to its line");
 
   AutoRubyTextContainerArray textContainers(aBaseContainer);
@@ -289,15 +274,17 @@ void nsRubyFrame::ReflowSegment(nsPresContext* aPresContext,
   // base container's rect, using a coordinate space that's relative to
   // the ruby frame. Right now, the base container's rect's block-axis
   // position is relative to the block container frame containing the
-  // lines, so we use 0 instead. (i.e. we assume that the base container
-  // is adjacent to the ruby frame's block-start edge.)
+  // lines, so here we reset it to the different between the ascents of
+  // the ruby container and the ruby base container, assuming they are
+  // aligned with the baseline.
   // XXX We may need to add border/padding here. See bug 1055667.
-  baseRect.BStart(lineWM) = 0;
+  baseRect.BStart(lineWM) = aBlockStartAscent - baseMetrics.BlockStartAscent();
   // The rect for offsets of text containers.
   LogicalRect offsetRect = baseRect;
   RubyBlockLeadings descLeadings = aBaseContainer->GetDescendantLeadings();
   offsetRect.BStart(lineWM) -= descLeadings.mStart;
   offsetRect.BSize(lineWM) += descLeadings.mStart + descLeadings.mEnd;
+  Maybe<LineRelativeDir> lastLineSide;
   for (uint32_t i = 0; i < rtcCount; i++) {
     nsRubyTextContainerFrame* textContainer = textContainers[i];
     WritingMode rtcWM = textContainer->GetWritingMode();
@@ -312,30 +299,47 @@ void nsRubyFrame::ReflowSegment(nsPresContext* aPresContext,
     // handled when reflowing the base containers.
     NS_ASSERTION(textReflowStatus.IsEmpty(),
                  "Ruby text container must not break itself inside");
-    // The metrics is initialized with reflow input of this ruby frame,
-    // hence the writing-mode is tied to rubyWM instead of rtcWM.
-    LogicalSize size = textMetrics.Size(rubyWM).ConvertTo(lineWM, rubyWM);
+    const LogicalSize size = textMetrics.Size(lineWM);
     textContainer->SetSize(lineWM, size);
 
     nscoord reservedISize = RubyUtils::GetReservedISize(textContainer);
     segmentISize = std::max(segmentISize, size.ISize(lineWM) + reservedISize);
 
-    uint8_t rubyPosition = textContainer->StyleText()->mRubyPosition;
-    MOZ_ASSERT(rubyPosition == NS_STYLE_RUBY_POSITION_OVER ||
-               rubyPosition == NS_STYLE_RUBY_POSITION_UNDER);
-    Maybe<LogicalSide> side;
-    if (rubyPosition == NS_STYLE_RUBY_POSITION_OVER) {
-      side.emplace(lineWM.LogicalSideForLineRelativeDir(eLineRelativeDirOver));
-    } else if (rubyPosition == NS_STYLE_RUBY_POSITION_UNDER) {
-      side.emplace(lineWM.LogicalSideForLineRelativeDir(eLineRelativeDirUnder));
-    } else {
-      // XXX inter-character support in bug 1055672
-      MOZ_ASSERT_UNREACHABLE("Unsupported ruby-position");
+    Maybe<LineRelativeDir> lineSide;
+    switch (textContainer->StyleText()->mRubyPosition) {
+      case StyleRubyPosition::Over:
+        lineSide.emplace(eLineRelativeDirOver);
+        break;
+      case StyleRubyPosition::Under:
+        lineSide.emplace(eLineRelativeDirUnder);
+        break;
+      case StyleRubyPosition::AlternateOver:
+        if (lastLineSide.isSome() &&
+            lastLineSide.value() == eLineRelativeDirOver) {
+          lineSide.emplace(eLineRelativeDirUnder);
+        } else {
+          lineSide.emplace(eLineRelativeDirOver);
+        }
+        break;
+      case StyleRubyPosition::AlternateUnder:
+        if (lastLineSide.isSome() &&
+            lastLineSide.value() == eLineRelativeDirUnder) {
+          lineSide.emplace(eLineRelativeDirOver);
+        } else {
+          lineSide.emplace(eLineRelativeDirUnder);
+        }
+        break;
+      default:
+        // XXX inter-character support in bug 1055672
+        MOZ_ASSERT_UNREACHABLE("Unsupported ruby-position");
     }
+    lastLineSide = lineSide;
 
     LogicalPoint position(lineWM);
-    if (side.isSome()) {
-      if (nsLayoutUtils::IsInterCharacterRubyEnabled() &&
+    if (lineSide.isSome()) {
+      LogicalSide logicalSide =
+          lineWM.LogicalSideForLineRelativeDir(lineSide.value());
+      if (StaticPrefs::layout_css_ruby_intercharacter_enabled() &&
           rtcWM.IsVerticalRL() &&
           lineWM.GetInlineDir() == WritingMode::eInlineLTR) {
         // Inter-character ruby annotations are only supported for vertical-rl
@@ -348,11 +352,11 @@ void nsRubyFrame::ReflowSegment(nsPresContext* aPresContext,
                 : 0);
         position = offsetRect.Origin(lineWM) + offset;
         aReflowInput.mLineLayout->AdvanceICoord(size.ISize(lineWM));
-      } else if (side.value() == eLogicalSideBStart) {
+      } else if (logicalSide == eLogicalSideBStart) {
         offsetRect.BStart(lineWM) -= size.BSize(lineWM);
         offsetRect.BSize(lineWM) += size.BSize(lineWM);
         position = offsetRect.Origin(lineWM);
-      } else if (side.value() == eLogicalSideBEnd) {
+      } else if (logicalSide == eLogicalSideBEnd) {
         position = offsetRect.Origin(lineWM) +
                    LogicalPoint(lineWM, 0, offsetRect.BSize(lineWM));
         offsetRect.BSize(lineWM) += size.BSize(lineWM);
@@ -365,7 +369,7 @@ void nsRubyFrame::ReflowSegment(nsPresContext* aPresContext,
     // reflowed.
     FinishReflowChild(textContainer, aPresContext, textMetrics,
                       &textReflowInput, lineWM, position, dummyContainerSize,
-                      0);
+                      ReflowChildFlags::Default);
   }
   MOZ_ASSERT(baseRect.ISize(lineWM) == offsetRect.ISize(lineWM),
              "Annotations should only be placed on the block directions");
@@ -378,9 +382,12 @@ void nsRubyFrame::ReflowSegment(nsPresContext* aPresContext,
     aReflowInput.mLineLayout->AdvanceICoord(deltaISize);
   }
 
-  // Set block leadings of the base container
-  nscoord startLeading = baseRect.BStart(lineWM) - offsetRect.BStart(lineWM);
-  nscoord endLeading = offsetRect.BEnd(lineWM) - baseRect.BEnd(lineWM);
+  // Set block leadings of the base container.
+  // The leadings are the difference between the offsetRect and the rect
+  // of this ruby container, which has block start zero and block size
+  // aBlockSize.
+  nscoord startLeading = -offsetRect.BStart(lineWM);
+  nscoord endLeading = offsetRect.BEnd(lineWM) - aBlockSize;
   // XXX When bug 765861 gets fixed, this warning should be upgraded.
   NS_WARNING_ASSERTION(startLeading >= 0 && endLeading >= 0,
                        "Leadings should be non-negative (because adding "
@@ -411,8 +418,7 @@ nsRubyBaseContainerFrame* nsRubyFrame::PullOneSegment(
   if (nsBlockFrame* newFloatCB =
           do_QueryFrame(aLineLayout->LineContainerFrame())) {
     if (oldFloatCB && oldFloatCB != newFloatCB) {
-      newFloatCB->ReparentFloats(baseFrame, oldFloatCB, true,
-                                 ReparentingDirection::Backwards);
+      newFloatCB->ReparentFloats(baseFrame, oldFloatCB, true);
     }
   }
 
