@@ -9,131 +9,147 @@
 #ifndef mozilla_css_Loader_h
 #define mozilla_css_Loader_h
 
-#include "nsIPrincipal.h"
+#include <tuple>
+#include <utility>
+
+#include "mozilla/Attributes.h"
+#include "mozilla/CORSMode.h"
+#include "mozilla/css/StylePreloadKind.h"
+#include "mozilla/dom/LinkStyle.h"
+#include "mozilla/MemoryReporting.h"
+#include "mozilla/UniquePtr.h"
 #include "nsCompatibility.h"
 #include "nsCycleCollectionParticipant.h"
-#include "nsDataHashtable.h"
-#include "nsRefPtrHashtable.h"
 #include "nsStringFwd.h"
 #include "nsTArray.h"
 #include "nsTObserverArray.h"
 #include "nsURIHashKey.h"
-#include "nsIStyleSheetLinkingElement.h"
-#include "mozilla/Attributes.h"
-#include "mozilla/CORSMode.h"
-#include "mozilla/StyleSheetInlines.h"
-#include "mozilla/Maybe.h"
-#include "mozilla/MemoryReporting.h"
-#include "mozilla/Move.h"
-#include "mozilla/StyleSheet.h"
-#include "mozilla/UniquePtr.h"
-#include "mozilla/net/ReferrerPolicy.h"
+#include "nsRefPtrHashtable.h"
 
 class nsICSSLoaderObserver;
 class nsIConsoleReportCollector;
 class nsIContent;
+class nsIPrincipal;
 
 namespace mozilla {
+
+class PreloadHashKey;
+class SharedStyleSheetCache;
+class SheetLoadDataHashKey;
+class StyleSheet;
+
 namespace dom {
 class DocGroup;
 class Element;
 }  // namespace dom
-}  // namespace mozilla
 
-namespace mozilla {
-
-class URIPrincipalReferrerPolicyAndCORSModeHashKey : public nsURIHashKey {
+// The load data for a <link> or @import style-sheet.
+//
+// This must contain all the state that affects CSS parsing.
+class SheetLoadDataHashKey : public PLDHashEntryHdr {
  public:
-  typedef URIPrincipalReferrerPolicyAndCORSModeHashKey* KeyType;
-  typedef const URIPrincipalReferrerPolicyAndCORSModeHashKey* KeyTypePointer;
-  typedef mozilla::net::ReferrerPolicy ReferrerPolicy;
+  using KeyType = const SheetLoadDataHashKey&;
+  using KeyTypePointer = const SheetLoadDataHashKey*;
 
-  explicit URIPrincipalReferrerPolicyAndCORSModeHashKey(
-      const URIPrincipalReferrerPolicyAndCORSModeHashKey* aKey)
-      : nsURIHashKey(aKey->mKey),
+  explicit SheetLoadDataHashKey(const SheetLoadDataHashKey* aKey)
+      : mURI(aKey->mURI),
         mPrincipal(aKey->mPrincipal),
+        mLoaderPrincipal(aKey->mLoaderPrincipal),
+        mPartitionPrincipal(aKey->mPartitionPrincipal),
+        mEncodingGuess(aKey->mEncodingGuess),
         mCORSMode(aKey->mCORSMode),
-        mReferrerPolicy(aKey->mReferrerPolicy) {
-    MOZ_COUNT_CTOR(URIPrincipalReferrerPolicyAndCORSModeHashKey);
+        mParsingMode(aKey->mParsingMode),
+        mCompatMode(aKey->mCompatMode),
+        mSRIMetadata(aKey->mSRIMetadata),
+        mIsLinkRelPreload(aKey->mIsLinkRelPreload) {
+    MOZ_COUNT_CTOR(SheetLoadDataHashKey);
   }
 
-  URIPrincipalReferrerPolicyAndCORSModeHashKey(nsIURI* aURI,
-                                               nsIPrincipal* aPrincipal,
-                                               CORSMode aCORSMode,
-                                               ReferrerPolicy aReferrerPolicy)
-      : nsURIHashKey(aURI),
+  SheetLoadDataHashKey(nsIURI* aURI, nsIPrincipal* aPrincipal,
+                       nsIPrincipal* aLoaderPrincipal,
+                       nsIPrincipal* aPartitionPrincipal,
+                       NotNull<const Encoding*> aEncodingGuess,
+                       CORSMode aCORSMode, css::SheetParsingMode aParsingMode,
+                       nsCompatibility aCompatMode,
+                       const dom::SRIMetadata& aSRIMetadata,
+                       css::StylePreloadKind aPreloadKind)
+      : mURI(aURI),
         mPrincipal(aPrincipal),
+        mLoaderPrincipal(aLoaderPrincipal),
+        mPartitionPrincipal(aPartitionPrincipal),
+        mEncodingGuess(aEncodingGuess),
         mCORSMode(aCORSMode),
-        mReferrerPolicy(aReferrerPolicy) {
-    MOZ_COUNT_CTOR(URIPrincipalReferrerPolicyAndCORSModeHashKey);
+        mParsingMode(aParsingMode),
+        mCompatMode(aCompatMode),
+        mSRIMetadata(aSRIMetadata),
+        mIsLinkRelPreload(IsLinkRelPreload(aPreloadKind)) {
+    MOZ_ASSERT(aURI);
+    MOZ_ASSERT(aPrincipal);
+    MOZ_ASSERT(aLoaderPrincipal);
+    MOZ_COUNT_CTOR(SheetLoadDataHashKey);
   }
 
-  URIPrincipalReferrerPolicyAndCORSModeHashKey(
-      URIPrincipalReferrerPolicyAndCORSModeHashKey&& toMove)
-      : nsURIHashKey(std::move(toMove)),
+  SheetLoadDataHashKey(SheetLoadDataHashKey&& toMove)
+      : mURI(std::move(toMove.mURI)),
         mPrincipal(std::move(toMove.mPrincipal)),
+        mLoaderPrincipal(std::move(toMove.mLoaderPrincipal)),
+        mPartitionPrincipal(std::move(toMove.mPartitionPrincipal)),
+        mEncodingGuess(std::move(toMove.mEncodingGuess)),
         mCORSMode(std::move(toMove.mCORSMode)),
-        mReferrerPolicy(std::move(toMove.mReferrerPolicy)) {
-    MOZ_COUNT_CTOR(URIPrincipalReferrerPolicyAndCORSModeHashKey);
+        mParsingMode(std::move(toMove.mParsingMode)),
+        mCompatMode(std::move(toMove.mCompatMode)),
+        mSRIMetadata(std::move(toMove.mSRIMetadata)),
+        mIsLinkRelPreload(std::move(toMove.mIsLinkRelPreload)) {
+    MOZ_COUNT_CTOR(SheetLoadDataHashKey);
   }
 
-  explicit URIPrincipalReferrerPolicyAndCORSModeHashKey(
-      css::SheetLoadData* aLoadData);
+  explicit SheetLoadDataHashKey(const css::SheetLoadData&);
 
-  ~URIPrincipalReferrerPolicyAndCORSModeHashKey() {
-    MOZ_COUNT_DTOR(URIPrincipalReferrerPolicyAndCORSModeHashKey);
+  MOZ_COUNTED_DTOR(SheetLoadDataHashKey)
+
+  const SheetLoadDataHashKey& GetKey() const { return *this; }
+  const SheetLoadDataHashKey* GetKeyPointer() const { return this; }
+
+  bool KeyEquals(const SheetLoadDataHashKey* aKey) const {
+    return KeyEquals(*aKey);
   }
 
-  URIPrincipalReferrerPolicyAndCORSModeHashKey* GetKey() const {
-    return const_cast<URIPrincipalReferrerPolicyAndCORSModeHashKey*>(this);
+  bool KeyEquals(const SheetLoadDataHashKey&) const;
+
+  static const SheetLoadDataHashKey* KeyToPointer(
+      const SheetLoadDataHashKey& aKey) {
+    return &aKey;
   }
-  const URIPrincipalReferrerPolicyAndCORSModeHashKey* GetKeyPointer() const {
-    return this;
+  static PLDHashNumber HashKey(const SheetLoadDataHashKey* aKey) {
+    return nsURIHashKey::HashKey(aKey->mURI);
   }
 
-  bool KeyEquals(
-      const URIPrincipalReferrerPolicyAndCORSModeHashKey* aKey) const {
-    if (!nsURIHashKey::KeyEquals(aKey->mKey)) {
-      return false;
-    }
+  nsIURI* URI() const { return mURI; }
 
-    if (!mPrincipal != !aKey->mPrincipal) {
-      // One or the other has a principal, but not both... not equal
-      return false;
-    }
+  nsIPrincipal* Principal() const { return mPrincipal; }
 
-    if (mCORSMode != aKey->mCORSMode) {
-      // Different CORS modes; we don't match
-      return false;
-    }
+  nsIPrincipal* LoaderPrincipal() const { return mLoaderPrincipal; }
 
-    if (mReferrerPolicy != aKey->mReferrerPolicy) {
-      // Different ReferrerPolicy; we don't match
-      return false;
-    }
+  nsIPrincipal* PartitionPrincipal() const { return mPartitionPrincipal; }
 
-    bool eq;
-    return !mPrincipal ||
-           (NS_SUCCEEDED(mPrincipal->Equals(aKey->mPrincipal, &eq)) && eq);
-  }
-
-  static const URIPrincipalReferrerPolicyAndCORSModeHashKey* KeyToPointer(
-      URIPrincipalReferrerPolicyAndCORSModeHashKey* aKey) {
-    return aKey;
-  }
-  static PLDHashNumber HashKey(
-      const URIPrincipalReferrerPolicyAndCORSModeHashKey* aKey) {
-    return nsURIHashKey::HashKey(aKey->mKey);
-  }
-
-  nsIURI* GetURI() const { return nsURIHashKey::GetKey(); }
+  css::SheetParsingMode ParsingMode() const { return mParsingMode; }
 
   enum { ALLOW_MEMMOVE = true };
 
  protected:
-  nsCOMPtr<nsIPrincipal> mPrincipal;
-  CORSMode mCORSMode;
-  ReferrerPolicy mReferrerPolicy;
+  const nsCOMPtr<nsIURI> mURI;
+  const nsCOMPtr<nsIPrincipal> mPrincipal;
+  const nsCOMPtr<nsIPrincipal> mLoaderPrincipal;
+  const nsCOMPtr<nsIPrincipal> mPartitionPrincipal;
+  // The encoding guess is the encoding the sheet would get if the request
+  // didn't have any encoding information like @charset or a Content-Encoding
+  // header.
+  const NotNull<const Encoding*> mEncodingGuess;
+  const CORSMode mCORSMode;
+  const css::SheetParsingMode mParsingMode;
+  const nsCompatibility mCompatMode;
+  dom::SRIMetadata mSRIMetadata;
+  const bool mIsLinkRelPreload;
 };
 
 namespace css {
@@ -147,7 +163,7 @@ class ImportRule;
 
 class MOZ_RAII LoaderReusableStyleSheets {
  public:
-  LoaderReusableStyleSheets() {}
+  LoaderReusableStyleSheets() = default;
 
   /**
    * Look for a reusable sheet (see AddReusableSheet) matching the
@@ -180,36 +196,25 @@ class MOZ_RAII LoaderReusableStyleSheets {
   nsTArray<RefPtr<StyleSheet>> mReusableSheets;
 };
 
-/***********************************************************************
- * Enum that describes the state of the sheet returned by CreateSheet. *
- ***********************************************************************/
-enum StyleSheetState {
-  eSheetStateUnknown = 0,
-  eSheetNeedsParser,
-  eSheetPending,
-  eSheetLoading,
-  eSheetComplete
-};
-
 class Loader final {
-  typedef mozilla::net::ReferrerPolicy ReferrerPolicy;
+  using ReferrerPolicy = dom::ReferrerPolicy;
 
  public:
-  typedef nsIStyleSheetLinkingElement::Completed Completed;
-  typedef nsIStyleSheetLinkingElement::HasAlternateRel HasAlternateRel;
-  typedef nsIStyleSheetLinkingElement::IsAlternate IsAlternate;
-  typedef nsIStyleSheetLinkingElement::IsInline IsInline;
-  typedef nsIStyleSheetLinkingElement::IsExplicitlyEnabled IsExplicitlyEnabled;
-  typedef nsIStyleSheetLinkingElement::MediaMatched MediaMatched;
-  typedef nsIStyleSheetLinkingElement::Update LoadSheetResult;
-  typedef nsIStyleSheetLinkingElement::SheetInfo SheetInfo;
+  using Completed = dom::LinkStyle::Completed;
+  using HasAlternateRel = dom::LinkStyle::HasAlternateRel;
+  using IsAlternate = dom::LinkStyle::IsAlternate;
+  using IsInline = dom::LinkStyle::IsInline;
+  using IsExplicitlyEnabled = dom::LinkStyle::IsExplicitlyEnabled;
+  using MediaMatched = dom::LinkStyle::MediaMatched;
+  using LoadSheetResult = dom::LinkStyle::Update;
+  using SheetInfo = dom::LinkStyle::SheetInfo;
 
   Loader();
   // aDocGroup is used for dispatching SheetLoadData in PostLoadEvent(). It
   // can be null if you want to use this constructor, and there's no
   // document when the Loader is constructed.
-  explicit Loader(mozilla::dom::DocGroup*);
-  explicit Loader(mozilla::dom::Document*);
+  explicit Loader(dom::DocGroup*);
+  explicit Loader(dom::Document*);
 
  private:
   // Private destructor, to discourage deletion outside of Release():
@@ -221,10 +226,26 @@ class Loader final {
 
   void DropDocumentReference();  // notification that doc is going away
 
+  void DeregisterFromSheetCache();
+  void RegisterInSheetCache();
+
   void SetCompatibilityMode(nsCompatibility aCompatMode) {
-    mCompatMode = aCompatMode;
+    mDocumentCompatMode = aCompatMode;
+  }
+
+  using StylePreloadKind = css::StylePreloadKind;
+
+  nsCompatibility CompatMode(StylePreloadKind aPreloadKind) const {
+    // For Link header preload, we guess non-quirks, because otherwise it is
+    // useless for modern pages.
+    //
+    // Link element preload is generally good because the speculative html
+    // parser deals with quirks mode properly.
+    if (aPreloadKind == StylePreloadKind::FromLinkRelPreloadHeader) {
+      return eCompatibility_FullStandards;
+    }
+    return mDocumentCompatMode;
   }
-  nsCompatibility GetCompatibilityMode() { return mCompatMode; }
 
   // TODO(emilio): Is the complexity of this method and carrying the titles
   // around worth it? The alternate sheets will load anyhow eventually...
@@ -273,16 +294,25 @@ class Loader final {
    *
    * @param aParentSheet the parent of this child sheet
    * @param aParentData the SheetLoadData corresponding to the load of the
-   *                    parent sheet.
+   *                    parent sheet. May be null for @import rules inserted via
+   *                    CSSOM.
    * @param aURL the URL of the child sheet
    * @param aMedia the already-parsed media list for the child sheet
    * @param aSavedSheets any saved style sheets which could be reused
    *              for this load
    */
-  nsresult LoadChildSheet(StyleSheet* aParentSheet, SheetLoadData* aParentData,
+  nsresult LoadChildSheet(StyleSheet& aParentSheet, SheetLoadData* aParentData,
                           nsIURI* aURL, dom::MediaList* aMedia,
                           LoaderReusableStyleSheets* aSavedSheets);
 
+  /**
+   * Called when we hit the internal memory cache with a complete stylesheet.
+   */
+  void DidHitCompleteSheetCache(const SheetLoadDataHashKey&,
+                                const StyleUseCounters* aCounters);
+
+  enum class UseSystemPrincipal { No, Yes };
+
   /**
    * Synchronously load and return the stylesheet at aURL.  Any child sheets
    * will also be loaded synchronously.  Note that synchronous loads over some
@@ -295,7 +325,6 @@ class Loader final {
    *        (see comments at enum SheetParsingMode, above).
    * @param aUseSystemPrincipal if true, give the resulting sheet the system
    * principal no matter where it's being loaded from.
-   * @param [out] aSheet the loaded, complete sheet.
    *
    * NOTE: At the moment, this method assumes the sheet will be UTF-8, but
    * ideally it would allow arbitrary encodings.  Callers should NOT depend on
@@ -305,16 +334,9 @@ class Loader final {
    * whether the data could be parsed as CSS and doesn't indicate anything
    * about the status of child sheets of the returned sheet.
    */
-  nsresult LoadSheetSync(nsIURI* aURL, SheetParsingMode aParsingMode,
-                         bool aUseSystemPrincipal, RefPtr<StyleSheet>* aSheet);
-
-  /**
-   * As above, but defaults aParsingMode to eAuthorSheetFeatures and
-   * aUseSystemPrincipal to false.
-   */
-  nsresult LoadSheetSync(nsIURI* aURL, RefPtr<StyleSheet>* aSheet) {
-    return LoadSheetSync(aURL, eAuthorSheetFeatures, false, aSheet);
-  }
+  Result<RefPtr<StyleSheet>, nsresult> LoadSheetSync(
+      nsIURI*, SheetParsingMode = eAuthorSheetFeatures,
+      UseSystemPrincipal = UseSystemPrincipal::No);
 
   /**
    * Asynchronously load the stylesheet at aURL.  If a successful result is
@@ -327,30 +349,27 @@ class Loader final {
    *        (see comments at enum SheetParsingMode, above).
    * @param aUseSystemPrincipal if true, give the resulting sheet the system
    * principal no matter where it's being loaded from.
+   * @param aReferrerInfo referrer information of the sheet.
    * @param aObserver the observer to notify when the load completes.
    *                  Must not be null.
-   * @param [out] aSheet the sheet to load. Note that the sheet may well
-   *              not be loaded by the time this method returns.
+   * @return the sheet to load. Note that the sheet may well not be loaded by
+   * the time this method returns.
    *
    * NOTE: At the moment, this method assumes the sheet will be UTF-8, but
    * ideally it would allow arbitrary encodings.  Callers should NOT depend on
    * non-UTF8 sheets being treated as UTF-8 by this method.
    */
-  nsresult LoadSheet(nsIURI* aURL, SheetParsingMode aParsingMode,
-                     bool aUseSystemPrincipal, nsICSSLoaderObserver* aObserver,
-                     RefPtr<StyleSheet>* aSheet);
+  Result<RefPtr<StyleSheet>, nsresult> LoadSheet(
+      nsIURI* aURI, StylePreloadKind, const Encoding* aPreloadEncoding,
+      nsIReferrerInfo* aReferrerInfo, nsICSSLoaderObserver* aObserver,
+      CORSMode = CORS_NONE, const nsAString& aIntegrity = u""_ns);
 
   /**
-   * Same as above, to be used when the caller doesn't care about the
-   * not-yet-loaded sheet.
+   * As above, but without caring for a couple things.
    */
-  nsresult LoadSheet(nsIURI* aURL, bool aIsPreload,
-                     nsIPrincipal* aOriginPrincipal,
-                     const Encoding* aPreloadEncoding,
-                     nsICSSLoaderObserver* aObserver,
-                     CORSMode aCORSMode = CORS_NONE,
-                     ReferrerPolicy aReferrerPolicy = mozilla::net::RP_Unset,
-                     const nsAString& aIntegrity = EmptyString());
+  Result<RefPtr<StyleSheet>, nsresult> LoadSheet(nsIURI*, SheetParsingMode,
+                                                 UseSystemPrincipal,
+                                                 nsICSSLoaderObserver*);
 
   /**
    * Stop loading all sheets.  All nsICSSLoaderObservers involved will be
@@ -374,10 +393,12 @@ class Loader final {
   bool GetEnabled() { return mEnabled; }
   void SetEnabled(bool aEnabled) { mEnabled = aEnabled; }
 
+  uint32_t ParsedSheetCount() const { return mParsedSheetCount; }
+
   /**
    * Get the document we live for. May return null.
    */
-  mozilla::dom::Document* GetDocument() const { return mDocument; }
+  dom::Document* GetDocument() const { return mDocument; }
 
   /**
    * Return true if this loader has pending loads (ones that would send
@@ -417,13 +438,45 @@ class Loader final {
   typedef nsTArray<RefPtr<SheetLoadData>> LoadDataArray;
 
   // Measure our size.
-  size_t SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) const;
+  size_t SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const;
+
+  enum class SheetState : uint8_t {
+    NeedsParser = 0,
+    Pending,
+    Loading,
+    Complete
+  };
+
+  // The loader principal is the document's node principal, if this loader is
+  // owned by a document, or the system principal otherwise.
+  nsIPrincipal* LoaderPrincipal() const;
+
+  // The partitioned principal is the document's partitioned principal, if this
+  // loader is owned by a document, or the system principal otherwise.
+  nsIPrincipal* PartitionedPrincipal() const;
+
+  bool ShouldBypassCache() const;
 
  private:
+  friend class mozilla::SharedStyleSheetCache;
   friend class SheetLoadData;
   friend class StreamLoader;
 
   // Helpers to conditionally block onload if mDocument is non-null.
+  void IncrementOngoingLoadCount() {
+    if (!mOngoingLoadCount++) {
+      BlockOnload();
+    }
+  }
+
+  void DecrementOngoingLoadCount() {
+    MOZ_DIAGNOSTIC_ASSERT(mOngoingLoadCount);
+    MOZ_DIAGNOSTIC_ASSERT(mOngoingLoadCount > mPendingLoadCount);
+    if (!--mOngoingLoadCount) {
+      UnblockOnload(false);
+    }
+  }
+
   void BlockOnload();
   void UnblockOnload(bool aFireSync);
 
@@ -434,71 +487,65 @@ class Loader final {
   nsresult CheckContentPolicy(nsIPrincipal* aLoadingPrincipal,
                               nsIPrincipal* aTriggeringPrincipal,
                               nsIURI* aTargetURI, nsINode* aRequestingNode,
-                              bool aIsPreload);
+                              const nsAString& aNonce, StylePreloadKind);
 
-  nsresult CreateSheet(const SheetInfo& aInfo, nsIPrincipal* aLoaderPrincipal,
-                       css::SheetParsingMode aParsingMode, bool aSyncLoad,
-                       StyleSheetState& aSheetState,
-                       RefPtr<StyleSheet>* aSheet) {
-    return CreateSheet(aInfo.mURI, aInfo.mContent, aLoaderPrincipal,
-                       aParsingMode, aInfo.mCORSMode, aInfo.mReferrerPolicy,
-                       aInfo.mIntegrity, aSyncLoad, aSheetState, aSheet);
+  std::tuple<RefPtr<StyleSheet>, SheetState> CreateSheet(
+      const SheetInfo& aInfo, css::SheetParsingMode aParsingMode,
+      bool aSyncLoad, css::StylePreloadKind aPreloadKind) {
+    nsIPrincipal* triggeringPrincipal = aInfo.mTriggeringPrincipal
+                                            ? aInfo.mTriggeringPrincipal.get()
+                                            : LoaderPrincipal();
+    return CreateSheet(aInfo.mURI, aInfo.mContent, triggeringPrincipal,
+                       aParsingMode, aInfo.mCORSMode,
+                       /* aPreloadOrParentDataEncoding = */ nullptr,
+                       aInfo.mIntegrity, aSyncLoad, aPreloadKind);
   }
 
   // For inline style, the aURI param is null, but the aLinkingContent
   // must be non-null then.  The loader principal must never be null
   // if aURI is not null.
-  nsresult CreateSheet(nsIURI* aURI, nsIContent* aLinkingContent,
-                       nsIPrincipal* aLoaderPrincipal,
-                       css::SheetParsingMode aParsingMode, CORSMode aCORSMode,
-                       ReferrerPolicy aReferrerPolicy,
-                       const nsAString& aIntegrity, bool aSyncLoad,
-                       StyleSheetState& aSheetState,
-                       RefPtr<StyleSheet>* aSheet);
+  std::tuple<RefPtr<StyleSheet>, SheetState> CreateSheet(
+      nsIURI* aURI, nsIContent* aLinkingContent,
+      nsIPrincipal* aTriggeringPrincipal, css::SheetParsingMode, CORSMode,
+      const Encoding* aPreloadOrParentDataEncoding, const nsAString& aIntegrity,
+      bool aSyncLoad, StylePreloadKind);
 
   // Pass in either a media string or the MediaList from the CSSParser.  Don't
   // pass both.
   //
-  // This method will set the sheet's enabled state based on aIsAlternate
-  MediaMatched PrepareSheet(StyleSheet* aSheet, const nsAString& aTitle,
+  // This method will set the sheet's enabled state based on IsAlternate and co.
+  MediaMatched PrepareSheet(StyleSheet&, const nsAString& aTitle,
                             const nsAString& aMediaString, dom::MediaList*,
                             IsAlternate, IsExplicitlyEnabled);
 
   // Inserts a style sheet in a document or a ShadowRoot.
-  void InsertSheetInTree(StyleSheet& aSheet, nsIContent* aLinkingContent);
+  void InsertSheetInTree(StyleSheet& aSheet, nsINode* aOwningNode);
   // Inserts a style sheet into a parent style sheet.
   void InsertChildSheet(StyleSheet& aSheet, StyleSheet& aParentSheet);
 
-  nsresult InternalLoadNonDocumentSheet(
-      nsIURI* aURL, bool aIsPreload, SheetParsingMode aParsingMode,
-      bool aUseSystemPrincipal, nsIPrincipal* aOriginPrincipal,
-      const Encoding* aPreloadEncoding, RefPtr<StyleSheet>* aSheet,
-      nsICSSLoaderObserver* aObserver, CORSMode aCORSMode = CORS_NONE,
-      ReferrerPolicy aReferrerPolicy = mozilla::net::RP_Unset,
-      const nsAString& aIntegrity = EmptyString());
+  Result<RefPtr<StyleSheet>, nsresult> InternalLoadNonDocumentSheet(
+      nsIURI* aURL, StylePreloadKind, SheetParsingMode aParsingMode,
+      UseSystemPrincipal, const Encoding* aPreloadEncoding,
+      nsIReferrerInfo* aReferrerInfo, nsICSSLoaderObserver* aObserver,
+      CORSMode aCORSMode, const nsAString& aIntegrity);
+
+  RefPtr<StyleSheet> LookupInlineSheetInCache(const nsAString&);
 
   // Post a load event for aObserver to be notified about aSheet.  The
   // notification will be sent with status NS_OK unless the load event is
   // canceled at some point (in which case it will be sent with
-  // NS_BINDING_ABORTED).  aWasAlternate indicates the state when the load was
-  // initiated, not the state at some later time.  aURI should be the URI the
-  // sheet was loaded from (may be null for inline sheets).  aElement is the
-  // owning element for this sheet.
-  nsresult PostLoadEvent(nsIURI* aURI, StyleSheet* aSheet,
-                         nsICSSLoaderObserver* aObserver,
-                         IsAlternate aWasAlternate, MediaMatched aMediaMatched,
-                         nsIStyleSheetLinkingElement* aElement);
+  // NS_BINDING_ABORTED).
+  nsresult PostLoadEvent(RefPtr<SheetLoadData>);
 
   // Start the loads of all the sheets in mPendingDatas
   void StartDeferredLoads();
 
-  // Handle an event posted by PostLoadEvent
-  void HandleLoadEvent(SheetLoadData* aEvent);
+  void HandleLoadEvent(SheetLoadData&);
 
-  // Note: LoadSheet is responsible for releasing aLoadData and setting the
-  // sheet to complete on failure.
-  nsresult LoadSheet(SheetLoadData* aLoadData, StyleSheetState aSheetState,
-                     bool aIsPreLoad);
+  // Note: LoadSheet is responsible for setting the sheet to complete on
+  // failure.
+  enum class PendingLoad { No, Yes };
+  nsresult LoadSheet(SheetLoadData&, SheetState, PendingLoad = PendingLoad::No);
 
   enum class AllowAsyncParse {
     Yes,
@@ -512,40 +559,37 @@ class Loader final {
   //
   // If this function returns Completed::Yes, then ParseSheet also called
   // SheetComplete on aLoadData.
-  Completed ParseSheet(const nsACString& aBytes, SheetLoadData*,
-                       AllowAsyncParse);
+  Completed ParseSheet(const nsACString&, SheetLoadData&, AllowAsyncParse);
 
-  // The load of the sheet in aLoadData is done, one way or another.  Do final
-  // cleanup, including releasing aLoadData.
-  void SheetComplete(SheetLoadData* aLoadData, nsresult aStatus);
+  // The load of the sheet in the load data is done, one way or another.
+  // Do final cleanup.
+  void SheetComplete(SheetLoadData&, nsresult);
 
-  // The guts of SheetComplete.  This may be called recursively on parent datas
-  // or datas that had glommed on to a single load.  The array is there so load
-  // datas whose observers need to be notified can be added to it.
-  void DoSheetComplete(SheetLoadData* aLoadData, LoadDataArray& aDatasToNotify);
+  // Notify observers on an individual data. This is different from
+  // SheetComplete for loads that are shared.
+  void NotifyObservers(SheetLoadData&, nsresult);
 
   // Mark the given SheetLoadData, as well as any of its siblings, parents, etc
   // transitively, as failed.  The idea is to mark as failed any load that was
   // directly or indirectly @importing the sheet this SheetLoadData represents.
-  void MarkLoadTreeFailed(SheetLoadData* aLoadData);
+  //
+  // if aOnlyForLoader is non-null, then only loads for a given loader will be
+  // marked as failing. This is useful to only cancel loads associated to a
+  // given loader, in case they were marked as canceled.
+  static void MarkLoadTreeFailed(SheetLoadData&,
+                                 Loader* aOnlyForLoader = nullptr);
 
-  // If there's Referrer-Policy reponse header, the loading data and key should
-  // be updated with the referer policy parsed from the header.
-  void UpdateLoadingData(URIPrincipalReferrerPolicyAndCORSModeHashKey* aOldKey,
-                         SheetLoadData* aData);
+  // A shorthand to mark a possible link preload as used to supress "unused"
+  // warning in the console.
+  void MaybeNotifyPreloadUsed(SheetLoadData&);
 
-  struct Sheets {
-    nsBaseHashtable<URIPrincipalReferrerPolicyAndCORSModeHashKey,
-                    RefPtr<StyleSheet>, StyleSheet*>
-        mCompleteSheets;
-    nsDataHashtable<URIPrincipalReferrerPolicyAndCORSModeHashKey,
-                    SheetLoadData*>
-        mLoadingDatas;  // weak refs
-    nsDataHashtable<URIPrincipalReferrerPolicyAndCORSModeHashKey,
-                    SheetLoadData*>
-        mPendingDatas;  // weak refs
-  };
-  UniquePtr<Sheets> mSheets;
+  nsRefPtrHashtable<nsStringHashKey, StyleSheet> mInlineSheets;
+
+  // A set with all the different loads we've done in a given document, for the
+  // purpose of not posting duplicate performance entries for them.
+  nsTHashtable<const SheetLoadDataHashKey> mLoadsPerformed;
+
+  RefPtr<SharedStyleSheetCache> mSheets;
 
   // The array of posted stylesheet loaded events (SheetLoadDatas) we have.
   // Note that these are rare.
@@ -556,26 +600,32 @@ class Loader final {
 
   // This reference is nulled by the Document in it's destructor through
   // DropDocumentReference().
-  mozilla::dom::Document* MOZ_NON_OWNING_REF
-      mDocument;  // the document we live for
+  dom::Document* MOZ_NON_OWNING_REF mDocument;  // the document we live for
 
   // For dispatching events via DocGroup::Dispatch() when mDocument is nullptr.
-  RefPtr<mozilla::dom::DocGroup> mDocGroup;
+  RefPtr<dom::DocGroup> mDocGroup;
 
-  // Number of datas still waiting to be notified on if we're notifying on a
-  // whole bunch at once (e.g. in one of the stop methods).  This is used to
-  // make sure that HasPendingLoads() won't return false until we're notifying
-  // on the last data we're working with.
-  uint32_t mDatasToNotifyOn;
-
-  nsCompatibility mCompatMode;
-
-  bool mEnabled;  // is enabled to load new styles
+  nsCompatibility mDocumentCompatMode;
 
   nsCOMPtr<nsIConsoleReportCollector> mReporter;
 
+  // Number of datas for asynchronous sheet loads still waiting to be notified.
+  // This includes pending stylesheets whose load hasn't started yet but which
+  // we need to, but not inline or constructable stylesheets, though the
+  // constructable stylesheets bit may change, see bug 1642227.
+  uint32_t mOngoingLoadCount = 0;
+
+  // The number of sheets that have been deferred / are in a pending state.
+  uint32_t mPendingLoadCount = 0;
+
+  // The number of stylesheets that we have parsed, for testing purposes.
+  uint32_t mParsedSheetCount = 0;
+
+  bool mEnabled = true;
+
 #ifdef DEBUG
-  bool mSyncCallback;
+  // Whether we're in a necko callback atm.
+  bool mSyncCallback = false;
 #endif
 };
 
