@@ -13,22 +13,27 @@ import java.util.Locale;
 
 import android.app.Service;
 import android.graphics.Rect;
-import android.os.Build;
 import android.os.Bundle;
 import android.os.LocaleList;
 import android.os.Parcel;
 import android.os.Parcelable;
-import android.support.annotation.AnyThread;
-import android.support.annotation.IntDef;
-import android.support.annotation.NonNull;
-import android.support.annotation.Nullable;
+import androidx.annotation.AnyThread;
+import androidx.annotation.IntDef;
+import androidx.annotation.NonNull;
+import androidx.annotation.Nullable;
 import android.text.TextUtils;
+import android.util.Log;
 
 import org.mozilla.gecko.EventDispatcher;
+import org.mozilla.gecko.GeckoSystemStateListener;
 import org.mozilla.gecko.util.GeckoBundle;
 
+import static android.os.Build.VERSION;
+
 @AnyThread
 public final class GeckoRuntimeSettings extends RuntimeSettings {
+    private static final String LOGTAG = "GeckoRuntimeSettings";
+
     /**
      * Settings builder used to construct the settings object.
      */
@@ -41,19 +46,6 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
             return new GeckoRuntimeSettings(settings);
         }
 
-        /**
-         * Set the content process hint flag.
-         *
-         * @param use If true, this will reload the content process for future use.
-         *            Default is false.
-         * @return This Builder instance.
-
-         */
-        public @NonNull Builder useContentProcessHint(final boolean use) {
-            getSettings().mUseContentProcess = use;
-            return this;
-        }
-
         /**
          * Set the custom Gecko process arguments.
          *
@@ -86,6 +78,10 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
          * Path to configuration file from which GeckoView will read configuration options such as
          * Gecko process arguments, environment variables, and preferences.
          *
+         * Note: this feature is only available for
+         * <code>{@link VERSION#SDK_INT} &gt; 21</code>, on older devices this will be
+         * silently ignored.
+         *
          * @param configFilePath Configuration file path to read from, or <code>null</code> to use
          *                       default location <code>/data/local/tmp/$PACKAGE-geckoview-config.yaml</code>.
          * @return This Builder instance.
@@ -157,6 +153,22 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
             return this;
         }
 
+        /**
+         * Set whether web manifest support is enabled.
+         *
+         * This controls if Gecko actually downloads, or "obtains", web
+         * manifests and processes them. Without setting this pref, trying
+         * to obtain a manifest throws.
+         *
+         * @param enabled A flag determining whether Web Manifest processing support is
+         *                enabled.
+         * @return The builder instance.
+         */
+        public @NonNull Builder webManifest(final boolean enabled) {
+            getSettings().mWebManifest.set(enabled);
+            return this;
+        }
+
         /**
          * Set whether or not web console messages should go to logcat.
          *
@@ -205,6 +217,20 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
             return this;
         }
 
+        /**
+         * Enable the Enteprise Roots feature.
+         *
+         * When Enabled, GeckoView will fetch the third-party root certificates added to the
+         * Android OS CA store and will use them internally.
+         *
+         * @param enabled whether to enable this feature or not
+         * @return The builder instance
+         */
+        public @NonNull Builder enterpiseRootsEnabled(final boolean enabled) {
+            getSettings().setEnterpriseRootsEnabled(enabled);
+            return this;
+        }
+
         /**
          * Set whether or not font inflation for non mobile-friendly pages should be enabled. The
          * default value of this setting is <code>false</code>.
@@ -265,6 +291,20 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
             return this;
         }
 
+        /**
+         * Set whether login forms should be filled automatically if only one
+         * viable candidate is provided via
+         * {@link Autocomplete.StorageDelegate#onLoginFetch onLoginFetch}.
+         *
+         * @param enabled A flag determining whether login autofill should be
+         *                enabled.
+         * @return The builder instance.
+         */
+        public @NonNull Builder loginAutofillEnabled(final boolean enabled) {
+            getSettings().setLoginAutofillEnabled(enabled);
+            return this;
+        }
+
         /**
          * When set, the specified {@link android.app.Service} will be started by
          * an {@link android.content.Intent} with action {@link GeckoRuntime#ACTION_CRASHED} when
@@ -314,23 +354,13 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
             return this;
         }
 
+        @SuppressWarnings("checkstyle:javadocmethod")
         public @NonNull Builder contentBlocking(
                 final @NonNull ContentBlocking.Settings cb) {
             getSettings().mContentBlocking = cb;
             return this;
         }
 
-        /**
-         * Sets video autoplay mode.
-         * May be either {@link GeckoRuntimeSettings#AUTOPLAY_DEFAULT_ALLOWED} or {@link GeckoRuntimeSettings#AUTOPLAY_DEFAULT_BLOCKED}
-         * @param autoplay Allows or blocks video autoplay.
-         * @return This Builder instance.
-         */
-        public @NonNull Builder autoplayDefault(final @AutoplayDefault int autoplay) {
-            getSettings().mAutoplayDefault.set(autoplay);
-            return this;
-        }
-
         /**
          * Sets the preferred color scheme override for web content.
          *
@@ -339,7 +369,7 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
          * @return This Builder instance.
          */
         public @NonNull Builder preferredColorScheme(final @ColorScheme int scheme) {
-            getSettings().mPreferredColorScheme.set(scheme);
+            getSettings().setPreferredColorScheme(scheme);
             return this;
         }
 
@@ -369,26 +399,95 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
          * Sets the WebGL MSAA level.
          *
          * @param level number of MSAA samples, 0 if MSAA should be disabled.
-         * @return This GeckoRuntimeSettings instance.
+         * @return This Builder instance.
          */
         public @NonNull Builder glMsaaLevel(final int level) {
             getSettings().mGlMsaaLevel.set(level);
             return this;
         }
+
+        /**
+         * Add a {@link RuntimeTelemetry.Delegate} instance to this
+         * GeckoRuntime.  This delegate can be used by the app to receive
+         * streaming telemetry data from GeckoView.
+         *
+         * @param delegate the delegate that will handle telemetry
+         * @return The builder instance.
+         */
+        public @NonNull Builder telemetryDelegate(
+                final @NonNull RuntimeTelemetry.Delegate delegate) {
+            getSettings().mTelemetryProxy = new RuntimeTelemetry.Proxy(delegate);
+            getSettings().mTelemetryEnabled.set(true);
+            return this;
+        }
+
+        /**
+         * Enables GeckoView and Gecko Logging.
+         * Logging is on by default. Does not control all logging in Gecko.
+         * Logging done in Java code must be stripped out at build time.
+         *
+         * @param enable True if logging is enabled.
+         * @return This Builder instance.
+         */
+        public @NonNull Builder debugLogging(final boolean enable) {
+            getSettings().mDevToolsConsoleToLogcat.set(enable);
+            getSettings().mConsoleServiceToLogcat.set(enable);
+            getSettings().mGeckoViewLogLevel.set(enable ? "Debug" : "Fatal");
+            return this;
+        }
+
+        /**
+         * Sets whether or not about:config should be enabled. This is a page that allows
+         * users to directly modify Gecko preferences. Modification of some preferences may
+         * cause the app to break in unpredictable ways -- crashes, performance issues, security
+         * vulnerabilities, etc.
+         *
+         * @param flag True if about:config should be enabled, false otherwise.
+         * @return This Builder instance.
+         */
+        public @NonNull Builder aboutConfigEnabled(final boolean flag) {
+            getSettings().mAboutConfig.set(flag);
+            return this;
+        }
+
+        /**
+         * Sets whether or not pinch-zooming should be enabled when <code>user-scalable=no</code> is set on the viewport.
+         *
+         * @param flag True if force user scalable zooming should be enabled, false otherwise.
+         * @return This Builder instance.
+         */
+        public @NonNull Builder forceUserScalableEnabled(final boolean flag) {
+            getSettings().mForceUserScalable.set(flag);
+            return this;
+        }
+
+        /**
+         * Sets whether and where insecure (non-HTTPS) connections are allowed.
+         *
+         * @param level One of the {@link GeckoRuntimeSettings#ALLOW_ALL HttpsOnlyMode} constants.
+         *
+         * @return This Builder instance.
+         */
+        public @NonNull Builder allowInsecureConnections(final @HttpsOnlyMode int level) {
+            getSettings().setAllowInsecureConnections(level);
+            return this;
+        }
     }
 
     private GeckoRuntime mRuntime;
-    /* package */ boolean mUseContentProcess;
     /* package */ String[] mArgs;
     /* package */ Bundle mExtras;
     /* package */ String mConfigFilePath;
 
     /* package */ ContentBlocking.Settings mContentBlocking;
 
+    @SuppressWarnings("checkstyle:javadocmethod")
     public @NonNull ContentBlocking.Settings getContentBlocking() {
         return mContentBlocking;
     }
 
+    /* package */ final Pref<Boolean> mWebManifest = new Pref<Boolean>(
+        "dom.manifest.enabled", true);
     /* package */ final Pref<Boolean> mJavaScript = new Pref<Boolean>(
         "javascript.enabled", true);
     /* package */ final Pref<Boolean> mRemoteDebugging = new Pref<Boolean>(
@@ -397,21 +496,42 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
         "browser.display.use_document_fonts", 1);
     /* package */ final Pref<Boolean> mConsoleOutput = new Pref<Boolean>(
         "geckoview.console.enabled", false);
-    /* package */ final Pref<Integer> mAutoplayDefault = new Pref<Integer>(
-        "media.autoplay.default", AUTOPLAY_DEFAULT_BLOCKED);
     /* package */ final Pref<Integer> mFontSizeFactor = new Pref<>(
         "font.size.systemFontScale", 100);
+    /* package */ final Pref<Boolean> mEnterpriseRootsEnabled = new Pref<>(
+            "security.enterprise_roots.enabled", false);
     /* package */ final Pref<Integer> mFontInflationMinTwips = new Pref<>(
         "font.size.inflation.minTwips", 0);
-    /* package */ final Pref<Integer> mPreferredColorScheme = new Pref<>(
-        "ui.systemUsesDarkTheme", -1);
     /* package */ final Pref<Boolean> mInputAutoZoom = new Pref<>(
             "formhelper.autozoom", true);
     /* package */ final Pref<Boolean> mDoubleTapZooming = new Pref<>(
             "apz.allow_double_tap_zooming", true);
     /* package */ final Pref<Integer> mGlMsaaLevel = new Pref<>(
             "gl.msaa-level", 0);
+    /* package */ final Pref<Boolean> mTelemetryEnabled = new Pref<>(
+            "toolkit.telemetry.geckoview.streaming", false);
+    /* package */ final Pref<String> mGeckoViewLogLevel = new Pref<>(
+            "geckoview.logging", "Debug");
+    /* package */ final Pref<Boolean> mConsoleServiceToLogcat = new Pref<>(
+            "consoleservice.logcat", true);
+    /* package */ final Pref<Boolean> mDevToolsConsoleToLogcat = new Pref<>(
+            "devtools.console.stdout.chrome", true);
+    /* package */ final Pref<Boolean> mAboutConfig = new Pref<>(
+            "general.aboutConfig.enable", false);
+    /* package */ final Pref<Boolean> mForceUserScalable = new Pref<>(
+            "browser.ui.zoom.force-user-scalable", false);
+    /* package */ final Pref<Boolean> mAutofillLogins = new Pref<Boolean>(
+        "signon.autofillForms", true);
+    /* package */ final Pref<Boolean> mHttpsOnly = new Pref<Boolean>(
+        "dom.security.https_only_mode", false);
+    /* package */ final Pref<Boolean> mHttpsOnlyPrivateMode = new Pref<Boolean>(
+        "dom.security.https_only_mode_pbm", false);
+    /* package */ final Pref<Integer> mProcessCount = new Pref<>(
+            "dom.ipc.processCount", 2);
 
+    /* package */ int mPreferredColorScheme = COLOR_SCHEME_SYSTEM;
+
+    /* package */ boolean mForceEnableAccessibility;
     /* package */ boolean mDebugPause;
     /* package */ boolean mUseMaxScreenDepth;
     /* package */ float mDisplayDensityOverride = -1.0f;
@@ -420,6 +540,7 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
     /* package */ int mScreenHeightOverride;
     /* package */ Class<? extends Service> mCrashHandler;
     /* package */ String[] mRequestedLocales;
+    /* package */ RuntimeTelemetry.Proxy mTelemetryProxy;
 
     /**
      * Attach and commit the settings to the given runtime.
@@ -428,6 +549,10 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
     /* package */ void attachTo(final @NonNull GeckoRuntime runtime) {
         mRuntime = runtime;
         commit();
+
+        if (mTelemetryProxy != null) {
+            mTelemetryProxy.attach();
+        }
     }
 
     @Override // RuntimeSettings
@@ -456,12 +581,12 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
     private void updateSettings(final @NonNull GeckoRuntimeSettings settings) {
         updatePrefs(settings);
 
-        mUseContentProcess = settings.getUseContentProcessHint();
         mArgs = settings.getArguments().clone();
         mExtras = new Bundle(settings.getExtras());
         mContentBlocking = new ContentBlocking.Settings(
                 this /* parent */, settings.mContentBlocking);
 
+        mForceEnableAccessibility = settings.mForceEnableAccessibility;
         mDebugPause = settings.mDebugPause;
         mUseMaxScreenDepth = settings.mUseMaxScreenDepth;
         mDisplayDensityOverride = settings.mDisplayDensityOverride;
@@ -471,6 +596,7 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
         mCrashHandler = settings.mCrashHandler;
         mRequestedLocales = settings.mRequestedLocales;
         mConfigFilePath = settings.mConfigFilePath;
+        mTelemetryProxy = settings.mTelemetryProxy;
     }
 
     /* package */ void commit() {
@@ -478,15 +604,6 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
         commitResetPrefs();
     }
 
-    /**
-     * Get the content process hint flag.
-     *
-     * @return The content process hint flag.
-     */
-    public boolean getUseContentProcessHint() {
-        return mUseContentProcess;
-    }
-
     /**
      * Get the custom Gecko process arguments.
      *
@@ -509,6 +626,8 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
      * Path to configuration file from which GeckoView will read configuration options such as
      * Gecko process arguments, environment variables, and preferences.
      *
+     * Note: this feature is only available for <code>{@link VERSION#SDK_INT} &gt; 21</code>.
+     *
      * @return Path to configuration file from which GeckoView will read configuration options,
      * or <code>null</code> for default location
      * <code>/data/local/tmp/$PACKAGE-geckoview-config.yaml</code>.
@@ -586,6 +705,29 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
         return mDebugPause;
     }
 
+    /**
+     * Gets whether accessibility is force enabled or not.
+     *
+     * @return true if accessibility is force enabled.
+     */
+    public boolean getForceEnableAccessibility() {
+        return mForceEnableAccessibility;
+    }
+
+    /**
+     * Sets whether accessibility is force enabled or not.
+     *
+     * Useful when testing accessibility.
+     *
+     * @param value whether accessibility is force enabled or not
+     * @return this GeckoRuntimeSettings instance.
+     */
+    public @NonNull GeckoRuntimeSettings setForceEnableAccessibility(final boolean value) {
+        mForceEnableAccessibility = value;
+        SessionAccessibility.setForceEnabled(value);
+        return this;
+    }
+
     /**
      * Gets whether the compositor should use the maximum screen depth when rendering.
      *
@@ -619,6 +761,7 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
         return null;
     }
 
+    @SuppressWarnings("checkstyle:javadocmethod")
     public @Nullable Class<? extends Service> getCrashHandler() {
         return mCrashHandler;
     }
@@ -663,19 +806,19 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
     }
 
     private String computeAcceptLanguages() {
-        ArrayList<String> locales = new ArrayList<String>();
+        final ArrayList<String> locales = new ArrayList<String>();
 
         // Explicitly-set app prefs come first:
         if (mRequestedLocales != null) {
-            for (String locale : mRequestedLocales) {
-                locales.add(locale.toLowerCase());
+            for (final String locale : mRequestedLocales) {
+                locales.add(locale.toLowerCase(Locale.ROOT));
             }
         }
         // OS prefs come second:
-        for (String locale : getDefaultLocales()) {
-            locale = locale.toLowerCase();
-            if (!locales.contains(locale)) {
-                locales.add(locale);
+        for (final String locale : getDefaultLocales()) {
+            final String localeLowerCase = locale.toLowerCase(Locale.ROOT);
+            if (!locales.contains(localeLowerCase)) {
+                locales.add(localeLowerCase);
             }
         }
 
@@ -683,17 +826,17 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
     }
 
     private static String[] getDefaultLocales() {
-        if (Build.VERSION.SDK_INT >= 24) {
+        if (VERSION.SDK_INT >= 24) {
             final LocaleList localeList = LocaleList.getDefault();
-            String[] locales = new String[localeList.size()];
+            final String[] locales = new String[localeList.size()];
             for (int i = 0; i < localeList.size(); i++) {
                 locales[i] = localeList.get(i).toLanguageTag();
             }
             return locales;
         }
-        String[] locales = new String[1];
+        final String[] locales = new String[1];
         final Locale locale = Locale.getDefault();
-        if (Build.VERSION.SDK_INT >= 21) {
+        if (VERSION.SDK_INT >= 21) {
             locales[0] = locale.toLanguageTag();
             return locales;
         }
@@ -716,6 +859,28 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
         return out.toString();
     }
 
+    /**
+     * Sets whether Web Manifest processing support is enabled.
+     *
+     * @param enabled A flag determining whether Web Manifest processing support is
+     *                enabled.
+     *
+     * @return This GeckoRuntimeSettings instance.
+     */
+    public @NonNull GeckoRuntimeSettings setWebManifestEnabled(final boolean enabled) {
+        mWebManifest.commit(enabled);
+        return this;
+    }
+
+    /**
+     * Get whether or not Web Manifest processing support is enabled.
+     *
+     * @return True if web manifest processing support is enabled.
+     */
+    public boolean getWebManifestEnabled() {
+        return mWebManifest.get();
+    }
+
     /**
      * Set whether or not web console messages should go to logcat.
      *
@@ -743,10 +908,10 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
 
     /**
      * Set whether or not font sizes in web content should be automatically scaled according to
-     * the device's current system font scale setting.
-     * Disabling this setting will restore the previously used values for
-     * {@link GeckoRuntimeSettings#getFontSizeFactor()} and
-     * {@link GeckoRuntimeSettings#getFontInflationEnabled()}.
+     * the device's current system font scale setting. Enabling this will prevent modification of
+     * the {@link GeckoRuntimeSettings#setFontSizeFactor font size factor}.
+     * Disabling this setting will restore the previously used value for the
+     * {@link GeckoRuntimeSettings#getFontSizeFactor font size factor}.
      *
      * @param enabled A flag determining whether or not font sizes should be scaled automatically
      *                to match the device's system font scale.
@@ -767,41 +932,6 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
         return GeckoFontScaleListener.getInstance().getEnabled();
     }
 
-    // Sync values with dom/media/nsIAutoplay.idl.
-    @Retention(RetentionPolicy.SOURCE)
-    @IntDef({ AUTOPLAY_DEFAULT_ALLOWED, AUTOPLAY_DEFAULT_BLOCKED })
-    /* package */ @interface AutoplayDefault {}
-
-    /**
-     * Autoplay video is allowed.
-     */
-    public static final int AUTOPLAY_DEFAULT_ALLOWED = 0;
-
-    /**
-     * Autoplay video is blocked.
-     */
-    public static final int AUTOPLAY_DEFAULT_BLOCKED = 1;
-
-    /**
-     * Sets video autoplay mode.
-     * May be either {@link GeckoRuntimeSettings#AUTOPLAY_DEFAULT_ALLOWED} or {@link GeckoRuntimeSettings#AUTOPLAY_DEFAULT_BLOCKED}
-     * @param autoplay Allows or blocks video autoplay.
-     * @return This GeckoRuntimeSettings instance.
-     */
-    public @NonNull GeckoRuntimeSettings setAutoplayDefault(final @AutoplayDefault int autoplay) {
-        mAutoplayDefault.commit(autoplay);
-        return this;
-    }
-
-    /**
-     * Gets the current video autoplay mode.
-     * @return The current video autoplay mode. Will be either {@link GeckoRuntimeSettings#AUTOPLAY_DEFAULT_ALLOWED}
-     * or {@link GeckoRuntimeSettings#AUTOPLAY_DEFAULT_BLOCKED}
-     */
-    public @AutoplayDefault int getAutoplayDefault() {
-        return mAutoplayDefault.get();
-    }
-
     private static final int FONT_INFLATION_BASE_VALUE = 120;
 
     /**
@@ -828,14 +958,48 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
         return setFontSizeFactorInternal(fontSizeFactor);
     }
 
-    /* package */ @NonNull GeckoRuntimeSettings setFontSizeFactorInternal(
-            final float fontSizeFactor) {
+    /*
+     * Enable the Enteprise Roots feature.
+     *
+     * When Enabled, GeckoView will fetch the third-party root certificates added to the
+     * Android OS CA store and will use them internally.
+     *
+     * @param enabled whether to enable this feature or not
+     * @return This GeckoRuntimeSettings instance
+     */
+    public @NonNull GeckoRuntimeSettings setEnterpriseRootsEnabled(final boolean enabled) {
+        mEnterpriseRootsEnabled.commit(enabled);
+        return this;
+    }
+
+    /**
+     * Gets whether the Enteprise Roots feature is enabled or not.
+     *
+     * @return true if the feature is enabled, false otherwise.
+     */
+    public boolean getEnterpriseRootsEnabled() {
+        return mEnterpriseRootsEnabled.get();
+    }
+
+    private final static float DEFAULT_FONT_SIZE_FACTOR = 1f;
+
+    private float sanitizeFontSizeFactor(final float fontSizeFactor) {
         if (fontSizeFactor < 0) {
-            throw new IllegalArgumentException("fontSizeFactor cannot be < 0");
+            if (BuildConfig.DEBUG) {
+                throw new IllegalArgumentException("fontSizeFactor cannot be < 0");
+            } else {
+                Log.e(LOGTAG, "fontSizeFactor cannot be < 0");
+                return DEFAULT_FONT_SIZE_FACTOR;
+            }
         }
 
-        final int fontSizePercentage = Math.round(fontSizeFactor * 100);
-        mFontSizeFactor.commit(Math.round(fontSizePercentage));
+        return fontSizeFactor;
+    }
+
+    /* package */ @NonNull GeckoRuntimeSettings setFontSizeFactorInternal(
+            final float fontSizeFactor) {
+        final int fontSizePercentage = Math.round(sanitizeFontSizeFactor(fontSizeFactor) * 100);
+        mFontSizeFactor.commit(fontSizePercentage);
         if (getFontInflationEnabled()) {
             final int scaledFontInflation = Math.round(FONT_INFLATION_BASE_VALUE * fontSizeFactor);
             mFontInflationMinTwips.commit(scaledFontInflation);
@@ -866,21 +1030,10 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
      *
      * <p>Currently, any changes only take effect after a reload of the session.
      *
-     * <p>This setting cannot be modified while
-     * {@link GeckoRuntimeSettings#setAutomaticFontSizeAdjustment automatic font size adjustment}
-     * is enabled.
-     *
      * @param enabled A flag determining whether or not font inflation should be enabled.
      * @return This GeckoRuntimeSettings instance.
      */
     public @NonNull GeckoRuntimeSettings setFontInflationEnabled(final boolean enabled) {
-        if (getAutomaticFontSizeAdjustment()) {
-            throw new IllegalStateException("Not allowed when automatic font size adjustment is enabled");
-        }
-        return setFontInflationEnabledInternal(enabled);
-    }
-
-    /* package */ @NonNull GeckoRuntimeSettings setFontInflationEnabledInternal(final boolean enabled) {
         final int minTwips =
                 enabled ? Math.round(FONT_INFLATION_BASE_VALUE * getFontSizeFactor()) : 0;
         mFontInflationMinTwips.commit(minTwips);
@@ -915,7 +1068,7 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
      * @return One of the {@link GeckoRuntimeSettings#COLOR_SCHEME_LIGHT COLOR_SCHEME_*} constants.
      */
     public @ColorScheme int getPreferredColorScheme() {
-        return mPreferredColorScheme.get();
+        return mPreferredColorScheme;
     }
 
     /**
@@ -926,7 +1079,10 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
      * @return This GeckoRuntimeSettings instance.
      */
     public @NonNull GeckoRuntimeSettings setPreferredColorScheme(final @ColorScheme int scheme) {
-        mPreferredColorScheme.commit(scheme);
+        if (mPreferredColorScheme != scheme) {
+            mPreferredColorScheme = scheme;
+            GeckoSystemStateListener.onDeviceChanged();
+        }
         return this;
     }
 
@@ -990,13 +1146,147 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
         return this;
     }
 
+    @SuppressWarnings("checkstyle:javadocmethod")
+    public @Nullable RuntimeTelemetry.Delegate getTelemetryDelegate() {
+        return mTelemetryProxy.getDelegate();
+    }
+
+    /**
+     * Gets whether about:config is enabled or not.
+     *
+     * @return True if about:config is enabled, false otherwise.
+     */
+    public boolean getAboutConfigEnabled() {
+        return mAboutConfig.get();
+    }
+
+    /**
+     * Sets whether or not about:config should be enabled. This is a page that allows
+     * users to directly modify Gecko preferences. Modification of some preferences may
+     * cause the app to break in unpredictable ways -- crashes, performance issues, security
+     * vulnerabilities, etc.
+     *
+     * @param flag True if about:config should be enabled, false otherwise.
+     * @return This GeckoRuntimeSettings instance.
+     */
+    public @NonNull GeckoRuntimeSettings setAboutConfigEnabled(final boolean flag) {
+        mAboutConfig.commit(flag);
+        return this;
+    }
+
+    /**
+     * Gets whether or not force user scalable zooming should be enabled or not.
+     *
+     * @return True if force user scalable zooming should be enabled, false otherwise.
+     */
+    public boolean getForceUserScalableEnabled() {
+        return mForceUserScalable.get();
+    }
+
+    /**
+     * Sets whether or not pinch-zooming should be enabled when <code>user-scalable=no</code> is set on the viewport.
+     *
+     * @param flag True if force user scalable zooming should be enabled, false otherwise.
+     * @return This GeckoRuntimeSettings instance.
+     */
+    public @NonNull GeckoRuntimeSettings setForceUserScalableEnabled(final boolean flag) {
+        mForceUserScalable.commit(flag);
+        return this;
+    }
+
+    /**
+     * Get whether login form autofill is enabled.
+     *
+     * @return True if login autofill is enabled.
+     */
+    public boolean getLoginAutofillEnabled() {
+        return mAutofillLogins.get();
+    }
+
+    /**
+     * Set whether login forms should be filled automatically if only one
+     * viable candidate is provided via
+     * {@link Autocomplete.StorageDelegate#onLoginFetch onLoginFetch}.
+     *
+     * @param enabled A flag determining whether login autofill should be
+     *                enabled.
+     * @return The builder instance.
+     */
+    public @NonNull GeckoRuntimeSettings setLoginAutofillEnabled(
+            final boolean enabled) {
+        mAutofillLogins.commit(enabled);
+        return this;
+    }
+
+    @Retention(RetentionPolicy.SOURCE)
+    @IntDef({ALLOW_ALL,
+             HTTPS_ONLY_PRIVATE,
+             HTTPS_ONLY})
+    /* package */ @interface HttpsOnlyMode {}
+
+    /** Allow all insecure connections */
+    public static final int ALLOW_ALL = 0;
+    /** Allow insecure connections in normal browsing, but only HTTPS in private browsing. */
+    public static final int HTTPS_ONLY_PRIVATE = 1;
+    /** Only allow HTTPS connections. */
+    public static final int HTTPS_ONLY = 2;
+
+    /**
+     * Get whether and where insecure (non-HTTPS) connections are allowed.
+     *
+     * @return One of the {@link GeckoRuntimeSettings#ALLOW_ALL HttpsOnlyMode} constants.
+     */
+    public @HttpsOnlyMode int getAllowInsecureConnections() {
+        final boolean httpsOnly = mHttpsOnly.get();
+        final boolean httpsOnlyPrivate = mHttpsOnlyPrivateMode.get();
+        if (httpsOnly) {
+            return HTTPS_ONLY;
+        } else if (httpsOnlyPrivate) {
+            return HTTPS_ONLY_PRIVATE;
+        }
+        return ALLOW_ALL;
+    }
+
+    /**
+     * Set whether and where insecure (non-HTTPS) connections are allowed.
+     *
+     * @param level One of the {@link GeckoRuntimeSettings#ALLOW_ALL HttpsOnlyMode} constants.
+     *
+     * @return This GeckoRuntimeSettings instance.
+     */
+    public @NonNull GeckoRuntimeSettings setAllowInsecureConnections(final @HttpsOnlyMode int level) {
+        switch (level) {
+            case ALLOW_ALL:
+                mHttpsOnly.commit(false);
+                mHttpsOnlyPrivateMode.commit(false);
+                break;
+            case HTTPS_ONLY_PRIVATE:
+                mHttpsOnly.commit(false);
+                mHttpsOnlyPrivateMode.commit(true);
+                break;
+            case HTTPS_ONLY:
+                mHttpsOnly.commit(true);
+                mHttpsOnlyPrivateMode.commit(false);
+                break;
+            default:
+                throw new IllegalArgumentException("Invalid setting for setAllowInsecureConnections");
+        }
+        return this;
+    }
+
+    // For internal use only
+    /* protected */ @NonNull GeckoRuntimeSettings setProcessCount(final int processCount) {
+        mProcessCount.commit(processCount);
+        return this;
+    }
+
     @Override // Parcelable
     public void writeToParcel(final Parcel out, final int flags) {
         super.writeToParcel(out, flags);
 
-        ParcelableUtils.writeBoolean(out, mUseContentProcess);
         out.writeStringArray(mArgs);
         mExtras.writeToParcel(out, flags);
+        ParcelableUtils.writeBoolean(out, mForceEnableAccessibility);
         ParcelableUtils.writeBoolean(out, mDebugPause);
         ParcelableUtils.writeBoolean(out, mUseMaxScreenDepth);
         out.writeFloat(mDisplayDensityOverride);
@@ -1009,12 +1299,13 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
     }
 
     // AIDL code may call readFromParcel even though it's not part of Parcelable.
+    @SuppressWarnings("checkstyle:javadocmethod")
     public void readFromParcel(final @NonNull Parcel source) {
         super.readFromParcel(source);
 
-        mUseContentProcess = ParcelableUtils.readBoolean(source);
         mArgs = source.createStringArray();
         mExtras.readFromParcel(source);
+        mForceEnableAccessibility = ParcelableUtils.readBoolean(source);
         mDebugPause = ParcelableUtils.readBoolean(source);
         mUseMaxScreenDepth = ParcelableUtils.readBoolean(source);
         mDisplayDensityOverride = source.readFloat();
@@ -1030,7 +1321,7 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
                         (Class<? extends Service>) Class.forName(crashHandlerName);
 
                 mCrashHandler = handler;
-            } catch (ClassNotFoundException e) {
+            } catch (final ClassNotFoundException e) {
             }
         }
 
