@@ -8,11 +8,11 @@
 
 #include "mozilla/Preferences.h"
 #include "nsContentUtils.h"
+#include "nsComponentManagerUtils.h"
 #include "nsIAsyncVerifyRedirectCallback.h"
 #include "nsIAuthPrompt.h"
 #include "nsIDHCPClient.h"
 #include "nsIHttpChannel.h"
-#include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
 #include "nsIPromptFactory.h"
 #include "nsIProtocolProxyService.h"
@@ -21,6 +21,7 @@
 #include "nsThreadUtils.h"
 #include "mozilla/Result.h"
 #include "mozilla/ResultExtensions.h"
+#include "mozilla/Telemetry.h"
 
 //-----------------------------------------------------------------------------
 
@@ -34,6 +35,10 @@ LazyLogModule gProxyLog("proxy");
 #define MOZ_WPAD_URL "http://wpad/wpad.dat"
 #define MOZ_DHCP_WPAD_OPTION 252
 
+// These pointers are declared in nsProtocolProxyService.cpp
+extern const char kProxyType_HTTPS[];
+extern const char kProxyType_DIRECT[];
+
 // The PAC thread does evaluations of both PAC files and
 // nsISystemProxySettings because they can both block the calling thread and we
 // don't want that on the main thread
@@ -182,7 +187,10 @@ class PACLoadComplete final : public Runnable {
 
   NS_IMETHOD Run() override {
     MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
-    mPACMan->mLoader = nullptr;
+    {
+      auto loader = mPACMan->mLoader.Lock();
+      loader.ref() = nullptr;
+    }
     mPACMan->PostProcessPendingQ();
     return NS_OK;
   }
@@ -301,10 +309,11 @@ class ExecutePACThreadAction final : public Runnable {
 //-----------------------------------------------------------------------------
 
 PendingPACQuery::PendingPACQuery(nsPACMan* pacMan, nsIURI* uri,
-                                 nsPACManCallback* callback,
+                                 nsPACManCallback* callback, uint32_t flags,
                                  bool mainThreadResponse)
     : Runnable("net::PendingPACQuery"),
       mPort(0),
+      mFlags(flags),
       mPACMan(pacMan),
       mCallback(callback),
       mOnMainThreadOnly(mainThreadResponse) {
@@ -318,10 +327,11 @@ void PendingPACQuery::Complete(nsresult status, const nsACString& pacString) {
   if (!mCallback) return;
   RefPtr<ExecuteCallback> runnable = new ExecuteCallback(mCallback, status);
   runnable->SetPACString(pacString);
-  if (mOnMainThreadOnly)
+  if (mOnMainThreadOnly) {
     mPACMan->Dispatch(runnable.forget());
-  else
+  } else {
     runnable->Run();
+  }
 }
 
 void PendingPACQuery::UseAlternatePACFile(const nsACString& pacURL) {
@@ -329,10 +339,11 @@ void PendingPACQuery::UseAlternatePACFile(const nsACString& pacURL) {
 
   RefPtr<ExecuteCallback> runnable = new ExecuteCallback(mCallback, NS_OK);
   runnable->SetPACURL(pacURL);
-  if (mOnMainThreadOnly)
+  if (mOnMainThreadOnly) {
     mPACMan->Dispatch(runnable.forget());
-  else
+  } else {
     runnable->Run();
+  }
 }
 
 NS_IMETHODIMP
@@ -350,8 +361,9 @@ static uint32_t sThreadLocalIndex = 0xdeadbeef;  // out of range
 static const char* kPACIncludePath =
     "network.proxy.autoconfig_url.include_path";
 
-nsPACMan::nsPACMan(nsIEventTarget* mainThreadEventTarget)
+nsPACMan::nsPACMan(nsISerialEventTarget* mainThreadEventTarget)
     : NeckoTargetHolder(mainThreadEventTarget),
+      mLoader("nsPACMan::mLoader"),
       mLoadPending(false),
       mShutdown(false),
       mLoadFailureCount(0),
@@ -381,7 +393,13 @@ nsPACMan::~nsPACMan() {
     }
   }
 
-  NS_ASSERTION(mLoader == nullptr, "pac man not shutdown properly");
+#ifdef DEBUG
+  {
+    auto loader = mLoader.Lock();
+    NS_ASSERTION(loader.ref() == nullptr, "pac man not shutdown properly");
+  }
+#endif
+
   NS_ASSERTION(mPendingQ.isEmpty(), "pac man not shutdown properly");
 }
 
@@ -428,6 +446,7 @@ nsresult nsPACMan::DispatchToPAC(already_AddRefed<nsIRunnable> aEvent,
 }
 
 nsresult nsPACMan::AsyncGetProxyForURI(nsIURI* uri, nsPACManCallback* callback,
+                                       uint32_t flags,
                                        bool mainThreadResponse) {
   MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
   if (mShutdown) return NS_ERROR_NOT_AVAILABLE;
@@ -437,15 +456,15 @@ nsresult nsPACMan::AsyncGetProxyForURI(nsIURI* uri, nsPACManCallback* callback,
       TimeStamp::Now() > mScheduledReload) {
     LOG(("nsPACMan::AsyncGetProxyForURI reload as scheduled\n"));
 
-    LoadPACFromURI(mAutoDetect ? EmptyCString() : mPACURISpec, false);
+    LoadPACFromURI(mAutoDetect ? ""_ns : mPACURISpec, false);
   }
 
   RefPtr<PendingPACQuery> query =
-      new PendingPACQuery(this, uri, callback, mainThreadResponse);
+      new PendingPACQuery(this, uri, callback, flags, mainThreadResponse);
 
   if (IsPACURI(uri)) {
     // deal with this directly instead of queueing it
-    query->Complete(NS_OK, EmptyCString());
+    query->Complete(NS_OK, ""_ns);
     return NS_OK;
   }
 
@@ -456,7 +475,7 @@ nsresult nsPACMan::PostQuery(PendingPACQuery* query) {
   MOZ_ASSERT(!NS_IsMainThread(), "wrong thread");
 
   if (mShutdown) {
-    query->Complete(NS_ERROR_NOT_AVAILABLE, EmptyCString());
+    query->Complete(NS_ERROR_NOT_AVAILABLE, ""_ns);
     return NS_OK;
   }
 
@@ -484,7 +503,10 @@ nsresult nsPACMan::LoadPACFromURI(const nsACString& aSpec,
 
   CancelExistingLoad();
 
-  mLoader = loader;
+  {
+    auto locked = mLoader.Lock();
+    locked.ref() = loader.forget();
+  }
   mPACURIRedirectSpec.Truncate();
   mNormalPACURISpec.Truncate();  // set at load time
   if (aResetLoadFailureCount) {
@@ -519,10 +541,9 @@ nsresult nsPACMan::LoadPACFromURI(const nsACString& aSpec,
   if (!mLoadPending) {
     nsCOMPtr<nsIRunnable> runnable = NewRunnableMethod(
         "nsPACMan::StartLoading", this, &nsPACMan::StartLoading);
-    nsresult rv =
-        NS_IsMainThread()
-            ? Dispatch(runnable.forget())
-            : GetCurrentThreadEventTarget()->Dispatch(runnable.forget());
+    nsresult rv = NS_IsMainThread()
+                      ? Dispatch(runnable.forget())
+                      : GetCurrentEventTarget()->Dispatch(runnable.forget());
     if (NS_FAILED(rv)) return rv;
     mLoadPending = true;
   }
@@ -590,10 +611,17 @@ void nsPACMan::StartLoading() {
   MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
   mLoadPending = false;
 
-  // CancelExistingLoad was called...
-  if (!mLoader) {
-    PostCancelPendingQ(NS_ERROR_ABORT);
-    return;
+  {
+    // CancelExistingLoad was called...
+    nsCOMPtr<nsIStreamLoader> loader;
+    {
+      auto locked = mLoader.Lock();
+      loader = locked.ref();
+    }
+    if (!loader) {
+      PostCancelPendingQ(NS_ERROR_ABORT);
+      return;
+    }
   }
 
   if (mAutoDetect) {
@@ -616,12 +644,18 @@ void nsPACMan::StartLoading() {
 void nsPACMan::ContinueLoadingAfterPACUriKnown() {
   MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
 
+  nsCOMPtr<nsIStreamLoader> loader;
+  {
+    auto locked = mLoader.Lock();
+    loader = locked.ref();
+  }
+
   // CancelExistingLoad was called...
-  if (!mLoader) {
+  if (!loader) {
     PostCancelPendingQ(NS_ERROR_ABORT);
     return;
   }
-  if (NS_SUCCEEDED(mLoader->Init(this, nullptr))) {
+  if (NS_SUCCEEDED(loader->Init(this, nullptr))) {
     // Always hit the origin server when loading PAC.
     nsCOMPtr<nsIIOService> ios = do_GetIOService();
     if (ios) {
@@ -635,9 +669,9 @@ void nsPACMan::ContinueLoadingAfterPACUriKnown() {
         MOZ_RELEASE_ASSERT(NS_SUCCEEDED(rv));
         NS_NewChannel(getter_AddRefs(channel), pacURI,
                       nsContentUtils::GetSystemPrincipal(),
-                      nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
+                      nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL,
                       nsIContentPolicy::TYPE_OTHER,
-                      nullptr,  // nsICookieSettings
+                      nullptr,  // nsICookieJarSettings
                       nullptr,  // PerformanceStorage
                       nullptr,  // aLoadGroup
                       nullptr,  // aCallbacks
@@ -648,9 +682,13 @@ void nsPACMan::ContinueLoadingAfterPACUriKnown() {
       }
 
       if (channel) {
+        // allow deprecated HTTP request from SystemPrincipal
+        nsCOMPtr<nsILoadInfo> loadInfo = channel->LoadInfo();
+        loadInfo->SetAllowDeprecatedSystemRequests(true);
+
         channel->SetLoadFlags(nsIRequest::LOAD_BYPASS_CACHE);
         channel->SetNotificationCallbacks(this);
-        if (NS_SUCCEEDED(channel->AsyncOpen(mLoader))) return;
+        if (NS_SUCCEEDED(channel->AsyncOpen(loader))) return;
       }
     }
   }
@@ -685,11 +723,17 @@ void nsPACMan::OnLoadFailure() {
 }
 
 void nsPACMan::CancelExistingLoad() {
-  if (mLoader) {
+  nsCOMPtr<nsIStreamLoader> loader;
+  {
+    auto locked = mLoader.Lock();
+    loader.swap(*locked);
+  }
+  if (loader) {
     nsCOMPtr<nsIRequest> request;
-    mLoader->GetRequest(getter_AddRefs(request));
-    if (request) request->Cancel(NS_ERROR_ABORT);
-    mLoader = nullptr;
+    loader->GetRequest(getter_AddRefs(request));
+    if (request) {
+      request->Cancel(NS_ERROR_ABORT);
+    }
   }
 }
 
@@ -712,7 +756,7 @@ void nsPACMan::CancelPendingQ(nsresult status, bool aShutdown) {
 
   while (!mPendingQ.isEmpty()) {
     query = dont_AddRef(mPendingQ.popLast());
-    query->Complete(status, EmptyCString());
+    query->Complete(status, ""_ns);
   }
 
   if (aShutdown) mPAC.Shutdown();
@@ -720,8 +764,9 @@ void nsPACMan::CancelPendingQ(nsresult status, bool aShutdown) {
 
 void nsPACMan::ProcessPendingQ() {
   MOZ_ASSERT(!NS_IsMainThread(), "wrong thread");
-  while (ProcessPending())
+  while (ProcessPending()) {
     ;
+  }
 
   if (mShutdown) {
     mPAC.Shutdown();
@@ -742,7 +787,7 @@ bool nsPACMan::ProcessPending() {
   RefPtr<PendingPACQuery> query(dont_AddRef(mPendingQ.popFirst()));
 
   if (mShutdown || IsLoading()) {
-    query->Complete(NS_ERROR_NOT_AVAILABLE, EmptyCString());
+    query->Complete(NS_ERROR_NOT_AVAILABLE, ""_ns);
     return true;
   }
 
@@ -766,6 +811,18 @@ bool nsPACMan::ProcessPending() {
       NS_SUCCEEDED(mSystemProxySettings->GetProxyForURI(
           query->mSpec, query->mScheme, query->mHost, query->mPort,
           pacString))) {
+    if (query->mFlags & nsIProtocolProxyService::RESOLVE_PREFER_SOCKS_PROXY &&
+        query->mFlags & nsIProtocolProxyService::RESOLVE_PREFER_HTTPS_PROXY) {
+      if (StringBeginsWith(pacString, nsDependentCString(kProxyType_DIRECT),
+                           nsCaseInsensitiveUTF8StringComparator)) {
+        // DIRECT indicates that system proxy settings are not configured to use
+        // SOCKS proxy. Try https proxy as a secondary preferrable proxy. This
+        // is mainly for websocket whose precedence is SOCKS > HTTPS > DIRECT.
+        NS_SUCCEEDED(mSystemProxySettings->GetProxyForURI(
+            query->mSpec, nsDependentCString(kProxyType_HTTPS), query->mHost,
+            query->mPort, pacString));
+      }
+    }
     LOG(("Use proxy from system settings: %s\n", pacString.get()));
     query->Complete(NS_OK, pacString);
     completed = true;
@@ -791,13 +848,17 @@ nsPACMan::OnStreamComplete(nsIStreamLoader* loader, nsISupports* context,
                            nsresult status, uint32_t dataLen,
                            const uint8_t* data) {
   MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
-  if (mLoader != loader) {
-    // If this happens, then it means that LoadPACFromURI was called more
-    // than once before the initial call completed.  In this case, status
-    // should be NS_ERROR_ABORT, and if so, then we know that we can and
-    // should delay any processing.
-    LOG(("OnStreamComplete: called more than once\n"));
-    if (status == NS_ERROR_ABORT) return NS_OK;
+
+  {
+    auto locked = mLoader.Lock();
+    if (locked.ref() != loader) {
+      // If this happens, then it means that LoadPACFromURI was called more
+      // than once before the initial call completed.  In this case, status
+      // should be NS_ERROR_ABORT, and if so, then we know that we can and
+      // should delay any processing.
+      LOG(("OnStreamComplete: called more than once\n"));
+      if (status == NS_ERROR_ABORT) return NS_OK;
+    }
   }
 
   LOG(("OnStreamComplete: entry\n"));
@@ -837,10 +898,11 @@ nsPACMan::OnStreamComplete(nsIStreamLoader* loader, nsISupports* context,
     OnLoadFailure();
   }
 
-  if (NS_SUCCEEDED(status))
+  if (NS_SUCCEEDED(status)) {
     PostProcessPendingQ();
-  else
+  } else {
     PostCancelPendingQ(status);
+  }
 
   return NS_OK;
 }
@@ -851,8 +913,13 @@ nsPACMan::GetInterface(const nsIID& iid, void** result) {
   if (iid.Equals(NS_GET_IID(nsIAuthPrompt))) {
     nsCOMPtr<nsIPromptFactory> promptFac =
         do_GetService("@mozilla.org/prompter;1");
-    NS_ENSURE_TRUE(promptFac, NS_ERROR_FAILURE);
-    return promptFac->GetPrompt(nullptr, iid, reinterpret_cast<void**>(result));
+    NS_ENSURE_TRUE(promptFac, NS_ERROR_NO_INTERFACE);
+    nsresult rv =
+        promptFac->GetPrompt(nullptr, iid, reinterpret_cast<void**>(result));
+    if (NS_FAILED(rv)) {
+      return NS_ERROR_NO_INTERFACE;
+    }
+    return NS_OK;
   }
 
   // In case loading the PAC file results in a redirect.
