@@ -15,7 +15,6 @@
 
 #include "nsCOMPtr.h"
 #include "nsRefPtrHashtable.h"
-#include "nsDataHashtable.h"
 #include "nsHashKeys.h"
 #include "nsString.h"
 #include "nsCOMArray.h"
@@ -25,8 +24,7 @@
 #include "mozilla/TimeStamp.h"
 
 static inline uint32_t PRTimeToSeconds(PRTime t_usec) {
-  PRTime usec_per_sec = PR_USEC_PER_SEC;
-  return uint32_t(t_usec /= usec_per_sec);
+  return uint32_t(t_usec / PR_USEC_PER_SEC);
 }
 
 #define NowInSeconds() PRTimeToSeconds(PR_Now())
@@ -64,17 +62,16 @@ class CacheEntry final : public nsIRunnable, public CacheFileListener {
   // Forwarded to from CacheEntryHandle : nsICacheEntry
   nsresult GetKey(nsACString& aKey);
   nsresult GetCacheEntryId(uint64_t* aCacheEntryId);
-  nsresult GetPersistent(bool* aPersistent);
+  nsresult GetPersistent(bool* aPersistToDisk);
   nsresult GetFetchCount(int32_t* aFetchCount);
   nsresult GetLastFetched(uint32_t* aLastFetched);
   nsresult GetLastModified(uint32_t* aLastModified);
   nsresult GetExpirationTime(uint32_t* aExpirationTime);
   nsresult SetExpirationTime(uint32_t expirationTime);
-  nsresult GetOnStartTime(uint64_t* aOnStartTime);
-  nsresult GetOnStopTime(uint64_t* aOnStopTime);
+  nsresult GetOnStartTime(uint64_t* aTime);
+  nsresult GetOnStopTime(uint64_t* aTime);
   nsresult SetNetworkTimes(uint64_t onStartTime, uint64_t onStopTime);
   nsresult SetContentType(uint8_t aContentType);
-  nsresult AddBaseDomainAccess(uint32_t aSiteID);
   nsresult ForceValidFor(uint32_t aSecondsToTheFuture);
   nsresult GetIsForcedValid(bool* aIsForcedValid);
   nsresult OpenInputStream(int64_t offset, nsIInputStream** _retval);
@@ -83,8 +80,8 @@ class CacheEntry final : public nsIRunnable, public CacheFileListener {
   nsresult GetSecurityInfo(nsISupports** aSecurityInfo);
   nsresult SetSecurityInfo(nsISupports* aSecurityInfo);
   nsresult GetStorageDataSize(uint32_t* aStorageDataSize);
-  nsresult AsyncDoom(nsICacheEntryDoomCallback* listener);
-  nsresult GetMetaDataElement(const char* key, char** _retval);
+  nsresult AsyncDoom(nsICacheEntryDoomCallback* aCallback);
+  nsresult GetMetaDataElement(const char* key, char** aRetval);
   nsresult SetMetaDataElement(const char* key, const char* value);
   nsresult VisitMetaData(nsICacheEntryMetaDataVisitor* visitor);
   nsresult MetaDataReady(void);
@@ -92,18 +89,18 @@ class CacheEntry final : public nsIRunnable, public CacheFileListener {
   nsresult GetDiskStorageSizeInKB(uint32_t* aDiskStorageSizeInKB);
   nsresult Recreate(bool aMemoryOnly, nsICacheEntry** _retval);
   nsresult GetDataSize(int64_t* aDataSize);
-  nsresult GetAltDataSize(int64_t* aAltDataSize);
+  nsresult GetAltDataSize(int64_t* aDataSize);
   nsresult GetAltDataType(nsACString& aAltDataType);
   nsresult OpenAlternativeOutputStream(const nsACString& type,
                                        int64_t predictedSize,
                                        nsIAsyncOutputStream** _retval);
   nsresult OpenAlternativeInputStream(const nsACString& type,
                                       nsIInputStream** _retval);
-  nsresult GetLoadContextInfo(nsILoadContextInfo** aLoadContextInfo);
+  nsresult GetLoadContextInfo(nsILoadContextInfo** aInfo);
   nsresult Close(void);
   nsresult MarkValid(void);
   nsresult MaybeMarkValid(void);
-  nsresult HasWriteAccess(bool aWriteAllowed, bool* _retval);
+  nsresult HasWriteAccess(bool aWriteAllowed, bool* aWriteAccess);
 
  public:
   uint32_t GetMetadataMemoryConsumption();
@@ -154,8 +151,9 @@ class CacheEntry final : public nsIRunnable, public CacheFileListener {
                              const nsACString& aURISpec, nsACString& aResult);
 
   // Accessed only on the service management thread
-  double mFrecency;
-  ::mozilla::Atomic<uint32_t, ::mozilla::Relaxed> mSortingExpirationTime;
+  double mFrecency{0};
+  ::mozilla::Atomic<uint32_t, ::mozilla::Relaxed> mSortingExpirationTime{
+      uint32_t(-1)};
 
   // Memory reporting
   size_t SizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf) const;
@@ -310,7 +308,7 @@ class CacheEntry final : public nsIRunnable, public CacheFileListener {
       bool aMemoryOnly, nsICacheEntryOpenCallback* aCallback);
   void TransferCallbacks(CacheEntry& aFromEntry);
 
-  mozilla::Mutex mLock;
+  mozilla::Mutex mLock{"CacheEntry"};
 
   // Reflects the number of existing handles for this entry
   ::mozilla::ThreadSafeAutoRefCnt mHandlesCount;
@@ -323,7 +321,7 @@ class CacheEntry final : public nsIRunnable, public CacheFileListener {
   // Using ReleaseAcquire since we only control access to mFile with this.
   // When mFileStatus is read and found success it is ensured there is mFile and
   // that it is after a successful call to Init().
-  ::mozilla::Atomic<nsresult, ::mozilla::ReleaseAcquire> mFileStatus;
+  Atomic<nsresult, ReleaseAcquire> mFileStatus{NS_ERROR_NOT_INITIALIZED};
   nsCString mURI;
   nsCString mEnhanceID;
   nsCString mStorageID;
@@ -337,7 +335,7 @@ class CacheEntry final : public nsIRunnable, public CacheFileListener {
   // Whether it should skip max size check.
   bool const mSkipSizeCheck;
   // Set when entry is doomed with AsyncDoom() or DoomAlreadyRemoved().
-  bool mIsDoomed;
+  bool mIsDoomed{false};
 
   // Following flags are all synchronized with the cache entry lock.
 
@@ -371,7 +369,7 @@ class CacheEntry final : public nsIRunnable, public CacheFileListener {
   };
 
   // State of this entry.
-  EState mState;
+  EState mState{NOTLOADED};
 
   enum ERegistration {
     NEVERREGISTERED = 0,  // The entry has never been registered
@@ -381,7 +379,7 @@ class CacheEntry final : public nsIRunnable, public CacheFileListener {
 
   // Accessed only on the management thread.  Records the state of registration
   // this entry in the memory pool intermediate cache.
-  ERegistration mRegistration;
+  ERegistration mRegistration{NEVERREGISTERED};
 
   // If a new (empty) entry is requested to open an input stream before
   // output stream has been opened, we must open output stream internally
@@ -392,7 +390,7 @@ class CacheEntry final : public nsIRunnable, public CacheFileListener {
   // Weak reference to the current writter.  There can be more then one
   // writer at a time and OnHandleClosed() must be processed only for the
   // current one.
-  CacheEntryHandle* mWriter;
+  CacheEntryHandle* mWriter{nullptr};
 
   // Background thread scheduled operation.  Set (under the lock) one
   // of this flags to tell the background thread what to do.
@@ -403,7 +401,7 @@ class CacheEntry final : public nsIRunnable, public CacheFileListener {
     static uint32_t const CALLBACKS = 1 << 2;
     static uint32_t const UNREGISTER = 1 << 3;
 
-    Ops() : mFlags(0) {}
+    Ops() = default;
     uint32_t Grab() {
       uint32_t flags = mFlags;
       mFlags = 0;
@@ -416,12 +414,12 @@ class CacheEntry final : public nsIRunnable, public CacheFileListener {
     }
 
    private:
-    uint32_t mFlags;
+    uint32_t mFlags{0};
   } mBackgroundOperations;
 
   nsCOMPtr<nsISupports> mSecurityInfo;
   mozilla::TimeStamp mLoadStart;
-  uint32_t mUseCount;
+  uint32_t mUseCount{0};
 
   const uint64_t mCacheEntryId;
 };
@@ -469,9 +467,6 @@ class CacheEntryHandle final : public nsICacheEntry {
   NS_IMETHOD SetContentType(uint8_t contentType) override {
     return mEntry->SetContentType(contentType);
   }
-  NS_IMETHOD AddBaseDomainAccess(uint32_t aSiteID) override {
-    return mEntry->AddBaseDomainAccess(aSiteID);
-  }
   NS_IMETHOD ForceValidFor(uint32_t aSecondsToTheFuture) override {
     return mEntry->ForceValidFor(aSecondsToTheFuture);
   }
@@ -553,7 +548,7 @@ class CacheEntryHandle final : public nsICacheEntry {
 
   // This is |false| until Dismiss() was called and prevents OnHandleClosed
   // being called more than once.
-  Atomic<bool, ReleaseAcquire> mClosed;
+  Atomic<bool, ReleaseAcquire> mClosed{false};
 };
 
 class CacheOutputCloseListener final : public Runnable {
