@@ -5,20 +5,22 @@
 #include "CacheLog.h"
 #include "CacheFileContextEvictor.h"
 #include "CacheFileIOManager.h"
+#include "CacheFileMetadata.h"
 #include "CacheIndex.h"
 #include "CacheIndexIterator.h"
 #include "CacheFileUtils.h"
+#include "CacheObserver.h"
 #include "nsIFile.h"
 #include "LoadContextInfo.h"
 #include "nsThreadUtils.h"
 #include "nsString.h"
-#include "nsISimpleEnumerator.h"
 #include "nsIDirectoryEnumerator.h"
 #include "mozilla/Base64.h"
 #include "mozilla/IntegerPrintfMacros.h"
+#include "nsContentUtils.h"
+#include "nsNetUtil.h"
 
-namespace mozilla {
-namespace net {
+namespace mozilla::net {
 
 #define CONTEXT_EVICTION_PREFIX "ce_"
 const uint32_t kContextEvictionPrefixLength =
@@ -26,8 +28,7 @@ const uint32_t kContextEvictionPrefixLength =
 
 bool CacheFileContextEvictor::sDiskAlreadySearched = false;
 
-CacheFileContextEvictor::CacheFileContextEvictor()
-    : mEvicting(false), mIndexIsUpToDate(false) {
+CacheFileContextEvictor::CacheFileContextEvictor() {
   LOG(("CacheFileContextEvictor::CacheFileContextEvictor() [this=%p]", this));
 }
 
@@ -35,7 +36,7 @@ CacheFileContextEvictor::~CacheFileContextEvictor() {
   LOG(("CacheFileContextEvictor::~CacheFileContextEvictor() [this=%p]", this));
 }
 
-nsresult CacheFileContextEvictor::Init(nsIFile *aCacheDirectory) {
+nsresult CacheFileContextEvictor::Init(nsIFile* aCacheDirectory) {
   LOG(("CacheFileContextEvictor::Init()"));
 
   nsresult rv;
@@ -51,7 +52,7 @@ nsresult CacheFileContextEvictor::Init(nsIFile *aCacheDirectory) {
     return rv;
   }
 
-  rv = mEntriesDir->AppendNative(NS_LITERAL_CSTRING(ENTRIES_DIR));
+  rv = mEntriesDir->AppendNative(nsLiteralCString(ENTRIES_DIR));
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -82,7 +83,8 @@ uint32_t CacheFileContextEvictor::ContextsCount() {
 }
 
 nsresult CacheFileContextEvictor::AddContext(
-    nsILoadContextInfo *aLoadContextInfo, bool aPinned) {
+    nsILoadContextInfo* aLoadContextInfo, bool aPinned,
+    const nsAString& aOrigin) {
   LOG(
       ("CacheFileContextEvictor::AddContext() [this=%p, loadContextInfo=%p, "
        "pinned=%d]",
@@ -92,12 +94,13 @@ nsresult CacheFileContextEvictor::AddContext(
 
   MOZ_ASSERT(CacheFileIOManager::IsOnIOThread());
 
-  CacheFileContextEvictorEntry *entry = nullptr;
+  CacheFileContextEvictorEntry* entry = nullptr;
   if (aLoadContextInfo) {
     for (uint32_t i = 0; i < mEntries.Length(); ++i) {
       if (mEntries[i]->mInfo && mEntries[i]->mInfo->Equals(aLoadContextInfo) &&
-          mEntries[i]->mPinned == aPinned) {
-        entry = mEntries[i];
+          mEntries[i]->mPinned == aPinned &&
+          mEntries[i]->mOrigin.Equals(aOrigin)) {
+        entry = mEntries[i].get();
         break;
       }
     }
@@ -108,7 +111,8 @@ nsresult CacheFileContextEvictor::AddContext(
     for (uint32_t i = mEntries.Length(); i > 0;) {
       --i;
       if (mEntries[i]->mInfo && mEntries[i]->mPinned == aPinned) {
-        RemoveEvictInfoFromDisk(mEntries[i]->mInfo, mEntries[i]->mPinned);
+        RemoveEvictInfoFromDisk(mEntries[i]->mInfo, mEntries[i]->mPinned,
+                                mEntries[i]->mOrigin);
         mEntries.RemoveElementAt(i);
       }
     }
@@ -118,12 +122,13 @@ nsresult CacheFileContextEvictor::AddContext(
     entry = new CacheFileContextEvictorEntry();
     entry->mInfo = aLoadContextInfo;
     entry->mPinned = aPinned;
-    mEntries.AppendElement(entry);
+    entry->mOrigin = aOrigin;
+    mEntries.AppendElement(WrapUnique(entry));
   }
 
   entry->mTimeStamp = PR_Now() / PR_USEC_PER_MSEC;
 
-  PersistEvictionInfoToDisk(aLoadContextInfo, aPinned);
+  PersistEvictionInfoToDisk(aLoadContextInfo, aPinned, aOrigin);
 
   if (mIndexIsUpToDate) {
     // Already existing context could be added again, in this case the iterator
@@ -153,7 +158,7 @@ nsresult CacheFileContextEvictor::AddContext(
   return NS_OK;
 }
 
-nsresult CacheFileContextEvictor::CacheIndexStateChanged() {
+void CacheFileContextEvictor::CacheIndexStateChanged() {
   LOG(("CacheFileContextEvictor::CacheIndexStateChanged() [this=%p]", this));
 
   MOZ_ASSERT(CacheFileIOManager::IsOnIOThread());
@@ -163,18 +168,18 @@ nsresult CacheFileContextEvictor::CacheIndexStateChanged() {
   if (mEntries.Length() == 0) {
     // Just save the state and exit, since there is nothing to do
     mIndexIsUpToDate = isUpToDate;
-    return NS_OK;
+    return;
   }
 
   if (!isUpToDate && !mIndexIsUpToDate) {
     // Index is outdated and status has not changed, nothing to do.
-    return NS_OK;
+    return;
   }
 
   if (isUpToDate && mIndexIsUpToDate) {
     // Status has not changed, but make sure the eviction is running.
     if (mEvicting) {
-      return NS_OK;
+      return;
     }
 
     // We're not evicting, but we should be evicting?!
@@ -192,19 +197,14 @@ nsresult CacheFileContextEvictor::CacheIndexStateChanged() {
   } else {
     CloseIterators();
   }
-
-  return NS_OK;
 }
 
-nsresult CacheFileContextEvictor::WasEvicted(const nsACString &aKey,
-                                             nsIFile *aFile,
-                                             bool *aEvictedAsPinned,
-                                             bool *aEvictedAsNonPinned) {
+void CacheFileContextEvictor::WasEvicted(const nsACString& aKey, nsIFile* aFile,
+                                         bool* aEvictedAsPinned,
+                                         bool* aEvictedAsNonPinned) {
   LOG(("CacheFileContextEvictor::WasEvicted() [key=%s]",
        PromiseFlatCString(aKey).get()));
 
-  nsresult rv;
-
   *aEvictedAsPinned = false;
   *aEvictedAsNonPinned = false;
 
@@ -214,24 +214,22 @@ nsresult CacheFileContextEvictor::WasEvicted(const nsACString &aKey,
   MOZ_ASSERT(info);
   if (!info) {
     LOG(("CacheFileContextEvictor::WasEvicted() - Cannot parse key!"));
-    return NS_OK;
+    return;
   }
 
   for (uint32_t i = 0; i < mEntries.Length(); ++i) {
-    CacheFileContextEvictorEntry *entry = mEntries[i];
+    const auto& entry = mEntries[i];
 
     if (entry->mInfo && !info->Equals(entry->mInfo)) {
       continue;
     }
 
     PRTime lastModifiedTime;
-    rv = aFile->GetLastModifiedTime(&lastModifiedTime);
-    if (NS_FAILED(rv)) {
+    if (NS_FAILED(aFile->GetLastModifiedTime(&lastModifiedTime))) {
       LOG(
           ("CacheFileContextEvictor::WasEvicted() - Cannot get last modified "
-           "time"
-           ", returning false."));
-      return NS_OK;
+           "time, returning."));
+      return;
     }
 
     if (lastModifiedTime > entry->mTimeStamp) {
@@ -250,12 +248,11 @@ nsresult CacheFileContextEvictor::WasEvicted(const nsACString &aKey,
       *aEvictedAsNonPinned = true;
     }
   }
-
-  return NS_OK;
 }
 
 nsresult CacheFileContextEvictor::PersistEvictionInfoToDisk(
-    nsILoadContextInfo *aLoadContextInfo, bool aPinned) {
+    nsILoadContextInfo* aLoadContextInfo, bool aPinned,
+    const nsAString& aOrigin) {
   LOG(
       ("CacheFileContextEvictor::PersistEvictionInfoToDisk() [this=%p, "
        "loadContextInfo=%p]",
@@ -266,14 +263,14 @@ nsresult CacheFileContextEvictor::PersistEvictionInfoToDisk(
   MOZ_ASSERT(CacheFileIOManager::IsOnIOThread());
 
   nsCOMPtr<nsIFile> file;
-  rv = GetContextFile(aLoadContextInfo, aPinned, getter_AddRefs(file));
+  rv = GetContextFile(aLoadContextInfo, aPinned, aOrigin, getter_AddRefs(file));
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
 
   nsCString path = file->HumanReadablePath();
 
-  PRFileDesc *fd;
+  PRFileDesc* fd;
   rv =
       file->OpenNSPRFileDesc(PR_RDWR | PR_CREATE_FILE | PR_TRUNCATE, 0600, &fd);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -295,7 +292,8 @@ nsresult CacheFileContextEvictor::PersistEvictionInfoToDisk(
 }
 
 nsresult CacheFileContextEvictor::RemoveEvictInfoFromDisk(
-    nsILoadContextInfo *aLoadContextInfo, bool aPinned) {
+    nsILoadContextInfo* aLoadContextInfo, bool aPinned,
+    const nsAString& aOrigin) {
   LOG(
       ("CacheFileContextEvictor::RemoveEvictInfoFromDisk() [this=%p, "
        "loadContextInfo=%p]",
@@ -306,7 +304,7 @@ nsresult CacheFileContextEvictor::RemoveEvictInfoFromDisk(
   MOZ_ASSERT(CacheFileIOManager::IsOnIOThread());
 
   nsCOMPtr<nsIFile> file;
-  rv = GetContextFile(aLoadContextInfo, aPinned, getter_AddRefs(file));
+  rv = GetContextFile(aLoadContextInfo, aPinned, aOrigin, getter_AddRefs(file));
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -339,13 +337,8 @@ nsresult CacheFileContextEvictor::LoadEvictInfoFromDisk() {
 
   sDiskAlreadySearched = true;
 
-  nsCOMPtr<nsISimpleEnumerator> enumerator;
-  rv = mCacheDirectory->GetDirectoryEntries(getter_AddRefs(enumerator));
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return rv;
-  }
-
-  nsCOMPtr<nsIDirectoryEnumerator> dirEnum = do_QueryInterface(enumerator, &rv);
+  nsCOMPtr<nsIDirectoryEnumerator> dirEnum;
+  rv = mCacheDirectory->GetDirectoryEntries(getter_AddRefs(dirEnum));
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -376,7 +369,7 @@ nsresult CacheFileContextEvictor::LoadEvictInfoFromDisk() {
       continue;
     }
 
-    if (!StringBeginsWith(leaf, NS_LITERAL_CSTRING(CONTEXT_EVICTION_PREFIX))) {
+    if (!StringBeginsWith(leaf, nsLiteralCString(CONTEXT_EVICTION_PREFIX))) {
       continue;
     }
 
@@ -400,8 +393,20 @@ nsresult CacheFileContextEvictor::LoadEvictInfoFromDisk() {
       decoded = Substring(decoded, 1);
     }
 
+    // Let's see if we have an origin.
+    nsAutoCString origin;
+    if (decoded.Contains('\t')) {
+      auto split = decoded.Split('\t');
+      MOZ_ASSERT(decoded.CountChar('\t') == 1);
+
+      auto splitIt = split.begin();
+      origin = *splitIt;
+      ++splitIt;
+      decoded = *splitIt;
+    }
+
     nsCOMPtr<nsILoadContextInfo> info;
-    if (!NS_LITERAL_CSTRING("*").Equals(decoded)) {
+    if (!"*"_ns.Equals(decoded)) {
       // "*" is indication of 'delete all', info left null will pass
       // to CacheFileContextEvictor::AddContext and clear all the cache data.
       info = CacheFileUtils::ParseKey(decoded);
@@ -421,9 +426,10 @@ nsresult CacheFileContextEvictor::LoadEvictInfoFromDisk() {
       continue;
     }
 
-    CacheFileContextEvictorEntry *entry = new CacheFileContextEvictorEntry();
+    CacheFileContextEvictorEntry* entry = new CacheFileContextEvictorEntry();
     entry->mInfo = info;
     entry->mPinned = pinned;
+    CopyUTF8toUTF16(origin, entry->mOrigin);
     entry->mTimeStamp = lastModifiedTime;
     mEntries.AppendElement(entry);
   }
@@ -432,12 +438,10 @@ nsresult CacheFileContextEvictor::LoadEvictInfoFromDisk() {
 }
 
 nsresult CacheFileContextEvictor::GetContextFile(
-    nsILoadContextInfo *aLoadContextInfo, bool aPinned, nsIFile **_retval) {
+    nsILoadContextInfo* aLoadContextInfo, bool aPinned,
+    const nsAString& aOrigin, nsIFile** _retval) {
   nsresult rv;
 
-  nsAutoCString leafName;
-  leafName.AssignLiteral(CONTEXT_EVICTION_PREFIX);
-
   nsAutoCString keyPrefix;
   if (aPinned) {
     // Mark pinned context files with a tab char at the start.
@@ -449,17 +453,21 @@ nsresult CacheFileContextEvictor::GetContextFile(
   } else {
     keyPrefix.Append('*');
   }
+  if (!aOrigin.IsEmpty()) {
+    keyPrefix.Append('\t');
+    keyPrefix.Append(NS_ConvertUTF16toUTF8(aOrigin));
+  }
 
-  nsAutoCString data64;
-  rv = Base64Encode(keyPrefix, data64);
+  nsAutoCString leafName;
+  leafName.AssignLiteral(CONTEXT_EVICTION_PREFIX);
+
+  rv = Base64EncodeAppend(keyPrefix, leafName);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
 
   // Replace '/' with '-' since '/' cannot be part of the filename.
-  data64.ReplaceChar('/', '-');
-
-  leafName.Append(data64);
+  leafName.ReplaceChar('/', '-');
 
   nsCOMPtr<nsIFile> file;
   rv = mCacheDirectory->Clone(getter_AddRefs(file));
@@ -516,7 +524,7 @@ void CacheFileContextEvictor::StartEvicting() {
   MOZ_ASSERT(CacheFileIOManager::IsOnIOThread());
 
   if (mEvicting) {
-    LOG(("CacheFileContextEvictor::StartEvicting() - already evicintg."));
+    LOG(("CacheFileContextEvictor::StartEvicting() - already evicting."));
     return;
   }
 
@@ -525,9 +533,9 @@ void CacheFileContextEvictor::StartEvicting() {
     return;
   }
 
-  nsCOMPtr<nsIRunnable> ev;
-  ev = NewRunnableMethod("net::CacheFileContextEvictor::EvictEntries", this,
-                         &CacheFileContextEvictor::EvictEntries);
+  nsCOMPtr<nsIRunnable> ev =
+      NewRunnableMethod("net::CacheFileContextEvictor::EvictEntries", this,
+                        &CacheFileContextEvictor::EvictEntries);
 
   RefPtr<CacheIOThread> ioThread = CacheFileIOManager::IOThread();
 
@@ -542,7 +550,7 @@ void CacheFileContextEvictor::StartEvicting() {
   mEvicting = true;
 }
 
-nsresult CacheFileContextEvictor::EvictEntries() {
+void CacheFileContextEvictor::EvictEntries() {
   LOG(("CacheFileContextEvictor::EvictEntries()"));
 
   nsresult rv;
@@ -555,7 +563,7 @@ nsresult CacheFileContextEvictor::EvictEntries() {
     LOG(
         ("CacheFileContextEvictor::EvictEntries() - Stopping evicting due to "
          "outdated index."));
-    return NS_OK;
+    return;
   }
 
   while (true) {
@@ -566,7 +574,7 @@ nsresult CacheFileContextEvictor::EvictEntries() {
       mEvicting =
           true;  // We don't want to start eviction again during shutdown
                  // process. Setting this flag to true ensures it.
-      return NS_OK;
+      return;
     }
 
     if (CacheIOThread::YieldAndRerun()) {
@@ -574,7 +582,7 @@ nsresult CacheFileContextEvictor::EvictEntries() {
           ("CacheFileContextEvictor::EvictEntries() - Breaking loop for higher "
            "level events."));
       mEvicting = true;
-      return NS_OK;
+      return;
     }
 
     if (mEntries.Length() == 0) {
@@ -585,7 +593,7 @@ nsresult CacheFileContextEvictor::EvictEntries() {
       // Allow index to notify AsyncGetDiskConsumption callbacks.  The size is
       // actual again.
       CacheIndex::OnAsyncEviction(false);
-      return NS_OK;
+      return;
     }
 
     SHA1Sum::Hash hash;
@@ -595,10 +603,12 @@ nsresult CacheFileContextEvictor::EvictEntries() {
           ("CacheFileContextEvictor::EvictEntries() - No more entries left in "
            "iterator. [iterator=%p, info=%p]",
            mEntries[0]->mIterator.get(), mEntries[0]->mInfo.get()));
-      RemoveEvictInfoFromDisk(mEntries[0]->mInfo, mEntries[0]->mPinned);
+      RemoveEvictInfoFromDisk(mEntries[0]->mInfo, mEntries[0]->mPinned,
+                              mEntries[0]->mOrigin);
       mEntries.RemoveElementAt(0);
       continue;
-    } else if (NS_FAILED(rv)) {
+    }
+    if (NS_FAILED(rv)) {
       LOG(
           ("CacheFileContextEvictor::EvictEntries() - Iterator failed to "
            "provide next hash (shutdown?), keeping eviction info on disk."
@@ -629,7 +639,7 @@ nsresult CacheFileContextEvictor::EvictEntries() {
 
     CacheIndex::EntryStatus status;
     bool pinned = false;
-    auto callback = [&pinned](const CacheIndexEntry *aEntry) {
+    auto callback = [&pinned](const CacheIndexEntry* aEntry) {
       pinned = aEntry->IsPinned();
     };
     rv = CacheIndex::HasEntry(hash, &status, callback);
@@ -646,6 +656,53 @@ nsresult CacheFileContextEvictor::EvictEntries() {
       continue;
     }
 
+    if (!mEntries[0]->mOrigin.IsEmpty()) {
+      nsCOMPtr<nsIFile> file;
+      CacheFileIOManager::gInstance->GetFile(&hash, getter_AddRefs(file));
+
+      // Read metadata from the file synchronously
+      RefPtr<CacheFileMetadata> metadata = new CacheFileMetadata();
+      rv = metadata->SyncReadMetadata(file);
+      if (NS_WARN_IF(NS_FAILED(rv))) {
+        continue;
+      }
+
+      // Now get the context + enhance id + URL from the key.
+      nsAutoCString uriSpec;
+      RefPtr<nsILoadContextInfo> info =
+          CacheFileUtils::ParseKey(metadata->GetKey(), nullptr, &uriSpec);
+      MOZ_ASSERT(info);
+      if (!info) {
+        continue;
+      }
+
+      nsCOMPtr<nsIURI> uri;
+      rv = NS_NewURI(getter_AddRefs(uri), uriSpec);
+      if (NS_FAILED(rv)) {
+        LOG(
+            ("CacheFileContextEvictor::EvictEntries() - Skipping entry since "
+             "NS_NewURI failed to parse the uriSpec"));
+        continue;
+      }
+
+      nsAutoString urlOrigin;
+      rv = nsContentUtils::GetUTFOrigin(uri, urlOrigin);
+      if (NS_FAILED(rv)) {
+        LOG(
+            ("CacheFileContextEvictor::EvictEntries() - Skipping entry since "
+             "We failed to extract an origin"));
+        continue;
+      }
+
+      if (!urlOrigin.Equals(mEntries[0]->mOrigin)) {
+        LOG(
+            ("CacheFileContextEvictor::EvictEntries() - Skipping entry since "
+             "origin "
+             "doesn't match"));
+        continue;
+      }
+    }
+
     nsAutoCString leafName;
     CacheFileIOManager::HashToStr(&hash, leafName);
 
@@ -678,9 +735,7 @@ nsresult CacheFileContextEvictor::EvictEntries() {
     CacheIndex::RemoveEntry(&hash);
   }
 
-  NS_NOTREACHED("We should never get here");
-  return NS_OK;
+  MOZ_ASSERT_UNREACHABLE("We should never get here");
 }
 
-}  // namespace net
-}  // namespace mozilla
+}  // namespace mozilla::net
