@@ -8,15 +8,21 @@
 
 #include "AlternateServices.h"
 #include "LoadInfo.h"
+#include "nsComponentManagerUtils.h"
 #include "nsEscape.h"
 #include "nsHttpConnectionInfo.h"
 #include "nsHttpChannel.h"
 #include "nsHttpHandler.h"
+#include "nsIOService.h"
 #include "nsThreadUtils.h"
 #include "nsHttpTransaction.h"
-#include "NullHttpTransaction.h"
 #include "nsISSLSocketControl.h"
 #include "nsIWellKnownOpportunisticUtils.h"
+#include "mozilla/Atomics.h"
+#include "mozilla/StaticPrefs_network.h"
+#include "mozilla/dom/PContent.h"
+#include "mozilla/SyncRunnable.h"
+#include "mozilla/net/AltSvcTransactionParent.h"
 
 /* RFC 7838 Alternative Services
    http://httpwg.org/http-extensions/opsec.html
@@ -38,7 +44,9 @@ static nsresult SchemeIsHTTPS(const nsACString& originScheme,
   outIsHTTPS = originScheme.EqualsLiteral("https");
 
   if (!outIsHTTPS && !originScheme.EqualsLiteral("http")) {
-    MOZ_ASSERT(false, "unexpected scheme");
+    MOZ_ASSERT(!originScheme.LowerCaseEqualsLiteral("https") &&
+                   !originScheme.LowerCaseEqualsLiteral("http"),
+               "The scheme should already be lowercase");
     return NS_ERROR_UNEXPECTED;
   }
   return NS_OK;
@@ -51,16 +59,16 @@ bool AltSvcMapping::AcceptableProxy(nsProxyInfo* proxyInfo) {
 void AltSvcMapping::ProcessHeader(
     const nsCString& buf, const nsCString& originScheme,
     const nsCString& originHost, int32_t originPort, const nsACString& username,
-    const nsACString& topWindowOrigin, bool privateBrowsing,
-    nsIInterfaceRequestor* callbacks, nsProxyInfo* proxyInfo, uint32_t caps,
-    const OriginAttributes& originAttributes) {
+    bool privateBrowsing, nsIInterfaceRequestor* callbacks,
+    nsProxyInfo* proxyInfo, uint32_t caps,
+    const OriginAttributes& originAttributes,
+    bool aDontValidate /* = false */) {  // aDontValidate is only used for
+                                         // testing
   MOZ_ASSERT(NS_IsMainThread());
   LOG(("AltSvcMapping::ProcessHeader: %s\n", buf.get()));
-  if (!callbacks) {
-    return;
-  }
 
-  if (!AcceptableProxy(proxyInfo)) {
+  if (StaticPrefs::network_http_altsvc_proxy_checks() &&
+      !AcceptableProxy(proxyInfo)) {
     LOG(("AltSvcMapping::ProcessHeader ignoring due to proxy\n"));
     return;
   }
@@ -78,12 +86,16 @@ void AltSvcMapping::ProcessHeader(
   ParsedHeaderValueListList parsedAltSvc(buf);
   int32_t numEntriesInHeader = parsedAltSvc.mValues.Length();
 
+  // Only use one http3 version.
+  bool http3Found = false;
+
   for (uint32_t index = 0; index < parsedAltSvc.mValues.Length(); ++index) {
     uint32_t maxage = 86400;  // default
     nsAutoCString hostname;
     nsAutoCString npnToken;
     int32_t portno = originPort;
     bool clearEntry = false;
+    bool isHttp3 = false;
 
     for (uint32_t pairIndex = 0;
          pairIndex < parsedAltSvc.mValues[index].mValues.Length();
@@ -101,8 +113,11 @@ void AltSvcMapping::ProcessHeader(
           break;
         }
 
-        // h2=[hostname]:443
+        // h2=[hostname]:443 or h3-xx=[hostname]:port
+        // XX is current version we support and it is define in nsHttp.h.
+        isHttp3 = gHttpHandler->IsHttp3VersionSupported(currentName);
         npnToken = currentName;
+
         int32_t colonIndex = currentValue.FindChar(':');
         if (colonIndex >= 0) {
           portno =
@@ -113,7 +128,6 @@ void AltSvcMapping::ProcessHeader(
         hostname.Assign(currentValue.BeginReading(), colonIndex);
       } else if (currentName.EqualsLiteral("ma")) {
         maxage = atoi(PromiseFlatCString(currentValue).get());
-        break;
       } else {
         LOG(("Alt Svc ignoring parameter %s", currentName.BeginReading()));
       }
@@ -124,8 +138,8 @@ void AltSvcMapping::ProcessHeader(
       originAttributes.CreateSuffix(suffix);
       LOG(("Alt Svc clearing mapping for %s:%d:%s", originHost.get(),
            originPort, suffix.get()));
-      gHttpHandler->ConnMgr()->ClearHostMapping(originHost, originPort,
-                                                originAttributes);
+      gHttpHandler->AltServiceCache()->ClearHostMapping(originHost, originPort,
+                                                        originAttributes);
       continue;
     }
 
@@ -139,29 +153,45 @@ void AltSvcMapping::ProcessHeader(
     nsUnescape(npnToken.BeginWriting());
     npnToken.SetLength(strlen(npnToken.BeginReading()));
 
+    if (http3Found && isHttp3) {
+      LOG(("Alt Svc ignore multiple Http3 options (%s)", npnToken.get()));
+      continue;
+    }
+
     uint32_t spdyIndex;
     SpdyInformation* spdyInfo = gHttpHandler->SpdyInfo();
     if (!(NS_SUCCEEDED(spdyInfo->GetNPNIndex(npnToken, &spdyIndex)) &&
-          spdyInfo->ProtocolEnabled(spdyIndex))) {
+          spdyInfo->ProtocolEnabled(spdyIndex)) &&
+        !(isHttp3 && gHttpHandler->IsHttp3Enabled() &&
+          !gHttpHandler->IsHttp3Excluded(hostname.IsEmpty() ? originHost
+                                                            : hostname))) {
       LOG(("Alt Svc unknown protocol %s, ignoring", npnToken.get()));
       continue;
     }
 
-    RefPtr<AltSvcMapping> mapping = new AltSvcMapping(
-        gHttpHandler->ConnMgr()->GetStoragePtr(),
-        gHttpHandler->ConnMgr()->StorageEpoch(), originScheme, originHost,
-        originPort, username, topWindowOrigin, privateBrowsing,
-        NowInSeconds() + maxage, hostname, portno, npnToken, originAttributes);
+    if (isHttp3) {
+      http3Found = true;
+    }
+
+    RefPtr<AltSvcMapping> mapping =
+        new AltSvcMapping(gHttpHandler->AltServiceCache()->GetStoragePtr(),
+                          gHttpHandler->AltServiceCache()->StorageEpoch(),
+                          originScheme, originHost, originPort, username,
+                          privateBrowsing, NowInSeconds() + maxage, hostname,
+                          portno, npnToken, originAttributes, isHttp3);
     if (mapping->TTL() <= 0) {
       LOG(("Alt Svc invalid map"));
       mapping = nullptr;
       // since this isn't a parse error, let's clear any existing mapping
       // as that would have happened if we had accepted the parameters.
-      gHttpHandler->ConnMgr()->ClearHostMapping(originHost, originPort,
-                                                originAttributes);
-    } else {
+      gHttpHandler->AltServiceCache()->ClearHostMapping(originHost, originPort,
+                                                        originAttributes);
+    } else if (!aDontValidate) {
       gHttpHandler->UpdateAltServiceMapping(mapping, proxyInfo, callbacks, caps,
                                             originAttributes);
+    } else {
+      gHttpHandler->UpdateAltServiceMappingWithoutValidation(
+          mapping, proxyInfo, callbacks, caps, originAttributes);
     }
   }
 
@@ -174,12 +204,12 @@ void AltSvcMapping::ProcessHeader(
 AltSvcMapping::AltSvcMapping(DataStorage* storage, int32_t epoch,
                              const nsACString& originScheme,
                              const nsACString& originHost, int32_t originPort,
-                             const nsACString& username,
-                             const nsACString& topWindowOrigin,
-                             bool privateBrowsing, uint32_t expiresAt,
+                             const nsACString& username, bool privateBrowsing,
+                             uint32_t expiresAt,
                              const nsACString& alternateHost,
                              int32_t alternatePort, const nsACString& npnToken,
-                             const OriginAttributes& originAttributes)
+                             const OriginAttributes& originAttributes,
+                             bool aIsHttp3)
     : mStorage(storage),
       mStorageEpoch(epoch),
       mAlternateHost(alternateHost),
@@ -187,14 +217,11 @@ AltSvcMapping::AltSvcMapping(DataStorage* storage, int32_t epoch,
       mOriginHost(originHost),
       mOriginPort(originPort),
       mUsername(username),
-      mTopWindowOrigin(topWindowOrigin),
       mPrivate(privateBrowsing),
       mExpiresAt(expiresAt),
-      mValidated(false),
-      mMixedScheme(false),
       mNPNToken(npnToken),
       mOriginAttributes(originAttributes),
-      mSyncOnlyOnSuccess(false) {
+      mIsHttp3(aIsHttp3) {
   MOZ_ASSERT(NS_IsMainThread());
 
   if (NS_FAILED(SchemeIsHTTPS(originScheme, mHttps))) {
@@ -218,14 +245,19 @@ AltSvcMapping::AltSvcMapping(DataStorage* storage, int32_t epoch,
   }
 
   if ((mAlternatePort == mOriginPort) &&
-      mAlternateHost.EqualsIgnoreCase(mOriginHost.get())) {
+      mAlternateHost.EqualsIgnoreCase(mOriginHost.get()) && !mIsHttp3) {
+    // Http2 on the same host:port does not make sense because we are
+    // connecting to the same end point over the same protocol (TCP) as with
+    // original host. On the other hand, for Http3 alt-svc can be hosted on
+    // the same host:port because protocol(UDP vs. TCP) is always different and
+    // we are not connecting to the same end point.
     LOG(("Alt Svc is also origin Svc - ignoring\n"));
     mExpiresAt = 0;  // invalid
   }
 
   if (mExpiresAt) {
     MakeHashKey(mHashKey, originScheme, mOriginHost, mOriginPort, mPrivate,
-                mOriginAttributes);
+                mOriginAttributes, mIsHttp3);
   }
 }
 
@@ -233,7 +265,8 @@ void AltSvcMapping::MakeHashKey(nsCString& outKey,
                                 const nsACString& originScheme,
                                 const nsACString& originHost,
                                 int32_t originPort, bool privateBrowsing,
-                                const OriginAttributes& originAttributes) {
+                                const OriginAttributes& originAttributes,
+                                bool aHttp3) {
   outKey.Truncate();
 
   if (originPort == -1) {
@@ -252,6 +285,9 @@ void AltSvcMapping::MakeHashKey(nsCString& outKey,
   nsAutoCString suffix;
   originAttributes.CreateSuffix(suffix);
   outKey.Append(suffix);
+  outKey.Append(':');
+
+  outKey.Append(aHttp3 ? '3' : '.');
 }
 
 int32_t AltSvcMapping::TTL() { return mExpiresAt - NowInSeconds(); }
@@ -317,8 +353,8 @@ void AltSvcMapping::GetConnectionInfo(
     nsHttpConnectionInfo** outCI, nsProxyInfo* pi,
     const OriginAttributes& originAttributes) {
   RefPtr<nsHttpConnectionInfo> ci = new nsHttpConnectionInfo(
-      mOriginHost, mOriginPort, mNPNToken, mUsername, mTopWindowOrigin, pi,
-      originAttributes, mAlternateHost, mAlternatePort);
+      mOriginHost, mOriginPort, mNPNToken, mUsername, pi, originAttributes,
+      mAlternateHost, mAlternatePort, mIsHttp3);
 
   // http:// without the mixed-scheme attribute needs to be segmented in the
   // connection manager connection information hash with this attribute
@@ -331,7 +367,7 @@ void AltSvcMapping::GetConnectionInfo(
 
 void AltSvcMapping::Serialize(nsCString& out) {
   // Be careful, when serializing new members, add them to the end of this list.
-  out = mHttps ? NS_LITERAL_CSTRING("https:") : NS_LITERAL_CSTRING("http:");
+  out = mHttps ? "https:"_ns : "http:"_ns;
   out.Append(mOriginHost);
   out.Append(':');
   out.AppendInt(mOriginPort);
@@ -358,14 +394,18 @@ void AltSvcMapping::Serialize(nsCString& out) {
   mOriginAttributes.CreateSuffix(suffix);
   out.Append(suffix);
   out.Append(':');
-  out.Append(mTopWindowOrigin);
-  out.Append('|');  // Be careful, the top window origin may contain colons!
+  out.Append(""_ns);  // Formerly topWindowOrigin. Now unused empty string.
+  out.Append('|');    // Be careful, the top window origin may contain colons!
+  out.Append('n');  // Formerly mIsolated. Now always 'n'. Should remove someday
+  out.Append(':');
+  out.Append(mIsHttp3 ? 'y' : 'n');
+  out.Append(':');
   // Add code to serialize new members here!
 }
 
 AltSvcMapping::AltSvcMapping(DataStorage* storage, int32_t epoch,
                              const nsCString& str)
-    : mStorage(storage), mStorageEpoch(epoch), mSyncOnlyOnSuccess(false) {
+    : mStorage(storage), mStorageEpoch(epoch) {
   mValidated = false;
   nsresult code;
   char separator = ':';
@@ -419,136 +459,154 @@ AltSvcMapping::AltSvcMapping(DataStorage* storage, int32_t epoch,
     // origin string can contain colons.
     separator = '|';
     _NS_NEXT_TOKEN;
-    mTopWindowOrigin = Substring(str, start, idx - start);
+    // TopWindowOrigin used to be encoded here. Now it's unused.
     separator = ':';
+    _NS_NEXT_TOKEN;
+    // mIsolated used to be encoded here. Now it's unused.
+    _NS_NEXT_TOKEN;
+    mIsHttp3 = Substring(str, start, idx - start).EqualsLiteral("y");
     // Add code to deserialize new members here!
 #undef _NS_NEXT_TOKEN
 
-    MakeHashKey(
-        mHashKey,
-        mHttps ? NS_LITERAL_CSTRING("https") : NS_LITERAL_CSTRING("http"),
-        mOriginHost, mOriginPort, mPrivate, mOriginAttributes);
+    MakeHashKey(mHashKey, mHttps ? "https"_ns : "http"_ns, mOriginHost,
+                mOriginPort, mPrivate, mOriginAttributes, mIsHttp3);
   } while (false);
 }
 
-// This is the asynchronous null transaction used to validate
-// an alt-svc advertisement only for https://
-class AltSvcTransaction final : public NullHttpTransaction {
- public:
-  AltSvcTransaction(AltSvcMapping* map, nsHttpConnectionInfo* ci,
-                    nsIInterfaceRequestor* callbacks, uint32_t caps)
-      : NullHttpTransaction(ci, callbacks, caps & ~NS_HTTP_ALLOW_KEEPALIVE),
-        mMapping(map),
-        mRunning(true),
-        mTriedToValidate(false),
-        mTriedToWrite(false) {
-    LOG(("AltSvcTransaction ctor %p map %p [%s -> %s]", this, map,
-         map->OriginHost().get(), map->AlternateHost().get()));
-    MOZ_ASSERT(mMapping);
-    MOZ_ASSERT(mMapping->HTTPS());
+AltSvcMappingValidator::AltSvcMappingValidator(AltSvcMapping* aMap)
+    : mMapping(aMap) {
+  LOG(("AltSvcMappingValidator ctor %p map %p [%s -> %s]", this, aMap,
+       aMap->OriginHost().get(), aMap->AlternateHost().get()));
+  MOZ_ASSERT(mMapping);
+  MOZ_ASSERT(mMapping->HTTPS());  // http:// uses the .wk path
+}
+
+void AltSvcMappingValidator::OnTransactionDestroy(bool aValidateResult) {
+  mMapping->SetValidated(aValidateResult);
+  if (!mMapping->Validated()) {
+    // try again later
+    mMapping->SetExpiresAt(NowInSeconds() + 2);
+  }
+  LOG(
+      ("AltSvcMappingValidator::OnTransactionDestroy %p map %p validated %d "
+       "[%s]",
+       this, mMapping.get(), mMapping->Validated(), mMapping->HashKey().get()));
+}
+
+void AltSvcMappingValidator::OnTransactionClose(bool aValidateResult) {
+  mMapping->SetValidated(aValidateResult);
+  LOG(
+      ("AltSvcMappingValidator::OnTransactionClose %p map %p validated %d "
+       "[%s]",
+       this, mMapping.get(), mMapping->Validated(), mMapping->HashKey().get()));
+}
+
+template <class Validator>
+AltSvcTransaction<Validator>::AltSvcTransaction(
+    nsHttpConnectionInfo* ci, nsIInterfaceRequestor* callbacks, uint32_t caps,
+    Validator* aValidator, bool aIsHttp3)
+    : SpeculativeTransaction(ci, callbacks, caps & ~NS_HTTP_ALLOW_KEEPALIVE),
+      mValidator(aValidator),
+      mIsHttp3(aIsHttp3),
+      mRunning(true),
+      mTriedToValidate(false),
+      mTriedToWrite(false),
+      mValidatedResult(false) {
+  MOZ_ASSERT_IF(nsIOService::UseSocketProcess(), XRE_IsSocketProcess());
+  MOZ_ASSERT_IF(!nsIOService::UseSocketProcess(), XRE_IsParentProcess());
+}
+
+template <class Validator>
+AltSvcTransaction<Validator>::~AltSvcTransaction() {
+  LOG(("AltSvcTransaction dtor %p running %d", this, mRunning));
+
+  if (mRunning) {
+    mValidatedResult = MaybeValidate(NS_OK);
+    mValidator->OnTransactionDestroy(mValidatedResult);
+  }
+}
+
+template <class Validator>
+bool AltSvcTransaction<Validator>::MaybeValidate(nsresult reason) {
+  if (mTriedToValidate) {
+    return mValidatedResult;
+  }
+  mTriedToValidate = true;
+
+  LOG(("AltSvcTransaction::MaybeValidate() %p reason=%" PRIx32
+       " running=%d conn=%p write=%d",
+       this, static_cast<uint32_t>(reason), mRunning, mConnection.get(),
+       mTriedToWrite));
+
+  if (mTriedToWrite && reason == NS_BASE_STREAM_CLOSED) {
+    // The normal course of events is to cause the transaction to fail with
+    // CLOSED on a write - so that's a success that means the HTTP/2 session
+    // is setup.
+    reason = NS_OK;
   }
 
-  ~AltSvcTransaction() override {
-    LOG(("AltSvcTransaction dtor %p map %p running %d", this, mMapping.get(),
-         mRunning));
-
-    if (mRunning) {
-      MaybeValidate(NS_OK);
-    }
-    if (!mMapping->Validated()) {
-      // try again later
-      mMapping->SetExpiresAt(NowInSeconds() + 2);
-    }
-    LOG(("AltSvcTransaction dtor %p map %p validated %d [%s]", this,
-         mMapping.get(), mMapping->Validated(), mMapping->HashKey().get()));
+  if (NS_FAILED(reason) || !mRunning || !mConnection) {
+    LOG(("AltSvcTransaction::MaybeValidate %p Failed due to precondition",
+         this));
+    return false;
   }
 
- private:
-  // check on alternate route.
-  // also evaluate 'reasonable assurances' for opportunistic security
-  void MaybeValidate(nsresult reason) {
-    MOZ_ASSERT(mMapping->HTTPS());  // http:// uses the .wk path
+  // insist on >= http/2
+  HttpVersion version = mConnection->Version();
+  LOG(("AltSvcTransaction::MaybeValidate() %p version %d\n", this,
+       static_cast<int32_t>(version)));
+  if ((!mIsHttp3 && (version != HttpVersion::v2_0)) ||
+      (mIsHttp3 && (version != HttpVersion::v3_0))) {
+    LOG(
+        ("AltSvcTransaction::MaybeValidate %p Failed due to protocol version"
+         " expacted %s.",
+         this, mIsHttp3 ? "Http3" : "Http2"));
+    return false;
+  }
 
-    if (mTriedToValidate) {
-      return;
-    }
-    mTriedToValidate = true;
+  nsCOMPtr<nsISupports> secInfo;
+  mConnection->GetSecurityInfo(getter_AddRefs(secInfo));
+  nsCOMPtr<nsISSLSocketControl> socketControl = do_QueryInterface(secInfo);
 
-    LOG(("AltSvcTransaction::MaybeValidate() %p reason=%" PRIx32
-         " running=%d conn=%p write=%d",
-         this, static_cast<uint32_t>(reason), mRunning, mConnection.get(),
-         mTriedToWrite));
-
-    if (mTriedToWrite && reason == NS_BASE_STREAM_CLOSED) {
-      // The normal course of events is to cause the transaction to fail with
-      // CLOSED on a write - so that's a success that means the HTTP/2 session
-      // is setup.
-      reason = NS_OK;
-    }
-
-    if (NS_FAILED(reason) || !mRunning || !mConnection) {
-      LOG(("AltSvcTransaction::MaybeValidate %p Failed due to precondition",
-           this));
-      return;
-    }
-
-    // insist on >= http/2
-    HttpVersion version = mConnection->Version();
-    LOG(("AltSvcTransaction::MaybeValidate() %p version %d\n", this,
-         static_cast<int32_t>(version)));
-    if (version != HttpVersion::v2_0) {
-      LOG(("AltSvcTransaction::MaybeValidate %p Failed due to protocol version",
-           this));
-      return;
-    }
-
-    nsCOMPtr<nsISupports> secInfo;
-    mConnection->GetSecurityInfo(getter_AddRefs(secInfo));
-    nsCOMPtr<nsISSLSocketControl> socketControl = do_QueryInterface(secInfo);
-
-    LOG(("AltSvcTransaction::MaybeValidate() %p socketControl=%p\n", this,
-         socketControl.get()));
-
-    if (socketControl->GetFailedVerification()) {
-      LOG(
-          ("AltSvcTransaction::MaybeValidate() %p "
-           "not validated due to auth error",
-           this));
-      return;
-    }
+  LOG(("AltSvcTransaction::MaybeValidate() %p socketControl=%p\n", this,
+       socketControl.get()));
 
+  if (socketControl->GetFailedVerification()) {
     LOG(
         ("AltSvcTransaction::MaybeValidate() %p "
-         "validating alternate service with successful auth check",
+         "not validated due to auth error",
          this));
-    mMapping->SetValidated(true);
+    return false;
   }
 
- public:
-  void Close(nsresult reason) override {
-    LOG(("AltSvcTransaction::Close() %p reason=%" PRIx32 " running %d", this,
-         static_cast<uint32_t>(reason), mRunning));
+  LOG(
+      ("AltSvcTransaction::MaybeValidate() %p "
+       "validating alternate service with successful auth check",
+       this));
 
-    MaybeValidate(reason);
-    if (!mMapping->Validated() && mConnection) {
-      mConnection->DontReuse();
-    }
-    NullHttpTransaction::Close(reason);
+  return true;
+}
+
+template <class Validator>
+void AltSvcTransaction<Validator>::Close(nsresult reason) {
+  LOG(("AltSvcTransaction::Close() %p reason=%" PRIx32 " running %d", this,
+       static_cast<uint32_t>(reason), mRunning));
+
+  mValidatedResult = MaybeValidate(reason);
+  mValidator->OnTransactionClose(mValidatedResult);
+  if (!mValidatedResult && mConnection) {
+    mConnection->DontReuse();
   }
+  NullHttpTransaction::Close(reason);
+}
 
-  nsresult ReadSegments(nsAHttpSegmentReader* reader, uint32_t count,
-                        uint32_t* countRead) override {
-    LOG(("AltSvcTransaction::ReadSegements() %p\n", this));
-    mTriedToWrite = true;
-    return NullHttpTransaction::ReadSegments(reader, count, countRead);
-  }
-
- private:
-  RefPtr<AltSvcMapping> mMapping;
-  uint32_t mRunning : 1;
-  uint32_t mTriedToValidate : 1;
-  uint32_t mTriedToWrite : 1;
-};
+template <class Validator>
+nsresult AltSvcTransaction<Validator>::ReadSegments(
+    nsAHttpSegmentReader* reader, uint32_t count, uint32_t* countRead) {
+  LOG(("AltSvcTransaction::ReadSegements() %p\n", this));
+  mTriedToWrite = true;
+  return NullHttpTransaction::ReadSegments(reader, count, countRead);
+}
 
 class WellKnownChecker {
  public:
@@ -571,9 +629,11 @@ class WellKnownChecker {
     LOG(("WellKnownChecker::Start %p\n", this));
     nsCOMPtr<nsILoadInfo> loadInfo =
         new LoadInfo(nsContentUtils::GetSystemPrincipal(), nullptr, nullptr,
-                     nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
+                     nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL,
                      nsIContentPolicy::TYPE_OTHER);
     loadInfo->SetOriginAttributes(mCI->GetOriginAttributes());
+    // allow deprecated HTTP request from SystemPrincipal
+    loadInfo->SetAllowDeprecatedSystemRequests(true);
 
     RefPtr<nsHttpChannel> chan = new nsHttpChannel();
     nsresult rv;
@@ -613,7 +673,7 @@ class WellKnownChecker {
         LOG(("WellKnownChecker::Done %p alternate was not 200 response code\n",
              this));
       } else if (!mTransactionAlternate->mVersionOK) {
-        LOG(("WellKnownChecker::Done %p alternate was not at least h2\n",
+        LOG(("WellKnownChecker::Done %p alternate was not at least h2 or h3\n",
              this));
       } else if (!mTransactionAlternate->mWKResponse.Equals(
                      mTransactionOrigin->mWKResponse)) {
@@ -640,38 +700,15 @@ class WellKnownChecker {
       if (accepted) {
         MOZ_ASSERT(!mMapping->HTTPS());  // https:// does not use .wk
 
-        nsresult rv = uu->Verify(mTransactionAlternate->mWKResponse, mOrigin,
-                                 mAlternatePort);
+        nsresult rv = uu->Verify(mTransactionAlternate->mWKResponse, mOrigin);
         if (NS_SUCCEEDED(rv)) {
           bool validWK = false;
-          bool mixedScheme = false;
-          int32_t lifetime = 0;
           Unused << uu->GetValid(&validWK);
-          Unused << uu->GetLifetime(&lifetime);
-          Unused << uu->GetMixed(&mixedScheme);
           if (!validWK) {
             LOG(("WellKnownChecker::Done %p json parser declares invalid\n%s\n",
                  this, mTransactionAlternate->mWKResponse.get()));
             accepted = false;
           }
-          if (accepted && (lifetime > 0)) {
-            if (mMapping->TTL() > lifetime) {
-              LOG((
-                  "WellKnownChecker::Done %p atl-svc lifetime reduced by .wk\n",
-                  this));
-              mMapping->SetExpiresAt(NowInSeconds() + lifetime);
-            } else {
-              LOG(
-                  ("WellKnownChecker::Done %p .wk lifetime exceeded alt-svc ma "
-                   "so ignored\n",
-                   this));
-            }
-          }
-          if (accepted && mixedScheme) {
-            mMapping->SetMixedScheme(true);
-            LOG(("WellKnownChecker::Done %p atl-svc .wk allows mixed scheme\n",
-                 this));
-          }
         } else {
           LOG(("WellKnownChecker::Done %p .wk jason eval failed to run\n",
                this));
@@ -704,9 +741,8 @@ class WellKnownChecker {
     uint64_t channelId;
     nsLoadFlags flags;
 
-    nsContentPolicyType contentPolicyType =
-        loadInfo ? loadInfo->GetExternalContentPolicyType()
-                 : nsIContentPolicy::TYPE_OTHER;
+    ExtContentPolicyType contentPolicyType =
+        loadInfo->GetExternalContentPolicyType();
 
     if (NS_FAILED(gHttpHandler->NewChannelId(channelId)) ||
         NS_FAILED(chan->Init(uri, caps, nullptr, 0, nullptr, channelId,
@@ -745,45 +781,28 @@ TransactionObserver::TransactionObserver(nsHttpChannel* channel,
     : mChannel(channel),
       mChecker(checker),
       mRanOnce(false),
+      mStatusOK(false),
       mAuthOK(false),
-      mVersionOK(false),
-      mStatusOK(false) {
+      mVersionOK(false) {
   LOG(("TransactionObserver ctor %p channel %p checker %p\n", this, channel,
        checker));
   mChannelRef = do_QueryInterface((nsIHttpChannel*)channel);
 }
 
-void TransactionObserver::Complete(nsHttpTransaction* aTrans, nsresult reason) {
-  // socket thread
-  MOZ_ASSERT(!NS_IsMainThread());
+void TransactionObserver::Complete(bool versionOK, bool authOK,
+                                   nsresult reason) {
   if (mRanOnce) {
     return;
   }
   mRanOnce = true;
 
-  RefPtr<nsAHttpConnection> conn = aTrans->Connection();
-  LOG(("TransactionObserver::Complete %p aTrans %p reason %" PRIx32
-       " conn %p\n",
-       this, aTrans, static_cast<uint32_t>(reason), conn.get()));
-  if (!conn) {
-    return;
-  }
-  HttpVersion version = conn->Version();
-  mVersionOK = (((reason == NS_BASE_STREAM_CLOSED) || (reason == NS_OK)) &&
-                conn->Version() == HttpVersion::v2_0);
+  mVersionOK = versionOK;
+  mAuthOK = authOK;
 
-  nsCOMPtr<nsISupports> secInfo;
-  conn->GetSecurityInfo(getter_AddRefs(secInfo));
-  nsCOMPtr<nsISSLSocketControl> socketControl = do_QueryInterface(secInfo);
-  LOG(("TransactionObserver::Complete version %u socketControl %p\n",
-       static_cast<int32_t>(version), socketControl.get()));
-  if (!socketControl) {
-    return;
-  }
-
-  mAuthOK = !socketControl->GetFailedVerification();
-  LOG(("TransactionObserve::Complete %p trans %p authOK %d versionOK %d\n",
-       this, aTrans, mAuthOK, mVersionOK));
+  LOG(
+      ("TransactionObserve::Complete %p authOK %d versionOK %d"
+       " reason %" PRIx32,
+       this, authOK, versionOK, static_cast<uint32_t>(reason)));
 }
 
 #define MAX_WK 32768
@@ -804,11 +823,11 @@ TransactionObserver::OnDataAvailable(nsIRequest* aRequest,
   uint32_t oldLen = mWKResponse.Length();
   uint64_t newLen = aCount + oldLen;
   if (newLen < MAX_WK) {
-    nsresult rv;
-    auto handle = mWKResponse.BulkWrite(newLen, oldLen, false, rv);
-    if (NS_FAILED(rv)) {
-      return rv;
+    auto handleOrErr = mWKResponse.BulkWrite(newLen, oldLen, false);
+    if (handleOrErr.isErr()) {
+      return handleOrErr.unwrapErr();
     }
+    auto handle = handleOrErr.unwrap();
     uint32_t amtRead;
     if (NS_SUCCEEDED(
             aStream->Read(handle.Elements() + oldLen, aCount, &amtRead))) {
@@ -840,6 +859,49 @@ TransactionObserver::OnStopRequest(nsIRequest* aRequest, nsresult code) {
   return NS_OK;
 }
 
+void AltSvcCache::EnsureStorageInited() {
+  static Atomic<bool> initialized(false);
+
+  if (initialized) {
+    return;
+  }
+
+  auto initTask = [&]() {
+    MOZ_ASSERT(NS_IsMainThread());
+
+    // DataStorage gives synchronous access to a memory based hash table
+    // that is backed by disk where those writes are done asynchronously
+    // on another thread
+    mStorage = DataStorage::Get(DataStorageClass::AlternateServices);
+    if (!mStorage) {
+      LOG(("AltSvcCache::EnsureStorageInited WARN NO STORAGE\n"));
+      return;
+    }
+
+    if (NS_FAILED(mStorage->Init())) {
+      mStorage = nullptr;
+    } else {
+      initialized = true;
+    }
+
+    mStorageEpoch = NowInSeconds();
+  };
+
+  if (NS_IsMainThread()) {
+    initTask();
+    return;
+  }
+
+  nsCOMPtr<nsIEventTarget> main = GetMainThreadEventTarget();
+  if (!main) {
+    return;
+  }
+
+  SyncRunnable::DispatchToThread(
+      main, new SyncRunnable(NS_NewRunnableFunction(
+                "AltSvcCache::EnsureStorageInited", initTask)));
+}
+
 already_AddRefed<AltSvcMapping> AltSvcCache::LookupMapping(
     const nsCString& key, bool privateBrowsing) {
   LOG(("AltSvcCache::LookupMapping %p %s\n", this, key.get()));
@@ -847,6 +909,13 @@ already_AddRefed<AltSvcMapping> AltSvcCache::LookupMapping(
     LOG(("AltSvcCache::LookupMapping %p no backing store\n", this));
     return nullptr;
   }
+
+  if (NS_IsMainThread() && !mStorage->IsReady()) {
+    LOG(("AltSvcCache::LookupMapping %p skip when storage is not ready\n",
+         this));
+    return nullptr;
+  }
+
   nsCString val(mStorage->Get(
       key, privateBrowsing ? DataStorage_Private : DataStorage_Persistent));
   if (val.IsEmpty()) {
@@ -864,6 +933,17 @@ already_AddRefed<AltSvcMapping> AltSvcCache::LookupMapping(
     return nullptr;
   }
 
+  if (rv->IsHttp3() &&
+      (!gHttpHandler->IsHttp3Enabled() ||
+       !gHttpHandler->IsHttp3VersionSupported(rv->NPNToken()) ||
+       gHttpHandler->IsHttp3Excluded(rv->AlternateHost()))) {
+    // If Http3 is disabled or the version not supported anymore, remove the
+    // mapping.
+    mStorage->Remove(
+        key, rv->Private() ? DataStorage_Private : DataStorage_Persistent);
+    return nullptr;
+  }
+
   if (rv->TTL() <= 0) {
     LOG(("AltSvcCache::LookupMapping %p expired hit - MISS\n", this));
     mStorage->Remove(
@@ -876,6 +956,25 @@ already_AddRefed<AltSvcMapping> AltSvcCache::LookupMapping(
   return rv.forget();
 }
 
+// This is only used for testing!
+void AltSvcCache::UpdateAltServiceMappingWithoutValidation(
+    AltSvcMapping* map, nsProxyInfo* pi, nsIInterfaceRequestor* aCallbacks,
+    uint32_t caps, const OriginAttributes& originAttributes) {
+  MOZ_ASSERT(NS_IsMainThread());
+  if (!mStorage) {
+    return;
+  }
+  RefPtr<AltSvcMapping> existing =
+      LookupMapping(map->HashKey(), map->Private());
+  LOG(
+      ("AltSvcCache::UpdateAltServiceMappingWithoutValidation %p map %p "
+       "existing %p %s",
+       this, map, existing.get(), map->AlternateHost().get()));
+  if (!existing) {
+    map->SetValidated(true);
+  }
+}
+
 void AltSvcCache::UpdateAltServiceMapping(
     AltSvcMapping* map, nsProxyInfo* pi, nsIInterfaceRequestor* aCallbacks,
     uint32_t caps, const OriginAttributes& originAttributes) {
@@ -965,10 +1064,22 @@ void AltSvcCache::UpdateAltServiceMapping(
          this));
     // for https resources we only establish a connection
     nsCOMPtr<nsIInterfaceRequestor> callbacks = new AltSvcOverride(aCallbacks);
-    RefPtr<AltSvcTransaction> nullTransaction =
-        new AltSvcTransaction(map, ci, aCallbacks, caps);
-    nsresult rv = gHttpHandler->ConnMgr()->SpeculativeConnect(
-        ci, callbacks, caps, nullTransaction);
+    RefPtr<AltSvcMappingValidator> validator = new AltSvcMappingValidator(map);
+    RefPtr<SpeculativeTransaction> transaction;
+    if (nsIOService::UseSocketProcess()) {
+      RefPtr<AltSvcTransactionParent> parent =
+          new AltSvcTransactionParent(ci, aCallbacks, caps, validator);
+      if (!parent->Init()) {
+        return;
+      }
+      transaction = parent;
+    } else {
+      transaction = new AltSvcTransaction<AltSvcMappingValidator>(
+          ci, aCallbacks, caps, validator, map->IsHttp3());
+    }
+
+    nsresult rv =
+        gHttpHandler->SpeculativeConnect(ci, callbacks, caps, transaction);
     if (NS_FAILED(rv)) {
       LOG(
           ("AltSvcCache::UpdateAltServiceMapping %p "
@@ -977,7 +1088,7 @@ void AltSvcCache::UpdateAltServiceMapping(
     }
   } else {
     // for http:// resources we fetch .well-known too
-    nsAutoCString origin(NS_LITERAL_CSTRING("http://"));
+    nsAutoCString origin("http://"_ns);
 
     // Check whether origin is an ipv6 address. In that case we need to add
     // '[]'.
@@ -1017,25 +1128,11 @@ void AltSvcCache::UpdateAltServiceMapping(
 
 already_AddRefed<AltSvcMapping> AltSvcCache::GetAltServiceMapping(
     const nsACString& scheme, const nsACString& host, int32_t port,
-    bool privateBrowsing, const OriginAttributes& originAttributes) {
-  bool isHTTPS;
-  MOZ_ASSERT(NS_IsMainThread());
-  if (!mStorage) {
-    // DataStorage gives synchronous access to a memory based hash table
-    // that is backed by disk where those writes are done asynchronously
-    // on another thread
-    mStorage = DataStorage::Get(DataStorageClass::AlternateServices);
-    if (mStorage) {
-      if (NS_FAILED(mStorage->Init(nullptr))) {
-        mStorage = nullptr;
-      }
-    }
-    if (!mStorage) {
-      LOG(("AltSvcCache::GetAltServiceMapping WARN NO STORAGE\n"));
-    }
-    mStorageEpoch = NowInSeconds();
-  }
+    bool privateBrowsing, const OriginAttributes& originAttributes,
+    bool aHttp2Allowed, bool aHttp3Allowed) {
+  EnsureStorageInited();
 
+  bool isHTTPS;
   if (NS_FAILED(SchemeIsHTTPS(scheme, isHTTPS))) {
     return nullptr;
   }
@@ -1046,19 +1143,39 @@ already_AddRefed<AltSvcMapping> AltSvcCache::GetAltServiceMapping(
     return nullptr;
   }
 
-  nsAutoCString key;
-  AltSvcMapping::MakeHashKey(key, scheme, host, port, privateBrowsing,
-                             originAttributes);
-  RefPtr<AltSvcMapping> existing = LookupMapping(key, privateBrowsing);
-  LOG(
-      ("AltSvcCache::GetAltServiceMapping %p key=%s "
-       "existing=%p validated=%d ttl=%d",
-       this, key.get(), existing.get(), existing ? existing->Validated() : 0,
-       existing ? existing->TTL() : 0));
-  if (existing && !existing->Validated()) {
-    existing = nullptr;
+  // First look for HTTP3
+  if (aHttp3Allowed) {
+    nsAutoCString key;
+    AltSvcMapping::MakeHashKey(key, scheme, host, port, privateBrowsing,
+                               originAttributes, true);
+    RefPtr<AltSvcMapping> existing = LookupMapping(key, privateBrowsing);
+    LOG(
+        ("AltSvcCache::GetAltServiceMapping %p key=%s "
+         "existing=%p validated=%d ttl=%d",
+         this, key.get(), existing.get(), existing ? existing->Validated() : 0,
+         existing ? existing->TTL() : 0));
+    if (existing && existing->Validated()) {
+      return existing.forget();
+    }
   }
-  return existing.forget();
+
+  // Now look for HTTP2.
+  if (aHttp2Allowed) {
+    nsAutoCString key;
+    AltSvcMapping::MakeHashKey(key, scheme, host, port, privateBrowsing,
+                               originAttributes, false);
+    RefPtr<AltSvcMapping> existing = LookupMapping(key, privateBrowsing);
+    LOG(
+        ("AltSvcCache::GetAltServiceMapping %p key=%s "
+         "existing=%p validated=%d ttl=%d",
+         this, key.get(), existing.get(), existing ? existing->Validated() : 0,
+         existing ? existing->TTL() : 0));
+    if (existing && existing->Validated()) {
+      return existing.forget();
+    }
+  }
+
+  return nullptr;
 }
 
 class ProxyClearHostMapping : public Runnable {
@@ -1072,7 +1189,8 @@ class ProxyClearHostMapping : public Runnable {
 
   NS_IMETHOD Run() override {
     MOZ_ASSERT(NS_IsMainThread());
-    gHttpHandler->ConnMgr()->ClearHostMapping(mHost, mPort, mOriginAttributes);
+    gHttpHandler->AltServiceCache()->ClearHostMapping(mHost, mPort,
+                                                      mOriginAttributes);
     return NS_OK;
   }
 
@@ -1084,6 +1202,8 @@ class ProxyClearHostMapping : public Runnable {
 
 void AltSvcCache::ClearHostMapping(const nsACString& host, int32_t port,
                                    const OriginAttributes& originAttributes) {
+  MOZ_ASSERT(XRE_IsParentProcess());
+
   if (!NS_IsMainThread()) {
     nsCOMPtr<nsIRunnable> event =
         new ProxyClearHostMapping(host, port, originAttributes);
@@ -1093,32 +1213,24 @@ void AltSvcCache::ClearHostMapping(const nsACString& host, int32_t port,
     return;
   }
   nsAutoCString key;
-  AltSvcMapping::MakeHashKey(key, NS_LITERAL_CSTRING("http"), host, port, true,
-                             originAttributes);
-  RefPtr<AltSvcMapping> existing = LookupMapping(key, true);
-  if (existing) {
-    existing->SetExpired();
-  }
-
-  AltSvcMapping::MakeHashKey(key, NS_LITERAL_CSTRING("https"), host, port, true,
-                             originAttributes);
-  existing = LookupMapping(key, true);
-  if (existing) {
-    existing->SetExpired();
-  }
-
-  AltSvcMapping::MakeHashKey(key, NS_LITERAL_CSTRING("http"), host, port, false,
-                             originAttributes);
-  existing = LookupMapping(key, false);
-  if (existing) {
-    existing->SetExpired();
-  }
-
-  AltSvcMapping::MakeHashKey(key, NS_LITERAL_CSTRING("https"), host, port,
-                             false, originAttributes);
-  existing = LookupMapping(key, false);
-  if (existing) {
-    existing->SetExpired();
+  for (int secure = 0; secure < 2; ++secure) {
+    constexpr auto http = "http"_ns;
+    constexpr auto https = "https"_ns;
+    const nsLiteralCString& scheme = secure ? https : http;
+    for (int pb = 1; pb >= 0; --pb) {
+      AltSvcMapping::MakeHashKey(key, scheme, host, port, bool(pb),
+                                 originAttributes, false);
+      RefPtr<AltSvcMapping> existing = LookupMapping(key, bool(pb));
+      if (existing) {
+        existing->SetExpired();
+      }
+      AltSvcMapping::MakeHashKey(key, scheme, host, port, bool(pb),
+                                 originAttributes, true);
+      existing = LookupMapping(key, bool(pb));
+      if (existing) {
+        existing->SetExpired();
+      }
+    }
   }
 }
 
@@ -1136,12 +1248,30 @@ void AltSvcCache::ClearAltServiceMappings() {
   }
 }
 
+nsresult AltSvcCache::GetAltSvcCacheKeys(nsTArray<nsCString>& value) {
+  MOZ_ASSERT(NS_IsMainThread());
+  if (gHttpHandler->AllowAltSvc() && mStorage) {
+    nsTArray<DataStorageItem> items;
+    mStorage->GetAll(&items);
+
+    for (const auto& item : items) {
+      value.AppendElement(item.key);
+    }
+  }
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 AltSvcOverride::GetInterface(const nsIID& iid, void** result) {
   if (NS_SUCCEEDED(QueryInterface(iid, result)) && *result) {
     return NS_OK;
   }
-  return mCallbacks->GetInterface(iid, result);
+
+  if (mCallbacks) {
+    return mCallbacks->GetInterface(iid, result);
+  }
+
+  return NS_ERROR_NO_INTERFACE;
 }
 
 NS_IMETHODIMP
