@@ -10,25 +10,32 @@
 #include "nsHttp.h"
 #include "CacheControlParser.h"
 #include "PLDHashTable.h"
-#include "mozilla/Mutex.h"
+#include "mozilla/DataMutex.h"
 #include "mozilla/HashFunctions.h"
 #include "mozilla/Preferences.h"
-#include "mozilla/Unused.h"
+#include "mozilla/StaticPrefs_network.h"
 #include "nsCRT.h"
+#include "nsContentUtils.h"
 #include "nsHttpRequestHead.h"
 #include "nsHttpResponseHead.h"
 #include "nsHttpHandler.h"
 #include "nsICacheEntry.h"
 #include "nsIRequest.h"
+#include "nsJSUtils.h"
 #include <errno.h>
 #include <functional>
+#include "nsLiteralString.h"
 
 namespace mozilla {
 namespace net {
 
+const uint32_t kHttp3VersionCount = 5;
+const nsCString kHttp3Versions[] = {"h3-29"_ns, "h3-30"_ns, "h3-31"_ns,
+                                    "h3-32"_ns, "h3"_ns};
+
 // define storage for all atoms
 namespace nsHttp {
-#define HTTP_ATOM(_name, _value) nsHttpAtom _name = {_value};
+#define HTTP_ATOM(_name, _value) nsHttpAtom _name(nsLiteralCString{_value});
 #include "nsHttpAtomList.h"
 #undef HTTP_ATOM
 }  // namespace nsHttp
@@ -41,124 +48,121 @@ enum {
 };
 #undef HTTP_ATOM
 
-// we keep a linked list of atoms allocated on the heap for easy clean up when
-// the atom table is destroyed.  The structure and value string are allocated
-// as one contiguous block.
+class nsCaseInsentitiveHashKey : public PLDHashEntryHdr {
+ public:
+  using KeyType = const nsACString&;
+  using KeyTypePointer = const nsACString*;
 
-struct HttpHeapAtom {
-  struct HttpHeapAtom* next;
-  char value[1];
+  explicit nsCaseInsentitiveHashKey(KeyTypePointer aStr) : mStr(*aStr) {
+    // take it easy just deal HashKey
+  }
+
+  nsCaseInsentitiveHashKey(const nsCaseInsentitiveHashKey&) = delete;
+  nsCaseInsentitiveHashKey(nsCaseInsentitiveHashKey&& aToMove) noexcept
+      : PLDHashEntryHdr(std::move(aToMove)), mStr(aToMove.mStr) {}
+  ~nsCaseInsentitiveHashKey() = default;
+
+  KeyType GetKey() const { return mStr; }
+  bool KeyEquals(const KeyTypePointer aKey) const {
+    return mStr.Equals(*aKey, nsCaseInsensitiveCStringComparator);
+  }
+
+  static KeyTypePointer KeyToPointer(KeyType aKey) { return &aKey; }
+  static PLDHashNumber HashKey(const KeyTypePointer aKey) {
+    nsAutoCString tmKey(*aKey);
+    ToLowerCase(tmKey);
+    return mozilla::HashString(tmKey);
+  }
+  enum { ALLOW_MEMMOVE = false };
+
+  // To avoid double-counting, only measure the string if it is unshared.
+  size_t SizeOfExcludingThis(mozilla::MallocSizeOf aMallocSizeOf) const {
+    return GetKey().SizeOfExcludingThisIfUnshared(aMallocSizeOf);
+  }
+
+ private:
+  const nsCString mStr;
 };
 
-static PLDHashTable* sAtomTable;
-static struct HttpHeapAtom* sHeapAtoms = nullptr;
-static Mutex* sLock = nullptr;
+static StaticDataMutex<nsTHashtable<nsCaseInsentitiveHashKey>> sAtomTable(
+    "nsHttp::sAtomTable");
 
-HttpHeapAtom* NewHeapAtom(const char* value) {
-  int len = strlen(value);
-
-  HttpHeapAtom* a = reinterpret_cast<HttpHeapAtom*>(malloc(sizeof(*a) + len));
-  if (!a) return nullptr;
-  memcpy(a->value, value, len + 1);
-
-  // add this heap atom to the list of all heap atoms
-  a->next = sHeapAtoms;
-  sHeapAtoms = a;
-
-  return a;
-}
-
-// Hash string ignore case, based on PL_HashString
-static PLDHashNumber StringHash(const void* key) {
-  PLDHashNumber h = 0;
-  for (const char* s = reinterpret_cast<const char*>(key); *s; ++s)
-    h = AddToHash(h, nsCRT::ToLower(*s));
-  return h;
-}
-
-static bool StringCompare(const PLDHashEntryHdr* entry, const void* testKey) {
-  const void* entryKey = reinterpret_cast<const PLDHashEntryStub*>(entry)->key;
-
-  return PL_strcasecmp(reinterpret_cast<const char*>(entryKey),
-                       reinterpret_cast<const char*>(testKey)) == 0;
-}
-
-static const PLDHashTableOps ops = {StringHash, StringCompare,
-                                    PLDHashTable::MoveEntryStub,
-                                    PLDHashTable::ClearEntryStub, nullptr};
+// This is set to true in DestroyAtomTable so we don't try to repopulate the
+// table if ResolveAtom gets called during shutdown for some reason.
+static Atomic<bool> sTableDestroyed{false};
 
 // We put the atoms in a hash table for speedy lookup.. see ResolveAtom.
 namespace nsHttp {
-nsresult CreateAtomTable() {
-  MOZ_ASSERT(!sAtomTable, "atom table already initialized");
 
-  if (!sLock) {
-    sLock = new Mutex("nsHttp.sLock");
+nsresult CreateAtomTable(nsTHashtable<nsCaseInsentitiveHashKey>& base) {
+  if (sTableDestroyed) {
+    return NS_ERROR_ILLEGAL_DURING_SHUTDOWN;
   }
-
-  // The initial length for this table is a value greater than the number of
-  // known atoms (NUM_HTTP_ATOMS) because we expect to encounter a few random
-  // headers right off the bat.
-  sAtomTable =
-      new PLDHashTable(&ops, sizeof(PLDHashEntryStub), NUM_HTTP_ATOMS + 10);
-
   // fill the table with our known atoms
-  const char* const atoms[] = {
-#define HTTP_ATOM(_name, _value) _name._val,
+  const nsHttpAtom* atoms[] = {
+#define HTTP_ATOM(_name, _value) &(_name),
 #include "nsHttpAtomList.h"
 #undef HTTP_ATOM
-      nullptr};
+  };
 
-  for (int i = 0; atoms[i]; ++i) {
-    auto stub =
-        static_cast<PLDHashEntryStub*>(sAtomTable->Add(atoms[i], fallible));
-    if (!stub) return NS_ERROR_OUT_OF_MEMORY;
-
-    MOZ_ASSERT(!stub->key, "duplicate static atom");
-    stub->key = atoms[i];
+  if (!base.IsEmpty()) {
+    return NS_OK;
+  }
+  for (const auto* atom : atoms) {
+    Unused << base.PutEntry(atom->val(), fallible);
   }
 
+  LOG(("Added static atoms to atomTable"));
   return NS_OK;
 }
 
-void DestroyAtomTable() {
-  delete sAtomTable;
-  sAtomTable = nullptr;
-
-  while (sHeapAtoms) {
-    HttpHeapAtom* next = sHeapAtoms->next;
-    free(sHeapAtoms);
-    sHeapAtoms = next;
-  }
-
-  delete sLock;
-  sLock = nullptr;
+nsresult CreateAtomTable() {
+  LOG(("CreateAtomTable"));
+  auto atomTable = sAtomTable.Lock();
+  return CreateAtomTable(atomTable.ref());
 }
 
-Mutex* GetLock() { return sLock; }
+void DestroyAtomTable() {
+  LOG(("DestroyAtomTable"));
+  sTableDestroyed = true;
+  auto atomTable = sAtomTable.Lock();
+  atomTable.ref().Clear();
+}
 
 // this function may be called from multiple threads
-nsHttpAtom ResolveAtom(const char* str) {
-  nsHttpAtom atom = {nullptr};
-
-  if (!str || !sAtomTable) return atom;
-
-  MutexAutoLock lock(*sLock);
-
-  auto stub = static_cast<PLDHashEntryStub*>(sAtomTable->Add(str, fallible));
-  if (!stub) return atom;  // out of memory
-
-  if (stub->key) {
-    atom._val = reinterpret_cast<const char*>(stub->key);
+nsHttpAtom ResolveAtom(const nsACString& str) {
+  nsHttpAtom atom;
+  if (str.IsEmpty()) {
     return atom;
   }
 
-  // if the atom could not be found in the atom table, then we'll go
-  // and allocate a new atom on the heap.
-  HttpHeapAtom* heapAtom = NewHeapAtom(str);
-  if (!heapAtom) return atom;  // out of memory
+  auto atomTable = sAtomTable.Lock();
 
-  stub->key = atom._val = heapAtom->value;
+  if (atomTable.ref().IsEmpty()) {
+    if (sTableDestroyed) {
+      NS_WARNING("ResolveAtom called during shutdown");
+      return atom;
+    }
+
+    NS_WARNING("ResolveAtom called before CreateAtomTable");
+    if (NS_FAILED(CreateAtomTable(atomTable.ref()))) {
+      return atom;
+    }
+  }
+
+  // Check if we already have an entry in the table
+  auto* entry = atomTable.ref().GetEntry(str);
+  if (entry) {
+    atom._val = entry->GetKey();
+    return atom;
+  }
+
+  LOG(("Putting %s header into atom table", nsPromiseFlatCString(str).get()));
+  // Put the string in the table. If it works create the atom.
+  entry = atomTable.ref().PutEntry(str, fallible);
+  if (entry) {
+    atom._val = entry->GetKey();
+  }
   return atom;
 }
 
@@ -210,6 +214,8 @@ bool IsValidToken(const char* start, const char* end) {
 
 const char* GetProtocolVersion(HttpVersion pv) {
   switch (pv) {
+    case HttpVersion::v3_0:
+      return "h3";
     case HttpVersion::v2_0:
       return "h2";
     case HttpVersion::v1_0:
@@ -262,7 +268,7 @@ const char* FindToken(const char* input, const char* token, const char* seps) {
   const char* inputTop = input;
   const char* inputEnd = input + inputLen - tokenLen;
   for (; input <= inputEnd; ++input) {
-    if (PL_strncasecmp(input, token, tokenLen) == 0) {
+    if (nsCRT::strncasecmp(input, token, tokenLen) == 0) {
       if (input > inputTop && !strchr(seps, *(input - 1))) continue;
       if (input < inputEnd && !strchr(seps, *(input + tokenLen))) continue;
       return input;
@@ -418,7 +424,8 @@ bool ValidationRequired(bool isForcedValid,
     LOG(("  not validating, expire time not in the past"));
   } else if (cachedResponseHead->MustValidateIfExpired()) {
     doValidation = true;
-  } else if (cachedResponseHead->StaleWhileRevalidate(now, expiration)) {
+  } else if (cachedResponseHead->StaleWhileRevalidate(now, expiration) &&
+             StaticPrefs::network_http_stale_while_revalidate_enabled()) {
     LOG(("  not validating, in the stall-while-revalidate window"));
     doValidation = false;
     if (performBackgroundRevalidation) {
@@ -430,10 +437,11 @@ bool ValidationRequired(bool isForcedValid,
     // or not this is the first access this session.  This behavior
     // is consistent with existing browsers and is generally expected
     // by web authors.
-    if (freshness == 0)
+    if (freshness == 0) {
       doValidation = true;
-    else
+    } else {
       doValidation = fromPreviousSession;
+    }
   } else {
     doValidation = true;
   }
@@ -516,13 +524,50 @@ bool IsBeforeLastActiveTabLoadOptimization(TimeStamp const& when) {
          gHttpHandler->IsBeforeLastActiveTabLoadOptimization(when);
 }
 
+nsCString ConvertRequestHeadToString(nsHttpRequestHead& aRequestHead,
+                                     bool aHasRequestBody,
+                                     bool aRequestBodyHasHeaders,
+                                     bool aUsingConnect) {
+  // Make sure that there is "Content-Length: 0" header in the requestHead
+  // in case of POST and PUT methods when there is no requestBody and
+  // requestHead doesn't contain "Transfer-Encoding" header.
+  //
+  // RFC1945 section 7.2.2:
+  //   HTTP/1.0 requests containing an entity body must include a valid
+  //   Content-Length header field.
+  //
+  // RFC2616 section 4.4:
+  //   For compatibility with HTTP/1.0 applications, HTTP/1.1 requests
+  //   containing a message-body MUST include a valid Content-Length header
+  //   field unless the server is known to be HTTP/1.1 compliant.
+  if ((aRequestHead.IsPost() || aRequestHead.IsPut()) && !aHasRequestBody &&
+      !aRequestHead.HasHeader(nsHttp::Transfer_Encoding)) {
+    DebugOnly<nsresult> rv =
+        aRequestHead.SetHeader(nsHttp::Content_Length, "0"_ns);
+    MOZ_ASSERT(NS_SUCCEEDED(rv));
+  }
+
+  nsCString reqHeaderBuf;
+  reqHeaderBuf.Truncate();
+
+  // make sure we eliminate any proxy specific headers from
+  // the request if we are using CONNECT
+  aRequestHead.Flatten(reqHeaderBuf, aUsingConnect);
+
+  if (!aRequestBodyHasHeaders || !aHasRequestBody) {
+    reqHeaderBuf.AppendLiteral("\r\n");
+  }
+
+  return reqHeaderBuf;
+}
+
 void NotifyActiveTabLoadOptimization() {
   if (gHttpHandler) {
     gHttpHandler->NotifyActiveTabLoadOptimization();
   }
 }
 
-TimeStamp const GetLastActiveTabLoadOptimizationHit() {
+TimeStamp GetLastActiveTabLoadOptimizationHit() {
   return gHttpHandler ? gHttpHandler->GetLastActiveTabLoadOptimizationHit()
                       : TimeStamp();
 }
@@ -533,13 +578,6 @@ void SetLastActiveTabLoadOptimizationHit(TimeStamp const& when) {
   }
 }
 
-HttpVersion GetHttpVersionFromSpdy(SpdyVersion sv) {
-  MOZ_DIAGNOSTIC_ASSERT(sv != SpdyVersion::NONE);
-  MOZ_ASSERT(sv == SpdyVersion::HTTP_2);
-
-  return HttpVersion::v2_0;
-}
-
 }  // namespace nsHttp
 
 template <typename T>
@@ -572,13 +610,10 @@ void EnsureBuffer(UniquePtr<uint8_t[]>& buf, uint32_t newSize,
 }
 
 static bool IsTokenSymbol(signed char chr) {
-  if (chr < 33 || chr == 127 || chr == '(' || chr == ')' || chr == '<' ||
-      chr == '>' || chr == '@' || chr == ',' || chr == ';' || chr == ':' ||
-      chr == '"' || chr == '/' || chr == '[' || chr == ']' || chr == '?' ||
-      chr == '=' || chr == '{' || chr == '}' || chr == '\\') {
-    return false;
-  }
-  return true;
+  return !(chr < 33 || chr == 127 || chr == '(' || chr == ')' || chr == '<' ||
+           chr == '>' || chr == '@' || chr == ',' || chr == ';' || chr == ':' ||
+           chr == '"' || chr == '/' || chr == '[' || chr == ']' || chr == '?' ||
+           chr == '=' || chr == '{' || chr == '}' || chr == '\\');
 }
 
 ParsedHeaderPair::ParsedHeaderPair(const char* name, int32_t nameLen,
@@ -690,8 +725,9 @@ void ParsedHeaderValueList::ParseNameAndValue(const char* input,
 
   for (; *input && *input != ';' && *input != ',' &&
          !nsCRT::IsAsciiSpace(*input) && *input != '=';
-       input++)
+       input++) {
     ;
+  }
 
   nameEnd = input;
 
@@ -730,8 +766,9 @@ void ParsedHeaderValueList::ParseNameAndValue(const char* input,
     valueStart = input;
     for (valueEnd = input; *valueEnd && !nsCRT::IsAsciiSpace(*valueEnd) &&
                            *valueEnd != ';' && *valueEnd != ',';
-         valueEnd++)
+         valueEnd++) {
       ;
+    }
     if (!allowInvalidValue) {
       for (const char* c = valueStart; c < valueEnd; c++) {
         if (!IsTokenSymbol(*c)) {
@@ -808,17 +845,13 @@ void LogCallingScriptLocation(void* instance) {
 }
 
 void LogHeaders(const char* lineStart) {
-  static bool sanitize = true;
-  static nsresult once = Preferences::AddBoolVarCache(
-      &sanitize, "network.http.sanitize-headers-in-logs", true);
-  Unused << once;
-
   nsAutoCString buf;
   char* endOfLine;
   while ((endOfLine = PL_strstr(lineStart, "\r\n"))) {
     buf.Assign(lineStart, endOfLine - lineStart);
-    if (sanitize && (PL_strcasestr(buf.get(), "authorization: ") ||
-                     PL_strcasestr(buf.get(), "proxy-authorization: "))) {
+    if (StaticPrefs::network_http_sanitize_headers_in_logs() &&
+        (nsCRT::strcasestr(buf.get(), "authorization: ") ||
+         nsCRT::strcasestr(buf.get(), "proxy-authorization: "))) {
       char* p = PL_strchr(buf.get(), ' ');
       while (p && *++p) {
         *p = '*';
@@ -829,5 +862,182 @@ void LogHeaders(const char* lineStart) {
   }
 }
 
+nsresult HttpProxyResponseToErrorCode(uint32_t aStatusCode) {
+  // In proxy CONNECT case, we treat every response code except 200 as an error.
+  // Even if the proxy server returns other 2xx codes (i.e. 206), this function
+  // still returns an error code.
+  MOZ_ASSERT(aStatusCode != 200);
+
+  nsresult rv;
+  switch (aStatusCode) {
+    case 300:
+    case 301:
+    case 302:
+    case 303:
+    case 307:
+    case 308:
+      // Bad redirect: not top-level, or it's a POST, bad/missing Location,
+      // or ProcessRedirect() failed for some other reason.  Legal
+      // redirects that fail because site not available, etc., are handled
+      // elsewhere, in the regular codepath.
+      rv = NS_ERROR_CONNECTION_REFUSED;
+      break;
+    // Squid sends 404 if DNS fails (regular 404 from target is tunneled)
+    case 404:  // HTTP/1.1: "Not Found"
+               // RFC 2616: "some deployed proxies are known to return 400 or
+               // 500 when DNS lookups time out."  (Squid uses 500 if it runs
+               // out of sockets: so we have a conflict here).
+    case 400:  // HTTP/1.1 "Bad Request"
+    case 500:  // HTTP/1.1: "Internal Server Error"
+      rv = NS_ERROR_UNKNOWN_HOST;
+      break;
+    case 401:
+      rv = NS_ERROR_PROXY_UNAUTHORIZED;
+      break;
+    case 402:
+      rv = NS_ERROR_PROXY_PAYMENT_REQUIRED;
+      break;
+    case 403:
+      rv = NS_ERROR_PROXY_FORBIDDEN;
+      break;
+    case 405:
+      rv = NS_ERROR_PROXY_METHOD_NOT_ALLOWED;
+      break;
+    case 406:
+      rv = NS_ERROR_PROXY_NOT_ACCEPTABLE;
+      break;
+    case 407:  // ProcessAuthentication() failed (e.g. no header)
+      rv = NS_ERROR_PROXY_AUTHENTICATION_FAILED;
+      break;
+    case 408:
+      rv = NS_ERROR_PROXY_REQUEST_TIMEOUT;
+      break;
+    case 409:
+      rv = NS_ERROR_PROXY_CONFLICT;
+      break;
+    case 410:
+      rv = NS_ERROR_PROXY_GONE;
+      break;
+    case 411:
+      rv = NS_ERROR_PROXY_LENGTH_REQUIRED;
+      break;
+    case 412:
+      rv = NS_ERROR_PROXY_PRECONDITION_FAILED;
+      break;
+    case 413:
+      rv = NS_ERROR_PROXY_REQUEST_ENTITY_TOO_LARGE;
+      break;
+    case 414:
+      rv = NS_ERROR_PROXY_REQUEST_URI_TOO_LONG;
+      break;
+    case 415:
+      rv = NS_ERROR_PROXY_UNSUPPORTED_MEDIA_TYPE;
+      break;
+    case 416:
+      rv = NS_ERROR_PROXY_REQUESTED_RANGE_NOT_SATISFIABLE;
+      break;
+    case 417:
+      rv = NS_ERROR_PROXY_EXPECTATION_FAILED;
+      break;
+    case 421:
+      rv = NS_ERROR_PROXY_MISDIRECTED_REQUEST;
+      break;
+    case 425:
+      rv = NS_ERROR_PROXY_TOO_EARLY;
+      break;
+    case 426:
+      rv = NS_ERROR_PROXY_UPGRADE_REQUIRED;
+      break;
+    case 428:
+      rv = NS_ERROR_PROXY_PRECONDITION_REQUIRED;
+      break;
+    case 429:
+      rv = NS_ERROR_PROXY_TOO_MANY_REQUESTS;
+      break;
+    case 431:
+      rv = NS_ERROR_PROXY_REQUEST_HEADER_FIELDS_TOO_LARGE;
+      break;
+    case 451:
+      rv = NS_ERROR_PROXY_UNAVAILABLE_FOR_LEGAL_REASONS;
+      break;
+    case 501:
+      rv = NS_ERROR_PROXY_NOT_IMPLEMENTED;
+      break;
+    case 502:
+      rv = NS_ERROR_PROXY_BAD_GATEWAY;
+      break;
+    case 503:
+      // Squid returns 503 if target request fails for anything but DNS.
+      /* User sees: "Failed to Connect:
+       *  Firefox can't establish a connection to the server at
+       *  www.foo.com.  Though the site seems valid, the browser
+       *  was unable to establish a connection."
+       */
+      rv = NS_ERROR_CONNECTION_REFUSED;
+      break;
+    // RFC 2616 uses 504 for both DNS and target timeout, so not clear what to
+    // do here: picking target timeout, as DNS covered by 400/404/500
+    case 504:
+      rv = NS_ERROR_PROXY_GATEWAY_TIMEOUT;
+      break;
+    case 505:
+      rv = NS_ERROR_PROXY_VERSION_NOT_SUPPORTED;
+      break;
+    case 506:
+      rv = NS_ERROR_PROXY_VARIANT_ALSO_NEGOTIATES;
+      break;
+    case 510:
+      rv = NS_ERROR_PROXY_NOT_EXTENDED;
+      break;
+    case 511:
+      rv = NS_ERROR_PROXY_NETWORK_AUTHENTICATION_REQUIRED;
+      break;
+    default:
+      rv = NS_ERROR_PROXY_CONNECTION_REFUSED;
+      break;
+  }
+
+  return rv;
+}
+
+Tuple<nsCString, bool> SelectAlpnFromAlpnList(
+    const nsTArray<nsCString>& aAlpnList, bool aNoHttp2, bool aNoHttp3) {
+  nsCString h3Value;
+  nsCString h2Value;
+  nsCString h1Value;
+  for (const auto& npnToken : aAlpnList) {
+    bool isHttp3 = gHttpHandler->IsHttp3VersionSupported(npnToken);
+    if (isHttp3 && h3Value.IsEmpty()) {
+      h3Value.Assign(npnToken);
+    }
+
+    uint32_t spdyIndex;
+    SpdyInformation* spdyInfo = gHttpHandler->SpdyInfo();
+    if (NS_SUCCEEDED(spdyInfo->GetNPNIndex(npnToken, &spdyIndex)) &&
+        spdyInfo->ProtocolEnabled(spdyIndex) && h2Value.IsEmpty()) {
+      h2Value.Assign(npnToken);
+    }
+
+    if (npnToken.LowerCaseEqualsASCII("http/1.1") && h1Value.IsEmpty()) {
+      h1Value.Assign(npnToken);
+    }
+  }
+
+  if (!h3Value.IsEmpty() && gHttpHandler->IsHttp3Enabled() && !aNoHttp3) {
+    return MakeTuple(h3Value, true);
+  }
+
+  if (!h2Value.IsEmpty() && gHttpHandler->IsSpdyEnabled() && !aNoHttp2) {
+    return MakeTuple(h2Value, false);
+  }
+
+  if (!h1Value.IsEmpty()) {
+    return MakeTuple(h1Value, false);
+  }
+
+  // If we are here, there is no supported alpn can be used.
+  return MakeTuple(EmptyCString(), false);
+}
+
 }  // namespace net
 }  // namespace mozilla
