@@ -14,8 +14,6 @@
 #include "nsICacheEntry.h"
 #include "nsICacheEntryOpenCallback.h"
 #include "nsIDNSListener.h"
-#include "nsIApplicationCacheChannel.h"
-#include "nsIChannelWithDivertableParentListener.h"
 #include "nsIProtocolProxyCallback.h"
 #include "nsIHttpAuthenticableChannel.h"
 #include "nsIAsyncVerifyRedirectCallback.h"
@@ -23,19 +21,21 @@
 #include "nsIThreadRetargetableStreamListener.h"
 #include "nsWeakReference.h"
 #include "TimingStruct.h"
-#include "ADivertableParentChannel.h"
 #include "AutoClose.h"
 #include "nsIStreamListener.h"
 #include "nsICorsPreflightCallback.h"
 #include "AlternateServices.h"
 #include "nsIRaceCacheWithNetwork.h"
+#include "mozilla/AtomicBitfields.h"
 #include "mozilla/Atomics.h"
 #include "mozilla/extensions/PStreamFilterParent.h"
+#include "mozilla/net/DocumentLoadListener.h"
 #include "mozilla/Mutex.h"
 
 class nsDNSPrefetch;
 class nsICancelable;
 class nsIDNSRecord;
+class nsIDNSHTTPSSVCRecord;
 class nsIHttpChannelAuthProvider;
 class nsInputStreamPump;
 class nsITransportSecurityInfo;
@@ -68,14 +68,12 @@ class nsHttpChannel final : public HttpBaseChannel,
                             public nsITransportEventSink,
                             public nsIProtocolProxyCallback,
                             public nsIHttpAuthenticableChannel,
-                            public nsIApplicationCacheChannel,
                             public nsIAsyncVerifyRedirectCallback,
                             public nsIThreadRetargetableRequest,
                             public nsIThreadRetargetableStreamListener,
                             public nsIDNSListener,
                             public nsSupportsWeakReference,
                             public nsICorsPreflightCallback,
-                            public nsIChannelWithDivertableParentListener,
                             public nsIRaceCacheWithNetwork,
                             public nsIRequestTailUnblockCallback,
                             public nsITimerCallback {
@@ -90,12 +88,9 @@ class nsHttpChannel final : public HttpBaseChannel,
   NS_DECL_NSITRANSPORTEVENTSINK
   NS_DECL_NSIPROTOCOLPROXYCALLBACK
   NS_DECL_NSIPROXIEDCHANNEL
-  NS_DECL_NSIAPPLICATIONCACHECONTAINER
-  NS_DECL_NSIAPPLICATIONCACHECHANNEL
   NS_DECL_NSIASYNCVERIFYREDIRECTCALLBACK
   NS_DECL_NSITHREADRETARGETABLEREQUEST
   NS_DECL_NSIDNSLISTENER
-  NS_DECL_NSICHANNELWITHDIVERTABLEPARENTLISTENER
   NS_DECLARE_STATIC_IID_ACCESSOR(NS_HTTPCHANNEL_IID)
   NS_DECL_NSIRACECACHEWITHNETWORK
   NS_DECL_NSITIMERCALLBACK
@@ -131,7 +126,7 @@ class nsHttpChannel final : public HttpBaseChannel,
   [[nodiscard]] virtual nsresult Init(
       nsIURI* aURI, uint32_t aCaps, nsProxyInfo* aProxyInfo,
       uint32_t aProxyResolveFlags, nsIURI* aProxyURI, uint64_t aChannelId,
-      nsContentPolicyType aContentPolicyType) override;
+      ExtContentPolicyType aContentPolicyType) override;
 
   [[nodiscard]] nsresult OnPush(uint32_t aPushedStreamId,
                                 const nsACString& aUrl,
@@ -153,7 +148,7 @@ class nsHttpChannel final : public HttpBaseChannel,
   // nsIHttpChannel
   NS_IMETHOD GetEncodedBodySize(uint64_t* aEncodedBodySize) override;
   // nsIHttpChannelInternal
-  NS_IMETHOD SetupFallbackChannel(const char* aFallbackKey) override;
+  NS_IMETHOD GetIsAuthChannel(bool* aIsAuthChannel) override;
   NS_IMETHOD SetChannelIsForDownload(bool aChannelIsForDownload) override;
   NS_IMETHOD GetNavigationStartTimeStamp(TimeStamp* aTimeStamp) override;
   NS_IMETHOD SetNavigationStartTimeStamp(TimeStamp aTimeStamp) override;
@@ -198,36 +193,25 @@ class nsHttpChannel final : public HttpBaseChannel,
   void SetWarningReporter(HttpChannelSecurityWarningReporter* aReporter);
   HttpChannelSecurityWarningReporter* GetWarningReporter();
 
-  bool OnDataAlreadySent() { return mDataAlreadySent; }
+  bool DataSentToChildProcess() { return LoadDataSentToChildProcess(); }
+
+  enum class SnifferType { Media, Image };
+  void DisableIsOpaqueResponseAllowedAfterSniffCheck(SnifferType aType);
 
  public: /* internal necko use only */
   uint32_t GetRequestTime() const { return mRequestTime; }
 
-  nsresult AsyncOpenFinal(TimeStamp aTimeStamp);
+  void AsyncOpenFinal(TimeStamp aTimeStamp);
 
-  [[nodiscard]] nsresult OpenCacheEntry(bool usingSSL);
-  [[nodiscard]] nsresult OpenCacheEntryInternal(
-      bool isHttps, nsIApplicationCache* applicationCache, bool noAppCache);
+  [[nodiscard]] nsresult OpenCacheEntry(bool isHttps);
+  [[nodiscard]] nsresult OpenCacheEntryInternal(bool isHttps);
   [[nodiscard]] nsresult ContinueConnect();
 
   [[nodiscard]] nsresult StartRedirectChannelToURI(nsIURI*, uint32_t);
 
-  // This allows cache entry to be marked as foreign even after channel itself
-  // is gone.  Needed for e10s (see
-  // HttpChannelParent::RecvDocumentChannelCleanup)
-  class OfflineCacheEntryAsForeignMarker {
-    nsCOMPtr<nsIApplicationCache> mApplicationCache;
-    nsCOMPtr<nsIURI> mCacheURI;
-
-   public:
-    OfflineCacheEntryAsForeignMarker(nsIApplicationCache* appCache,
-                                     nsIURI* aURI)
-        : mApplicationCache(appCache), mCacheURI(aURI) {}
-
-    nsresult MarkAsForeign();
-  };
-
-  OfflineCacheEntryAsForeignMarker* GetOfflineCacheEntryAsForeignMarker();
+  SnifferCategoryType GetSnifferCategoryType() const {
+    return mSnifferCategoryType;
+  }
 
   // Helper to keep cache callbacks wait flags consistent
   class AutoCacheWaitFlags {
@@ -235,9 +219,7 @@ class nsHttpChannel final : public HttpBaseChannel,
     explicit AutoCacheWaitFlags(nsHttpChannel* channel)
         : mChannel(channel), mKeep(0) {
       // Flags must be set before entering any AsyncOpenCacheEntry call.
-      mChannel->mCacheEntriesToWaitFor =
-          nsHttpChannel::WAIT_FOR_CACHE_ENTRY |
-          nsHttpChannel::WAIT_FOR_OFFLINE_CACHE_ENTRY;
+      mChannel->StoreWaitForCacheEntry(nsHttpChannel::WAIT_FOR_CACHE_ENTRY);
     }
 
     void Keep(uint32_t flags) {
@@ -247,12 +229,13 @@ class nsHttpChannel final : public HttpBaseChannel,
 
     ~AutoCacheWaitFlags() {
       // Keep only flags those are left to be wait for.
-      mChannel->mCacheEntriesToWaitFor &= mKeep;
+      mChannel->StoreWaitForCacheEntry(mChannel->LoadWaitForCacheEntry() &
+                                       mKeep);
     }
 
    private:
     nsHttpChannel* mChannel;
-    uint32_t mKeep : 2;
+    uint32_t mKeep : 1;
   };
 
   bool AwaitingCacheCallbacks();
@@ -265,7 +248,8 @@ class nsHttpChannel final : public HttpBaseChannel,
   base::ProcessId ProcessId();
 
   using ChildEndpointPromise =
-      MozPromise<ipc::Endpoint<extensions::PStreamFilterChild>, bool, true>;
+      MozPromise<mozilla::ipc::Endpoint<extensions::PStreamFilterChild>, bool,
+                 true>;
   [[nodiscard]] RefPtr<ChildEndpointPromise> AttachStreamFilter(
       base::ProcessId aChildProcessId);
 
@@ -279,13 +263,13 @@ class nsHttpChannel final : public HttpBaseChannel,
   }
   TransactionObserver* GetTransactionObserver() { return mTransactionObserver; }
 
-  CacheDisposition mCacheDisposition;
+  CacheDisposition mCacheDisposition{kCacheUnresolved};
 
  protected:
   virtual ~nsHttpChannel();
 
  private:
-  typedef nsresult (nsHttpChannel::*nsContinueRedirectionFunc)(nsresult result);
+  using nsContinueRedirectionFunc = nsresult (nsHttpChannel::*)(nsresult);
 
   // Directly call |aFunc| if the channel is not canceled and not suspended.
   // Otherwise, set |aFunc| to |mCallOnResume| and wait until the channel
@@ -300,7 +284,7 @@ class nsHttpChannel final : public HttpBaseChannel,
 
   // Connections will only be established in this function.
   // (including DNS prefetch and speculative connection.)
-  nsresult MaybeResolveProxyAndBeginConnect();
+  void MaybeResolveProxyAndBeginConnect();
   nsresult MaybeStartDNSPrefetch();
 
   // Tells the channel to resolve the origin of the end server we are connecting
@@ -325,14 +309,12 @@ class nsHttpChannel final : public HttpBaseChannel,
   // is required, this funciton will just return NS_OK and BeginConnect()
   // will be called when callback. See Bug 1325054 for more information.
   nsresult BeginConnect();
-  [[nodiscard]] nsresult ContinueBeginConnectWithResult();
-  void ContinueBeginConnect();
   [[nodiscard]] nsresult PrepareToConnect();
-  void HandleOnBeforeConnect();
   [[nodiscard]] nsresult OnBeforeConnect();
-  [[nodiscard]] nsresult ContinueOnBeforeConnect(bool aShouldUpgrade,
-                                                 nsresult aStatus);
-  void OnBeforeConnectContinue();
+  [[nodiscard]] nsresult ContinueOnBeforeConnect(
+      bool aShouldUpgrade, nsresult aStatus, bool aUpgradeWithHTTPSRR = false);
+  nsresult MaybeUseHTTPSRRForUpgrade(bool aShouldUpgrade, nsresult aStatus);
+  void OnHTTPSRRAvailable(nsIDNSHTTPSSVCRecord* aRecord);
   [[nodiscard]] nsresult Connect();
   void SpeculativeConnect();
   [[nodiscard]] nsresult SetupTransaction();
@@ -355,12 +337,10 @@ class nsHttpChannel final : public HttpBaseChannel,
           aContinueProcessResponseFunc);
   [[nodiscard]] nsresult ContinueProcessResponseAfterNotModified(nsresult aRv);
 
-  [[nodiscard]] nsresult AsyncProcessRedirection(uint32_t httpStatus);
+  [[nodiscard]] nsresult AsyncProcessRedirection(uint32_t redirectType);
   [[nodiscard]] nsresult ContinueProcessRedirection(nsresult);
   [[nodiscard]] nsresult ContinueProcessRedirectionAfterFallback(nsresult);
   [[nodiscard]] nsresult ProcessFailedProxyConnect(uint32_t httpStatus);
-  [[nodiscard]] nsresult ProcessFallback(bool* waitingForRedirectCallback);
-  [[nodiscard]] nsresult ContinueProcessFallback(nsresult);
   void HandleAsyncAbort();
   [[nodiscard]] nsresult EnsureAssocReq();
   void ProcessSSLInformation();
@@ -378,12 +358,11 @@ class nsHttpChannel final : public HttpBaseChannel,
   void HandleAsyncAPIRedirect();
   [[nodiscard]] nsresult ContinueHandleAsyncRedirect(nsresult);
   void HandleAsyncNotModified();
-  void HandleAsyncFallback();
-  [[nodiscard]] nsresult ContinueHandleAsyncFallback(nsresult);
   [[nodiscard]] nsresult PromptTempRedirect();
   [[nodiscard]] virtual nsresult SetupReplacementChannel(
       nsIURI*, nsIChannel*, bool preserveMethod,
       uint32_t redirectFlags) override;
+  void HandleAsyncRedirectToUnstrippedURI();
 
   // proxy specific methods
   [[nodiscard]] nsresult ProxyFailover();
@@ -392,33 +371,23 @@ class nsHttpChannel final : public HttpBaseChannel,
   [[nodiscard]] nsresult ResolveProxy();
 
   // cache specific methods
-  [[nodiscard]] nsresult OnOfflineCacheEntryAvailable(
-      nsICacheEntry* aEntry, bool aNew, nsIApplicationCache* aAppCache,
-      nsresult aResult);
   [[nodiscard]] nsresult OnNormalCacheEntryAvailable(nsICacheEntry* aEntry,
                                                      bool aNew,
-                                                     nsresult aResult);
-  [[nodiscard]] nsresult OpenOfflineCacheEntryForWriting();
-  [[nodiscard]] nsresult OnOfflineCacheEntryForWritingAvailable(
-      nsICacheEntry* aEntry, nsIApplicationCache* aAppCache, nsresult aResult);
-  [[nodiscard]] nsresult OnCacheEntryAvailableInternal(
-      nsICacheEntry* entry, bool aNew, nsIApplicationCache* aAppCache,
-      nsresult status);
+                                                     nsresult aEntryStatus);
+  [[nodiscard]] nsresult OnCacheEntryAvailableInternal(nsICacheEntry* entry,
+                                                       bool aNew,
+                                                       nsresult status);
   [[nodiscard]] nsresult GenerateCacheKey(uint32_t postID, nsACString& key);
   [[nodiscard]] nsresult UpdateExpirationTime();
   [[nodiscard]] nsresult CheckPartial(nsICacheEntry* aEntry, int64_t* aSize,
                                       int64_t* aContentLength);
-  bool ShouldUpdateOfflineCacheEntry();
   [[nodiscard]] nsresult ReadFromCache(bool alreadyMarkedValid);
   void CloseCacheEntry(bool doomOnFailure);
-  void CloseOfflineCacheEntry();
   [[nodiscard]] nsresult InitCacheEntry();
   void UpdateInhibitPersistentCachingFlag();
-  [[nodiscard]] nsresult InitOfflineCacheEntry();
   [[nodiscard]] nsresult AddCacheEntryHeaders(nsICacheEntry* entry);
   [[nodiscard]] nsresult FinalizeCacheEntry();
   [[nodiscard]] nsresult InstallCacheListener(int64_t offset = 0);
-  [[nodiscard]] nsresult InstallOfflineCacheListener(int64_t offset = 0);
   void MaybeInvalidateCacheEntryForSubsequentGet();
   void AsyncOnExamineCachedResponse();
 
@@ -441,8 +410,8 @@ class nsHttpChannel final : public HttpBaseChannel,
       HttpTransactionShell* aTransWithStickyConn,
       const std::function<nsresult(nsHttpChannel*, nsresult)>&
           aContinueOnStopRequestFunc);
-  [[nodiscard]] nsresult DoConnect(
-      HttpTransactionShell* aTransWithStickyConn = nullptr);
+  [[nodiscard]] MOZ_NEVER_INLINE nsresult
+  DoConnect(HttpTransactionShell* aTransWithStickyConn = nullptr);
   [[nodiscard]] nsresult DoConnectActual(
       HttpTransactionShell* aTransWithStickyConn);
   [[nodiscard]] nsresult ContinueOnStopRequestAfterAuthRetry(
@@ -479,17 +448,12 @@ class nsHttpChannel final : public HttpBaseChannel,
       nsHttpResponseHead* aResponseHead);
 
   /**
-   * A function that will, if the feature is enabled, send security reports.
-   */
-  void ProcessSecurityReport(nsresult status);
-
-  /**
-   * A function to process a single security header (STS or PKP), assumes
-   * some basic sanity checks have been applied to the channel. Called
+   * A function to process HTTP Strict Transport Security (HSTS) headers.
+   * Some basic consistency checks have been applied to the channel. Called
    * from ProcessSecurityHeaders.
    */
-  [[nodiscard]] nsresult ProcessSingleSecurityHeader(
-      uint32_t aType, nsITransportSecurityInfo* aSecInfo, uint32_t aFlags);
+  [[nodiscard]] nsresult ProcessHSTSHeader(nsITransportSecurityInfo* aSecInfo,
+                                           uint32_t aFlags);
 
   void InvalidateCacheEntryForLocation(const char* location);
   void AssembleCacheKey(const char* spec, uint32_t postID, nsACString& key);
@@ -534,21 +498,14 @@ class nsHttpChannel final : public HttpBaseChannel,
   void UntieByteRangeRequest();
   void UntieValidationRequest();
   [[nodiscard]] nsresult OpenCacheInputStream(nsICacheEntry* cacheEntry,
-                                              bool startBuffering,
-                                              bool checkingAppCacheEntry);
+                                              bool startBuffering);
 
   void SetPushedStreamTransactionAndId(
       HttpTransactionShell* aTransWithPushedStream, uint32_t aPushedStreamId);
 
-  void MaybeWarnAboutAppCache();
-
   void SetOriginHeader();
   void SetDoNotTrack();
 
-  bool IsIsolated();
-
-  const nsCString& GetTopWindowOrigin();
-
   already_AddRefed<nsChannelClassifier> GetOrCreateChannelClassifier();
 
   // Start an internal redirect to a new InterceptedHttpChannel which will
@@ -559,18 +516,13 @@ class nsHttpChannel final : public HttpBaseChannel,
   // writing a new entry. The content type is used in cache internally only.
   void SetCachedContentType();
 
-  // This function updates all the fields used by anti-tracking when a channel
-  // is opened. We have to do this in the parent to access cross-origin info
-  // that is not exposed to child processes.
-  void UpdateAntiTrackingInfo();
-
  private:
   // this section is for main-thread-only object
   // all the references need to be proxy released on main thread.
-  nsCOMPtr<nsIApplicationCache> mApplicationCacheForWrite;
   // auth specific data
   nsCOMPtr<nsIHttpChannelAuthProvider> mAuthProvider;
   nsCOMPtr<nsIURI> mRedirectURI;
+  nsCOMPtr<nsIURI> mUnstrippedRedirectURI;
   nsCOMPtr<nsIChannel> mRedirectChannel;
   nsCOMPtr<nsIChannel> mPreflightChannel;
 
@@ -603,7 +555,7 @@ class nsHttpChannel final : public HttpBaseChannel,
   nsCOMPtr<nsIRequest> mTransactionPump;
   RefPtr<HttpTransactionShell> mTransaction;
 
-  uint64_t mLogicalOffset;
+  uint64_t mLogicalOffset{0};
 
   // cache specific data
   nsCOMPtr<nsICacheEntry> mCacheEntry;
@@ -622,11 +574,10 @@ class nsHttpChannel final : public HttpBaseChannel,
   RefPtr<nsInputStreamPump> mCachePump;
   UniquePtr<nsHttpResponseHead> mCachedResponseHead;
   nsCOMPtr<nsISupports> mCachedSecurityInfo;
-  uint32_t mPostID;
-  uint32_t mRequestTime;
+  uint32_t mPostID{0};
+  uint32_t mRequestTime{0};
 
-  nsCOMPtr<nsICacheEntry> mOfflineCacheEntry;
-  uint32_t mOfflineCacheLastModifiedTime;
+  nsTArray<StreamFilterRequest> mStreamFilterRequests;
 
   mozilla::TimeStamp mOnStartRequestTimestamp;
   // Timestamp of the time the channel was suspended.
@@ -638,119 +589,111 @@ class nsHttpChannel final : public HttpBaseChannel,
 #endif
   // Total time the channel spent suspended. This value is reported to
   // telemetry in nsHttpChannel::OnStartRequest().
-  uint32_t mSuspendTotalTime;
-
-  // If the channel is associated with a cache, and the URI matched
-  // a fallback namespace, this will hold the key for the fallback
-  // cache entry.
-  nsCString mFallbackKey;
+  uint32_t mSuspendTotalTime{0};
 
   friend class AutoRedirectVetoNotifier;
   friend class HttpAsyncAborter<nsHttpChannel>;
 
-  uint32_t mRedirectType;
+  uint32_t mRedirectType{0};
 
   static const uint32_t WAIT_FOR_CACHE_ENTRY = 1;
-  static const uint32_t WAIT_FOR_OFFLINE_CACHE_ENTRY = 2;
 
-  bool mCacheOpenWithPriority;
-  uint32_t mCacheQueueSizeWhenOpen;
+  bool mCacheOpenWithPriority{false};
+  uint32_t mCacheQueueSizeWhenOpen{0};
 
-  Atomic<bool, Relaxed> mCachedContentIsValid;
+  Atomic<bool, Relaxed> mCachedContentIsValid{false};
+  Atomic<bool> mIsAuthChannel{false};
+  Atomic<bool> mAuthRetryPending{false};
 
+  // clang-format off
   // state flags
-  uint32_t mCachedContentIsPartial : 1;
-  uint32_t mCacheOnlyMetadata : 1;
-  uint32_t mTransactionReplaced : 1;
-  uint32_t mAuthRetryPending : 1;
-  uint32_t mProxyAuthPending : 1;
-  // Set if before the first authentication attempt a custom authorization
-  // header has been set on the channel.  This will make that custom header
-  // go to the server instead of any cached credentials.
-  uint32_t mCustomAuthHeader : 1;
-  uint32_t mResuming : 1;
-  uint32_t mInitedCacheEntry : 1;
-  // True if we are loading a fallback cache entry from the
-  // application cache.
-  uint32_t mFallbackChannel : 1;
-  // True if consumer added its own If-None-Match or If-Modified-Since
-  // headers. In such a case we must not override them in the cache code
-  // and also we want to pass possible 304 code response through.
-  uint32_t mCustomConditionalRequest : 1;
-  uint32_t mFallingBack : 1;
-  uint32_t mWaitingForRedirectCallback : 1;
-  // True if mRequestTime has been set. In such a case it is safe to update
-  // the cache entry's expiration time. Otherwise, it is not(see bug 567360).
-  uint32_t mRequestTimeInitialized : 1;
-  uint32_t mCacheEntryIsReadOnly : 1;
-  uint32_t mCacheEntryIsWriteOnly : 1;
-  // see WAIT_FOR_* constants above
-  uint32_t mCacheEntriesToWaitFor : 2;
-  // whether cache entry data write was in progress during cache entry check
-  // when true, after we finish read from cache we must check all data
-  // had been loaded from cache. If not, then an error has to be propagated
-  // to the consumer.
-  uint32_t mConcurrentCacheAccess : 1;
-  // whether the request is setup be byte-range
-  uint32_t mIsPartialRequest : 1;
-  // true iff there is AutoRedirectVetoNotifier on the stack
-  uint32_t mHasAutoRedirectVetoNotifier : 1;
-  // consumers set this to true to use cache pinning, this has effect
-  // only when the channel is in an app context
-  uint32_t mPinCacheContent : 1;
-  // True if CORS preflight has been performed
-  uint32_t mIsCorsPreflightDone : 1;
+  MOZ_ATOMIC_BITFIELDS(mAtomicBitfields5, 32, (
+    (uint32_t, CachedContentIsPartial, 1),
+    (uint32_t, CacheOnlyMetadata, 1),
+    (uint32_t, TransactionReplaced, 1),
+    (uint32_t, ProxyAuthPending, 1),
+    // Set if before the first authentication attempt a custom authorization
+    // header has been set on the channel.  This will make that custom header
+    // go to the server instead of any cached credentials.
+    (uint32_t, CustomAuthHeader, 1),
+    (uint32_t, Resuming, 1),
+    (uint32_t, InitedCacheEntry, 1),
+    // True if consumer added its own If-None-Match or If-Modified-Since
+    // headers. In such a case we must not override them in the cache code
+    // and also we want to pass possible 304 code response through.
+    (uint32_t, CustomConditionalRequest, 1),
+    (uint32_t, WaitingForRedirectCallback, 1),
+    // True if mRequestTime has been set. In such a case it is safe to update
+    // the cache entry's expiration time. Otherwise, it is not(see bug 567360).
+    (uint32_t, RequestTimeInitialized, 1),
+    (uint32_t, CacheEntryIsReadOnly, 1),
+    (uint32_t, CacheEntryIsWriteOnly, 1),
+    // see WAIT_FOR_* constants above
+    (uint32_t, WaitForCacheEntry, 1),
+    // whether cache entry data write was in progress during cache entry check
+    // when true, after we finish read from cache we must check all data
+    // had been loaded from cache. If not, then an error has to be propagated
+    // to the consumer.
+    (uint32_t, ConcurrentCacheAccess, 1),
+    // whether the request is setup be byte-range
+    (uint32_t, IsPartialRequest, 1),
+    // true iff there is AutoRedirectVetoNotifier on the stack
+    (uint32_t, HasAutoRedirectVetoNotifier, 1),
+    // consumers set this to true to use cache pinning, this has effect
+    // only when the channel is in an app context
+    (uint32_t, PinCacheContent, 1),
+    // True if CORS preflight has been performed
+    (uint32_t, IsCorsPreflightDone, 1),
 
-  // if the http transaction was performed (i.e. not cached) and
-  // the result in OnStopRequest was known to be correctly delimited
-  // by chunking, content-length, or h2 end-stream framing
-  uint32_t mStronglyFramed : 1;
+    // if the http transaction was performed (i.e. not cached) and
+    // the result in OnStopRequest was known to be correctly delimited
+    // by chunking, content-length, or h2 end-stream framing
+    (uint32_t, StronglyFramed, 1),
 
-  // true if an HTTP transaction is created for the socket thread
-  uint32_t mUsedNetwork : 1;
+    // true if an HTTP transaction is created for the socket thread
+    (uint32_t, UsedNetwork, 1),
 
-  // the next authentication request can be sent on a whole new connection
-  uint32_t mAuthConnectionRestartable : 1;
+    // the next authentication request can be sent on a whole new connection
+    (uint32_t, AuthConnectionRestartable, 1),
 
-  // True if the channel classifier has marked the channel to be cancelled due
-  // to the safe-browsing classifier rules, but the asynchronous cancellation
-  // process hasn't finished yet.
-  uint32_t mChannelClassifierCancellationPending : 1;
+    // True if the channel classifier has marked the channel to be cancelled due
+    // to the safe-browsing classifier rules, but the asynchronous cancellation
+    // process hasn't finished yet.
+    (uint32_t, ChannelClassifierCancellationPending, 1),
 
-  // True only when we are between Resume and async fire of mCallOnResume.
-  // Used to suspend any newly created pumps in mCallOnResume handler.
-  uint32_t mAsyncResumePending : 1;
+    // True only when we are between Resume and async fire of mCallOnResume.
+    // Used to suspend any newly created pumps in mCallOnResume handler.
+    (uint32_t, AsyncResumePending, 1),
 
-  // True only when we have checked whether this channel has been isolated for
-  // anti-tracking purposes.
-  uint32_t mHasBeenIsolatedChecked : 1;
-  // True only when we have determined this channel should be isolated for
-  // anti-tracking purposes.  Can never ben true unless mHasBeenIsolatedChecked
-  // is true.
-  uint32_t mIsIsolated : 1;
+    // True if the data will be sent from the socket process to the
+    // content process directly.
+    (uint32_t, DataSentToChildProcess, 1),
 
-  // True only when we have computed the value of the top window origin.
-  uint32_t mTopWindowOriginComputed : 1;
+    (uint32_t, UseHTTPSSVC, 1),
+    (uint32_t, WaitHTTPSSVCRecord, 1)
+  ))
 
-  // True if the data has already been sent from the socket process to the
-  // content process.
-  uint32_t mDataAlreadySent : 1;
-
-  // The origin of the top window, only valid when mTopWindowOriginComputed is
-  // true.
-  nsCString mTopWindowOrigin;
+  // Broken up into two bitfields to avoid alignment requirements of uint64_t.
+  // (Too many bits used for one uint32_t.)
+  MOZ_ATOMIC_BITFIELDS(mAtomicBitfields6, 32, (
+    // Only set to true when we receive an HTTPSSVC record before the
+    // transaction is created.
+    (uint32_t, HTTPSSVCTelemetryReported, 1),
+    (uint32_t, EchConfigUsed, 1)
+  ))
+  // clang-format on
 
   nsTArray<nsContinueRedirectionFunc> mRedirectFuncStack;
 
   // Needed for accurate DNS timing
   RefPtr<nsDNSPrefetch> mDNSPrefetch;
 
-  uint32_t mPushedStreamId;
+  uint32_t mPushedStreamId{0};
   RefPtr<HttpTransactionShell> mTransWithPushedStream;
 
   // True if the channel's principal was found on a phishing, malware, or
   // tracking (if tracking protection is enabled) blocklist
-  bool mLocalBlocklist;
+  bool mLocalBlocklist{false};
 
   [[nodiscard]] nsresult WaitForRedirectCallback();
   void PushRedirectAsyncFunc(nsContinueRedirectionFunc func);
@@ -770,7 +713,8 @@ class nsHttpChannel final : public HttpBaseChannel,
 
   // A function we trigger when untail callback is triggered by our request
   // context in case this channel was tail-blocked.
-  nsresult (nsHttpChannel::*mOnTailUnblock)();
+  using TailUnblockCallback = nsresult (nsHttpChannel::*)();
+  TailUnblockCallback mOnTailUnblock{nullptr};
   // Called on untail when tailed during AsyncOpen execution.
   nsresult AsyncOpenOnTailUnblock();
   // Called on untail when tailed because of being a tracking resource.
@@ -781,10 +725,8 @@ class nsHttpChannel final : public HttpBaseChannel,
   // If non-null, warnings should be reported to this object.
   RefPtr<HttpChannelSecurityWarningReporter> mWarningReporter;
 
-  RefPtr<ADivertableParentChannel> mParentChannel;
-
   // True if the channel is reading from cache.
-  Atomic<bool> mIsReadingFromCache;
+  Atomic<bool> mIsReadingFromCache{false};
 
   // These next members are only used in unit tests to delay the call to
   // cache->AsyncOpenURI in order to race the cache with the network.
@@ -798,11 +740,11 @@ class nsHttpChannel final : public HttpBaseChannel,
     RESPONSE_FROM_CACHE = 1,   // response coming from cache. no network.
     RESPONSE_FROM_NETWORK = 2  // response coming from the network
   };
-  Atomic<ResponseSource, Relaxed> mFirstResponseSource;
+  Atomic<ResponseSource, Relaxed> mFirstResponseSource{RESPONSE_PENDING};
 
   // Determines if it's possible and advisable to race the network request
   // with the cache fetch, and proceeds to do so.
-  nsresult MaybeRaceCacheWithNetwork();
+  void MaybeRaceCacheWithNetwork();
 
   // Creates a new cache entry when network wins the race to ensure we have
   // the latest version of the resource in the cache. Otherwise we might return
@@ -812,6 +754,9 @@ class nsHttpChannel final : public HttpBaseChannel,
   nsresult TriggerNetworkWithDelay(uint32_t aDelay);
   nsresult TriggerNetwork();
   void CancelNetworkRequest(nsresult aStatus);
+
+  void SetHTTPSSVCRecord(already_AddRefed<nsIDNSHTTPSSVCRecord>&& aRecord);
+
   // Timer used to delay the network request, or to trigger the network
   // request if retrieving the cache entry takes too long.
   nsCOMPtr<nsITimer> mNetworkTriggerTimer;
@@ -821,15 +766,15 @@ class nsHttpChannel final : public HttpBaseChannel,
   bool mStaleRevalidation = false;
   // Will be true if the onCacheEntryAvailable callback is not called by the
   // time we send the network request
-  Atomic<bool> mRaceCacheWithNetwork;
-  uint32_t mRaceDelay;
+  Atomic<bool> mRaceCacheWithNetwork{false};
+  uint32_t mRaceDelay{0};
   // If true then OnCacheEntryAvailable should ignore the entry, because
   // SetupTransaction removed conditional headers and decisions made in
   // OnCacheEntryCheck are no longer valid.
-  bool mIgnoreCacheEntry;
+  bool mIgnoreCacheEntry{false};
   // Lock preventing SetupTransaction/MaybeCreateCacheEntryWhenRCWN and
   // OnCacheEntryCheck being called at the same time.
-  mozilla::Mutex mRCWNLock;
+  mozilla::Mutex mRCWNLock{"nsHttpChannel.mRCWNLock"};
 
   TimeStamp mNavigationStartTimeStamp;
 
@@ -843,7 +788,12 @@ class nsHttpChannel final : public HttpBaseChannel,
 
   // We update the value of mProxyConnectResponseCode when OnStartRequest is
   // called and reset the value when we switch to another failover proxy.
-  int32_t mProxyConnectResponseCode;
+  int32_t mProxyConnectResponseCode{0};
+
+  // If mHTTPSSVCRecord has value, it means OnHTTPSRRAvailable() is called and
+  // we got the result of HTTPS RR query. Otherwise, it means we are still
+  // waiting for the result or the query is not performed.
+  Maybe<nsCOMPtr<nsIDNSHTTPSSVCRecord>> mHTTPSSVCRecord;
 
  protected:
   virtual void DoNotifyListenerCleanup() override;
@@ -855,7 +805,7 @@ class nsHttpChannel final : public HttpBaseChannel,
   virtual void DoAsyncAbort(nsresult aStatus) override;
 
  private:  // cache telemetry
-  bool mDidReval;
+  bool mDidReval{false};
 };
 
 NS_DEFINE_STATIC_IID_ACCESSOR(nsHttpChannel, NS_HTTPCHANNEL_IID)
