@@ -17,6 +17,7 @@
 #include <netinet/if_ether.h>
 #include <arpa/inet.h>
 #include <ifaddrs.h>
+#include <resolv.h>
 
 #include "nsCOMPtr.h"
 #include "nsIObserverService.h"
@@ -51,6 +52,12 @@ static LazyLogModule gNotifyAddrLog("nsNotifyAddr");
 // the calaulation of network id.
 static const uint32_t kNetworkIdDelayAfterChange = 3000;
 
+// When you remove search domains from the settings page and hit Apply a
+// network change event is generated, but res.dnsrch is not updated to the
+// correct values. Thus, after a network change, we add a small delay to
+// the runnable so the OS has the chance to update the values.
+static const uint32_t kDNSSuffixDelayAfterChange = 50;
+
 // If non-successful, extract the error code and return it.  This
 // error code dance is inspired by
 // http://developer.apple.com/technotes/tn/tn1145.html
@@ -78,7 +85,7 @@ static void CFReleaseSafe(CFTypeRef cf) {
   }
 }
 
-NS_IMPL_ISUPPORTS(nsNetworkLinkService, nsINetworkLinkService, nsIObserver)
+NS_IMPL_ISUPPORTS(nsNetworkLinkService, nsINetworkLinkService, nsIObserver, nsITimerCallback)
 
 nsNetworkLinkService::nsNetworkLinkService()
     : mLinkUp(true),
@@ -124,201 +131,32 @@ nsNetworkLinkService::GetPlatformDNSIndications(uint32_t* aPlatformDNSIndication
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-// Note that this function is copied from xpcom/io/nsLocalFileUnix.cpp.
-static nsresult CFStringReftoUTF8(CFStringRef aInStrRef, nsACString& aOutStr) {
-  // first see if the conversion would succeed and find the length of the result
-  CFIndex usedBufLen, inStrLen = ::CFStringGetLength(aInStrRef);
-  CFIndex charsConverted =
-      ::CFStringGetBytes(aInStrRef, CFRangeMake(0, inStrLen), kCFStringEncodingUTF8, 0, false,
-                         nullptr, 0, &usedBufLen);
-  if (charsConverted == inStrLen) {
-    // all characters converted, do the actual conversion
-    aOutStr.SetLength(usedBufLen);
-    if (aOutStr.Length() != (unsigned int)usedBufLen) {
-      return NS_ERROR_OUT_OF_MEMORY;
-    }
-    UInt8* buffer = (UInt8*)aOutStr.BeginWriting();
-    ::CFStringGetBytes(aInStrRef, CFRangeMake(0, inStrLen), kCFStringEncodingUTF8, 0, false, buffer,
-                       usedBufLen, &usedBufLen);
-    return NS_OK;
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
-static void GetDNSSearchDomains(const SCDynamicStoreRef aStoreRef, const CFStringRef aPattern,
-                                nsTArray<nsCString>& aResult) {
-  CFDictionaryRef dnsDict = (CFDictionaryRef)SCDynamicStoreCopyValue(aStoreRef, aPattern);
-  if (dnsDict) {
-    CFTypeRef domains;
-    if (::CFDictionaryGetValueIfPresent(dnsDict, kSCPropNetDNSSearchDomains, &domains)) {
-      if (domains && ::CFGetTypeID(domains) == ::CFArrayGetTypeID()) {
-        int count = ::CFArrayGetCount(static_cast<CFArrayRef>(domains));
-        for (int i = 0; i < count; i++) {
-          CFTypeRef domain = ::CFArrayGetValueAtIndex(static_cast<CFArrayRef>(domains), i);
-          if (domain && ::CFGetTypeID(domain) == ::CFStringGetTypeID()) {
-            nsAutoCString domainStr;
-            if (NS_SUCCEEDED(CFStringReftoUTF8(static_cast<CFStringRef>(domain), domainStr))) {
-              LOG(("DNS search domain [%s]\n", domainStr.get()));
-              aResult.AppendElement(domainStr);
-            }
-          }
-        }
-      }
-    }
-  }
-  CFReleaseSafe(dnsDict);
-}
-
-static OSStatus GetPrimaryServiceInfo(const SCDynamicStoreRef aStoreRef, nsACString& aServiceName,
-                                      nsACString& aServiceId) {
-  OSStatus err = getErrorCodePtr(aStoreRef);
-  if (err != noErr) {
-    return err;
-  }
-
-  CFStringRef globalIPv4StateKey = ::SCDynamicStoreKeyCreateNetworkGlobalEntity(
-      nullptr, kSCDynamicStoreDomainState, kSCEntNetIPv4);
-  err = getErrorCodePtr(globalIPv4StateKey);
-
-  // Get the primary interface.
-  CFDictionaryRef primaryInterface = nullptr;
-  if (err == noErr) {
-    primaryInterface = (CFDictionaryRef)::SCDynamicStoreCopyValue(aStoreRef, globalIPv4StateKey);
-    err = getErrorCodePtr(primaryInterface);
-  }
-
-  CFTypeRef serviceId;
-  if (err == noErr && ::CFDictionaryGetValueIfPresent(
-                          primaryInterface, kSCDynamicStorePropNetPrimaryService, &serviceId)) {
-    err = -1;
-    if (serviceId && ::CFGetTypeID(serviceId) == ::CFStringGetTypeID()) {
-      if (NS_SUCCEEDED(CFStringReftoUTF8(static_cast<CFStringRef>(serviceId), aServiceId))) {
-        err = noErr;
-      }
-    }
-  }
-
-  CFTypeRef serviceName;
-  if (err == noErr && ::CFDictionaryGetValueIfPresent(
-                          primaryInterface, kSCDynamicStorePropNetPrimaryInterface, &serviceName)) {
-    err = -1;
-    if (serviceName && ::CFGetTypeID(serviceName) == ::CFStringGetTypeID()) {
-      if (NS_SUCCEEDED(CFStringReftoUTF8(static_cast<CFStringRef>(serviceName), aServiceName))) {
-        err = noErr;
-      }
-    }
-  }
-
-  CFReleaseSafe(globalIPv4StateKey);
-  CFReleaseSafe(primaryInterface);
-
-  if (err == noErr) {
-    return noErr;
-  }
-
-  return err;
-}
-
-static OSStatus IsInterfaceActive(const SCDynamicStoreRef aStoreRef, const char* aInterfaceName,
-                                  bool& aResult) {
-  aResult = false;
-
-  OSStatus err = getErrorCodePtr(aStoreRef);
-  if (err != noErr) {
-    return err;
-  }
-
-  CFStringRef serviceNameRef = nullptr;
-  if (err == noErr) {
-    serviceNameRef = CFStringCreateWithCString(nullptr, aInterfaceName, kCFStringEncodingUTF8);
-    err = getErrorCodePtr(serviceNameRef);
-  }
-
-  CFStringRef linkPattern = nullptr;
-  if (err == noErr) {
-    linkPattern = ::SCDynamicStoreKeyCreateNetworkInterfaceEntity(
-        nullptr, kSCDynamicStoreDomainState, serviceNameRef, kSCEntNetLink);
-    err = getErrorCodePtr(linkPattern);
-  }
-
-  CFDictionaryRef dict = nullptr;
-  if (err == noErr) {
-    dict = (CFDictionaryRef)SCDynamicStoreCopyValue(aStoreRef, linkPattern);
-    err = getErrorCodePtr(dict);
-  }
-
-  if (err == noErr) {
-    CFTypeRef activeRef;
-    err = -1;
-    if (::CFDictionaryGetValueIfPresent(dict, kSCPropNetLinkActive, &activeRef)) {
-      if (activeRef && ::CFGetTypeID(activeRef) == ::CFBooleanGetTypeID()) {
-        aResult = ::CFBooleanGetValue(static_cast<CFBooleanRef>(activeRef));
-        err = noErr;
-      }
-    }
-  }
-
-  CFReleaseSafe(dict);
-  CFReleaseSafe(linkPattern);
-  CFReleaseSafe(serviceNameRef);
-  return err;
-}
-
 void nsNetworkLinkService::GetDnsSuffixListInternal() {
   MOZ_ASSERT(!NS_IsMainThread());
+  LOG(("GetDnsSuffixListInternal"));
 
-  RefPtr<nsNetworkLinkService> self = this;
-  auto sendNotification = mozilla::MakeScopeExit([self] {
+  auto sendNotification = mozilla::MakeScopeExit([self = RefPtr{this}] {
     NS_DispatchToMainThread(NS_NewRunnableFunction(
         "nsNetworkLinkService::GetDnsSuffixListInternal",
         [self]() { self->NotifyObservers(NS_DNS_SUFFIX_LIST_UPDATED_TOPIC, nullptr); }));
   });
 
-  nsAutoCString primaryServiceName;
-  nsAutoCString primaryServiceId;
-  OSStatus err = GetPrimaryServiceInfo(mStoreRef, primaryServiceName, primaryServiceId);
-
-  bool active = false;
-  if (err == noErr) {
-    err = IsInterfaceActive(mStoreRef, primaryServiceName.get(), active);
-    LOG(("primaryServiceName:[%s] active=%d", primaryServiceName.get(), active));
-  }
-
-  if (err != noErr || !active) {
-    // Primary interface is not active. Clear the DNS suffix list.
-    MutexAutoLock lock(mMutex);
-    mDNSSuffixList.Clear();
-    return;
-  }
-
   nsTArray<nsCString> result;
 
-  CFStringRef serviceIdRef =
-      CFStringCreateWithCString(nullptr, primaryServiceId.get(), kCFStringEncodingUTF8);
-  err = getErrorCodePtr(serviceIdRef);
-  if (err == noErr) {
-    CFStringRef dnsSetupPattern = ::SCDynamicStoreKeyCreateNetworkServiceEntity(
-        nullptr, kSCDynamicStoreDomainSetup, serviceIdRef, kSCEntNetDNS);
-    CFStringRef dnsStatePattern = ::SCDynamicStoreKeyCreateNetworkServiceEntity(
-        nullptr, kSCDynamicStoreDomainState, serviceIdRef, kSCEntNetDNS);
-    err = getErrorCodePtr(dnsSetupPattern);
-    if (err == noErr) {
-      err = getErrorCodePtr(dnsStatePattern);
-      if (err == noErr) {
-        GetDNSSearchDomains(mStoreRef, dnsSetupPattern, result);
-        GetDNSSearchDomains(mStoreRef, dnsStatePattern, result);
+  struct __res_state res;
+  if (res_ninit(&res) == 0) {
+    for (int i = 0; i < MAXDNSRCH; i++) {
+      if (!res.dnsrch[i]) {
+        break;
       }
+      LOG(("DNS search domain from [%s]\n", res.dnsrch[i]));
+      result.AppendElement(nsCString(res.dnsrch[i]));
     }
-    CFReleaseSafe(dnsStatePattern);
-    CFReleaseSafe(dnsSetupPattern);
+    res_nclose(&res);
   }
-  CFReleaseSafe(serviceIdRef);
 
-  if (err == noErr) {
-    MutexAutoLock lock(mMutex);
-    mDNSSuffixList = std::move(result);
-  }
+  MutexAutoLock lock(mMutex);
+  mDNSSuffixList = std::move(result);
 }
 
 NS_IMETHODIMP
@@ -330,6 +168,16 @@ nsNetworkLinkService::GetDnsSuffixList(nsTArray<nsCString>& aDnsSuffixList) {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsNetworkLinkService::GetResolvers(nsTArray<RefPtr<nsINetAddr>>& aResolvers) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsNetworkLinkService::GetNativeResolvers(nsTArray<mozilla::net::NetAddr>& aResolvers) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 #ifndef SA_SIZE
 #  define SA_SIZE(sa)                                 \
     ((!(sa) || ((struct sockaddr*)(sa))->sa_len == 0) \
@@ -826,18 +674,25 @@ void nsNetworkLinkService::NetworkConfigChanged(SCDynamicStoreRef aStoreREf,
   }
 
   if (dnsConfigChanged) {
-    service->DNSConfigChanged();
+    service->DNSConfigChanged(kDNSSuffixDelayAfterChange);
   }
 }
 
-void nsNetworkLinkService::DNSConfigChanged() {
+void nsNetworkLinkService::DNSConfigChanged(uint32_t aDelayMs) {
   LOG(("nsNetworkLinkService::DNSConfigChanged"));
   nsCOMPtr<nsIEventTarget> target = do_GetService(NS_STREAMTRANSPORTSERVICE_CONTRACTID);
-  if (target) {
-    RefPtr<nsNetworkLinkService> self = this;
-    MOZ_ALWAYS_SUCCEEDS(
-        target->Dispatch(NS_NewRunnableFunction("nsNetworkLinkService::GetDnsSuffixListInternal",
-                                                [self]() { self->GetDnsSuffixListInternal(); })));
+  if (!target) {
+    return;
+  }
+  if (aDelayMs) {
+    MOZ_ALWAYS_SUCCEEDS(target->DelayedDispatch(
+        NS_NewRunnableFunction("nsNetworkLinkService::GetDnsSuffixListInternal",
+                               [self = RefPtr{this}]() { self->GetDnsSuffixListInternal(); }),
+        aDelayMs));
+  } else {
+    MOZ_ALWAYS_SUCCEEDS(target->Dispatch(
+        NS_NewRunnableFunction("nsNetworkLinkService::GetDnsSuffixListInternal",
+                               [self = RefPtr{this}]() { self->GetDnsSuffixListInternal(); })));
   }
 }
 
@@ -957,7 +812,7 @@ nsresult nsNetworkLinkService::Init(void) {
 
   calculateNetworkIdWithDelay(0);
 
-  DNSConfigChanged();
+  DNSConfigChanged(0);
 
   return NS_OK;
 }
@@ -1061,5 +916,5 @@ void nsNetworkLinkService::ReachabilityChanged(SCNetworkReachabilityRef target,
   service->calculateNetworkIdWithDelay(kNetworkIdDelayAfterChange);
   // If a new interface is up or the order of interfaces is changed, we should
   // update the DNS suffix list.
-  service->DNSConfigChanged();
+  service->DNSConfigChanged(0);
 }
