@@ -9,8 +9,7 @@ const { AppConstants } = ChromeUtils.import(
 const { ObjectUtils } = ChromeUtils.import(
   "resource://gre/modules/ObjectUtils.jsm"
 );
-
-const IS_ANDROID = AppConstants.platform == "android";
+const { setTimeout } = ChromeUtils.import("resource://gre/modules/Timer.jsm");
 
 const { RemoteSettings } = ChromeUtils.import(
   "resource://services-settings/remote-settings.js"
@@ -23,12 +22,19 @@ const { TelemetryTestUtils } = ChromeUtils.import(
   "resource://testing-common/TelemetryTestUtils.jsm"
 );
 
+const IS_ANDROID = AppConstants.platform == "android";
+
 const BinaryInputStream = CC(
   "@mozilla.org/binaryinputstream;1",
   "nsIBinaryInputStream",
   "setInputStream"
 );
 
+const TELEMETRY_EVENTS_FILTERS = {
+  category: "uptake.remotecontent.result",
+  method: "uptake",
+};
+
 let server;
 let client;
 let clientWithDump;
@@ -68,7 +74,7 @@ function run_test() {
 
   server.registerPathHandler("/v1/", handleResponse);
   server.registerPathHandler(
-    "/v1/buckets/monitor/collections/changes/records",
+    "/v1/buckets/monitor/collections/changes/changeset",
     handleResponse
   );
   server.registerPathHandler(
@@ -101,6 +107,12 @@ add_task(async function test_records_obtained_from_server_are_stored_in_db() {
   // Our test data has a single record; it should be in the local collection
   const list = await client.get();
   equal(list.length, 1);
+
+  const timestamp = await client.db.getLastModified();
+  equal(timestamp, 3000, "timestamp was stored");
+
+  const { signature } = await client.db.getMetadata();
+  equal(signature.signature, "abcdef", "metadata was stored");
 });
 add_task(clear_state);
 
@@ -216,13 +228,10 @@ add_task(clear_state);
 add_task(
   async function test_records_changes_are_overwritten_by_server_changes() {
     // Create some local conflicting data, and make sure it syncs without error.
-    await client.db.create(
-      {
-        website: "",
-        id: "9d500963-d80e-3a91-6e74-66f3811b99cc",
-      },
-      { useRecordId: true }
-    );
+    await client.db.create({
+      website: "",
+      id: "9d500963-d80e-3a91-6e74-66f3811b99cc",
+    });
 
     await client.maybeSync(2000);
 
@@ -257,6 +266,21 @@ add_task(async function test_get_does_not_load_dump_when_pref_is_false() {
 });
 add_task(clear_state);
 
+add_task(async function test_get_loads_dump_only_once_if_called_in_parallel() {
+  const backup = clientWithDump._importJSONDump;
+  let callCount = 0;
+  clientWithDump._importJSONDump = async () => {
+    callCount++;
+    // eslint-disable-next-line mozilla/no-arbitrary-setTimeout
+    await new Promise(resolve => setTimeout(resolve, 100));
+    return 42;
+  };
+  await Promise.all([clientWithDump.get(), clientWithDump.get()]);
+  equal(callCount, 1, "JSON dump was called more than once");
+  clientWithDump._importJSONDump = backup;
+});
+add_task(clear_state);
+
 add_task(async function test_get_falls_back_to_dump_if_db_fails() {
   if (IS_ANDROID) {
     // Skip test: we don't ship remote settings dumps on Android (see package-manifest).
@@ -267,7 +291,7 @@ add_task(async function test_get_falls_back_to_dump_if_db_fails() {
     throw new Error("Unknown error");
   };
 
-  const records = await clientWithDump.get();
+  const records = await clientWithDump.get({ dumpFallback: true });
   ok(records.length > 0, "dump content is returned");
 
   // If fallback is disabled, error is thrown.
@@ -283,6 +307,78 @@ add_task(async function test_get_falls_back_to_dump_if_db_fails() {
 });
 add_task(clear_state);
 
+add_task(async function test_get_sorts_results_if_specified() {
+  await client.db.importChanges(
+    {},
+    42,
+    [
+      {
+        field: 12,
+        id: "9d500963-d80e-3a91-6e74-66f3811b99cc",
+      },
+      {
+        field: 7,
+        id: "d83444a4-f348-4cd8-8228-842cb927db9f",
+      },
+    ],
+    { clear: true }
+  );
+
+  const records = await client.get({ order: "field" });
+  ok(
+    records[0].field < records[records.length - 1].field,
+    "records are sorted"
+  );
+});
+add_task(clear_state);
+
+add_task(async function test_get_falls_back_sorts_results() {
+  if (IS_ANDROID) {
+    // Skip test: we don't ship remote settings dumps on Android (see package-manifest).
+    return;
+  }
+  const backup = clientWithDump.db.getLastModified;
+  clientWithDump.db.getLastModified = () => {
+    throw new Error("Unknown error");
+  };
+
+  const records = await clientWithDump.get({
+    dumpFallback: true,
+    order: "-id",
+  });
+
+  ok(records[0].id > records[records.length - 1].id, "records are sorted");
+
+  clientWithDump.db.getLastModified = backup;
+});
+add_task(clear_state);
+
+add_task(async function test_get_falls_back_to_dump_if_db_fails_later() {
+  if (IS_ANDROID) {
+    // Skip test: we don't ship remote settings dumps on Android (see package-manifest).
+    return;
+  }
+  const backup = clientWithDump.db.list;
+  clientWithDump.db.list = () => {
+    throw new Error("Unknown error");
+  };
+
+  const records = await clientWithDump.get({ dumpFallback: true });
+  ok(records.length > 0, "dump content is returned");
+
+  // If fallback is disabled, error is thrown.
+  let error;
+  try {
+    await clientWithDump.get({ dumpFallback: false });
+  } catch (e) {
+    error = e;
+  }
+  equal(error.message, "Unknown error");
+
+  clientWithDump.db.list = backup;
+});
+add_task(clear_state);
+
 add_task(async function test_get_does_not_sync_if_empty_dump_is_provided() {
   if (IS_ANDROID) {
     // Skip test: we don't ship remote settings dumps on Android (see package-manifest).
@@ -437,6 +533,26 @@ add_task(async function test_get_does_not_verify_signature_if_load_dump() {
 });
 add_task(clear_state);
 
+add_task(
+  async function test_get_does_verify_signature_if_json_loaded_in_parallel() {
+    const backup = clientWithDump._verifier;
+    let callCount = 0;
+    clientWithDump._verifier = {
+      async asyncVerifyContentSignature(serialized, signature) {
+        callCount++;
+        return true;
+      },
+    };
+    await Promise.all([
+      clientWithDump.get({ verifySignature: true }),
+      clientWithDump.get({ verifySignature: true }),
+    ]);
+    equal(callCount, 0, "No need to verify signatures if JSON dump is loaded");
+    clientWithDump._verifier = backup;
+  }
+);
+add_task(clear_state);
+
 add_task(async function test_sync_runs_once_only() {
   const backup = Utils.log.warn;
   const messages = [];
@@ -669,19 +785,22 @@ add_task(
     await withFakeChannel("nightly", async () => {
       await client.maybeSync(2000);
 
-      TelemetryTestUtils.assertEvents([
+      TelemetryTestUtils.assertEvents(
         [
-          "uptake.remotecontent.result",
-          "uptake",
-          "remotesettings",
-          UptakeTelemetry.STATUS.SUCCESS,
-          {
-            source: client.identifier,
-            duration: v => v > 0,
-            trigger: "manual",
-          },
+          [
+            "uptake.remotecontent.result",
+            "uptake",
+            "remotesettings",
+            UptakeTelemetry.STATUS.SUCCESS,
+            {
+              source: client.identifier,
+              duration: v => v > 0,
+              trigger: "manual",
+            },
+          ],
         ],
-      ]);
+        TELEMETRY_EVENTS_FILTERS
+      );
     });
   }
 );
@@ -704,7 +823,7 @@ add_task(async function test_telemetry_reports_if_application_fails() {
 add_task(clear_state);
 
 add_task(async function test_telemetry_reports_if_sync_fails() {
-  await client.db.saveLastModified(9999);
+  await client.db.importChanges({}, 9999);
 
   const startHistogram = getUptakeTelemetrySnapshot(client.identifier);
 
@@ -719,7 +838,7 @@ add_task(async function test_telemetry_reports_if_sync_fails() {
 add_task(clear_state);
 
 add_task(async function test_telemetry_reports_if_parsing_fails() {
-  await client.db.saveLastModified(10000);
+  await client.db.importChanges({}, 10000);
 
   const startHistogram = getUptakeTelemetrySnapshot(client.identifier);
 
@@ -734,7 +853,7 @@ add_task(async function test_telemetry_reports_if_parsing_fails() {
 add_task(clear_state);
 
 add_task(async function test_telemetry_reports_if_fetching_signature_fails() {
-  await client.db.saveLastModified(11000);
+  await client.db.importChanges({}, 11000);
 
   const startHistogram = getUptakeTelemetrySnapshot(client.identifier);
 
@@ -799,20 +918,23 @@ add_task(async function test_telemetry_reports_error_name_as_event_nightly() {
       await client.maybeSync(2000);
     } catch (e) {}
 
-    TelemetryTestUtils.assertEvents([
+    TelemetryTestUtils.assertEvents(
       [
-        "uptake.remotecontent.result",
-        "uptake",
-        "remotesettings",
-        UptakeTelemetry.STATUS.UNKNOWN_ERROR,
-        {
-          source: client.identifier,
-          trigger: "manual",
-          duration: v => v >= 0,
-          errorName: "ThrownError",
-        },
+        [
+          "uptake.remotecontent.result",
+          "uptake",
+          "remotesettings",
+          UptakeTelemetry.STATUS.UNKNOWN_ERROR,
+          {
+            source: client.identifier,
+            trigger: "manual",
+            duration: v => v >= 0,
+            errorName: "ThrownError",
+          },
+        ],
       ],
-    ]);
+      TELEMETRY_EVENTS_FILTERS
+    );
   });
 
   client.db.list = backup;
@@ -831,6 +953,20 @@ add_task(async function test_bucketname_changes_when_bucket_pref_changes() {
 });
 add_task(clear_state);
 
+add_task(async function test_bucket_pref_ignored_when_bucketName_set() {
+  let clientWithBucket = RemoteSettings("coll", { bucketName: "buck" });
+  equal(clientWithBucket.collectionName, "coll");
+  equal(clientWithBucket.bucketName, "buck");
+
+  Services.prefs.setCharPref(
+    "services.settings.default_bucket",
+    "coll-preview"
+  );
+
+  equal(clientWithBucket.bucketName, "buck");
+});
+add_task(clear_state);
+
 add_task(
   async function test_get_loads_default_records_from_a_local_dump_if_preview_collection() {
     if (IS_ANDROID) {
@@ -947,7 +1083,7 @@ function getSampleResponse(req, port) {
         hello: "kinto",
       },
     },
-    "GET:/v1/buckets/monitor/collections/changes/records": {
+    "GET:/v1/buckets/monitor/collections/changes/changeset": {
       sampleHeaders: [
         "Access-Control-Allow-Origin: *",
         "Access-Control-Expose-Headers: Retry-After, Content-Length, Alert, Backoff",
@@ -958,7 +1094,8 @@ function getSampleResponse(req, port) {
       ],
       status: { status: 200, statusText: "OK" },
       responseBody: {
-        data: [
+        timestamp: 5000,
+        changes: [
           {
             id: "4676f0c7-9757-4796-a0e8-b40a5a37a9c9",
             bucket: "main",
@@ -1141,7 +1278,7 @@ wNuvFqc=
         error: "Service Unavailable",
       },
     },
-    "GET:/v1/buckets/monitor/collections/changes/records?collection=password-fields&bucket=main": {
+    "GET:/v1/buckets/monitor/collections/changes/changeset?collection=password-fields&bucket=main&_expected=0": {
       sampleHeaders: [
         "Access-Control-Allow-Origin: *",
         "Access-Control-Expose-Headers: Retry-After, Content-Length, Alert, Backoff",
@@ -1152,7 +1289,8 @@ wNuvFqc=
       ],
       status: { status: 200, statusText: "OK" },
       responseBody: {
-        data: [
+        timestamp: 1338,
+        changes: [
           {
             id: "fe5758d0-c67a-42d0-bb4f-8f2d75106b65",
             bucket: "main",
@@ -1276,7 +1414,7 @@ wNuvFqc=
         ],
       },
     },
-    "GET:/v1/buckets/monitor/collections/changes/records?collection=no-mocked-responses&bucket=main": {
+    "GET:/v1/buckets/monitor/collections/changes/changeset?collection=no-mocked-responses&bucket=main&_expected=0": {
       sampleHeaders: [
         "Access-Control-Allow-Origin: *",
         "Access-Control-Expose-Headers: Retry-After, Content-Length, Alert, Backoff",
