@@ -6,6 +6,10 @@
 #ifndef mozilla_widget_IMEData_h_
 #define mozilla_widget_IMEData_h_
 
+#include "mozilla/CheckedInt.h"
+#include "mozilla/EventForwards.h"
+#include "mozilla/ToString.h"
+
 #include "nsPoint.h"
 #include "nsRect.h"
 #include "nsString.h"
@@ -18,6 +22,167 @@ namespace mozilla {
 
 class WritingMode;
 
+// Helper class to logging string which may contain various Unicode characters
+// and/or may be too long string for logging.
+class MOZ_STACK_CLASS PrintStringDetail : public nsAutoCString {
+ public:
+  static constexpr uint32_t kMaxLengthForCompositionString = 8;
+  static constexpr uint32_t kMaxLengthForSelectedString = 12;
+  static constexpr uint32_t kMaxLengthForEditor = 20;
+
+  PrintStringDetail() = delete;
+  explicit PrintStringDetail(const nsAString& aString,
+                             uint32_t aMaxLength = UINT32_MAX);
+
+ private:
+  static nsCString PrintCharData(char32_t aChar);
+};
+
+// StartAndEndOffsets represents a range in flat-text.
+template <typename IntType>
+class StartAndEndOffsets {
+ protected:
+  static IntType MaxOffset() { return std::numeric_limits<IntType>::max(); }
+
+ public:
+  StartAndEndOffsets() = delete;
+  explicit StartAndEndOffsets(IntType aStartOffset, IntType aEndOffset)
+      : mStartOffset(aStartOffset),
+        mEndOffset(aStartOffset <= aEndOffset ? aEndOffset : aStartOffset) {
+    MOZ_ASSERT(aStartOffset <= mEndOffset);
+  }
+
+  IntType StartOffset() const { return mStartOffset; }
+  IntType Length() const { return mEndOffset - mStartOffset; }
+  IntType EndOffset() const { return mEndOffset; }
+
+  bool IsOffsetInRange(IntType aOffset) const {
+    return aOffset >= mStartOffset && aOffset < mEndOffset;
+  }
+  bool IsOffsetInRangeOrEndOffset(IntType aOffset) const {
+    return aOffset >= mStartOffset && aOffset <= mEndOffset;
+  }
+
+  void MoveTo(IntType aNewStartOffset) {
+    auto delta = static_cast<int64_t>(mStartOffset) - aNewStartOffset;
+    mStartOffset += delta;
+    mEndOffset += delta;
+  }
+  void SetOffsetAndLength(IntType aNewOffset, IntType aNewLength) {
+    mStartOffset = aNewOffset;
+    CheckedInt<IntType> endOffset(aNewOffset + aNewLength);
+    mEndOffset = endOffset.isValid() ? endOffset.value() : MaxOffset();
+  }
+  void SetEndOffset(IntType aEndOffset) {
+    MOZ_ASSERT(mStartOffset <= aEndOffset);
+    mEndOffset = std::max(aEndOffset, mStartOffset);
+  }
+  void SetStartAndEndOffsets(IntType aStartOffset, IntType aEndOffset) {
+    MOZ_ASSERT(aStartOffset <= aEndOffset);
+    mStartOffset = aStartOffset;
+    mEndOffset = aStartOffset <= aEndOffset ? aEndOffset : aStartOffset;
+  }
+  void SetLength(IntType aNewLength) {
+    CheckedInt<IntType> endOffset(mStartOffset + aNewLength);
+    mEndOffset = endOffset.isValid() ? endOffset.value() : MaxOffset();
+  }
+
+  friend std::ostream& operator<<(
+      std::ostream& aStream,
+      const StartAndEndOffsets<IntType>& aStartAndEndOffsets) {
+    aStream << "{ mStartOffset=" << aStartAndEndOffsets.mStartOffset
+            << ", mEndOffset=" << aStartAndEndOffsets.mEndOffset
+            << ", Length()=" << aStartAndEndOffsets.Length() << " }";
+    return aStream;
+  }
+
+ private:
+  IntType mStartOffset;
+  IntType mEndOffset;
+};
+
+// OffsetAndData class is designed for storing composition string and its
+// start offset.  Length() and EndOffset() return only valid length or
+// offset.  I.e., if the string is too long for inserting at the offset,
+// the length is shrunken.  However, the string itself is not shrunken.
+// Therefore, moving it to where all of the string can be contained,
+// they will return longer/bigger value.
+enum class OffsetAndDataFor {
+  CompositionString,
+  SelectedString,
+  EditorString,
+};
+template <typename IntType>
+class OffsetAndData {
+ protected:
+  static IntType MaxOffset() { return std::numeric_limits<IntType>::max(); }
+
+ public:
+  OffsetAndData() = delete;
+  explicit OffsetAndData(
+      IntType aStartOffset, const nsAString& aData,
+      OffsetAndDataFor aFor = OffsetAndDataFor::CompositionString)
+      : mData(aData), mOffset(aStartOffset), mFor(aFor) {}
+
+  IntType StartOffset() const { return mOffset; }
+  IntType Length() const {
+    CheckedInt<IntType> endOffset(mOffset + mData.Length());
+    return endOffset.isValid() ? mData.Length() : MaxOffset() - mOffset;
+  }
+  IntType EndOffset() const { return mOffset + Length(); }
+  StartAndEndOffsets<IntType> CreateStartAndEndOffsets() const {
+    return StartAndEndOffsets<IntType>(StartOffset(), EndOffset());
+  }
+  const nsString& DataRef() const {
+    // In strictly speaking, we should return substring which may be shrunken
+    // for rounding to the max offset.  However, it's unrealistic edge case,
+    // and creating new string is not so cheap job in a hot path.  Therefore,
+    // this just returns the data as-is.
+    return mData;
+  }
+  bool IsDataEmpty() const { return mData.IsEmpty(); }
+
+  bool IsOffsetInRange(IntType aOffset) const {
+    return aOffset >= mOffset && aOffset < EndOffset();
+  }
+  bool IsOffsetInRangeOrEndOffset(IntType aOffset) const {
+    return aOffset >= mOffset && aOffset <= EndOffset();
+  }
+
+  void MoveTo(IntType aNewOffset) { mOffset = aNewOffset; }
+  void SetOffsetAndData(IntType aStartOffset, const nsAString& aData) {
+    mOffset = aStartOffset;
+    mData = aData;
+  }
+  void SetData(const nsAString& aData) { mData = aData; }
+  void TruncateData(uint32_t aLength = 0) { mData.Truncate(aLength); }
+  void ReplaceData(nsAString::size_type aCutStart,
+                   nsAString::size_type aCutLength,
+                   const nsAString& aNewString) {
+    mData.Replace(aCutStart, aCutLength, aNewString);
+  }
+
+  friend std::ostream& operator<<(
+      std::ostream& aStream, const OffsetAndData<IntType>& aOffsetAndData) {
+    const auto maxDataLength =
+        aOffsetAndData.mFor == OffsetAndDataFor::CompositionString
+            ? PrintStringDetail::kMaxLengthForCompositionString
+            : (aOffsetAndData.mFor == OffsetAndDataFor::SelectedString
+                   ? PrintStringDetail::kMaxLengthForSelectedString
+                   : PrintStringDetail::kMaxLengthForEditor);
+    aStream << "{ mOffset=" << aOffsetAndData.mOffset << ", mData="
+            << PrintStringDetail(aOffsetAndData.mData, maxDataLength).get()
+            << ", Length()=" << aOffsetAndData.Length()
+            << ", EndOffset()=" << aOffsetAndData.EndOffset() << " }";
+    return aStream;
+  }
+
+ private:
+  nsString mData;
+  IntType mOffset;
+  OffsetAndDataFor mFor;
+};
+
 namespace widget {
 
 /**
@@ -91,52 +256,43 @@ struct IMENotificationRequests final {
   Notifications mWantUpdates;
 };
 
+/**
+ * IME enabled states.
+ *
+ * WARNING: If you change these values, you also need to edit:
+ *   nsIDOMWindowUtils.idl
+ */
+enum class IMEEnabled {
+  /**
+   * 'Disabled' means the user cannot use IME. So, the IME open state should
+   * be 'closed' during 'disabled'.
+   */
+  Disabled,
+  /**
+   * 'Enabled' means the user can use IME.
+   */
+  Enabled,
+  /**
+   * 'Password' state is a special case for the password editors.
+   * E.g., on mac, the password editors should disable the non-Roman
+   * keyboard layouts at getting focus. Thus, the password editor may have
+   * special rules on some platforms.
+   */
+  Password,
+  /**
+   * 'Unknown' is useful when you cache this enum.  So, this shouldn't be
+   * used with nsIWidget::SetInputContext().
+   */
+  Unknown,
+};
+
 /**
  * Contains IMEStatus plus information about the current
  * input context that the IME can use as hints if desired.
  */
 
 struct IMEState final {
-  /**
-   * IME enabled states, the mEnabled value of
-   * SetInputContext()/GetInputContext() should be one value of following
-   * values.
-   *
-   * WARNING: If you change these values, you also need to edit:
-   *   nsIDOMWindowUtils.idl
-   *   nsContentUtils::GetWidgetStatusFromIMEStatus
-   */
-  enum Enabled {
-    /**
-     * 'Disabled' means the user cannot use IME. So, the IME open state should
-     * be 'closed' during 'disabled'.
-     */
-    DISABLED,
-    /**
-     * 'Enabled' means the user can use IME.
-     */
-    ENABLED,
-    /**
-     * 'Password' state is a special case for the password editors.
-     * E.g., on mac, the password editors should disable the non-Roman
-     * keyboard layouts at getting focus. Thus, the password editor may have
-     * special rules on some platforms.
-     */
-    PASSWORD,
-    /**
-     * This state is used when a plugin is focused.
-     * When a plug-in is focused content, we should send native events
-     * directly. Because we don't process some native events, but they may
-     * be needed by the plug-in.
-     */
-    PLUGIN,
-    /**
-     * 'Unknown' is useful when you cache this enum.  So, this shouldn't be
-     * used with nsIWidget::SetInputContext().
-     */
-    UNKNOWN
-  };
-  Enabled mEnabled;
+  IMEEnabled mEnabled;
 
   /**
    * IME open states the mOpen value of SetInputContext() should be one value of
@@ -171,22 +327,17 @@ struct IMEState final {
   };
   Open mOpen;
 
-  IMEState() : mEnabled(ENABLED), mOpen(DONT_CHANGE_OPEN_STATE) {}
+  IMEState() : mEnabled(IMEEnabled::Enabled), mOpen(DONT_CHANGE_OPEN_STATE) {}
 
-  explicit IMEState(Enabled aEnabled, Open aOpen = DONT_CHANGE_OPEN_STATE)
+  explicit IMEState(IMEEnabled aEnabled, Open aOpen = DONT_CHANGE_OPEN_STATE)
       : mEnabled(aEnabled), mOpen(aOpen) {}
 
   // Returns true if the user can input characters.
   // This means that a plain text editor, an HTML editor, a password editor or
   // a plain text editor whose ime-mode is "disabled".
   bool IsEditable() const {
-    return mEnabled == ENABLED || mEnabled == PASSWORD;
+    return mEnabled == IMEEnabled::Enabled || mEnabled == IMEEnabled::Password;
   }
-  // Returns true if the user might be able to input characters.
-  // This means that a plain text editor, an HTML editor, a password editor,
-  // a plain text editor whose ime-mode is "disabled" or a windowless plugin
-  // has focus.
-  bool MaybeEditable() const { return IsEditable() || mEnabled == PLUGIN; }
 };
 
 // NS_ONLY_ONE_NATIVE_IME_CONTEXT is a special value of native IME context.
@@ -245,12 +396,20 @@ struct InputContext final {
     mHTMLInputType.Truncate();
     mHTMLInputInputmode.Truncate();
     mActionHint.Truncate();
+    mAutocapitalize.Truncate();
   }
 
   bool IsPasswordEditor() const {
     return mHTMLInputType.LowerCaseEqualsLiteral("password");
   }
 
+  // https://html.spec.whatwg.org/dev/interaction.html#autocapitalization
+  bool IsAutocapitalizeSupported() const {
+    return !mHTMLInputType.EqualsLiteral("password") &&
+           !mHTMLInputType.EqualsLiteral("url") &&
+           !mHTMLInputType.EqualsLiteral("email");
+  }
+
   IMEState mIMEState;
 
   /* The type of the input if the input is a html input field */
@@ -262,6 +421,9 @@ struct InputContext final {
   /* A hint for the action that is performed when the input is submitted */
   nsString mActionHint;
 
+  /* A hint for autocapitalize */
+  nsString mAutocapitalize;
+
   /**
    * mOrigin indicates whether this focus event refers to main or remote
    * content.
@@ -321,6 +483,8 @@ struct InputContextAction final {
     CAUSE_MOUSE,
     // The cause is user's touch operation (implies mouse)
     CAUSE_TOUCH,
+    // The cause is users' long press operation.
+    CAUSE_LONGPRESS,
     // The cause is unknown but it occurs during user input except keyboard
     // input.  E.g., an event handler of a user input event moves focus.
     CAUSE_UNKNOWN_DURING_NON_KEYBOARD_INPUT,
@@ -388,6 +552,7 @@ struct InputContextAction final {
       case CAUSE_KEY:
       case CAUSE_MOUSE:
       case CAUSE_TOUCH:
+      case CAUSE_LONGPRESS:
       case CAUSE_UNKNOWN_DURING_NON_KEYBOARD_INPUT:
       case CAUSE_UNKNOWN_DURING_KEYBOARD_INPUT:
         return true;
@@ -472,8 +637,8 @@ struct IMENotification final {
       case NOTIFY_IME_OF_MOUSE_BUTTON_EVENT:
         mMouseButtonEventData.mEventMessage = eVoidEvent;
         mMouseButtonEventData.mOffset = UINT32_MAX;
-        mMouseButtonEventData.mCursorPos.Set(nsIntPoint(0, 0));
-        mMouseButtonEventData.mCharRect.Set(nsIntRect(0, 0, 0, 0));
+        mMouseButtonEventData.mCursorPos.MoveTo(0, 0);
+        mMouseButtonEventData.mCharRect.SetRect(0, 0, 0, 0);
         mMouseButtonEventData.mButton = -1;
         mMouseButtonEventData.mButtons = 0;
         mMouseButtonEventData.mModifiers = 0;
@@ -549,29 +714,6 @@ struct IMENotification final {
 
   IMEMessage mMessage;
 
-  struct Point {
-    int32_t mX;
-    int32_t mY;
-
-    void Set(const nsIntPoint& aPoint) {
-      mX = aPoint.x;
-      mY = aPoint.y;
-    }
-    nsIntPoint AsIntPoint() const { return nsIntPoint(mX, mY); }
-  };
-
-  struct Rect {
-    int32_t mX;
-    int32_t mY;
-    int32_t mWidth;
-    int32_t mHeight;
-
-    void Set(const nsIntRect& aRect) {
-      aRect.GetRect(&mX, &mY, &mWidth, &mHeight);
-    }
-    nsIntRect AsIntRect() const { return nsIntRect(mX, mY, mWidth, mHeight); }
-  };
-
   // NOTIFY_IME_OF_SELECTION_CHANGE specific data
   struct SelectionChangeDataBase {
     // Selection range.
@@ -758,9 +900,9 @@ struct IMENotification final {
     // Character offset from the start of the focused editor under the cursor
     uint32_t mOffset;
     // Cursor position in pixels relative to the widget
-    Point mCursorPos;
+    LayoutDeviceIntPoint mCursorPos;
     // Character rect in pixels under the cursor relative to the widget
-    Rect mCharRect;
+    LayoutDeviceIntRect mCharRect;
     // The value of WidgetMouseEventBase::button and buttons
     int16_t mButton;
     int16_t mButtons;
@@ -801,6 +943,21 @@ struct CandidateWindowPosition {
   bool mExcludeRect;
 };
 
+std::ostream& operator<<(std::ostream& aStream, const IMEEnabled& aEnabled);
+std::ostream& operator<<(std::ostream& aStream, const IMEState::Open& aOpen);
+std::ostream& operator<<(std::ostream& aStream, const IMEState& aState);
+std::ostream& operator<<(std::ostream& aStream,
+                         const InputContext::Origin& aOrigin);
+std::ostream& operator<<(std::ostream& aStream, const InputContext& aContext);
+std::ostream& operator<<(std::ostream& aStream,
+                         const InputContextAction::Cause& aCause);
+std::ostream& operator<<(std::ostream& aStream,
+                         const InputContextAction::FocusChange& aFocusChange);
+std::ostream& operator<<(std::ostream& aStream,
+                         const IMENotification::SelectionChangeDataBase& aData);
+std::ostream& operator<<(std::ostream& aStream,
+                         const IMENotification::TextChangeDataBase& aData);
+
 }  // namespace widget
 }  // namespace mozilla
 
