@@ -22,13 +22,16 @@ class PBrowserBridgeParent;
 }  // namespace dom
 
 class WidgetPointerEvent;
+}  // namespace mozilla
+
+namespace mozilla {
 class WidgetPointerEventHolder final {
  public:
   nsTArray<WidgetPointerEvent> mEvents;
   NS_INLINE_DECL_REFCOUNTING(WidgetPointerEventHolder)
 
  private:
-  virtual ~WidgetPointerEventHolder() {}
+  virtual ~WidgetPointerEventHolder() = default;
 };
 
 /******************************************************************************
@@ -62,14 +65,7 @@ class WidgetPointerHelper {
         tangentialPressure(aTangentialPressure),
         convertToPointer(true) {}
 
-  explicit WidgetPointerHelper(const WidgetPointerHelper& aHelper)
-      : pointerId(aHelper.pointerId),
-        tiltX(aHelper.tiltX),
-        tiltY(aHelper.tiltY),
-        twist(aHelper.twist),
-        tangentialPressure(aHelper.tangentialPressure),
-        convertToPointer(aHelper.convertToPointer),
-        mCoalescedWidgetEvents(aHelper.mCoalescedWidgetEvents) {}
+  explicit WidgetPointerHelper(const WidgetPointerHelper& aHelper) = default;
 
   void AssignPointerHelperData(const WidgetPointerHelper& aEvent,
                                bool aCopyCoalescedEvents = false) {
@@ -100,8 +96,7 @@ class WidgetMouseEventBase : public WidgetInputEvent {
       : mPressure(0),
         mButton(0),
         mButtons(0),
-        mInputSource(/* MouseEvent_Binding::MOZ_SOURCE_MOUSE = */ 1),
-        mHitCluster(false) {}
+        mInputSource(/* MouseEvent_Binding::MOZ_SOURCE_MOUSE = */ 1) {}
   // Including MouseEventBinding.h here leads to an include loop, so
   // we have to hardcode MouseEvent_Binding::MOZ_SOURCE_MOUSE.
 
@@ -111,8 +106,7 @@ class WidgetMouseEventBase : public WidgetInputEvent {
         mPressure(0),
         mButton(0),
         mButtons(0),
-        mInputSource(/* MouseEvent_Binding::MOZ_SOURCE_MOUSE = */ 1),
-        mHitCluster(false) {}
+        mInputSource(/* MouseEvent_Binding::MOZ_SOURCE_MOUSE = */ 1) {}
   // Including MouseEventBinding.h here leads to an include loop, so
   // we have to hardcode MouseEvent_Binding::MOZ_SOURCE_MOUSE.
 
@@ -140,14 +134,11 @@ class WidgetMouseEventBase : public WidgetInputEvent {
   // Possible values a in MouseEvent
   uint16_t mInputSource;
 
-  // Touch near a cluster of links (true)
-  bool mHitCluster;
-
   bool IsLeftButtonPressed() const {
-    return !!(mButtons & MouseButtonsFlag::eLeftFlag);
+    return !!(mButtons & MouseButtonsFlag::ePrimaryFlag);
   }
   bool IsRightButtonPressed() const {
-    return !!(mButtons & MouseButtonsFlag::eRightFlag);
+    return !!(mButtons & MouseButtonsFlag::eSecondaryFlag);
   }
   bool IsMiddleButtonPressed() const {
     return !!(mButtons & MouseButtonsFlag::eMiddleFlag);
@@ -166,7 +157,6 @@ class WidgetMouseEventBase : public WidgetInputEvent {
     mButton = aEvent.mButton;
     mButtons = aEvent.mButtons;
     mPressure = aEvent.mPressure;
-    mHitCluster = aEvent.mHitCluster;
     mInputSource = aEvent.mInputSource;
   }
 
@@ -174,7 +164,7 @@ class WidgetMouseEventBase : public WidgetInputEvent {
    * Returns true if left click event.
    */
   bool IsLeftClickEvent() const {
-    return mMessage == eMouseClick && mButton == MouseButton::eLeft;
+    return mMessage == eMouseClick && mButton == MouseButton::ePrimary;
   }
 };
 
@@ -193,30 +183,43 @@ class WidgetMouseEvent : public WidgetMouseEventBase,
   typedef bool ReasonType;
   enum Reason : ReasonType { eReal, eSynthesized };
 
-  typedef bool ContextMenuTriggerType;
-  enum ContextMenuTrigger : ContextMenuTriggerType { eNormal, eContextMenuKey };
+  typedef uint8_t ContextMenuTriggerType;
+  enum ContextMenuTrigger : ContextMenuTriggerType {
+    eNormal,
+    eContextMenuKey,
+    eControlClick
+  };
 
-  typedef bool ExitFromType;
-  enum ExitFrom : ExitFromType { eChild, eTopLevel };
+  typedef uint8_t ExitFromType;
+  enum ExitFrom : ExitFromType {
+    ePlatformChild,
+    ePlatformTopLevel,
+    ePuppet,
+    ePuppetParentToPuppetChild
+  };
 
  protected:
   WidgetMouseEvent()
       : mReason(eReal),
         mContextMenuTrigger(eNormal),
-        mExitFrom(eChild),
-        mIgnoreRootScrollFrame(false),
         mClickCount(0),
-        mUseLegacyNonPrimaryDispatch(false) {}
+        mIgnoreRootScrollFrame(false),
+        mUseLegacyNonPrimaryDispatch(false),
+        mClickEventPrevented(false) {}
 
   WidgetMouseEvent(bool aIsTrusted, EventMessage aMessage, nsIWidget* aWidget,
                    EventClassID aEventClassID, Reason aReason)
       : WidgetMouseEventBase(aIsTrusted, aMessage, aWidget, aEventClassID),
         mReason(aReason),
         mContextMenuTrigger(eNormal),
-        mExitFrom(eChild),
-        mIgnoreRootScrollFrame(false),
         mClickCount(0),
-        mUseLegacyNonPrimaryDispatch(false) {}
+        mIgnoreRootScrollFrame(false),
+        mUseLegacyNonPrimaryDispatch(false),
+        mClickEventPrevented(false) {}
+
+#ifdef DEBUG
+  void AssertContextMenuEventButtonConsistency() const;
+#endif
 
  public:
   virtual WidgetMouseEvent* AsMouseEvent() override { return this; }
@@ -227,24 +230,18 @@ class WidgetMouseEvent : public WidgetMouseEventBase,
       : WidgetMouseEventBase(aIsTrusted, aMessage, aWidget, eMouseEventClass),
         mReason(aReason),
         mContextMenuTrigger(aContextMenuTrigger),
-        mExitFrom(eChild),
-        mIgnoreRootScrollFrame(false),
         mClickCount(0),
-        mUseLegacyNonPrimaryDispatch(false) {
+        mIgnoreRootScrollFrame(false),
+        mUseLegacyNonPrimaryDispatch(false),
+        mClickEventPrevented(false) {
     if (aMessage == eContextMenu) {
-      mButton = (mContextMenuTrigger == eNormal) ? MouseButton::eRight
-                                                 : MouseButton::eLeft;
+      mButton = (mContextMenuTrigger == eNormal) ? MouseButton::eSecondary
+                                                 : MouseButton::ePrimary;
     }
   }
 
 #ifdef DEBUG
-  virtual ~WidgetMouseEvent() {
-    NS_WARNING_ASSERTION(
-        mMessage != eContextMenu ||
-            mButton == ((mContextMenuTrigger == eNormal) ? MouseButton::eRight
-                                                         : MouseButton::eLeft),
-        "Wrong button set to eContextMenu event?");
-  }
+  virtual ~WidgetMouseEvent() { AssertContextMenuEventButtonConsistency(); }
 #endif
 
   virtual WidgetEvent* Duplicate() const override {
@@ -273,30 +270,35 @@ class WidgetMouseEvent : public WidgetMouseEventBase,
   // other reasons (typically, a click of right mouse button).
   ContextMenuTrigger mContextMenuTrigger;
 
-  // mExitFrom is valid only when mMessage is eMouseExitFromWidget.
-  // This indicates if the mouse cursor exits from a top level widget or
-  // a child widget.
-  ExitFrom mExitFrom;
-
-  // Whether the event should ignore scroll frame bounds during dispatch.
-  bool mIgnoreRootScrollFrame;
+  // mExitFrom contains a value only when mMessage is eMouseExitFromWidget.
+  // This indicates if the mouse cursor exits from a top level platform widget,
+  // a child widget or a puppet widget.
+  Maybe<ExitFrom> mExitFrom;
 
   // mClickCount may be non-zero value when mMessage is eMouseDown, eMouseUp,
   // eMouseClick or eMouseDoubleClick. The number is count of mouse clicks.
   // Otherwise, this must be 0.
   uint32_t mClickCount;
 
+  // Whether the event should ignore scroll frame bounds during dispatch.
+  bool mIgnoreRootScrollFrame;
+
   // Indicates whether the event should dispatch click events for non-primary
   // mouse buttons on window and document.
   bool mUseLegacyNonPrimaryDispatch;
 
+  // Whether the event shouldn't cause click event.
+  bool mClickEventPrevented;
+
   void AssignMouseEventData(const WidgetMouseEvent& aEvent, bool aCopyTargets) {
     AssignMouseEventBaseData(aEvent, aCopyTargets);
     AssignPointerHelperData(aEvent, /* aCopyCoalescedEvents */ true);
 
-    mIgnoreRootScrollFrame = aEvent.mIgnoreRootScrollFrame;
+    mExitFrom = aEvent.mExitFrom;
     mClickCount = aEvent.mClickCount;
+    mIgnoreRootScrollFrame = aEvent.mIgnoreRootScrollFrame;
     mUseLegacyNonPrimaryDispatch = aEvent.mUseLegacyNonPrimaryDispatch;
+    mClickEventPrevented = aEvent.mClickEventPrevented;
   }
 
   /**
@@ -366,6 +368,14 @@ class WidgetDragEvent : public WidgetMouseEvent {
     mUserCancelled = false;
     mDefaultPreventedOnContent = aEvent.mDefaultPreventedOnContent;
   }
+
+  /**
+   * Should be called before dispatching the DOM tree if this event is
+   * synthesized for tests because drop effect is initialized before
+   * dispatching from widget if it's not synthesized event, but synthesized
+   * events are not initialized in the path.
+   */
+  void InitDropEffectForTests();
 };
 
 /******************************************************************************
@@ -506,6 +516,33 @@ class WidgetWheelEvent : public WidgetMouseEventBase {
   double mDeltaY;
   double mDeltaZ;
 
+  // The mousewheel tick counts.
+  double mWheelTicksX = 0.0;
+  double mWheelTicksY = 0.0;
+
+  enum class DeltaModeCheckingState : uint8_t {
+    // Neither deltaMode nor the delta values have been accessed.
+    Unknown,
+    // The delta values have been accessed, without checking deltaMode first.
+    Unchecked,
+    // The deltaMode has been checked.
+    Checked,
+  };
+
+  // For compat reasons, we might expose a DOM_DELTA_LINE event as
+  // DOM_DELTA_PIXEL instead. Whether we do that depends on whether the event
+  // has been asked for the deltaMode before the deltas. If it has, we assume
+  // that the page will correctly handle DOM_DELTA_LINE. This variable tracks
+  // that state. See bug 1392460.
+  DeltaModeCheckingState mDeltaModeCheckingState =
+      DeltaModeCheckingState::Unknown;
+
+  // The amount of scrolling per line or page, without accounting for mouse
+  // wheel transactions etc.
+  //
+  // Computed by EventStateManager::DeltaAccumulator::InitLineOrPageDelta.
+  nsSize mScrollAmount;
+
   // overflowed delta values for scroll, these values are set by
   // EventStateManger.  If the default action of the wheel event isn't scroll,
   // these values are always zero.  Otherwise, remaining delta values which are
@@ -602,6 +639,7 @@ class WidgetWheelEvent : public WidgetMouseEventBase {
     mDeltaY = aEvent.mDeltaY;
     mDeltaZ = aEvent.mDeltaZ;
     mDeltaMode = aEvent.mDeltaMode;
+    mScrollAmount = aEvent.mScrollAmount;
     mCustomizedByUserPrefs = aEvent.mCustomizedByUserPrefs;
     mMayHaveMomentum = aEvent.mMayHaveMomentum;
     mIsMomentum = aEvent.mIsMomentum;
@@ -649,7 +687,8 @@ class WidgetPointerEvent : public WidgetMouseEvent {
   friend class mozilla::dom::PBrowserParent;
   friend class mozilla::dom::PBrowserChild;
 
-  WidgetPointerEvent() : mWidth(1), mHeight(1), mIsPrimary(true) {}
+  WidgetPointerEvent()
+      : mWidth(1), mHeight(1), mIsPrimary(true), mFromTouchEvent(false) {}
 
  public:
   virtual WidgetPointerEvent* AsPointerEvent() override { return this; }
@@ -658,10 +697,15 @@ class WidgetPointerEvent : public WidgetMouseEvent {
       : WidgetMouseEvent(aIsTrusted, aMsg, w, ePointerEventClass, eReal),
         mWidth(1),
         mHeight(1),
-        mIsPrimary(true) {}
+        mIsPrimary(true),
+        mFromTouchEvent(false) {}
 
   explicit WidgetPointerEvent(const WidgetMouseEvent& aEvent)
-      : WidgetMouseEvent(aEvent), mWidth(1), mHeight(1), mIsPrimary(true) {
+      : WidgetMouseEvent(aEvent),
+        mWidth(1),
+        mHeight(1),
+        mIsPrimary(true),
+        mFromTouchEvent(false) {
     mClass = ePointerEventClass;
   }
 
@@ -679,6 +723,7 @@ class WidgetPointerEvent : public WidgetMouseEvent {
   uint32_t mWidth;
   uint32_t mHeight;
   bool mIsPrimary;
+  bool mFromTouchEvent;
 
   // XXX Not tested by test_assign_event_data.html
   void AssignPointerEventData(const WidgetPointerEvent& aEvent,
@@ -688,6 +733,7 @@ class WidgetPointerEvent : public WidgetMouseEvent {
     mWidth = aEvent.mWidth;
     mHeight = aEvent.mHeight;
     mIsPrimary = aEvent.mIsPrimary;
+    mFromTouchEvent = aEvent.mFromTouchEvent;
   }
 };
 
