@@ -7,7 +7,8 @@
 #define nsDeviceContextSpecX_h_
 
 #include "nsIDeviceContextSpec.h"
-#include "nsIPrinterEnumerator.h"
+#include "nsIPrinter.h"
+#include "nsIPrinterList.h"
 
 #include "nsCOMPtr.h"
 
@@ -46,17 +47,4 @@ class nsDeviceContextSpecX : public nsIDeviceContextSpec {
 #endif
 };
 
-//----------------------------------------------------------------------
-// nsPrinterErnumeratorX
-
-class nsPrinterEnumeratorX final : public nsIPrinterEnumerator {
- public:
-  NS_DECL_ISUPPORTS
-  NS_DECL_NSIPRINTERENUMERATOR
-  nsPrinterEnumeratorX() = default;
-
- private:
-  ~nsPrinterEnumeratorX() = default;
-};
-
 #endif  // nsDeviceContextSpecX_h_
