@@ -19,13 +19,15 @@
 #include "nsString.h"
 #include "nsReadableUtils.h"
 #include "nsPrimitiveHelpers.h"
-#include "nsIServiceManager.h"
 #include "nsImageToPixbuf.h"
 #include "nsStringStream.h"
+#include "nsIFileURL.h"
 #include "nsIObserverService.h"
 #include "mozilla/Services.h"
 #include "mozilla/RefPtr.h"
+#include "mozilla/SchedulerGroup.h"
 #include "mozilla/TimeStamp.h"
+#include "WidgetUtilsGtk.h"
 
 #include "imgIContainer.h"
 
@@ -44,6 +46,8 @@ const int kClipboardTimeout = 500000;
 static const char kHTMLMarkupPrefix[] =
     R"(<meta http-equiv="content-type" content="text/html; charset=utf-8">)";
 
+static const char kURIListMime[] = "text/uri-list";
+
 // Callback when someone asks us for the data
 void clipboard_get_cb(GtkClipboard* aGtkClipboard,
                       GtkSelectionData* aSelectionData, guint info,
@@ -52,10 +56,11 @@ void clipboard_get_cb(GtkClipboard* aGtkClipboard,
 // Callback when someone asks us to clear a clipboard
 void clipboard_clear_cb(GtkClipboard* aGtkClipboard, gpointer user_data);
 
-static void ConvertHTMLtoUCS2(const char* data, int32_t dataLength,
-                              char16_t** unicodeData, int32_t& outUnicodeLen);
+static bool ConvertHTMLtoUCS2(const char* data, int32_t dataLength,
+                              nsCString& charset, char16_t** unicodeData,
+                              int32_t& outUnicodeLen);
 
-static void GetHTMLCharset(const char* data, int32_t dataLength,
+static bool GetHTMLCharset(const char* data, int32_t dataLength,
                            nsCString& str);
 
 GdkAtom GetSelectionAtom(int32_t aWhichClipboard) {
@@ -65,7 +70,16 @@ GdkAtom GetSelectionAtom(int32_t aWhichClipboard) {
   return GDK_SELECTION_PRIMARY;
 }
 
-nsClipboard::nsClipboard() {}
+int GetGeckoClipboardType(GtkClipboard* aGtkClipboard) {
+  if (aGtkClipboard == gtk_clipboard_get(GDK_SELECTION_PRIMARY))
+    return nsClipboard::kSelectionClipboard;
+  else if (aGtkClipboard == gtk_clipboard_get(GDK_SELECTION_CLIPBOARD))
+    return nsClipboard::kGlobalClipboard;
+
+  return -1;  // THAT AIN'T NO CLIPBOARD I EVER HEARD OF
+}
+
+nsClipboard::nsClipboard() = default;
 
 nsClipboard::~nsClipboard() {
   // We have to clear clipboard before gdk_display_close() call.
@@ -81,41 +95,37 @@ nsClipboard::~nsClipboard() {
 NS_IMPL_ISUPPORTS(nsClipboard, nsIClipboard, nsIObserver)
 
 nsresult nsClipboard::Init(void) {
-  GdkDisplay* display = gdk_display_get_default();
-
-  // Create a nsRetrievalContext. If there's no default display
-  // create the X11 one as a fallback.
-  if (!display || GDK_IS_X11_DISPLAY(display)) {
-    mContext = new nsRetrievalContextX11();
+  if (widget::GdkIsX11Display()) {
+    mContext = MakeUnique<nsRetrievalContextX11>();
 #if defined(MOZ_WAYLAND)
-  } else {
-    mContext = new nsRetrievalContextWayland();
+  } else if (widget::GdkIsWaylandDisplay()) {
+    mContext = MakeUnique<nsRetrievalContextWayland>();
 #endif
+  } else {
+    NS_WARNING("Missing nsRetrievalContext for nsClipboard!");
+    return NS_OK;
   }
-  NS_ASSERTION(mContext, "Missing nsRetrievalContext for nsClipboard!");
 
   nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
   if (os) {
-    os->AddObserver(this, "quit-application", false);
     os->AddObserver(this, "xpcom-shutdown", false);
   }
 
   return NS_OK;
 }
 
-nsresult nsClipboard::Store(void) {
-  if (mGlobalTransferable) {
-    GtkClipboard* clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
-    gtk_clipboard_store(clipboard);
-  }
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsClipboard::Observe(nsISupports* aSubject, const char* aTopic,
                      const char16_t* aData) {
-  Store();
-  return NS_OK;
+  // Save global clipboard content to CLIPBOARD_MANAGER.
+  // gtk_clipboard_store() can run an event loop, so call from a dedicated
+  // runnable.
+  return SchedulerGroup::Dispatch(
+      TaskCategory::Other,
+      NS_NewRunnableFunction("gtk_clipboard_store()", []() {
+        LOGCLIP(("nsClipboard storing clipboard content\n"));
+        gtk_clipboard_store(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD));
+      }));
 }
 
 NS_IMETHODIMP
@@ -131,8 +141,8 @@ nsClipboard::SetData(nsITransferable* aTransferable, nsIClipboardOwner* aOwner,
     return NS_OK;
   }
 
-  // Clear out the clipboard in order to set the new data
-  EmptyClipboard(aWhichClipboard);
+  LOGCLIP(("nsClipboard::SetData (%s)\n",
+           aWhichClipboard == kSelectionClipboard ? "primary" : "clipboard"));
 
   // List of suported targets
   GtkTargetList* list = gtk_target_list_new(nullptr, 0);
@@ -141,16 +151,19 @@ nsClipboard::SetData(nsITransferable* aTransferable, nsIClipboardOwner* aOwner,
   nsTArray<nsCString> flavors;
   nsresult rv = aTransferable->FlavorsTransferableCanExport(flavors);
   if (NS_FAILED(rv)) {
-    return rv;
+    LOGCLIP(("    FlavorsTransferableCanExport failed!\n"));
+    // Fall through.  |gtkTargets| will be null below.
   }
 
   // Add all the flavors to this widget's supported type.
   bool imagesAdded = false;
   for (uint32_t i = 0; i < flavors.Length(); i++) {
     nsCString& flavorStr = flavors[i];
+    LOGCLIP(("    processing target %s\n", flavorStr.get()));
 
     // Special case text/unicode since we can handle all of the string types.
     if (flavorStr.EqualsLiteral(kUnicodeMime)) {
+      LOGCLIP(("    adding TEXT targets\n"));
       gtk_target_list_add_text_targets(list, 0);
       continue;
     }
@@ -159,6 +172,7 @@ nsClipboard::SetData(nsITransferable* aTransferable, nsIClipboardOwner* aOwner,
       // Don't bother adding image targets twice
       if (!imagesAdded) {
         // accept any writable image type
+        LOGCLIP(("    adding IMAGE targets\n"));
         gtk_target_list_add_image_targets(list, 0, TRUE);
         imagesAdded = true;
       }
@@ -166,6 +180,7 @@ nsClipboard::SetData(nsITransferable* aTransferable, nsIClipboardOwner* aOwner,
     }
 
     // Add this to our list of valid targets
+    LOGCLIP(("    adding OTHER target %s\n", flavorStr.get()));
     GdkAtom atom = gdk_atom_intern(flavorStr.get(), FALSE);
     gtk_target_list_add(list, atom, 0, 0);
   }
@@ -177,10 +192,16 @@ nsClipboard::SetData(nsITransferable* aTransferable, nsIClipboardOwner* aOwner,
   gint numTargets;
   GtkTargetEntry* gtkTargets =
       gtk_target_table_new_from_list(list, &numTargets);
+  if (!gtkTargets) {
+    LOGCLIP(("    gtk_clipboard_set_with_data() failed!\n"));
+    // Clear references to the any old data and let GTK know that it is no
+    // longer available.
+    EmptyClipboard(aWhichClipboard);
+    return NS_ERROR_FAILURE;
+  }
 
   // Set getcallback and request to store data after an application exit
-  if (gtkTargets &&
-      gtk_clipboard_set_with_data(gtkClipboard, gtkTargets, numTargets,
+  if (gtk_clipboard_set_with_data(gtkClipboard, gtkTargets, numTargets,
                                   clipboard_get_cb, clipboard_clear_cb, this)) {
     // We managed to set-up the clipboard so update internal state
     // We have to set it now because gtk_clipboard_set_with_data() calls
@@ -196,6 +217,8 @@ nsClipboard::SetData(nsITransferable* aTransferable, nsIClipboardOwner* aOwner,
 
     rv = NS_OK;
   } else {
+    LOGCLIP(("    gtk_clipboard_set_with_data() failed!\n"));
+    EmptyClipboard(aWhichClipboard);
     rv = NS_ERROR_FAILURE;
   }
 
@@ -209,6 +232,8 @@ void nsClipboard::SetTransferableData(nsITransferable* aTransferable,
                                       nsCString& aFlavor,
                                       const char* aClipboardData,
                                       uint32_t aClipboardDataLength) {
+  LOGCLIP(("nsClipboard::SetTransferableData MIME %s\n", aFlavor.get()));
+
   nsCOMPtr<nsISupports> wrapper;
   nsPrimitiveHelpers::CreatePrimitiveForData(
       aFlavor, aClipboardData, aClipboardDataLength, getter_AddRefs(wrapper));
@@ -217,15 +242,28 @@ void nsClipboard::SetTransferableData(nsITransferable* aTransferable,
 
 NS_IMETHODIMP
 nsClipboard::GetData(nsITransferable* aTransferable, int32_t aWhichClipboard) {
-  if (!aTransferable) return NS_ERROR_FAILURE;
+  LOGCLIP(("nsClipboard::GetData (%s)\n",
+           aWhichClipboard == kSelectionClipboard ? "primary" : "clipboard"));
+
+  if (!aTransferable || !mContext) {
+    return NS_ERROR_FAILURE;
+  }
 
   // Get a list of flavors this transferable can import
   nsTArray<nsCString> flavors;
   nsresult rv = aTransferable->FlavorsTransferableCanImport(flavors);
   if (NS_FAILED(rv)) {
+    LOGCLIP(("    FlavorsTransferableCanImport falied!\n"));
     return rv;
   }
 
+#ifdef MOZ_LOGGING
+  LOGCLIP(("Flavors which can be imported:\n"));
+  for (uint32_t i = 0; i < flavors.Length(); i++) {
+    LOGCLIP(("    %s\n", flavors[i].get()));
+  }
+#endif
+
   for (uint32_t i = 0; i < flavors.Length(); i++) {
     nsCString& flavorStr = flavors[i];
 
@@ -238,16 +276,22 @@ nsClipboard::GetData(nsITransferable* aTransferable, int32_t aWhichClipboard) {
         flavorStr.Assign(kJPEGImageMime);
       }
 
+      LOGCLIP(("    Getting image %s MIME clipboard data\n", flavorStr.get()));
+
       uint32_t clipboardDataLength;
       const char* clipboardData = mContext->GetClipboardData(
           flavorStr.get(), aWhichClipboard, &clipboardDataLength);
-      if (!clipboardData) continue;
+      if (!clipboardData) {
+        LOGCLIP(("    %s type is missing\n", flavorStr.get()));
+        continue;
+      }
 
       nsCOMPtr<nsIInputStream> byteStream;
       NS_NewByteInputStream(getter_AddRefs(byteStream),
-                            MakeSpan(clipboardData, clipboardDataLength),
+                            Span(clipboardData, clipboardDataLength),
                             NS_ASSIGNMENT_COPY);
       aTransferable->SetTransferData(flavorStr.get(), byteStream);
+      LOGCLIP(("    got %s MIME data\n", flavorStr.get()));
 
       mContext->ReleaseClipboardData(clipboardData);
       return NS_OK;
@@ -256,8 +300,12 @@ nsClipboard::GetData(nsITransferable* aTransferable, int32_t aWhichClipboard) {
     // Special case text/unicode since we can convert any
     // string into text/unicode
     if (flavorStr.EqualsLiteral(kUnicodeMime)) {
+      LOGCLIP(
+          ("    Getting unicode %s MIME clipboard data\n", flavorStr.get()));
+
       const char* clipboardData = mContext->GetClipboardText(aWhichClipboard);
       if (!clipboardData) {
+        LOGCLIP(("    failed to get unicode data\n"));
         // If the type was text/unicode and we couldn't get
         // text off the clipboard, run the next loop
         // iteration.
@@ -272,25 +320,71 @@ nsClipboard::GetData(nsITransferable* aTransferable, int32_t aWhichClipboard) {
                           unicodeDataLength);
       free((void*)unicodeData);
 
+      LOGCLIP(("    got unicode data, length %d\n", ucs2string.Length()));
+
       mContext->ReleaseClipboardData(clipboardData);
       return NS_OK;
     }
 
+    if (flavorStr.EqualsLiteral(kFileMime)) {
+      LOGCLIP(("    Getting %s file clipboard data\n", flavorStr.get()));
+
+      uint32_t clipboardDataLength;
+      const char* clipboardData = mContext->GetClipboardData(
+          kURIListMime, aWhichClipboard, &clipboardDataLength);
+      if (!clipboardData) {
+        LOGCLIP(("    text/uri-list type is missing\n"));
+        continue;
+      }
+
+      nsDependentCSubstring data(clipboardData, clipboardDataLength);
+      nsTArray<nsCString> uris = mozilla::widget::ParseTextURIList(data);
+      if (!uris.IsEmpty()) {
+        nsCOMPtr<nsIURI> fileURI;
+        NS_NewURI(getter_AddRefs(fileURI), uris[0]);
+        if (nsCOMPtr<nsIFileURL> fileURL = do_QueryInterface(fileURI, &rv)) {
+          nsCOMPtr<nsIFile> file;
+          rv = fileURL->GetFile(getter_AddRefs(file));
+          if (NS_SUCCEEDED(rv)) {
+            aTransferable->SetTransferData(flavorStr.get(), file);
+            LOGCLIP(("    successfully set file to clipboard\n"));
+          }
+        }
+      }
+
+      mContext->ReleaseClipboardData(clipboardData);
+      return NS_OK;
+    }
+
+    LOGCLIP(("    Getting %s MIME clipboard data\n", flavorStr.get()));
+
     uint32_t clipboardDataLength;
     const char* clipboardData = mContext->GetClipboardData(
         flavorStr.get(), aWhichClipboard, &clipboardDataLength);
 
+#ifdef MOZ_LOGGING
+    if (!clipboardData) {
+      LOGCLIP(("    %s type is missing\n", flavorStr.get()));
+    }
+#endif
+
     if (clipboardData) {
+      LOGCLIP(("    got %s mime type data.\n", flavorStr.get()));
+
       // Special case text/html since we can convert into UCS2
       if (flavorStr.EqualsLiteral(kHTMLMime)) {
         char16_t* htmlBody = nullptr;
         int32_t htmlBodyLen = 0;
         // Convert text/html into our unicode format
-        ConvertHTMLtoUCS2(clipboardData, clipboardDataLength, &htmlBody,
-                          htmlBodyLen);
-
-        // Try next data format?
-        if (!htmlBodyLen) {
+        nsAutoCString charset;
+        if (!GetHTMLCharset(clipboardData, clipboardDataLength, charset)) {
+          // Fall back to utf-8 in case html/data is missing kHTMLMarkupPrefix.
+          LOGCLIP(("Failed to get html/text encoding, fall back to utf-8.\n"));
+          charset.AssignLiteral("utf-8");
+        }
+        if (!ConvertHTMLtoUCS2(clipboardData, clipboardDataLength, charset,
+                               &htmlBody, htmlBodyLen)) {
+          LOGCLIP(("    failed to convert text/html to UCS2.\n"));
           mContext->ReleaseClipboardData(clipboardData);
           continue;
         }
@@ -308,11 +402,30 @@ nsClipboard::GetData(nsITransferable* aTransferable, int32_t aWhichClipboard) {
     }
   }
 
+  LOGCLIP(("    failed to get clipboard content.\n"));
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsClipboard::EmptyClipboard(int32_t aWhichClipboard) {
+  LOGCLIP(("nsClipboard::EmptyClipboard (%s)\n",
+           aWhichClipboard == kSelectionClipboard ? "primary" : "clipboard"));
+  if (aWhichClipboard == kSelectionClipboard) {
+    if (mSelectionTransferable) {
+      gtk_clipboard_clear(gtk_clipboard_get(GDK_SELECTION_PRIMARY));
+      MOZ_ASSERT(!mSelectionTransferable);
+    }
+  } else {
+    if (mGlobalTransferable) {
+      gtk_clipboard_clear(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD));
+      MOZ_ASSERT(!mGlobalTransferable);
+    }
+  }
+
+  return NS_OK;
+}
+
+void nsClipboard::ClearTransferable(int32_t aWhichClipboard) {
   if (aWhichClipboard == kSelectionClipboard) {
     if (mSelectionOwner) {
       mSelectionOwner->LosingOwnership(mSelectionTransferable);
@@ -326,28 +439,55 @@ nsClipboard::EmptyClipboard(int32_t aWhichClipboard) {
     }
     mGlobalTransferable = nullptr;
   }
-
-  return NS_OK;
 }
 
 NS_IMETHODIMP
-nsClipboard::HasDataMatchingFlavors(const char** aFlavorList, uint32_t aLength,
+nsClipboard::HasDataMatchingFlavors(const nsTArray<nsCString>& aFlavorList,
                                     int32_t aWhichClipboard, bool* _retval) {
-  if (!aFlavorList || !_retval) return NS_ERROR_NULL_POINTER;
+  if (!_retval) {
+    return NS_ERROR_NULL_POINTER;
+  }
+
+  LOGCLIP(("nsClipboard::HasDataMatchingFlavors (%s)\n",
+           aWhichClipboard == kSelectionClipboard ? "primary" : "clipboard"));
 
   *_retval = false;
 
+  if (!mContext) {
+    return NS_ERROR_FAILURE;
+  }
+
   int targetNums;
   GdkAtom* targets = mContext->GetTargets(aWhichClipboard, &targetNums);
-  if (!targets) return NS_OK;
+  if (!targets) {
+    LOGCLIP(("    no targes at clipboard (null)\n"));
+    return NS_OK;
+  }
+
+#ifdef MOZ_LOGGING
+  LOGCLIP(("    Clipboard content (target nums %d):\n", targetNums));
+  for (int32_t j = 0; j < targetNums; j++) {
+    gchar* atom_name = gdk_atom_name(targets[j]);
+    if (!atom_name) {
+      LOGCLIP(("        failed to get MIME\n"));
+      continue;
+    }
+    LOGCLIP(("        MIME %s\n", atom_name));
+  }
+  LOGCLIP(("    Asking for content:\n"));
+  for (auto& flavor : aFlavorList) {
+    LOGCLIP(("        MIME %s\n", flavor.get()));
+  }
+#endif
 
   // Walk through the provided types and try to match it to a
   // provided type.
-  for (uint32_t i = 0; i < aLength && !*_retval; i++) {
+  for (auto& flavor : aFlavorList) {
     // We special case text/unicode here.
-    if (!strcmp(aFlavorList[i], kUnicodeMime) &&
+    if (flavor.EqualsLiteral(kUnicodeMime) &&
         gtk_targets_include_text(targets, targetNums)) {
       *_retval = true;
+      LOGCLIP(("    has kUnicodeMime\n"));
       break;
     }
 
@@ -355,13 +495,22 @@ nsClipboard::HasDataMatchingFlavors(const char** aFlavorList, uint32_t aLength,
       gchar* atom_name = gdk_atom_name(targets[j]);
       if (!atom_name) continue;
 
-      if (!strcmp(atom_name, aFlavorList[i])) *_retval = true;
-
+      if (flavor.Equals(atom_name)) {
+        *_retval = true;
+        LOGCLIP(("    has %s\n", atom_name));
+      }
       // X clipboard supports image/jpeg, but we want to emulate support
       // for image/jpg as well
-      if (!strcmp(aFlavorList[i], kJPGImageMime) &&
-          !strcmp(atom_name, kJPEGImageMime)) {
+      else if (flavor.EqualsLiteral(kJPGImageMime) &&
+               !strcmp(atom_name, kJPEGImageMime)) {
         *_retval = true;
+        LOGCLIP(("    has image/jpg\n"));
+      }
+      // application/x-moz-file should be treated like text/uri-list
+      else if (flavor.EqualsLiteral(kFileMime) &&
+               !strcmp(atom_name, kURIListMime)) {
+        *_retval = true;
+        LOGCLIP(("    has text/uri-list treating as application/x-moz-file"));
       }
 
       g_free(atom_name);
@@ -370,13 +519,19 @@ nsClipboard::HasDataMatchingFlavors(const char** aFlavorList, uint32_t aLength,
     }
   }
 
+#ifdef MOZ_LOGGING
+  if (!(*_retval)) {
+    LOGCLIP(("    no targes at clipboard (bad match)\n"));
+  }
+#endif
+
   g_free(targets);
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsClipboard::SupportsSelectionClipboard(bool* _retval) {
-  *_retval = mContext->HasSelectionSupport();
+  *_retval = mContext ? mContext->HasSelectionSupport() : false;
   return NS_OK;
 }
 
@@ -415,13 +570,14 @@ void nsClipboard::SelectionGetEvent(GtkClipboard* aClipboard,
   else
     return;  // THAT AIN'T NO CLIPBOARD I EVER HEARD OF
 
+  LOGCLIP(("nsClipboard::SelectionGetEvent (%s)\n",
+           whichClipboard == kSelectionClipboard ? "primary" : "clipboard"));
+
   nsCOMPtr<nsITransferable> trans = GetTransferable(whichClipboard);
   if (!trans) {
     // We have nothing to serve
-#ifdef DEBUG_CLIPBOARD
-    printf("nsClipboard::SelectionGetEvent() - %s clipboard is empty!\n",
-           whichClipboard == kSelectionClipboard ? "Selection" : "Global");
-#endif
+    LOGCLIP(("nsClipboard::SelectionGetEvent() - %s clipboard is empty!\n",
+             whichClipboard == kSelectionClipboard ? "Primary" : "Clipboard"));
     return;
   }
 
@@ -429,14 +585,17 @@ void nsClipboard::SelectionGetEvent(GtkClipboard* aClipboard,
   nsCOMPtr<nsISupports> item;
 
   GdkAtom selectionTarget = gtk_selection_data_get_target(aSelectionData);
+  LOGCLIP(("  selection target %s\n", gdk_atom_name(selectionTarget)));
 
   // Check to see if the selection data is some text type.
   if (gtk_targets_include_text(&selectionTarget, 1)) {
+    LOGCLIP(("  providing text/unicode data\n"));
     // Try to convert our internal type into a text string.  Get
     // the transferable for this clipboard and try to get the
     // text/unicode type for it.
     rv = trans->GetTransferData("text/unicode", getter_AddRefs(item));
     if (NS_FAILED(rv) || !item) {
+      LOGCLIP(("  GetTransferData() failed to get text/unicode!\n"));
       return;
     }
 
@@ -448,13 +607,25 @@ void nsClipboard::SelectionGetEvent(GtkClipboard* aClipboard,
     wideString->GetData(ucs2string);
     NS_ConvertUTF16toUTF8 utf8string(ucs2string);
 
-    gtk_selection_data_set_text(aSelectionData, utf8string.get(),
-                                utf8string.Length());
+    LOGCLIP(("  sent %d bytes of utf-8 data\n", utf8string.Length()));
+    if (selectionTarget == gdk_atom_intern("text/plain;charset=utf-8", FALSE)) {
+      LOGCLIP(
+          ("  using gtk_selection_data_set for 'text/plain;charset=utf-8'\n"));
+      // Bypass gtk_selection_data_set_text, which will convert \n to \r\n
+      // in some versions of GTK.
+      gtk_selection_data_set(aSelectionData, selectionTarget, 8,
+                             reinterpret_cast<const guchar*>(utf8string.get()),
+                             utf8string.Length());
+    } else {
+      gtk_selection_data_set_text(aSelectionData, utf8string.get(),
+                                  utf8string.Length());
+    }
     return;
   }
 
   // Check to see if the selection data is an image type
   if (gtk_targets_include_image(&selectionTarget, 1, TRUE)) {
+    LOGCLIP(("  providing image data\n"));
     // Look through our transfer data for the image
     static const char* const imageMimeTypes[] = {kNativeImageMime,
                                                  kPNGImageMime, kJPEGImageMime,
@@ -464,36 +635,49 @@ void nsClipboard::SelectionGetEvent(GtkClipboard* aClipboard,
     for (uint32_t i = 0; i < ArrayLength(imageMimeTypes); i++) {
       rv = trans->GetTransferData(imageMimeTypes[i], getter_AddRefs(imageItem));
       if (NS_FAILED(rv)) {
+        LOGCLIP(
+            ("    %s is missing at GetTransferData()\n", imageMimeTypes[i]));
         continue;
       }
 
       image = do_QueryInterface(imageItem);
       if (image) {
+        LOGCLIP(
+            ("    %s is available at GetTransferData()\n", imageMimeTypes[i]));
         break;
       }
     }
 
     if (!image) {  // Not getting an image for an image mime type!?
+      LOGCLIP(("    Failed to get any image mime from GetTransferData()!\n"));
       return;
     }
 
     GdkPixbuf* pixbuf = nsImageToPixbuf::ImageToPixbuf(image);
-    if (!pixbuf) return;
+    if (!pixbuf) {
+      LOGCLIP(("    nsImageToPixbuf::ImageToPixbuf() failed!\n"));
+      return;
+    }
 
+    LOGCLIP(("    Setting pixbuf image data as %s\n",
+             gdk_atom_name(selectionTarget)));
     gtk_selection_data_set_pixbuf(aSelectionData, pixbuf);
     g_object_unref(pixbuf);
     return;
   }
 
   if (selectionTarget == gdk_atom_intern(kHTMLMime, FALSE)) {
+    LOGCLIP(("  providing %s data\n", kHTMLMime));
     rv = trans->GetTransferData(kHTMLMime, getter_AddRefs(item));
     if (NS_FAILED(rv) || !item) {
+      LOGCLIP(("  failed to get %s data by GetTransferData()!\n", kHTMLMime));
       return;
     }
 
     nsCOMPtr<nsISupportsString> wideString;
     wideString = do_QueryInterface(item);
     if (!wideString) {
+      LOGCLIP(("  failed to get wideString interface!"));
       return;
     }
 
@@ -505,19 +689,28 @@ void nsClipboard::SelectionGetEvent(GtkClipboard* aClipboard,
     html.AppendLiteral(kHTMLMarkupPrefix);
     AppendUTF16toUTF8(ucs2string, html);
 
+    LOGCLIP(("  Setting %d bytest of %s data\n", html.Length(),
+             gdk_atom_name(selectionTarget)));
     gtk_selection_data_set(aSelectionData, selectionTarget, 8,
                            (const guchar*)html.get(), html.Length());
     return;
   }
 
+  LOGCLIP(("  Try if we have anything at GetTransferData() for %s\n",
+           gdk_atom_name(selectionTarget)));
+
   // Try to match up the selection data target to something our
   // transferable provides.
   gchar* target_name = gdk_atom_name(selectionTarget);
-  if (!target_name) return;
+  if (!target_name) {
+    LOGCLIP(("  Failed to get target name!\n"));
+    return;
+  }
 
   rv = trans->GetTransferData(target_name, getter_AddRefs(item));
   // nothing found?
   if (NS_FAILED(rv) || !item) {
+    LOGCLIP(("  Failed to get anything from GetTransferData()!\n"));
     g_free(target_name);
     return;
   }
@@ -528,37 +721,41 @@ void nsClipboard::SelectionGetEvent(GtkClipboard* aClipboard,
                                               item, &primitive_data, &dataLen);
 
   if (primitive_data) {
+    LOGCLIP(("  Setting %s as a primitive data type, %d bytes\n", target_name,
+             dataLen));
     gtk_selection_data_set(aSelectionData, selectionTarget,
                            8, /* 8 bits in a unit */
                            (const guchar*)primitive_data, dataLen);
     free(primitive_data);
+  } else {
+    LOGCLIP(("  Failed to get primitive data!\n"));
   }
 
   g_free(target_name);
 }
 
 void nsClipboard::SelectionClearEvent(GtkClipboard* aGtkClipboard) {
-  int32_t whichClipboard;
+  int32_t whichClipboard = GetGeckoClipboardType(aGtkClipboard);
+  if (whichClipboard < 0) {
+    return;
+  }
 
-  // which clipboard?
-  if (aGtkClipboard == gtk_clipboard_get(GDK_SELECTION_PRIMARY))
-    whichClipboard = kSelectionClipboard;
-  else if (aGtkClipboard == gtk_clipboard_get(GDK_SELECTION_CLIPBOARD))
-    whichClipboard = kGlobalClipboard;
-  else
-    return;  // THAT AIN'T NO CLIPBOARD I EVER HEARD OF
+  LOGCLIP(("nsClipboard::SelectionClearEvent (%s)\n",
+           whichClipboard == kSelectionClipboard ? "primary" : "clipboard"));
 
-  EmptyClipboard(whichClipboard);
+  ClearTransferable(whichClipboard);
 }
 
 void clipboard_get_cb(GtkClipboard* aGtkClipboard,
                       GtkSelectionData* aSelectionData, guint info,
                       gpointer user_data) {
+  LOGCLIP(("clipboard_get_cb() callback\n"));
   nsClipboard* aClipboard = static_cast<nsClipboard*>(user_data);
   aClipboard->SelectionGetEvent(aGtkClipboard, aSelectionData);
 }
 
 void clipboard_clear_cb(GtkClipboard* aGtkClipboard, gpointer user_data) {
+  LOGCLIP(("clipboard_clear_cb() callback\n"));
   nsClipboard* aClipboard = static_cast<nsClipboard*>(user_data);
   aClipboard->SelectionClearEvent(aGtkClipboard);
 }
@@ -582,68 +779,66 @@ void clipboard_clear_cb(GtkClipboard* aGtkClipboard, gpointer user_data) {
  * body      : pass to Mozilla
  * bodyLength: pass to Mozilla
  */
-void ConvertHTMLtoUCS2(const char* data, int32_t dataLength,
+bool ConvertHTMLtoUCS2(const char* data, int32_t dataLength, nsCString& charset,
                        char16_t** unicodeData, int32_t& outUnicodeLen) {
-  nsAutoCString charset;
-  GetHTMLCharset(data, dataLength, charset);  // get charset of HTML
-  if (charset.EqualsLiteral("UTF-16")) {      // current mozilla
+  if (charset.EqualsLiteral("UTF-16")) {  // current mozilla
     outUnicodeLen = (dataLength / 2) - 1;
     *unicodeData = reinterpret_cast<char16_t*>(
         moz_xmalloc((outUnicodeLen + sizeof('\0')) * sizeof(char16_t)));
     memcpy(*unicodeData, data + sizeof(char16_t),
            outUnicodeLen * sizeof(char16_t));
     (*unicodeData)[outUnicodeLen] = '\0';
-  } else if (charset.EqualsLiteral("UNKNOWN")) {
-    outUnicodeLen = 0;
-    return;
-  } else {
-    // app which use "text/html" to copy&paste
-    // get the decoder
-    auto encoding = Encoding::ForLabelNoReplacement(charset);
-    if (!encoding) {
-#ifdef DEBUG_CLIPBOARD
-      g_print("        get unicode decoder error\n");
-#endif
-      outUnicodeLen = 0;
-      return;
-    }
-
-    auto dataSpan = MakeSpan(data, dataLength);
-    // Remove kHTMLMarkupPrefix again, it won't necessarily cause any
-    // issues, but might confuse other users.
-    const size_t prefixLen = ArrayLength(kHTMLMarkupPrefix) - 1;
-    if (dataSpan.Length() >= prefixLen &&
-        Substring(data, prefixLen).EqualsLiteral(kHTMLMarkupPrefix)) {
-      dataSpan = dataSpan.From(prefixLen);
-    }
-
-    auto decoder = encoding->NewDecoder();
-    CheckedInt<size_t> needed =
-        decoder->MaxUTF16BufferLength(dataSpan.Length());
-    if (!needed.isValid() || needed.value() > INT32_MAX) {
-      outUnicodeLen = 0;
-      return;
-    }
-
-    outUnicodeLen = 0;
-    if (needed.value()) {
-      *unicodeData = reinterpret_cast<char16_t*>(
-          moz_xmalloc((needed.value() + 1) * sizeof(char16_t)));
-      uint32_t result;
-      size_t read;
-      size_t written;
-      bool hadErrors;
-      Tie(result, read, written, hadErrors) = decoder->DecodeToUTF16(
-          AsBytes(dataSpan), MakeSpan(*unicodeData, needed.value()), true);
-      MOZ_ASSERT(result == kInputEmpty);
-      MOZ_ASSERT(read == size_t(dataSpan.Length()));
-      MOZ_ASSERT(written <= needed.value());
-      Unused << hadErrors;
-      outUnicodeLen = written;
-      // null terminate.
-      (*unicodeData)[outUnicodeLen] = '\0';
-    }  // if valid length
+    return true;
   }
+  if (charset.EqualsLiteral("UNKNOWN")) {
+    outUnicodeLen = 0;
+    return false;
+  }
+  // app which use "text/html" to copy&paste
+  // get the decoder
+  auto encoding = Encoding::ForLabelNoReplacement(charset);
+  if (!encoding) {
+    LOGCLIP(("ConvertHTMLtoUCS2: get unicode decoder error\n"));
+    outUnicodeLen = 0;
+    return false;
+  }
+
+  auto dataSpan = Span(data, dataLength);
+  // Remove kHTMLMarkupPrefix again, it won't necessarily cause any
+  // issues, but might confuse other users.
+  const size_t prefixLen = ArrayLength(kHTMLMarkupPrefix) - 1;
+  if (dataSpan.Length() >= prefixLen &&
+      Substring(data, prefixLen).EqualsLiteral(kHTMLMarkupPrefix)) {
+    dataSpan = dataSpan.From(prefixLen);
+  }
+
+  auto decoder = encoding->NewDecoder();
+  CheckedInt<size_t> needed = decoder->MaxUTF16BufferLength(dataSpan.Length());
+  if (!needed.isValid() || needed.value() > INT32_MAX) {
+    outUnicodeLen = 0;
+    return false;
+  }
+
+  outUnicodeLen = 0;
+  if (needed.value()) {
+    *unicodeData = reinterpret_cast<char16_t*>(
+        moz_xmalloc((needed.value() + 1) * sizeof(char16_t)));
+    uint32_t result;
+    size_t read;
+    size_t written;
+    bool hadErrors;
+    Tie(result, read, written, hadErrors) = decoder->DecodeToUTF16(
+        AsBytes(dataSpan), Span(*unicodeData, needed.value()), true);
+    MOZ_ASSERT(result == kInputEmpty);
+    MOZ_ASSERT(read == size_t(dataSpan.Length()));
+    MOZ_ASSERT(written <= needed.value());
+    Unused << hadErrors;
+    outUnicodeLen = written;
+    // null terminate.
+    (*unicodeData)[outUnicodeLen] = '\0';
+    return true;
+  }  // if valid length
+  return false;
 }
 
 /*
@@ -653,12 +848,13 @@ void ConvertHTMLtoUCS2(const char* data, int32_t dataLength,
  *  2. "UNKNOWN":     mozilla can't detect what encode it use
  *  3. other:         "text/html" with other charset than utf-16
  */
-void GetHTMLCharset(const char* data, int32_t dataLength, nsCString& str) {
+bool GetHTMLCharset(const char* data, int32_t dataLength, nsCString& str) {
   // if detect "FFFE" or "FEFF", assume UTF-16
   char16_t* beginChar = (char16_t*)data;
   if ((beginChar[0] == 0xFFFE) || (beginChar[0] == 0xFEFF)) {
     str.AssignLiteral("UTF-16");
-    return;
+    LOGCLIP(("GetHTMLCharset: Charset of HTML is UTF-16\n"));
+    return true;
   }
   // no "FFFE" and "FEFF", assume ASCII first to find "charset" info
   const nsDependentCSubstring htmlStr(data, dataLength);
@@ -667,13 +863,11 @@ void GetHTMLCharset(const char* data, int32_t dataLength, nsCString& str) {
   htmlStr.EndReading(end);
   nsACString::const_iterator valueStart(start), valueEnd(start);
 
-  if (CaseInsensitiveFindInReadable(NS_LITERAL_CSTRING("CONTENT=\"text/html;"),
-                                    start, end)) {
+  if (CaseInsensitiveFindInReadable("CONTENT=\"text/html;"_ns, start, end)) {
     start = end;
     htmlStr.EndReading(end);
 
-    if (CaseInsensitiveFindInReadable(NS_LITERAL_CSTRING("charset="), start,
-                                      end)) {
+    if (CaseInsensitiveFindInReadable("charset="_ns, start, end)) {
       valueStart = end;
       start = end;
       htmlStr.EndReading(end);
@@ -685,10 +879,10 @@ void GetHTMLCharset(const char* data, int32_t dataLength, nsCString& str) {
   if (valueStart != valueEnd) {
     str = Substring(valueStart, valueEnd);
     ToUpperCase(str);
-#ifdef DEBUG_CLIPBOARD
-    printf("Charset of HTML = %s\n", charsetUpperStr.get());
-#endif
-    return;
+    LOGCLIP(("GetHTMLCharset: Charset of HTML = %s\n", str.get()));
+    return true;
   }
   str.AssignLiteral("UNKNOWN");
+  LOGCLIP(("GetHTMLCharset: Failed to get HTML Charset!\n"));
+  return false;
 }
