@@ -8,24 +8,21 @@
 #include <stdlib.h>
 
 #include "mozilla/ArrayUtils.h"
+#include "mozilla/Services.h"
 
-#include "mozcontainer.h"
+#include "MozContainer.h"
 #include "nsIPrintSettings.h"
 #include "nsIWidget.h"
 #include "nsPrintDialogGTK.h"
 #include "nsPrintSettingsGTK.h"
 #include "nsString.h"
 #include "nsReadableUtils.h"
-#include "nsIFile.h"
 #include "nsIStringBundle.h"
 #include "nsIPrintSettingsService.h"
-#include "nsIDOMWindow.h"
 #include "nsPIDOMWindow.h"
-#include "nsIBaseWindow.h"
-#include "nsIDocShellTreeItem.h"
-#include "nsIDocShell.h"
 #include "nsIGIOService.h"
 #include "WidgetUtils.h"
+#include "WidgetUtilsGtk.h"
 #include "nsIObserverService.h"
 
 // for gdk_x11_window_get_xid
@@ -138,9 +135,6 @@ class nsPrintDialogWidgetGTK {
 
  private:
   GtkWidget* dialog;
-  GtkWidget* radio_as_laid_out;
-  GtkWidget* radio_selected_frame;
-  GtkWidget* radio_separate_frames;
   GtkWidget* shrink_to_fit_toggle;
   GtkWidget* print_bg_colors_toggle;
   GtkWidget* print_bg_images_toggle;
@@ -160,7 +154,6 @@ class nsPrintDialogWidgetGTK {
    *   "Import" means to copy from NS to GTK
    *   "Export" means to copy from GTK to NS
    */
-  void ExportFramePrinting(nsIPrintSettings* aNS, GtkPrintSettings* aSettings);
   void ExportHeaderFooter(nsIPrintSettings* aNS);
 };
 
@@ -182,9 +175,9 @@ nsPrintDialogWidgetGTK::nsPrintDialogWidgetGTK(nsPIDOMWindowOuter* aParent,
   gtk_print_unix_dialog_set_manual_capabilities(
       GTK_PRINT_UNIX_DIALOG(dialog),
       GtkPrintCapabilities(
-          GTK_PRINT_CAPABILITY_PAGE_SET | GTK_PRINT_CAPABILITY_COPIES |
-          GTK_PRINT_CAPABILITY_COLLATE | GTK_PRINT_CAPABILITY_REVERSE |
-          GTK_PRINT_CAPABILITY_SCALE | GTK_PRINT_CAPABILITY_GENERATE_PDF));
+          GTK_PRINT_CAPABILITY_COPIES | GTK_PRINT_CAPABILITY_COLLATE |
+          GTK_PRINT_CAPABILITY_REVERSE | GTK_PRINT_CAPABILITY_SCALE |
+          GTK_PRINT_CAPABILITY_GENERATE_PDF));
 
   // The vast majority of magic numbers in this widget construction are padding.
   // e.g. for the set_border_width below, 12px matches that of just about every
@@ -194,49 +187,6 @@ nsPrintDialogWidgetGTK::nsPrintDialogWidgetGTK(nsPIDOMWindowOuter* aParent,
   GtkWidget* tab_label =
       gtk_label_new(GetUTF8FromBundle("optionsTabLabelGTK").get());
 
-  int16_t frameUIFlag;
-  aSettings->GetHowToEnableFrameUI(&frameUIFlag);
-  radio_as_laid_out = gtk_radio_button_new_with_mnemonic(
-      nullptr, GetUTF8FromBundle("asLaidOut").get());
-  if (frameUIFlag == nsIPrintSettings::kFrameEnableNone)
-    gtk_widget_set_sensitive(radio_as_laid_out, FALSE);
-
-  radio_selected_frame = gtk_radio_button_new_with_mnemonic_from_widget(
-      GTK_RADIO_BUTTON(radio_as_laid_out),
-      GetUTF8FromBundle("selectedFrame").get());
-  if (frameUIFlag == nsIPrintSettings::kFrameEnableNone ||
-      frameUIFlag == nsIPrintSettings::kFrameEnableAsIsAndEach)
-    gtk_widget_set_sensitive(radio_selected_frame, FALSE);
-
-  radio_separate_frames = gtk_radio_button_new_with_mnemonic_from_widget(
-      GTK_RADIO_BUTTON(radio_as_laid_out),
-      GetUTF8FromBundle("separateFrames").get());
-  if (frameUIFlag == nsIPrintSettings::kFrameEnableNone)
-    gtk_widget_set_sensitive(radio_separate_frames, FALSE);
-
-  // "Print Frames" options label, bold and center-aligned
-  GtkWidget* print_frames_label = gtk_label_new(nullptr);
-  char* pangoMarkup = g_markup_printf_escaped(
-      "<b>%s</b>", GetUTF8FromBundle("printFramesTitleGTK").get());
-  gtk_label_set_markup(GTK_LABEL(print_frames_label), pangoMarkup);
-  g_free(pangoMarkup);
-  gtk_misc_set_alignment(GTK_MISC(print_frames_label), 0, 0);
-
-  // Align the radio buttons slightly so they appear to fall under the
-  // aforementioned label as per the GNOME HIG
-  GtkWidget* frames_radio_container = gtk_alignment_new(0, 0, 0, 0);
-  gtk_alignment_set_padding(GTK_ALIGNMENT(frames_radio_container), 8, 0, 12, 0);
-
-  // Radio buttons for the print frames options
-  GtkWidget* frames_radio_list = gtk_vbox_new(TRUE, 2);
-  gtk_box_pack_start(GTK_BOX(frames_radio_list), radio_as_laid_out, FALSE,
-                     FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(frames_radio_list), radio_selected_frame, FALSE,
-                     FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(frames_radio_list), radio_separate_frames, FALSE,
-                     FALSE, 0);
-  gtk_container_add(GTK_CONTAINER(frames_radio_container), frames_radio_list);
-
   // Check buttons for shrink-to-fit and print selection
   GtkWidget* check_buttons_container = gtk_vbox_new(TRUE, 2);
   shrink_to_fit_toggle = gtk_check_button_new_with_mnemonic(
@@ -247,9 +197,7 @@ nsPrintDialogWidgetGTK::nsPrintDialogWidgetGTK(nsPIDOMWindowOuter* aParent,
   // GTK+2.18 and above allow us to add a "Selection" option to the main
   // settings screen, rather than adding an option on a custom tab like we must
   // do on older versions.
-  bool canSelectText;
-  aSettings->GetPrintOptions(nsIPrintSettings::kEnableSelectionRB,
-                             &canSelectText);
+  bool canSelectText = aSettings->GetIsPrintSelectionRBEnabled();
   if (gtk_major_version > 2 ||
       (gtk_major_version == 2 && gtk_minor_version >= 18)) {
     useNativeSelection = true;
@@ -277,7 +225,7 @@ nsPrintDialogWidgetGTK::nsPrintDialogWidgetGTK(nsPIDOMWindowOuter* aParent,
 
   // "Appearance" options label, bold and center-aligned
   GtkWidget* appearance_label = gtk_label_new(nullptr);
-  pangoMarkup = g_markup_printf_escaped(
+  char* pangoMarkup = g_markup_printf_escaped(
       "<b>%s</b>", GetUTF8FromBundle("printBGOptions").get());
   gtk_label_set_markup(GTK_LABEL(appearance_label), pangoMarkup);
   g_free(pangoMarkup);
@@ -355,10 +303,6 @@ nsPrintDialogWidgetGTK::nsPrintDialogWidgetGTK(nsPIDOMWindowOuter* aParent,
                      header_footer_container, FALSE, FALSE, 0);
 
   // Construction of everything
-  gtk_box_pack_start(GTK_BOX(custom_options_tab), print_frames_label, FALSE,
-                     FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(custom_options_tab), frames_radio_container, FALSE,
-                     FALSE, 0);
   gtk_box_pack_start(GTK_BOX(custom_options_tab), check_buttons_container,
                      FALSE, FALSE, 10);  // 10px padding
   gtk_box_pack_start(GTK_BOX(custom_options_tab), appearance_vertical_squasher,
@@ -387,8 +331,7 @@ const char* nsPrintDialogWidgetGTK::OptionWidgetToString(GtkWidget* dropdown) {
 
   if (index == CUSTOM_VALUE_INDEX)
     return (const char*)g_object_get_data(G_OBJECT(dropdown), "custom-text");
-  else
-    return header_footer_tags[index];
+  return header_footer_tags[index];
 }
 
 gint nsPrintDialogWidgetGTK::Run() {
@@ -397,20 +340,6 @@ gint nsPrintDialogWidgetGTK::Run() {
   return response;
 }
 
-void nsPrintDialogWidgetGTK::ExportFramePrinting(nsIPrintSettings* aNS,
-                                                 GtkPrintSettings* aSettings) {
-  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio_as_laid_out)))
-    aNS->SetPrintFrameType(nsIPrintSettings::kFramesAsIs);
-  else if (gtk_toggle_button_get_active(
-               GTK_TOGGLE_BUTTON(radio_selected_frame)))
-    aNS->SetPrintFrameType(nsIPrintSettings::kSelectedFrame);
-  else if (gtk_toggle_button_get_active(
-               GTK_TOGGLE_BUTTON(radio_separate_frames)))
-    aNS->SetPrintFrameType(nsIPrintSettings::kEachFrameSep);
-  else
-    aNS->SetPrintFrameType(nsIPrintSettings::kNoFrames);
-}
-
 void nsPrintDialogWidgetGTK::ExportHeaderFooter(nsIPrintSettings* aNS) {
   const char* header_footer_str;
   header_footer_str = OptionWidgetToString(header_dropdown[0]);
@@ -447,13 +376,11 @@ nsresult nsPrintDialogWidgetGTK::ImportSettings(nsIPrintSettings* aNSSettings) {
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(shrink_to_fit_toggle),
                                geckoBool);
 
-  aNSSettings->GetPrintBGColors(&geckoBool);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(print_bg_colors_toggle),
-                               geckoBool);
+                               aNSSettings->GetPrintBGColors());
 
-  aNSSettings->GetPrintBGImages(&geckoBool);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(print_bg_images_toggle),
-                               geckoBool);
+                               aNSSettings->GetPrintBGImages());
 
   gtk_print_unix_dialog_set_settings(GTK_PRINT_UNIX_DIALOG(dialog), settings);
   gtk_print_unix_dialog_set_page_setup(GTK_PRINT_UNIX_DIALOG(dialog), setup);
@@ -472,7 +399,6 @@ nsresult nsPrintDialogWidgetGTK::ExportSettings(nsIPrintSettings* aNSSettings) {
   GtkPrinter* printer =
       gtk_print_unix_dialog_get_selected_printer(GTK_PRINT_UNIX_DIALOG(dialog));
   if (settings && setup && printer) {
-    ExportFramePrinting(aNSSettings, settings);
     ExportHeaderFooter(aNSSettings);
 
     aNSSettings->SetOutputFormat(nsIPrintSettings::kOutputFormatNative);
@@ -506,7 +432,7 @@ nsresult nsPrintDialogWidgetGTK::ExportSettings(nsIPrintSettings* aNSSettings) {
         printSelectionOnly = gtk_toggle_button_get_active(
             GTK_TOGGLE_BUTTON(selection_only_toggle));
       }
-      aNSSettingsGTK->SetForcePrintSelectionOnly(printSelectionOnly);
+      aNSSettingsGTK->SetPrintSelectionOnly(printSelectionOnly);
     }
   }
 
@@ -555,9 +481,9 @@ GtkWidget* nsPrintDialogWidgetGTK::ConstructHeaderFooterDropdown(
 
 NS_IMPL_ISUPPORTS(nsPrintDialogServiceGTK, nsIPrintDialogService)
 
-nsPrintDialogServiceGTK::nsPrintDialogServiceGTK() {}
+nsPrintDialogServiceGTK::nsPrintDialogServiceGTK() = default;
 
-nsPrintDialogServiceGTK::~nsPrintDialogServiceGTK() {}
+nsPrintDialogServiceGTK::~nsPrintDialogServiceGTK() = default;
 
 NS_IMETHODIMP
 nsPrintDialogServiceGTK::Init() { return NS_OK; }
@@ -599,7 +525,7 @@ static void wayland_window_handle_exported(GdkWindow* window,
 static gboolean window_export_handle(GtkWindow* window,
                                      GtkWindowHandleExported callback,
                                      gpointer user_data) {
-  if (GDK_IS_X11_DISPLAY(gtk_widget_get_display(GTK_WIDGET(window)))) {
+  if (GdkIsX11Display()) {
     GdkWindow* gdk_window = gtk_widget_get_window(GTK_WIDGET(window));
     char* handle_str;
     guint32 xid = (guint32)gdk_x11_window_get_xid(gdk_window);
@@ -610,7 +536,7 @@ static gboolean window_export_handle(GtkWindow* window,
     return true;
   }
 #ifdef MOZ_WAYLAND
-  else {
+  else if (GdkIsWaylandDisplay()) {
     GdkWindow* gdk_window = gtk_widget_get_window(GTK_WIDGET(window));
     WaylandWindowHandleExportedData* data;
 
@@ -628,9 +554,8 @@ static gboolean window_export_handle(GtkWindow* window,
             gdk_window, wayland_window_handle_exported, data, g_free)) {
       g_free(data);
       return false;
-    } else {
-      return true;
     }
+    return true;
   }
 #endif
 
@@ -754,7 +679,7 @@ void nsFlatpakPrintPortal::PreparePrint(GtkWindow* aWindow,
 
   // We need to remember GtkWindow to unexport window handle after it is
   // no longer needed by the portal dialog (apply only on non-X11 sessions).
-  if (!GDK_IS_X11_DISPLAY(gdk_display_get_default())) {
+  if (GdkIsWaylandDisplay()) {
     mParentWindow = aWindow;
   }
 
@@ -853,11 +778,8 @@ void nsFlatpakPrintPortal::FinishPrintDialog(GVariant* parameters) {
   g_variant_get(parameters, "(u@a{sv})", &response, &options);
   mResult = GTK_PRINT_OPERATION_RESULT_CANCEL;
   if (response == 0) {
-    GVariant* v;
-
-    char* filename;
-    char* uri;
-    v = g_variant_lookup_value(options, "settings", G_VARIANT_TYPE_VARDICT);
+    GVariant* v =
+        g_variant_lookup_value(options, "settings", G_VARIANT_TYPE_VARDICT);
     static auto s_gtk_print_settings_new_from_gvariant =
         reinterpret_cast<GtkPrintSettings* (*)(GVariant*)>(
             dlsym(RTLD_DEFAULT, "gtk_print_settings_new_from_gvariant"));
@@ -874,14 +796,6 @@ void nsFlatpakPrintPortal::FinishPrintDialog(GVariant* parameters) {
 
     g_variant_lookup(options, "token", "u", &mToken);
 
-    // Force printing to file because only filedescriptor of the file
-    // can be passed to portal
-    int fd = g_file_open_tmp("gtkprintXXXXXX", &filename, NULL);
-    uri = g_filename_to_uri(filename, NULL, NULL);
-    gtk_print_settings_set(printSettings, GTK_PRINT_SETTINGS_OUTPUT_URI, uri);
-    g_free(uri);
-    close(fd);
-
     // Save native settings in the session object
     mPrintAndPageSettings->SetGtkPrintSettings(printSettings);
     mPrintAndPageSettings->SetGtkPageSetup(pageSetup);
@@ -988,8 +902,7 @@ nsFlatpakPrintPortal::~nsFlatpakPrintPortal() {
 
 NS_IMETHODIMP
 nsPrintDialogServiceGTK::Show(nsPIDOMWindowOuter* aParent,
-                              nsIPrintSettings* aSettings,
-                              nsIWebBrowserPrint* aWebBrowserPrint) {
+                              nsIPrintSettings* aSettings) {
   MOZ_ASSERT(aParent, "aParent must not be null");
   MOZ_ASSERT(aSettings, "aSettings must not be null");
 
@@ -1013,7 +926,6 @@ nsPrintDialogServiceGTK::Show(nsPIDOMWindowOuter* aParent,
     // This blocks until nsFlatpakPrintPortal::FinishPrintDialog is called
     GtkPrintOperationResult printDialogResult = fpPrintPortal->GetResult();
 
-    rv = NS_OK;
     switch (printDialogResult) {
       case GTK_PRINT_OPERATION_RESULT_APPLY: {
         nsCOMPtr<nsIObserverService> os =
@@ -1086,19 +998,43 @@ nsPrintDialogServiceGTK::ShowPageSetup(nsPIDOMWindowOuter* aParent,
     nsString printName;
     aNSSettings->GetPrinterName(printName);
     if (printName.IsVoid()) {
-      psService->GetDefaultPrinterName(printName);
+      psService->GetLastUsedPrinterName(printName);
       aNSSettings->SetPrinterName(printName);
     }
     psService->InitPrintSettingsFromPrefs(aNSSettings, true,
                                           nsIPrintSettings::kInitSaveAll);
   }
 
+  // Frustratingly, gtk_print_run_page_setup_dialog doesn't tell us whether
+  // the user cancelled or confirmed the dialog! So to avoid needlessly
+  // refreshing the preview when Page Setup was cancelled, we compare the
+  // serializations of old and new settings; if they're the same, bail out.
   GtkPrintSettings* gtkSettings = aNSSettingsGTK->GetGtkPrintSettings();
   GtkPageSetup* oldPageSetup = aNSSettingsGTK->GetGtkPageSetup();
+  GKeyFile* oldKeyFile = g_key_file_new();
+  gtk_page_setup_to_key_file(oldPageSetup, oldKeyFile, nullptr);
+  gsize oldLength;
+  gchar* oldData = g_key_file_to_data(oldKeyFile, &oldLength, nullptr);
+  g_key_file_free(oldKeyFile);
 
   GtkPageSetup* newPageSetup =
       gtk_print_run_page_setup_dialog(gtkParent, oldPageSetup, gtkSettings);
 
+  GKeyFile* newKeyFile = g_key_file_new();
+  gtk_page_setup_to_key_file(newPageSetup, newKeyFile, nullptr);
+  gsize newLength;
+  gchar* newData = g_key_file_to_data(newKeyFile, &newLength, nullptr);
+  g_key_file_free(newKeyFile);
+
+  bool unchanged =
+      (oldLength == newLength && !memcmp(oldData, newData, oldLength));
+  g_free(oldData);
+  g_free(newData);
+  if (unchanged) {
+    g_object_unref(newPageSetup);
+    return NS_ERROR_ABORT;
+  }
+
   aNSSettingsGTK->SetGtkPageSetup(newPageSetup);
 
   // Now newPageSetup has a refcount of 2 (SetGtkPageSetup will addref), put it
@@ -1106,8 +1042,11 @@ nsPrintDialogServiceGTK::ShowPageSetup(nsPIDOMWindowOuter* aParent,
   g_object_unref(newPageSetup);
 
   if (psService)
-    psService->SavePrintSettingsToPrefs(aNSSettings, true,
-                                        nsIPrintSettings::kInitSaveAll);
+    psService->SavePrintSettingsToPrefs(
+        aNSSettings, true,
+        nsIPrintSettings::kInitSaveOrientation |
+            nsIPrintSettings::kInitSavePaperSize |
+            nsIPrintSettings::kInitSaveUnwriteableMargins);
 
   return NS_OK;
 }
