@@ -98,7 +98,7 @@ JumpListBuilder::JumpListBuilder()
   // Instantiate mJumpListMgr in the multithreaded apartment so that proxied
   // calls on that object do not need to interact with the main thread's message
   // pump.
-  mscom::EnsureMTA([this]() {
+  mscom::EnsureMTA([&]() {
     RefPtr<ICustomDestinationList> jumpListMgr;
     HRESULT hr = ::CoCreateInstance(
         CLSID_DestinationList, nullptr, CLSCTX_INPROC_SERVER,
@@ -117,8 +117,7 @@ JumpListBuilder::JumpListBuilder()
   }
 
   // Make a lazy thread for any IO
-  mIOThread = new LazyIdleThread(DEFAULT_THREAD_TIMEOUT_MS,
-                                 NS_LITERAL_CSTRING("Jump List"),
+  mIOThread = new LazyIdleThread(DEFAULT_THREAD_TIMEOUT_MS, "Jump List"_ns,
                                  LazyIdleThread::ManualShutdown);
   Preferences::AddStrongObserver(this, kPrefTaskbarEnabled);
 
