@@ -7,7 +7,6 @@
 #include "MemoryTelemetry.h"
 #include "nsMemoryReporterManager.h"
 
-#include "GCTelemetry.h"
 #include "mozilla/ClearOnShutdown.h"
 #include "mozilla/Result.h"
 #include "mozilla/ResultExtensions.h"
@@ -24,6 +23,7 @@
 #include "nsIMemoryReporter.h"
 #include "nsIWindowMediator.h"
 #include "nsImportModule.h"
+#include "nsITelemetry.h"
 #include "nsNetCID.h"
 #include "nsObserverService.h"
 #include "nsReadableUtils.h"
@@ -47,19 +47,6 @@ static constexpr const char* kTopicCycleCollectorBegin =
 // How long to wait in millis for all the child memory reports to come in
 static constexpr uint32_t kTotalMemoryCollectorTimeout = 200;
 
-static Result<nsCOMPtr<mozIGCTelemetry>, nsresult> GetGCTelemetry() {
-  nsresult rv;
-
-  nsCOMPtr<mozIGCTelemetryJSM> jsm =
-      do_ImportModule("resource://gre/modules/GCTelemetry.jsm", &rv);
-  MOZ_TRY(rv);
-
-  nsCOMPtr<mozIGCTelemetry> gcTelemetry;
-  MOZ_TRY(jsm->GetGCTelemetry(getter_AddRefs(gcTelemetry)));
-
-  return std::move(gcTelemetry);
-}
-
 namespace {
 
 enum class PrevValue : uint32_t {
@@ -141,13 +128,6 @@ nsresult MemoryTelemetry::DelayedInit() {
 
   GatherReports();
 
-  if (Telemetry::CanRecordExtended()) {
-    nsCOMPtr<mozIGCTelemetry> gcTelemetry;
-    MOZ_TRY_VAR(gcTelemetry, GetGCTelemetry());
-
-    MOZ_TRY(gcTelemetry->Init());
-  }
-
   return NS_OK;
 }
 
@@ -157,13 +137,6 @@ nsresult MemoryTelemetry::Shutdown() {
 
   obs->RemoveObserver(this, kTopicCycleCollectorBegin);
 
-  if (Telemetry::CanRecordExtended()) {
-    nsCOMPtr<mozIGCTelemetry> gcTelemetry;
-    MOZ_TRY_VAR(gcTelemetry, GetGCTelemetry());
-
-    MOZ_TRY(gcTelemetry->Shutdown());
-  }
-
   return NS_OK;
 }
 
@@ -246,9 +219,8 @@ nsresult MemoryTelemetry::GatherReports(
 
   // If we're running in the parent process, collect data from all processes for
   // the MEMORY_TOTAL histogram.
-  if (XRE_IsParentProcess() && !mTotalMemoryGatherer) {
-    mTotalMemoryGatherer = new TotalMemoryGatherer();
-    mTotalMemoryGatherer->Begin(mThreadPool);
+  if (XRE_IsParentProcess() && !mGatheringTotalMemory) {
+    GatherTotalMemory();
   }
 
   if (!Telemetry::CanRecordReleaseData()) {
@@ -285,10 +257,6 @@ nsresult MemoryTelemetry::GatherReports(
          UNITS_BYTES);
   RECORD(MEMORY_STORAGE_SQLITE, StorageSQLite, UNITS_BYTES);
 #ifdef XP_WIN
-  RECORD(LOW_MEMORY_EVENTS_VIRTUAL, LowMemoryEventsVirtual,
-         UNITS_COUNT_CUMULATIVE);
-  RECORD(LOW_MEMORY_EVENTS_COMMIT_SPACE, LowMemoryEventsCommitSpace,
-         UNITS_COUNT_CUMULATIVE);
   RECORD(LOW_MEMORY_EVENTS_PHYSICAL, LowMemoryEventsPhysical,
          UNITS_COUNT_CUMULATIVE);
 #endif
@@ -332,62 +300,107 @@ nsresult MemoryTelemetry::GatherReports(
   return NS_OK;
 }
 
-NS_IMPL_ISUPPORTS(MemoryTelemetry::TotalMemoryGatherer, nsITimerCallback)
+namespace {
+struct ChildProcessInfo {
+  GeckoProcessType mType;
+#if defined(XP_WIN)
+  HANDLE mHandle;
+#elif defined(XP_MACOSX)
+  task_t mHandle;
+#else
+  pid_t mHandle;
+#endif
+};
+}  // namespace
 
 /**
- * Polls all child processes for their unique set size, and populates the
- * MEMORY_TOTAL and MEMORY_DISTRIBUTION_AMONG_CONTENT histograms with the
- * results.
+ * Runs a task on the background thread pool to fetch the memory usage of all
+ * processes.
  */
-void MemoryTelemetry::TotalMemoryGatherer::Begin(nsIEventTarget* aThreadPool) {
-  nsCOMPtr<nsISerialEventTarget> target = GetMainThreadSerialEventTarget();
+void MemoryTelemetry::GatherTotalMemory() {
+  MOZ_ASSERT(!mGatheringTotalMemory);
+  mGatheringTotalMemory = true;
 
-  nsTArray<ContentParent*> parents;
-  ContentParent::GetAll(parents);
-  for (auto& parent : parents) {
-    mRemainingChildCount++;
-    parent->SendGetMemoryUniqueSetSize()->Then(
-        target, "TotalMemoryGather::Begin", this,
-        &TotalMemoryGatherer::CollectResult, &TotalMemoryGatherer::OnFailure);
-  }
+  nsTArray<ChildProcessInfo> infos;
+  mozilla::ipc::GeckoChildProcessHost::GetAll(
+      [&](mozilla::ipc::GeckoChildProcessHost* aGeckoProcess) {
+        if (!aGeckoProcess->GetChildProcessHandle()) {
+          return;
+        }
 
-  mChildSizes.SetCapacity(mRemainingChildCount);
+        ChildProcessInfo info{};
+        info.mType = aGeckoProcess->GetProcessType();
 
-  RefPtr<TotalMemoryGatherer> self{this};
+        // NOTE: For now we ignore non-content processes here for compatibility
+        // with the existing probe. We may want to introduce a new probe in the
+        // future which also collects data for non-content processes.
+        if (info.mType != GeckoProcessType_Content) {
+          return;
+        }
 
-  aThreadPool->Dispatch(
-      NS_NewRunnableFunction(
-          "TotalMemoryGather::Begin",
-          [self]() {
-            RefPtr<nsMemoryReporterManager> mgr =
-                nsMemoryReporterManager::GetOrCreate();
-            MOZ_RELEASE_ASSERT(mgr);
+#if defined(XP_WIN)
+        if (!::DuplicateHandle(::GetCurrentProcess(),
+                               aGeckoProcess->GetChildProcessHandle(),
+                               ::GetCurrentProcess(), &info.mHandle, 0, false,
+                               DUPLICATE_SAME_ACCESS)) {
+          return;
+        }
+#elif defined(XP_MACOSX)
+        info.mHandle = aGeckoProcess->GetChildTask();
+        if (mach_port_mod_refs(mach_task_self(), info.mHandle,
+                               MACH_PORT_RIGHT_SEND, 1) != KERN_SUCCESS) {
+          return;
+        }
+#else
+        info.mHandle = base::GetProcId(aGeckoProcess->GetChildProcessHandle());
+#endif
 
-            NS_DispatchToMainThread(NewRunnableMethod<int64_t>(
-                "TotalMemoryGather::CollectParentSize", self,
-                &TotalMemoryGatherer::CollectParentSize, mgr->ResidentFast()));
-          }),
-      NS_DISPATCH_NORMAL);
+        infos.AppendElement(info);
+      });
 
-  NS_NewTimerWithCallback(getter_AddRefs(mTimeout), this,
-                          kTotalMemoryCollectorTimeout,
-                          nsITimer::TYPE_ONE_SHOT);
+  mThreadPool->Dispatch(NS_NewRunnableFunction(
+      "MemoryTelemetry::GatherTotalMemory", [infos = std::move(infos)] {
+        RefPtr<nsMemoryReporterManager> mgr =
+            nsMemoryReporterManager::GetOrCreate();
+        MOZ_RELEASE_ASSERT(mgr);
+
+        int64_t totalMemory = mgr->ResidentFast();
+        nsTArray<int64_t> childSizes(infos.Length());
+
+        // Use our handle for the remote process to collect resident unique set
+        // size information for that process.
+        for (const auto& info : infos) {
+          int64_t memory =
+              nsMemoryReporterManager::ResidentUnique(info.mHandle);
+          if (memory > 0) {
+            childSizes.AppendElement(memory);
+            totalMemory += memory;
+          }
+
+#if defined(XP_WIN)
+          ::CloseHandle(info.mHandle);
+#elif defined(XP_MACOSX)
+          mach_port_deallocate(mach_task_self(), info.mHandle);
+#endif
+        }
+
+        NS_DispatchToMainThread(NS_NewRunnableFunction(
+            "MemoryTelemetry::FinishGatheringTotalMemory",
+            [totalMemory, childSizes = std::move(childSizes)] {
+              MemoryTelemetry::Get().FinishGatheringTotalMemory(totalMemory,
+                                                                childSizes);
+            }));
+      }));
 }
 
-nsresult MemoryTelemetry::TotalMemoryGatherer::MaybeFinish() {
-  // If we timed out waiting for a response from any child, we don't report
-  // anything for this attempt.
-  if (!mTimeout || !mHaveParentSize || mRemainingChildCount) {
-    return NS_OK;
-  }
-
-  mTimeout = nullptr;
-  MemoryTelemetry::Get().mTotalMemoryGatherer = nullptr;
+nsresult MemoryTelemetry::FinishGatheringTotalMemory(
+    int64_t aTotalMemory, const nsTArray<int64_t>& aChildSizes) {
+  mGatheringTotalMemory = false;
 
   HandleMemoryReport(Telemetry::MEMORY_TOTAL, nsIMemoryReporter::UNITS_BYTES,
-                     mTotalMemory);
+                     aTotalMemory);
 
-  if (mChildSizes.Length() > 1) {
+  if (aChildSizes.Length() > 1) {
     int32_t tabsCount;
     MOZ_TRY_VAR(tabsCount, GetOpenTabsCount());
 
@@ -402,10 +415,10 @@ nsresult MemoryTelemetry::TotalMemoryGatherer::MaybeFinish() {
 
     // Mean of the USS of all the content processes.
     int64_t mean = 0;
-    for (auto size : mChildSizes) {
+    for (auto size : aChildSizes) {
       mean += size;
     }
-    mean /= mChildSizes.Length();
+    mean /= aChildSizes.Length();
 
     // For some users, for unknown reasons (though most likely because they're
     // in a sandbox without procfs mounted), we wind up with 0 here, which
@@ -418,7 +431,7 @@ nsresult MemoryTelemetry::TotalMemoryGatherer::MaybeFinish() {
     // Absolute error of USS for each content process, normalized by the mean
     // (*100 to get it in percentage). 20% means for a content process that it
     // is using 20% more or 20% less than the mean.
-    for (auto size : mChildSizes) {
+    for (auto size : aChildSizes) {
       int64_t diff = llabs(size - mean) * 100 / mean;
 
       HandleMemoryReport(Telemetry::MEMORY_DISTRIBUTION_AMONG_CONTENT,
@@ -434,37 +447,6 @@ nsresult MemoryTelemetry::TotalMemoryGatherer::MaybeFinish() {
   return NS_OK;
 }
 
-void MemoryTelemetry::TotalMemoryGatherer::CollectParentSize(
-    int64_t aResident) {
-  mTotalMemory += aResident;
-  mHaveParentSize = true;
-
-  MaybeFinish();
-}
-
-void MemoryTelemetry::TotalMemoryGatherer::CollectResult(int64_t aChildUSS) {
-  mChildSizes.AppendElement(aChildUSS);
-
-  mTotalMemory += aChildUSS;
-  mRemainingChildCount--;
-
-  MaybeFinish();
-}
-
-void MemoryTelemetry::TotalMemoryGatherer::OnFailure(
-    mozilla::ipc::ResponseRejectReason aReason) {
-  // Treat failure of any request the same as a timeout.
-  Notify(nullptr);
-}
-
-nsresult MemoryTelemetry::TotalMemoryGatherer::Notify(nsITimer* aTimer) {
-  // Set mTimeout null to indicate the timeout has fired. After this, all
-  // results for this attempt will be ignored.
-  mTimeout = nullptr;
-  MemoryTelemetry::Get().mTotalMemoryGatherer = nullptr;
-  return NS_OK;
-}
-
 /* static */ Result<uint32_t, nsresult> MemoryTelemetry::GetOpenTabsCount() {
   nsresult rv;
 
