@@ -6,7 +6,6 @@
 #ifndef nsTraceRefcnt_h
 #define nsTraceRefcnt_h
 
-#include <stdio.h>  // for FILE
 #include "nscore.h"
 
 class nsTraceRefcnt {
@@ -17,14 +16,16 @@ class nsTraceRefcnt {
 
   static void ResetStatistics();
 
-  static void WalkTheStack(FILE* aStream);
-
   /**
    * Tell nsTraceRefcnt whether refcounting, allocation, and destruction
    * activity is legal.  This is used to trigger assertions for any such
    * activity that occurs because of static constructors or destructors.
    */
   static void SetActivityIsLegal(bool aLegal);
+
+#ifdef MOZ_ENABLE_FORKSERVER
+  static void ResetLogFiles(const char* aProcType = nullptr);
+#endif
 };
 
 #endif  // nsTraceRefcnt_h
