@@ -27,6 +27,10 @@
 #  include "mozilla/java/GeckoAppShellWrappers.h"
 #endif
 
+#ifdef MOZ_BACKGROUNDTASKS
+#  include "mozilla/BackgroundTasks.h"
+#endif
+
 #include "mozilla/Services.h"
 
 #include "nsCRT.h"
@@ -156,11 +160,11 @@ void LogMessageWithContext(FileLocation& aFile, uint32_t aLineNumber,
     return;
   }
 
-  nsresult rv = error->Init(
-      NS_ConvertUTF8toUTF16(formatted.get()), NS_ConvertUTF8toUTF16(file),
-      EmptyString(), aLineNumber, 0, nsIScriptError::warningFlag,
-      "chrome registration", false /* from private window */,
-      true /* from chrome context */);
+  nsresult rv = error->Init(NS_ConvertUTF8toUTF16(formatted.get()),
+                            NS_ConvertUTF8toUTF16(file), u""_ns, aLineNumber, 0,
+                            nsIScriptError::warningFlag, "chrome registration",
+                            false /* from private window */,
+                            true /* from chrome context */);
   if (NS_FAILED(rv)) {
     return;
   }
@@ -270,8 +274,7 @@ static bool CheckOsFlag(const nsAString& aFlag, const nsAString& aData,
   bool result = CheckStringFlag(aFlag, aData, aValue, aResult);
 #if defined(XP_UNIX) && !defined(XP_DARWIN) && !defined(ANDROID)
   if (result && aResult == eBad) {
-    result =
-        CheckStringFlag(aFlag, aData, NS_LITERAL_STRING("likeunix"), aResult);
+    result = CheckStringFlag(aFlag, aData, u"likeunix"_ns, aResult);
   }
 #endif
   return result;
@@ -390,25 +393,29 @@ void ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
   nsChromeRegistry::ManifestProcessingContext chromecx(aType, aFile);
   nsresult rv;
 
-  NS_NAMED_LITERAL_STRING(kContentAccessible, "contentaccessible");
-  NS_NAMED_LITERAL_STRING(kRemoteEnabled, "remoteenabled");
-  NS_NAMED_LITERAL_STRING(kRemoteRequired, "remoterequired");
-  NS_NAMED_LITERAL_STRING(kApplication, "application");
-  NS_NAMED_LITERAL_STRING(kAppVersion, "appversion");
-  NS_NAMED_LITERAL_STRING(kGeckoVersion, "platformversion");
-  NS_NAMED_LITERAL_STRING(kOs, "os");
-  NS_NAMED_LITERAL_STRING(kOsVersion, "osversion");
-  NS_NAMED_LITERAL_STRING(kABI, "abi");
-  NS_NAMED_LITERAL_STRING(kProcess, "process");
+  constexpr auto kContentAccessible = u"contentaccessible"_ns;
+  constexpr auto kRemoteEnabled = u"remoteenabled"_ns;
+  constexpr auto kRemoteRequired = u"remoterequired"_ns;
+  constexpr auto kApplication = u"application"_ns;
+  constexpr auto kAppVersion = u"appversion"_ns;
+  constexpr auto kGeckoVersion = u"platformversion"_ns;
+  constexpr auto kOs = u"os"_ns;
+  constexpr auto kOsVersion = u"osversion"_ns;
+  constexpr auto kABI = u"abi"_ns;
+  constexpr auto kProcess = u"process"_ns;
 #if defined(MOZ_WIDGET_ANDROID)
-  NS_NAMED_LITERAL_STRING(kTablet, "tablet");
+  constexpr auto kTablet = u"tablet"_ns;
 #endif
+  // You might expect this to be guarded by MOZ_BACKGROUNDTASKS, but it's not
+  // possible to have conditional manifest contents, so we need to recognize and
+  // discard these tokens even when MOZ_BACKGROUNDTASKS is not set.
+  constexpr auto kBackgroundTask = u"backgroundtask"_ns;
 
-  NS_NAMED_LITERAL_STRING(kMain, "main");
-  NS_NAMED_LITERAL_STRING(kContent, "content");
+  constexpr auto kMain = u"main"_ns;
+  constexpr auto kContent = u"content"_ns;
 
   // Obsolete
-  NS_NAMED_LITERAL_STRING(kXPCNativeWrappers, "xpcnativewrappers");
+  constexpr auto kXPCNativeWrappers = u"xpcnativewrappers"_ns;
 
   nsAutoString appID;
   nsAutoString appVersion;
@@ -571,6 +578,14 @@ void ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
     TriState stProcess = eUnspecified;
 #if defined(MOZ_WIDGET_ANDROID)
     TriState stTablet = eUnspecified;
+#endif
+#ifdef MOZ_BACKGROUNDTASKS
+    // When in background task mode, default to not registering
+    // category directivies unless backgroundtask=1 is specified.
+    TriState stBackgroundTask = (BackgroundTasks::IsBackgroundTaskMode() &&
+                                 strcmp("category", directive->directive) == 0)
+                                    ? eBad
+                                    : eUnspecified;
 #endif
     int flags = 0;
 
@@ -598,6 +613,18 @@ void ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
       }
 #endif
 
+      // You might expect this to be guarded by MOZ_BACKGROUNDTASKS, it's not
+      // possible to have conditional manifest contents.
+      bool flag;
+      if (CheckFlag(kBackgroundTask, wtoken, flag)) {
+#if defined(MOZ_BACKGROUNDTASKS)
+        // Background task mode is active: filter.
+        stBackgroundTask =
+            (flag == BackgroundTasks::IsBackgroundTaskMode()) ? eOK : eBad;
+#endif /* defined(MOZ_BACKGROUNDTASKS) */
+        continue;
+      }
+
       if (directive->contentflags) {
         bool flag;
         if (CheckFlag(kContentAccessible, wtoken, flag)) {
@@ -631,6 +658,9 @@ void ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
         stGeckoVersion == eBad || stOs == eBad || stOsVersion == eBad ||
 #ifdef MOZ_WIDGET_ANDROID
         stTablet == eBad ||
+#endif
+#ifdef MOZ_BACKGROUNDTASKS
+        stBackgroundTask == eBad ||
 #endif
         stABI == eBad || stProcess == eBad) {
       continue;
@@ -642,8 +672,7 @@ void ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
       }
 
       if (!nsChromeRegistry::gChromeRegistry) {
-        nsCOMPtr<nsIChromeRegistry> cr =
-            mozilla::services::GetChromeRegistryService();
+        nsCOMPtr<nsIChromeRegistry> cr = mozilla::services::GetChromeRegistry();
         if (!nsChromeRegistry::gChromeRegistry) {
           LogMessageWithContext(aFile, line,
                                 "Chrome registry isn't available yet.");
