@@ -27,10 +27,11 @@
 #include "nsThreadUtils.h"
 #include "mozilla/ArenaAllocatorExtensions.h"
 #include "mozilla/MemoryReporting.h"
+#include "mozilla/ProfilerLabels.h"
+#include "mozilla/ProfilerMarkers.h"
 #include "mozilla/Services.h"
 #include "mozilla/SimpleEnumerator.h"
 
-#include "GeckoProfiler.h"
 #include "ManifestParser.h"
 #include "nsSimpleEnumerator.h"
 
@@ -134,11 +135,11 @@ CategoryEnumerator* CategoryEnumerator::Create(
     return nullptr;
   }
 
-  for (auto iter = aTable.Iter(); !iter.Done(); iter.Next()) {
+  for (const auto& entry : aTable) {
     // if a category has no entries, we pretend it doesn't exist
-    CategoryNode* aNode = iter.UserData();
+    CategoryNode* aNode = entry.GetWeak();
     if (aNode->Count()) {
-      enumObj->mArray[enumObj->mCount++] = iter.Key();
+      enumObj->mArray[enumObj->mCount++] = entry.GetKey();
     }
   }
 
@@ -393,9 +394,9 @@ size_t nsCategoryManager::SizeOfIncludingThis(
   n += mArena.SizeOfExcludingThis(aMallocSizeOf);
 
   n += mTable.ShallowSizeOfExcludingThis(aMallocSizeOf);
-  for (auto iter = mTable.ConstIter(); !iter.Done(); iter.Next()) {
+  for (const auto& data : mTable.Values()) {
     // We don't measure the key string because it's a non-owning pointer.
-    n += iter.Data()->SizeOfExcludingThis(aMallocSizeOf);
+    n += data->SizeOfExcludingThis(aMallocSizeOf);
   }
 
   return n;
@@ -511,9 +512,11 @@ void nsCategoryManager::AddCategoryEntry(const nsACString& aCategoryName,
 
     if (!category) {
       // That category doesn't exist yet; let's make it.
-      category = CategoryNode::Create(&mArena);
-
-      mTable.Put(MaybeStrdup(aCategoryName, &mArena), category);
+      category = mTable
+                     .InsertOrUpdate(
+                         MaybeStrdup(aCategoryName, &mArena),
+                         UniquePtr<CategoryNode>{CategoryNode::Create(&mArena)})
+                     .get();
     }
   }
 
@@ -674,15 +677,12 @@ void NS_CreateServicesFromCategory(const char* aCategory, nsISupports* aOrigin,
       // try an observer, if it implements it.
       nsCOMPtr<nsIObserver> observer = do_QueryInterface(instance);
       if (observer) {
-#ifdef MOZ_GECKO_PROFILER
         nsPrintfCString profilerStr("%s (%s)", aObserverTopic,
                                     entryString.get());
-        AUTO_PROFILER_TEXT_MARKER_CAUSE("Category observer notification",
-                                        profilerStr, OTHER, Nothing(),
-                                        profiler_get_backtrace());
+        AUTO_PROFILER_MARKER_TEXT("Category observer notification", OTHER,
+                                  MarkerStack::Capture(), profilerStr);
         AUTO_PROFILER_LABEL_DYNAMIC_NSCSTRING_NONSENSITIVE(
             "Category observer notification -", OTHER, profilerStr);
-#endif
 
         observer->Observe(aOrigin, aObserverTopic,
                           aObserverData ? aObserverData : u"");
