@@ -7,20 +7,24 @@
 
 """Print a C++ header file for the IDL files specified on the command line"""
 
-import sys
+from __future__ import absolute_import
+
 import os.path
 import re
-import xpidl
+from xpidl import xpidl
 import itertools
-import glob
 
 printdoccomments = False
 
 if printdoccomments:
+
     def printComments(fd, clist, indent):
         for c in clist:
             fd.write("%s%s\n" % (indent, c))
+
+
 else:
+
     def printComments(fd, clist, indent):
         pass
 
@@ -33,8 +37,8 @@ def attributeParamName(a):
     return "a" + firstCap(a.name)
 
 
-def attributeParamNames(a, getter):
-    if getter and a.notxpcom:
+def attributeParamNames(a, getter, return_param=True):
+    if getter and (a.notxpcom or not return_param):
         l = []
     else:
         l = [attributeParamName(a)]
@@ -45,14 +49,33 @@ def attributeParamNames(a, getter):
 
 def attributeNativeName(a, getter):
     binaryname = a.binaryname is not None and a.binaryname or firstCap(a.name)
-    return "%s%s" % (getter and 'Get' or 'Set', binaryname)
+    return "%s%s" % (getter and "Get" or "Set", binaryname)
+
+
+def attributeAttributes(a, getter):
+    ret = ""
+
+    if a.must_use:
+        ret = "[[nodiscard]] " + ret
+
+    # Ideally, we'd set MOZ_CAN_RUN_SCRIPT in the "scriptable and not
+    # builtinclass" case too, so we'd just have memberCanRunScript() check
+    # explicit_setter_can_run_script/explicit_setter_can_run_script and call it
+    # here.  But that would likely require a fair amount of Gecko-side
+    # annotation work.  See bug 1534292.
+    if (a.explicit_getter_can_run_script and getter) or (
+        a.explicit_setter_can_run_script and not getter
+    ):
+        ret = "MOZ_CAN_RUN_SCRIPT " + ret
+
+    return ret
 
 
 def attributeReturnType(a, getter, macro):
     """macro should be NS_IMETHOD or NS_IMETHODIMP"""
     # Pick the type to be returned from the getter/setter.
     if a.notxpcom:
-        ret = a.realtype.nativeType('in').strip() if getter else "void"
+        ret = a.realtype.nativeType("in").strip() if getter else "void"
     else:
         ret = "nsresult"
 
@@ -67,33 +90,29 @@ def attributeReturnType(a, getter, macro):
         else:
             ret = "%s_(%s)" % (macro, ret)
 
-    if a.must_use:
-        ret = "MOZ_MUST_USE " + ret
-    # Ideally, we'd set MOZ_CAN_RUN_SCRIPT in the "scriptable and not
-    # builtinclass" case too, so we'd just have memberCanRunScript() check
-    # can_explicitly_run_script and call it here.  But that would likely require
-    # a fair amount of Gecko-side annotation work.  See bug 1534292.
-    if a.explicit_can_run_script:
-        ret = "MOZ_CAN_RUN_SCRIPT " + ret
-    return ret
+    return attributeAttributes(a, getter) + ret
 
 
-def attributeParamlist(a, getter):
-    if getter and a.notxpcom:
+def attributeParamlist(a, getter, return_param=True):
+    if getter and (a.notxpcom or not return_param):
         l = []
     else:
-        l = ["%s%s" % (a.realtype.nativeType(getter and 'out' or 'in'),
-                       attributeParamName(a))]
+        l = [
+            "%s%s"
+            % (a.realtype.nativeType(getter and "out" or "in"), attributeParamName(a))
+        ]
     if a.implicit_jscontext:
         l.insert(0, "JSContext* cx")
 
     return ", ".join(l)
 
 
-def attributeAsNative(a, getter, declType='NS_IMETHOD'):
-    params = {'returntype': attributeReturnType(a, getter, declType),
-              'binaryname': attributeNativeName(a, getter),
-              'paramlist': attributeParamlist(a, getter)}
+def attributeAsNative(a, getter, declType="NS_IMETHOD"):
+    params = {
+        "returntype": attributeReturnType(a, getter, declType),
+        "binaryname": attributeNativeName(a, getter),
+        "paramlist": attributeParamlist(a, getter),
+    }
     return "%(returntype)s %(binaryname)s(%(paramlist)s)" % params
 
 
@@ -101,10 +120,26 @@ def methodNativeName(m):
     return m.binaryname is not None and m.binaryname or firstCap(m.name)
 
 
+def methodAttributes(m):
+    ret = ""
+
+    if m.must_use:
+        ret = "[[nodiscard]] " + ret
+
+    # Ideally, we'd set MOZ_CAN_RUN_SCRIPT in the "scriptable and not
+    # builtinclass" case too, so we'd just have memberCanRunScript() check
+    # explicit_can_run_script and call it here.  But that would likely require
+    # a fair amount of Gecko-side annotation work.  See bug 1534292.
+    if m.explicit_can_run_script:
+        ret = "MOZ_CAN_RUN_SCRIPT " + ret
+
+    return ret
+
+
 def methodReturnType(m, macro):
     """macro should be NS_IMETHOD or NS_IMETHODIMP"""
     if m.notxpcom:
-        ret = m.realtype.nativeType('in').strip()
+        ret = m.realtype.nativeType("in").strip()
     else:
         ret = "nsresult"
 
@@ -119,46 +154,49 @@ def methodReturnType(m, macro):
         else:
             ret = "%s_(%s)" % (macro, ret)
 
-    if m.must_use:
-        ret = "MOZ_MUST_USE " + ret
-    # Ideally, we'd set MOZ_CAN_RUN_SCRIPT in the "scriptable and not
-    # builtinclass" case too, so we'd just have memberCanRunScript() check
-    # can_explicitly_run_script and call it here.  But that would likely require
-    # a fair amount of Gecko-side annotation work.  See bug 1534292.
-    if m.explicit_can_run_script:
-        ret = "MOZ_CAN_RUN_SCRIPT " + ret
-    return ret
+    return methodAttributes(m) + ret
 
 
-def methodAsNative(m, declType='NS_IMETHOD'):
-    return "%s %s(%s)" % (methodReturnType(m, declType),
-                          methodNativeName(m),
-                          paramlistAsNative(m))
+def methodAsNative(m, declType="NS_IMETHOD"):
+    return "%s %s(%s)" % (
+        methodReturnType(m, declType),
+        methodNativeName(m),
+        paramlistAsNative(m),
+    )
 
 
-def paramlistAsNative(m, empty='void'):
+def paramlistAsNative(m, empty="void", return_param=True):
     l = [paramAsNative(p) for p in m.params]
 
     if m.implicit_jscontext:
         l.append("JSContext* cx")
 
     if m.optional_argc:
-        l.append('uint8_t _argc')
+        l.append("uint8_t _argc")
 
-    if not m.notxpcom and m.realtype.name != 'void':
-        l.append(paramAsNative(xpidl.Param(paramtype='out',
-                                           type=None,
-                                           name='_retval',
-                                           attlist=[],
-                                           location=None,
-                                           realtype=m.realtype)))
+    if not m.notxpcom and m.realtype.name != "void" and return_param:
+        l.append(
+            paramAsNative(
+                xpidl.Param(
+                    paramtype="out",
+                    type=None,
+                    name="_retval",
+                    attlist=[],
+                    location=None,
+                    realtype=m.realtype,
+                )
+            )
+        )
 
     # Set any optional out params to default to nullptr. Skip if we just added
     # extra non-optional args to l.
     if len(l) == len(m.params):
         paramIter = len(m.params) - 1
-        while (paramIter >= 0 and m.params[paramIter].optional and
-                m.params[paramIter].paramtype == "out"):
+        while (
+            paramIter >= 0
+            and m.params[paramIter].optional
+            and m.params[paramIter].paramtype == "out"
+        ):
             t = m.params[paramIter].type
             # Strings can't be optional, so this shouldn't happen, but let's make sure:
             if t == "AString" or t == "ACString" or t == "AUTF8String":
@@ -182,33 +220,31 @@ def runScriptAnnotation(member):
 
 
 def paramAsNative(p):
-    default_spec = ''
+    default_spec = ""
     if p.default_value:
         default_spec = " = " + p.default_value
-    return "%s%s%s" % (p.nativeType(),
-                       p.name,
-                       default_spec)
+    return "%s%s%s" % (p.nativeType(), p.name, default_spec)
 
 
-def paramlistNames(m):
+def paramlistNames(m, return_param=True):
     names = [p.name for p in m.params]
 
     if m.implicit_jscontext:
-        names.append('cx')
+        names.append("cx")
 
     if m.optional_argc:
-        names.append('_argc')
+        names.append("_argc")
 
-    if not m.notxpcom and m.realtype.name != 'void':
-        names.append('_retval')
+    if not m.notxpcom and m.realtype.name != "void" and return_param:
+        names.append("_retval")
 
     if len(names) == 0:
-        return ''
-    return ', '.join(names)
+        return ""
+    return ", ".join(names)
 
 
 header = """/*
- * DO NOT EDIT.  THIS FILE IS GENERATED FROM %(filename)s
+ * DO NOT EDIT.  THIS FILE IS GENERATED FROM $SRCDIR/%(relpath)s
  */
 
 #ifndef __gen_%(basename)s_h__
@@ -252,70 +288,69 @@ forward_decl = """class %(name)s; /* forward declaration */
 
 def idl_basename(f):
     """returns the base name of a file with the last extension stripped"""
-    return os.path.basename(f).rpartition('.')[0]
+    return os.path.basename(f).rpartition(".")[0]
 
 
-def print_header(idl, fd, filename):
-    fd.write(header % {'filename': filename,
-                       'basename': idl_basename(filename)})
+def print_header(idl, fd, filename, relpath):
+    fd.write(header % {"relpath": relpath, "basename": idl_basename(filename)})
 
     foundinc = False
     for inc in idl.includes():
         if not foundinc:
             foundinc = True
-            fd.write('\n')
-        fd.write(include % {'basename': idl_basename(inc.filename)})
+            fd.write("\n")
+        fd.write(include % {"basename": idl_basename(inc.filename)})
 
     if idl.needsJSTypes():
         fd.write(jsvalue_include)
 
     # Include some extra files if any attributes are infallible.
-    interfaces = [p for p in idl.productions if p.kind == 'interface']
+    interfaces = [p for p in idl.productions if p.kind == "interface"]
     wroteRunScriptIncludes = False
+    wroteInfallibleIncludes = False
     for iface in interfaces:
-        attrs = [m for m in iface.members if isinstance(m, xpidl.Attribute)]
-        for attr in attrs:
-            if attr.infallible:
+        for member in iface.members:
+            if not isinstance(member, xpidl.Attribute) and not isinstance(
+                member, xpidl.Method
+            ):
+                continue
+            if not wroteInfallibleIncludes and member.infallible:
                 fd.write(infallible_includes)
-                break
+                wroteInfallibleIncludes = True
+            if not wroteRunScriptIncludes and memberCanRunScript(member):
+                fd.write(can_run_script_includes)
+                wroteRunScriptIncludes = True
+        if wroteRunScriptIncludes and wroteInfallibleIncludes:
+            break
 
-        if not wroteRunScriptIncludes:
-            methods = [m for m in iface.members if isinstance(m, xpidl.Method)]
-            for member in itertools.chain(attrs, methods):
-                if memberCanRunScript(member):
-                    fd.write(can_run_script_includes)
-                    wroteRunScriptIncludes = True
-                    break
-
-    fd.write('\n')
+    fd.write("\n")
     fd.write(header_end)
 
     for p in idl.productions:
-        if p.kind == 'include':
+        if p.kind == "include":
             continue
-        if p.kind == 'cdata':
+        if p.kind == "cdata":
             fd.write(p.data)
             continue
 
-        if p.kind == 'webidl':
+        if p.kind == "webidl":
             write_webidl(p, fd)
             continue
-        if p.kind == 'forward':
-            fd.write(forward_decl % {'name': p.name})
+        if p.kind == "forward":
+            fd.write(forward_decl % {"name": p.name})
             continue
-        if p.kind == 'interface':
+        if p.kind == "interface":
             write_interface(p, fd)
             continue
-        if p.kind == 'typedef':
-            printComments(fd, p.doccomments, '')
-            fd.write("typedef %s %s;\n\n" % (p.realtype.nativeType('in'),
-                                             p.name))
+        if p.kind == "typedef":
+            printComments(fd, p.doccomments, "")
+            fd.write("typedef %s %s;\n\n" % (p.realtype.nativeType("in"), p.name))
 
-    fd.write(footer % {'basename': idl_basename(filename)})
+    fd.write(footer % {"basename": idl_basename(filename)})
 
 
 def write_webidl(p, fd):
-    path = p.native.split('::')
+    path = p.native.split("::")
     for seg in path[:-1]:
         fd.write("namespace %s {\n" % seg)
     fd.write("class %s; /* webidl %s */\n" % (path[-1], p.name))
@@ -334,11 +369,14 @@ iface_header = r"""
 
 """
 
-uuid_decoder = re.compile(r"""(?P<m0>[a-f0-9]{8})-
+uuid_decoder = re.compile(
+    r"""(?P<m0>[a-f0-9]{8})-
                               (?P<m1>[a-f0-9]{4})-
                               (?P<m2>[a-f0-9]{4})-
                               (?P<m3>[a-f0-9]{4})-
-                              (?P<m4>[a-f0-9]{12})$""", re.X)
+                              (?P<m4>[a-f0-9]{12})$""",
+    re.X,
+)
 
 iface_prolog = """ {
  public:
@@ -347,6 +385,12 @@ iface_prolog = """ {
 
 """
 
+iface_scriptable = """\
+  /* Used by ToJSValue to check which scriptable interface is implemented. */
+  using ScriptableInterfaceType = %(name)s;
+
+"""
+
 iface_epilog = """};
 
   NS_DEFINE_STATIC_IID_ACCESSOR(%(name)s, %(defname)s_IID)
@@ -370,8 +414,8 @@ iface_forward_safe = """
 /* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
 #define NS_FORWARD_SAFE_%(macroname)s(_to) """  # NOQA: E501
 
-attr_builtin_infallible_tmpl = """\
-  inline %(realtype)s%(nativename)s(%(args)s)
+builtin_infallible_tmpl = """\
+  %(attributes)sinline %(realtype)s %(nativename)s(%(args)s)
   {
     %(realtype)sresult;
     mozilla::DebugOnly<nsresult> rv = %(nativename)s(%(argnames)s&result);
@@ -383,8 +427,8 @@ attr_builtin_infallible_tmpl = """\
 # NOTE: We don't use RefPtr::forget here because we don't want to need the
 # definition of %(realtype)s in scope, which we would need for the
 # AddRef/Release calls.
-attr_refcnt_infallible_tmpl = """\
-  inline already_AddRefed<%(realtype)s>%(nativename)s(%(args)s)
+refcnt_infallible_tmpl = """\
+  %(attributes)s inline already_AddRefed<%(realtype)s> %(nativename)s(%(args)s)
   {
     %(realtype)s* result = nullptr;
     mozilla::DebugOnly<nsresult> rv = %(nativename)s(%(argnames)s&result);
@@ -394,6 +438,39 @@ attr_refcnt_infallible_tmpl = """\
 """
 
 
+def infallibleDecl(member):
+    isattr = isinstance(member, xpidl.Attribute)
+    ismethod = isinstance(member, xpidl.Method)
+    assert isattr or ismethod
+
+    realtype = member.realtype.nativeType("in")
+    tmpl = builtin_infallible_tmpl
+
+    if member.realtype.kind != "builtin" and member.realtype.kind != "cenum":
+        assert realtype.endswith(" *"), "bad infallible type"
+        tmpl = refcnt_infallible_tmpl
+        realtype = realtype[:-2]  # strip trailing pointer
+
+    if isattr:
+        nativename = attributeNativeName(member, getter=True)
+        args = attributeParamlist(member, getter=True, return_param=False)
+        argnames = attributeParamNames(member, getter=True, return_param=False)
+        attributes = attributeAttributes(member, getter=True)
+    else:
+        nativename = methodNativeName(member)
+        args = paramlistAsNative(member, return_param=False)
+        argnames = paramlistNames(member, return_param=False)
+        attributes = methodAttributes(member)
+
+    return tmpl % {
+        "attributes": attributes,
+        "realtype": realtype,
+        "nativename": nativename,
+        "args": args,
+        "argnames": argnames + ", " if argnames else "",
+    }
+
+
 def write_interface(iface, fd):
     if iface.namemap is None:
         raise Exception("Interface was not resolved.")
@@ -403,9 +480,12 @@ def write_interface(iface, fd):
 
     def record_name(name):
         if name in names:
-            raise Exception("Unexpected overloaded virtual method %s in interface %s"
-                            % (name, iface.name))
+            raise Exception(
+                "Unexpected overloaded virtual method %s in interface %s"
+                % (name, iface.name)
+            )
         names.add(name)
+
     for m in iface.members:
         if type(m) == xpidl.Attribute:
             record_name(attributeNativeName(m, getter=True))
@@ -418,13 +498,17 @@ def write_interface(iface, fd):
         fd.write("  enum {\n")
         enums = []
         for c in g:
-            printComments(fd, c.doccomments, '  ')
+            printComments(fd, c.doccomments, "  ")
             basetype = c.basetype
             value = c.getValue()
-            enums.append("    %(name)s = %(value)s%(signed)s" % {
-                         'name': c.name,
-                         'value': value,
-                         'signed': (not basetype.signed) and 'U' or ''})
+            enums.append(
+                "    %(name)s = %(value)s%(signed)s"
+                % {
+                    "name": c.name,
+                    "value": value,
+                    "signed": (not basetype.signed) and "U" or "",
+                }
+            )
         fd.write(",\n".join(enums))
         fd.write("\n  };\n\n")
 
@@ -435,60 +519,55 @@ def write_interface(iface, fd):
         fd.write("  };\n\n")
 
     def write_method_decl(m):
-        printComments(fd, m.doccomments, '  ')
+        printComments(fd, m.doccomments, "  ")
 
         fd.write("  /* %s */\n" % m.toIDL())
-        fd.write("  %s%s = 0;\n\n" % (runScriptAnnotation(m),
-                                      methodAsNative(m)))
+        fd.write("  %s%s = 0;\n\n" % (runScriptAnnotation(m), methodAsNative(m)))
+
+        if m.infallible:
+            fd.write(infallibleDecl(m))
 
     def write_attr_decl(a):
-        printComments(fd, a.doccomments, '  ')
+        printComments(fd, a.doccomments, "  ")
 
         fd.write("  /* %s */\n" % a.toIDL())
 
-        fd.write("  %s%s = 0;\n" % (runScriptAnnotation(a),
-                                    attributeAsNative(a, True)))
+        fd.write("  %s%s = 0;\n" % (runScriptAnnotation(a), attributeAsNative(a, True)))
         if a.infallible:
-            realtype = a.realtype.nativeType('in')
-            tmpl = attr_builtin_infallible_tmpl
-
-            if a.realtype.kind != 'builtin' and a.realtype.kind != 'cenum':
-                assert realtype.endswith(' *'), "bad infallible type"
-                tmpl = attr_refcnt_infallible_tmpl
-                realtype = realtype[:-2]  # strip trailing pointer
-
-            fd.write(tmpl % {'realtype': realtype,
-                             'nativename': attributeNativeName(a, getter=True),
-                             'args': '' if not a.implicit_jscontext else 'JSContext* cx',
-                             'argnames': '' if not a.implicit_jscontext else 'cx, '})
+            fd.write(infallibleDecl(a))
 
         if not a.readonly:
-            fd.write("  %s%s = 0;\n" % (runScriptAnnotation(a),
-                                        attributeAsNative(a, False)))
+            fd.write(
+                "  %s%s = 0;\n" % (runScriptAnnotation(a), attributeAsNative(a, False))
+            )
         fd.write("\n")
 
     defname = iface.name.upper()
-    if iface.name[0:2] == 'ns':
-        defname = 'NS_' + defname[2:]
+    if iface.name[0:2] == "ns":
+        defname = "NS_" + defname[2:]
 
     names = uuid_decoder.match(iface.attributes.uuid).groupdict()
-    m3str = names['m3'] + names['m4']
-    names['m3joined'] = ", ".join(["0x%s" % m3str[i:i+2] for i in xrange(0, 16, 2)])
+    m3str = names["m3"] + names["m4"]
+    names["m3joined"] = ", ".join(["0x%s" % m3str[i : i + 2] for i in range(0, 16, 2)])
 
-    if iface.name[2] == 'I':
+    if iface.name[2] == "I":
         implclass = iface.name[:2] + iface.name[3:]
     else:
-        implclass = '_MYCLASS_'
+        implclass = "_MYCLASS_"
 
-    names.update({'defname': defname,
-                  'macroname': iface.name.upper(),
-                  'name': iface.name,
-                  'iid': iface.attributes.uuid,
-                  'implclass': implclass})
+    names.update(
+        {
+            "defname": defname,
+            "macroname": iface.name.upper(),
+            "name": iface.name,
+            "iid": iface.attributes.uuid,
+            "implclass": implclass,
+        }
+    )
 
     fd.write(iface_header % names)
 
-    printComments(fd, iface.doccomments, '')
+    printComments(fd, iface.doccomments, "")
 
     fd.write("class ")
     foundcdata = False
@@ -504,6 +583,9 @@ def write_interface(iface, fd):
         fd.write(" : public %s" % iface.base)
     fd.write(iface_prolog % names)
 
+    if iface.attributes.scriptable:
+        fd.write(iface_scriptable % names)
+
     for key, group in itertools.groupby(iface.members, key=type):
         if key == xpidl.ConstMember:
             write_const_decls(group)  # iterator of all the consts
@@ -528,17 +610,24 @@ def write_interface(iface, fd):
         for member in iface.members:
             if isinstance(member, xpidl.Attribute):
                 if member.infallible:
-                    fd.write("\\\n  using %s::%s; " %
-                             (iface.name, attributeNativeName(member, True)))
-                fd.write("\\\n  %s%s; " % (attributeAsNative(member, True, declType), suffix))
+                    fd.write(
+                        "\\\n  using %s::%s; "
+                        % (iface.name, attributeNativeName(member, True))
+                    )
+                fd.write(
+                    "\\\n  %s%s; " % (attributeAsNative(member, True, declType), suffix)
+                )
                 if not member.readonly:
-                    fd.write("\\\n  %s%s; " % (attributeAsNative(member, False, declType), suffix))
+                    fd.write(
+                        "\\\n  %s%s; "
+                        % (attributeAsNative(member, False, declType), suffix)
+                    )
             elif isinstance(member, xpidl.Method):
                 fd.write("\\\n  %s%s; " % (methodAsNative(member, declType), suffix))
         if len(iface.members) == 0:
-            fd.write('\\\n  /* no methods! */')
-        elif member.kind not in ('attribute', 'method'):
-            fd.write('\\')
+            fd.write("\\\n  /* no methods! */")
+        elif member.kind not in ("attribute", "method"):
+            fd.write("\\")
 
     writeDeclaration(fd, iface, True)
     fd.write(iface_nonvirtual % names)
@@ -551,60 +640,74 @@ def write_interface(iface, fd):
         for member in iface.members:
             if isinstance(member, xpidl.Attribute):
                 if forward_infallible and member.infallible:
-                    fd.write("\\\n  using %s::%s; " %
-                             (iface.name, attributeNativeName(member, True)))
+                    fd.write(
+                        "\\\n  using %s::%s; "
+                        % (iface.name, attributeNativeName(member, True))
+                    )
                 attr_tmpl = tmpl_notxpcom if member.notxpcom else tmpl
-                fd.write(attr_tmpl % {'asNative': attributeAsNative(member, True),
-                                      'nativeName': attributeNativeName(member, True),
-                                      'paramList': attributeParamNames(member, True)})
+                fd.write(
+                    attr_tmpl
+                    % {
+                        "asNative": attributeAsNative(member, True),
+                        "nativeName": attributeNativeName(member, True),
+                        "paramList": attributeParamNames(member, True),
+                    }
+                )
                 if not member.readonly:
-                    fd.write(attr_tmpl % {'asNative': attributeAsNative(member, False),
-                                          'nativeName': attributeNativeName(member, False),
-                                          'paramList': attributeParamNames(member, False)})
+                    fd.write(
+                        attr_tmpl
+                        % {
+                            "asNative": attributeAsNative(member, False),
+                            "nativeName": attributeNativeName(member, False),
+                            "paramList": attributeParamNames(member, False),
+                        }
+                    )
             elif isinstance(member, xpidl.Method):
                 if member.notxpcom:
-                    fd.write(tmpl_notxpcom % {'asNative': methodAsNative(member),
-                                              'nativeName': methodNativeName(member),
-                                              'paramList': paramlistNames(member)})
+                    fd.write(
+                        tmpl_notxpcom
+                        % {
+                            "asNative": methodAsNative(member),
+                            "nativeName": methodNativeName(member),
+                            "paramList": paramlistNames(member),
+                        }
+                    )
                 else:
-                    fd.write(tmpl % {'asNative': methodAsNative(member),
-                                     'nativeName': methodNativeName(member),
-                                     'paramList': paramlistNames(member)})
+                    fd.write(
+                        tmpl
+                        % {
+                            "asNative": methodAsNative(member),
+                            "nativeName": methodNativeName(member),
+                            "paramList": paramlistNames(member),
+                        }
+                    )
         if len(iface.members) == 0:
-            fd.write('\\\n  /* no methods! */')
-        elif member.kind not in ('attribute', 'method'):
-            fd.write('\\')
+            fd.write("\\\n  /* no methods! */")
+        elif member.kind not in ("attribute", "method"):
+            fd.write("\\")
 
-    emitTemplate(True,
-                 "\\\n  %(asNative)s override { return _to %(nativeName)s(%(paramList)s); } ")
+    emitTemplate(
+        True,
+        "\\\n  %(asNative)s override { return _to %(nativeName)s(%(paramList)s); } ",
+    )
 
     fd.write(iface_forward_safe % names)
 
     # Don't try to safely forward notxpcom functions, because we have no
     # sensible default error return.  Instead, the caller will have to
     # implement them.
-    emitTemplate(False,
-                 "\\\n  %(asNative)s override { return !_to ? NS_ERROR_NULL_POINTER : _to->%(nativeName)s(%(paramList)s); } ",  # NOQA: E501
-                 "\\\n  %(asNative)s override; ")
+    emitTemplate(
+        False,
+        "\\\n  %(asNative)s override { return !_to ? NS_ERROR_NULL_POINTER : _to->%(nativeName)s(%(paramList)s); } ",  # NOQA: E501
+        "\\\n  %(asNative)s override; ",
+    )
 
-    fd.write('\n\n')
+    fd.write("\n\n")
 
 
 def main(outputfile):
-    cachedir = os.path.dirname(outputfile.name if outputfile else '') or '.'
-    if not os.path.isdir(cachedir):
-        os.mkdir(cachedir)
-    sys.path.append(cachedir)
-
-    # Delete the lex/yacc files.  Ply is too stupid to regenerate them
-    # properly
-    for fileglobs in [os.path.join(cachedir, f) for f in ["xpidllex.py*", "xpidlyacc.py*"]]:
-        for filename in glob.glob(fileglobs):
-            os.remove(filename)
-
-    # Instantiate the parser.
-    xpidl.IDLParser(outputdir=cachedir)
+    xpidl.IDLParser()
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main(None)
