@@ -11,18 +11,8 @@
 #![allow(non_snake_case)]
 #![allow(non_camel_case_types)]
 
-extern crate libc;
-extern crate nserror;
-extern crate nsstring;
-extern crate thin_vec;
-extern crate threadbound;
-
 // re-export the xpcom_macros macro
-#[macro_use]
-#[allow(unused_imports)]
-extern crate xpcom_macros;
-#[doc(hidden)]
-pub use xpcom_macros::*;
+pub use xpcom_macros::xpcom;
 
 // Helper functions and data structures are exported in the root of the crate.
 mod base;
@@ -32,6 +22,10 @@ pub use base::*;
 mod method;
 pub use method::*;
 
+// dom::Promise resolving.
+mod promise;
+pub use promise::*;
+
 mod refptr;
 pub use refptr::*;
 
