@@ -26,20 +26,14 @@
  * ASCII case-insensitive comparator.  (for Unicode case-insensitive
  * comparision, see nsUnicharUtils.h)
  */
-class nsCaseInsensitiveCStringComparator : public nsCStringComparator {
- public:
-  nsCaseInsensitiveCStringComparator() {}
-  typedef char char_type;
-
-  virtual int operator()(const char_type*, const char_type*, uint32_t,
-                         uint32_t) const override;
-};
+int nsCaseInsensitiveCStringComparator(const char*, const char*, uint32_t,
+                                       uint32_t);
 
 class nsCaseInsensitiveCStringArrayComparator {
  public:
   template <class A, class B>
   bool Equals(const A& aStrA, const B& aStrB) const {
-    return aStrA.Equals(aStrB, nsCaseInsensitiveCStringComparator());
+    return aStrA.Equals(aStrB, nsCaseInsensitiveCStringComparator);
   }
 };
 
