@@ -71,6 +71,9 @@ class nsTString : public nsTSubstring<T> {
     this->Assign(aData, aLength);
   }
 
+  explicit nsTString(mozilla::Span<const char_type> aData)
+      : nsTString(aData.Elements(), aData.Length()) {}
+
 #if defined(MOZ_USE_CHAR16_WRAPPER)
   template <typename Q = T, typename EnableIfChar16 = mozilla::Char16OnlyT<Q>>
   explicit nsTString(char16ptr_t aStr, size_type aLength = size_type(-1))
@@ -278,35 +281,6 @@ class nsTString : public nsTSubstring<T> {
     return RFindCharInSet(aString.get(), aOffset);
   }
 
-  /**
-   * Compares a given string to this string.
-   *
-   * @param   aString is the string to be compared
-   * @param   aIgnoreCase tells us how to treat case
-   * @param   aCount tells us how many chars to compare
-   * @return  -1,0,1
-   */
-  template <typename Q = T, typename EnableIfChar = mozilla::CharOnlyT<Q>>
-  int32_t Compare(const char_type* aString, bool aIgnoreCase = false,
-                  int32_t aCount = -1) const;
-
-  /**
-   * Equality check between given string and this string.
-   *
-   * @param   aString is the string to check
-   * @param   aIgnoreCase tells us how to treat case
-   * @param   aCount tells us how many chars to compare
-   * @return  boolean
-   */
-  template <typename Q = T, typename EnableIfChar = mozilla::CharOnlyT<Q>>
-  bool EqualsIgnoreCase(const char_type* aString, int32_t aCount = -1) const {
-    return Compare(aString, true, aCount) == 0;
-  }
-
-  template <typename Q = T, typename EnableIfChar16 = mozilla::Char16OnlyT<Q>>
-  bool EqualsIgnoreCase(const incompatible_char_type* aString,
-                        int32_t aCount = -1) const;
-
   /**
    * Perform string to double-precision float conversion.
    *
@@ -411,12 +385,12 @@ class nsTString : public nsTSubstring<T> {
    */
   void ReplaceSubstring(const self_type& aTarget, const self_type& aNewValue);
   void ReplaceSubstring(const char_type* aTarget, const char_type* aNewValue);
-  MOZ_MUST_USE bool ReplaceSubstring(const self_type& aTarget,
-                                     const self_type& aNewValue,
-                                     const fallible_t&);
-  MOZ_MUST_USE bool ReplaceSubstring(const char_type* aTarget,
-                                     const char_type* aNewValue,
-                                     const fallible_t&);
+  [[nodiscard]] bool ReplaceSubstring(const self_type& aTarget,
+                                      const self_type& aNewValue,
+                                      const fallible_t&);
+  [[nodiscard]] bool ReplaceSubstring(const char_type* aTarget,
+                                      const char_type* aNewValue,
+                                      const fallible_t&);
 
   /**
    *  This method trims characters found in aTrimSet from
@@ -619,7 +593,7 @@ class MOZ_NON_MEMMOVABLE nsTAutoStringN : public nsTString<T> {
  protected:
   friend class nsTSubstring<T>;
 
-  size_type mInlineCapacity;
+  const size_type mInlineCapacity;
 
  private:
   char_type mStorage[N];
@@ -653,6 +627,11 @@ class nsTArrayElementTraits<nsTAutoString<T>> {
       Instead_Use_nsTArray_of<nsTString<T>>* aE, const A& aArg) {
     return 0;
   }
+  template <class... Args>
+  static Dont_Instantiate_nsTArray_of<nsTAutoString<T>>* Construct(
+      Instead_Use_nsTArray_of<nsTString<T>>* aE, Args&&... aArgs) {
+    return 0;
+  }
   static Dont_Instantiate_nsTArray_of<nsTAutoString<T>>* Destruct(
       Instead_Use_nsTArray_of<nsTString<T>>* aE) {
     return 0;
