@@ -2,6 +2,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+
 function run_test() {
   const PROPERTIES = [
     "name",
@@ -10,10 +12,9 @@ function run_test() {
     "pagesize",
     "pageshift",
     "memmapalign",
-    "cpucount",
     "memsize",
   ];
-  let sysInfo = Cc["@mozilla.org/system-info;1"].getService(Ci.nsIPropertyBag2);
+  let sysInfo = Services.sysinfo;
 
   PROPERTIES.forEach(function(aPropertyName) {
     print("Testing property: " + aPropertyName);
