@@ -17,14 +17,11 @@
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/BinarySearch.h"
 #include "mozilla/OperatorNewExtensions.h"
+#include "mozilla/StaticPrefs_timer.h"
 
 #include <math.h>
 
 using namespace mozilla;
-#ifdef MOZ_TASK_TRACER
-#  include "GeckoTaskTracerImpl.h"
-using namespace mozilla::tasktracer;
-#endif
 
 NS_IMPL_ISUPPORTS_INHERITED(TimerThread, Runnable, nsIObserver)
 
@@ -152,6 +149,7 @@ class nsTimerEvent final : public CancelableRunnable {
   }
   void operator delete(void* aPtr) {
     sAllocator->Free(aPtr);
+    sAllocatorUsers--;
     DeleteAllocatorIfNeeded();
   }
 
@@ -166,7 +164,6 @@ class nsTimerEvent final : public CancelableRunnable {
     MOZ_ASSERT(!sCanDeleteAllocator || sAllocatorUsers > 0,
                "This will result in us attempting to deallocate the "
                "nsTimerEvent allocator twice");
-    sAllocatorUsers--;
   }
 
   TimeStamp mInitTime;
@@ -176,12 +173,12 @@ class nsTimerEvent final : public CancelableRunnable {
   static TimerEventAllocator* sAllocator;
 
   static Atomic<int32_t, SequentiallyConsistent> sAllocatorUsers;
-  static bool sCanDeleteAllocator;
+  static Atomic<bool, SequentiallyConsistent> sCanDeleteAllocator;
 };
 
 TimerEventAllocator* nsTimerEvent::sAllocator = nullptr;
 Atomic<int32_t, SequentiallyConsistent> nsTimerEvent::sAllocatorUsers;
-bool nsTimerEvent::sCanDeleteAllocator = false;
+Atomic<bool, SequentiallyConsistent> nsTimerEvent::sCanDeleteAllocator;
 
 namespace {
 
@@ -412,7 +409,10 @@ TimerThread::Run() {
           // We are going to let the call to PostTimerEvent here handle the
           // release of the timer so that we don't end up releasing the timer
           // on the TimerThread instead of on the thread it targets.
-          timerRef = PostTimerEvent(timerRef.forget());
+          {
+            LogTimerEvent::Run run(timerRef.get());
+            timerRef = PostTimerEvent(timerRef.forget());
+          }
 
           if (timerRef) {
             // We got our reference back due to an error.
@@ -629,6 +629,8 @@ bool TimerThread::AddTimerInternal(nsTimerImpl* aTimer) {
 
   TimeStamp now = TimeStamp::Now();
 
+  LogTimerEvent::LogDispatch(aTimer);
+
   UniquePtr<Entry>* entry = mTimers.AppendElement(
       MakeUnique<Entry>(now, aTimer->mTimeout, aTimer), mozilla::fallible);
   if (!entry) {
@@ -637,12 +639,6 @@ bool TimerThread::AddTimerInternal(nsTimerImpl* aTimer) {
 
   std::push_heap(mTimers.begin(), mTimers.end(), Entry::UniquePtrLessThan);
 
-#ifdef MOZ_TASK_TRACER
-  // Caller of AddTimer is the parent task of its timer event, so we store the
-  // TraceInfo here for later used.
-  aTimer->GetTLSTraceInfo();
-#endif
-
   return true;
 }
 
@@ -674,11 +670,8 @@ void TimerThread::RemoveLeadingCanceledTimersInternal() {
   }
 
   // Finally, remove the canceled timers from the back of the
-  // nsTArray.  Note, since std::pop_heap() uses iterators
-  // we must convert to nsTArray indices and number of
-  // elements here.
-  mTimers.RemoveElementsAt(sortedEnd - mTimers.begin(),
-                           mTimers.end() - sortedEnd);
+  // nsTArray.
+  mTimers.RemoveLastElements(mTimers.end() - sortedEnd);
 }
 
 void TimerThread::RemoveFirstTimerInternal() {
@@ -706,14 +699,6 @@ already_AddRefed<nsTimerImpl> TimerThread::PostTimerEvent(
   // event, so we can avoid firing a timer that was re-initialized after being
   // canceled.
 
-#ifdef MOZ_TASK_TRACER
-  // During the dispatch of TimerEvent, we overwrite the current TraceInfo
-  // partially with the info saved in timer earlier, and restore it back by
-  // AutoSaveCurTraceInfo.
-  AutoSaveCurTraceInfo saveCurTraceInfo;
-  (timer->GetTracedTask()).SetTLSTraceInfo();
-#endif
-
   nsCOMPtr<nsIEventTarget> target = timer->mEventTarget;
 
   void* p = nsTimerEvent::operator new(sizeof(nsTimerEvent));
@@ -761,6 +746,10 @@ void TimerThread::DoAfterSleep() {
 NS_IMETHODIMP
 TimerThread::Observe(nsISupports* /* aSubject */, const char* aTopic,
                      const char16_t* /* aData */) {
+  if (StaticPrefs::timer_ignore_sleep_wake_notifications()) {
+    return NS_OK;
+  }
+
   if (strcmp(aTopic, "sleep_notification") == 0 ||
       strcmp(aTopic, "suspend_process_notification") == 0) {
     DoBeforeSleep();
